--|$Log:	btrees.a,v $
--|Revision 8.0  91/01/17  10:37:20  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|

with Lists;
generic

    type ItemType is private; 
                       --| Information being contained in a node of tree


    with function "<"(X,Y: in ItemType) return boolean;
                       --| Function which defines ordering of nodes

package BTrees is 


--| Overview
--| This package creates an ordered binary tree.  This will allow for 
--| quick insertion, and search.  
--|
--| The tree is organized such that 
--|  
--|  leftchild < root    root < rightchild
--| 
--| This means that by doing a left to right search of the tree will can
--| produce the nodes of the tree in ascending order.





--                             Types
--                             -----

type Tree is  private;     --| This is the type exported to represent the
                           --| tree.


type TreeIter is private;  --| This is the type which is used to iterate
                           --| over the set.

--|                          Exceptions
--|                          ----------

--|                          Operations
--|                          ----------
--|
--| Create           Creates a tree.
--| Deposit          Replaces the given node's information with 
--|                  the given information.
--| DestroyTree      Destroys the given tree and returns the spaces.
--| InsertNode       This inserts a node n into a tree t.
--| MakeTreeIter     This returns an iterator to the user in order to start
--|                  an iteration.
--| More             This returns true if there are more elements to iterate
--|                  over in the tree.
--| Next             This returns the information associated with the current
--|                  iterator and advances the iterator.
       

---------------------------------------------------------------------------

function Create             --| This function creates the tree.

return Tree;

--| Effects
--| This creates a tree containing no information and no children.  An 
--| emptytree.

-------------------------------------------------------------------------------

procedure Deposit (              --| This deposits the information I in the
                                 --| root of the Tree S.
          I :in     ItemType;    --| The information being deposited.
          S :in     Tree         --| The tree where the information is being
                                 --| stored.
);

--| Modifies
--| This changes the information stored at the root of the tree S.

-------------------------------------------------------------------------------


procedure DestroyTree (         --| Destroys a tree.
          T  :in out Tree       --| Tree being destroyed.
);

--| Effects
--| Destroys a tree and returns the space which it is occupying.

--------------------------------------------------------------------------

Procedure Insertnode(           --| This Procedure Inserts A Node Into The 
                                --| Specified Tree.
       N      :In Out Itemtype; --| The Information To Be Contained In The 
                                --| Node Being Inserted.   
                              
       T      :In Out Tree;     --| Tree Being Inserted Into.
       Root   :   Out Tree;     --| Root of the subtree which Node N heads. 
                                --| This is the position of the node N in T.
       Exists :   out boolean   --| If this node already exists in the tree
                                --| Exists is true.  If this is the first
                                --| insertion Exists is false.
); 

Procedure Insertnode(           --| This Procedure Inserts A Node Into The 
                                --| Specified Tree.
       N      :In Out Itemtype; --| The Information To Be Contained In The 
                                --| Node Being Inserted.   
                              
       T      :In Out Tree;     --| Tree Being Inserted Into.
       Root   :   Out Tree      --| Root of the subtree which Node N heads. 
                                --| This is the position of the node N in T.
); 


--| Effects
--| This adds the node N to the tree T inserting in the proper postion.

--| Modifies
--| This modifies the tree T by add the node N to it.

------------------------------------------------------------------------------

function MakeTreeIter (         --| Sets a variable to a position in the
                                --| tree
                                --| where the iteration is to begin.  In this 
                                --| case the position is a pointer to the  
                                --| the deepest leftmost leaf in the tree.
        T:in Tree               --| Tree being iterated over 
) return TreeIter;


--| Effects


-----------------------------------------------------------------------------

function More (                 --| Returns true if there are more elements 
                                --| in the tree to iterate over.
          I :in TreeIter  
) return boolean;


-----------------------------------------------------------------------------

procedure Next (                --| This is the iterator operation.  Given 
                                --| an Iter in the Tree it returns the 
                                --| item Iter points to and updates the
                                --| iter. If Iter is at the end of the Tree, 
                                --| yielditer returns false otherwise it 
                                --| returns true.
    I        :in out TreeIter;  --| The iter which marks the position in the 
                                --| Tree.

    Info     :   out ItemType   --| Information being returned from a node.
);    

function Get_Info (T : in Tree) return Itemtype;


procedure Copy (From : in Tree; To : in out Tree);


procedure Find (Info : in ItemType; Root : in Tree; Data : out ItemType);
---------------------------------------------------------------------------

private

   type Node;
   type Tree is access Node;

   type Node is 
        record
            Info           :ItemType;
            LeftChild      :Tree;
            RightChild     :Tree;
        end record;

   package NodeOrder is new Lists (Tree);


   type TreeIter is
      record
          NodeList :NodeOrder.List;
          State    :NodeOrder.ListIter;
      end record;
 

end BTrees;

