--|$Log:	btrees_.a,v $
--|Revision 8.0  91/01/17  10:37:27  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|

with Unchecked_Deallocation;

package body Btrees is

   ----------------------------------------------------------------------------
   --                   Local Subprograms
   ----------------------------------------------------------------------------

   procedure Free is new Unchecked_Deallocation (Node, Tree);

   function Equal (X, Y : in Itemtype) return Boolean is

   begin
      return (not (X < Y)) and (not (Y < X));
   end Equal;

  -----------------------------------------------------------------------------

   function Generate (T : in Tree) return Nodeorder.List is
      L : Nodeorder.List;

   begin
      L := Nodeorder.Create;
      if T /= null then
         L := Generate (T.Leftchild);
         Nodeorder.Attach (L, T);
         Nodeorder.Attach (L, Generate (T.Rightchild));
      end if;
      return L;
   end Generate;

  -----------------------------------------------------------------------------



  -----------------------------------------------------------------------------
   --                    Visible Subprograms
  -----------------------------------------------------------------------------

   function Create return Tree is

   begin
      return null;
   end Create;

  -----------------------------------------------------------------------------

   procedure Deposit (I : in Itemtype;
                      S : in Tree) is

   begin
      S.Info := I;
   end Deposit;

  -----------------------------------------------------------------------------

   procedure Destroytree (T : in out Tree) is

   begin
      if T.Leftchild /= null then
         Destroytree (T.Leftchild);
         Destroytree (T.Rightchild);
         Free (T);
      end if;
   end Destroytree;

  -----------------------------------------------------------------------------

   procedure Insertnode (N      : in out Itemtype;
                         T      : in out Tree;
                         Root   : out Tree;
                         Exists : out Boolean) is
   begin
      if T = null then
         T      := new Node'(Info       => N,
                        Leftchild  => null,
                        Rightchild => null);
         Root   := T;
         Exists := False;
         N      := T.Info;
      elsif N < T.Info then
         Insertnode (N, T.Leftchild, Root, Exists);
      elsif T.Info < N then
         Insertnode (N, T.Rightchild, Root, Exists);
      else
         Root   := T;
         Exists := True;
         N      := T.Info;

      end if;
   end Insertnode;

   procedure Insertnode (N      : in out Itemtype;
                         T      : in out Tree;
                         Root   : out Tree) is
   begin
      if T = null then
         T      := new Node'(Info       => N,
                        Leftchild  => null,
                        Rightchild => null);
         Root   := T;
         N      := T.Info;
      elsif N < T.Info then
         Insertnode (N, T.Leftchild, Root);
      elsif T.Info < N then
         Insertnode (N, T.Rightchild, Root);
      else
         Root   := T;
         N      := T.Info;
      end if;
   end Insertnode;


  -----------------------------------------------------------------------------

   function Maketreeiter (T : in Tree) return Treeiter is

      I : Treeiter;

   begin
      I.Nodelist := Nodeorder.Create;
      if T /= null then
         I.Nodelist := Generate (T.Leftchild);
         Nodeorder.Attach (I.Nodelist, T);
         Nodeorder.Attach (I.Nodelist, Generate (T.Rightchild));
      end if;
      I.State := Nodeorder.Makelistiter (I.Nodelist);
      return I;
   end Maketreeiter;

  -----------------------------------------------------------------------------

   function More (I : in Treeiter) return Boolean is

   begin
      return Nodeorder.More (I.State);
   end More;

  -----------------------------------------------------------------------------

   procedure Next (I    : in out Treeiter;
                   Info : out Itemtype) is
      T : Tree;
   begin
      Nodeorder.Next (I.State, T);
      Info := T.Info;
   end Next;

   function Get_Info (T : in Tree) return Itemtype is
   begin
     return T.info;
   end Get_Info;

   procedure Copy (From : in Tree;
                   To   : in out Tree) is
      Item : Itemtype;
      Iter : TreeIter;
      Temp_Tree : Tree;
      Temp_Bool : Boolean;
   begin
      if To /= null then
         Destroytree (To);
      end if;
      Iter := Maketreeiter (From);
      To := Create;
      while More (Iter) loop
         Next (Iter, Item);
         Insertnode (Item, To, Temp_Tree, Temp_Bool);
      end loop;
   end Copy;


   procedure Find (Info : in Itemtype;
                   Root : in Tree;
                   Data : out Itemtype) is
      T : Tree := Root;
   begin
      if T = null then
         null; --TBD
      elsif Info < T.Info then
         Find (Info, T.Leftchild, Data);
      elsif T.Info < Info then
         Find (Info, T.Rightchild, Data);
      else
         Data := T.Info;
      end if;
   end Find;

  -----------------------------------------------------------------------------

end Btrees;
