--|$Log:	hash_table_.a,v $
--|Revision 8.0  91/01/17  10:37:28  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
with Unchecked_Conversion;
with Unchecked_Deallocation;
with Text_Io;
package body Hash_Table is

   type Hash_Node;
   type Node_Ptr is access Hash_Node;
   subtype Hash_Range is Positive range 1 .. Table_Size;

   type Hash_Node is
      record
         Node_Num : Node_Id;
         Key      : Key_Type;
         Rec_Info : Info_Type;
         Next     : Node_Ptr;
      end record;

   Hash_Table : array (Hash_Range) of Node_Ptr;

   Node_Count : Node_Id := 0;

   procedure Check_Hash is
      Node_Count : Integer;
      Curr_Node  : Node_Ptr;
   begin
      for I in Hash_Range loop
         Node_Count := 0;
			Curr_Node := Hash_Table (I);
         while curr_node /= null loop
            Node_Count := Node_Count + 1;
				if curr_node /= null then
	            Curr_Node  := Curr_Node.Next;
				end if;
         end loop;

         Text_Io.Put_Line (Integer'IMAGE (I) & " ==> " &
                           Integer'IMAGE (Node_Count));
      end loop;
   end Check_Hash;

   procedure Destroy_Table is
      Zap_Node : Node_Ptr;
      procedure Free is new Unchecked_Deallocation (Hash_Node, Node_Ptr);
   begin
      for Index in Hash_Range loop
         while Hash_Table (Index) /= null loop
            Zap_Node := Hash_Table (Index);
            Hash_Table (Index) := Hash_Table (Index).Next;
            Free (Zap_Node);
         end loop;
      end loop;
      Node_Count := 0;
   end Destroy_Table;

   function Convert_Key is new Unchecked_Conversion (Key_Type, Integer);

   function Hash (Key : in Key_Type) return Natural is
   begin
      return (Convert_Key (Key) mod Table_Size + 1);
   end Hash;

   function Save_Rec (Key  : in Key_Type;
                      Info : Info_Type) return Node_Id is
      Hash_Index : Hash_Range := Hash (Key);
      New_Node   : Node_Ptr;
   begin
      if Hash_Table (Hash_Index) = null then
         Node_Count              := Node_Count + 1;
         New_Node                := new Hash_Node'(Node_Count, Key, Info, null);
         Hash_Table (Hash_Index) := New_Node;
         return (Node_Count);
      else
         declare
            Curr_Node : Node_Ptr := Hash_Table (Hash_Index);
            Prev_Node : Node_Ptr := null;
         begin
            while Curr_Node /= null and then
               Convert_Key (Curr_Node.Key) < Convert_Key (Key) loop
               Prev_Node := Curr_Node;
               Curr_Node := Curr_Node.Next;
            end loop;

            if (Curr_Node = null or else
               Convert_Key (Curr_Node.Key) /= Convert_Key (Key)) then
               Node_Count := Node_Count + 1;
               New_Node   := new Hash_Node'(Node_Count, Key, Info, Curr_Node);

               if Prev_Node = null then
                  -- insert at head
                  Hash_Table (Hash_Index) := New_Node;
               else
                  Prev_Node.Next := New_Node;
               end if;
               return (Node_Count);
            else
               return (- Curr_Node.Node_Num);
            end if;
         end;
      end if;

   end Save_Rec;

   function Get_Rec (Key : in Key_Type) return Info_Type is
      Dummy : Info_Type;
   begin
      return Dummy;
   end Get_Rec;

end Hash_Table;
