--|$Log:	keymap.a,v $
--|Revision 8.0  91/01/17  10:37:29  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Key_Bindings
--
----------------------------------------------------------------------------
--
--
-- Abstract
--
-- The Key_Bindings package provides the data
-- structures and procedures needed to bind the
-- keyboard keys to both history and line editing
-- functions, and to user supplied functions.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------

package Key_Bindings is

   type String_Ptr is access String;

   type Commands is
      (Error,
       Unbound,
       Line_Separator,
       Interpret_String,
       -- the optional string tells what function to invoke
       History_Back,
       History_Forward,
       Move_Left,
       Move_Right,
       Beginning_Of_Line,
       End_of_Line,
       Start_Of_History,
       End_Of_History,
       Delete, -- deletes the char to the left of the current position
       Delete_This_Char,
       Show_History,
       Show_History_Limit,
       Set_History_Limit,
       Kill_Line,
       Quoted_Insert,
       Refresh_Current_Line,
       Refresh_Current_Line_and_Prompt,
       Rewrite_Current_Line,
       Rewrite_Current_Line_and_Prompt,
       Get_Current_Line,
       Get_Current_Character,
       Get_Current_Column,
       Insert_String,
       Self_Insert);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Make_Default_Bindings
   --
   --    Last Mod:    
   --    Author:      R. Pollack
   --
   --  <Description>
   --
   -- This procedure establishes a basic set of bindings to enable the user
   -- to take advantage of the history and line editing features of
   -- Sanctified_Input.
   --  --------------------------------------------------------------------
   procedure Make_Default_Bindings;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Top_Binding
   --
   --    Last Mod:    
   --    Author:      R. Pollack
   --
   --  <Description>
   --
   --    This procedure is used to find a key binding when there is a
   --    requirement that the escape sequence be only one character long.
   --  --------------------------------------------------------------------
   function Top_Binding (C : in Character) return Commands;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Find_Binding
   --
   --    Last Mod:    
   --    Author:      C. Smotherman
   --
   --  <Description>
   --
   --    This procedure traces a character sequence through the arrays until
   --    it comes to the entry which identifies the command.
   --  --------------------------------------------------------------------
   procedure Find_Binding (First_Char      : in Character;
                           Binding         : out Commands;
                           Optional_String : out String_Ptr);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Make_Binding
   --
   --    Last Mod:    
   --    Author:    R. Pollack  
   --    Art:       C. Smotherman
   --
   --  <Description>
   --    
   --    This procedure comes in two flavors; the first
   --    simply allows the user to make the call with a
   --    character as the first argument; the second
   --    procedure does the work.
   --    
   --    This procedure traces a character sequence through
   --    a series of arrays; for the earlier characters in
   --    the sequence, if needed, it creates a new array
   --    and puts a pointer to the new array in the
   --    previous array, but if the needed array is already
   --    there, just proceeds to the next character in the
   --    sequence. For the last character it creates an
   --    array if necessary and writes the command name in
   --    the proper entry. There is a requirement that no
   --    character sequence used for the first argument be
   --    a prefix of another character sequence, used in a
   --    different Make_Binding call. If the user in the
   --    example below had chosen to make the last call:
   --    
   --        Make_Binding( "ac", move);
   --    
   --    they would have caused the binding for "aca" to be
   --    lost.
   --    
   --    The picture below is a shrunk down example of the
   --    arrays that would be built by three calls to
   --    Make_Binding:
   --    
   --          	 Make_Binding( "aab", insert);
   --            Make_Binding( "aca", delete);
   --            Make_Binding( "ca", move);
   --    
   --   -----     _____            _____
   --   | a |---> | a |----------> | a |
   --   |   |     |   |            | b |- Command = insert
   --   |   |     | b |            | c |
   --   | b |     |   |       ____ -----
   --   |   |     | c |-----> | a |- Command = delete
   --   |   |     -----       | b |
   --   |   |     _____       | c |
   --   | c |---> | a |       -----
   --   -----     | b |- Command = move
   --             | c |
   --             -----
   --  --------------------------------------------------------------------
   procedure Make_Binding (C               : in Character;
                           Binding         : in Commands;
                           Optional_String : in String := "");

   procedure Make_Binding (Char_Seq        : in String;
                           Binding         : in Commands;
                           Optional_String : in String := "");

end Key_Bindings;
