--|$Log:	sinput.a,v $
--|Revision 8.0  91/01/17  10:37:33  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Sanctified_Input
--
---------------------------------------------------------------
--
--
-- Abstract
--
-- This package accepts user input, line by line, and keeps a record of
-- the text of the lines and their order, and provides commands to manipulate
-- this body of lines and allow specified lines to be reissued without
-- being retyped.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------


with Text_Io;
with Tty_Operations;

generic
   with procedure Interpret_String (Command : String);
   with function Prompt return String;

package Sanctified_Input is

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Line, et al.
   --
   --    Last Mod:    Wed Apr 11 22:26:15 1990
   --    Author:      Cathy Smotherman
   --
   --  <Description>
   --
   --   Terminal_Reset is a function which restores the
   --   terminal to its normal state.  It should be called
   --   by the exception handler of a program using
   --   Sanctified_Input's Get_Line. This is because the
   --   Tty_Operations routines that Get_Line uses put the
   --   termninal into some altered states (raw, noecho
   --   and such). The terminal is automatically reset
   --   when Get_Line returns successfully.
   --
   --  --------------------------------------------------------------------
   procedure Get_Line (File : in Text_Io.File_Type;
                       Item : out String;
                       Last : out Natural);
   function  ask_for_integer (request : in string) return integer;
   procedure Kill_Line;
   procedure Get_Current_Line (Line   : out String;
                               Length : out Natural);
   function  Get_Current_Line return String;
   function  Get_Current_Column return Natural;
   function  Get_Current_Character return Character;
   procedure Do_History_Back;
   procedure Do_History_Forward;
   procedure Goto_Beg_Of_Line;
   procedure Goto_End_Of_Line;
   procedure Show_History;
   procedure Goto_Start_Of_History;
   procedure Goto_End_Of_History;
   procedure Move_Left;
   procedure Move_Right;
   procedure Delete;
   procedure Delete_This_Char;
   procedure Set_History_Limit;
   procedure Show_History_Limit;
   procedure Quoted_Insert;
   procedure Refresh_Current_Line;
   procedure Refresh_Current_Line_And_Prompt;
   procedure Rewrite_Current_Line (Str : in String);
   procedure Rewrite_Current_Line_And_Prompt (Str : in String);
   procedure Insert_String (The_String : in String);
   procedure Terminal_Reset renames Tty_Operations.Terminal_Reset;
   procedure Terminal_Set renames Tty_Operations.Terminal_Set;
   function  Maximum_Line_Length return Natural;
end Sanctified_Input;
