--|$Log:	unix_types_.a,v $
--|Revision 8.0  91/01/17  10:40:28  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Unix_Types
--
---------------------------------------------------------------
--
--
-- Abstract
--
--  Simple package that declares several types common to unix and
--  operations to convert to appropriate Ada types.
--
-- Update Log - : 
--
-- End of Header
----------------------------------------------------------------------------
with Unchecked_Conversion;
with System;

package body Unix_Types is

   --==========================================================================
   -- <Subprogram Name>
   --
   -- Convert_To_Cstring
   --
   -- <Description>
   --
   -- Conversion from an Ada String to a Unix String.
   --
   --==========================================================================
   function Convert_To_Cstring (Str : String) return Cstring is

      function Convert_To_Cstring is new Unchecked_Conversion
          (System.Address, Cstring);

      type Allocated_String is access String;
      Allocation      : Allocated_String := new String'(Str & Ascii.Nul);
      First_Character : Natural          := Allocation.all'FIRST;

   begin
      return Convert_To_Cstring (Allocation (First_Character)'ADDRESS);
   end Convert_To_Cstring;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Convert_To_Astring
   --
   --    Last Mod:    Tue Nov 28 17:05:42 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --   converts a Cstring into and Ada string (It tries to get around
   --   dope vectors by using a constrained array rather then an unconstrained
   --   array).
   --
   --  --------------------------------------------------------------------
   function Convert_To_Astring (Str : Cstring) return String is

      function Cstrlen (Str : Cstring) return Positive;
      pragma INTERFACE (C, Cstrlen);
      pragma INTERFACE_NAME (Cstrlen, "_strlen");

      subtype Constrained_String is String (1 .. Cstrlen (Str));
      type Access_Constrained_String is access Constrained_String;

      function Convert_To_Constrained_String is new Unchecked_Conversion
          (Cstring, Access_Constrained_String);
   begin
      return Convert_To_Constrained_String (Str).all;
   exception
      when others =>
         return ""; -- null Cstring
   end Convert_To_Astring;

end Unix_Types;
