--|$Log:	widget_.a,v $
--|Revision 8.0  91/01/17  10:40:29  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Widget_Package
--
----------------------------------------------------------------------------
--
--
-- Abstract
--
-- This package offers a sample selection of Xt toolkit procedures,
-- implemented as calls to the Xt object library.  Implementers
-- should feel free to add as many more as they wish, following the
-- pattern laid down here.
--
-- This package will eventually be replaced by the STARS Ada
-- implementation of the Xt toolkit.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
with Unchecked_Conversion;
with String_Encapsulation;
use String_Encapsulation;
with Xt_Stringdefs;
use Xt_Stringdefs;
with X_Windows_Interface;
use X_Windows_Interface;
with Text_Io;
use Text_Io;
with Os_Dependent;
with Compiler_Dependent;

package body Widget_Package is

   use System;
   use Intrinsics;
   use X_Windows;
   use Renamed_Xlib_Types;
   use X_Windows.Resource_Manager;
   use X_Windows.Fonts;
   use X_Windows.Events;
   use Intrinsics.Xt_Ancillary_Types;

   -- TeleSoft doesn't always handle Unchecked_Conversion properly
   -- if Unchecked_Conversion is called as part of an actual parameter

   -- function To_Caddr_T is new Unchecked_Conversion (Cardinal, Caddr_T);
   -- function To_Caddr_T is new Unchecked_Conversion (C_String, Caddr_T);
   -- function To_Caddr_T is new Unchecked_Conversion (Long, Caddr_T);
   -- function To_Event is new Unchecked_Conversion (Address, Event);
   -- function To_Address is new Unchecked_Conversion (Event, Address);

   function To_Caddr_T (Cardinal_Parameter : in Cardinal) return Caddr_T;
   function To_Caddr_T (C_String_Parameter : in C_String) return Caddr_T;
   function To_Caddr_T (Long_Parameter : in Long) return Caddr_T;
   function To_Event (Address_Parameter : in Address) return Event;
   function To_Address (Event_Parameter : in Event) return Address;

   C_Event_Record : String (1 .. 96);

   -- Be sure this is as big as the largest event structure
   -- It is o.k. as of 5/15/90 (rhp)

   type Arg_Type is
      record
         Name  : Address;
         Value : Caddr_T;
      end record;
   type Arg_Array is array (Integer range <>) of Arg_Type;

   type Argv_Array is array (Cardinal range <>) of Address;

   function Set_Font_Values (Font_Info : Font_Record) return X_Font_Record;
   function Set_Font_Values (X_Font_Info : X_Font_Record) return Font_Record;

   --=========================================================================
   --  <Subprogram Name>
   --    Make_Xt_String
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Converts Ada strings to strings suitable for the Xt toolkit.
   --
   --=========================================================================
   function Make_Xt_String (S : in String) return Xt_String is
      Local_Xt_String : Xt_String;
   begin
      Local_Xt_String     := new String (S'RANGE);
      Local_Xt_String.all := S;
      return Local_Xt_String;
   end Make_Xt_String;

   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Initialize
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the Xt toolkit routine XtInitialize.
   --
   --=========================================================================
   procedure Xt_Initialize (Out_Value : out Widget;
                            Name      : in String;
                            Classname : in String;
                            Urlist    : in Xrm_Option_Desc_List;
                            Num_Urs   : in Cardinal;
                            Argc      : in out Cardinal;
                            Argv      : in out Argv_Type) is

      Local_Argv : Argv_Array (0 .. Argc);

      function C_Xt_Initialize (Name      : in Address;
                                Classname : in Address;
                                Urlist    : in Address;
                                Num_Urs   : in Cardinal;
                                Argc      : in Address;
                                Argv      : in Address) return Address;

      pragma INTERFACE (C, C_Xt_Initialize);
      pragma INTERFACE_NAME (C_Xt_Initialize, "_XtInitialize");

      Local_Name   : Address;
      Local_Urlist : Address;

   begin
      if Name = "" then
         Local_Name := Compiler_Dependent.Null_Address;
      else
         Local_Name := Name'ADDRESS;
      end if;

      if Num_Urs = 0 or else Urlist = Null_Xrm_Option_List then
         Local_Urlist := Compiler_Dependent.Null_Address;
      else
         Local_Urlist := Urlist'ADDRESS;
      end if;

      for I in Argv.all'RANGE loop
         Local_Argv (Cardinal (I)) := Address (Make_C_String (Argv (I).all));
      end loop;
      Local_Argv (Local_Argv'LAST) := Compiler_Dependent.Null_Address;

      Out_Value := Widget (C_Xt_Initialize (Name'ADDRESS, Classname'ADDRESS,
                                            Urlist'ADDRESS, Num_Urs,
                                            Argc'ADDRESS, Local_Argv'ADDRESS));

      declare
         Argv_Out_Array : String_Pointer_Array (0 .. Integer (Argc) - 1);
      begin
         for I in 0 .. Integer (Argc) - 1 loop
            Argv_Out_Array (I) :=
               new String'(Make_Ada_String (Local_Argv (Cardinal (I))));
         end loop;
         Argv := new String_Pointer_Array'(Argv_Out_Array);
      end;
   end Xt_Initialize;

   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Set_Arg
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   -- This set of overloaded procedures provides a strongly typed,
   -- Ada version of the C macro XtSetArg.
   --
   -- Current overloading permits the Value parameter to be one
   -- of the following types:
   --
   --       Boolean
   --       String
   --       Integer
   --       Address
   --       Dimension
   --       Position
   --
   --=========================================================================
   procedure Xt_Set_Arg (Arg   : out Xt_Arg;
                         Name  : in String;
                         Value : in Boolean) is
   begin
      Arg.Name := Make_Xt_String (Name);
      if Value then
         Arg.Value := To_Caddr_T (Cardinal'(1));
      else
         Arg.Value := To_Caddr_T (Cardinal'(0));
      end if;
   end Xt_Set_Arg;

   procedure Xt_Set_Arg (Arg   : out Xt_Arg;
                         Name  : in String;
                         Value : in String) is
   begin
      Arg.Name  := Make_Xt_String (Name);
      Arg.Value := To_Caddr_T (Make_C_String (Value));
   end Xt_Set_Arg;

   procedure Xt_Set_Arg (Arg   : out Xt_Arg;
                         Name  : in String;
                         Value : in Integer) is
   begin
      Arg.Name  := Make_Xt_String (Name);
      Arg.Value := To_Caddr_T (Long (Value));
   end Xt_Set_Arg;

   procedure Xt_Set_Arg (Arg   : out Xt_Arg;
                         Name  : in String;
                         Value : in Address) is
   begin
      Arg.Name  := Make_Xt_String (Name);
      Arg.Value := Caddr_T (Value);
   end Xt_Set_Arg;

   procedure Xt_Set_Arg (Arg   : out Xt_Arg;
                         Name  : in String;
                         Value : in Dimension) is
   begin
      Arg.Name  := Make_Xt_String (Name);
      Arg.Value := To_Caddr_T (Cardinal (Value));
   end Xt_Set_Arg;

   procedure Xt_Set_Arg (Arg   : out Xt_Arg;
                         Name  : in String;
                         Value : in Position) is
   begin
      Arg.Name  := Make_Xt_String (Name);
      Arg.Value := To_Caddr_T (Cardinal (Value));
   end Xt_Set_Arg;

   procedure Xt_Set_Arg (Arg   : out Xt_Arg;
                         Name  : in String;
                         Value : in Gravity_Type) is
   begin
      Arg.Name  := Make_Xt_String (Name);
      Arg.Value := To_Caddr_T (Cardinal (Gravity_Type'POS (Value)));
   end Xt_Set_Arg;

   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Set_Values
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the toolkit procedure XtSetValues.
   --
   --=========================================================================
   procedure Xt_Set_Values (W        : in Widget;
                            Arg_List : in Xt_Arg_List) is

      Local_Args : Arg_Array (Arg_List'RANGE);

      procedure C_Xt_Set_Values (W        : in Address;
                                 Arg_List : in Address;
                                 N        : in Long);
      pragma INTERFACE (C, C_Xt_Set_Values);
      pragma INTERFACE_NAME (C_Xt_Set_Values, "_XtSetValues");

   begin
      for I in Arg_List'RANGE loop
         Local_Args (I).Name := Address (Make_C_String (Arg_List (I).Name.all));
         Local_Args (I).Value := Arg_List (I).Value;
      end loop;

      C_Xt_Set_Values (Address (W), Local_Args'ADDRESS, Local_Args'LENGTH);

      for I in Arg_List'RANGE loop
         Free (C_String (Local_Args (I).Name));

         -- We should also free some of the value fields,
         -- but how can we know which ones?

      end loop;
   end Xt_Set_Values;

   procedure C_Xt_Get_Values (W        : in Address;
                              Arg_List : in Address;
                              N        : in Long);
   pragma INTERFACE (C, C_Xt_Get_Values);
   pragma INTERFACE_NAME (C_Xt_Get_Values, "_XtGetValues");

   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Get_Value
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   -- This set of overloaded functions provides a strongly typed,
   -- Ada alternative to the Xt toolkit procedure XtGetValues.
   -- Since the toolkit procedure returns a list of heterogeneous
   -- elements, it cannot be easily modelled in Ada.  Moreover,
   -- a true modelling would not take advantage of Ada's type checking.
   -- Therefore, there is a function to return each type of resource.
   --
   -- Current overloading permits the returned value to be one
   -- of the following types:
   --
   --       Boolean
   --       Dimension
   --       Font_Record
   --
   --=========================================================================
   function Xt_Get_Value (W    : in Widget;
                          Name : in String) return Boolean is

      Local_Arg   : Arg_Type;
      Local_Value : Integer;

   begin
      Local_Arg.Name  := Address (Make_C_String (Name));
      Local_Arg.Value := Caddr_T (Local_Value'ADDRESS);

      C_Xt_Get_Values (Address (W), Local_Arg'ADDRESS, 1);

      Free (C_String (Local_Arg.Name));

      if Local_Value = 0 then
         return False;
      else
         return True;
      end if;
   end Xt_Get_Value;

   function Xt_Get_Value (W    : in Widget;
                          Name : in String) return Dimension is

      Local_Arg   : Arg_Type;
      Local_Value : Dimension;

   begin
      Local_Arg.Name  := Address (Make_C_String (Name));
      Local_Arg.Value := Caddr_T (Local_Value'ADDRESS);

      C_Xt_Get_Values (Address (W), Local_Arg'ADDRESS, 1);

      Free (C_String (Local_Arg.Name));

      return Local_Value;
   end Xt_Get_Value;

   function Xt_Get_Value (W    : in Widget;
                          Name : in String) return Font_Record is

      Local_Arg   : Arg_Type;
      Local_Value : X_Font_Record_Pointer;

   begin
      Local_Arg.Name  := Address (Make_C_String (Name));
      Local_Arg.Value := Caddr_T (Local_Value'ADDRESS);

      C_Xt_Get_Values (Address (W), Local_Arg'ADDRESS, 1);

      Free (C_String (Local_Arg.Name));

      return Set_Font_Values (Local_Value.all);
   end Xt_Get_Value;

   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Add_Callback
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   -- This set of overloaded procedures provides a strongly typed,
   -- Ada interface to the XtAddCallback toolkit procedure.
   --
   -- Current overloading permits the Closure parameter to be one
   -- of the following types:
   --
   --       Address
   --       Integer
   --
   --=========================================================================
   procedure Xt_Add_Callback (W             : in Widget;
                              Name          : in String;
                              Callback_Proc : in Address;
                              Closure       : in Address) is

      procedure C_Xt_Add_Callback (W             : in Address;
                                   Name          : in Address;
                                   Callback_Proc : in Address;
                                   Closure       : in Address);
      pragma INTERFACE (C, C_Xt_Add_Callback);
      pragma INTERFACE_NAME (C_Xt_Add_Callback, "_XtAddCallback");

      Local_C_String : C_String;

   begin
      Local_C_String := Make_C_String (Name);
      C_Xt_Add_Callback (Address (W), Address (Local_C_String), Callback_Proc,
                         Closure);
      Free (Local_C_String);
   end Xt_Add_Callback;

   procedure Xt_Add_Callback (W             : in Widget;
                              Name          : in String;
                              Callback_Proc : in Address;
                              Closure       : in Integer) is

      procedure C_Xt_Add_Callback (W             : in Address;
                                   Name          : in Address;
                                   Callback_Proc : in Address;
                                   Closure       : in Long);
      pragma INTERFACE (C, C_Xt_Add_Callback);
      pragma INTERFACE_NAME (C_Xt_Add_Callback, "_XtAddCallback");

      Local_C_String : C_String;

   begin
      Local_C_String := Make_C_String (Name);
      C_Xt_Add_Callback (Address (W), Address (Local_C_String), Callback_Proc,
                         Long (Closure));
      Free (Local_C_String);
   end Xt_Add_Callback;

   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Create_Widget
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the Xt toolkit routine XtCreateWidget.
   --
   --=========================================================================
   procedure Xt_Create_Widget (Out_Value   : out Widget;
                               Name        : in String;
                               Widgetclass : in Widget_Class;
                               Parent      : in Widget;
                               Arg_List    : in Xt_Arg_List) is

      function C_Xt_Create_Widget (Name        : in Address;
                                   Widgetclass : in Address;
                                   Parent      : in Address;
                                   Arg_List    : in Address;
                                   N           : in Long) return Address;
      pragma INTERFACE (C, C_Xt_Create_Widget);
      pragma INTERFACE_NAME (C_Xt_Create_Widget, "_XtCreateWidget");

      Local_Name         : Address;
      Local_Args         : Arg_Array (Arg_List'RANGE);
      Local_Args_Address : Address;
      Local_N            : Long;

   begin
      if Name = "" then
         Local_Name := Compiler_Dependent.Null_Address;
      else
         Local_Name := Name'ADDRESS;
      end if;

      if Arg_List = Null_Xt_Arg_List then
         Local_Args_Address := Compiler_Dependent.Null_Address;
         Local_N            := 0;
      else
         Local_Args_Address := Local_Args'ADDRESS;
         Local_N            := Arg_List'LENGTH;
         for I in Arg_List'RANGE loop
            Local_Args (I).Name :=
               Address (Make_C_String (Arg_List (I).Name.all));
            Local_Args (I).Value := Arg_List (I).Value;
         end loop;
      end if;

      Out_Value := Widget (C_Xt_Create_Widget (Local_Name,
                                               Address (Widgetclass),
                                               Address (Parent),
                                               Local_Args_Address, Local_N));

      for I in Arg_List'RANGE loop
         Free (C_String (Local_Args (I).Name));

         -- We should also free some of the value fields,
         -- but how can we know which ones?

      end loop;
   end Xt_Create_Widget;

   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Create_Managed_Widget
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the Xt toolkit routine XtCreateManagedWidget.
   --
   --=========================================================================
   procedure Xt_Create_Managed_Widget (Out_Value   : out Widget;
                                       Name        : in String;
                                       Widgetclass : in Widget_Class;
                                       Parent      : in Widget;
                                       Arg_List    : in Xt_Arg_List) is

      function C_Xt_Create_Managed_Widget
          (Name        : in Address;
           Widgetclass : in Address;
           Parent      : in Address;
           Arg_List    : in Address;
           N           : in Long) return Address;
      pragma INTERFACE (C, C_Xt_Create_Managed_Widget);
      pragma INTERFACE_NAME (C_Xt_Create_Managed_Widget,
                             "_XtCreateManagedWidget");

      Local_Name         : Address;
      Local_Args         : Arg_Array (Arg_List'RANGE);
      Local_Args_Address : Address;
      Local_N            : Long;

   begin
      if Name = "" then
         Local_Name := Compiler_Dependent.Null_Address;
      else
         Local_Name := Name'ADDRESS;
      end if;

      if Arg_List = Null_Xt_Arg_List then
         Local_Args_Address := Compiler_Dependent.Null_Address;
         Local_N            := 0;
      else
         Local_Args_Address := Local_Args'ADDRESS;
         Local_N            := Local_Args'LENGTH;
         for I in Arg_List'RANGE loop
            Local_Args (I).Name :=
               Address (Make_C_String (Arg_List (I).Name.all));
            Local_Args (I).Value := Arg_List (I).Value;
         end loop;
      end if;

      Out_Value := Widget (C_Xt_Create_Managed_Widget (Local_Name,
                                                       Address (Widgetclass),
                                                       Address (Parent),
                                                       Local_Args_Address,
                                                       Local_N));

      for I in Arg_List'RANGE loop
         Free (C_String (Local_Args (I).Name));

         -- We should also free some of the value fields,
         -- but how can we know which ones?

      end loop;
   end Xt_Create_Managed_Widget;

   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Main_Loop
   --
   --    Last Mod:    Thu May 24 11:16:56 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the Xt toolkit routine XtMainLoop.
   --
   --=========================================================================
   procedure Xt_Main_Loop is

      procedure C_Xt_Main_Loop;
      pragma INTERFACE (C, C_Xt_Main_Loop);
      pragma INTERFACE_NAME (C_Xt_Main_Loop, "_XtMainLoop");

   begin
      C_Xt_Main_Loop;
   end Xt_Main_Loop;

   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Realize_Widget
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the Xt toolkit routine XtRealizeWidget.
   --
   --=========================================================================
   procedure Xt_Realize_Widget (W : in Widget) is

      procedure C_Xt_Realize_Widget (W : in Address);
      pragma INTERFACE (C, C_Xt_Realize_Widget);
      pragma INTERFACE_NAME (C_Xt_Realize_Widget, "_XtRealizeWidget");

   begin
      C_Xt_Realize_Widget (Address (W));
   end Xt_Realize_Widget;

   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Default_App_Context
   --
   --    Last Mod:    Fri May 11 15:37:53 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the Xt toolkit routine _XtDefaultAppContext.
   --
   --=========================================================================
   function Xt_Default_App_Context return Xt_App_Context is

      function C_Xt_Default_App_Context return Address;
      pragma INTERFACE (C, C_Xt_Default_App_Context);
      pragma INTERFACE_NAME (C_Xt_Default_App_Context, "__XtDefaultAppContext");
   begin
      return Xt_App_Context (C_Xt_Default_App_Context);
   end Xt_Default_App_Context;
   --=========================================================================
   --  <Subprogram Name>
   --    Xt_App_Next_Event
   --
   --    Last Mod:    Fri May 11 15:37:53 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the Xt toolkit routine XtAppNextEvent.
   --
   --=========================================================================
   function Xt_App_Next_Event (App : in Xt_App_Context) return Event is

      procedure C_Xt_App_Next_Event (App   : in Address;
                                     Event : in Address);
      pragma INTERFACE (C, C_Xt_App_Next_Event);
      pragma INTERFACE_NAME (C_Xt_App_Next_Event, "_XtAppNextEvent");

   begin
      C_Xt_App_Next_Event (Address (App), C_Event_Record'ADDRESS);
      return To_Event (C_Event_Record'ADDRESS);
   end Xt_App_Next_Event;

   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Dispatch_Event
   --
   --    Last Mod:    Fri May 11 15:37:53 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the Xt toolkit routine XtDispatchEvent.
   --
   --=========================================================================
   procedure Xt_Dispatch_Event (E : in Event) is

      procedure C_Xt_Dispatch_Event (Event : in Address);
      pragma INTERFACE (C, C_Xt_Dispatch_Event);
      pragma INTERFACE_NAME (C_Xt_Dispatch_Event, "_XtDispatchEvent");

   begin
      C_Xt_Dispatch_Event (To_Address (E));
   end Xt_Dispatch_Event;

   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Destroy_Widget
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the Xt toolkit routine XtDestroyWidget.
   --
   --=========================================================================
   procedure Xt_Destroy_Widget (W : in Widget) is

      procedure C_Xt_Destroy_Widget (W : in Address);
      pragma INTERFACE (C, C_Xt_Destroy_Widget);
      pragma INTERFACE_NAME (C_Xt_Destroy_Widget, "_XtDestroyWidget");

   begin
      C_Xt_Destroy_Widget (Address (W));
   end Xt_Destroy_Widget;

   --=========================================================================
   --  <Subprogram Name>
   --    X_Text_Width
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the X library routine XTextWidth.
   --
   --=========================================================================
   function X_Text_Width (F : in Font_Record;
                          S : in String) return Dimension is

      function C_X_Text_Width (F   : in Address;
                               Str : in Address;
                               Lth : in Long) return Long;
      pragma INTERFACE (C, C_X_Text_Width);
      pragma INTERFACE_NAME (C_X_Text_Width, "_XTextWidth");

      Local_C_String      : C_String;
      Local_X_Font_Record : X_Font_Record;
      Return_Value        : Dimension;

   begin
      Local_C_String      := Make_C_String (S);
      Local_X_Font_Record := Set_Font_Values (F);
      Return_Value := Dimension (C_X_Text_Width (Local_X_Font_Record'ADDRESS,
                                                 Address (Local_C_String),
                                                 S'LENGTH));
      Free (Local_C_String);
      return Return_Value;
   end X_Text_Width;

   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Parse_Translation_Table
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the Xt toolkit routine XtParseTranslationTable.
   --
   --=========================================================================
   function Xt_Parse_Translation_Table (S : in String) return Xt_Translations
                                       is

      function C_Xt_Parse_Translation_Table (S : in Address) return Address;
      pragma INTERFACE (C, C_Xt_Parse_Translation_Table);
      pragma INTERFACE_NAME (C_Xt_Parse_Translation_Table,
                             "_XtParseTranslationTable");

      Local_C_String : C_String;
      Return_Value   : Xt_Translations;

   begin
      Local_C_String := Make_C_String (S);
      Return_Value   :=
         Xt_Translations (C_Xt_Parse_Translation_Table (Local_C_String));
      Free (Local_C_String);
      return Return_Value;
   end Xt_Parse_Translation_Table;

   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Override_Translations
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the Xt toolkit routine XtOverrideTranslations.
   --
   --=========================================================================
   procedure Xt_Override_Translations (W : in Widget;
                                       T : in Xt_Translations) is

      procedure C_Xt_Override_Translations (W : in Address;
                                            T : in Address);
      pragma INTERFACE (C, C_Xt_Override_Translations);
      pragma INTERFACE_NAME (C_Xt_Override_Translations,
                             "_XtOverrideTranslations");

   begin
      C_Xt_Override_Translations (Address (W), Address (T));
   end Xt_Override_Translations;

   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Augment_Translations
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the Xt toolkit routine XtAugmentTranslations.
   --
   --=========================================================================
   procedure Xt_Augment_Translations (W : in Widget;
                                      T : in Xt_Translations) is

      procedure C_Xt_Augment_Translations (W : in Address;
                                           T : in Address);
      pragma INTERFACE (C, C_Xt_Augment_Translations);
      pragma INTERFACE_NAME (C_Xt_Augment_Translations,
                             "_XtAugmentTranslations");

   begin
      C_Xt_Augment_Translations (Address (W), Address (T));
   end Xt_Augment_Translations;

   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Add_Actions
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the Xt toolkit routine XtAddActions.
   --
   --=========================================================================
   procedure Xt_Add_Actions (Arg_List : in Xt_Arg_List) is

      Local_Args : Arg_Array (Arg_List'RANGE);

      procedure C_Xt_Add_Actions (Arg_List : in Address;
                                  N        : in Long);
      pragma INTERFACE (C, C_Xt_Add_Actions);
      pragma INTERFACE_NAME (C_Xt_Add_Actions, "_XtAddActions");

   begin
      for I in Arg_List'RANGE loop
         Local_Args (I).Name := Address (Make_C_String (Arg_List (I).Name.all));
         Local_Args (I).Value := Arg_List (I).Value;
      end loop;

      C_Xt_Add_Actions (Local_Args'ADDRESS, Local_Args'LENGTH);

      for I in Arg_List'RANGE loop
         Free (C_String (Local_Args (I).Name));

         -- We should also free some of the value fields,
         -- but how can we know which ones?

      end loop;
   end Xt_Add_Actions;

   --=========================================================================
   --  <Subprogram Name>
   --    Set_Character_Record_Values
   --    Xbcopy
   --    Fix_X_Character_List
   --    Fix_Font_Property_List
   --    Set_Character_Array_Values
   --    Set_Character_Record_Values
   --    Set_Font_Values
   --    Set_Font_Values
   --
   --    Last Mod:    Thu Apr 19 14:59:58 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Copies of certain hidden parts of the Ada X libary bindings;
   --  these will not be needed when we are using the Ada Xt toolkit.
   --
   --=========================================================================

   function Set_Character_Record_Values
       (X_Char_Rec : X_Character_Record) return Character_Record is
   begin
      return (Left_Bearing  => Pixels (X_Char_Rec.Left_Bearing),
              Right_Bearing => Pixels (X_Char_Rec.Right_Bearing),
              Width         => Pixels (X_Char_Rec.Width),
              Ascent        => Pixels (X_Char_Rec.Ascent),
              Descent       => Pixels (X_Char_Rec.Descent),
              Attributes    => Pixels (X_Char_Rec.Attributes));
   end Set_Character_Record_Values;

   -- The procedure Bcopy is an external interface to the Bcopy routine
   -- supplied in the C library. It is used because we need a vehicle
   -- for copying bytes of information from the source to the
   -- newly made destination.

   procedure Xbcopy (Source : in System.Address;
                     Target : in System.Address;
                     Length : in Long);
   pragma INTERFACE (C, Xbcopy);

   function Fix_X_Character_List
       (Source : in X_Character_List;
        Length : in Natural;
        First  : in Natural := 1) return X_Character_List is

      New_List : X_Character_Array (First .. First + Length - 1);

   begin
      if Source = null then
         return null;
      else
         Xbcopy (Source => Source.all'ADDRESS,
                 Target => New_List (New_List'FIRST)'ADDRESS,
                 Length => (Length * (X_Character_Record'SIZE / 8)));
         return new X_Character_Array'(New_List);
      end if;
   end Fix_X_Character_List;

   function Fix_Font_Property_List
       (Source : in Font_Property_List;
        Length : in Natural;
        First  : in Natural := 1) return Font_Property_List is

      New_List : Font_Property_Array (First .. First + Length - 1);

   begin
      if Source = null then
         return null;
      else
         Xbcopy (Source => Source.all'ADDRESS,
                 Target => New_List (New_List'FIRST)'ADDRESS,
                 Length => (Length * (Font_Property_Record'SIZE / 8)));
         return new Font_Property_Array'(New_List);
      end if;
   end Fix_Font_Property_List;

   function Set_Character_Array_Values
       (X_Chars : X_Character_Array) return Character_Array is
      Chars : Character_Array (X_Chars'RANGE);
   begin
      for Each_Character in X_Chars'RANGE loop
         Chars (Each_Character) :=
            Set_Character_Record_Values (X_Chars (Each_Character));
      end loop;
      return Chars;
   end Set_Character_Array_Values;

   function Set_Character_Record_Values
       (Char_Rec : Character_Record) return X_Character_Record is
   begin
      return (Left_Bearing  => Short (Char_Rec.Left_Bearing),
              Right_Bearing => Short (Char_Rec.Right_Bearing),
              Width         => Short (Char_Rec.Width),
              Ascent        => Short (Char_Rec.Ascent),
              Descent       => Short (Char_Rec.Descent),
              Attributes    => Short (Char_Rec.Attributes));
   end Set_Character_Record_Values;

   function Set_Character_Array_Values
       (Chars : Character_Array) return X_Character_Array is
      X_Chars : X_Character_Array (Chars'RANGE);
   begin
      for Each_Character in Chars'RANGE loop
         X_Chars (Each_Character) :=
            Set_Character_Record_Values (Chars (Each_Character));
      end loop;
      return X_Chars;
   end Set_Character_Array_Values;

   function Set_Font_Values (Font_Info : Font_Record) return X_Font_Record is

      --| originally died if font_record per_char field was null
      --| added Per_char_field to test for null and set up the correct value
      --| RCS
      Per_Char_Field : X_Character_List; -- RCS
      function To_X_Character_List is new Unchecked_Conversion
          (Source => System.Address, Target => X_Character_List);

   begin
      if Font_Info.Per_Char = null then -- RCS
         Per_Char_Field := null; -- RCS
      else -- RCS
         Per_Char_Field :=  -- kcw
            new X_Character_Array'(
               Set_Character_Array_Values (Font_Info.Per_Char.all));
         Per_Char_Field :=
            To_X_Character_List (Per_Char_Field (Per_Char_Field'FIRST)'ADDRESS);
      end if;

      return
         (Ext_Data     => Font_Info.Ext_Data,
          Font_Id      => Font_Info.Font_Id,
          Direction    => Font_Direction'POS (Font_Info.Direction),
          First_Char   => Int (Font_Info.First_Char),
          Last_Char    => Int (Font_Info.Last_Char),
          First_Row    => Int (Font_Info.First_Row),
          Last_Row     => Int (Font_Info.Last_Row),
          All_Exist    => Boolean'POS (Font_Info.All_Exist),
          Default_Char => Int (Font_Info.Default_Char),
          N_Properties => Font_Info.Properties'LENGTH,
          Properties   => Font_Info.Properties,
          Min_Bounds   => Set_Character_Record_Values (Font_Info.Min_Bounds),
          Max_Bounds   => Set_Character_Record_Values (Font_Info.Max_Bounds),
          Per_Char     => Per_Char_Field,  -- RCS
          Ascent       => Int (Font_Info.Ascent),
          Descent      => Int (Font_Info.Descent));
   end Set_Font_Values;

   function Set_Font_Values (X_Font_Info : X_Font_Record) return Font_Record is

      Test_Font_Info : Font_Record;
      Temp_Per_Char  : X_Character_List;

      use Os_Dependent;

   begin
      if X_Font_Info.Per_Char /= null then

         Temp_Per_Char :=
            Fix_X_Character_List
                (Source => X_Font_Info.Per_Char,
                 Length => Natural (X_Font_Info.Last_Char -
                                    X_Font_Info.First_Char + 1),
                 First => Natural (X_Font_Info.First_Char));
      else
         Temp_Per_Char := null;
      end if;

      return
         (Ext_Data     => X_Font_Info.Ext_Data,
          Font_Id      => X_Font_Info.Font_Id,
          Direction    => Font_Direction'VAL (X_Font_Info.Direction),
          First_Char   => X_Character (X_Font_Info.First_Char),
          Last_Char    => X_Character (X_Font_Info.Last_Char),
          First_Row    => X_Character (X_Font_Info.First_Row),
          Last_Row     => X_Character (X_Font_Info.Last_Row),
          All_Exist    => Boolean'VAL (X_Font_Info.All_Exist),
          Default_Char => X_Character (X_Font_Info.Default_Char),
          Properties   =>
             Fix_Font_Property_List (X_Font_Info.Properties,
                                     Natural (X_Font_Info.N_Properties)),
          Min_Bounds => Set_Character_Record_Values (X_Font_Info.Min_Bounds),
          Max_Bounds => Set_Character_Record_Values (X_Font_Info.Max_Bounds),
          Per_Char   =>
             new Character_Array'(
                Set_Character_Array_Values (Temp_Per_Char.all)),
          Ascent  => Pixels (X_Font_Info.Ascent),
          Descent => Pixels (X_Font_Info.Descent));
   end Set_Font_Values;

   -- TeleSoft-compatible unchecked conversion functions

   function To_Caddr_T (Cardinal_Parameter : in Cardinal) return Caddr_T is

      Cardinal_Input : Cardinal;
      Caddr_T_Result : Caddr_T;
      function Convert_Cardinal is new Unchecked_Conversion (Cardinal, Caddr_T);

   begin
      Cardinal_Input := Cardinal_Parameter;
      Caddr_T_Result := Convert_Cardinal (Cardinal_Input);
      return Caddr_T_Result;
   end To_Caddr_T;

   function To_Caddr_T (C_String_Parameter : in C_String) return Caddr_T is

      C_String_Input : C_String;
      Caddr_T_Result : Caddr_T;
      function Convert_C_String is new Unchecked_Conversion (C_String, Caddr_T);

   begin
      C_String_Input := C_String_Parameter;
      Caddr_T_Result := Convert_C_String (C_String_Input);
      return Caddr_T_Result;
   end To_Caddr_T;

   function To_Caddr_T (Long_Parameter : in Long) return Caddr_T is

      Long_Input     : Long;
      Caddr_T_Result : Caddr_T;
      function Convert_Long is new Unchecked_Conversion (Long, Caddr_T);

   begin
      Long_Input     := Long_Parameter;
      Caddr_T_Result := Convert_Long (Long_Input);
      return Caddr_T_Result;
   end To_Caddr_T;

   function To_Event (Address_Parameter : in Address) return Event is

      Address_Input : Address;
      Event_Result  : Event;
      function Convert_Address is new Unchecked_Conversion (Address, Event);

   begin
      Address_Input := Address_Parameter;
      Event_Result  := Convert_Address (Address_Input);
      return Event_Result;
   end To_Event;

   function To_Address (Event_Parameter : in Event) return Address is

      Event_Input    : Event;
      Address_Result : Address;
      function Convert_Event is new Unchecked_Conversion (Event, Address);

   begin
      Event_Input    := Event_Parameter;
      Address_Result := Convert_Event (Event_Input);
      return Address_Result;
   end To_Address;

end Widget_Package;
