--|$Log:	widget.a,v $
--|Revision 8.0  91/01/17  11:04:15  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Widget_Package
--
----------------------------------------------------------------------------
--
--
-- Abstract
--
-- This package offers a sample selection of Xt toolkit procedures,
-- implemented as calls to the Xt object library.  Implementers
-- should feel free to add as many more as they wish, following the
-- pattern laid down here.
--
-- This package will eventually be replaced by the STARS Ada
-- implementation of the Xt toolkit.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
with System;
with Intrinsics;
with X_Windows;
with Renamed_Xlib_Types;

package Widget_Package is

   Null_Xrm_Option_List : X_Windows.Resource_Manager.Xrm_Option_Desc_List
    (1 .. 0);

   type Xt_Arg_List is
      array (Integer range <>) of Intrinsics.Xt_Ancillary_Types.Xt_Arg;
   Null_Xt_Arg_List : Xt_Arg_List (1 .. 0);

   type Xt_Translations is new System.Address;

   --=========================================================================
   --  <Subprogram Name>
   --    Make_Xt_String
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Converts Ada strings to strings suitable for the Xt toolkit.
   --
   --=========================================================================
   function Make_Xt_String (S : in String) return Renamed_Xlib_Types.Xt_String;

   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Initialize
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the Xt toolkit routine XtInitialize.
   --
   --=========================================================================
   procedure Xt_Initialize
       (Out_Value : out Intrinsics.Widget;
        Name      : in String;
        Classname : in String;
        Urlist    : in X_Windows.Resource_Manager.Xrm_Option_Desc_List;
        Num_Urs   : in Intrinsics.Cardinal;
        Argc      : in out Intrinsics.Cardinal;
        Argv      : in out Renamed_Xlib_Types.Argv_Type);

   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Set_Arg
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   -- This set of overloaded procedures provides a strongly typed,
   -- Ada version of the C macro XtSetArg.
   --
   -- Current overloading permits the Value parameter to be one
   -- of the following types:
   --
   --       Boolean
   --       String
   --       Integer
   --       Address
   --       Dimension
   --       Position
   --       Gravity_Type
   --
   --=========================================================================
   procedure Xt_Set_Arg (Arg   : out Intrinsics.Xt_Ancillary_Types.Xt_Arg;
                         Name  : in String;
                         Value : in Boolean);

   procedure Xt_Set_Arg (Arg   : out Intrinsics.Xt_Ancillary_Types.Xt_Arg;
                         Name  : in String;
                         Value : in String);

   procedure Xt_Set_Arg (Arg   : out Intrinsics.Xt_Ancillary_Types.Xt_Arg;
                         Name  : in String;
                         Value : in Integer);

   procedure Xt_Set_Arg (Arg   : out Intrinsics.Xt_Ancillary_Types.Xt_Arg;
                         Name  : in String;
                         Value : in System.Address);

   procedure Xt_Set_Arg (Arg   : out Intrinsics.Xt_Ancillary_Types.Xt_Arg;
                         Name  : in String;
                         Value : in Intrinsics.Dimension);

   procedure Xt_Set_Arg (Arg   : out Intrinsics.Xt_Ancillary_Types.Xt_Arg;
                         Name  : in String;
                         Value : in Intrinsics.Position);

   procedure Xt_Set_Arg (Arg   : out Intrinsics.Xt_Ancillary_Types.Xt_Arg;
                         Name  : in String;
                         Value : in X_Windows.Gravity_Type);

   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Set_Values
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the toolkit procedure XtSetValues.
   --
   --=========================================================================
   procedure Xt_Set_Values (W        : in Intrinsics.Widget;
                            Arg_List : in Xt_Arg_List);

   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Get_Value
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   -- This set of overloaded functions provides a strongly typed,
   -- Ada alternative to the Xt toolkit procedure XtGetValues.
   -- Since the toolkit procedure returns a list of heterogeneous
   -- elements, it cannot be easily modelled in Ada.  Moreover,
   -- a true modelling would not take advantage of Ada's type checking.
   -- Therefore, there is a function to return each type of resource.
   --
   -- Current overloading permits the returned value to be one
   -- of the following types:
   --
   --       Boolean
   --       Dimension
   --       Font_Record_Pointer
   --
   --=========================================================================
   function Xt_Get_Value (W    : in Intrinsics.Widget;
                          Name : in String) return Boolean;

   function Xt_Get_Value (W    : in Intrinsics.Widget;
                          Name : in String) return Intrinsics.Dimension;

   function Xt_Get_Value (W    : in Intrinsics.Widget;
                          Name : in String) return X_Windows.Fonts.Font_Record;

   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Add_Callback
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   -- This set of overloaded procedures provides a strongly typed,
   -- Ada interface to the XtAddCallback toolkit procedure.
   --
   -- Current overloading permits the Closure parameter to be one
   -- of the following types:
   --
   --       Address
   --       Integer
   --
   --=========================================================================
   procedure Xt_Add_Callback (W             : in Intrinsics.Widget;
                              Name          : in String;
                              Callback_Proc : in System.Address;
                              Closure       : in System.Address);

   procedure Xt_Add_Callback (W             : in Intrinsics.Widget;
                              Name          : in String;
                              Callback_Proc : in System.Address;
                              Closure       : in Integer);

   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Create_Widget
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the Xt toolkit routine XtCreateWidget.
   --
   --=========================================================================
   procedure Xt_Create_Widget (Out_Value   : out Intrinsics.Widget;
                               Name        : in String;
                               Widgetclass : in Intrinsics.Widget_Class;
                               Parent      : in Intrinsics.Widget;
                               Arg_List    : in Xt_Arg_List);

   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Create_Managed_Widget
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the Xt toolkit routine XtCreateManagedWidget.
   --
   --=========================================================================
   procedure Xt_Create_Managed_Widget (Out_Value   : out Intrinsics.Widget;
                                       Name        : in String;
                                       Widgetclass : in Intrinsics.Widget_Class;
                                       Parent      : in Intrinsics.Widget;
                                       Arg_List    : in Xt_Arg_List);

   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Realize_Widget
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the Xt toolkit routine XtRealizeWidget.
   --
   --=========================================================================
   procedure Xt_Realize_Widget (W : in Intrinsics.Widget);

   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Main_Loop
   --
   --    Last Mod:    Thu May 24 11:16:56 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the Xt toolkit routine XtMainLoop.
   --
   --=========================================================================
   procedure Xt_Main_Loop;
   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Default_App_Context
   --
   --    Last Mod:    Fri May 11 15:37:53 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the Xt toolkit routine _XtDefaultAppContext.
   --
   --=========================================================================
   function Xt_Default_App_Context return Intrinsics.Xt_App_Context;
   --=========================================================================
   --  <Subprogram Name>
   --    Xt_App_Next_Event
   --
   --    Last Mod:    Tue May 15 16:47:02 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the Xt toolkit routine XtAppNextEvent.
   --
   --=========================================================================
   function Xt_App_Next_Event
       (App : in Intrinsics.Xt_App_Context) return
         X_Windows.Events.Event;
   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Dispatch_Event
   --
   --    Last Mod:    Tue May 15 16:47:12 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the Xt toolkit routine XtDispatchEvent.
   --
   --=========================================================================
   procedure Xt_Dispatch_Event (E : in X_Windows.Events.Event);
   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Destroy_Widget
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the Xt toolkit routine XtDestroyWidget.
   --
   --=========================================================================
   procedure Xt_Destroy_Widget (W : in Intrinsics.Widget);
   --=========================================================================
   --  <Subprogram Name>
   --    X_Text_Width
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the X library routine XTextWidth.
   --
   --=========================================================================
   function X_Text_Width (F : in X_Windows.Fonts.Font_Record;
                          S : in String) return Intrinsics.Dimension;
   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Parse_Translation_Table
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the Xt toolkit routine XtParseTranslationTable.
   --
   --=========================================================================
   function Xt_Parse_Translation_Table (S : in String) return Xt_Translations;
   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Override_Translations
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the Xt toolkit routine XtOverrideTranslations.
   --
   --=========================================================================
   procedure Xt_Override_Translations (W : in Intrinsics.Widget;
                                       T : in Xt_Translations);
   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Augment_Translations
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the Xt toolkit routine XtAugmentTranslations.
   --
   --=========================================================================
   procedure Xt_Augment_Translations (W : in Intrinsics.Widget;
                                      T : in Xt_Translations);

   --=========================================================================
   --  <Subprogram Name>
   --    Xt_Add_Actions
   --
   --    Last Mod:    Wed Apr 18 15:33:19 EDT 1990 (rhp)
   --    Author:      Robert H. Pollack
   --
   --  <Description>
   --
   --  Ada version of the Xt toolkit routine XtAddActions.
   --
   --=========================================================================
   procedure Xt_Add_Actions (Arg_List : in Xt_Arg_List);
end Widget_Package;
