--|$Log:	ace_adt.a,v $
--|Revision 8.0  91/01/17  10:40:47  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Ace_Adt
--
---------------------------------------------------------------
--
--
-- Abstract
--
--   This ADT provides a programmatic interface to the ACLI, allowing
--   user to control interpretation of ACE constructs.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------

with Ace_Text_Io;
use Ace_Text_Io;
with Statement_Database;
use Statement_Database;

with Context_Control;

with System;

package Ace_Adt is

   type Ace_Statement_Database is new System.Address;


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Compile
   --
   --  <Description>
   --
   --    procedure Compile will take a string or file containing Ada code
   --  and "compile" it into the ACE internal form.  A Tree cross reference
   --  in the statement database will be returned that can be given to the
   --  appropriate interpret routine.
   --
   --  <Called By>
   --
   --  External, and Internal
   --
   --  <Exceptions Raised>
   --
   --  None.
   --
   --  <Exceptions Handled>
   --
   --  No exception is allowed to propagate from these routines.
   --  The Error flag is set if any exceptions were handled.
   --
   --==========================================================================
   procedure Compile (Context : in out Context_Control.Context_Definition;
                      Str     : in String;
                      Tree    : in out Ace_Statement_Database;
                      Error   : out Boolean);

   procedure Compile (Context : in out Context_Control.Context_Definition;
                      File    : in File_Type;
                      Tree    : in out Ace_Statement_Database;
                      Error   : out Boolean);


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Delete
   --
   --  <Description>
   --
   --    Deletes a Ace_Statement_Database Tree.
   --
   --  <Called By>
   --
   --    External and Internal
   --
   --  <Exceptions Raised>
   --
   --    [None]
   --
   --  <Exceptions Handled>
   --
   --    [None]
   --==========================================================================
   procedure Delete (Tree : in Ace_Statement_Database);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Interpret
   --
   --  <Description>
   --
   --    Interpret will take a statement database reference, string, or file
   --  and interpret its contents.  These routines can be said to
   --  programatically create any object in the currently executing
   --  environment.  Each of the following routines operate on a different
   --  type and allow the user to test for or ignore errors.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    [none]
   --
   --  <Exceptions Handled>
   --
   --    No exception is allowed to propagate from these routines.
   --    The Error flag is set if any exceptions were handled.
   --==========================================================================


   procedure Interpret (Context : in out Context_Control.Context_Definition;
                        Tree    : in Ref_Statement_Database;
                        Error   : out Boolean);

   procedure Interpret (Context : in out Context_Control.Context_Definition;
                        Tree    : in Ace_Statement_Database;
                        Error   : out Boolean);

   procedure Interpret (Context : in out Context_Control.Context_Definition;
                        Str     : in String;
                        Error   : out Boolean);

   procedure Interpret (Context : in out Context_Control.Context_Definition;
                        File    : in File_Type;
                        Error   : out Boolean);


   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Interpret_File
   --
   --    Last Mod:    Fri Nov 10 11:24:45 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --     Interpret_File is now defined in the Initialize_Symtbl routine
   --     so that startup files can interpret a different set of routines.
   --
   --  --------------------------------------------------------------------
   procedure Interpret_File
       (Context   : in out Context_Control.Context_Definition;
        File_Name : in String);

end Ace_Adt;
