--|$Log:	astd.a,v $
--|Revision 8.0  91/01/17  10:40:55  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Ace_Standard
--
---------------------------------------------------------------
--
--
-- Abstract
--
--    This package defines the standard environment manipulation operations
--    that are applicable to objects defined within ACE.
--    The user is automatically given access to these operations
--    when the ACE system elaborates itself.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------

with Literals;
use Literals;

with Io_Exceptions;

with Context_Control;

package Ace_Standard is

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Delete
   --
   --  <Description>
   --
   --    This procedure deletes the named ACE object.  The deletion is
   --    accomplished by marking the symbol table entry associated with the
   --    object as "Removed".
   --
   --  <Exceptions Raised>
   --
   --    Name_Error is raised if no object exists with the given name
   --
   --==========================================================================

   procedure Delete (Context     : in out Context_Control.Context_Definition;
                     Object_Name : in Ref_String;
                     Form        : String);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Defuse
   --
   --  <Description>
   --
   --    This procedure deletes an ACE subprogram object.  The deletion is
   --    accomplished by marking the symbol table entry associated with the
   --    object as "Removed".
   --
   --  <Exceptions Raised>
   --
   --    Name_Error is raised if the object does not exist or the object is
   --      not a package.
   --
   --==========================================================================

   procedure Defuse (Context      : in out Context_Control.Context_Definition;
                     Package_Name : in Ref_String;
                     Form         : String);

   -- Exceptions from Ada standard Text_Io that are raised by this package
   Name_Error : exception renames Io_Exceptions.Name_Error;

end Ace_Standard;
