--|$Log:	atextio.a,v $
--|Revision 8.0  91/01/17  10:40:55  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Ace_Text_Io
--
---------------------------------------------------------------
--
--
-- Abstract
--
--  Ace_Text_Io is an interface from ACE to Ada's Text_Io.
--  Text_Io uses a limited private type for the data types.
--  A limited private type can not be transferred from the interpreted
--  ACE to the compiled ACE. Therefore, we represent File_Type as
--  an integer and turn the integer into a Text_Io.File_Type internally.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
with Text_Io;
with Io_Exceptions;
package Ace_Text_Io is

   -- File_Limit is the maximum number of opened files allowed.
   File_Limit : constant := 100;

   -- TYPES
   type File_Type is new Integer range 0 .. File_Limit;

   type File_Mode is new Text_Io.File_Mode;

   type Count is  range 0 .. Text_Io.Count'LAST;
   subtype Positive_Count is Count range 1 .. Count'LAST;

   subtype Field is Integer range 0 .. Text_Io.Field'LAST;
   subtype Number_Base is Integer range 2 .. 16;

   -- Procedures that mimic Text_Io of ACE.

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create
   --
   --  <Description>
   --
   --    Same as Text_Io.Create
   --
   --==========================================================================

   procedure Create (File : in out File_Type;
                     Mode : in File_Mode := Out_File;
                     Name : in String    := "";
                     Form : in String    := "");

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Open
   --
   --  <Description>
   --
   --    Same as Text_Io.Open
   --
   --==========================================================================

   procedure Open (File : in out File_Type;
                   Mode : in File_Mode;
                   Name : in String;
                   Form : in String := "");

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Is_Open
   --
   --  <Description>
   --
   --    Same as Text_Io.Is_Open
   --
   --==========================================================================

   function Is_Open (File : in File_Type) return Boolean;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Mode
   --
   --  <Description>
   --
   --    Same as Text_Io.Mode
   --
   --==========================================================================

   function Mode (File : in File_Type) return File_Mode;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Form
   --
   --  <Description>
   --
   --    Same as Text_Io.Form
   --
   --==========================================================================

   function Form (File : in File_Type) return String;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Close
   --
   --  <Description>
   --
   --    Same as Text_Io.Close.
   --
   --==========================================================================

   procedure Close (File : in out File_Type);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Delete
   --
   --  <Description>
   --
   --    Same as Text_Io.Delete.
   --
   --==========================================================================

   procedure Delete (File : in out File_Type);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Reset
   --
   --  <Description>
   --
   --    Same as Text_Io.Reset
   --
   --==========================================================================

   procedure Reset (File : in File_Type;
                    Mode : in File_Mode);

--==========================================================================
--+--  <Subprogram Name>
   --
   --    Reset
   --
   --  <Description>
   --
   --    Same as Text_Io.Reset
   --
 --==========================================================================
   procedure Reset (File : in File_Type);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Get
   --
   --  <Description>
   --
   --    Same as Text_Io.Get
   --
   --==========================================================================

   procedure Get (File : in File_Type;
                  Item : out Character);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Get
   --
   --  <Description>
   --
   --    Same as Text_Io.Get
   --
   --==========================================================================

   procedure Get (Item : out Character);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Get
   --
   --  <Description>
   --
   --    Same as Text_Io.Get
   --
   --==========================================================================

   procedure Get (File : in File_Type;
                  Item : out String);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Get
   --
   --  <Description>
   --
   --    Same as Text_Io.Get
   --
   --==========================================================================

   procedure Get (Item : out String);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Get
   --
   --  <Description>
   --
   --    Same as Text_Io.Get_Line
   --
   --==========================================================================

   procedure Get_Line (File : in File_Type;
                       Item : out String;
                       Last : out Natural);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Get
   --
   --  <Description>
   --
   --    Same as Text_Io.Get_Line
   --
   --==========================================================================

   procedure Get_Line (Item : out String;
                       Last : out Natural);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Skip_Line
   --
   --  <Description>
   --
   --    Same as Text_Io.Skip_Line
   --
   --==========================================================================

   procedure Skip_Line (File    : in File_Type;
                        Spacing : in Positive_Count := 1);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Skip_Line
   --
   --  <Description>
   --
   --    Same as Text_Io.Skip_Line
   --
   --==========================================================================

   procedure Skip_Line (Spacing : in Positive_Count := 1);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Put
   --
   --  <Description>
   --
   --    Same as Text_Io.Put
   --
   --==========================================================================

   procedure Put (File : in File_Type;
                  Item : in Character);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Put
   --
   --  <Description>
   --
   --    Same as Text_Io.Put
   --
   --==========================================================================

   procedure Put (Item : in Character);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Put
   --
   --  <Description>
   --
   --    Same as Text_Io.Put
   --
   --==========================================================================

   procedure Put (File : in File_Type;
                  Item : in String);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Put
   --
   --  <Description>
   --
   --    Same as Text_Io.Put
   --
   --==========================================================================

   procedure Put (Item : in String);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Put_Line
   --
   --  <Description>
   --
   --    Same as Text_Io.Put_Line
   --
   --==========================================================================


   procedure Put_Line (File : in File_Type;
                       Item : in String);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Put_Line
   --
   --  <Description>
   --
   --    Same as Text_Io.Put_Line
   --
   --==========================================================================

   procedure Put_Line (Item : in String);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    New_Line
   --
   --  <Description>
   --
   --    Same as Text_Io.New_Line
   --
   --==========================================================================

   procedure New_Line (File    : in File_Type;
                       Spacing : in Positive_Count := 1);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    New_Line
   --
   --  <Description>
   --
   --    Same as Text_Io.New_Line
   --
   --==========================================================================

   procedure New_Line (Spacing : in Positive_Count := 1);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    End_Of_Line
   --
   --  <Description>
   --
   --    Same as Text_Io.End_Of_Line
   --
   --==========================================================================

   function End_Of_Line (File : in File_Type) return Boolean;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    End_Of_Line
   --
   --  <Description>
   --
   --    Same as Text_Io.End_Of_Line
   --
   --==========================================================================

   function End_Of_Line return Boolean;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    End_Of_File
   --
   --  <Description>
   --
   --    Same as Text_Io.End_Of_File
   --
   --==========================================================================

   function End_Of_File (File : in File_Type) return Boolean;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    End_Of_File
   --
   --  <Description>
   --
   --    Same as Text_Io.End_Of_File
   --
   --==========================================================================

   function End_Of_File return Boolean;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Input
   --
   --  <Description>
   --
   --    Same as Text_Io.Set_Input
   --
   --==========================================================================

   procedure Set_Input (File : in File_Type);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Output
   --
   --  <Description>
   --
   --    Same as Text_Io.Set_Output
   --
   --==========================================================================

   procedure Set_Output (File : in File_Type);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Standard_Input
   --
   --  <Description>
   --
   --    Same as Text_Io.Standard_Input
   --
   --==========================================================================

   function Standard_Input return File_Type;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Standard_Output
   --
   --  <Description>
   --
   --    Same as Text_Io.Standard_Output
   --
   --==========================================================================
   function Standard_Output return File_Type;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Current_Input
   --
   --  <Description>
   --
   --    Same as Text_Io.Current_Input
   --
   --==========================================================================

   function Current_Input return File_Type;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Current_Output
   --
   --  <Description>
   --
   --    Same as Text_Io.Current_Output
   --
   --==========================================================================
   function Current_Output return File_Type;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Ace_Input
   --
   --  <Description>
   --
   --    Same as Text_Io.Set_Input, except that this routine controls the
   --    input for the ACE interpreter.
   --
   --==========================================================================

   procedure Set_Ace_Input (File : in File_Type);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Ace_Output
   --
   --  <Description>
   --
   --    Same as Text_Io.Set_Output, except that this routine controls the
   --    output for the ACE interpreter.
   --
   --==========================================================================

   procedure Set_Ace_Output (File : in File_Type);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Standard_Ace_Input
   --
   --  <Description>
   --
   --    Same as Text_Io.Standard_Input, except that this routine returns the
   --    Standard_Input for the ACE interpreter.
   --
   --==========================================================================

   function Standard_Ace_Input return File_Type;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Standard_Ace_Output
   --
   --  <Description>
   --
   --    Same as Text_Io.Standard_Output, except that this routine returns the
   --    Standard_Output for the ACE interpreter.
   --
   --==========================================================================

   function Standard_Ace_Output return File_Type;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Current_Ace_Input
   --
   --  <Description>
   --
   --    Same as Text_Io.Current_Input, except that this routine returns the
   --    Current_Input for the ACE interpreter.
   --
   --==========================================================================

   function Current_Ace_Input return File_Type;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Current_Ace_Output
   --
   --  <Description>
   --
   --    Same as Text_Io.Current_Output, except that this routine returns the
   --    Current_Output for the ACE interpreter.
   --
   --==========================================================================
   function Current_Ace_Output return File_Type;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Name
   --
   --  <Description>
   --
   --    Same as Text_Io.Name
   --
   --==========================================================================
   function Name (File : in File_Type) return String;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Line_Length
   --
   --  <Description>
   --
   --    Same as Text_Io
   --
   --==========================================================================

   procedure Set_Line_Length (File : in File_Type;
                              To   : in Count);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Line_Length
   --
   --  <Description>
   --
   --    Same as Text_Io.Set_Line_Length
   --
   --==========================================================================
   procedure Set_Line_Length (To : in Count);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Page_Length
   --
   --  <Description>
   --
   --    Same as Text_Io.Set_Page_Length
   --
   --==========================================================================

   procedure Set_Page_Length (File : in File_Type;
                              To   : in Count);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Page_Length
   --
   --  <Description>
   --
   --    Same as Text_Io.Set_Page_Length
   --
   --==========================================================================

   procedure Set_Page_Length (To : in Count);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Line_Length
   --
   --  <Description>
   --
   --    Same as Text_Io.Line_Length
   --
   --==========================================================================

   function Line_Length (File : in File_Type) return Count;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Line_Length
   --
   --  <Description>
   --
   --    Same as Text_Io.Line_Length
   --
   --==========================================================================

   function Line_Length return Count;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Page_Length
   --
   --  <Description>
   --
   --    Same as Text_Io.Page_Length
   --
   --==========================================================================

   function Page_Length (File : in File_Type) return Count;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Page_Length
   --
   --  <Description>
   --
   --    Same as Text_Io.Page_Length
   --
   --==========================================================================

   function Page_Length return Count;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    New_Page
   --
   --  <Description>
   --
   --    Same as Text_Io.New_Page
   --
   --==========================================================================

   procedure New_Page (File : in File_Type);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    New_Page
   --
   --  <Description>
   --
   --    Same as Text_Io.New_Page
   --
   --==========================================================================

   procedure New_Page;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Skip_Page
   --
   --  <Description>
   --
   --    Same as Text_Io.Skip_Page
   --
   --==========================================================================

   procedure Skip_Page (File : in File_Type);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Skip_Page
   --
   --  <Description>
   --
   --    Same as Text_Io.Skip_Page
   --
   --==========================================================================

   procedure Skip_Page;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    End_Of_Page
   --
   --  <Description>
   --
   --    Same as Text_Io.End_Of_Page
   --
   --==========================================================================

   function End_Of_Page (File : in File_Type) return Boolean;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    End_Of_Page
   --
   --  <Description>
   --
   --    Same as Text_Io.End_Of_Page
   --
   --==========================================================================

   function End_Of_Page return Boolean;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Col
   --
   --  <Description>
   --
   --    Same as Text_Io.Set_Col
   --
   --==========================================================================

   procedure Set_Col (File : in File_Type;
                      To   : in Positive_Count);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Col
   --
   --  <Description>
   --
   --    Same as Text_Io.Set_Col
   --
   --==========================================================================
   procedure Set_Col (To : in Positive_Count);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Line
   --
   --  <Description>
   --
   --    Same as Text_Io.Set_Line
   --
   --==========================================================================

   procedure Set_Line (File : in File_Type;
                       To   : in Positive_Count);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Line
   --
   --  <Description>
   --
   --    Same as Text_Io.Set_Line
   --
   --==========================================================================

   procedure Set_Line (To : in Positive_Count);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Col
   --
   --  <Description>
   --
   --    Same as Text_Io.Col
   --
   --==========================================================================

   function Col (File : in File_Type) return Positive_Count;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Col
   --
   --  <Description>
   --
   --    Same as Text_Io.Col
   --
   --==========================================================================

   function Col return Positive_Count;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Line
   --
   --  <Description>
   --
   --    Same as Text_Io.Line
   --
   --==========================================================================

   function Line (File : in File_Type) return Positive_Count;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Line
   --
   --  <Description>
   --
   --    Same as Text_Io.Line
   --
   --==========================================================================

   function Line return Positive_Count;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Page
   --
   --  <Description>
   --
   --    Same as Text_Io.Page
   --
   --==========================================================================

   function Page (File : in File_Type) return Positive_Count;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Page
   --
   --  <Description>
   --
   --    Same as Text_Io.Page
   --
   --==========================================================================

   function Page return Positive_Count;


   -- These are the exceptions that are raised from the Text_Io routines.
   -- Since this is an interface to Text_Io, any of these may be raised.

   Status_Error : exception renames Io_Exceptions.Status_Error;
   Mode_Error   : exception renames Io_Exceptions.Mode_Error;
   Name_Error   : exception renames Io_Exceptions.Name_Error;
   Use_Error    : exception renames Io_Exceptions.Use_Error;
   Device_Error : exception renames Io_Exceptions.Device_Error;
   End_Error    : exception renames Io_Exceptions.End_Error;
   Data_Error   : exception renames Io_Exceptions.Data_Error;
   Layout_Error : exception renames Io_Exceptions.Layout_Error;


   --==========================================================================
   --  <Package Name>
   --
   --    Integer_Io
   --
   --  <Description>
   --
   --    Integer_Io is an instantiation of Text_Io.Integer_Io.  ACE
   --    doesn't support generics.
   --
   --==========================================================================
   --   generic
   --      type Num is range <>;
   package Integer_Io is


      -- Constants for outputing integers
      Default_Width : Field       := 10;
      Default_Base  : Number_Base := 10;

--========================================================================
--+--  <Subprogram Name>
      --
      --    Get
      --
      --  <Description>
      --
      --    Same as Text_Io.Integer_Io.Get
      --
   --========================================================================
      procedure Get (File  : in File_Type;
                     Item  : out Integer;
                     Width : in Field := 0);

--========================================================================
--+--  <Subprogram Name>
      --
      --    Get
      --
      --  <Description>
      --
      --    Same as Text_Io.Integer_Io.Get
      --
   --========================================================================

      procedure Get (Item  : out Integer;
                     Width : in Field := 0);

--========================================================================
--+--  <Subprogram Name>
      --
      --    Get
      --
      --  <Description>
      --
      --    Same as Text_Io.Integer_Io.Get
      --
   --========================================================================
      procedure Get (From : in String;
                     Item : out Integer;
                     Last : out Positive);

      --=======================================================================
      --  <Subprogram Name>
      --
      --    Put
      --
      --  <Description>
      --
      --    Same as Text_Io.Integer_Io.Put
      --
      --=======================================================================

      procedure Put (File  : in File_Type;
                     Item  : in Integer;
                     Width : in Field       := Default_Width;
                     Base  : in Number_Base := Default_Base);

      --=======================================================================
      --  <Subprogram Name>
      --
      --    Put
      --
      --  <Description>
      --
      --    Same as Text_Io.Integer_Io.Put
      --
      --=======================================================================

      procedure Put (Item  : in Integer;
                     Width : in Field       := Default_Width;
                     Base  : in Number_Base := Default_Base);

      --=======================================================================
      --  <Subprogram Name>
      --
      --    Put
      --
      --  <Description>
      --
      --    Same as Text_Io.Integer_Io.Put
      --
      --=======================================================================

      procedure Put (To   : out String;
                     Item : in Integer;
                     Base : in Number_Base := Default_Base);

   end Integer_Io;

end Ace_Text_Io;
