--|$Log:	auntyp.a,v $
--|Revision 8.0  91/01/17  10:40:57  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Ace_Universal_Types
--
---------------------------------------------------------------
--
--
-- Abstract
--
--    This package contains Items which are universal to the Ada Command
--    Environment.   The definitions of types and exceptions are included
--    within this package.  This provides a single set of definitions that
--    may be used through out ACE and assures consistency.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------

package Ace_Universal_Types is

   -- -----
   -- The Ada predefined Integer type is implementation specific in the
   -- setting of the upper and lower bounds of the range.  To provide a
   -- portable integer type, ACE defines the range of its integer.
   --
   -- Assumption:
   -- This definition assumes that the implementation can support
   -- a type that covers the range of
   -- -2147483647 to +2147483647, inclusive.
   -- -----

   type Ace_Integer is  range - 2147483647 .. + 2147483647;

   subtype Ace_Natural is Ace_Integer range 0 .. Ace_Integer'LAST;

   subtype Ace_Positive is Ace_Integer range 1 .. Ace_Integer'LAST;

   subtype Ace_Max_Levels is Ace_Natural range Ace_Natural'FIRST .. 20;

   -- -----
   -- The set of exceptions that ACE may raise, either caused by user
   -- errors or internal checks that denote a failure within ACE, are
   -- defined here.
   -- -----

   Tree_Builder_Error                 : exception;
   Parser_Error : exception;
   Evaluator_Error                    : exception;
   Scanner_Error : exception;
   Main_Error : exception;
   Semantic_Error                     : exception;
   Duplicate_Symbol_Table_Entry_Error : exception;
   User_Error : exception;
   Terminate_Current_Ace              : exception;
   Super_Terminate                    : exception;
   Interrupt_Error                    : exception;
   End_Ace : exception;
   End_Break : exception;
   Exit_Statement_Encountered         : exception;
   Invalid_Reference                  : exception;

end Ace_Universal_Types;
