--|$Log:	context.a,v $
--|Revision 8.0  91/01/17  10:40:59  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Context_Definition
--
---------------------------------------------------------------
--
-- Abstract
--
--  This package contains the global data that the symbol table routines
--  and statement evaluator use to retain state (i.e., the context in which
--  statements are executed).  Currently, it is a collection of unrelated
--  data, but as ACE evolves a clearer picture of what is actually
--  needed to retain state should emerge (i.e., some of this information
--  is not inherently global).
--  -----------------------------------------------------------
--
with Symbol_Table_Structures;
with Statement_Database;
with Ace_Universal_Types;
with Storage_Management;
with Text_Io;

use Symbol_Table_Structures;
use Statement_Database;
use Ace_Universal_Types;
use Storage_Management;
use Text_Io;



package Context_Control is

   type Context_Definition is private;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Create
   --
   --    Last Mod:    Thu Aug 17 17:11:32 1989 (wpl)
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --   This routine will create an initial context, and is called
   --   when ACE starts to create the context for all programs not
   --   nested inside a task.
   --
   --  --------------------------------------------------------------------
   function Create return Context_Definition;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Create
   --
   --    Last Mod:    Thu Aug 17 17:13:31 1989 (wpl)
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --    Create a context from another context (called when a task is
   --    is activated, basing the new context on the context passed in).
   --
   --  --------------------------------------------------------------------
   function Create (Context : in Context_Definition) return Context_Definition;


   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Enclosing_Scope
   --
   --    Last Mod:    Wed Aug 16 14:07:49 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --    Return a reference to the scoping block (subprogram, package, or
   --    block) we are currently executing in.
   --
   --  --------------------------------------------------------------------
   function Get_Enclosing_Scope
       (Context : in Context_Definition) return Ref_Statement_Database;


   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_Enclosing_Scope
   --
   --    Last Mod:    Wed Aug 16 14:07:49 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --    Sets the enclosing scope.
   --
   --  --------------------------------------------------------------------
   procedure Set_Enclosing_Scope (Context : in out Context_Definition;
                                  Item    : in Ref_Statement_Database);


   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Last_Storage_Location_Pointer
   --
   --    Last Mod:    Wed Aug 16 14:07:49 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   function Get_Last_Storage_Location_Pointer
       (Context : in Context_Definition) return Ace_Natural;


   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_Last_Storage_Location_Pointer
   --
   --    Last Mod:    Wed Aug 16 14:07:49 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Set_Last_Storage_Location_Pointer
       (Context : in out Context_Definition;
        Item    : in Ace_Natural);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Push_Last_Storage_Location_Pointer
   --
   --    Last Mod:    Fri Oct 20 17:19:32 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Push_Last_Storage_Location_Pointer
       (Context : in out Context_Definition);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Pop_Last_Storage_Location_Pointer
   --
   --    Last Mod:    Fri Oct 20 17:19:37 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Pop_Last_Storage_Location_Pointer
       (Context : in out Context_Definition);

   -- --
   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Display_For_Level
   --
   --    Last Mod:    Wed Aug 16 14:07:49 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   function Get_Display_For_Level (Context : in Context_Definition;
                                   Level   : in Ace_Natural) return Ace_Natural;

   -- --
   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_Display_For_Level
   --
   --    Last Mod:    Wed Aug 16 14:07:50 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Set_Display_For_Level (Context     : in out Context_Definition;
                                    Level, Item : in Ace_Natural);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Push_Display_For_Level
   --
   --    Last Mod:    Fri Oct 20 17:18:51 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Push_Display_For_Level (Context : in out Context_Definition;
                                     Level   : in Ace_Natural);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Pop_Display_For_Level
   --
   --    Last Mod:    Fri Oct 20 17:18:56 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Pop_Display_For_Level (Context : in out Context_Definition;
                                    Level   : in Ace_Natural);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Current_Level
   --
   --    Last Mod:    Wed Aug 16 14:07:50 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   function Get_Current_Level
       (Context : in Context_Definition) return Ace_Max_Levels;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_Current_Level
   --
   --    Last Mod:    Wed Aug 16 14:07:50 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Set_Current_Level (Context : in out Context_Definition;
                                Item    : in Ace_Max_Levels);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Saved_Level
   --
   --    Last Mod:    Wed Aug 16 14:07:50 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   function Get_Saved_Level
       (Context : in Context_Definition) return Ace_Max_Levels;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_Saved_Level
   --
   --    Last Mod:    Wed Aug 16 14:07:50 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Set_Saved_Level (Context : in out Context_Definition;
                              Item    : in Ace_Max_Levels);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Current_Offset
   --
   --    Last Mod:    Wed Aug 16 14:07:50 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   function Get_Current_Offset
       (Context : in Context_Definition) return Ace_Natural;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_Current_Offset
   --
   --    Last Mod:    Wed Aug 16 14:07:50 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Set_Current_Offset (Context : in out Context_Definition;
                                 Item    : in Ace_Natural);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Saved_Offset
   --
   --    Last Mod:    Wed Aug 16 14:07:50 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   function Get_Saved_Offset
       (Context : in Context_Definition) return Ace_Natural;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_Saved_Offset
   --
   --    Last Mod:    Wed Aug 16 14:07:50 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Set_Saved_Offset (Context : in out Context_Definition;
                               Item    : in Ace_Natural);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Offset_Stack
   --
   --    Last Mod:    Wed Aug 16 14:07:50 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   function Get_Offset_Stack (Context : in Context_Definition;
                              Level   : in Ace_Max_Levels) return Ace_Natural;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_Offset_Stack
   --
   --    Last Mod:    Wed Aug 16 14:07:51 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Set_Offset_Stack (Context : in out Context_Definition;
                               Level   : in Ace_Max_Levels;
                               Item    : in Ace_Natural);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Saved_Offset_Stack
   --
   --    Last Mod:    Wed Aug 16 14:07:51 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   function Get_Saved_Offset_Stack
       (Context : in Context_Definition;
        Level   : in Ace_Max_Levels) return Ace_Natural;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_Saved_Offset_Stack
   --
   --    Last Mod:    Wed Aug 16 14:07:51 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Set_Saved_Offset_Stack (Context : in out Context_Definition;
                                     Level   : in Ace_Max_Levels;
                                     Item    : in Ace_Natural);

   --=====================================================================
   --
   --   These routines set and get the currrently executing statement
   --   w.r.t. the debuggers perspective
   --
   --=====================================================================
   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_Current_Statement
   --
   --    Last Mod:    Wed Aug 16 14:07:51 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Set_Current_Statement (Context : in out Context_Definition;
                                    Node    : Ref_Statement_Database);
   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Current_Statement
   --
   --    Last Mod:    Wed Aug 16 14:07:51 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   function Get_Current_Statement
       (Context : in Context_Definition) return Ref_Statement_Database;

   --====================================================================
   --
   --   These routines provide access to a marker of when a routine
   --   is being executed within a subprogram call.
   --
   --=====================================================================

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_In_Subprogram_Call
   --
   --    Last Mod:    Wed Aug 16 14:07:51 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Set_In_Subprogram_Call (Context : in out Context_Definition);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Unset_In_Subprogram_Call
   --
   --    Last Mod:    Wed Aug 16 14:07:51 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Unset_In_Subprogram_Call (Context : in out Context_Definition);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Is_In_Subprogram_Call
   --
   --    Last Mod:    Wed Aug 16 14:07:51 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   function Is_In_Subprogram_Call
       (Context : in Context_Definition) return Boolean;
   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Increment_Call_Level
   --
   --    Last Mod:    Wed Aug 16 14:07:51 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Increment_Call_Level (Context : in out Context_Definition);
   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Decrement_Call_Level
   --
   --    Last Mod:    Wed Aug 16 14:07:51 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Decrement_Call_Level (Context : in out Context_Definition);
   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Call_Level
   --
   --    Last Mod:    Wed Aug 16 14:07:51 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   function Get_Call_Level (Context : in Context_Definition) return Ace_Natural;

   --=====================================================================
   --
   --   These routines provide access to various symbol table markers.
   --
   --=====================================================================

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Top_Of_Symtbl
   --
   --    Last Mod:    Wed Aug 16 14:07:52 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   function Get_Top_Of_Symtbl
       (Context : in Context_Definition) return Ref_Symbol_Table_Entry;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_Top_Of_Symtbl
   --
   --    Last Mod:    Wed Aug 16 14:07:52 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Set_Top_Of_Symtbl (Context : in out Context_Definition;
                                Item    : in Ref_Symbol_Table_Entry);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Push_Top_Of_Symtbl
   --
   --    Last Mod:    Wed Oct 18 12:14:46 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --    Push a new top of symbol_table.
   --
   --  --------------------------------------------------------------------
   procedure Push_Top_Of_Symtbl (Context : in out Context_Definition;
                                 Item    : in Ref_Symbol_Table_Entry);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Pop_Top_Of_Symtbl
   --
   --    Last Mod:    Wed Oct 18 12:14:31 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Pop_Top_Of_Symtbl (Context : in out Context_Definition);
   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_To_Zero_Level
   --
   --    Last Mod:    Wed Oct 18 14:19:27 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Set_To_Zero_Level (Context : in out Context_Definition);


   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Reset_From_Zero_Level
   --
   --    Last Mod:    Wed Oct 18 14:19:35 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Reset_From_Zero_Level (Context : in out Context_Definition);


   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Saved_Main_Level_Symbol_Table_Head
   --
   --    Last Mod:    Wed Aug 16 14:07:52 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   function Get_Saved_Main_Level_Symbol_Table_Head
       (Context : in Context_Definition) return Ref_Symbol_Table_Entry;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_Saved_Main_Level_Symbol_Table_Head
   --
   --    Last Mod:    Wed Aug 16 14:07:52 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Set_Saved_Main_Level_Symbol_Table_Head
       (Context      : in out Context_Definition;
        Symtbl_Entry : in Ref_Symbol_Table_Entry);


   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Reset_If_Symtbl_Head
   --
   --    Last Mod:    Wed Nov 15 16:33:00a 1989
   --    Author:      Ralph A. Foy
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------

   procedure Reset_If_Symtbl_Head (Context   : in out Context_Definition;
                                   Old_Entry : Ref_Symbol_Table_Entry;
                                   New_Head  : Ref_Symbol_Table_Entry);



   --=====================================================================
   --
   --   These routines provide access to various symbol table markers for
   --   the declarative region routines.
   --
   --=====================================================================

   type List_Of_Entries;
   type Ref_List_Of_Entries is access List_Of_Entries;
   type List_Of_Entries is
      record
         Hidden_Entry      : Ref_Symbol_Table_Entry;
         Next_Hidden_Entry : Ref_List_Of_Entries;
      end record;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Context_Clause_Holder
   --
   --    Last Mod:    Wed Aug 16 14:07:52 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   function Get_Context_Clause_Holder
       (Context : in Context_Definition) return Ref_Symbol_Table_Entry;
   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_Context_Clause_Holder
   --
   --    Last Mod:    Wed Aug 16 14:07:52 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Set_Context_Clause_Holder (Context : in out Context_Definition;
                                        Item    : in Ref_Symbol_Table_Entry);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Dregion_Count
   --
   --    Last Mod:    Wed Aug 16 14:07:52 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   function Get_Dregion_Count
       (Context : in Context_Definition) return Ace_Natural;
   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_Dregion_Count
   --
   --    Last Mod:    Wed Aug 16 14:07:52 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Set_Dregion_Count (Context : in out Context_Definition;
                                Item    : in Ace_Natural);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Dr_Count_Keep
   --
   --    Last Mod:    Wed Aug 16 14:07:52 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   function Get_Dr_Count_Keep
       (Context : in Context_Definition) return Ace_Natural;
   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_Dr_Count_Keep
   --
   --    Last Mod:    Wed Aug 16 14:07:52 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Set_Dr_Count_Keep (Context : in out Context_Definition;
                                Item    : in Ace_Natural);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Declarative_Region
   --
   --    Last Mod:    Wed Aug 16 14:07:52 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   function Get_Declarative_Region
       (Context : in Context_Definition;
        Level   : in Ace_Max_Levels) return Ref_Symbol_Table_Entry;
   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_Declarative_Region
   --
   --    Last Mod:    Wed Aug 16 14:07:52 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Set_Declarative_Region (Context : in out Context_Definition;
                                     Level   : in Ace_Max_Levels;
                                     Item    : in Ref_Symbol_Table_Entry);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Declarative_Region_Entry
   --
   --    Last Mod:    Wed Aug 16 14:07:53 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   function Get_Declarative_Region_Entry
       (Context : in Context_Definition;
        Level   : in Ace_Max_Levels) return Ref_Symbol_Table_Entry;
   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_Declarative_Region_Entry
   --
   --    Last Mod:    Wed Aug 16 14:07:53 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Set_Declarative_Region_Entry (Context : in out Context_Definition;
                                           Level   : in Ace_Max_Levels;
                                           Item    : in Ref_Symbol_Table_Entry);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Have_Encountered_Valid_Return_Stmt
   --
   --    Last Mod:    Wed Aug 16 14:07:53 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   function Get_Have_Encountered_Valid_Return_Stmt
       (Context : in Context_Definition;
        Level   : in Ace_Max_Levels) return Boolean;
   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_Have_Encountered_Valid_Return_Stmt
   --
   --    Last Mod:    Wed Aug 16 14:07:53 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Set_Have_Encountered_Valid_Return_Stmt
       (Context : in out Context_Definition;
        Level   : in Ace_Max_Levels;
        Item    : in Boolean);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Hidden_Entries
   --
   --    Last Mod:    Wed Aug 16 14:07:53 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   function Get_Hidden_Entries
       (Context : in Context_Definition;
        Level   : in Ace_Max_Levels) return Ref_List_Of_Entries;
   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_Hidden_Entries
   --
   --    Last Mod:    Wed Aug 16 14:07:53 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Set_Hidden_Entries (Context : in out Context_Definition;
                                 Level   : in Ace_Max_Levels;
                                 Item    : in Ref_List_Of_Entries);


   --=====================================================================
   --
   --   These routines provide access to various storage management
   --   data structures.
   --
   --=====================================================================

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Data_Storage_Stack
   --
   --    Last Mod:    Wed Aug 16 14:07:53 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   function Get_Data_Storage_Stack
       (Context  : in Context_Definition;
        Location : in Ace_Natural) return Ref_Data_Storage;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_Data_Storage_Stack
   --
   --    Last Mod:    Wed Aug 16 14:07:53 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Set_Data_Storage_Stack (Context  : in out Context_Definition;
                                     Location : in Ace_Natural;
                                     Item     : in Ref_Data_Storage);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Threshold
   --
   --    Last Mod:    Wed Aug 16 14:07:54 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   function Get_Threshold (Context : in Context_Definition) return Ace_Natural;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Previous_Context
   --
   --    Last Mod:    Wed Aug 16 14:07:54 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --   This routine will provide access to the previous context (i.e., the
   --   point at which the current context diverged).
   --  --------------------------------------------------------------------
   function Get_Previous_Context
       (Context : in Context_Definition) return Context_Definition;


   -- ------------------

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Is_Empty
   -- 
   --    Last Mod:    Wed Apr 11 11:57:51 1990
   --    Author:      William P Loftus <wpl@huey>
   -- 
   --  <Description> 
   -- 
   --  --------------------------------------------------------------------
   function Is_Empty (Context : in Context_Definition) return Boolean;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Is_In_Interpret
   -- 
   --    Last Mod:    Wed Apr 11 11:57:48 1990
   --    Author:      William P Loftus <wpl@huey>
   -- 
   --  <Description> 
   -- 
   --  --------------------------------------------------------------------
   function Is_In_Interpret (Context : in Context_Definition) return Boolean;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Top_Of_Symtbl_Save
   -- 
   --    Last Mod:    Wed Apr 11 11:57:46 1990
   --    Author:      William P Loftus <wpl@huey>
   -- 
   --  <Description> 
   -- 
   --  --------------------------------------------------------------------
   function Get_Top_Of_Symtbl_Save
       (Context : in Context_Definition) return Ref_Symbol_Table_Entry;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Call_Level_Save
   -- 
   --    Last Mod:    Wed Apr 11 11:57:43 1990
   --    Author:      William P Loftus <wpl@huey>
   -- 
   --  <Description> 
   -- 
   --  --------------------------------------------------------------------
   function Get_Call_Level_Save
       (Context : in Context_Definition) return Ace_Natural;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Dump_Subprogram_Call_Stack
   -- 
   --    Last Mod:    Wed Apr 11 11:57:40 1990
   --    Author:      William P Loftus <wpl@huey>
   -- 
   --  <Description> 
   -- 
   --  --------------------------------------------------------------------
   procedure Dump_Subprogram_Call_Stack (Outfile : File_Type;
                                         Context : in Context_Definition);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Dump_Dss
   -- 
   --    Last Mod:    Wed Apr 11 11:57:37 1990
   --    Author:      William P Loftus <wpl@huey>
   -- 
   --  <Description> 
   -- 
   --  --------------------------------------------------------------------
   procedure Dump_Dss (Outfile : File_Type;
                       Context : in Context_Definition);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Dump_Dss_Save
   -- 
   --    Last Mod:    Wed Apr 11 11:57:35 1990
   --    Author:      William P Loftus <wpl@huey>
   -- 
   --  <Description> 
   -- 
   --  --------------------------------------------------------------------
   procedure Dump_Dss_Save (Outfile : File_Type;
                            Context : in Context_Definition);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Dump_Storage_Pointers_Stack
   -- 
   --    Last Mod:    Wed Apr 11 11:57:32 1990
   --    Author:      William P Loftus <wpl@huey>
   -- 
   --  <Description> 
   -- 
   --  --------------------------------------------------------------------
   procedure Dump_Storage_Pointers_Stack (Outfile : File_Type;
                                          Context : in Context_Definition);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Dump_Display_Items_Stack
   -- 
   --    Last Mod:    Wed Apr 11 11:57:30 1990
   --    Author:      William P Loftus <wpl@huey>
   -- 
   --  <Description> 
   -- 
   --  --------------------------------------------------------------------
   procedure Dump_Display_Items_Stack (Outfile : in File_Type;
                                       Context : in Context_Definition);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Dump_Valid_Returns
   -- 
   --    Last Mod:    Wed Apr 11 11:57:27 1990
   --    Author:      William P Loftus <wpl@huey>
   -- 
   --  <Description> 
   -- 
   --  --------------------------------------------------------------------
   procedure Dump_Valid_Returns (Outfile : in File_Type;
                                 Context : in Context_Definition);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Dump_Hidden_Entries
   -- 
   --    Last Mod:    Wed Apr 11 11:57:24 1990
   --    Author:      William P Loftus <wpl@huey>
   -- 
   --  <Description> 
   -- 
   --  --------------------------------------------------------------------
   procedure Dump_Hidden_Entries (Context : in Context_Definition);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_In_Interpret
   -- 
   --    Last Mod:    Wed Apr 11 11:57:20 1990
   --    Author:      William P Loftus <wpl@huey>
   -- 
   --  <Description> 
   -- 
   --  --------------------------------------------------------------------
   function Get_In_Interpret (Context : in Context_Definition) return Boolean;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Dump_Top_Of_Symtbl_Save
   -- 
   --    Last Mod:    Wed Apr 11 11:57:17 1990
   --    Author:      William P Loftus <wpl@huey>
   -- 
   --  <Description> 
   -- 
   --  --------------------------------------------------------------------
   procedure Dump_Top_Of_Symtbl_Save (Outfile : in File_Type;
                                      Context : in Context_Definition);


   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Dump_Suprogram_Call_Stack
   -- 
   --    Last Mod:    Wed Apr 11 11:57:13 1990
   --    Author:      William P Loftus <wpl@huey>
   -- 
   --  <Description> 
   -- 
   --  --------------------------------------------------------------------
   procedure Dump_Suprogram_Call_Stack (Outfile : File_Type;
                                        Context : in Context_Definition);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Dump_Dregion
   -- 
   --    Last Mod:    Wed Apr 11 11:57:11 1990
   --    Author:      William P Loftus <wpl@huey>
   -- 
   --  <Description> 
   -- 
   --  --------------------------------------------------------------------
   procedure Dump_Dregion (Outfile : File_Type;
                           Context : in Context_Definition);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Dump_Dregion_Entry
   -- 
   --    Last Mod:    Wed Apr 11 11:57:08 1990
   --    Author:      William P Loftus <wpl@huey>
   -- 
   --  <Description> 
   -- 
   --  --------------------------------------------------------------------
   procedure Dump_Dregion_Entry (Outfile : File_Type;
                                 Context : in Context_Definition);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Dump_Offset_Stack
   -- 
   --    Last Mod:    Wed Apr 11 11:57:05 1990
   --    Author:      William P Loftus <wpl@huey>
   -- 
   --  <Description> 
   -- 
   --  --------------------------------------------------------------------
   procedure Dump_Offset_Stack (Outfile : File_Type;
                                Context : in Context_Definition);
   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Dump_Saved_Offset_Stack
   -- 
   --    Last Mod:    Wed Apr 11 11:57:02 1990
   --    Author:      William P Loftus <wpl@huey>
   -- 
   --  <Description> 
   -- 
   --  --------------------------------------------------------------------
   procedure Dump_Saved_Offset_Stack (Outfile : File_Type;
                                      Context : in Context_Definition);

private

   type Context_Record;

   type Context_Definition is access Context_Record;

end Context_Control;
