--|$Log:	create.a,v $
--|Revision 8.0  91/01/17  10:41:02  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Node_Create_Routines
--
---------------------------------------------------------------
--
--
-- Abstract
--
--    This package provides a set of CREATE routines for statement database
--    nodes.  This set of routines provides a level of data abstraction and
--    information hiding of the statement database node structures from the
--    tree building portion of the Ada command language interpreter.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
-- ==========================================================================

with Statement_Database;
use Statement_Database;

with Symbol_Table_Structures;
use Symbol_Table_Structures;

with Literals;
use Literals;

package Node_Create_Routines is

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Process_Comments
   --
   --  <Description>
   --
   --    This routine stores up comments, so that they may be
   --    associated with the nearest unit.  This needs some
   --    improvement.  Comments sometimes move from their original
   --    position to a position next to the nearest unit.
   --
   --==========================================================================

   procedure Process_Comments (Comment : String);


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Node
   --
   --  <Description>
   --
   --    This routine creates a statement database node (statements only).
   --
   --==========================================================================

   function Create_Node
       (Stmt_Kind : in Statement_Enumeration) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Node
   --
   --  <Description>
   --
   --    This routine creates a statement database node (expressions only).
   --
   --==========================================================================

   function Create_Node
       (Expr_Kind : in Expression_Enumeration) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Statement_Database_Item
   --
   --  <Description>
   --
   --   The tree builder is separated into two package one package is the
   --   general routines, and the other is the specific routines.
   --   This package contain the specification of the general routines.
   --
   --   The general routines can be separated into three categories:
   --
   --     creating statement database nodes
   --
   --     creating expression nodes
   --
   --     creating statement nodes
   --
   --   These categories are the only assumptions made about the
   --   underlying structure of the statement database.
   --   These routines are functional in nature returning small sections of
   --   the abstract syntax tree that will be hooked together to form a complete
   --   tree by the specific tree building routines.
   --   The general tree building routines interface directly with the statement
   --   database's data structures.
   --
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    Turns all exceptions to Tree_Builder_Error
   --
   --==========================================================================

   function Create_Statement_Database_Item (Item : Ref_Statement_Record := null
                                            ) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --   Create_Statement_Database_Item
   --
   --  <Description>
   --
   --   The routine Create_Statement_Database_Item will hook together pointers
   --   to creation one Statement_Database_Record.  The kind of variant that
   --   will be created is based on the parameter Kind, the rest of the items
   --   are the pointers to subtrees to be "hooked" in.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    Turns all exceptions to Tree_Builder_Error
   --==========================================================================
   function Create_Statement_Database_Item (Item : Ref_Expression_Record :=
      null) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --   Create_Statement_Item
   --
   --  <Description>
   --
   --   The routine Create_Statement_Item will hook together pointers
   --   to creation one Statement_Record.  The kind of variant that
   --   will be created is based on the parameter Kind, the rest of the items
   --   are the pointers to subtrees to be "hooked" in.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    Turns all exceptions to Tree_Builder_Error
   --==========================================================================

   function Create_Statement_Item
       (Kind : Statement_Enumeration;
        Item1, Item2, Item3, Item4 : Ref_Statement_Database := null) return
         Ref_Statement_Record;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Expression_Item
   --
   --  <Description>
   --
   --   The routine Create_Expression_Item will hook together pointers
   --   to creation one Expression_Record.  The kind of variant that
   --   will be created is based on the parameter Kind, the rest of the items
   --   are the pointers to subtrees to be "hooked" in.
   --
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    Turns all exceptions to Tree_Builder_Error
   --==========================================================================
   function Create_Expression_Item
       (Kind                : Expression_Enumeration;
        Item1, Item2, Item3 : Ref_Statement_Database := null;
        Symbol_Table        : Ref_Symbol_Table_Entry := null;
        Str                 : Ref_String             := Create;
        Enum                : Lp_Node_Enumeration    := Unknown) return
         Ref_Expression_Record;

end Node_Create_Routines;
