--|$Log:	ctree.a,v $
--|Revision 8.0  91/01/17  10:41:02  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Create_Statement_Database
--
---------------------------------------------------------------
--
--
-- Abstract
--
--   This package contains specific create routines for the
--   statement database.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
with Statement_Database;
use Statement_Database;
package Create_Statement_Database is

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Pragma_Node
   --
   --  <Description>
   --
   --    Create a statement database node for pragmas.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Pragma_Node
       (Ident, List : Ref_Statement_Database) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_With_Node
   --
   --  <Description>
   --
   --    Create a statement database node for with.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_With_Node
       (List : Ref_Statement_Database) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Case_Node
   --
   --  <Description>
   --
   --    Create a statement database node for case.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Case_Node
       (Expr, Altern : Ref_Statement_Database) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Case_Alt_Node
   --
   --  <Description>
   --
   --    Create a statement database node for case alternatives.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Case_Alt_Node
       (Choice, Seq_Choice, Seq_Stmt : Ref_Statement_Database) return
         Ref_Statement_Database;


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Left_Case_Alt_Node
   --
   --  <Description>
   --
   --    Create a statement database node for case alternatives.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Left_Case_Alt_Node
       (Altern, Seq_Altern : Ref_Statement_Database) return
         Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Right_Case_Alt_Node
   --
   --  <Description>
   --
   --    Create a statement database node for case alternatives.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Right_Case_Alt_Node
       (Seq_Altern, Altern : Ref_Statement_Database) return
         Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Block_Node
   --
   --  <Description>
   --
   --    Create a statement database node for blocks.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Block_Node
       (Name, Declarations, Statements, Exceptions : Ref_Statement_Database)
       return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Exception_Alt_Node
   --
   --  <Description>
   --
   --    Create a statement database node for case alternatives.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Exception_Alt_Node
       (Choice, Seq_Choice, Seq_Stmt : Ref_Statement_Database) return
         Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Left_Exception_Alt_Node
   --
   --  <Description>
   --
   --    Create a statement database node for case alternatives.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Left_Exception_Alt_Node
       (Altern, Seq_Altern : Ref_Statement_Database) return
         Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Right_Exception_Alt_Node
   --
   --  <Description>
   --
   --    Create a statement database node for case alternatives.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Right_Exception_Alt_Node
       (Seq_Altern, Altern : Ref_Statement_Database) return
         Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Procedure_Call_Node
   --
   --  <Description>
   --
   --    Create a statement database node for procedure calls.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Procedure_Call_Node
       (Name : Ref_Statement_Database) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Label_Node
   --
   --  <Description>
   --
   --    Create a statement database node for the labels.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Label_Node
       (Name : Ref_Statement_Database) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Exit_Node
   --
   --  <Description>
   --
   --    Create a statement database node for exits.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Exit_Node
       (Name, Expr : Ref_Statement_Database) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Goto_Node
   --
   --  <Description>
   --
   --    Create a statement database node for the goto.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Goto_Node
       (Name : Ref_Statement_Database) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Return_Node
   --
   --  <Description>
   --
   --    Create a statement database node for the return statement.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Return_Node
       (Expr : Ref_Statement_Database) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Loop_Node
   --
   --  <Description>
   --
   --    Create a statement database node for loops.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Loop_Node
       (Name, Scheme, Seq_Of_Stmt : Ref_Statement_Database) return
         Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_For_Node
   --
   --  <Description>
   --
   --    Create a statement database node for fors.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_For_Node
       (Variable, Reverse_Flag, Dscr_Rng : Ref_Statement_Database) return
         Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Null_Stmt_Node
   --
   --  <Description>
   --
   --    Create a statement database node for the null.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Null_Stmt_Node return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Left_Stmt_Seq_Node
   --
   --  <Description>
   --
   --    Hooks statement database nodes together.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Left_Stmt_Seq_Node (Current_Stmt_Seq, Statement,
                                       Optional_Pragma : Ref_Statement_Database
                                       ) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Right_Stmt_Seq_Node
   --
   --  <Description>
   --
   --    Hooks statement database nodes together.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Right_Stmt_Seq_Node
       (Optional_Pragma, Statement, Current_Stmt_Seq : Ref_Statement_Database)
       return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Compilation_Node
   --
   --  <Description>
   --
   --    Returns a statement database node that hooks a context clause
   --    into a compilation unit.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Compilation_Node
       (Context_Cl, Comp_Unit : Ref_Statement_Database) return
         Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Stmt_Seq_Node
   --
   --  <Description>
   --
   --    Returns a statement database node that hooks two nodes together.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Stmt_Seq_Node
       (Stmt1, Stmt2 : Ref_Statement_Database) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Assign_Node
   --
   --  <Description>
   --
   --    Create a statement database node for assignment.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Assign_Node
       (Name, Expr : Ref_Statement_Database) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_If_Node
   --
   --  <Description>
   --
   --    Create a statement database node for ifs.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_If_Node
       (Condition, Elsif_True_Stmts, Previous_Elsif_Stmt :
        Ref_Statement_Database) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_If_Node
   --
   --  <Description>
   --
   --    Create a statement database node for ifs.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_If_Node
       (Condition, If_True_Stmts, Elsif_Stmts, Final_Else_Part :
        Ref_Statement_Database) return Ref_Statement_Database;


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Expr_Node
   --
   --  <Description>
   --
   --    Create a statement database expression node.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Expr_Node
       (Exp1 : Ref_Statement_Database;
        Exp2 : Ref_Statement_Database) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Name_Range_Expr_Node
   --
   --  <Description>
   --
   --    Create a statement database expression node for ranges.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Name_Range_Expr_Node
       (Exp1, Exp2 : Ref_Statement_Database) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_New_Expr_Node
   --
   --  <Description>
   --
   --    Create a statement database expression node for the New operator.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_New_Expr_Node
       (Exp1 : Ref_Statement_Database;
        Exp2 : Ref_Statement_Database) return Ref_Statement_Database;

   -- [TBD]

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Expr_Node
   --
   --  <Description>
   --
   --    Create a statement database expression node.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Expr_Node
       (Exp1     : Ref_Statement_Database;
        Exp2     : Ref_Statement_Database;
        Operator : Integer;
        Exp3     : Ref_Statement_Database) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Aggr_Expr_Node
   --
   --  <Description>
   --
   --    Create a statement database expression node for aggregates.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Aggr_Expr_Node
       (Exp1 : Ref_Statement_Database;
        Exp2 : Ref_Statement_Database;
        Exp3 : Ref_Statement_Database) return Ref_Statement_Database;

   --
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Expr_Node
   --
   --  <Description>
   --
   --    Create a statement database expression node.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================

   function Create_Expr_Node (Operator : Integer) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Attr_Expr_Node
   --
   --  <Description>
   --
   --    Create a statement database expression node for attributes.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Attr_Expr_Node
       (Exp1 : Ref_Statement_Database;
        Exp2 : Ref_Statement_Database) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Range_Expr_Node
   --
   --  <Description>
   --
   --    Create a statement database expression node for ranges.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Range_Expr_Node
       (Exp1 : Ref_Statement_Database;
        Exp2 : Ref_Statement_Database) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Expr_Node
   --
   --  <Description>
   --
   --    Create a statement database expression node.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Expr_Node
       (Exp1     : Ref_Statement_Database;
        Operator : Integer;
        Exp2     : Ref_Statement_Database) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Expr_Node
   --
   --  <Description>
   --
   --    Create a statement database expression node.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Expr_Node
       (Exp1      : Ref_Statement_Database;
        Operator1 : Integer;
        Operator2 : Integer;
        Exp2      : Ref_Statement_Database) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Expr_Node
   --
   --  <Description>
   --
   --    Create a statement database expression node.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Expr_Node
       (Exp1 : Ref_Statement_Database;
        Exp2 : Ref_Statement_Database;
        Exp3 : Ref_Statement_Database) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Expr_Node
   --
   --  <Description>
   --
   --    Create a statement database node expression.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Expr_Node (Str      : String;
                              Str_Leng : Integer) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_String_Node
   --
   --  <Description>
   --
   --    Create a statement database node expression for Strings.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_String_Node
       (Str      : String;
        Str_Leng : Integer) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Character_Node
   --
   --  <Description>
   --
   --    Create a statement database expression node for characters.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Character_Node
       (Str      : String;
        Str_Leng : Integer) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Constant_Node
   --
   --  <Description>
   --
   --    Create a statement database expression node for constants.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Constant_Node
       (Str      : String;
        Str_Leng : Integer) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Dot_Expr_Node
   --
   --  <Description>
   --
   --    Create a statement database expression node for expanded names.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Dot_Expr_Node
       (Exp1 : Ref_Statement_Database;
        Exp2 : Ref_Statement_Database) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Unary_Expr_Node
   --
   --  <Description>
   --
   --    Create a statement database expression node.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Unary_Expr_Node
       (Operator : Integer;
        Exp1     : Ref_Statement_Database) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Expr_Node
   --
   --  <Description>
   --
   --    Create a statement database expression node.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Expr_Node
       (Operator : Integer;
        Exp1     : Ref_Statement_Database) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Left_Expr_Node
   --
   --  <Description>
   --
   --    Create a statement database expression node.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Left_Expr_Node
       (Exp1 : Ref_Statement_Database;
        Exp2 : Ref_Statement_Database) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Right_Expr_Node
   --
   --  <Description>
   --
   --    Create a statement database expression node.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Right_Expr_Node
       (Exp1 : Ref_Statement_Database;
        Exp2 : Ref_Statement_Database) return Ref_Statement_Database;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Unary_Add_Expr_Node
   --
   --  <Description>
   --
   --    Create a statement database expression node for unary operators.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Unary_Add_Expr_Node
       (Exp1 : Ref_Statement_Database;
        Exp2 : Ref_Statement_Database) return Ref_Statement_Database;


end Create_Statement_Database;









