--|$Log:	debug.a,v $
--|Revision 8.0  91/01/17  10:41:04  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - ACE Debugger
--
------------------------------------------------------------
--
--
-- Abstract
--
--  This package contains the routines that perform symbolic
--  debugging in ACE.
--
--
-- Update Log - :
--
-- End of Header
-------------------------------------------------------------------------

with Statement_Database;
use Statement_Database;

with Symbol_Table_Structures;
use Symbol_Table_Structures;

with Ace_Universal_Types;
use Ace_Universal_Types;

package Debugger is

   -- Overview:
   -- The package "Debugger" provides debugging capabilities for the
   -- testing of compilation units entered into the Ada Command Language
   -- Interpreter.  The debugging capabilities provide execution control
   -- and the display of status information.

   Do_Delay : Boolean := True;

   --====================================================================
   --  <Subprogram Name>
   --
   --    Any_Windows_Created
   --
   --  <Description>
   --
   --    Returns true if a window was created by ACE, during the
   --    current session.
   --
   --  <Exceptions Raised>
   --
   --    none
   --
   --  <Exceptions Handled>
   --
   --    none
   --====================================================================
   function Any_Windows_Created return Boolean;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Set_Statements_Trace_On
   --    Set_Statements_Trace_Off
   --    Is_Statements_Trace_On
   --
   --  <Description>
   --
   --    These routines turn on, turn off, and query a flag that marks
   --    whether or not the dump routines will output the statement
   --    number that is associated with every statement.
   --
   --  <Exceptions Raised>
   --
   --    none
   --
   --  <Exceptions Handled>
   --
   --    none
   --=======================================================================
   procedure Set_Statements_Trace_On;
   procedure Set_Statements_Trace_Off;
   function Is_Statements_Trace_On return Boolean;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Set_Observe_Statements_On
   --    Set_Observe_Statements_Off
   --    Is_Statements_Observe_On
   --
   --  <Description>
   --
   --    These routines control the statements observation window.
   --
   --=======================================================================
   procedure Set_Observe_Statements_On;
   procedure Set_Observe_Statements_Off;
   function Is_Statements_Observe_On return Boolean;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Set_Observe_Objects_On
   --    Set_Observe_Objects_Off
   --    Is_Objects_Observe_On
   --
   --  <Description>
   --
   --    These routines control the objects observation window.
   --
   --=======================================================================
   procedure Set_Observe_Objects_On;
   procedure Set_Observe_Objects_Off;
   function Is_Objects_Observe_On return Boolean;

   --=======================================================================
   --     <Subprogram Name>
   --
   --    Set_Observe_Subprograms_On
   --    Set_Observe_Subprograms_Off
   --    Is_Subprograms_Observe_On
   --
   --  <Description>
   --
   --    These routines control the subprograms observation window.
   --
   --======================================================================
   procedure Set_Observe_Subprograms_On;
   procedure Set_Observe_Subprograms_Off;
   function Is_Subprograms_Observe_On return Boolean;

   --=======================================================================
   --     <Subprogram Name>
   --
   --    Set_Observe_Tasks_On
   --    Set_Observe_Tasks_Off
   --    Is_Tasks_Observe_On
   --
   --  <Description>
   --
   --    These routines control the Tasks observation window.
   --
   --======================================================================
   procedure Set_Observe_Tasks_On;
   procedure Set_Observe_Tasks_Off;
   function Is_Tasks_Observe_On return Boolean;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Set_Symbol_Table_Debugging_On
   --    Set_Symbol_Table_Debugging_Off
   --    Is_Symbol_Table_Debugging_On
   --
   --  <Description>
   --
   --    These routines control the whether or not the symbol_table is
   --    set up (retaining symbolic information) for debugging.
   --
   --======================================================================
   procedure Set_Symbol_Table_Debugging_On;
   procedure Set_Symbol_Table_Debugging_Off;
   function Is_Symbol_Table_Debugging_On return Boolean;

   --=======================================================================
   --     <Subprogram Name>
   --
   --    Set_Overall_Debugging_On
   --    Set_Overall_Debugging_Off
   --    Is_Overall_Debugging_On
   --
   --  <Description>
   --
   --    These routines control the whether or not the ACE is
   --    set up (retaining symbolic information) for debugging.
   --======================================================================
   procedure Set_Overall_Debugging_On;
   procedure Set_Overall_Debugging_Off;
   function Is_Overall_Debugging_On return Boolean;

   -- -------------------------------------------------------------------

   --=======================================================================
   --     <Subprogram Name>
   --
   --    Observe_Statement_If_Statements_Observation_Is_On
   --    Observe_Object_If_Objects_Observation_Is_On
   --    Observe_Subprogram_If_Subprogram_Observation_Is_On
   --
   --  <Description>
   --
   --    These routines query which obeervation windows are currently
   --    active.
   --
   --======================================================================
   procedure Observe_Statement_If_Statements_Observation_Is_On
       (Node : in Ref_Statement_Database);
   procedure Observe_Object_If_Objects_Observation_Is_On
       (Symtbl_Entry : in Ref_Symbol_Table_Entry);
   procedure Observe_Subprogram_If_Subprogram_Observation_Is_On
       (Node : in Ref_Statement_Database);

   -- -------------------------------------------------------------------

   --=======================================================================
   --     <Subprogram Name>
   --
   --    Initialize_Statements_Observation_If_Statements_Observation_Is_On1
   --    Reset_Statements_Observation_If_Statements_Observation_Is_On
   --    Is_Subprograms_Observe_On
   --
   --  <Description>
   --
   --    These routines control the Statements observation window if its
   --    activity is toggled (i.e., turned on and off).
   --
   --======================================================================
   procedure Initialize_Statements_Observation_If_Statements_Observation_Is_On
       (Symtbl_Entry : in Ref_Symbol_Table_Entry);

   procedure Reset_Statements_Observation_If_Statements_Observation_Is_On
      ;
   procedure Remove_Entry_From_List (Symtbl_Entry : Ref_Symbol_Table_Entry);

   -- The following are routines that the debugging ADT in ACE uses, but
   -- are located in another package for simplicities sake.
   -- -------------------------------------------------------------------
   -- -------------------------------------------------------------------

   -- Dumps the all the entries contained in the ACE Symbol Table
   --
   -- procedure Dump_Symbol_Table;

   -- -------------------------------------------------------------------

   -- Dumps the symbol table entry for the given identifier
   --
   -- procedure Dump (Name : in String);

   -- -------------------------------------------------------------------
   --
   -- procedure Dump_Dss;

   -- -------------------------------------------------------------------
   --
   -- procedure Dump_Display;


   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Break
   --
   --    Last Mod:    Thu Aug 17 18:04:51 1989 (wpl)
   --    Author:      C Oei <wpl@huey>
   --
   --  <Description>
   --
   -- Temporarily suspend execution for debugging
   --  --------------------------------------------------------------------
   procedure Break (Current_Statement : in Ref_Statement_Database;
                    Notice_Message    : String := "Breakpoint encountered");

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Continue
   --
   --    Last Mod:    Thu Aug 17 18:05:44 1989 (wpl)
   --    Author:      C. Oei <wpl@huey>
   --
   --  <Description>
   -- Continue execution after a Break statement is encountered
   --
   --  --------------------------------------------------------------------
   procedure Continue;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Is_Breakpoint_Set
   --
   --    Last Mod:    Thu Aug 17 18:05:57 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --    Returns true if a break point has been set on the "line" (i.e., node)
   --    passed in.
   --
   --  --------------------------------------------------------------------
   function Is_Breakpoint_Set (Node : in Ref_Statement_Database) return Boolean;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_Break
   --
   --    Last Mod:    Thu Aug 17 18:03:34 1989 (wpl)
   --    Author:      C. Oei <wpl@huey>
   --
   --  <Description>
   --
   -- Set_Break establishes at breakpoint at the indicated statement.  When
   -- the breakpointed statement is encountered during interpretation,
   -- execution will be suspended and control given to ther user.
   -- Internally, a statement node of type "Breakpoint" will be created and
   -- placed prior to the indicated statement.
   --  --------------------------------------------------------------------
   procedure Set_Break (Statement_Number : in Ace_Natural);
   -- procedure Set_Break; -- at "current" statement [really uses above]
   procedure Set_Break (Name : in String); -- subprogram or label


   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Clear_Break
   --
   --    Last Mod:    Thu Aug 17 18:08:22 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   -- Clear_Break removes a breakpoint from the indicated location.
   --  --------------------------------------------------------------------
   procedure Clear_Break (Statement_Number : in Ace_Natural);
   -- procedure Clear_Break;
   -- clear breakpoint at "current" breakpoint statement [really uses above]
   procedure Clear_Break (Name : in String); -- subprogram or label



   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Display
   --
   --    Last Mod:    Thu Aug 17 18:08:44 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   -- Display produces output giving the text of the Ada statement associated
   -- with the requested location(s).  The location may be specified by a
   -- statement or a name.  If the name refers to a subprogram or package,
   -- the entire subprogram or package is displayed.  Statements are displayed
   -- with their corresponding statement number.
   -- Note: Statements may be displayed at any time.
   --  --------------------------------------------------------------------
   procedure Display (Statement_Number : in Ace_Natural);
   procedure Display (Name : in String); -- subprogram, package or label

   -- -------------------------------------------------------------------

   -- Display produces the text of the last statement executed (i.e. the
   -- "current" statement).
   --
   -- procedure Display; -- "current" statement [really uses above]


   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Display_Next
   --
   --    Last Mod:    Thu Aug 17 18:09:21 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   -- Display_Next produces the text of the next statement that was
   -- entered after the current.  The next statement becomes the
   -- "current" statement for displaying purposes.
   --
   --  --------------------------------------------------------------------
   procedure Display_Next (Reference_Statement_Number : in Ace_Natural);

   -- -------------------------------------------------------------------

   --
   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Display_Previous
   --
   --    Last Mod:    Thu Aug 17 18:09:42 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   -- Display_Previous produces the text of the previous statement that was
   -- entered prior to the current statement.  The previous statement
   -- becomes the "current" statement for displaying purposes.
   --  --------------------------------------------------------------------
   procedure Display_Previous (Reference_Statement_Number : in Ace_Natural);

   -- -------------------------------------------------------------------

   --
   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Dump_Call_Stack
   --
   --    Last Mod:    Thu Aug 17 18:10:01 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   -- The call stack for the currently active subprogram is dumped.
   --
   --  --------------------------------------------------------------------
   procedure Dump_Call_Stack;

   -- -------------------------------------------------------------------

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Is_Stepping_Through
   --
   --    Last Mod:    Thu Aug 17 18:10:24 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --    Returns true if the debugger is currently stepping through a
   --    routine.
   --
   --  --------------------------------------------------------------------
   function Is_Stepping_Through return Boolean;
   --
   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Step
   --
   --    Last Mod:    Thu Aug 17 18:10:29 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --     Steps to the next (N) statements.
   --
   --  --------------------------------------------------------------------
   procedure Step (Count : Ace_Positive := 1);

   -- -------------------------------------------------------------------
   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    List_Breakpoints
   --
   --    Last Mod:    Thu Aug 17 18:10:37 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --    Lists all the current break points.
   --
   --  --------------------------------------------------------------------
   procedure List_Breakpoints;

   -- -------------------------------------------------------------------
   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Within_Breakpoint
   --
   --    Last Mod:    Thu Aug 17 18:10:43 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --    returns true if currently executing after a break point.
   --  --------------------------------------------------------------------
   function Within_Breakpoint return Boolean;

   -- -------------------------------------------------------------------

end Debugger;
