--|$Log:	drsup.a,v $
--|Revision 8.0  91/01/17  10:41:06  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Declarative_Region_Support_Routines
--
------------------------------------------------------------
--
--
-- Abstract
--
-- Support routines for manipulating Ada declarative regions
--
-- Update Log - :
--
-- End of Header
-------------------------------------------------------------------------

with Symbol_Table_Structures;
use Symbol_Table_Structures;

with Context_Control;

package Declarative_Region_Support_Routines is

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Within_Declarative_Region
   --
   --  <Description>
   --
   --    This routine returns a boolean indicating whether or not we are
   --    currently in a declarative region.
   --
   --=======================================================================
   function Within_Declarative_Region
       (Context : in Context_Control.Context_Definition) return Boolean;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Valid_Return_Statement_Encountered
   --
   --  <Description>
   --
   --    This routine marks a declarative region that a return statement
   --    has been encounters.   We need this to be able to unwind
   --    the stack when executing a return.
   --=======================================================================
   procedure Valid_Return_Statement_Encountered
       (Context : in out Context_Control.Context_Definition);

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Declarative_Region_Entry
   --
   --  <Description>
   --
   --    This routine returns the position in the symbol table of
   --    the current declarative region.
   --
   --=======================================================================
   function Get_Declarative_Region_Entry
       (Context : in Context_Control.Context_Definition) return
         Ref_Symbol_Table_Entry;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Declarative_Region_Mark
   --
   --  <Description>
   --
   --    This routine returns the position in the symbol table of
   --    the current marked declarative region.
   --
   --=======================================================================

   function Get_Declarative_Region_Mark
       (Context : in Context_Control.Context_Definition) return
         Ref_Symbol_Table_Entry;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Set_Decl_Region_Top_Of_Symtbl
   --
   --  <Description>
   --
   --    This routine explicitly sets the declarative region.
   --
   --=======================================================================

   procedure Set_Decl_Region_Top_Of_Symtbl
       (Context      : in out Context_Control.Context_Definition;
        Symtbl_Entry : in Ref_Symbol_Table_Entry);

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Level_Zero_Entry
   --
   --  <Description>
   --
   --    This routine returns the position in the symbol table of
   --    the current declarative region.
   --
   --=======================================================================
   function Get_Level_Zero_Entry
       (Context : in Context_Control.Context_Definition) return
         Ref_Symbol_Table_Entry;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Push_Declarative_Region
   --
   --  <Description>
   --
   --    This routine will push a declarative region, thus
   --    making it the current declarative region.
   --
   --=======================================================================

   procedure Push_Declarative_Region
       (Context      : in out Context_Control.Context_Definition;
        Symtbl_Entry : in Ref_Symbol_Table_Entry);

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Pop_Declarative_Region
   --
   --  <Description>
   --
   --    This routine will pop off the current declarative region, thus
   --    restoring the previous declarative region.
   --
   --=======================================================================

   procedure Pop_Declarative_Region
       (Context : in out Context_Control.Context_Definition);

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Is_In_AceAdt_Compile, Set_In_AceAdt_Compile, UnSet_In_AceAdt_Compile
   --
   --  <Description>
   --
   --    These routines provide access to a marker of when an ACE ADT
   --    "compile" routine is being executed.
   --
   --
   --=======================================================================

   function Is_In_Aceadt_Compile return Boolean;

   procedure Set_In_Aceadt_Compile;

   procedure Unset_In_Aceadt_Compile;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Set_DR_To_Top_Level
   --
   --  <Description>
   --
   --    Sets the declarative region temporarily back to the top level
   --    (level 0) environment.
   --
   --=======================================================================

   procedure Set_Dr_To_Top_Level
       (Context : in out Context_Control.Context_Definition);

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Reset_DR_To_Previous_Level
   --
   --  <Description>
   --
   --    Undoes the declarative region temporarily back to the top level
   --    (level 0) environment.
   --
   --=======================================================================

   procedure Reset_Dr_To_Previous_Level
       (Context : in out Context_Control.Context_Definition);

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Push_Hidden_Entry
   --
   --  <Description>
   --
   --    This routine will add to a list of hidden entries, so that the
   --    entry may be made visible again at the end of the decalaritive
   --    region.
   --
   --=======================================================================

   procedure Push_Hidden_Entry
       (Context      : in out Context_Control.Context_Definition;
        Symtbl_Entry : in Ref_Symbol_Table_Entry);

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Reset_Declarative_Regions
   --
   --  <Description>
   --
   --    This routine will reset a declarative region, that is it will
   --    recover from an error that was in a declarative region.
   --
   --=======================================================================
   procedure Reset_Declarative_Regions
       (Context : in out Context_Control.Context_Definition);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Push_Context_Clause
   --
   --    Last Mod:    Fri Aug 18 11:30:52 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --      Used to allow visibility of the context clauses during
   --      parsing.
   --  --------------------------------------------------------------------
   procedure Push_Context_Clause
       (Context : in out Context_Control.Context_Definition);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Pop_Context_Clause
   --
   --    Last Mod:    Fri Aug 18 11:30:46 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --     Use to remove visibility of the context clauses after parsing a
   --     compilation unit.
   --
   --  --------------------------------------------------------------------
   procedure Pop_Context_Clause
       (Context : in out Context_Control.Context_Definition);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Start_Of_Block
   --
   --    Last Mod:    Fri Aug 18 11:30:44 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --    Set up the symbol table for the beginning of a block.
   --
   --  --------------------------------------------------------------------
   procedure Start_Of_Block
       (Context : in out Context_Control.Context_Definition);

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    End_Of_Block
   --
   --    Last Mod:    Fri Aug 18 11:30:13 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --    modify the symbol table to end a block
   --
   --  --------------------------------------------------------------------
   procedure End_Of_Block (Context : in out Context_Control.Context_Definition);

end Declarative_Region_Support_Routines;




