--|$Log:	dspprt.a,v $
--|Revision 8.0  91/01/17  10:41:08  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Declaration_Support_Routines
--
---------------------------------------------------------------
--
--
-- Abstract
--
--   This package is a collection of routines that provide support during
--   the parsing process of Ada statements by the Ada Command Language
--   Interpreter.  These routines provide ancillary support and are not
--   called directly by the parser.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------

with Symbol_Table_Structures;
use Symbol_Table_Structures;

-- Only needed for "All_Specs_Have_Bodies" routine
with Statement_Database;
use Statement_Database;

with Literals;
use Literals;

with Context_Control;

package Declaration_Support_Routines is


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Subprogram_Profiles_Match
   --
   --  <Description>
   --
   --    The subprogram profiles (parameters and return value for function)
   --    are compared against each other.
   --    If the two profiles match, then True is returned; otherwise False.
   --
   --==========================================================================

   function Subprogram_Profiles_Match
       (Subprogram_Spec_One : in Ref_Symbol_Table_Entry;
        Subprogram_Spec_Two : in Ref_Symbol_Table_Entry) return Boolean;


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Init_For_Duplicate_Check
   --
   --  <Description>
   --
   --    This routine works in conjunction with Check_For_Duplicates.
   --    The local state information is initialized to empty for the subsequent
   --    calls made to Check_For_Duplicates.  Any information remaining from
   --    a previous series of call to Init_For_Duplicate_Check and
   --    Check_For_Duplicates is cleaned up.
   --
   --==========================================================================

   procedure Init_For_Duplicate_Check;


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Check_For_Duplicates
   --
   --  <Description>
   --
   --    The identifier name is added to the running list of identifiers.
   --    The identifier is checked against all the current identifiers within
   --    the list to determine whether it is identical to another identifier
   --    within the list.
   --
   --==========================================================================

   function Check_For_Duplicates (Id_Name : in Ref_String) return Boolean;


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Name_Clash
   --
   --  <Description>
   --
   --    The identifier name is searched for within the symbol table to
   --    determine whether the name already exists.  For a name to clash, the
   --    name must exist at the current static subprogram nesting level.
   --
   --==========================================================================

   function Name_Clash (Context : in Context_Control.Context_Definition;
                        Id_Name : in Ref_String) return Boolean;


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Name_Clash_For_Enum_Literal
   --
   --  <Description>
   --
   --    The enumeration literal name is searched for within the symbol table
   --    to determine whether the name already exists.  Note that an
   --    enumeration literal may be overloaded (i.e. the name may already be
   --    used as an enumeration literal).  For an enumeration literal name to
   --    clash, the name must exist as something other than an enumeration
   --    literal at the current static subprogram nesting level.
   --
   --==========================================================================

   function Name_Clash_For_Enum_Literal
       (Context      : in Context_Control.Context_Definition;
        Literal_Name : in Ref_String) return Boolean;


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Name_Clash_For_Subprogram
   --
   --  <Description>
   --
   --    The subprogram name is searched for within the symbol table to
   --    determine whether the name already exists.  If the name is used
   --    for something other than a subprogram, then a name clash exists.
   --    If the name exists as a subprogram name, then the signatures of
   --    the subprograms must be checked.  A name clash exists if the
   --    signature of a subprogram spec matches the signature of another
   --    spec or body, or if the signature of a subprogram body matches the
   --    signature of another body.
   --    If the specification is found for a subprogram body, then the
   --    symbol table entry for the specification is returned; otherwise,
   --    null is returned.
   --
   --==========================================================================

   function Name_Clash_For_Subprogram
       (Context                  : in Context_Control.Context_Definition;
        Subprogram_Entry         : in Ref_Symbol_Table_Entry;
        Subprogram_Entry_Is_Body : in Boolean) return Ref_Symbol_Table_Entry;


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Locate_Package_Spec
   --
   --  <Description>
   --
   --    The current scope is searched to locate the package specification
   --    associated with the package name.
   --
   --==========================================================================

   function Locate_Package_Spec
       (Context      : in Context_Control.Context_Definition;
        Package_Name : in Ref_String) return Ref_Symbol_Table_Entry;


   --==========================================================================
   --  <Subprogram Name>
   --
   --    All_Specs_Have_Bodies
   --
   --  <Description>
   --
   --    The package is examined to determine whether all declarations
   --    within the package spec have a body within the package body.
   --
   --==========================================================================

   procedure All_Specs_Have_Bodies
       (Package_Spec_Decls : in Ref_Statement_Database);


end Declaration_Support_Routines;
