--|$Log:	dsubsup.a,v $
--|Revision 8.0  91/01/17  10:41:09  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Derived_Subprogram_Support
--
---------------------------------------------------------------
--
--
-- Abstract
--
--    This package contains a set of routines that support the generation
--    of derived subprograms.  During the processing of package specifications,
--    a list of derivable subprograms is associated with each appropriate
--    type.  When additional types are derived from these types, the list
--    of derivable subprograms is traversed and a derived subprogram entry
--    is created and entered appropriately at the location where it was
--    derived.
--
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------

with Symbol_Table_Structures;
use Symbol_Table_Structures;

--with Statement_Database;
--use Statement_Database;

with Context_Control;

package Derived_Subprogram_Support is

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Determine_Derivable_Subprograms
   --
   --  <Description>
   --
   --    This procedure determines the derivable subprograms associated with
   --    a package specification.  For each subprogram within the package
   --    specification declarations, mark the type definitions within the
   --    package spec that are referenced in either the formal parameters or
   --    the return value of a function.
   --
   --==========================================================================

   procedure Determine_Derivable_Subprograms
       (Package_Symtbl_Entry : in Ref_Symbol_Table_Entry);


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Derive_Subprograms
   --
   --  <Description>
   --
   --    This procedure creates a derived subprogram for the list of derivable
   --    subprograms associated with the parent type.  The parent type in the
   --    specification for the subprogram should be substituted with the
   --    derived type for each occurrence of the parent type in the formal
   --    parameters or the return type.
   --
   --==========================================================================

   procedure Derive_Subprograms
       (Context                   : in out Context_Control.Context_Definition;
        Derived_Type_Symtbl_Entry : in Ref_Symbol_Table_Entry);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Add_Subprogram_Entry_To_List
   --
   --  <Description>
   --
   --    This procedure adds the subprogram
   --    hidden or visible.  This function returns True if the derived
   --    subprogram will be hidden; False otherwise.
   --    This procedure adds a derived subprogram onto a subprogram list.
   --    Add the subprogram onto the derivable subprogram list for the type or
   --    derived type.  Also, add the derived program onto the list for a
   --    package spec which contains all the subprograms derived within the
   --    package spec.  Do not add onto the list if the subprogram entry
   --    already exists on the list (will be at the head of the list).
   --
   --==========================================================================
   procedure Add_Subprogram_Entry_To_List
       (Derived_Subprogram_Entry : in Ref_Symbol_Table_Entry;
        Symtbl_Entry             : in Ref_Symbol_Table_Entry);

end Derived_Subprogram_Support;
