--|$Log:	error.a,v $
--|Revision 8.0  91/01/17  10:41:11  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Error_Handler
--
---------------------------------------------------------------
--
--
-- Abstract
--
--    This package provides the display of error messages (fatal, warnings,
--    diagnostic, and internal), as well as routines to restore internal
--    ACE data structures when errors occur within the middle of
--    Ada constructs.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------

--with Symbol_Table_Structures;
--use Symbol_Table_Structures;

with Ace_Universal_Types;
use Ace_Universal_Types;

with Context_Control;

package Error_Handler is

   type Message_Level_Enumeration is (Warning, Diagnostic, Fatal, Internal);


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Show_Error
   --
   --  <Description>
   --
   --    This procedure displays the text associated with an error message,
   --    followed by any optional message text.  Errors that are fatal user
   --    errors then raise a User_Error exception which is the mechanism
   --    for indicating that error recovery should be performed by ACE.
   --
   --  <Exceptions Raised>
   --
   --    User_Error is raised if the message level of the error is a Fatal_Error
   --
   --==========================================================================

   procedure Show_Error (Error_Number     : in Ace_Integer;
                         Message_Level    : in Message_Level_Enumeration;
                         Optional_Message : in String := "");

   -- Notes:
   --   Parameter "Optional_Message" might typically contain the subprogram
   --   name which is calling this routine


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Reset_From_Subprogram_Or_Package_Declaration
   --
   --  <Description>
   --
   --    This procedure performs restoring actions for the command language
   --    interpreter when ACE needs to be reset.  Actions taken are dependent
   --    upon whether parsing is occurring within a declarative region when
   --    restoration is required.
   --
   --==========================================================================

   procedure Reset_From_Subprogram_Or_Package_Declaration
       (Context : in out Context_Control.Context_Definition);

end Error_Handler;
