--|$Log:	files.a,v $
--|Revision 8.0  91/01/17  10:41:12  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Ace_File_System
--
---------------------------------------------------------------
--
--
-- Abstract
--
--  This package contains the builtin routines for the ACE file
-- system.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------

with Ace_Universal_Types;
use Ace_Universal_Types;

--with Ooe;
--use Ooe;

with Ace_Text_Io;
use Ace_Text_Io;

package Ace_File_System is

   -- the File_Object type is derived from File_Type, and
   -- has all the operations of File_Type, except where new
   -- operations are defined.
   subtype File_Object is File_Type;

   -- The mode of object are similar to Text_Io File_Type modes.
   type Object_Mode is (In_Object, Out_Object);

   -- Used in indicating the method of execution.
   type Execution_Mode is (Foreground, Background);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create
   --
   --  <Description>
   --
   --    Create routine for File_Objects (similar to Text_Io.)
   --
   --  <Exceptions Raised>
   --
   --    Same as Text_Io Create.
   --
   --==========================================================================
   procedure Create (Obj  : in out File_Object;
                     Mode : in Object_Mode := Out_Object;
                     Name : in String      := "";
                     Form : in String      := "");

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Open
   --
   --  <Description>
   --
   --    Open routine for File_Objects (similar to Text_Io.)
   --
   --  <Exceptions Raised>
   --
   --    Same as Text_Io Open.
   --
   --==========================================================================
   procedure Open (Obj  : in out File_Object;
                   Mode : in Object_Mode;
                   Name : in String;
                   Form : in String := "");

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Delete
   --
   --  <Description>
   --
   --    Delete routine for File_Objects (similar to Text_Io.)
   --
   --  <Exceptions Raised>
   --
   --    Same as Text_Io Delete.
   --
   --==========================================================================
   procedure Delete (Obj : in out File_Object);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Rename
   --
   --  <Description>
   --
   --    This routine will rename a file from one name to another.
   --
   --  <Exceptions Raised>
   --
   --    Text_Io exceptions
   --
   --==========================================================================
   procedure Rename (Obj1, Obj2 : in out File_Object);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Copy
   --
   --  <Description>
   --
   --    This routine will copy the contents of one file to another file.
   --
   --  <Exceptions Raised>
   --
   --    Text_Io exceptions
   --
   --==========================================================================
   procedure Copy (Obj1, Obj2 : in out File_Object);


   --==========================================================================
   --  <Subprogram Name>
   --
   --    List
   --
   --  <Description>
   --
   --    This routine will list the contents of an object (directory, or
   --    text);
   --
   --  <Exceptions Raised>
   --
   --    Text_Io exceptions
   --
   --==========================================================================
   procedure List (Obj   : in File_Object;
                   Which : in Ace_Integer);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Change_Directory
   --
   --  <Description>
   --
   --    The following routine will change the directory to which the users
   --    actions will default.  (i.e., if the user doesn't specify the full
   --    path name on an object the system will use the current working
   --    directory).
   --
   --  <Exceptions Raised>
   --
   --    Text_Io exceptions
   --
   --==========================================================================
   procedure Change_Directory (Obj : in File_Object);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Edit
   --
   --  <Description>
   --
   --    Edit will call the host operating system edit command on
   --    a specified file.
   --
   --  <Exceptions Raised>
   --
   --    Text_Io exceptions
   --
   --==========================================================================
   procedure Edit (Obj : in out File_Object;
                   How : in Execution_Mode);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Compile
   --
   --  <Description>
   --
   --    Compile will call the host operating system compile command on
   --    a specified file.
   --
   --  <Exceptions Raised>
   --
   --    Text_Io exceptions
   --
   --==========================================================================
   procedure Compile (Obj  : in File_Object;
                      Form : in String);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Close
   --
   --  <Description>
   --
   --    Close routine for File_Objects (similar to Text_Io.)
   --
   --  <Exceptions Raised>
   --
   --    Same as Text_Io Close.
   --
   --==========================================================================
   procedure Close (Obj : in out File_Object);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Stubber
   --
   --  <Description>
   --
   --    This is the stubber that is found in the repository.
   --    It has been modified slightly to become integrated with the
   --    ACE File_System.
   --
   --==========================================================================
   procedure Stubber (Obj1, Obj2 : in out File_Object);


end Ace_File_System;
