--|$Log:	get.a,v $
--|Revision 8.0  91/01/17  10:41:13  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Get
------------------------------------------------------------
--
--
-- Abstract
--
--    This package contains a set of GET routines that provide access
--    to statement database nodes.  These routines support access to both
--    statement kind and expression kind statement database nodes.
--
-- Update Log - :
--
-- End of Header
-------------------------------------------------------------------------
-- =======================================================================

with Statement_Database;
use Statement_Database;

with Symbol_Table_Structures;
use Symbol_Table_Structures;

with Storage_Management;
use Storage_Management;

with Ace_Universal_Types;
use Ace_Universal_Types;

with Literals;
use Literals;

package Node_Get_Routines is


   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Database_Node
   --
   --    Last Mod:    Thu Sep  7 09:44:22 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --    Returns the expanded name's real statement_database node.
   --
   --  --------------------------------------------------------------------
   function Get_Database_Node
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Is_Statement_Node
   --
   --  <Description>
   --
   --    Returns TRUE if the given NODE is a Statement Node
   --
   --=======================================================================

   function Is_Statement_Node (Node : in Ref_Statement_Database) return Boolean;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Is_Expression_Node
   --
   --  <Description>
   --
   --    Returns TRUE if the given NODE is an Expression Node
   --
   --=======================================================================

   function Is_Expression_Node
       (Node : in Ref_Statement_Database) return Boolean;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Is_Attribute_Tick
   --
   --  <Description>
   --
   --    Returns TRUE if the given NODE is an Attribute_Tick expression
   --
   --=======================================================================

   function Is_Attribute_Tick (Node : in Ref_Statement_Database) return Boolean;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Is_Removed
   --
   --  <Description>
   --
   --   Return whether the symbol table entry associated with node has
   --    been "removed" from the symbol table
   --=======================================================================

   function Is_Removed (Node : in Ref_Statement_Database) return Boolean;


   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Next_Statement
   --
   --  <Description>
   --
   --    Returns the Next Statement Database Node
   --
   --=======================================================================

   function Get_Next_Statement
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Next_Statement

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Holder
   --
   --  <Description>
   --
   --    Returns the "holder" (temporary) node of the current Node
   --
   --=======================================================================

   function Get_Holder
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Holder
   -- Next_Expression

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Comments_List
   --
   --  <Description>
   --
   --    Returns the comments list for the current Node
   --
   --=======================================================================

   function Get_Comments_List
       (Node : in Ref_Statement_Database) return Ref_Comment;
   -- Comment_Holder

   function Get_Comments_List (Node : in Ref_Comment) return Ref_Comment;
   -- Next

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Comment
   --
   --  <Description>
   --
   --    Returns the comment(s) for the current Node
   --
   --=======================================================================

   function Get_Comment (Node : in Ref_Comment) return String;
   -- Comment

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Should_Suspend_Evaluation_At_This_Statement
   --
   --  <Description>
   --
   --    Returns the whether evaluation should be suspend at current Node
   --
   --=======================================================================
   function Should_Suspend_Evaluation_At_This_Statement
       (Node : in Ref_Statement_Database) return Boolean;
   -- Suspend_Evaluation_At_This_Statement

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Statement_Number
   --
   --  <Description>
   --
   --    Returns the statement number for the current Node
   --
   --=======================================================================
   function Get_Statement_Number
       (Node : in Ref_Statement_Database) return Ace_Natural;
   -- Statement_Number

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Subprogram_Info
   --
   --  <Description>
   --
   --    Returns the subprogram information node for the current Node
   --
   --=======================================================================

   function Get_Subprogram_Info
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Procedure_Info
   -- Lp_Object (for function calls)

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Symbol_Table_Entry
   --
   --  <Description>
   --
   --    Returns the symbol table entry for the current Node
   --
   --=======================================================================

   function Get_Symbol_Table_Entry
       (Node : in Ref_Statement_Database) return Ref_Symbol_Table_Entry;
   -- Symbol_Table_Entry
   -- Terminal_Symbol

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Lhs
   --
   --  <Description>
   --
   --    Returns the LHS for the current Assignment Statement Node
   --
   --=======================================================================

   function Get_Lhs
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Lhs

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Rhs
   --
   --  <Description>
   --
   --    Returns the RHS for the current Assignment Statement Node
   --
   --=======================================================================

   function Get_Rhs
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Rhs

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Condition
   --
   --  <Description>
   --
   --    Returns the condition expression for the current Node
   --
   --=======================================================================

   function Get_Condition
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- If_Condition
   -- Exit_Condition

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Loop_Scheme
   --
   --  <Description>
   --
   --    Returns the loop scheme for the current Node
   --
   --=======================================================================

   function Get_Loop_Scheme
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Loop_Scheme

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_For_Variable
   --
   --  <Description>
   --
   --    Returns the FOR variable for the current Node
   --
   --=======================================================================

   function Get_For_Variable
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- For_Variable

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_For_Range
   --
   --  <Description>
   --
   --    Returns the FOR range for the current Node
   --
   --=======================================================================

   function Get_For_Range
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- For_Range

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Name_Range
   --
   --  <Description>
   --
   --    Returns the name range for the current Node
   --
   --=======================================================================

   function Get_Name_Range
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Name_Range

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Head_Of_Statements_List
   --
   --  <Description>
   --
   --    Returns the head of the statements list for the current Node
   --
   --=======================================================================

   function Get_Head_Of_Statements_List
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Loop_Body
   -- Block_Statements

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Head_Of_True_Task_Statements_List
   --
   --  <Description>
   --
   --    Returns the head of the true task statement for the current Node
   --
   --=======================================================================

   function Get_Head_Of_True_Task_Statements_List
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- True_Task

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Head_Of_False_Task_Statements_List
   --
   --  <Description>
   --
   --    Returns the head of the false task statement for the current Node
   --
   --=======================================================================

   function Get_Head_Of_False_Task_Statements_List
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- False_Task

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Expression
   --
   --  <Description>
   --
   --    Returns the expression for the current Node
   --
   --=======================================================================

   function Get_Expression
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Case_Expression
   -- Return_Expression
   -- Item_Expression
   -- Init_Expression

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Subprogram_Reference
   --
   --  <Description>
   --
   --    Returns the subprogram reference for the current Node
   --
   --=======================================================================

   function Get_Subprogram_Reference
       (Node : in Ref_Statement_Database) return Ref_Symbol_Table_Entry;
   -- Subprogram_Reference

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Alternatives_List
   --
   --  <Description>
   --
   --    Returns the CASE alternatives list for the current Node
   --
   --=======================================================================

   function Get_Alternatives_List
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Case_Alternatives_List

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Alternative_Choice
   --
   --  <Description>
   --
   --    Returns the CASE alternative choice(s) for the current Node
   --
   --=======================================================================

   function Get_Alternative_Choice
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Case_Alternative_Choice

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Alternative_Statements
   --
   --  <Description>
   --
   --    Returns the CASE alternative statements for the current Node
   --
   --=======================================================================

   function Get_Alternative_Statements
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Case_Alternative_Statements

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Next_Alternative
   --
   --  <Description>
   --
   --    Returns the next CASE alternative for the current Node
   --
   --=======================================================================

   function Get_Next_Alternative
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Next_Case_Alternative

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Label_Entry
   --
   --  <Description>
   --
   --    Returns the label entry for the current Node
   --
   --=======================================================================

   function Get_Label_Entry
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Label_Entry

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Goto_Label
   --
   --  <Description>
   --
   --    Returns the goto label for the current Node
   --
   --=======================================================================

   function Get_Goto_Label
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Goto_Label

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Name
   --
   --  <Description>
   --
   --    Returns the name for the current Node
   --
   --=======================================================================

   function Get_Name
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Loop_Name
   -- Exit_Loop_Name
   -- Pragma_Name

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Block_Declarations
   --
   --  <Description>
   --
   --    Returns the Block Declarations for the current Node
   --
   --=======================================================================

   function Get_Block_Declarations
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Block_Declarations

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Exception_Handler
   --
   --  <Description>
   --
   --    Returns the Block Declarations for the current Node
   --
   --=======================================================================
   function Get_Exception_Handler
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Exception_Handler

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Name_List
   --
   --  <Description>
   --
   --    Returns the head of the name list for the current Node
   --
   --=======================================================================

   function Get_Name_List
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Unit_Name_List
   -- Name_List

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Parameter_List
   --
   --  <Description>
   --
   --    Returns the parameter list for the current Node
   --
   --=======================================================================

   function Get_Parameter_List
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Actual_Procedure_Parameters_List
   -- Lp_Items_List (for function calls)

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Argument_List
   --
   --  <Description>
   --
   --    Returns the argument list for the current Node
   --
   --=======================================================================

   function Get_Argument_List
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Pragma_Arg

   --

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Value
   --
   --  <Description>
   --
   --    Returns the value for the current Node
   --
   --=======================================================================

   function Get_Value
       (Node : in Ref_Statement_Database) return Ref_Data_Storage;
   -- Node_Value

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Type
   --
   --  <Description>
   --
   --    Returns the type for the current Node
   --
   --=======================================================================

   function Get_Type
       (Node : in Ref_Statement_Database) return Ref_Symbol_Table_Entry;
   -- Node_Type

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Is_A_Default_Expression
   --
   --  <Description>
   --
   --    Returns TRUE if the given NODE is a default expression
   --
   --=======================================================================
   function Is_A_Default_Expression
       (Node : in Ref_Statement_Database) return Boolean;
   -- Is_Default_Expression

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Is_Type_Universal
   --
   --  <Description>
   --
   --    Returns TRUE if the given NODE is a universal type
   --
   --=======================================================================
   function Is_Type_Universal (Node : in Ref_Statement_Database) return Boolean;
   -- Is_Node_Type_Universal

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Symbolic_Name
   --
   --  <Description>
   --
   --    Returns the symbolic name for the current Node
   --
   --=======================================================================

   function Get_Symbolic_Name
       (Node : in Ref_Statement_Database) return Ref_String;
   -- Symbol_Name
   -- Character_Symbol
   -- String_Symbol
   -- Constant_Symbol

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Node_Kind
   --
   --  <Description>
   --
   --    Returns the kind of the current Node
   --
   --=======================================================================

   function Get_Node_Kind
       (Node : in Ref_Statement_Database) return Expression_Enumeration;
   -- Dot_Node_Kind
   -- Lp_Node_Kind

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Object
   --
   --  <Description>
   --
   --    Returns the object for the current Node
   --
   --=======================================================================

   function Get_Object
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Dot_Object
   -- Lp_Object

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Dot_Component
   --
   --  <Description>
   --
   --    Returns the dot component for the current Node
   --
   --=======================================================================

   function Get_Dot_Component
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Dot_Component

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_List_Of_Items
   --
   --  <Description>
   --
   --    Returns the head of the list of items for the current Node
   --
   --=======================================================================

   function Get_List_Of_Items
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Lp_Items_List

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Next_List_Item
   --
   --  <Description>
   --
   --    Returns the next List Item for the current Node
   --
   --=======================================================================

   function Get_Next_List_Item
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Next_List_Item

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Aggregate_Choice
   --
   --  <Description>
   --
   --    Returns the aggregate choice for the current Node
   --
   --=======================================================================

   function Get_Aggregate_Choice
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Aggregate_Choice

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Aggregate_Expression
   --
   --  <Description>
   --
   --    Returns the Aggregate Expression for the current Node
   --
   --=======================================================================

   function Get_Aggregate_Expression
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Aggregate_Expression

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Lower_Bound
   --
   --  <Description>
   --
   --    Returns the Lower Bound for the current Node
   --
   --=======================================================================

   function Get_Lower_Bound
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Lower_Bound

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Upper_Bound
   --
   --  <Description>
   --
   --    Returns the Upper Bound for the current Node
   --
   --=======================================================================

   function Get_Upper_Bound
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Upper_Bound

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Attribute_Prefix
   --
   --  <Description>
   --
   --    Returns the Attribute Prefix Node for the current Node
   --
   --=======================================================================

   function Get_Attribute_Prefix
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Attribute_Prefix

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Attribute
   --
   --  <Description>
   --
   --    Returns the Attribute Statement Database Node for the current Node
   --
   --=======================================================================

   function Get_Attribute
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Attribute

   --

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Parent_Node
   --
   --  <Description>
   --
   --    Returns the Parent Statement Database Node for the current Node
   --
   --=======================================================================

   function Get_Parent_Node
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;
   -- Parent_Node

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Expression_Kind
   --
   --  <Description>
   --
   --    Returns the Expression_Kind Statement Database Node
   --    for the current Node
   --
   --=======================================================================

   function Get_Expression_Kind
       (Node : in Ref_Statement_Database) return Expression_Enumeration;
   -- Expression_Kind

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Reference_Count
   --
   --  <Description>
   --
   --
   --=======================================================================
   function Get_Reference_Count
       (Node : in Ref_Statement_Database) return Ace_Positive;
   -- Reference_Count


   -- pragma inline(Is_Statement_Node);
   -- pragma inline(Is_Expression_Node);
   -- pragma inline(Is_Attribute_Tick);
   -- pragma inline(Get_Next_Statement);
   -- pragma inline(Get_Holder);
   -- pragma inline(Get_Comments_List);
   -- pragma inline(Get_Comments_List);
   -- pragma inline(Get_Comment);
   -- pragma inline(Should_Suspend_Evaluation_At_This_Statement);
   -- pragma inline(Get_Statement_Number);
   -- pragma inline(Get_Subprogram_Info);
   -- pragma inline(Get_Symbol_Table_Entry);
   -- pragma inline(Get_Lhs);
   -- pragma inline(Get_Rhs);
   -- pragma inline(Get_Condition);
   -- pragma inline(Get_Loop_Scheme);
   -- pragma inline(Get_For_Variable);
   -- pragma inline(Get_For_Range);
   -- pragma inline(Get_Name_Range);
   -- pragma inline(Get_Head_Of_Statements_List);
   -- pragma inline(Get_Head_Of_True_Task_Statements_List);
   -- pragma inline(Get_Head_Of_False_Task_Statements_List);
   -- pragma inline(Get_Expression);
   -- pragma inline(Get_Subprogram_Reference);
   -- pragma inline(Get_Alternatives_List);
   -- pragma inline(Get_Alternative_Choice);
   -- pragma inline(Get_Alternative_Statements);
   -- pragma inline(Get_Next_Alternative);
   -- pragma inline(Get_Label_Entry);
   -- pragma inline(Get_Goto_Label);
   -- pragma inline(Get_Name);
   -- pragma inline(Get_Block_Declarations);
   -- pragma inline(Get_Exception_Handler);
   -- pragma inline(Get_Name_List);
   -- pragma inline(Get_Parameter_List);
   -- pragma inline(Get_Argument_List);
   -- pragma inline(Get_Value);
   -- pragma inline(Get_Type);
   -- pragma inline(Is_A_Default_Expression);
   -- pragma inline(Is_Type_Universal);
   -- pragma inline(Get_Symbolic_Name);
   -- pragma inline(Get_Node_Kind);
   -- pragma inline(Get_Object);
   -- pragma inline(Get_Dot_Component);
   -- pragma inline(Get_List_Of_Items);
   -- pragma inline(Get_Next_List_Item);
   -- pragma inline(Get_Aggregate_Choice);
   -- pragma inline(Get_Aggregate_Expression);
   -- pragma inline(Get_Lower_Bound);
   -- pragma inline(Get_Upper_Bound);
   -- pragma inline(Get_Attribute_Prefix);
   -- pragma inline(Get_Attribute);
   -- pragma inline(Get_Parent_Node);
   -- pragma inline(Get_Expression_Kind);
   -- pragma inline(Get_Reference_Count);

end Node_Get_Routines;
