--|$Log:	help.a,v $
--|Revision 8.0  91/01/17  10:41:15  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Help_Routines
--
---------------------------------------------------------------
--
--
-- Abstract
--
--   This package contains the definitions and support routines for Help
--   Comments which are embedded in ACE code.  On the surface Help Comments
--   are very similar to standard Ada comments.

-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
-- ==========================================================================

package Help_Pkg is
   type String_Ptr is access String;

   -- Data structure to hold Help_Comments

   type Help;

   type Ref_Help is access Help;

   type Help is
      record
         Comment : String_Ptr;
         Next    : Ref_Help;
      end record;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Process_Help
   --
   --  <Description>
   --
   --    This routine stores up help comments, so that they may be
   --    associated with the nearest unit.  This needs some
   --    improvement.  Help comments sometimes move from their original
   --    position to a position next to the nearest unit.
   --
   --==========================================================================
   procedure Process_Help (Comment : String);


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Get_Help
   --
   --  <Description>
   --
   --    This routine returns the stored help information, so that it may be
   --    associated with the nearest unit.  After it is called the
   --    stored help comments are emptied, so that they are not associated
   --    twice.
   --
   --==========================================================================
   function Get_Help return Ref_Help;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Clear_Help
   --
   --  <Description>
   --
   --    Clear_Help will clear out the help buffer.
   --
   --==========================================================================
   procedure Clear_Help;
   -- -------------------------------------------------------------------------

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Get_Help_List
   --
   --  <Description>
   --
   --    Get_Help_List returns the next help_comment in the list.
   --
   --==========================================================================
   function Get_Help_List (Node : in Ref_Help) return Ref_Help;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Get_Help_Info
   --
   --  <Description>
   --
   --    Get_Help_Info will return a line of help information from the given
   --   Help_Node.  The Strip flag will instruct the routine to strip off
   --   the help_comment symbol "--?" from the string.
   --==========================================================================
   function Get_Help_Info (Help_Node : in Ref_Help;
                           Strip     : in Boolean) return String;

   -- ------------------------------------------------------------


   procedure Nq_Pkg_Help (Help_Comments : in Ref_Help);
   function Dq_Pkg_Help return Ref_Help;

end Help_Pkg;
