--|$Log:	lexact.a,v $
--|Revision 8.0  91/01/17  10:41:16  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Lexacts
--
---------------------------------------------------------------
--
--
-- Abstract
--
--   This package provides the interface from the scanner to
--   the parser (with the exception of the token returned from
--   the scanner).  Routines included deal with the textual
--   representation of tokens, and dealing with the CR which
--   in ACE is a special token (it's meaning changes while the
--   user is typing, and is used to help make ACE interactive).
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
package Lexacts is

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Display
   --
   --  <Description>
   --
   --    This routine is used to convert the internal representation of
   --    tokens into the string equivalent.
   --
   --==========================================================================
   function Display (Tok : Integer) return String;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Pop_Eol
   --
   --  <Description>
   --
   --    This routine is used to keep track of syntatic nesting
   --    of constructs.  So that ACE can tell when a construct is
   --    finished. This routine is called everytime
   --    the user exits a construct.
   --    When the stack is empty and the use hit a newline,
   --    then the constuct is over.
   --
   --==========================================================================
   procedure Pop_Eol;
   pragma INLINE (Pop_Eol);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Push_Eol
   --
   --  <Description>
   --
   --    This routine is used to keep track of syntatic nesting
   --    of constructs.  So that ACE can tell when a construct is
   --    finished. This routine is called everytime
   --    the user enters a construct.
   --    When the stack is empty and the use hit a newline,
   --    then the constuct is over.
   --
   --==========================================================================
   procedure Push_Eol;
   pragma INLINE (Push_Eol);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Reset_Eol
   --
   --  <Description>
   --
   --    This routine is used to initialize the stack used in the previous
   --    two procedures.
   --==========================================================================
   procedure Reset_Eol;
   pragma INLINE (Reset_Eol);

   Eol_Flag : Integer;

end Lexacts;
