--|$Log:	misc.a,v $
--|Revision 8.0  91/01/17  10:41:17  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Misc
--
---------------------------------------------------------------
--
--
-- Abstract
--
--   This routine handles miscellaneous tasks (such as changing the
--   users prompt from "ACE> " to "    |" when a construct is being
--   continued on more than one line).
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
package Misc is
   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Prompt
   -- 
   --    Last Mod:    Thu May 24 11:56:01 1990
   --    Author:      William P Loftus <wpl@huey>
   -- 
   --  <Description> 
   --    returns a string that represents the current prompt
   -- 
   --  --------------------------------------------------------------------
   function Prompt return String;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    prompt
   --
   --  <Description>
   --
   --    Prompts the users with the given parameter.
   --
   --==========================================================================
   procedure Prompt (Str : String);


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Prompt_On
   --
   --  <Description>
   --
   --    Set a flag so the that the prompt will be send to the user's screen.
   --
   --==========================================================================
   procedure Set_Prompt_On;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Prompt_Off
   --
   --  <Description>
   --
   --    Set a flag so the that the prompt will not be given.
   --
   --==========================================================================
   procedure Set_Prompt_Off;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Main_Prompt
   --
   --  <Description>
   --
   --    Returns a string containing the Main prompt.
   --
   --==========================================================================
   function Main_Prompt return String;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Main_Prompt
   --
   --  <Description>
   --
   --    Sets the main user prompt.
   --
   --==========================================================================
   procedure Set_Main_Prompt (Str : String);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Continuation_Prompt
   --
   --  <Description>
   --
   --    Returns a string containing the continuation prompt.
   --
   --==========================================================================
   function Continuation_Prompt return String;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Continuation_Prompt
   --
   --  <Description>
   --
   --    Sets the continuation prompt.
   --
   --==========================================================================
   procedure Set_Continuation_Prompt (Str : String);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Exit_On_First_Error
   --
   --  <Description>
   --
   --   Set a flag that if true will cause ACE to exit to the host O/S
   --   when the first error is detected.
   --
   --==========================================================================

   procedure Set_Exit_On_First_Error (Switch : Boolean);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Exit_On_First_Error
   --
   --  <Description>
   --
   --   If true the first error should cause ACE to exit to the host O/S.
   --
   --==========================================================================

   function Exit_On_First_Error return Boolean;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Reset_Input
   --
   --  <Description>
   --
   --   Flushes out the input left in the user's buffer and resets
   --   the parser and lexical analyzer.
   --
   --==========================================================================
   procedure Reset_Input;

end Misc;
