--|$Log:	miscs.a,v $
--|Revision 8.0  91/01/17  10:41:18  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Misc_Support_Routines
--
---------------------------------------------------------------
--
--
-- Abstract
--
--    This package contains a collection of miscellaneous routines that
--    did not fall logically within any of the other packages for the
--    Ada Command Language Interpreter.  These routines include conversion
--    routines between a reference to a statement database node and a fake
--    reference to a statement database node; routines to determine whether
--    a particular data structure (e.g. statement database node, symbol table
--    entry, ...) is empty; routines to acquire the discriminant value from
--    a record; and routine to duplicate a statement database node.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------

with Statement_Database;
use Statement_Database;

with Symbol_Table_Structures;
use Symbol_Table_Structures;

with Help_Pkg;
use Help_Pkg;

with Storage_Management;
use Storage_Management;

with Ace_Universal_Types;
use Ace_Universal_Types;

with Literals;
use Literals;

package Misc_Support_Routines is


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Convert_Fake
   --
   --  <Description>
   --
   --    A fake reference to the statement database is converted into an
   --    actual reference to the statement database through use of the Ada
   --    UNCHECKED_CONVERSION generic subprogram.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    [none]
   --
   --  <Exceptions Handled>
   --
   --    [none]
   --==========================================================================

   function Convert_Fake
       (Node : in Ref_Statement_Database_Fake) return Ref_Statement_Database;


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Convert_To_Fake
   --
   --  <Description>
   --
   --    An actual reference to the statement database is converted into a
   --    fake reference to the statement database through use of the Ada
   --    UNCHECKED_CONVERSION generic subprogram.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    [none]
   --
   --  <Exceptions Handled>
   --
   --    [none]
   --==========================================================================

   function Convert_To_Fake
       (Node : in Ref_Statement_Database) return Ref_Statement_Database_Fake;


   -- -------------------------------------------------------------------------

   -- ::::::::::::::::::::::::::::::::::::::::::::::::::::
   -- : Tests for Emptiness of different data structures :
   -- ::::::::::::::::::::::::::::::::::::::::::::::::::::

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Is_Empty
   --
   --  <Description>
   --
   --    Returns True if the statement database node is empty;
   --    otherwise returns False.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    [none]
   --
   --  <Exceptions Handled>
   --
   --    [none]
   --==========================================================================

   function Is_Empty (Node : in Ref_Statement_Database) return Boolean;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Is_Empty
   --
   --  <Description>
   --
   --    Returns True if the symbol table entry is empty;
   --    otherwise returns False.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    [none]
   --
   --  <Exceptions Handled>
   --
   --    [none]
   --==========================================================================

   function Is_Empty (Node : in Ref_Symbol_Table_Entry) return Boolean;


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Is_Empty
   --
   --  <Description>
   --
   --    Returns True if the comment list entry is empty;
   --    otherwise returns False.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    [none]
   --
   --  <Exceptions Handled>
   --
   --    [none]
   --==========================================================================

   function Is_Empty (Node : in Ref_Comment) return Boolean;


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Is_Empty
   --
   --  <Description>
   --
   --    Returns True if the data storage location is empty;
   --    otherwise returns False.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    [none]
   --
   --  <Exceptions Handled>
   --
   --    [none]
   --==========================================================================

   function Is_Empty (Node : in Ref_Data_Storage) return Boolean;


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Is_Empty
   --
   --  <Description>
   --
   --    Returns True if the string pool element is empty;
   --    otherwise returns False.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    [none]
   --
   --  <Exceptions Handled>
   --
   --    [none]
   --==========================================================================

   function Is_Empty (Node : in Ref_String) return Boolean;


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Is_Empty
   --
   --  <Description>
   --
   --    Returns True if the Ace_Natural integer element is empty;
   --    otherwise returns False.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    [none]
   --
   --  <Exceptions Handled>
   --
   --    [none]
   --==========================================================================

   function Is_Empty (Node : in Ace_Natural) return Boolean;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Is_Empty
   --
   --  <Description>
   --
   --    Returns True if the help information storage location is empty;
   --    otherwise returns False.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    [none]
   --
   --  <Exceptions Handled>
   --
   --    [none]
   --==========================================================================

   function Is_Empty (Node : in Ref_Help) return Boolean;


   --==========================================================================
   -- return an empty value (usually a null)
   --==========================================================================
   function Empty return Ref_Statement_Database;
   function Empty return Ref_Symbol_Table_Entry;
   function Empty return Ref_Data_Storage;
   function Empty return Ref_String;
   function Empty return Ref_Help;
   function Empty return Ref_Comment;
   function Empty return Ace_Natural;


   --==========================================================================
   -- free node or entire substructure
   --==========================================================================
   type Depth_Kind is
      (One_Level,
       One_Level_And_Constants,
       Entire_Substructure,
       Entire_Substructure_And_Constants);
   procedure Free (Item  : in out Ref_Statement_Database;
                   Depth : Depth_Kind := One_Level);
   procedure Free (Item  : in out Ref_Symbol_Table_Entry;
                   Depth : Depth_Kind := One_Level);
   procedure Free (Item  : in out Ref_Data_Storage;
                   Depth : Depth_Kind := Entire_Substructure_And_Constants);
   -----
   procedure Free (Item  : in out Ref_Comment;
                   Depth : Depth_Kind := One_Level);
   procedure Free (Item  : in out Ref_Help;
                   Depth : Depth_Kind := One_Level);
   procedure Free (Item  : in out Ref_String;
                   Depth : Depth_Kind := One_Level);


   -- -------------------------------------------------------------------------

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Check_Stmt_Node
   --
   --  <Description>
   --
   --    Returns the kind of statement that is represented in the
   --    statement database node
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    Program_Error
   --
   --  <Exceptions Handled>
   --
   --    Constraint_Error
   --==========================================================================

   function Check_Stmt_Node
       (Node : in Ref_Statement_Database) return Statement_Enumeration;


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Check_Expr_Node
   --
   --  <Description>
   --
   --    Returns the kind of expression that is represented in the
   --    statement database node
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    Program_Error
   --
   --  <Exceptions Handled>
   --
   --    Constraint_Error
   --==========================================================================

   function Check_Expr_Node
       (Node : in Ref_Statement_Database) return Expression_Enumeration;


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Duplicate_Node
   --
   --  <Description>
   --
   --    A duplicate copy of the statement database node is created.
   --    This routine currently only supports the duplication of statement
   --    database nodes that are expresssion nodes of kind terminal_node.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    [none]
   --
   --  <Exceptions Handled>
   --
   --    [none]
   --==========================================================================

   function Duplicate_Node
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;


   -- -------------------------------------------------------------------------

   function Anonymous_Name return String;

   -- pragma inline(Convert_Fake);
   -- pragma inline(Convert_To_Fake);
   -- pragma inline(Is_Empty);
   -- pragma inline(Empty);
   -- pragma inline(Free);
   -- pragma inline(Check_Stmt_Node);
   -- pragma inline(Check_Expr_Node);
   -- pragma inline(Duplicate_Node);

end Misc_Support_Routines;
