--|$Log:	set.a,v $
--|Revision 8.0  91/01/17  10:41:24  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Node_Set_Routines
--
---------------------------------------------------------------
--
--
-- Abstract
--
--    These routines are statment database node SET routines
--    for each node/component kind of the statement database.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
-- ==========================================================================

with Statement_Database;
use Statement_Database;

with Symbol_Table_Structures;
use Symbol_Table_Structures;

with Storage_Management;
use Storage_Management;

with Ace_Universal_Types;
use Ace_Universal_Types;

with Literals;
use Literals;

package Node_Set_Routines is

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_Database_Node
   --
   --    Last Mod:    Thu Sep  7 09:49:04 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --    Set the real statement database node for an expanded name.
   --
   --  --------------------------------------------------------------------
   procedure Set_Database_Node (Node : in Ref_Statement_Database;
                                Item : in Ref_Statement_Database);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Various
   --
   --  <Description>
   --
   --    These routines operate on the Statement_Database.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None
   --
   --  <Exceptions Handled>
   --
   --    None
   --==========================================================================

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Next_Statement
   --
   --  <Description>
   --
   --    This procedure will set the Next_Statement of a Statement node in the
   --  Statement Database to Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None
   --
   --  <Exceptions Handled>
   --
   --    None
   --==========================================================================

   procedure Set_Next_Statement (Node : in Ref_Statement_Database;
                                 Item : in Ref_Statement_Database);
   -- Next_Statement

   procedure Set_Holder (Node : in Ref_Statement_Database;
                         Item : in Ref_Statement_Database);
   -- Holder
   -- Next_Expression

   --=======================================================================
   --  <Subprogram Name>
   --
   --    UnSet_Suspend_Evaluation_At_This_Statement
   --
   --  <Description>
   --
   --    Sets that evaluation should be suspended at current Node
   --
   --=======================================================================
   procedure Unset_Suspend_Evaluation_At_This_Statement
       (Node : in Ref_Statement_Database);
   -- Suspend_Evaluation_At_This_Statement

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Set_Suspend_Evaluation_At_This_Statement
   --
   --  <Description>
   --
   --    Sets that evaluation should be suspended at current Node
   --
   --=======================================================================
   procedure Set_Suspend_Evaluation_At_This_Statement
       (Node : in Ref_Statement_Database);
   -- Suspend_Evaluation_At_This_Statement

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Statement_Number
   --
   --  <Description>
   --
   --    This procedure will set the Statement Number of a Statement node
   --    in the Statement Database to number "Item".
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None
   --
   --  <Exceptions Handled>
   --
   --    None
   --==========================================================================

   procedure Set_Statement_Number (Node : in Ref_Statement_Database;
                                   Item : in Ace_Natural);
   -- Statement_Number

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Subprogram_Info
   --
   --  <Description>
   --
   --    If the Node is a procedure node, set the Procedure_info field
   --    to Item; else, when Node is a function, set the Lp_Object field
   --    to Item
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None
   --
   --  <Exceptions Handled>
   --
   --    None
   --==========================================================================

   procedure Set_Subprogram_Info (Node : in Ref_Statement_Database;
                                  Item : in Ref_Statement_Database);
   -- Procedure_Info
   -- Lp_Object (for function calls)


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Symbol_Table_Entry
   --
   --  <Description>
   --
   --    If the Node is a Statement, the Symbol table entry field is set to
   --    Item.  If the node is an Expression, the termial symbol is set
   --    to item
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    none.
   --
   --  <Exceptions Handled>
   --
   --    none.
   --==========================================================================
   procedure Set_Symbol_Table_Entry (Node : in Ref_Statement_Database;
                                     Item : in Ref_Symbol_Table_Entry);
   -- Symbol_Table_Entry
   -- Terminal_Symbol

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Lhs
   --
   --  <Description>
   --
   --    The Left hand side of an Assignment Statenemt Node is set to Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    none.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================

   procedure Set_Lhs (Node : in Ref_Statement_Database;
                      Item : in Ref_Statement_Database);
   -- Lhs

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Rhs
   --
   --  <Description>
   --
   --    The Right hand side of an Assignment Statement Node is set to Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================

   procedure Set_Rhs (Node : in Ref_Statement_Database;
                      Item : in Ref_Statement_Database);
   -- Rhs

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Condition
   --
   --  <Description>
   --
   --    The If_Condition is set to Item, if an if statement, else
   --    the Exit_Condition of an Exit Statement is set ot Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Condition (Node : in Ref_Statement_Database;
                            Item : in Ref_Statement_Database);
   -- If_Condition
   -- Exit_Condition

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Loop_Scheme
   --
   --  <Description>
   --
   --    The loop_scheme field of a loop is set to item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Loop_Scheme (Node : in Ref_Statement_Database;
                              Item : in Ref_Statement_Database);
   -- Loop_Scheme

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_For_Variable
   --
   --  <Description>
   --
   --    The variable of a for loop is set to Item.
   --
   --  <Called By>
   --
   --    External.
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_For_Variable (Node : in Ref_Statement_Database;
                               Item : in Ref_Statement_Database);
   -- For_Variable

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_For_Range
   --
   --  <Description>
   --
   --    The range of a For Loop Statement Node is set to Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    none.
   --
   --  <Exceptions Handled>
   --
   --    none.
   --==========================================================================
   procedure Set_For_Range (Node : in Ref_Statement_Database;
                            Item : in Ref_Statement_Database);
   -- For_Range

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Name_Range
   --
   --  <Description>
   --
   --    The Range_Name field of a Range Expression Node is set to Item
   --
   --  <Called By>
   --
   --    External.
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Name_Range (Node : in Ref_Statement_Database;
                             Item : in Ref_Statement_Database);
   -- Range_Name

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Head_Of_Statements_List
   --
   --  <Description>
   --
   --    Set the first statement of a Loop Statement or Block Statement
   --    to Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Head_Of_Statements_List (Node : in Ref_Statement_Database;
                                          Item : in Ref_Statement_Database);
   -- Loop_Body
   -- Block_Statements

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Head_Of_True_Task_Statements_List

   --
   --  <Description>
   --
   --    Associates the first of the statements to be executed
   --    with the true_task field of the If statement when
   --    the condition of the If_Statement is TRUE
   --
   --  <Called By>
   --
   -- External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Head_Of_True_Task_Statements_List
       (Node : in Ref_Statement_Database;
        Item : in Ref_Statement_Database);
   -- True_Task


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Head_Of_False_Task_Statements_List
   --
   --  <Description>
   --
   --    Associates the first of the statements to be executed
   --    with the false_task field of the If statement when
   --    the condition of the If_Statement is false
   --
   --  <Called By>
   --
   -- External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Head_Of_False_Task_Statements_List
       (Node : in Ref_Statement_Database;
        Item : in Ref_Statement_Database);
   -- False_Task

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Expression
   --
   --  <Description>
   --
   --    Sets the expression nodes of Case, Return, initialization
   --    and array index fields to Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Expression (Node : in Ref_Statement_Database;
                             Item : in Ref_Statement_Database);
   -- Case_Expression
   -- Return_Expression
   -- Item_Expression
   -- Init_Expression

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Subprogram_Reference
   --
   --  <Description>
   --
   --    Associates a return statement with the symbol_table entry of
   --    the subprogram which "owns" it.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Subprogram_Reference (Node : in Ref_Statement_Database;
                                       Item : Ref_Symbol_Table_Entry);
   -- Subprogram_Reference

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Alternatives_List
   --
   --  <Description>
   --
   --    Will associate a case statement with its alternatives.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Alternatives_List (Node : in Ref_Statement_Database;
                                    Item : in Ref_Statement_Database);
   -- Case_Alternatives_List

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Alternative_Choice
   --
   --  <Description>
   --
   --  Associates a choice with a case alternative
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Alternative_Choice (Node : in Ref_Statement_Database;
                                     Item : in Ref_Statement_Database);
   -- Case_Alternative_Choice

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Alternative_Statements
   --
   --  <Description>
   --
   --    Sets the case_alternative_statements field of the case alternative
   --    statement
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Alternative_Statements (Node : in Ref_Statement_Database;
                                         Item : in Ref_Statement_Database);
   -- Case_Alternative_Statements

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Next_Alternative
   --
   --  <Description>
   --
   --    Sets the Next Alternative to a case alternative to Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Next_Alternative (Node : in Ref_Statement_Database;
                                   Item : in Ref_Statement_Database);
   -- Next_Case_Alternative

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Label_Entry
   --
   --  <Description>
   --
   --    Sets the Label_entry field of a labeled statemnt to Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Label_Entry (Node : in Ref_Statement_Database;
                              Item : in Ref_Statement_Database);
   -- Label_Entry

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Goto_Label
   --
   --  <Description>
   --
   --    Sets the goto_label field of a statement to Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Goto_Label (Node : in Ref_Statement_Database;
                             Item : in Ref_Statement_Database);
   -- Goto_Label

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Name
   --
   --  <Description>
   --
   --    Sets the names of exit loops, named loops, and pragmas to item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Name (Node : in Ref_Statement_Database;
                       Item : in Ref_Statement_Database);
   -- Exit_Loop_Name
   -- Loop_Name
   -- Pragma_Name

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Block_Declarations
   --
   --  <Description>
   --
   --    Sets the Block_Declarations field of the Node to Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Block_Declarations (Node : in Ref_Statement_Database;
                                     Item : in Ref_Statement_Database);
   -- Block_Declarations

   -- -------------------------------------------------------------------------
   --  <Subprogram Name>
   --
   --    Set_Exception_Handler
   --
   --  <Description>
   --
   --    Returns the Block Declarations for the current Node
   --
   --=======================================================================

   procedure Set_Exception_Handler (Node : in Ref_Statement_Database;
                                    Item : in Ref_Statement_Database);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Name_List
   --
   --  <Description>
   --
   --    Sets the Name_list field of an object or Unit to Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Name_List (Node : in Ref_Statement_Database;
                            Item : in Ref_Statement_Database);
   -- Unit_Name_List
   -- Name_List

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Parameter_List
   --
   --  <Description>
   --
   --    Sets the parameter list of a procedure or function to Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Parameter_List (Node : in Ref_Statement_Database;
                                 Item : in Ref_Statement_Database);
   -- Actual_Procedure_Parameters_List
   -- Lp_Items_List (for function calls)

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Argument_List
   --
   --  <Description>
   --
   --    Sets the Argument_list of a pragma statment to Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Argument_List (Node : in Ref_Statement_Database;
                                Item : in Ref_Statement_Database);
   -- Pragma_Arg

   -----

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Value
   --
   --  <Description>
   --
   --    Sets the value of an expression to Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Value (Node : in Ref_Statement_Database;
                        Item : in Ref_Data_Storage);
   -- Node_Value

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Set_Is_A_Default_Expression
   --
   --  <Description>
   --
   --    Marks the given NODE as being a default expression
   --
   --=======================================================================
   procedure Set_Is_A_Default_Expression (Node : in Ref_Statement_Database);
   -- Is_Default_Expression

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_As_Universal_Type
   --
   --  <Description>
   --
   --    Sets the Node as a universal type.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_As_Universal_Type (Node : in Ref_Statement_Database);
   -- Is_Node_Type_Universal

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Type
   --
   --  <Description>
   --
   --    Sets the statement database node to be of type Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Type (Node : in Ref_Statement_Database;
                       Item : in Ref_Symbol_Table_Entry);
   -- Node_Type

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Symbolic_Name
   --
   --  <Description>
   --
   --    Sets the symbolic name of the symbol to Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Symbolic_Name (Node : in Ref_Statement_Database;
                                Item : in Ref_String);
   -- Symbol_Name
   -- Character_Symbol
   -- String_Symbol
   -- Constant_Symbol

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Node_Kind
   --
   --  <Description>
   --
   --    Sets the statement database node to be of kind Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Node_Kind (Node : in Ref_Statement_Database;
                            Item : in Expression_Enumeration);
   -- Dot_Node_Kind
   -- Lp_Node_Kind

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Object
   --
   --  <Description>
   --
   --    Sets node to be Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Object (Node : in Ref_Statement_Database;
                         Item : in Ref_Statement_Database);
   -- Dot_Object
   -- Lp_Object

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Dot_Component
   --
   --  <Description>
   --
   --    Sets the dot_component of node to be Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Dot_Component (Node : in Ref_Statement_Database;
                                Item : in Ref_Statement_Database);
   -- Dot_Component

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_List_Of_Items
   --
   --  <Description>
   --
   --    Sets the list of items in a parenthesis to be item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_List_Of_Items (Node : in Ref_Statement_Database;
                                Item : in Ref_Statement_Database);
   -- Lp_Items_List

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Next_List_Item
   --
   --  <Description>
   --
   --    Sets the next item on the list to be Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Next_List_Item (Node : in Ref_Statement_Database;
                                 Item : in Ref_Statement_Database);
   -- Next_List_Item

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Aggregate_Choice
   --
   --  <Description>
   --
   --    Sets the aggregate choice of an expression to Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Aggregate_Choice (Node : in Ref_Statement_Database;
                                   Item : in Ref_Statement_Database);
   -- Aggregate_Choice

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Aggregate_Expression
   --
   --  <Description>
   --
   --    Sets the aggregate expression to Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Aggregate_Expression (Node : in Ref_Statement_Database;
                                       Item : in Ref_Statement_Database);
   -- Aggregate_Expression

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Lower_Bound
   --
   --  <Description>
   --
   --    Sets the lower bound of a range expression to Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Lower_Bound (Node : in Ref_Statement_Database;
                              Item : in Ref_Statement_Database);
   -- Lower_Bound

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Upper_Bound
   --
   --  <Description>
   --
   --    Sets the upper bound of a range expression to Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Upper_Bound (Node : in Ref_Statement_Database;
                              Item : in Ref_Statement_Database);
   -- Upper_Bound

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Attribute_Prefix
   --
   --  <Description>
   --
   --    Sete the prefix of an attribute_tick node to Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Attribute_Prefix (Node : in Ref_Statement_Database;
                                   Item : in Ref_Statement_Database);
   -- Attribute_Prefix

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Attribute
   --
   --  <Description>
   --
   --    Sets the attribute of an attribute_Tick node to Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Attribute (Node : in Ref_Statement_Database;
                            Item : in Ref_Statement_Database);
   -- Attribute

   -----

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Parent_Node
   --
   --  <Description>
   --
   --    Sets the parent node of a statement database record to Item.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    None.
   --
   --  <Exceptions Handled>
   --
   --    None.
   --==========================================================================
   procedure Set_Parent_Node (Node : in Ref_Statement_Database;
                              Item : in Ref_Statement_Database);
   -- Parent_Node

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Increment_Reference_Count
   --
   --  <Description>
   --
   --
   --=======================================================================
   procedure Increment_Reference_Count (Node : in Ref_Statement_Database);
   -- Reference_Count


   -- pragma inline(Set_Next_Statement);
   -- pragma inline(Set_Holder);
   -- pragma inline(Unset_Suspend_Evaluation_At_This_Statement);
   -- pragma inline(Set_Suspend_Evaluation_At_This_Statement);
   -- pragma inline(Set_Statement_Number);
   -- pragma inline(Set_Subprogram_Info);
   -- pragma inline(Set_Symbol_Table_Entry);
   -- pragma inline(Set_Lhs);
   -- pragma inline(Set_Rhs);
   -- pragma inline(Set_Condition);
   -- pragma inline(Set_Loop_Scheme);
   -- pragma inline(Set_For_Variable);
   -- pragma inline(Set_For_Range);
   -- pragma inline(Set_Name_Range);
   -- pragma inline(Set_Head_Of_Statements_List);
   -- pragma inline(Set_Head_Of_True_Task_Statements_List);
   -- pragma inline(Set_Head_Of_False_Task_Statements_List);
   -- pragma inline(Set_Expression);
   -- pragma inline(Set_Subprogram_Reference);
   -- pragma inline(Set_Alternatives_List);
   -- pragma inline(Set_Alternative_Choice);
   -- pragma inline(Set_Alternative_Statements);
   -- pragma inline(Set_Next_Alternative);
   -- pragma inline(Set_Label_Entry);
   -- pragma inline(Set_Goto_Label);
   -- pragma inline(Set_Name);
   -- pragma inline(Set_Block_Declarations);
   -- pragma inline(Set_Exception_Handler);
   -- pragma inline(Set_Name_List);
   -- pragma inline(Set_Parameter_List);
   -- pragma inline(Set_Argument_List);
   -- pragma inline(Set_Value);
   -- pragma inline(Set_Is_A_Default_Expression);
   -- pragma inline(Set_As_Universal_Type);
   -- pragma inline(Set_Type);
   -- pragma inline(Set_Symbolic_Name);
   -- pragma inline(Set_Node_Kind);
   -- pragma inline(Set_Object);
   -- pragma inline(Set_Dot_Component);
   -- pragma inline(Set_List_Of_Items);
   -- pragma inline(Set_Next_List_Item);
   -- pragma inline(Set_Aggregate_Choice);
   -- pragma inline(Set_Aggregate_Expression);
   -- pragma inline(Set_Lower_Bound);
   -- pragma inline(Set_Upper_Bound);
   -- pragma inline(Set_Attribute_Prefix);
   -- pragma inline(Set_Attribute);
   -- pragma inline(Set_Parent_Node);
   -- pragma inline(Increment_Reference_Count);

end Node_Set_Routines;
