--|$Log:	smgms.a,v $
--|Revision 8.0  91/01/17  10:41:26  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Storage_Management
--
--
-- Abstract
--
-- The package Storage_Management is responsible for the run-time storage
-- management of data objects defined within the user's Ada statements.
-- A set of routines are provided as the interface to the manipulation of
-- data objects.  These routines may be invoked by other modules of the
-- ACLI.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
with Ace_Universal_Types;
use Ace_Universal_Types;

with Literals;
use Literals;

with Btrees;

package Storage_Management is
   -- -----
   -- The basic data representation unit for objects.  Void is used for
   -- the record composite objects.
   -- -----
   type Data_Representation is
      (Integer_Data,
       Char_Data,
       String_Data,
       Float_Data,
       Array_Data,
       Void);

   -- incomplete type definition
   type Data_Storage (Data_Type : Data_Representation);

   -- Currently used to track down pointers that are freed twice.
   type Indirect_Data_Storage is access Data_Storage;

   type Ref_Data_Storage is access Indirect_Data_Storage;


   -- Array Storage Definitions.

   type Index is new Ace_Integer;

   type Array_Index is
      record
         Item : Ref_Data_Storage;
         Indx : Index;
   end record;
   
   -- `lessthan' is needed because Array_Index needs Ref_Data_Storage, so there
   -- is no other way to statisfy the Btree generic without having depended
   -- specs.
   function Lessthan (X, Y : Array_Index) return Boolean;

   package Array_Heap is new Btrees (Array_Index, Lessthan);

   type Array_Definition is record
     Upper_Bound : Index;
     Lower_Bound : Index;
     Tree : Array_Heap.Tree;
   end record;

   -- -----
   -- The Data_Storage node is the unit for allocation of memory for a data
   -- object.  For scalar objects, the appropriate type of memory
   -- (integer, character, or string [via access to allow allocation of a
   -- string of the exact length needed]) is selected.  Composite objects
   -- are composed of a series of scalar objects.  The Next_Item field
   -- is used to link together elements of an array.  The Next_Component
   -- field is used to link together components of a record.  The record
   -- object itself (with a Data_Type of Void) only contains a reference to
   -- the first component of the record.
   -- -----
   type Data_Storage (Data_Type : Data_Representation) is
      record

         Next_Item      : Ref_Data_Storage := null;
         Next_Component : Ref_Data_Storage := null;
         case Data_Type is
            when Integer_Data  =>
               Int_Item : Ace_Integer;
            when Char_Data     =>
               Char_Item : Character;
            when String_Data   =>
               String_Item : Ref_String;
            when Float_Data    =>
               Float_Item : Float;
            when Array_Data =>
               Array_Item : Array_Definition;
            when Void     =>
               null;
         end case;
      end record;

end Storage_Management;
