--|$Log:	smgmt.a,v $
--|Revision 8.0  91/01/17  10:41:26  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Storage_Management_Routines
--
---------------------------------------------------------------
--
--
-- Abstract
--
-- The Storage_Management_Routines package provides a set of routines
-- to the statement evaluator of the ACLI for storage management.
-- These routines handle the creation, deletion, and manipulation of
-- storage associated with data objects and the invocation of subprograms.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
with Storage_Management;
use Storage_Management;

with Symbol_Table_Structures;
use Symbol_Table_Structures;

with Ace_Universal_Types;
use Ace_Universal_Types;

with Literals;
use Literals;

with Context_Control;
with System;

package Storage_Management_Routines is

   -- --------------------------------------------------------------------------

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Allocate_Storage
   --
   --  <Description>
   --
   --    Allocate_Storage creates a data storage location based upon the type
   -- of symtbl entry item.
   --
   --==========================================================================
   procedure Allocate_Storage
       (Context      : in out Context_Control.Context_Definition;
        Symtbl_Entry : in Ref_Symbol_Table_Entry);


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Deallocate_Storage
   --
   --  <Description>
   --
   --  Deallocate_Storage frees the data storage location associated with
   -- the specified symtbl entry item.
   --==========================================================================
   procedure Deallocate_Storage
       (Context      : in out Context_Control.Context_Definition;
        Symtbl_Entry : in Ref_Symbol_Table_Entry);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Fetch_Value
   --
   --  <Description>
   --
   --   Fetch_Value returns the value from the storage stack given
   --   a symtbl entry.
   --
   --=========================================================================
   function Fetch_Value
       (Context      : in Context_Control.Context_Definition;
        Symtbl_Entry : in Ref_Symbol_Table_Entry) return Ref_Data_Storage;


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Store_Value
   --
   --  <Description>
   --
   --  Store_Value places the value from the data storage structure into the
   --  item.
   --
   --=========================================================================
   procedure Store_Value (Context : in out Context_Control.Context_Definition;
                          Symtbl_Entry : in Ref_Symbol_Table_Entry;
                          Value        : in Ref_Data_Storage);

   -- --------------------------------------------------------------------------
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Copy_Value
   --
   --  <Description>
   --
   --   copies current data storage node (succeeding nodes (next_item
   --   and next_component nodes) are not copied).
   --
   --=======================================================================
   function Copy_Value (Value_Node : Ref_Data_Storage) return Ref_Data_Storage;

   -- --------------------------------------------------------------------------
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Get_Current_Level, Incr_Level, Decr_Level,
   --    Reset_LevelOffset_To_Top_Level,
   --    Reset_LevelOffset_To_Previous_DR_Level,
   --    Get_Next_Offset_Value, Init_Offset_Value.
   --
   --  <Description>
   --
   --    Level and Offset manipulation and access routines
   --
   --==========================================================================

   function Get_Current_Level
       (Context : in Context_Control.Context_Definition) return Ace_Max_Levels
       renames Context_Control.Get_Current_Level;

   procedure Incr_Level (Context : in out Context_Control.Context_Definition);

   procedure Decr_Level (Context : in out Context_Control.Context_Definition);

   -- reset level/offset to top level (level zero)
   procedure Reset_Leveloffset_To_Top_Level
       (Context : in out Context_Control.Context_Definition);

   -- reset level/offset to previous declarative region level
   procedure Reset_Leveloffset_To_Previous_Dr_Level
       (Context : in out Context_Control.Context_Definition);

   procedure Next_Offset_Value
       (Context : in out Context_Control.Context_Definition;
        Value   : out Ace_Natural);

   procedure Init_Offset_Value
       (Context : in out Context_Control.Context_Definition);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Get_Last_Storage_Location_Pointer, Set_Last_Storage_Location_Pointer
   --    Get_Display_For_Level, Set_Display_For_Level
   --
   --  <Description>
   --
   --    Dss access & manipulation routines (for subprogram calls)
   --    Display access & manipulation routines (for subprogram calls)
   --
   --==========================================================================

   function Get_Last_Storage_Location_Pointer
       (Context : in Context_Control.Context_Definition) return Ace_Natural
       renames Context_Control.Get_Last_Storage_Location_Pointer;

   procedure Set_Last_Storage_Location_Pointer
       (Context : in out Context_Control.Context_Definition;
        Item    : in Ace_Natural) renames Context_Control.
         Set_Last_Storage_Location_Pointer;

   function Get_Display_For_Level
       (Context : in Context_Control.Context_Definition;
        Level   : in Ace_Natural) return Ace_Natural renames Context_Control.
         Get_Display_For_Level;

   procedure Set_Display_For_Level
       (Context     : in out Context_Control.Context_Definition;
        Level, Item : in Ace_Natural) renames Context_Control.
         Set_Display_For_Level;

   function Get_Data_Type (Item : Ref_Data_Storage) return Data_Representation;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Dump_Dss, Dump_Display
   --
   --  <Description>
   --
   --    Output routines for Display and Dss (used in debuggin).
   --
   --==========================================================================
   procedure Dump_Dss (Context : in Context_Control.Context_Definition);
   procedure Dump_Display (Context : in Context_Control.Context_Definition);

   -- --------------------------------------------------------------------------

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Storage_For_Integer, Create_Storage_For_Character,
   --    Create_Storage_For_String, Create_Storage_For_Array.
   --
   --  <Description>
   --
   --    Routines to create storage for the three basic types in ACE.
   --
   --==========================================================================
   function Create_Storage_For_Integer
       (Value : in Ace_Integer := 0) return Ref_Data_Storage;

   function Create_Storage_For_Character
       (Value : in Character := ' ') return Ref_Data_Storage;

   function Create_Storage_For_String
       (Value : in Ref_String := Create) return Ref_Data_Storage;

   function Create_Storage_For_Array
       (Lower, Upper : in Index) return Ref_Data_Storage;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Get_Integer_Value, Get_Character_Value,
   --    Get_String_Value.
   --
   --  <Description>
   --
   --    Routines to get the object stored for the three basic types in ACE.
   --
   --==========================================================================

   function Get_Integer_Value (Node : in Ref_Data_Storage) return Ace_Integer;

   function Get_Character_Value (Node : in Ref_Data_Storage) return Character;

   function Get_String_Value (Node : in Ref_Data_Storage) return Ref_String;

   function Get_Array_Value
       (Node : in Ref_Data_Storage) return Array_Definition;

   -----
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Integer_Value, Set_Character_Value,
   --    Set_String_Value.
   --
   --  <Description>
   --
   --    Routines to set the object stored for the three basic types in ACE.
   --
   --=========================================================================
   procedure Set_Integer_Value (Node : in Ref_Data_Storage;
                                Item : in Ace_Integer);

   procedure Set_Character_Value (Node : in Ref_Data_Storage;
                                  Item : in Character);

   procedure Set_String_Value (Node : in Ref_Data_Storage;
                               Item : in Ref_String);

   procedure Set_Array_Value (Node : in Ref_Data_Storage;
                              Item : in Array_Definition);

   -----
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Get_Next_Item, Get_Next_Component.
   --
   --  <Description>
   --
   --    Get routines for sequentially traversing the storage structure.
   --
   --==========================================================================
   function Get_Next_Item (Node : in Ref_Data_Storage) return Ref_Data_Storage;

   function Get_Next_Component
       (Node : in Ref_Data_Storage) return Ref_Data_Storage;

   -----
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Next_Item, Set_Next_Component.
   --
   --  <Description>
   --
   --    Set routines for sequentially traversing and setting
   --    the storage structure.
   --
   --==========================================================================

   procedure Set_Next_Item (Node : in Ref_Data_Storage;
                            Item : in Ref_Data_Storage);

   procedure Set_Next_Component (Node : in Ref_Data_Storage;
                                 Item : in Ref_Data_Storage);

   -- --------------------------------------------------------------------------
   -- Arrays


   -- Creation

   procedure Create (Arry                     : in out Array_Definition;
                     Lower_Bound, Upper_Bound : in Index);

   -- Add and Find Array Values

   procedure Store_Value (Arry     : in out Array_Definition;
                          Position : in Index;
                          Data     : in Ref_Data_Storage);

   function Fetch_Value (Arry     : in Array_Definition;
                         Position : in Index) return Ref_Data_Storage;

   -- Slices and Copies

   procedure Slice (From        : in Array_Definition;
                    To          : in out Array_Definition;
                    Start, Stop : in Index);

   procedure Copy (From : in Array_Definition;
                   To   : in out Array_Definition);

   -- Attribute functions

   function Attr_Address (Arry : in Array_Definition) return System.Address;
   function Attr_Constrained (Arry : in Array_Definition) return Boolean;
   function Attr_First (Arry : in Array_Definition) return Index;
   function Attr_Last (Arry : in Array_Definition) return Index;
   function Attr_Length (Arry : in Array_Definition) return Ace_Integer;
   -- function Attr_Range (Arry : in Array_Definition) return <range>;

   -- Pragma Procedures

   procedure Pack (Arry : in out Array_Definition);

   -- Garbage Collection
   procedure Free (Arry : in out Array_Definition);

end Storage_Management_Routines;
