--|$Log:	x.a,v $
--|Revision 8.0  91/01/17  10:43:27  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - X_Window_System
--
---------------------------------------------------------------
--
--
-- Abstract
--
--   This package contains a small set of X window tool kit
--   routines. (Ada bindings).
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------

package X_Window_System is

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Observe_Subprograms_Window
   --
   --  <Description>
   --
   --    This procedure creates an observation window for subprograms
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    [none]
   --
   --  <Exceptions Handled>
   --
   --    [none]
   --==========================================================================
   procedure Create_Observe_Subprograms_Window;
   pragma INTERFACE (C, Create_Observe_Subprograms_Window);
   pragma LINKNAME (Create_Observe_Subprograms_Window,
                          "x_create_obs_subpgms_win");
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Observe_Tasks_Window
   --
   --  <Description>
   --
   --    This procedure creates an observation window for tasks
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    [none]
   --
   --  <Exceptions Handled>
   --
   --    [none]
   --==========================================================================
   procedure Create_Observe_Tasks_Window;
   pragma INTERFACE (C, Create_Observe_Tasks_Window);
   pragma LINKNAME (Create_Observe_Tasks_Window,
                          "x_create_obs_tasks_win");

   --==========================================================================
   --  <Subprogram Name>
   --
   --    End_Of_Displaying_Lines
   --
   --  <Description>
   --
   --    This procedure handles the end of displaying lines into the
   --    subprogram observation window.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    [none]
   --
   --  <Exceptions Handled>
   --
   --    [none]
   --==========================================================================
   procedure End_Of_Displaying_Lines;
   pragma INTERFACE (C, End_Of_Displaying_Lines);
   pragma LINKNAME (End_Of_Displaying_Lines, "x_end_dsp_lines");

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Clear_Textwindow
   --
   --  <Description>
   --
   --    This procedure clears a text window within the SunView windowing
   --    environment.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    [none]
   --
   --  <Exceptions Handled>
   --
   --    [none]
   --==========================================================================
   procedure Clear_Textwindow;
   pragma INTERFACE (C, Clear_Textwindow);
   pragma LINKNAME (Clear_Textwindow, "x_clear_text_win");

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Synchronize
   --
   --  <Description>
   --
   --    This routine synchronize the lines being displayed on a window
   --    with the interpreter. (i.e., it pauses the interpreter until
   --    the window is synchronize).
   --
   --==========================================================================
   procedure Synchronize;
   pragma INTERFACE (C, Synchronize);
   pragma LINKNAME (Synchronize, "x_synchronize");

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Find_String
   --
   --  <Description>
   --
   --    This routine will find a string in an X window.
   --
   --==========================================================================
   procedure Find_String (Str : in String);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Display_Line
   --
   --  <Description>
   --
   --    This routine will display a line in an X window.
   --
   --==========================================================================
   procedure Display_Line (Str : in String);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Kill_Window
   --
   --  <Description>
   --
   --    This routine will kill/destroy the X window.
   --
   --==========================================================================
   procedure Kill_Window;
   pragma INTERFACE (C, Kill_Window);
   pragma LINKNAME (Kill_Window, "x_kill_window");

   -- -----------------------------------------------------------------------

end X_Window_System;
