--|$Log:	treeb.a,v $
--|Revision 8.0  91/01/17  10:41:36  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Tree_Builder
--
---------------------------------------------------------------
--
--
-- Abstract
--
--   This package contains the routines for building specific statement
--   database nodes.  The functions correspond with Ada constructs.
--   These routines are called by the parser to construct entire
--   abstract syntax trees.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
with Statement_Database;
use Statement_Database;
package Tree_Builder is


   --==========================================================================
   --  <Subprogram Name>
   --
   --     Create_Expression_Node
   --
   --  <Description>
   --
   --   The expression routines are used to build up expressions (the need
   --   for many expression routines is due to the complexity of
   --   Ada expressions)
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None.
   --
   --==========================================================================
   function Create_Expression_Node
       (Kind  : Expression_Enumeration;
        Item1 : Ref_Statement_Database) return Ref_Statement_Database;

   function Create_Expression_Node
       (Kind         : Expression_Enumeration;
        Item1, Item2 : Ref_Statement_Database) return Ref_Statement_Database;

   function Create_Expression_Node
       (Kind                : Expression_Enumeration;
        Item1, Item2, Item3 : Ref_Statement_Database) return
         Ref_Statement_Database;

   function Create_Expression_Node
       (Kind : Expression_Enumeration;
        Item1, Item2, Item3, Item4 : Ref_Statement_Database) return
         Ref_Statement_Database;

   function Create_Expression_Node
       (Kind : Expression_Enumeration;
        Item1, Item2, Item3, Item4, Item5 : Ref_Statement_Database) return
         Ref_Statement_Database;


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_If_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for if statements.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_If_Node
       (Condition, True_Part, False_Part : Ref_Statement_Database) return
         Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Loop_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for loop statements.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_Loop_Node
       (Condition, Body_Part : Ref_Statement_Database) return
         Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Assignment_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for assignment statements.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_Assignment_Node
       (Lhs, Rhs : Ref_Statement_Database) return Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Case_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for case statements.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_Case_Node (Expression, Alternatives : Ref_Statement_Database
                              ) return Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Labeled_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for labels.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_Labeled_Node
       (Label : Ref_Statement_Database) return Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Goto_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for goto statements.  Currently not supported by all of ACE.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_Goto_Node
       (Label : Ref_Statement_Database) return Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Exit_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for exit statements.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_Exit_Node
       (Condition, Label : Ref_Statement_Database) return
         Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Return_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for return statements.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_Return_Node
       (Expression : Ref_Statement_Database) return Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Block_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for blocks.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_Block_Node
       (Decls, Stmts : Ref_Statement_Database) return Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_With_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for with clauses.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_With_Node
       (Unit_Name : Ref_Statement_Database) return Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Use_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for use clauses.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_Use_Node
       (Unit_Name : Ref_Statement_Database) return Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Procedure_Call_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for procedure calls.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_Procedure_Call_Node
       (Name, Actuals : Ref_Statement_Database) return Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Procedure_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for procedures.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_Procedure_Node
       (Name, Formals, Stmt_List : Ref_Statement_Database) return
         Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Function_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for functions.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_Function_Node
       (Name, Formals, Return_Type, Stmt_List : Ref_Statement_Database) return
         Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Package_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for packages.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_Package_Node
       (Name, Stmt_List : Ref_Statement_Database) return Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Package_Body_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for package bodies.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_Package_Body_Node
       (Name, Stmt_List : Ref_Statement_Database) return Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Array_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for array declarations.  Currently not supported by all of ACE.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_Array_Node
       (Index, Component : Ref_Statement_Database) return
         Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Record_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for record types.  Currently not supported by all of ACE.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_Record_Node
       (Components : Ref_Statement_Database) return Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Constraint_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for constraints.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_Constraint_Node
       (Type_Mark, Constraint : Ref_Statement_Database) return
         Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Type_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for a type declaration.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_Type_Node
       (Lhs, Rhs : Ref_Statement_Database) return Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Object_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for object declarations.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_Object_Node
       (Lhs, Rhs : Ref_Statement_Database) return Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Subtype_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for subtypes.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_Subtype_Node
       (Lhs, Rhs : Ref_Statement_Database) return Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Derived_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for derived types.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_Derived_Node
       (Lhs, Rhs : Ref_Statement_Database) return Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Pragma_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for pragmas.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_Pragma_Node
       (Lhs, Rhs : Ref_Statement_Database) return Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Private_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for private.  Currently not supported by other parts of
   --   ACE.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_Private_Node
       (Lhs, Rhs : Ref_Statement_Database) return Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Limited_Private_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for limited privates.  Currently not supported by other parts of
   --   ACE.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_Limited_Private_Node
       (Lhs, Rhs : Ref_Statement_Database) return Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_In_Parameter_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for in parameters.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_In_Parameter_Node
       (Lhs, Rhs : Ref_Statement_Database) return Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Out_Parameter_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for out parameters.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_Out_Parameter_Node
       (Lhs, Rhs : Ref_Statement_Database) return Ref_Statement_Database;
   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_In_Out_Parameter_Node
   --
   --  <Description>
   --
   --   These routines create specific parts of the abstract syntax tree
   --   for in out parameters.
   --
   --  <Called By>
   --
   --   External
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    None
   --
   --==========================================================================
   function Create_In_Out_Parameter_Node
       (Lhs, Rhs : Ref_Statement_Database) return Ref_Statement_Database;

end Tree_Builder;
