--|$Log:	user.a,v $
--|Revision 8.0  91/01/17  10:41:38  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - User_Pkg
--
---------------------------------------------------------------
--
--
-- Abstract
--
--
--   This package constains the input routines for ACE.
--
--   The input for ACE is complicated.  The following is a description of
--   what goes on:
--
--   Places input can come from:
--
--     1) Standard_Input  (The users terminal)
--     2) A named file    (A file named by the user that is loaded into ACE)
--     3) "Unput" buffer  (The buffer used to scan input ahead of its time)
--
--   If there is input in the Unput buffer (object Buf) then we take input
--   from there.
--   If there is input in the Standard_Input buffer (object Abuf) then
--   we get another input line from "Ace_Current_Input" which is sometimes
--   the Standard_Input of the users terminal and sometimes a named file.
--
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
with Text_Io;
use Text_Io;
package User_Pkg is

   -- Overview:
   --   This package is support routines for the scanner.
   --

   Nul     : constant Character := Ascii.Nul;
   Newline : constant Character := Ascii.Lf;

   Lineno  : Natural            := 0;


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Echo
   --
   --  <Description>
   --
   --    Starts the echoing of all input.  This is useful for the
   --    echoing of files when they are read into ACE.
   --
   --==========================================================================
   procedure Echo (Bool : Boolean);
   pragma INLINE (Echo);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Output
   --
   --  <Description>
   --
   --    This routine is used by the scanner to output a message
   --
   --==========================================================================
   procedure Output (C : in Character);
   pragma INLINE (Output);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Unput
   --
   --  <Description>
   --
   --    This routine is used by the pre_parser to put back the tokens that
   --    where read to determine what parser should be executed.
   --
   --==========================================================================
   procedure Unput (C : in Character);
   pragma INLINE (Unput);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Initbuf
   --
   --  <Description>
   --
   --    This routine is called to initialize all the buffers used
   --    in the input routines.
   --
   --==========================================================================
   procedure Init_Buf;
   pragma INLINE (Init_Buf);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Reset_Buffer
   --
   --  <Description>
   --
   --    This routine is called after an error has occurred and cleanup is
   --    needed.
   --
   --==========================================================================
   procedure Reset_Buffer;
   pragma INLINE (Reset_Buffer);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Yywrap
   --
   --  <Description>
   --
   --    This routine is called after all the parsing is finished and will
   --    perform any "wrap-up" that is needed (currently is does nothing).
   --
   --==========================================================================
   function Yywrap return Boolean;
   pragma INLINE (Yywrap);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Input
   --
   --  <Description>
   --
   --    This routine gets the next input character from where it should
   --    come from. That can mean standard_input, a file, or the unput buffer.
   --
   --==========================================================================
   function Input return Character;
   pragma INLINE (Input);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Allow_EOF and Disallow_EOF
   --
   --  <Description>
   --
   -- When we are reading from a file sometimes it is OK to get an
   -- end_of_file, such as when a person uses the environment
   -- command "get_file"
   -- However, when we are using the Interpret command the end_of_file
   -- will be significant. These routines control whether or not
   -- the end_of_file will signal an error, or the continuation of
   -- input from Standard_Ace_Input.
   --
   --==========================================================================
   procedure Allow_Eof;
   pragma INLINE (Allow_Eof);
   procedure Disallow_Eof;
   pragma INLINE (Disallow_Eof);

end User_Pkg;
