--|$Log:	system.a,v $
--|Revision 8.0  91/01/17  10:42:15  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - System_Dependent_Routines
--
---------------------------------------------------------------
--
--
-- Abstract
--
--  This package contains all the code that needs to be ported and
--  the system dependent routines.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
with System;

with Literals;
use Literals;

with Ace_File_System;
use Ace_File_System;

with Directory_Objects;
use Directory_Objects;

with Ace_Universal_Types;
use Ace_Universal_Types;

with Unix_Types;
use Unix_Types;

package System_Dependent_Routines is

   -- pass on some routines declared in Unix_Types that are used for conversion
   -- from Ada types to C types and back.

   subtype Thirty_Two_Int is Unix_Types.Thirty_Two_Int;

   subtype Cstring is Unix_Types.Cstring;

   function Convert_To_Cstring (Str : String) return Cstring renames Unix_Types
      .Convert_To_Cstring;

   function Convert_To_Astring (Str : Cstring) return String renames Unix_Types
      .Convert_To_Astring;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Flush
   --
   --  <Description>
   --
   --    This routine will "flush" all current input from the input
   --    queue.  This is a system dependent function, that needs to
   --    interface with the host O/S.
   --
   --  <Called By>
   --
   --    Error recovery routines
   --
   --  <Exceptions Raised>
   --
   --    Host O/S depended..
   --
   --  <Exceptions Handled>
   --
   --    Host O/S depended..
   --==========================================================================
   procedure Flush;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    System
   --
   --  <Description>
   --
   --    Spawns a process that will execute the contents of the string
   --    Command.
   --
   --  <Called By>
   --
   --    Various
   --
   --  <Exceptions Raised>
   --
   --    Host O/S depended.
   --
   --  <Exceptions Handled>
   --
   --    Host O/S depended.
   --==========================================================================
   procedure System (Command : String;
                     Args    : String         := "";
                     How     : Execution_Mode := Foreground);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Rename
   --
   --  <Description>
   --
   --    Renames a file from Obj1 to Obj2
   --
   --  <Called By>
   --
   --    Ace_File_System
   --
   --  <Exceptions Raised>
   --
   --    Host O/S depended.
   --
   --  <Exceptions Handled>
   --
   --    Host O/S depended.
   --==========================================================================
   procedure Rename (Obj1, Obj2 : in out Ref_String);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Copy
   --
   --  <Description>
   --
   --    Copies Obj1 to Obj2
   --
   --  <Called By>
   --
   --    Ace_File_System
   --
   --  <Exceptions Raised>
   --
   --    Host O/S depended.
   --
   --  <Exceptions Handled>
   --
   --    Host O/S depended.
   --==========================================================================
   procedure Copy (Obj1, Obj2 : in out Ref_String);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Mkdir
   --
   --  <Description>
   --
   --    Creates the directory specified
   --
   --  <Called By>
   --
   --    Ace_File_System
   --
   --  <Exceptions Raised>
   --
   --    Host O/S depended.
   --
   --  <Exceptions Handled>
   --
   --    Host O/S depended.
   --==========================================================================
   procedure Mkdir (Obj1 : in Ref_String);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Chdir
   --
   --  <Description>
   --
   --    Changes the current working directory
   --
   --  <Called By>
   --
   --    Ace_File_System
   --
   --  <Exceptions Raised>
   --
   --    Host O/S depended.
   --
   --  <Exceptions Handled>
   --
   --    Host O/S depended.
   --==========================================================================
   procedure Chdir (Obj1 : in Ref_String);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Get_Directory_Name
   --
   --  <Description>
   --
   --    This routine takes as a parameter a relative directory name, and
   --    returns the full directory name. For example, it will change "." to
   --    "/mn1/wpl".
   --
   --  <Called By>
   --
   --    Ace_File_System
   --
   --  <Exceptions Raised>
   --
   --    Host O/S depended.
   --
   --  <Exceptions Handled>
   --
   --    Host O/S depended.
   --==========================================================================
   function Get_Directory_Name (Str : String) return String;


   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Opendir
   --
   --    Last Mod:    Thu Nov 30 16:33:39 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   function Opendir (Filename : String) return Ace_Integer;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Readdir
   --
   --    Last Mod:    Thu Nov 30 16:33:43 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   function Readdir (Dirfd : Ace_Integer) return String;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Closedir
   --
   --    Last Mod:    Thu Nov 30 16:33:46 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   function Closedir (Dirfd : Ace_Integer) return Ace_Integer;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Directory_Lister
   --
   --  <Description>
   --
   --    This procedure will list all the files contained in the directory Obj.
   --
   --  <Called By>
   --
   --    Ace_File_System
   --
   --  <Exceptions Raised>
   --
   --    Host O/S depended.
   --
   --  <Exceptions Handled>
   --
   --    Host O/S depended.
   --==========================================================================
   procedure Directory_Lister (Obj : Ref_String);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Home_Directory
   --
   --  <Description>
   --
   --    Returns the File_Object associated with the users home directory
   --    (Startup directory).
   --
   --  <Called By>
   --
   --    ACE_File_System
   --
   --  <Exceptions Raised>
   --
   --    Host O/S depended.
   --
   --  <Exceptions Handled>
   --
   --    Host O/S depended.
   --==========================================================================
   function Home_Directory return Directory_Object;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Current_Directory
   --
   --  <Description>
   --
   --    Returns the File_OBject associated with the user current directory.
   --
   --  <Called By>
   --
   --    Ace_File_System
   --
   --  <Exceptions Raised>
   --
   --    Host O/S depended.
   --
   --  <Exceptions Handled>
   --
   --    Host O/S depended.
   --==========================================================================
   function Current_Directory return Directory_Object;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Edit
   --
   --  <Description>
   --
   --    This routine calls a host operating system editor.
   --
   --  <Called By>
   --
   --    ACE_File_System
   --
   --  <Exceptions Raised>
   --
   --    Host O/S depended.
   --
   --  <Exceptions Handled>
   --
   --    Host O/S depended.
   --==========================================================================
   procedure Edit (Obj : in out File_Object;
                   How : in Execution_Mode := Foreground);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Compile
   --
   --  <Description>
   --
   --    This routine compiles an Ada program in the host operating system.
   --
   --  <Called By>
   --
   --    Ace_File_System
   --
   --  <Exceptions Raised>
   --
   --    Host O/S depended.
   --
   --  <Exceptions Handled>
   --
   --    Host O/S depended.
   --==========================================================================
   procedure Compile (Obj  : in File_Object;
                      Form : in String);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Command_Line_Argument
   --
   --  <Description>
   --
   --    This is an Ada compiler dependent subprogram, that will return
   --    a string that contains the first command line argument.  This
   --    routine will have to re-written for each Ada compilation system.
   --
   --  <Called By>
   --
   --    Pre_Parser
   --
   --==========================================================================
   function Command_Line_Argument return String;


   --==========================================================================
   --+--  <Subprogram Name>
   --
   --    Abort_Check
   --
   --  <Description>
   --
   --    This routine will check if the user has typed an abort (i.e., ^C).
   --    If so, then setup for another abort, and raise interrupt_error.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    Interrupt_Error
   --
   --==========================================================================

   function Abort_Check return Boolean;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    System_Exit
   --
   --  <Description>
   --
   --    Ada interface to a system-dependent that terminates the
   --    calling process, regardless of its state.
   --
   --  <Called By>
   --
   --   Interpreted Xt callback procedures
   --
   --==========================================================================

   procedure System_Exit (Exit_Code : in Integer);
   pragma INTERFACE (C, System_Exit);
   pragma INTERFACE_NAME (System_Exit, "_exit");

   ----

   package Cpu is

      type Time is new Ace_Integer;

      function Clock return Time;

      function "-" (Stop_Time, Start_Time : Time) return Time;

      procedure Put_Time (A_Time : in Time);

   end Cpu;

end System_Dependent_Routines;
