--|$Log:	xadt.a,v $
--|Revision 8.0  91/01/17  10:41:40  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - X_Window_Adt [Spec]
--
---------------------------------------------------------------
--
--
-- Abstract
--
--    This package provides the ACE Abstract Data Type implementation for
--    the X Window System.  The routines within this package interface
--    with the X-Ada bindings to implement a selected portion of window
--    creation, deletion, and text/drawing manipulation operations.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------

with Window_Objects;
use Window_Objects;

package X_Window_Adt is

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Window
   --
   --  <Description>
   --
   --    This procedure creates a new window object under the X Window System.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    [none]
   --
   --  <Exceptions Handled>
   --
   --    [none]
   --==========================================================================

   procedure Create_Window (Window              : in out Ace_Window_Type;
                            Horizontal_Position : in Ace_Coordinate := 0;
                            Vertical_Position   : in Ace_Coordinate := 0;
                            Horizontal_Size     : in Ace_Pixels     := 100;
                            Vertical_Size       : in Ace_Pixels     := 100);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Destroy_Window
   --
   --  <Description>
   --
   --    This procedure destroys an X Window that was previously created.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    [none]
   --
   --  <Exceptions Handled>
   --
   --    [none]
   --==========================================================================

   procedure Destroy_Window (Window : in out Ace_Window_Type);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Clear_Window
   --
   --  <Description>
   --
   --    This procedure clears the all the elements from within the window.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    [none]
   --
   --  <Exceptions Handled>
   --
   --    [none]
   --==========================================================================

   procedure Clear_Window (Window : in Ace_Window_Type);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Draw_Line
   --
   --  <Description>
   --
   --    This procedure draws a line within an X window as specified by the
   --    X and Y coordinate values.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    [none]
   --
   --  <Exceptions Handled>
   --
   --    [none]
   --==========================================================================

   procedure Draw_Line (Window         : in Ace_Window_Type;
                        X1, Y1, X2, Y2 : in Ace_Coordinate);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Draw_Dashed_Line
   --
   --  <Description>
   --
   --    This procedure draws a dashed line within an X window as specified
   --    by the X and Y coordinate values.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    [none]
   --
   --  <Exceptions Handled>
   --
   --    [none]
   --==========================================================================

   procedure Draw_Dashed_Line (Window         : in Ace_Window_Type;
                               X1, Y1, X2, Y2 : in Ace_Coordinate);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Draw_Text
   --
   --  <Description>
   --
   --    This procedure draws text within an X window at the given coordinates.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    [none]
   --
   --  <Exceptions Handled>
   --
   --    [none]
   --==========================================================================

   procedure Draw_Text (Window : in Ace_Window_Type;
                        X, Y   : in Ace_Coordinate;
                        Text   : in String);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Draw_Rectangle_Builtin
   --
   --  <Description>
   --
   --    This procedure draws a rectangle within an X window by making use of
   --    the draw rectangle function provided by X.  Rectangle is drawn using
   --    the X, Y coordinates and the width and height of the rectangle in
   --    pixels.
   --
   --  <Called By>
   --
   --    External
   --
   --  <Exceptions Raised>
   --
   --    [none]
   --
   --  <Exceptions Handled>
   --
   --    [none]
   --==========================================================================

   procedure Draw_Rectangle_Builtin (Window        : in Ace_Window_Type;
                                     X, Y          : in Ace_Coordinate;
                                     Width, Height : in Ace_Pixels);

   -- -------------------------------------------------------------------------

end X_Window_Adt;
