--|$Log:	alloc.a,v $
--|Revision 8.0  91/01/17  10:43:33  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Allocate_And_Init_Storage_For_Package_Decls
--
---------------------------------------------------------------
--
--
-- Abstract
--
--   This is a "separate" compilation unit (procedure) that will allocate
--   all the space on the heap for package declarations.  In addition,
--   when space is allocated it is initialized to the proper values.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------

separate (Statement_Evaluator.Evaluate_Statement.Evaluate_Individual_Statement)

-- -----------------------------------------------------------------------
--
procedure Allocate_And_Init_Storage_For_Package_Decls
    (Context : in out Context_Control.Context_Definition;
     Node    : Ref_Statement_Database) is

   Current : Ref_Statement_Database := Node;

begin

   while (not (Is_Empty (Current))) loop
      case Check_Stmt_Node (Current) is

         -- Allocate the space for an object declaration
         -- Objects are special, since they have a value associated with
         -- each declaration.

         when Object_Declaration    =>
            declare
               List_Node : Ref_Statement_Database := Get_Name_List (Current);
               Is_An_Init_Value_To_Assign : Boolean := False;
               Init_Expr_Result           : Ref_Statement_Database;
            begin
               if (not Is_Empty (Get_Expression (Current))) then
                  Is_An_Init_Value_To_Assign := True;
                  Init_Expr_Result :=
                     Evaluate_Expression (Get_Expression (Current));
               end if;
               while (not Is_Empty (List_Node)) loop
                  declare
                     Object_Symtbl_Entry : Ref_Symbol_Table_Entry :=
                        Get_Symbol_Table_Entry (Get_Expression (List_Node));
                  begin

                     -- allocation of the object declaration and
                     -- the space for the value of the object.

                     Allocate_Storage (Context, Object_Symtbl_Entry);
                     if (Is_An_Init_Value_To_Assign) then
                        Store_Value (Context, Object_Symtbl_Entry,
                                     Copy_Value (Get_Value (Init_Expr_Result)));
                     end if;
                     List_Node := Get_Next_List_Item (List_Node);

                  end;
               end loop;
            end;

         when Type_Declaration | Subtype_Declaration    =>

            -- allocate space for the declaration (no values are associated
            -- with types).

            Allocate_Storage (Context, Get_Symbol_Table_Entry (Current));

         when Package_Declaration   =>

            -- recurse on this routine for packages within packages.

            Allocate_And_Init_Storage_For_Package_Decls
                (Context,
                 Get_Package_Spec_Decls (Get_Symbol_Table_Entry (Current)));

         when others =>
            null; -- ignore
      end case;

      Current := Get_Next_Statement (Current);
   end loop;

end Allocate_And_Init_Storage_For_Package_Decls;
