--|$Log:	atextio.a,v $
--|Revision 8.0  91/01/17  10:43:37  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Ace_Text_Io
--
---------------------------------------------------------------
--
--
-- Abstract
--
--  Ace_Text_Io is an interface from ACE to Ada's Text_Io.
--  Text_Io uses a limted private type for the data types.
--  A limited private type can not be transfered from the interpreted
--  ACE to the compiled ACE. Therefore, we represent File_Type as
--  an integer and turn the integer into a Text_Io.File_Type internally.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
with Misc;

with Ooe;
use Ooe;

with User_Pkg;

with Text_Io;

with Error_Handler;

package body Ace_Text_Io is

   -- Objects to hold predefined values of Ace_Text_Io.
   -- These predefined values control the input features
   -- of ACE (input and output of executing programs,
   -- and input and output of ACE itself).

   Active_Input      : File_Type := File_Type'LAST;
   Active_Ace_Input  : File_Type := File_Type'LAST;

   Active_Output     : File_Type := File_Type'LAST - 1;
   Active_Ace_Output : File_Type := File_Type'LAST - 1;

   -- Procedures that mimic Text_Io for ACE.

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create
   --
   --  <Description>
   --
   --    Same as Text_Io.Create
   --
   --==========================================================================
   procedure Create (File : in out File_Type;
                     Mode : in File_Mode := Out_File;
                     Name : in String    := "";
                     Form : in String    := "") is

   begin

      for I in File_Type'FIRST + 1 .. File_Type'LAST - 2 loop

         -- find an unused file entry

         if not Text_Io.Is_Open (File_Entries (I)) then
            Create (Files (I), Mode, Name, Form);
            File := I;
            return ;
         end if;

      end loop;

      -- reached the limit on number of files

      raise Use_Error;

   end Create;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Open
   --
   --  <Description>
   --
   --    Same as Text_Io.Open
   --
   --==========================================================================
   procedure Open (File : in out File_Type;
                   Mode : in File_Mode;
                   Name : in String;
                   Form : in String := "") is

      Index : File_Type;

   begin

      for I in File_Type'FIRST + 1 .. File_Type'LAST - 2 loop

         -- find an unused file entry

         if not Text_Io.Is_Open (File_Entries (I)) then
            Open (Files (I), Mode, Name, Form);
            File := I;
            return ;
         end if;

      end loop;

      -- reached the limit on number of files

      raise Use_Error;

   exception

       when Name_Error =>

          Error_Handler.Show_Error (3027, Error_Handler.Warning, Name);
          raise;

   end Open;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Is_Open
   --
   --  <Description>
   --
   --    Same as Text_Io.Is_Open
   --
   --==========================================================================
   function Is_Open (File : in File_Type) return Boolean is
      Return_Value : Boolean;
   begin
      return Text_Io.Is_Open (File_Entries (File));
   end Is_Open;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Mode
   --
   --  <Description>
   --
   --    Same as Text_Io.Mode
   --
   --==========================================================================
   function Mode (File : in File_Type) return File_Mode is
   begin
      return Mode (File_Entries (File));
   end Mode;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Form
   --
   --  <Description>
   --
   --    Same as Text_Io.Form
   --
   --==========================================================================
   function Form (File : in File_Type) return String is
   begin
      return Text_Io.Form (File_Entries (File));
   end Form;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Close
   --
   --  <Description>
   --
   --    Same as Text_Io.Close.
   --
   --==========================================================================
   procedure Close (File : in out File_Type) is
   begin
      Text_Io.Close (Files (File));
      File := File_Type'FIRST;
   end Close;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Delete
   --
   --  <Description>
   --
   --    Same as Text_Io.Delete.
   --
   --==========================================================================
   procedure Delete (File : in out File_Type) is
   begin
      Text_Io.Delete (Files (File));
   end Delete;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Reset
   --
   --  <Description>
   --
   --    Same as Text_Io.Reset
   --
   --==========================================================================
   procedure Reset (File : in File_Type;
                    Mode : in File_Mode) is
   begin
      Text_Io.Reset (Files (File),
                     Text_Io.File_Mode'VAL (Ace_Text_Io.File_Mode'POS (Mode)));
   end Reset;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Reset
   --
   --  <Description>
   --
   --    Same as Text_Io.Reset
   --
   --==========================================================================
   procedure Reset (File : in File_Type) is
   begin
      Text_Io.Reset (Files (File));
   end Reset;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Get
   --
   --  <Description>
   --
   --    Same as Text_Io.Get
   --
   --==========================================================================
   procedure Get (File : in File_Type;
                  Item : out Character) is
   begin
      Text_Io.Get (File_Entries (File), Item);
   end Get;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Get
   --
   --  <Description>
   --
   --    Same as Text_Io.Get
   --
   --==========================================================================
   procedure Get (Item : out Character) is
   begin
      Text_Io.Get (Item);
   end Get;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Get
   --
   --  <Description>
   --
   --    Same as Text_Io.Get
   --
   --==========================================================================
   procedure Get (File : in File_Type;
                  Item : out String) is
   begin
      Text_Io.Get (File_Entries (File), Item);
   end Get;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Get
   --
   --  <Description>
   --
   --    Same as Text_Io.Get
   --
   --==========================================================================
   procedure Get (Item : out String) is
   begin
      Text_Io.Get (Item);
   end Get;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Get_Line
   --
   --  <Description>
   --
   --    Same as Text_Io.Get_Line
   --
   --==========================================================================
   procedure Get_Line (File : in File_Type;
                       Item : out String;
                       Last : out Natural) is
   begin
      Text_Io.Get_Line (File_Entries (File), Item, Last);
   end Get_Line;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Get_Line
   --
   --  <Description>
   --
   --    Same as Text_Io.Get_Line
   --
   --==========================================================================
   procedure Get_Line (Item : out String;
                       Last : out Natural) is
   begin
      Text_Io.Get_Line (Item, Last);
   end Get_Line;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Skip_Line
   --
   --  <Description>
   --
   --    Same as Text_Io.Skip_Line
   --
   --==========================================================================
   procedure Skip_Line (File    : in File_Type;
                        Spacing : in Positive_Count := 1) is
   begin
      Text_Io.Skip_Line (File_Entries (File), Text_Io.Positive_Count (Spacing));
   end Skip_Line;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Skip_Line
   --
   --  <Description>
   --
   --    Same as Text_Io.Skip_Line
   --
   --==========================================================================
   procedure Skip_Line (Spacing : in Positive_Count := 1) is
   begin
      Text_Io.Skip_Line (Text_Io.Positive_Count (Spacing));
   end Skip_Line;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Put
   --
   --  <Description>
   --
   --    Same as Text_Io.Put
   --
   --==========================================================================
   procedure Put (File : in File_Type;
                  Item : in Character) is
   begin
      Text_Io.Put (File_Entries (File), Item);
   end Put;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Put
   --
   --  <Description>
   --
   --    Same as Text_Io.Put
   --
   --==========================================================================
   procedure Put (Item : in Character) is
   begin
      Text_Io.Put (Item);
   end Put;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Put
   --
   --  <Description>
   --
   --    Same as Text_Io.Put
   --
   --==========================================================================
   procedure Put (File : in File_Type;
                  Item : in String) is
   begin
      Text_Io.Put (File_Entries (File), Item);
   end Put;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Put
   --
   --  <Description>
   --
   --    Same as Text_Io.Put
   --
   --==========================================================================
   procedure Put (Item : in String) is
   begin
      Text_Io.Put (Item);
   end Put;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Put_Line
   --
   --  <Description>
   --
   --    Same as Text_Io.Put_Line
   --
   --==========================================================================
   procedure Put_Line (File : in File_Type;
                       Item : in String) is
   begin
      Text_Io.Put_Line (File_Entries (File), Item);
   end Put_Line;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Put_Line
   --
   --  <Description>
   --
   --    Same as Text_Io.Put_Line
   --
   --==========================================================================
   procedure Put_Line (Item : in String) is
   begin
      Text_Io.Put_Line (Item);
   end Put_Line;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    New_Line
   --
   --  <Description>
   --
   --    Same as Text_Io.New_Line
   --
   --==========================================================================
   procedure New_Line (File    : in File_Type;
                       Spacing : in Positive_Count := 1) is
   begin
      Text_Io.New_Line (File_Entries (File), Text_Io.Positive_Count (Spacing));
   end New_Line;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    New_Line
   --
   --  <Description>
   --
   --    Same as Text_Io.New_Line
   --
   --==========================================================================
   procedure New_Line (Spacing : in Positive_Count := 1) is
   begin
      Text_Io.New_Line (Text_Io.Positive_Count (Spacing));
   end New_Line;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    End_Of_Line
   --
   --  <Description>
   --
   --    Same as Text_Io.End_Of_Line
   --
   --==========================================================================
   function End_Of_Line (File : in File_Type) return Boolean is
   begin
      return Text_Io.End_Of_Line (File_Entries (File));
   end End_Of_Line;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    End_Of_Line
   --
   --  <Description>
   --
   --    Same as Text_Io.End_Of_Line
   --
   --==========================================================================
   function End_Of_Line return Boolean is
   begin
      return Text_Io.End_Of_Line;
   end End_Of_Line;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    End_Of_File
   --
   --  <Description>
   --
   --    Same as Text_Io.End_Of_File
   --
   --==========================================================================
   function End_Of_File (File : in File_Type) return Boolean is
   begin
      return Text_Io.End_Of_File (File_Entries (File));
   end End_Of_File;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    End_Of_File
   --
   --  <Description>
   --
   --    Same as Text_Io.End_Of_File
   --
   --==========================================================================
   function End_Of_File return Boolean is
   begin
      return Text_Io.End_Of_File;
   end End_Of_File;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Input
   --
   --  <Description>
   --
   --    Same as Text_Io.Set_Input
   --
   --==========================================================================
   procedure Set_Input (File : in File_Type) is
   begin
      Text_Io.Set_Input (File_Entries (File));
      Active_Input := File;
   end Set_Input;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Output
   --
   --  <Description>
   --
   --    Same as Text_Io.Set_Output
   --
   --==========================================================================
   procedure Set_Output (File : in File_Type) is
   begin
      Text_Io.Set_Output (File_Entries (File));
      Active_Output := File;
   end Set_Output;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Standard_Input
   --
   --  <Description>
   --
   --    Same as Text_Io.Standard_Input
   --
   --==========================================================================
   function Standard_Input return File_Type is
   begin
      return File_Type'LAST;
   end Standard_Input;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Standard_Output
   --
   --  <Description>
   --
   --    Same as Text_Io.Standard_Output
   --
   --==========================================================================
   function Standard_Output return File_Type is
   begin
      return File_Type'LAST - 1;
   end Standard_Output;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Current_Input
   --
   --  <Description>
   --
   --    Same as Text_Io.Current_Input
   --
   --==========================================================================
   function Current_Input return File_Type is
   begin
      return Active_Input;
   end Current_Input;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Current_Output
   --
   --  <Description>
   --
   --    Same as Text_Io.Current_Output
   --
   --==========================================================================
   function Current_Output return File_Type is
   begin
      return Active_Output;
   end Current_Output;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Ace_Input
   --
   --  <Description>
   --
   --    Same as Text_Io.Set_Input, except that this routine controls the
   --    input for the ACE interpreter.
   --
   --==========================================================================
   procedure Set_Ace_Input (File : in File_Type) is
   begin
      if Is_Open (File) then
         Active_Ace_Input := File;
         if File /= Standard_Ace_Input then
            Misc.Set_Prompt_Off;
            User_Pkg.Allow_Eof;
         else
            Misc.Set_Prompt_On;
         end if;
      else
         raise Status_Error;
      end if;
   end Set_Ace_Input;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Ace_Output
   --
   --  <Description>
   --
   --    Same as Text_Io.Set_Output, except that this routine controls the
   --    output for the ACE interpreter.
   --
   --==========================================================================
   procedure Set_Ace_Output (File : in File_Type) is
   begin
      Active_Ace_Output := File;
   end Set_Ace_Output;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Standard_Ace_Input
   --
   --  <Description>
   --
   --    Same as Text_Io.Standard_Input, except that this routine returns the
   --    Standard_Input for the ACE interpreter.
   --
   --==========================================================================
   function Standard_Ace_Input return File_Type is
   begin
      return File_Type'LAST;
   end Standard_Ace_Input;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Standard_Ace_Output
   --
   --  <Description>
   --
   --    Same as Text_Io.Standard_Output, except that this routine returns the
   --    Standard_Output for the ACE interpreter.
   --
   --==========================================================================
   function Standard_Ace_Output return File_Type is
   begin
      return File_Type'LAST - 1;
   end Standard_Ace_Output;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Current_Ace_Input
   --
   --  <Description>
   --
   --    Same as Text_Io.Current_Input, except that this routine returns the
   --    Current_Input for the ACE interpreter.
   --
   --==========================================================================
   function Current_Ace_Input return File_Type is
   begin
      return Active_Ace_Input;
   end Current_Ace_Input;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Current_Ace_Output
   --
   --  <Description>
   --
   --    Same as Text_Io.Current_Output, except that this routine returns the
   --    Current_Output for the ACE interpreter.
   --
   --==========================================================================
   function Current_Ace_Output return File_Type is
   begin
      return Active_Ace_Output;
   end Current_Ace_Output;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Name
   --
   --  <Description>
   --
   --    Same as Text_Io.Name
   --
   --==========================================================================
   function Name (File : in File_Type) return String is
   begin
      return Text_Io.Name (File_Entries (File));
   end Name;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Line_Length
   --
   --  <Description>
   --
   --    Same as Text_Io
   --
   --==========================================================================
   procedure Set_Line_Length (File : in File_Type;
                              To   : in Count) is
   begin
      Text_Io.Set_Line_Length (File_Entries (File), Text_Io.Count (To));
   end Set_Line_Length;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Line_Length
   --
   --  <Description>
   --
   --    Same as Text_Io.Set_Line_Length
   --
   --==========================================================================
   procedure Set_Line_Length (To : in Count) is
   begin
      Text_Io.Set_Line_Length (Text_Io.Count (To));
   end Set_Line_Length;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Page_Length
   --
   --  <Description>
   --
   --    Same as Text_Io.Set_Page_Length
   --
   --==========================================================================
   procedure Set_Page_Length (File : in File_Type;
                              To   : in Count) is
   begin
      Text_Io.Set_Page_Length (File_Entries (File), Text_Io.Count (To));
   end Set_Page_Length;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Page_Length
   --
   --  <Description>
   --
   --    Same as Text_Io.Set_Page_Length
   --
   --==========================================================================
   procedure Set_Page_Length (To : in Count) is
   begin
      Text_Io.Set_Line_Length (Text_Io.Count (To));
   end Set_Page_Length;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Line_Length
   --
   --  <Description>
   --
   --    Same as Text_Io.Line_Length
   --
   --==========================================================================
   function Line_Length (File : in File_Type) return Count is
   begin
      return Count (Text_Io.Line_Length (File_Entries (File)));
   end Line_Length;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Line_Length
   --
   --  <Description>
   --
   --    Same as Text_Io.Line_Length
   --
   --==========================================================================
   function Line_Length return Count is
   begin
      return Count (Text_Io.Line_Length);
   end Line_Length;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Page_Length
   --
   --  <Description>
   --
   --    Same as Text_Io.Page_Length
   --
   --==========================================================================
   function Page_Length (File : in File_Type) return Count is
   begin
      return Count (Text_Io.Page_Length (File_Entries (File)));
   end Page_Length;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Page_Length
   --
   --  <Description>
   --
   --    Same as Text_Io.Page_Length
   --
   --==========================================================================
   function Page_Length return Count is
   begin
      return Count (Text_Io.Page_Length);
   end Page_Length;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    New_Page
   --
   --  <Description>
   --
   --    Same as Text_Io.New_Page
   --
   --==========================================================================
   procedure New_Page (File : in File_Type) is
   begin
      Text_Io.New_Page (File_Entries (File));
   end New_Page;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    New_Page
   --
   --  <Description>
   --
   --    Same as Text_Io.New_Page
   --
   --==========================================================================
   procedure New_Page is
   begin
      Text_Io.New_Page;
   end New_Page;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Skip_Page
   --
   --  <Description>
   --
   --    Same as Text_Io.Skip_Page
   --
   --==========================================================================
   procedure Skip_Page (File : in File_Type) is
   begin
      Text_Io.New_Page (File_Entries (File));
   end Skip_Page;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Skip_Page
   --
   --  <Description>
   --
   --    Same as Text_Io.Skip_Page
   --
   --==========================================================================
   procedure Skip_Page is
   begin
      Text_Io.Skip_Page;
   end Skip_Page;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    End_Of_Page
   --
   --  <Description>
   --
   --    Same as Text_Io.End_Of_Page
   --
   --==========================================================================
   function End_Of_Page (File : in File_Type) return Boolean is
   begin
      return Text_Io.End_Of_Page (File_Entries (File));
   end End_Of_Page;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    End_Of_Page
   --
   --  <Description>
   --
   --    Same as Text_Io.End_Of_Page
   --
   --==========================================================================
   function End_Of_Page return Boolean is
   begin
      return Text_Io.End_Of_Page;
   end End_Of_Page;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Col
   --
   --  <Description>
   --
   --    Same as Text_Io.Set_Col
   --
   --==========================================================================
   procedure Set_Col (File : in File_Type;
                      To   : in Positive_Count) is
   begin
      Text_Io.Set_Col (File_Entries (File), Text_Io.Positive_Count (To));
   end Set_Col;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Col
   --
   --  <Description>
   --
   --    Same as Text_Io.Set_Col
   --
   --==========================================================================
   procedure Set_Col (To : in Positive_Count) is
   begin
      Text_Io.Set_Col (Text_Io.Positive_Count (To));
   end Set_Col;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Line
   --
   --  <Description>
   --
   --    Same as Text_Io.Set_Line
   --
   --==========================================================================
   procedure Set_Line (File : in File_Type;
                       To   : in Positive_Count) is
   begin
      Text_Io.Set_Line (File_Entries (File), Text_Io.Positive_Count (To));
   end Set_Line;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Line
   --
   --  <Description>
   --
   --    Same as Text_Io.Set_Line
   --
   --==========================================================================
   procedure Set_Line (To : in Positive_Count) is
   begin
      Text_Io.Set_Line (Text_Io.Positive_Count (To));
   end Set_Line;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Col
   --
   --  <Description>
   --
   --    Same as Text_Io.Col
   --
   --==========================================================================
   function Col (File : in File_Type) return Positive_Count is
   begin
      return Positive_Count (Text_Io.Col (File_Entries (File)));
   end Col;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Col
   --
   --  <Description>
   --
   --    Same as Text_Io.Col
   --
   --==========================================================================
   function Col return Positive_Count is
   begin
      return Positive_Count (Text_Io.Col);
   end Col;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Line
   --
   --  <Description>
   --
   --    Same as Text_Io.Line
   --
   --==========================================================================
   function Line (File : in File_Type) return Positive_Count is
   begin
      return Positive_Count (Text_Io.Line (File_Entries (File)));
   end Line;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Line
   --
   --  <Description>
   --
   --    Same as Text_Io.Line
   --
   --==========================================================================
   function Line return Positive_Count is
   begin
      return Positive_Count (Text_Io.Line);
   end Line;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Page
   --
   --  <Description>
   --
   --    Same as Text_Io.Page
   --
   --==========================================================================
   function Page (File : in File_Type) return Positive_Count is
   begin
      return Positive_Count (Text_Io.Page (File_Entries (File)));
   end Page;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Page
   --
   --  <Description>
   --
   --    Same as Text_Io.Page
   --
   --==========================================================================
   function Page return Positive_Count is
   begin
      return Positive_Count (Text_Io.Page);
   end Page;

   --==========================================================================
   --  <Package Name>
   --
   --    Integer_Io
   --
   --  <Description>
   --
   --    Integer_Io is an instantiation of Text_Io.Integer_Io.  ACE
   --    doesn't support generics.
   --
   --==========================================================================
   package body Integer_Io is

      package Text_Io_Integer_Io is new Text_Io.Integer_Io
          (Integer);

      --========================================================================
      --  <Subprogram Name>
      --
      --    Get
      --
      --  <Description>
      --
      --    Same as Text_Io.Get
      --
      --========================================================================
      procedure Get (File  : in File_Type;
                     Item  : out Integer;
                     Width : in Field := 0) is
      begin
         Text_Io_Integer_Io.Get (File_Entries (File), Item,
                                 Text_Io.Field (Width));
      end Get;

      --=======================================================================
      --  <Subprogram Name>
      --
      --    Get
      --
      --  <Description>
      --
      --    Same as Text_Io.Get
      --
      --========================================================================
      procedure Get (Item  : out Integer;
                     Width : in Field := 0) is
      begin
         Text_Io_Integer_Io.Get (Item, Text_Io.Field (Width));
      end Get;

      --=======================================================================
      --  <Subprogram Name>
      --
      --    Get
      --
      --  <Description>
      --
      --    Same as Text_Io.Get
      --
      --=======================================================================
      procedure Get (From : in String;
                     Item : out Integer;
                     Last : out Positive) is
      begin
         Text_Io_Integer_Io.Get (From, Item, Last);
      end Get;

      --=======================================================================
      --  <Subprogram Name>
      --
      --    Put
      --
      --  <Description>
      --
      --    Same as Text_Io.Put
      --
      --=======================================================================
      procedure Put (File  : in File_Type;
                     Item  : in Integer;
                     Width : in Field       := Default_Width;
                     Base  : in Number_Base := Default_Base) is
      begin
         Text_Io_Integer_Io.Put (File_Entries (File), Item,
                                 Text_Io.Field (Width),
                                 Text_Io.Number_Base (Base));
      end Put;

      --=======================================================================
      --  <Subprogram Name>
      --
      --    Put
      --
      --  <Description>
      --
      --    Same as Text_Io.Put
      --
      --=======================================================================
      procedure Put (Item  : in Integer;
                     Width : in Field       := Default_Width;
                     Base  : in Number_Base := Default_Base) is
      begin
         Text_Io_Integer_Io.Put (Item, Text_Io.Field (Width),
                                 Text_Io.Number_Base (Base));
      end Put;

      --=======================================================================
      --  <Subprogram Name>
      --
      --    Put
      --
      --  <Description>
      --    Same as Text_Io.Put
      --
      --=======================================================================
      procedure Put (To   : out String;
                     Item : in Integer;
                     Base : in Number_Base := Default_Base) is
      begin
         Text_Io_Integer_Io.Put (To, Item, Text_Io.Number_Base (Base));
      end Put;

   end Integer_Io;

end Ace_Text_Io;
