/*
--|$Log:	sunws.c,v $
--|Revision 8.0  91/01/17  10:47:45  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
*/
/*****************************************************************
--
-- Company Name - : Unisys Corporation
--
-- Address -    : Paoli Research Center
--              : P.O. Box 517
--              : Paoli, PA 19301-0517
--
------------------ Disclaimer ---------------------------------
--
-- This software and its documentation are provided "AS IS" and
-- without any expressed or implied warranties whatsoever.
-- No warranties as to performance, merchantability, or fitness
-- for a particular purpose exist.
--
-- Because of the diversity of conditions and hardware under
-- which this software may be used, no warranty of fitness for
-- a particular purpose is offered.  The user is advised to
-- test the software thoroughly before relying on it.  The user
-- must assume the entire risk and liability of using this
-- software.
--
-- In no event shall any person or organization of people be
-- held responsible for any direct, indirect, consequential
-- or inconsequential damages or lost profits.
---------------------------------------------------------------
--
*****************************************************************/

#include <stdio.h>
#include <suntool/sunview.h>
#include <suntool/textsw.h>

#define BUFFER_SIZE 128

void sun_create_obs_subpgms_win ();
static void show_new_file ();
static Notify_value my_pipe_reader();
static void quit_proc ();

static Frame baseframe;
static Textsw observe_stmts_textsw;
static Textsw_index tindex, lindex;
static FILE *fp_tochild, *fp_fromchild;
static int childpid;
static int fildes[2];

static short toolicon[258] = {
#include "observe_window.icn"
};
mpr_static (toolicon_pixrect, 64, 64, 1, toolicon);

int
is_under_Sun ()
{
    if ((char *) getenv("WINDOW_PARENT"))
	return TRUE;
    else
	return FALSE;
}

void
sun_find_string (str)
    char *str;
{
    putc ('/', fp_tochild);
    fputs (str, fp_tochild);
    putc ('\n', fp_tochild);
}

void
sun_display_line (str)
    char *str;
{
    putc ('+', fp_tochild);
    fputs (str, fp_tochild);
    putc ('\n', fp_tochild);
}

void
sun_end_dsp_lines ()
{
    fputs (".\n", fp_tochild);
}

void
sun_clear_text_win ()
{
    fputs ("-\n", fp_tochild);
}

void
sun_create_obs_subpgms_win ()
{
    if (pipe(fildes) == -1)
    {
	perror ("pipe");
	exit (1);
    }

    fp_tochild = fdopen (fildes[1], "w");
    fp_fromchild = fdopen (fildes[0], "r");
    setbuf(fp_tochild,NULL);

    switch (childpid = fork())
    {
    case -1:
	perror ("ACE_sunview_observe_window");
	exit(1);
    
    case 0: /* child */

	putenv ("DEFAULT_FONT=/usr/lib/fonts/fixedwidthfonts/cour.r.10");
	baseframe =
	    window_create (NULL, FRAME,
		FRAME_LABEL,
		    "ACE Debugger: Subprograms Observation",
		FRAME_SHOW_LABEL,
		    TRUE,
		WIN_X,
		    0,
		WIN_Y,
		    0,
		WIN_COLUMNS,
		    80,
		FRAME_ICON,
		    icon_create (
			ICON_IMAGE, &toolicon_pixrect,
			ICON_LABEL, "Stmts",
			0),
		0);

	observe_stmts_textsw =
	    window_create (baseframe,
		TEXTSW, WIN_ROWS, 24,
		TEXTSW_READ_ONLY, TRUE,
		TEXTSW_UPDATE_SCROLLBAR,
		0);

	notify_set_input_func (baseframe, my_pipe_reader, fildes[0]);

	window_fit_height (baseframe);
	window_main_loop (baseframe);

	exit (0);

    default: /* parent */
	return;
    }
}

static void
quit_proc ()
{
    textsw_reset (observe_stmts_textsw, 400, 500);
    window_destroy (baseframe);
}

static Notify_value
my_pipe_reader (the_frame,fd)
    Frame the_frame;
    int fd;
{
    char buf[ BUFFER_SIZE ], *b;

    fgets (buf, BUFFER_SIZE, fp_fromchild);

    b = buf;

    switch (*b)
    {
    case '/': /* find string */
	if (textsw_find_bytes (
		observe_stmts_textsw,
		&tindex,
		&lindex,
		++b, strlen(b)-2, 0) >= 0)
	{
	    textsw_set_selection (observe_stmts_textsw, tindex, lindex, 1);
	    textsw_possibly_normalize (observe_stmts_textsw, tindex);
	    tindex = lindex;
	}
	/* else item not found -- can't happen in ACE observe stmt window! */
	break;

    case '+': /* insert one line */
	textsw_insert (observe_stmts_textsw, ++b, strlen(b)-1);
	break;

    case '.': /* end of inserting lines */
	tindex = textsw_index_for_file_line (observe_stmts_textsw, 0);
	textsw_normalize_view (observe_stmts_textsw, tindex);
	break;
	
    case '-': /* reset textsw */
	textsw_reset (observe_stmts_textsw, 400, 500);
	break;

    default:
	break; /* ignore */
    }

    return (NOTIFY_DONE);
}
