--|$Log:	calendar.a,v $
--|Revision 8.0  91/01/17  10:43:39  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Ace_Calendar
--
---------------------------------------------------------------
--
--
-- Abstract
--
--  This package provides an interface to the Standard.Calender.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
with Calendar;
with Unchecked_Conversion;
package body Ace_Calendar is

   type Ref_Time is access Calendar.Time;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Convert_To_Time, Convert_To_Ref_Time, Convert
   --
   --  <Description>
   --
   --    These are various conversion routines (there is no semantics,
   --    except for the type conversion).
   --
   --==========================================================================
   function Convert_To_Time is new Unchecked_Conversion
       (Ref_Time, Time);
   function Convert_To_Ref_Time is new Unchecked_Conversion
       (Time, Ref_Time);
   function Convert (T : Time) return Calendar.Time is
   begin
      return Convert_To_Ref_Time (T).all;
   end Convert;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Clock
   --
   --  <Description>
   --
   --    Same as the Standard.Calendar.Clock
   --
   --==========================================================================
   function Clock return Time is
   begin
      return Convert_To_Time (new Calendar.Time'(Calendar.Clock));
   end Clock;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Year
   --
   --  <Description>
   --
   --    Same as the Standard.Calendar.Year
   --
   --==========================================================================
   function Year (Date : Time) return Year_Number is
   begin
      return Ace_Integer (Calendar.Year (Convert (Date)));
   end Year;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Month
   --
   --  <Description>
   --
   --    Same as the Standard.Calendar.Month
   --
   --==========================================================================
   function Month (Date : Time) return Month_Number is
   begin
      return Ace_Integer (Calendar.Month (Convert (Date)));
   end Month;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Day
   --
   --  <Description>
   --
   --    Same as the Standard.Calendar.Day
   --
   --==========================================================================
   function Day (Date : Time) return Day_Number is
   begin
      return Ace_Integer (Calendar.Day (Convert (Date)));
   end Day;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Seconds
   --
   --  <Description>
   --
   --    Same as the Standard.Calendar.Seconds
   --
   --==========================================================================
   function Seconds (Date : Time) return Day_Duration is
   begin
      return Ace_Duration (Calendar.Seconds (Convert (Date)));
   end Seconds;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Split
   --
   --  <Description>
   --
   --    Same as the Standard.Calendar.Split
   --
   --==========================================================================
   procedure Split (Date    : in Time;
                    Year    : out Year_Number;
                    Month   : out Month_Number;
                    Day     : out Day_Number;
                    Seconds : out Day_Duration) is
      Tyear    : Calendar.Year_Number;
      Tmonth   : Calendar.Month_Number;
      Tday     : Calendar.Day_Number;
      Tseconds : Calendar.Day_Duration;
   begin
      Calendar.Split (Convert (Date), Tyear, Tmonth, Tday, Tseconds);
      Year := Year_Number (Tyear);
      Month := Month_Number (Tmonth);
      Day := Day_Number (Tday);
      Seconds := Day_Duration (Tseconds);
   end Split;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Time_Of
   --
   --  <Description>
   --
   --    Same as the Standard.Calendar.Time_Of
   --
   --==========================================================================
   function Time_Of (Year    : Year_Number;
                     Month   : Month_Number;
                     Day     : Day_Number;
                     Seconds : Day_Duration := 0) return Time is
   begin
      return
         Convert_To_Time
             (new Calendar.Time'(
                 Calendar.Time_Of (Calendar.Year_Number (Year),
                                   Calendar.Month_Number (Month),
                                   Calendar.Day_Number (Day),
                                   Duration (Seconds))));
   end Time_Of;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    "+"
   --
   --  <Description>
   --
   --    Same as the Standard.Calendar."+"
   --
   --==========================================================================
   function "+" (Left  : Time;
                 Right : Ace_Duration) return Time is
   begin
      return
         Convert_To_Time (new Calendar.Time'(Calendar."+" (Convert (Left),
                                                           Duration (Right))));
   end "+";

   --==========================================================================
   --  <Subprogram Name>
   --
   --    "+"
   --
   --  <Description>
   --
   --    Same as the Standard.Calendar."+"
   --
   --==========================================================================
   function "+" (Left  : Ace_Duration;
                 Right : Time) return Time is
   begin
      return
         Convert_To_Time (new Calendar.Time'(Calendar."+" (Duration (Left),
                                                           Convert (Right))));
   end "+";

   --==========================================================================
   --  <Subprogram Name>
   --
   --    "-"
   --
   --  <Description>
   --
   --    Same as the Standard.Calendar."-"
   --
   --==========================================================================
   function "-" (Left  : Time;
                 Right : Ace_Duration) return Time is
   begin
      return
         Convert_To_Time (new Calendar.Time'(Calendar."-" (Convert (Left),
                                                           Duration (Right))));
   end "-";

   --==========================================================================
   --  <Subprogram Name>
   --
   --    "-"
   --
   --  <Description>
   --
   --    Same as the Standard.Calendar."-"
   --
   --==========================================================================
   function "-" (Left  : Time;
                 Right : Time) return Ace_Duration is
   begin
      return Ace_Duration (Calendar."-" (Convert (Left), Convert (Right)));
   end "-";

   --==========================================================================
   --  <Subprogram Name>
   --
   --    "<"
   --
   --  <Description>
   --
   --    Same as the Standard.Calendar."<"
   --
   --==========================================================================
   function "<" (Left, Right : Time) return Boolean is
   begin
      return (Calendar."<" (Convert (Left), Convert (Right)));
   end "<";

   --==========================================================================
   --  <Subprogram Name>
   --
   --    "<="
   --
   --  <Description>
   --
   --    Same as the Standard.Calendar."<="
   --
   --==========================================================================
   function "<=" (Left, Right : Time) return Boolean is
   begin
      return (Calendar."<=" (Convert (Left), Convert (Right)));
   end "<=";

   --==========================================================================
   --  <Subprogram Name>
   --
   --    ">"
   --
   --  <Description>
   --
   --    Same as the Standard.Calendar.">"
   --
   --==========================================================================
   function ">" (Left, Right : Time) return Boolean is
   begin
      return (Calendar.">" (Convert (Left), Convert (Right)));
   end ">";

   --==========================================================================
   --  <Subprogram Name>
   --
   --    ">="
   --
   --  <Description>
   --
   --    Same as the Standard.Calendar.">="
   --
   --==========================================================================
   function ">=" (Left, Right : Time) return Boolean is
   begin
      return (Calendar.">=" (Convert (Left), Convert (Right)));
   end ">=";

end Ace_Calendar;
