--|$Log:	carray.a,v $
--|Revision 8.0  91/01/17  10:43:40  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Subunit for Package Name -  Check_Array_Reference.Check_Expression
--
---------------------------------------------------------------
--
--
-- Abstract
--
--   This routine handle semantic checks for Array references, e.g., A(2).
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------

--==========================================================================
--  <Subprogram Name>
--
--    Check_Array_Reference
--
--  <Description>
--
--    This routine handles expanded names, etc. (e.g., record notation,
--    access notation).
--
--==========================================================================
with Text_Io;
use Text_Io;

separate (Semantic_Analyzer.Check_Expression)

procedure Check_Array_Reference (Node : in Ref_Statement_Database) is

begin
   if (Is_Empty (Get_Symbol_Table_Entry (Node))) then
      -- symbol table entry was _not_ found
      -- symbol is undeclared
      Show_Error (1100, Fatal, Convert_To_String (Get_Symbolic_Name (Node)));
   else
      if Is_Empty (Get_List_Of_Items (Node)) then
         -- no index
         Set_Type (Node, Get_Type (Get_Symbol_Table_Entry (Node)));
      else
         case Get_Expression_Kind (Get_Expression (Get_List_Of_Items (Node)))
             is
            when Range_Expression   =>
               -- index is range
               Set_Type (Node, Get_Type (Get_Symbol_Table_Entry (Node)));
            when Terminal_Node | Constant_Node     =>
               -- should be single a constant_node, or terminal_node.
               -- index is single descrete value
               -- therefore, we make array reference the same as an array
               -- element.
               Set_Type
                   (Node,
                    Get_Symbol_Table_Entry
                        (Get_Element_Type (Get_Type (Get_Object (Node)))));
            when others   =>
               Put_Line
                   ("Hmm.... must be more than a Terminal or Constant_Node");
               Put_Line
                   ("It's...." &
                    Expression_Enumeration'IMAGE (
                       Get_Expression_Kind
                           (Get_Expression (Get_List_Of_Items (Node)))));
         end case;
         --      else
         --        Show_Error -- Illegal index
      end if;
   end if;
end Check_Array_Reference;
