--|$Log:	cattr.a,v $
--|Revision 8.0  91/01/17  10:43:41  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Subunits for Package - Semantic_Analyzer
--
---------------------------------------------------------------
--
--
-- Abstract
--
--  The following routines are separate routines for
--  Semantic_Analyzer.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------

with Text_Io;
with Statement_Evaluator;

with Symbol_Table_Routines;
use Symbol_Table_Routines;

with Symtbl_Entry_Get_Routines;
use Symtbl_Entry_Get_Routines;

with Storage_Management_Routines;

with String_Constants;
use String_Constants;

with Misc_Support_Routines;
use Misc_Support_Routines;

separate (Semantic_Analyzer.Check_Expression)

--==========================================================================
--  <Subprogram Name>
--
--    Check_attribute
--
--  <Description>
--
--    This routine will check semantics of attributes.
--
--==========================================================================
procedure Check_Attribute (Node : in Ref_Statement_Database) is

   Attr_Name         : Ref_String;
   Attr_Node         : Ref_Statement_Database := null;
   Attr_Param_Node   : Ref_Statement_Database := null;
   Prefix_Entry_Node : Ref_Symbol_Table_Entry;

   -- just a small function to check and see if the given structure is
   -- a type mark.  It will be compiled INLINE

   function Is_Type_Mark (Ref : in Ref_Symbol_Table_Entry) return Boolean is
      Prefix_Entry_Kind : constant Type_Entry_Enumeration :=
         Get_Entry_Kind (Ref);
   begin
      return (Prefix_Entry_Kind = Type_Entry or else
             Prefix_Entry_Kind = Subtype_Entry or else
             Prefix_Entry_Kind = Derived_Type_Entry);
   end Is_Type_Mark;
   pragma INLINE (Is_Type_Mark);

   -- This Function returns TRUE if the given attribute name should
   -- have a parameter.  It will be compiled INLINE
   function Attribute_Should_Have_Parameter (Name : Ref_String) return Boolean
                                            is
   begin
      return Are_Symbolic_Names_Equal (Name, Ref_Image_String) or else
             Are_Symbolic_Names_Equal (Name, Ref_Pos_String) or else
             Are_Symbolic_Names_Equal (Name, Ref_Pred_String) or else
             Are_Symbolic_Names_Equal (Name, Ref_Succ_String) or else
             Are_Symbolic_Names_Equal (Name, Ref_Val_String) or else
             Are_Symbolic_Names_Equal (Name, Ref_Range_String);
   end Attribute_Should_Have_Parameter;
   pragma INLINE (Attribute_Should_Have_Parameter);

   -- This function returns TRUE if the given Attribute node has
   -- only one parameter attatched to it.
   -- NOTE: THIS DOES NOT CONFORM TO THE LRM. It facilitates the currently
   -- implemented ACE attributes and must be ammeded/enhanced to support
   --others,
   function Has_Only_One_Parameter
       (Attr_Node : Ref_Statement_Database) return Boolean is
   begin
      return Is_Empty (Get_Next_List_Item (Get_List_Of_Items (Attr_Node)));

   end Has_Only_One_Parameter;

   -- This function will return TRUE if the given node is a discrete type.
   -- It will be compiled INLINE
   function Is_A_Discrete_Type
       (Type_Node : Ref_Symbol_Table_Entry) return Boolean is
      Root_Type : Ref_Symbol_Table_Entry := Get_Root_Type (Type_Node);
   begin
      return Is_Type_An_Integer_Type (Root_Type) or
             Is_Type_An_Enumeration_Type (Root_Type);
   end Is_A_Discrete_Type;
   pragma INLINE (Is_A_Discrete_Type);

   -- This function will return TRUE if the given attribute is not currently
   -- supported by ACE. NOTE: In some case, a form of the attribute might not
   -- supported, while another is (e.g. "first").  Such attributes are
   -- nontheless considered supported, in either form (though another
   -- error will probably be reported.
   -- This subroutine will be compiled INLINE
   function Non_Supported_Attribute (Attr_Name : Ref_String) return Boolean is
   begin
      return
         Are_Symbolic_Names_Equal (Attr_Name, Ref_Address_String) or else
         Are_Symbolic_Names_Equal (Attr_Name, Ref_Aft_String) or else
         Are_Symbolic_Names_Equal (Attr_Name, Ref_Base_String) or else
         Are_Symbolic_Names_Equal (Attr_Name, Ref_Callable_String) or else
         Are_Symbolic_Names_Equal (Attr_Name, Ref_Constrained_String) or else
         Are_Symbolic_Names_Equal (Attr_Name, Ref_Count_String) or else
         Are_Symbolic_Names_Equal (Attr_Name, Ref_Delta_String) or else
         Are_Symbolic_Names_Equal (Attr_Name, Ref_Digits_String) or else
         Are_Symbolic_Names_Equal (Attr_Name, Ref_Emax_String) or else
         Are_Symbolic_Names_Equal (Attr_Name, Ref_Epsilon_String) or else
         Are_Symbolic_Names_Equal (Attr_Name, Ref_First_Bit_String) or else
         Are_Symbolic_Names_Equal (Attr_Name, Ref_Fore_String) or else
         Are_Symbolic_Names_Equal (Attr_Name, Ref_Large_String) or else
         Are_Symbolic_Names_Equal (Attr_Name, Ref_Last_Bit_String) or else
         Are_Symbolic_Names_Equal (Attr_Name, Ref_Machine_Emax_String) or else
         Are_Symbolic_Names_Equal (Attr_Name, Ref_Machine_Emin_String) or else
         Are_Symbolic_Names_Equal (Attr_Name, Ref_Machine_Mantissa_String)
          or else
         Are_Symbolic_Names_Equal (Attr_Name, Ref_Machine_Overflows_String)
          or else
         Are_Symbolic_Names_Equal (Attr_Name, Ref_Machine_Radix_String)
          or else
         Are_Symbolic_Names_Equal (Attr_Name, Ref_Machine_Rounds_String)
          or else Are_Symbolic_Names_Equal (Attr_Name, Ref_Mantissa_String)
          or else Are_Symbolic_Names_Equal (Attr_Name, Ref_Position_String)
          or else Are_Symbolic_Names_Equal (Attr_Name, Ref_Range_String)
          or else Are_Symbolic_Names_Equal (Attr_Name, Ref_Safe_Emax_String)
          or else Are_Symbolic_Names_Equal (Attr_Name, Ref_Safe_Large_String)
          or else Are_Symbolic_Names_Equal (Attr_Name, Ref_Safe_Small_String)
          or else Are_Symbolic_Names_Equal (Attr_Name, Ref_Size_String)
          or else Are_Symbolic_Names_Equal (Attr_Name, Ref_Small_String)
          or else Are_Symbolic_Names_Equal (Attr_Name, Ref_Storage_Size_String)
          or else Are_Symbolic_Names_Equal (Attr_Name, Ref_Terminated_String)
          or else Are_Symbolic_Names_Equal (Attr_Name, Ref_Value_String)
          or else Are_Symbolic_Names_Equal (Attr_Name, Ref_Width_String);
   end Non_Supported_Attribute;
   pragma INLINE (Non_Supported_Attribute);
begin

   -----
   Check_Expression (Context, Get_Attribute_Prefix (Node));

   Prefix_Entry_Node := Get_Symbol_Table_Entry (Get_Attribute_Prefix (Node));

   Attr_Node := Get_Attribute (Node);

   -- Does it have a parameter list?...
   if (Check_Expr_Node (Attr_Node) = Left_Parenthesis_Operator) then
      Attr_Name := Get_Symbolic_Name (Get_Object (Attr_Node));

      -- is is supported?
      if Non_Supported_Attribute (Attr_Name) then
         Show_Error (4103, Fatal, Convert_To_String (Attr_Name));

         -- it is, but should it have any parameters anyway?...
      elsif not Attribute_Should_Have_Parameter (Attr_Name) then
         Show_Error (4105, Fatal, Convert_To_String (Attr_Name));

         -- it should, but does it have more than one?...
      elsif not Has_Only_One_Parameter (Attr_Node) then
         Show_Error (4106, Fatal, Convert_To_String (Attr_Name));
      else

         -- just one. Everything is kosher.
         Attr_Param_Node := Get_List_Of_Items (Attr_Node);
         Check_Expression (Context, Attr_Param_Node);
      end if;
   else
      --it doesn't have any parameters...
      Attr_Name :=
         Node_Get_Routines.Get_Symbolic_Name
             (Node_Get_Routines.Get_Attribute (Node));
      -- (see if we support the thing)...
      if Non_Supported_Attribute (Attr_Name) then
         Show_Error (4103, Fatal, Convert_To_String (Attr_Name));
         -- but perhaps it should have parameters...
      elsif Attribute_Should_Have_Parameter (Attr_Name) then
         Show_Error (4107, Fatal, Convert_To_String (Attr_Name));
      end if;
   end if;

   -- now see if
   if Are_Symbolic_Names_Equal (Attr_Name, Ref_First_String) or else
      Are_Symbolic_Names_Equal (Attr_Name, Ref_Last_String) then
      if (Check_Expr_Node (Get_Attribute_Prefix (Node)) /= Terminal_Node) then
         Show_Error (4100, Fatal);
      elsif Is_Type_Mark (Prefix_Entry_Node) then
         Set_Type (Node, Prefix_Entry_Node);
      else
         Show_Error (4100, Fatal,
                     Convert_To_String (Get_Symbolic_Name (Prefix_Entry_Node)));
      end if;
   elsif Are_Symbolic_Names_Equal (Attr_Name, Ref_Pos_String) then
      if not Is_Type_Mark (Prefix_Entry_Node) then
         Show_Error (4100, Fatal,
                     Convert_To_String (Get_Symbolic_Name (Prefix_Entry_Node)));
      elsif (Check_Expr_Node (Get_Attribute_Prefix (Node)) /= Terminal_Node)
             then
         Show_Error (4100, Fatal);
      elsif Is_Empty (Get_Type (Attr_Param_Node)) or else
            (Get_Base_Type (Get_Type (Attr_Param_Node)) /=
            Get_Base_Type (Prefix_Entry_Node)) then
         Show_Error (4101, Fatal,
                     Convert_To_String (Get_Symbolic_Name (Prefix_Entry_Node)));
      else
         Set_Type (Node, Integer_Entry);
         Set_As_Universal_Type (Node);
      end if;

   elsif Are_Symbolic_Names_Equal (Attr_Name, Ref_Pred_String) or else
         Are_Symbolic_Names_Equal (Attr_Name, Ref_Succ_String) then
      if not Is_Type_Mark (Prefix_Entry_Node) then
         Show_Error (4100, Fatal,
                     Convert_To_String (Get_Symbolic_Name (Prefix_Entry_Node)));
      elsif (Check_Expr_Node (Get_Attribute_Prefix (Node)) /= Terminal_Node)
             then
         Show_Error (4100, Fatal);
      elsif not Is_A_Discrete_Type (Prefix_Entry_Node) then
         Show_Error (4108, Fatal,
                     Convert_To_String (Get_Symbolic_Name (Prefix_Entry_Node)));
      elsif Is_Empty (Get_Type (Attr_Param_Node)) or else
            (Get_Base_Type (Get_Type (Attr_Param_Node)) /=
            Get_Base_Type (Prefix_Entry_Node)) then
         Show_Error (4101, Fatal,
                     Convert_To_String (Get_Symbolic_Name (Prefix_Entry_Node)));
      else
         Set_Type (Node, Prefix_Entry_Node);
      end if;

   elsif Are_Symbolic_Names_Equal (Attr_Name, Ref_Image_String) then
      if not Is_Type_Mark (Prefix_Entry_Node) then
         Show_Error (4100, Fatal,
                     Convert_To_String (Get_Symbolic_Name (Prefix_Entry_Node)));
      elsif (Check_Expr_Node (Get_Attribute_Prefix (Node)) /= Terminal_Node)
             then
         Show_Error (4100, Fatal);
      elsif not Is_A_Discrete_Type (Prefix_Entry_Node) then
         Show_Error (4108, Fatal,
                     Convert_To_String (Get_Symbolic_Name (Prefix_Entry_Node)));
      elsif Is_Empty (Get_Type (Attr_Param_Node)) or else
            (Get_Base_Type (Get_Type (Attr_Param_Node)) /=
            Get_Base_Type (Prefix_Entry_Node)) then
         Show_Error (4101, Fatal,
                     Convert_To_String (Get_Symbolic_Name (Prefix_Entry_Node)));
      else
         Set_Type (Node, String_Entry);
      end if;

   elsif Are_Symbolic_Names_Equal (Attr_Name, Ref_Val_String) then
      if not Is_Type_Mark (Prefix_Entry_Node) then
         Show_Error (4100, Fatal,
                     Convert_To_String (Get_Symbolic_Name (Prefix_Entry_Node)));
      elsif (Check_Expr_Node (Get_Attribute_Prefix (Node)) /= Terminal_Node)
             then
         Show_Error (4100, Fatal);
      elsif not Is_A_Discrete_Type (Prefix_Entry_Node) then
         Show_Error (4108, Fatal,
                     Convert_To_String (Get_Symbolic_Name (Prefix_Entry_Node)));
      elsif Is_Empty (Get_Type (Attr_Param_Node)) or else
            not Is_Type_An_Integer_Type (Get_Type (Attr_Param_Node)) then
         Show_Error (4102, Fatal,
                     Convert_To_String (Get_Symbolic_Name (Attr_Param_Node)));
      else
         Set_Type (Node, Prefix_Entry_Node);
      end if;

   elsif Are_Symbolic_Names_Equal (Attr_Name, Ref_Length_String) then
      if not (Are_Symbolic_Names_Equal
             (Get_Symbolic_Name (Get_Type (Prefix_Entry_Node)),
              Convert_To_Ref_String ("string"))) then
         Show_Error (4110, Fatal, "String expected");
      else
         Set_Type (Node, Integer_Entry);
      end if;
      -- non-supported attribute...
   elsif Non_Supported_Attribute (Attr_Name) then
      Show_Error (4103, Fatal, Convert_To_String (Attr_Name));
   else
      -- don't recognize this attribute prefix...
      Show_Error (4104, Fatal, Convert_To_String (Attr_Name));
   end if;

end Check_Attribute;
