--|$Log:	cdot.a,v $
--|Revision 8.0  91/01/17  10:43:42  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Subunit for Package Name -  Check_Dot_Notation.Check_Expression
--
---------------------------------------------------------------
--
--
-- Abstract
--
--   This routine handles expanded names, etc. (e.g., record notation,
--   access notation).
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------

--==========================================================================
--  <Subprogram Name>
--
--    Check_Dot_Notation
--
--  <Description>
--
--    This routine handles expanded names, etc. (e.g., record notation,
--    access notation).
--
--==========================================================================
separate (Semantic_Analyzer.Check_Expression)

procedure Check_Dot_Notation (Node : in Ref_Statement_Database) is

   -- (sdb) Get_Object
   -- (sdb) Get_Dot_Component

   Current      : Ref_Statement_Database := Node;
   Found_Entry  : Ref_Symbol_Table_Entry := Empty;
   Terminal_Sdb : Ref_Statement_Database := Empty;
   Function_Sdb : Ref_Statement_Database := Empty;

   function Find_Corresponding_Item
       (Prefix   : in Ref_Symbol_Table_Entry;
        Selector : in Ref_Statement_Database) return Ref_Symbol_Table_Entry is
      List          : Ref_Statement_Database := Empty;
      Selector_Name : Ref_String := Get_Symbolic_Name (Selector);
      -- for possibly overloaded (subpgm) entries
      Found_Entry   : Ref_Symbol_Table_Entry := Empty;
   begin
      -- get correct head of possible selectors (decls)
      -- [tbd] do only those visible w/in "scope"
      case Get_Entry_Kind (Prefix) is
         when Package_Entry    =>
            List := Get_Package_Spec_Decls (Prefix);
         when Procedure_Entry  | Function_Entry    =>
            List := Get_Head_Of_Declarations_List (Prefix);
         when others =>
            Show_Error (1991, Fatal,
                        Convert_To_String (Get_Symbolic_Name (Prefix)));
      end case;

      -- look for selector
      -- and return selector's symtbl entry
   Look_Loop:
      while not Is_Empty (List) loop
         case Check_Stmt_Node (List) is

            when Object_Declaration =>
               declare
                  Object : Ref_Statement_Database := Get_Name_List (List);
                  Name   : Ref_String := Empty;
               begin
                  -- go through the list if apropos
                  while not Is_Empty (Object) loop
                     Name :=
                        Get_Symbolic_Name
                            (Get_Symbol_Table_Entry (Get_Expression (Object)));
                     if Are_Symbolic_Names_Equal (Name, Selector_Name) then
                        Found_Entry :=
                           Get_Symbol_Table_Entry (Get_Expression (Object));
                        exit Look_Loop; -- should only be one anyway (right?)
                     end if;
                     Object := Get_Next_List_Item (Object);
                  end loop;
               end;

            when Type_Declaration   =>

               if Are_Symbolic_Names_Equal
                      (Get_Symbolic_Name (Get_Symbol_Table_Entry (List)),
                       Selector_Name) then
                  -- mark this one, the last "marked" will be the
                  --   head of where overload resoltion will start looking
                  Found_Entry := Get_Symbol_Table_Entry (List);
               elsif Get_Entry_Kind (Get_Symbol_Table_Entry (List)) =
                     Type_Entry and then
                     (Is_Type_An_Enumeration_Type
                         (Get_Symbol_Table_Entry (List))) then
                  -- is type an enumeration type, then look at all the
                  -- enumerations
                  declare
                     Iterator : Ref_Symbol_Table_Entry;
                  begin
                     Iterator :=
                        Get_First_Literal (Get_Symbol_Table_Entry (List));
                     while (not Is_Empty (Iterator)) loop
                        if Are_Symbolic_Names_Equal
                               (Get_Symbolic_Name (Iterator), Selector_Name)
                            then
                           -- mark this one, the last "marked" will be the
                        --   head of where overload resoltion will start looking
                           Found_Entry := Iterator;
                        end if;
                        Iterator := Get_Successor (Iterator);
                     end loop;
                  end;
               end if;

            when Subtype_Declaration        |
                 Procedure_Declaration      |
                 Procedure_Spec_Declaration |
                 Function_Spec_Declaration  |
                 Function_Declaration       |
                 Package_Declaration        |
                 Package_Body_Declaration     =>

               if Are_Symbolic_Names_Equal
                      (Get_Symbolic_Name (Get_Symbol_Table_Entry (List)),
                       Selector_Name) then
                  -- mark this one, the last "marked" will be the
                  --   head of where overload resoltion will start looking
                  Found_Entry := Get_Symbol_Table_Entry (List);
               end if;

            when Use_Statement =>
               -- just ignore
               null;

            when Pragma_Statement   =>
               -- ignore these
               null;

            when others   =>
               Show_Error
                   (0, Fatal,
                    "Ada feature not yet implemented" &
                    "or yet unchecked semantic error (Check_Dot_Notation)");
         end case;
         List := Get_Next_Statement (List);
      end loop Look_Loop;

      if Is_Empty (Found_Entry) then
         -- no valid selector entry found
         Show_Error (1100, Fatal, Convert_To_String (Selector_Name));
         raise Program_Error;
      else
         return Found_Entry;
      end if;

   end Find_Corresponding_Item;

begin

   --- (variant) Dot_Notation_Variable_Reference

   -- it's either a record or dot notation variable reference

   -- for expanded names:
   --    - (grammar) prefix '.' selector
   --    - prefix can be: package (spec), program unit, (do these now)
   --        block stmt, loop stmt, accept stmt (forget these for now)

   -- [assume] it's a (simple) expanded name (for now):
   --    - lhs is only a single terminal
   --    - rhs is either another "dot_node" or single terminal

   -- [assume] "main"/"first" dot_object is references only visible items
   --    (no objects that are w/in a body or private part)

   -- get "main" ("first") prefix entry
   Check_Expression (Context, Get_Object (Node));
   Found_Entry := Get_Symbol_Table_Entry (Get_Object (Node));
   -- [tbd] test if it is a proper type of entry (can only be certain things)

   loop
      Current := Get_Dot_Component (Current);
      case Check_Expr_Node (Current) is
         when Terminal_Node    =>
            -- this is the last selector
            Found_Entry := Find_Corresponding_Item (Found_Entry, Current);
            exit ;
         when Dot_Operator     =>
            -- lhs of this current is the selector to use now
            Found_Entry := Find_Corresponding_Item (Found_Entry,
                                                    Get_Object (Current));
         when others =>
            -- can't be this here
            Show_Error (0, Fatal, "dot notation");
      end case;
   end loop;

   -- set the symtbl entry for this dot_operator node, specifying
   --    that the expanded name has been resolved
   --
   Set_Symbol_Table_Entry (Node, Found_Entry);

   if Get_Entry_Kind (Found_Entry) = Function_Entry then
      Function_Sdb := Create_Node (Function_Call);
      Terminal_Sdb := Create_Node (Terminal_Node);
      Set_Symbol_Table_Entry (Terminal_Sdb, Found_Entry);
      Set_Symbolic_Name (Terminal_Sdb, Get_Symbolic_Name (Found_Entry));
      Set_Subprogram_Info (Function_Sdb, Terminal_Sdb);
      Set_Database_Node (Node, Function_Sdb);
   end if;

end Check_Dot_Notation;
