--|$Log:	cinfix.a,v $
--|Revision 8.0  91/01/17  10:43:45  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Subunit for Package Name - Semantic_Analyzer
--
---------------------------------------------------------------
--
--
-- Abstract
--
--  Contains Check_Infix_Operations (a separate unit).
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------

--==========================================================================
-- <Subprogram Name>
--
-- Check_Infix_Operations
--
-- <Description>
--
-- Checks the semantic correctness of infix operations which ACE
-- treats as function calls.
--
-- <Notes>
--
-- If this routine is called, then this subpgm call is really a built-in,
-- infix operation.
--
--==========================================================================
with String_Constants;
use String_Constants;

separate (Semantic_Analyzer.Check_Subprogram_Call)

procedure Check_Infix_Operation (Node : in Ref_Statement_Database) is

   Subprogram_Name : Ref_String :=
      Get_Symbolic_Name (Get_Subprogram_Info (Node));

   First_Argument  : Ref_Statement_Database :=
      Get_Expression (Get_Parameter_List (Node));
   Second_Argument : Ref_Statement_Database;

   -- --------------------
   procedure Must_Be_Of_Some_Integer_Type (Node : in Ref_Statement_Database) is
   begin
      Set_Type (Universal_Type_Symtbl_Entry_Holder_Node, Integer_Entry);
      if not Is_Same_Type (Node, Universal_Type_Symtbl_Entry_Holder_Node) then
         Show_Error (1111, Fatal,
                     "expression expected of some integer type: " &
                     Convert_To_String (Subprogram_Name));
      end if;
   end Must_Be_Of_Some_Integer_Type;

   -- --------------------
   procedure Must_Be_Of_Some_Boolean_Type (Node : in Ref_Statement_Database) is
   begin
      Set_Type (Universal_Type_Symtbl_Entry_Holder_Node, Boolean_Entry);
      if not Is_Same_Type (Node, Universal_Type_Symtbl_Entry_Holder_Node) then
         Show_Error (1111, Fatal,
                     "expression expected of some boolean type: " &
                     Convert_To_String (Subprogram_Name));
      end if;
   end Must_Be_Of_Some_Boolean_Type;

   -- --------------------
   procedure Must_Be_Of_Some_String_Type (Node : in Ref_Statement_Database) is
   begin
      Set_Type (Universal_Type_Symtbl_Entry_Holder_Node, String_Entry);
      if not Is_Same_Type (Node, Universal_Type_Symtbl_Entry_Holder_Node) then
         Show_Error (1111, Fatal,
                     "expression expected of some string type: " &
                     Convert_To_String (Subprogram_Name));
      end if;
   end Must_Be_Of_Some_String_Type;

   -- --------------------
   procedure Must_Be_Of_Some_String_Or_Char_Type
       (Node : in Ref_Statement_Database) is
   begin
      Set_Type (Universal_Type_Symtbl_Entry_Holder_Node, String_Entry);
      if not Is_Same_Type (Node, Universal_Type_Symtbl_Entry_Holder_Node) then
         Set_Type (Universal_Type_Symtbl_Entry_Holder_Node, Character_Entry);
         if not Is_Same_Type (Node, Universal_Type_Symtbl_Entry_Holder_Node)
             then
            Show_Error (1111, Fatal,
                        "expression expected of string or character type: " &
                        Convert_To_String (Subprogram_Name));
         end if;
      end if;
   end Must_Be_Of_Some_String_Or_Char_Type;

   -- --------------------
   procedure Must_Be_Of_Some_Scalar_Type (Node : in Ref_Statement_Database) is
      Item_Root_Type : Ref_Symbol_Table_Entry :=
         Get_Root_Type (Get_Type (Node));
   begin
      if ((not Is_Type_An_Enumeration_Type (Item_Root_Type)) and
         (not Is_Type_An_Integer_Type (Item_Root_Type))) then
         Show_Error
             (1111, Fatal,
              "[TBD] --oei-- expression expected of some scalar type: " &
              Convert_To_String (Subprogram_Name));
      end if;
   end Must_Be_Of_Some_Scalar_Type;

   -- --------------------
   -- [TBD] This should really be Must_Be_Of_Some_Scalar_or_Discrete_Array_Type.
   --
   procedure Must_Be_Of_Some_Scalar_Or_String_Type
       (Node : in Ref_Statement_Database) is
      Item_Root_Type : Ref_Symbol_Table_Entry :=
         Get_Root_Type (Get_Type (Node));
   begin
      if ((not Is_Type_An_Enumeration_Type (Item_Root_Type)) and
         (not Is_Type_An_Integer_Type (Item_Root_Type))) then
         Set_Type (Universal_Type_Symtbl_Entry_Holder_Node, String_Entry);
         if not Is_Same_Type (Node, Universal_Type_Symtbl_Entry_Holder_Node)
             then
            Show_Error
                (1111, Fatal,
                 "[TBD] expression expected of some scalar type or String: " &
                 Convert_To_String (Subprogram_Name));
         end if;
      end if;
   end Must_Be_Of_Some_Scalar_Or_String_Type;
   -- --------------------
   -- [tbd] more numeric types
   procedure Must_Be_Of_Some_Numeric_Type (Node : in Ref_Statement_Database) is
   begin
      Must_Be_Of_Some_Integer_Type (Node);
   end Must_Be_Of_Some_Numeric_Type;

   -- --------------------
   procedure Set_Node_Type is
   begin
      if Is_Type_Universal (First_Argument) and
         Is_Type_Universal (Second_Argument) then
         Set_Type (Node, Get_Type (First_Argument));
         -- types for First_Argument and Second_Argument must be the same,
         -- so use either
         Set_As_Universal_Type (Node);
      elsif Is_Type_Universal (First_Argument) then
         Set_Type (Node, Get_Type (Second_Argument));
      elsif Is_Type_Universal (Second_Argument) then
         Set_Type (Node, Get_Type (First_Argument));
      else
         -- types for First_Argument and Second_Argument must be the same,
         -- so use either
         Set_Type (Node, Get_Type (First_Argument));
      end if;
   end Set_Node_Type;

   -- --------------------

begin

   if not Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Not_String) and
      not Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Abs_String) then
      Second_Argument :=
         Get_Expression (Get_Next_List_Item (Get_Parameter_List (Node)));
   end if;

   if Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Plus_String) or else
      Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Minus_String) or else
      Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Times_String) or else
      Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Divide_String) then
      -- arguments should be of any same numeric type
      Must_Be_Of_Some_Numeric_Type (First_Argument);
      Must_Be_Of_Some_Numeric_Type (Second_Argument);
      if not Is_Same_Type (First_Argument, Second_Argument) then
         Show_Error (1111, Fatal,
                     "arguments must be of same numeric type: " &
                     Convert_To_String (Subprogram_Name));
      else
         Set_Node_Type;
      end if;

   elsif Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Mod_String) or else
         Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Rem_String) then
      -- arguments should be of any same integer type
      Must_Be_Of_Some_Integer_Type (First_Argument);
      Must_Be_Of_Some_Integer_Type (Second_Argument);
      if not Is_Same_Type (First_Argument, Second_Argument) then
         Show_Error (1111, Fatal,
                     "arguments must be of same integer type: " &
                     Convert_To_String (Subprogram_Name));
      else
         Set_Node_Type;
      end if;

   elsif Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Exp_String) then
      Must_Be_Of_Some_Numeric_Type (First_Argument);
      Must_Be_Of_Some_Integer_Type (Second_Argument);
      Set_Type (Node, Get_Type (First_Argument));
      if Is_Type_Universal (First_Argument) then
         Set_As_Universal_Type (Node);
      end if;

   elsif Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Abs_String) then
      -- [tbd] actually only integer or floating point type allowed
      Must_Be_Of_Some_Numeric_Type (First_Argument);
      Set_Type (Node, Get_Type (First_Argument));
      if Is_Type_Universal (First_Argument) then
         Set_As_Universal_Type (Node);
      end if;

   elsif Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Concat_String) then
      -- arguments should be of any array or the component type
      -- [tbd] Must_Be_Of_Some_component_or_array_Type (First_Argument);
      Must_Be_Of_Some_String_Or_Char_Type (First_Argument);
      -- [tbd] Must_Be_Of_Some_component_or_array_Type (Second_Argument);
      Must_Be_Of_Some_String_Or_Char_Type (Second_Argument);
      -- [tbd] always returns the same array type
      -- or _any_ array type if both args are components
      Set_Type (Node, String_Entry);

   elsif Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Equal_String) or else
         Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Not_Equal_String) then
      -- arguments should be of any same type
      if not Is_Same_Type (First_Argument, Second_Argument) then
         Show_Error (1111, Fatal,
                     "arguments must be of same type: " &
                     Convert_To_String (Subprogram_Name));
      end if;
      Set_Type (Node, Boolean_Entry);

   elsif Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Greater_Eq_String)
          or else
         Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Greaterthan_String)
          or else
         Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Lessthan_String)
          or else
         Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Less_Eq_String) then
      -- arguments should be of the any same scalar type or
      -- [TBD] a discrete array type.
      -- Currently Strings are an exception.
      Must_Be_Of_Some_Scalar_Or_String_Type (First_Argument);
      Must_Be_Of_Some_Scalar_Or_String_Type (Second_Argument);
      if not Is_Same_Type (First_Argument, Second_Argument) then
         Show_Error (1111, Fatal,
                     "arguments must be of same scalar type: " &
                     Convert_To_String (Subprogram_Name));
      end if;
      Set_Type (Node, Boolean_Entry);

   elsif Are_Symbolic_Names_Equal (Subprogram_Name, Ref_And_String) or else
         Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Or_String) or else
         Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Xor_String) then
      -- arguments should be of the any same boolean type
      Must_Be_Of_Some_Boolean_Type (First_Argument);
      Must_Be_Of_Some_Boolean_Type (Second_Argument);
      if not Is_Same_Type (First_Argument, Second_Argument) then
         Show_Error (1111, Fatal,
                     "arguments must be of same boolean type: " &
                     Convert_To_String (Subprogram_Name));
      else
         Set_Node_Type;
      end if;

   elsif Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Not_String) then
      Must_Be_Of_Some_Boolean_Type (First_Argument);
      Set_Type (Node, Get_Type (First_Argument));
      if Is_Type_Universal (First_Argument) then
         Set_As_Universal_Type (Node);
      end if;

   else
      Show_Error (1990, Internal,
                  "Check_Infix_Operation: " &
                  Convert_To_String (Subprogram_Name));
   end if;

end Check_Infix_Operation;
