--|$Log:	cmsppt.a,v $
--|Revision 8.0  91/01/17  10:43:47  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Subunit for Package - Semantic_Analyzer
--
---------------------------------------------------------------
--
--
-- Abstract
--
--   The following contains several "separate" unit for Semantic_Analyzer.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------

-- -----------------------------------------------------------------------
-- [private to package]
--
--==========================================================================
--  <Subprogram Name>
--
--    Is_Mode_In
--
--  <Description>
--
--    returns a boolean if an object is readable.
--
--==========================================================================
separate (Semantic_Analyzer)

function Is_Mode_In (Node : in Ref_Statement_Database) return Boolean is
begin
   case Check_Expr_Node (Node) is
      when Terminal_Node |
           Array_Reference =>
         return Is_Mode_In (Get_Symbol_Table_Entry (Node));
      when List_Item =>
         -- return mode of FIRST item in list
         return Is_Mode_In (Get_Expression (Node));
      when Constant_Node  |
           String_Node    |
           Character_Node |
           Function_Call  |
           Type_Conversion     =>
         return True;
      when Dot_Operator   =>
         if not Is_Empty (Get_Symbol_Table_Entry (Node)) then
            return Is_Mode_In (Get_Symbol_Table_Entry (Node));
         end if;
         Show_Error (1003, Internal,
                     "Is_Mode_In: " &
                     Expression_Enumeration'IMAGE (Check_Expr_Node (Node)));
         return False;
      when Unknown   | Left_Parenthesis_Operator   =>
         Show_Error (1003, Internal,
                     "Is_Mode_In: " &
                     Expression_Enumeration'IMAGE (Check_Expr_Node (Node)));
         return False;
      when Dot_Notation_Variable_Reference |
           Record_Reference                |
           Range_Expression                |
           Aggregate_Component             |
           Attribute_Tick =>
         Show_Error (1011, Internal,
                     "Is_Mode_In: " &
                     Expression_Enumeration'IMAGE (Check_Expr_Node (Node)));
         return False;
   end case;
end Is_Mode_In;

-- -----------------------------------------------------------------------
--==========================================================================
--  <Subprogram Name>
--
--    Is_Mode_Out
--
--  <Description>
--
--    returns a boolean true if an object if writeable
--
--==========================================================================
separate (Semantic_Analyzer)

function Is_Mode_Out (Node : in Ref_Statement_Database) return Boolean is
begin
   case Check_Expr_Node (Node) is
      when Terminal_Node  |
           Array_Reference =>
         case Get_Entry_Kind (Get_Symbol_Table_Entry (Node)) is
            when Object_Entry  | Function_Entry    =>
               return Is_Mode_Out (Get_Symbol_Table_Entry (Node));
            when others   =>
               return False;
         end case;
      when List_Item =>
         -- return mode of FIRST item in list
         return Is_Mode_Out (Get_Expression (Node));
      when Constant_Node       |
           String_Node         |
           Character_Node      |
           Attribute_Tick      |
           Function_Call       |
           Aggregate_Component |
           Type_Conversion     =>
         return False;
      when Dot_Operator   =>
         if not Is_Empty (Get_Symbol_Table_Entry (Node)) then
            return Is_Mode_Out (Get_Symbol_Table_Entry (Node));
         end if;
         Show_Error (1003, Internal,
                     "Is_Mode_Out: " &
                     Expression_Enumeration'IMAGE (Check_Expr_Node (Node)));
         return False;
      when Unknown   | Left_Parenthesis_Operator   =>
         Show_Error (1003, Internal,
                     "Is_Mode_Out: " &
                     Expression_Enumeration'IMAGE (Check_Expr_Node (Node)));
         return False;
      when Dot_Notation_Variable_Reference |
           Record_Reference                |
           Range_Expression    =>
         Show_Error (1011, Internal,
                     "Is_Mode_Out: " &
                     Expression_Enumeration'IMAGE (Check_Expr_Node (Node)));
         return False;
   end case;
end Is_Mode_Out;

-- -----------------------------------------------------------------------
--
--==========================================================================
--  <Subprogram Name>
--
--    Must_Be_Boolean_Type
--
--  <Description>
--
--    Outputs an error message if an expression is not a boolean
--    expression.
--
--==========================================================================
with Ace;

separate (Semantic_Analyzer)

procedure Must_Be_Boolean_Type (Node : in Ref_Statement_Database) is

   -- Effects:
   --   The given expression must be of type "boolean".

begin
   -- suggest that resulting expression should be of type "boolean"
   Set_Type (Node, Boolean_Entry);

   -- Do we need this check here? TBD (wpl)
   Check_Expression (Ace.Main_Thread_Context, Node);

   Set_Type (Universal_Type_Symtbl_Entry_Holder_Node, Boolean_Entry);
   if not Is_Same_Type (Node, Universal_Type_Symtbl_Entry_Holder_Node) then
      Show_Error (1111, Fatal, "boolean expression expected");
   end if;
end Must_Be_Boolean_Type;
