--|$Log:	compilation_unit.a,v $
--|Revision 8.0  91/01/17  10:43:50  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|

with Yyval_Error;
use Yyval_Error;

with User_Pkg;
use User_Pkg;

with Lexacts;
use Lexacts;

with Error_Handler;
use Error_Handler;

with Help_Pkg;
use Help_Pkg;

with Declaration_Routines;
use Declaration_Routines;

with Declarative_Region_Support_Routines;
use Declarative_Region_Support_Routines;

with Semantic_Analyzer;
use Semantic_Analyzer;

with Create_Statement_Database;
use Create_Statement_Database;

with Ace_Text_Io;
use  Ace_Text_Io;

with Parserdefs; --  Remove later [TBD] wpl

with Ace;

-- Parser packages

with Common_Parser_Routines;
use  Common_Parser_Routines;

with Tokens_Definition, compilation_unit_shift_reduce, compilation_unit_goto;

package body compilation_unit_parser is

procedure YYParse is

   -- Rename User Defined Packages to Internal Names.
    package yy_goto_tables         renames
      Compilation_Unit_Goto;
    package yy_shift_reduce_tables renames
      Compilation_Unit_Shift_Reduce;
    package yy_tokens              renames
      Tokens_Definition;

   use yy_tokens, yy_goto_tables, yy_shift_reduce_tables;

   procedure yyerrok;
   procedure yyclearin;

   package yy is

       -- the size of the value and state stacks
       stack_size : constant Natural := 300;

       -- subtype rule         is natural;
       subtype parse_state  is natural;
       -- subtype nonterminal  is integer;

       -- encryption constants
       default           : constant := -1;
       first_shift_entry : constant :=  0;
       accept_code       : constant := -1001;
       error_code        : constant := -1000;

       -- stack data used by the parser
       tos                : natural := 0;
       value_stack        : array(0..stack_size) of yy_tokens.yystype;
       state_stack        : array(0..stack_size) of parse_state;

       -- current input symbol and action the parser is on
       action             : integer;
       rule_id            : rule;
       input_symbol       : yy_tokens.token;


       -- error recovery flag
       error_flag : natural := 0;
          -- indicates  3 - (number of valid shifts after an error occurs)

       look_ahead : boolean := true;
       index      : integer;

       -- Is Debugging option on or off
        DEBUG : constant boolean := FALSE;

    end yy;


    function goto_state
      (state : yy.parse_state;
       sym   : nonterminal) return yy.parse_state;

    function parse_action
      (state : yy.parse_state;
       t     : yy_tokens.token) return integer;

    pragma inline(goto_state, parse_action);


    function goto_state(state : yy.parse_state;
                        sym   : nonterminal) return yy.parse_state is
        index : integer;
    begin
        index := goto_offset(state);
        while  integer(goto_matrix(index).nonterm) /= sym loop
            index := index + 1;
        end loop;
        return integer(goto_matrix(index).newstate);
    end goto_state;


    function parse_action(state : yy.parse_state;
                          t     : yy_tokens.token) return integer is
        index      : integer;
        tok_pos    : integer;
        default    : constant integer := -1;
    begin
        tok_pos := yy_tokens.token'pos(t);
        index   := shift_reduce_offset(state);
        while integer(shift_reduce_matrix(index).t) /= tok_pos and then
              integer(shift_reduce_matrix(index).t) /= default
        loop
            index := index + 1;
        end loop;
        return integer(shift_reduce_matrix(index).act);
    end parse_action;

-- error recovery stuff

    procedure handle_error is
      temp_action : integer;
    begin

      if yy.error_flag = 3 then -- no shift yet, clobber input.
      if yy.debug then
          put_line("Ayacc.YYParse: Error Recovery Clobbers " &
                   yy_tokens.token'image(yy.input_symbol));
      end if;
        if yy.input_symbol = yy_tokens.end_of_input then  -- don't discard,
        if yy.debug then
            put_line("Ayacc.YYParse: Can't discard END_OF_INPUT, quiting...");
        end if;
        raise yy_tokens.syntax_error;
        end if;

            yy.look_ahead := true;   -- get next token
        return;                  -- and try again...
    end if;

    if yy.error_flag = 0 then -- brand new error
        yyerror("Syntax Error");
    end if;

    yy.error_flag := 3;

    -- find state on stack where error is a valid shift --

    if yy.debug then
        put_line("Ayacc.YYParse: Looking for state with error as valid shift");
    end if;

    loop
        if yy.debug then
        put_line("Ayacc.YYParse: Examining State " &
              yy.parse_state'image(yy.state_stack(yy.tos)));
        end if;
        temp_action := parse_action(yy.state_stack(yy.tos), error);

            if temp_action >= yy.first_shift_entry then
                yy.tos := yy.tos + 1;
                yy.state_stack(yy.tos) := temp_action;
                exit;
            end if;

        Decrement_Stack_Pointer :
        begin
          yy.tos := yy.tos - 1;
        exception
          when Constraint_Error =>
            yy.tos := 0;
        end Decrement_Stack_Pointer;

        if yy.tos = 0 then
          if yy.debug then
            put_line("Ayacc.YYParse: Error recovery popped entire stack, aborting...");
          end if;
          raise yy_tokens.syntax_error;
        end if;
    end loop;

    if yy.debug then
        put_line("Ayacc.YYParse: Shifted error token in state " &
              yy.parse_state'image(yy.state_stack(yy.tos)));
    end if;

    end handle_error;

   -- print debugging information for a shift operation
   procedure shift_debug(state_id: yy.parse_state; lexeme: yy_tokens.token) is
   begin
       put_line("Ayacc.YYParse: Shift "& yy.parse_state'image(state_id)&" on input symbol "&
               yy_tokens.token'image(lexeme) );
   end;

   -- print debugging information for a reduce operation
   procedure reduce_debug(rule_id: rule; state_id: yy.parse_state) is
   begin
       put_line("Ayacc.YYParse: Reduce by rule "&rule'image(rule_id)&" goto state "&
               yy.parse_state'image(state_id));
   end;

   -- make the parser believe that 3 valid shifts have occured.
   -- used for error recovery.
   procedure yyerrok is
   begin
       yy.error_flag := 0;
   end yyerrok;

   -- called to clear input symbol that caused an error.
   procedure yyclearin is
   begin
       -- yy.input_symbol := yylex;
       yy.look_ahead := true;
   end yyclearin;


begin
    -- initialize by pushing state 0 and getting the first input symbol
    yy.state_stack(yy.tos) := 0;

    loop

        yy.index := shift_reduce_offset(yy.state_stack(yy.tos));
        if integer(shift_reduce_matrix(yy.index).t) = yy.default then
            yy.action := integer(shift_reduce_matrix(yy.index).act);
        else
            if yy.look_ahead then
                yy.look_ahead   := false;
                yy.input_symbol := yylex;
            end if;
            yy.action :=
             parse_action(yy.state_stack(yy.tos), yy.input_symbol);
        end if;

        if yy.action >= yy.first_shift_entry then  -- SHIFT

            if yy.debug then
                shift_debug(yy.action, yy.input_symbol);
            end if;

            -- Enter new state
            yy.tos := yy.tos + 1;
            yy.state_stack(yy.tos) := yy.action;
            yy.value_stack(yy.tos) := yylval;

        if yy.error_flag > 0 then  -- indicate a valid shift
            yy.error_flag := yy.error_flag - 1;
        end if;

            -- Advance lookahead
            yy.look_ahead := true;

        elsif yy.action = yy.error_code then       -- ERROR
            handle_error;

        elsif yy.action = yy.accept_code then
            if yy.debug then
                put_line("Ayacc.YYParse: Accepting Grammar...");
            end if;
            exit;

        else -- Reduce Action

            -- Convert action into a rule
            yy.rule_id  := -1 * yy.action;

            -- Execute User Action
            -- user_action(yy.rule_id);
            case yy.rule_id is

when  1 =>
--#line  92
 Push_Eol; 

when  2 =>
--#line  94
 Pop_Eol; 

when  3 =>
--#line  96
 NQ_Pkg_Help(Get_Help); 

when  4 =>
--#line  98
 Lval := 
yy.value_stack(yy.tos); 

when  5 =>
--#line  103
 
yyval := Create_Pragma_Node(
yy.value_stack(yy.tos-3), 
yy.value_stack(yy.tos-2));
	     Check_Statement (Ace.Main_Thread_Context, 
yyval); 

when  6 =>
--#line  108
 
yyval := 
yy.value_stack(yy.tos); 

when  7 =>
--#line  110
 Show_Error(3016, Fatal, "Named Association"); 

when  8 =>
--#line  118
 Check_Statement (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos));
                          
yyval := 
yy.value_stack(yy.tos); 

when  9 =>
--#line  120
 Check_Statement (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos));
                          
yyval := 
yy.value_stack(yy.tos); 

when  10 =>
--#line  122
 Check_Statement (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos));
                          
yyval := 
yy.value_stack(yy.tos); 

when  11 =>
--#line  124
 
yyval := 
yy.value_stack(yy.tos); 

when  12 =>
--#line  125
 
yyval := 
yy.value_stack(yy.tos); 

when  13 =>
--#line  126
 Show_Error(3016, Fatal, "Tasking"); 

when  14 =>
--#line  127
 Show_Error(3016, Fatal, "Generics"); 

when  15 =>
--#line  128
 Check_Statement (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos));
                          
yyval := 
yy.value_stack(yy.tos); 

when  16 =>
--#line  130
 Show_Error(3016, Fatal, "Generics"); 

when  17 =>
--#line  131
 Show_Error(3016, Fatal, "Renaming"); 

when  18 =>
--#line  132
 
yyval := 
yy.value_stack(yy.tos); 

when  19 =>
--#line  140
 Create_Object_Decl (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos-5), FALSE, 
yy.value_stack(yy.tos-3), 
yy.value_stack(yy.tos-2), 
yyval); 

when  20 =>
--#line  142
 Create_Object_Decl (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos-6), TRUE, 
yy.value_stack(yy.tos-3), 
yy.value_stack(yy.tos-2), 
yyval); 

when  21 =>
--#line  144
 Create_Object_Decl (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos-5), FALSE, 
yy.value_stack(yy.tos-3), 
yy.value_stack(yy.tos-2), 
yyval); 

when  22 =>
--#line  146
 Create_Object_Decl (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos-6), TRUE, 
yy.value_stack(yy.tos-3), 
yy.value_stack(yy.tos-2), 
yyval); 

when  23 =>
--#line  152
  Create_Number_Decl (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos-6), 
yy.value_stack(yy.tos-2), 
yyval);
       Check_Statement (Ace.Main_Thread_Context, 
yyval); 

when  24 =>
--#line  157
 
yyval := Link_Id_Onto_Front_Of_List (
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos)); 

when  25 =>
--#line  164
 
yyval := 
yy.value_stack(yy.tos); 

when  26 =>
--#line  165
 Show_Error(3016, Fatal, "Incomplete Declarations"); 

when  27 =>
--#line  166
 Show_Error(3016, Fatal, "Private Declarations"); 

when  28 =>
--#line  172
 
yyval := Create_Type_Decl (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos-4), null, 
yy.value_stack(yy.tos-2)); 

when  29 =>
--#line  174
 
yyval := Create_Type_Decl (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos-5), 
yy.value_stack(yy.tos-4), 
yy.value_stack(yy.tos-2)); 

when  30 =>
--#line  181
 
yyval := Create_Enum_Type_Def_Entry (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos)); 

when  31 =>
--#line  182
 
yyval := 
yy.value_stack(yy.tos); 

when  32 =>
--#line  183
 
yyval := 
yy.value_stack(yy.tos); 

when  33 =>
--#line  184
 
yyval := 
yy.value_stack(yy.tos); 

when  34 =>
--#line  185
 Show_Error(3016, Fatal, "Records"); 

when  35 =>
--#line  186
 Show_Error(3016, Fatal, "Access"); 

when  36 =>
--#line  187
 
yyval := 
yy.value_stack(yy.tos); 

when  37 =>
--#line  194
 
yyval := Create_Subtype_Decl (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos-4), 
yy.value_stack(yy.tos-2)); 

when  38 =>
--#line  198
 
yyval := Build_Subtype_Indication (
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos)); 

when  39 =>
--#line  201
  Check_Expression(Ace.Main_Thread_Context, 
yy.value_stack(yy.tos)); 
                        
yyval := Resolve_Type_Mark (
yy.value_stack(yy.tos));


when  40 =>
--#line  210
 Show_Error(3016, Warning, "Constraints"); 

when  41 =>
--#line  211
 Show_Error(3016, Fatal, "Floating Point"); 

when  42 =>
--#line  212
 Show_Error(3016, Fatal, "Fixed Point"); 

when  43 =>
--#line  213
 
yyval := 
yy.value_stack(yy.tos); Check_Expression (Ace.Main_Thread_Context, 
yyval);  

when  44 =>
--#line  220
 
yyval := Mark_Derived_Type (
yy.value_stack(yy.tos)); 

when  45 =>
--#line  224
 ---$$ := Build_Range_Constraint ($2);
         ---THIS IS STILL UNDER DEVELOPMENT
         ---COMPLETE IMPLEMENTATION NOT YET DONE  [TBD]
         
yyval := 
yy.value_stack(yy.tos);
       

when  46 =>
--#line  232
 
yyval := 
yy.value_stack(yy.tos); 

when  47 =>
--#line  233
 
yyval := Create_Range_Expr_Node (
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos)); 

when  48 =>
--#line  240
 
yyval := Link_Enum_Literal_On_Front (
yy.value_stack(yy.tos-2), 
yy.value_stack(yy.tos-1)); 

when  49 =>
--#line  244
 
yyval := Create_Enum_Literal_Entry (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos)); 

when  50 =>
--#line  247
 
yyval := 
yy.value_stack(yy.tos); 

when  51 =>
--#line  248
 
yyval := 
yy.value_stack(yy.tos); 

when  52 =>
--#line  255
 
yyval := Build_Integer_Type (
yy.value_stack(yy.tos)); 

when  53 =>
--#line  261
 Show_Error(3016, Fatal, "Float"); 

when  54 =>
--#line  262
 Show_Error(3016, Fatal, "Fixed Point"); 

when  55 =>
--#line  269
 
yyval := 
yy.value_stack(yy.tos-1); 

when  56 =>
--#line  273
 
yyval := 
yy.value_stack(yy.tos-1); 

when  57 =>
--#line  279
 
yyval := 
yy.value_stack(yy.tos-1); 

when  58 =>
--#line  283
 
yyval := 
yy.value_stack(yy.tos-1); 

when  59 =>
--#line  289
 
yyval := 
yy.value_stack(yy.tos); 

when  60 =>
--#line  290
 
yyval := 
yy.value_stack(yy.tos); 

when  61 =>
--#line  297
 
yyval := Build_Unconstrained_Array_Decl (
yy.value_stack(yy.tos-4), 
yy.value_stack(yy.tos-3), 
yy.value_stack(yy.tos-1)); 

when  62 =>
--#line  303
 
yyval := Build_Constrained_Array_Decl (
yy.value_stack(yy.tos-2), 
yy.value_stack(yy.tos)); 

when  63 =>
--#line  307
 
yyval := 
yy.value_stack(yy.tos-2); 

when  64 =>
--#line  311
 
yyval := Link_Id_Onto_Rear_Of_List (
yy.value_stack(yy.tos-2), 
yy.value_stack(yy.tos-1)); 

when  65 =>
--#line  314
 
yyval := 
yy.value_stack(yy.tos); 

when  66 =>
--#line  315
 
yyval := 
yy.value_stack(yy.tos-1);
                     --Show_Error(3016, Warning, "Constrained Ranges");
                    

when  67 =>
--#line  325
 
yyval := Create_Record_Type_Entry (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos-2)); 

when  68 =>
--#line  330
 
yyval := Link_Record_Component_Entries (
yy.value_stack(yy.tos-2), 
yy.value_stack(yy.tos-1)); 

when  69 =>
--#line  331
 
yyval := 
yy.value_stack(yy.tos-3); 

when  70 =>
--#line  333
 
yyval := null; 

when  71 =>
--#line  340
 
yyval := Build_Record_Component (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos-5), 
yy.value_stack(yy.tos-3), 
yy.value_stack(yy.tos-2)); 

when  72 =>
--#line  344
 
yyval := 
yy.value_stack(yy.tos); 

when  73 =>
--#line  350
 
yyval := 
yy.value_stack(yy.tos-3); 

when  74 =>
--#line  354
 
yyval := 
yy.value_stack(yy.tos-3); 

when  75 =>
--#line  373
 
yyval := 
yy.value_stack(yy.tos-10); 

when  76 =>
--#line  378
 
yyval := 
yy.value_stack(yy.tos-4); 

when  77 =>
--#line  382
 
yyval := 
yy.value_stack(yy.tos); 

when  78 =>
--#line  383
 
yyval := Create_Expr_Node (
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos)); 

when  79 =>
--#line  385
 
yyval := Create_Range_Expr_Node (
yy.value_stack(yy.tos-2), 
yy.value_stack(yy.tos)); 

when  80 =>
--#line  386
 
yyval := Create_Expr_Node (parserdefs.OTHERS_TK); 

when  81 =>
--#line  392
 
yyval := 
yy.value_stack(yy.tos-1); 

when  82 =>
--#line  398
 
yyval := 
yy.value_stack(yy.tos-4); 

when  83 =>
--#line  399
 
yyval := 
yy.value_stack(yy.tos-5); 

when  84 =>
--#line  405
 
yyval := 
yy.value_stack(yy.tos); 

when  85 =>
--#line  407
 
yyval := Create_Right_Stmt_Seq_Node (
yy.value_stack(yy.tos-2), 
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos)); 

when  86 =>
--#line  411
 
yyval := 
yy.value_stack(yy.tos); 

when  87 =>
--#line  412
 
yyval := 
yy.value_stack(yy.tos); 

when  88 =>
--#line  413
 
yyval := 
yy.value_stack(yy.tos); Check_Statement(Ace.Main_Thread_Context, 
yyval);

when  89 =>
--#line  418
 
yyval := 
yy.value_stack(yy.tos); 

when  90 =>
--#line  419
 
yyval := 
yy.value_stack(yy.tos); 

when  91 =>
--#line  420
 
yyval := 
yy.value_stack(yy.tos); 

when  92 =>
--#line  421
 Show_Error(3016, Fatal, "Tasking"); 

when  93 =>
--#line  422
 Show_Error(3016, Fatal, "Generics"); 

when  94 =>
--#line  423
 
yyval := 
yy.value_stack(yy.tos); Check_Statement(Ace.Main_Thread_Context, 
yyval);

when  95 =>
--#line  424
 Show_Error(3016, Fatal, "Generics"); 

when  96 =>
--#line  427
 
yyval := 
yy.value_stack(yy.tos); 

when  97 =>
--#line  428
 Show_Error(3016, Fatal, "Separate"); 

when  98 =>
--#line  432
 
yyval := 
yy.value_stack(yy.tos); 

when  99 =>
--#line  433
 
yyval := 
yy.value_stack(yy.tos); 

when  100 =>
--#line  434
 Show_Error(3016, Fatal, "Tasking"); 

when  101 =>
--#line  439
 
yyval := 
yy.value_stack(yy.tos); 

when  102 =>
--#line  440
 
yyval := 
yy.value_stack(yy.tos); 

when  103 =>
--#line  441
 
yyval := 
yy.value_stack(yy.tos); 

when  104 =>
--#line  442
 
yyval := 
yy.value_stack(yy.tos); 

when  105 =>
--#line  443
 
yyval := 
yy.value_stack(yy.tos); 

when  106 =>
--#line  445
  
yyval := 
yy.value_stack(yy.tos); 

when  107 =>
--#line  448
 
yyval := 
yy.value_stack(yy.tos); 

when  108 =>
--#line  456
 
yyval := Create_Expr_Node (
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos)); 

when  109 =>
--#line  470
 
yyval := Create_Dot_Expr_Node (
yy.value_stack(yy.tos-2), 
yy.value_stack(yy.tos)); 

when  110 =>
--#line  473
 
yyval := 
yy.value_stack(yy.tos); 

when  111 =>
--#line  474
 
yyval := 
yy.value_stack(yy.tos); 

when  112 =>
--#line  475
 
yyval := 
yy.value_stack(yy.tos); 

when  113 =>
--#line  476
 
yyval := Create_Expr_Node (parserdefs.ALL_TK); 

when  114 =>
--#line  483
 
yyval := Create_Attr_Expr_Node (
yy.value_stack(yy.tos-2), 
yy.value_stack(yy.tos)); 

when  115 =>
--#line  486
 
yyval := 
yy.value_stack(yy.tos); 

when  116 =>
--#line  492
 
yyval := 
yy.value_stack(yy.tos); 

when  117 =>
--#line  500
 
yyval := Link_Id_Onto_Front_Of_List (
yy.value_stack(yy.tos-2),
yy.value_stack(yy.tos-1));
       

when  118 =>
--#line  506
 
yyval := 
yy.value_stack(yy.tos); 

when  119 =>
--#line  508
 Show_Error(3016, Fatal, "Named Association"); 

when  120 =>
--#line  510
 
yyval := Create_Range_Expr_Node (
yy.value_stack(yy.tos-2), 
yy.value_stack(yy.tos)); 

when  121 =>
--#line  512
 
yyval := Create_Name_Range_Expr_Node (
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos)); 

when  122 =>
--#line  518
 
yyval := 
yy.value_stack(yy.tos); 

when  123 =>
--#line  519
 
yyval := 
yy.value_stack(yy.tos); 

when  124 =>
--#line  520
 
yyval := 
yy.value_stack(yy.tos); 

when  125 =>
--#line  521
 
yyval := 
yy.value_stack(yy.tos); 

when  126 =>
--#line  522
 
yyval := 
yy.value_stack(yy.tos); 

when  127 =>
--#line  526
 
yyval := Create_Left_Expr_Node (
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos)); 

when  128 =>
--#line  527
 
yyval := 
yy.value_stack(yy.tos); 

when  129 =>
--#line  531
 
yyval := 
yy.value_stack(yy.tos); 

when  130 =>
--#line  534
 
yyval := 
yy.value_stack(yy.tos); 

when  131 =>
--#line  537
 
yyval := Create_Left_Expr_Node (
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos)); 

when  132 =>
--#line  538
 
yyval := Create_Unary_Expr_Node (parserdefs.ABS_TK, 
yy.value_stack(yy.tos)); 

when  133 =>
--#line  539
 
yyval := Create_Unary_Expr_Node (parserdefs.NOT_TK, 
yy.value_stack(yy.tos)); 

when  134 =>
--#line  543
 
yyval := 
yy.value_stack(yy.tos); 

when  135 =>
--#line  544
 
yyval := Create_Expr_Node (parserdefs.NULL_TK); 

when  136 =>
--#line  545
 
yyval := 
yy.value_stack(yy.tos); 

when  137 =>
--#line  546
 
yyval := 
yy.value_stack(yy.tos); 

when  138 =>
--#line  547
 
yyval := 
yy.value_stack(yy.tos); 

when  139 =>
--#line  548
 
yyval := 
yy.value_stack(yy.tos); 

when  140 =>
--#line  551
 
yyval := Create_Expr_Node (character'pos('=')); 

when  141 =>
--#line  552
 
yyval := Create_Expr_Node (parserdefs.NOTEQL_TK);          

when  142 =>
--#line  553
 
yyval := Create_Expr_Node (character'pos('<')); 

when  143 =>
--#line  554
 
yyval := Create_Expr_Node (parserdefs.LTEQ_TK);            

when  144 =>
--#line  555
 
yyval := Create_Expr_Node (character'pos('>')); 

when  145 =>
--#line  556
 
yyval := Create_Expr_Node (parserdefs.GTEQL_TK);           

when  146 =>
--#line  559
 
yyval := Create_Expr_Node (character'pos('+')); 

when  147 =>
--#line  560
 
yyval := Create_Expr_Node (character'pos('-')); 

when  148 =>
--#line  561
 
yyval := Create_Expr_Node (character'pos('&')); 

when  149 =>
--#line  564
 
yyval := Create_Expr_Node (character'pos('+')); 

when  150 =>
--#line  565
 
yyval := Create_Expr_Node (character'pos('-')); 

when  151 =>
--#line  568
 
yyval := Create_Expr_Node (character'pos('*')); 

when  152 =>
--#line  569
 
yyval := Create_Expr_Node (character'pos('/')); 

when  153 =>
--#line  570
 
yyval := Create_Expr_Node (parserdefs.MOD_TK);             

when  154 =>
--#line  571
 
yyval := Create_Expr_Node (parserdefs.REM_TK);             

when  155 =>
--#line  575
 
yyval := 
yy.value_stack(yy.tos); 

when  156 =>
--#line  580
 
yyval := Create_Expr_Node (parserdefs.NEW_TK, 
yy.value_stack(yy.tos)); 

when  157 =>
--#line  581
 
yyval := Create_New_Expr_Node (
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos)); 

when  158 =>
--#line  582
 
yyval := Create_New_Expr_Node (
yy.value_stack(yy.tos-2), 
yy.value_stack(yy.tos)); 

when  159 =>
--#line  588
 
yyval := Create_Right_Stmt_Seq_Node (
yy.value_stack(yy.tos-2), 
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos)); 

when  160 =>
--#line  593
 
yyval := Create_Stmt_Seq_Node (
yy.value_stack(yy.tos-1),
yy.value_stack(yy.tos)); 

when  161 =>
--#line  595
 
yyval := Create_Stmt_Seq_Node (
yy.value_stack(yy.tos-1),
yy.value_stack(yy.tos)); 

when  162 =>
--#line  598
 
yyval := 
yy.value_stack(yy.tos); 

when  163 =>
--#line  600
 Check_Statement (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos)); 
yyval := 
yy.value_stack(yy.tos); 

when  164 =>
--#line  602
 Check_Statement (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos)); 
yyval := 
yy.value_stack(yy.tos); 

when  165 =>
--#line  604
 Check_Statement (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos)); 
yyval := 
yy.value_stack(yy.tos); 

when  166 =>
--#line  605
 Show_Error(3016, Fatal, "Goto"); 

when  167 =>
--#line  606
 Show_Error(3016, Fatal, "Delay"); 

when  168 =>
--#line  607
 Show_Error(3016, Fatal, "Abort"); 

when  169 =>
--#line  608
 Show_Error(3016, Fatal, "Raise"); 

when  170 =>
--#line  609
 Show_Error(3016, Fatal, "Code"); 

when  171 =>
--#line  611
 
yyval := Create_Procedure_Call_Node (
yy.value_stack(yy.tos-2));
	      Check_Statement (Ace.Main_Thread_Context, 
yyval); 

when  172 =>
--#line  614
 
yyval := Create_Null_Stmt_Node; 

when  173 =>
--#line  621
 
yyval := 
yy.value_stack(yy.tos); 

when  174 =>
--#line  622
 Check_Statement (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos)); 
yyval := 
yy.value_stack(yy.tos); 

when  175 =>
--#line  623
 Check_Statement (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos)); 
yyval := 
yy.value_stack(yy.tos); 

when  176 =>
--#line  624
 Check_Statement (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos)); 
yyval := 
yy.value_stack(yy.tos); 

when  177 =>
--#line  625
 Show_Error(3016, Fatal, "Accept"); 

when  178 =>
--#line  626
 Show_Error(3016, Fatal, "Select"); 

when  179 =>
--#line  630
 Show_Error(3016, Warning, "Label");
                                    
yyval := Create_Label_Node (
yy.value_stack(yy.tos-1)); 

when  180 =>
--#line  635
 
yyval := Create_Null_Stmt_Node; 

when  181 =>
--#line  641
 
yyval := Create_Assign_Node (
yy.value_stack(yy.tos-4), 
yy.value_stack(yy.tos-2)); 

when  182 =>
--#line  650
 Must_Be_Boolean_Type (
yy.value_stack(yy.tos-1)); 

when  183 =>
--#line  655
 
yyval := Create_If_Node (
yy.value_stack(yy.tos-9), 
yy.value_stack(yy.tos-6), 
yy.value_stack(yy.tos-5), 
yy.value_stack(yy.tos-4)); 

when  184 =>
--#line  658
 
yyval := 
yy.value_stack(yy.tos); 

when  185 =>
--#line  665
 Check_Expression (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos-1)); 

when  186 =>
--#line  668
 
yyval := Create_Case_Node (
yy.value_stack(yy.tos-7), 
yy.value_stack(yy.tos-4)); 

when  187 =>
--#line  675
 
yyval := Create_Case_Alt_Node (
yy.value_stack(yy.tos-3), 
yy.value_stack(yy.tos-2), 
yy.value_stack(yy.tos)); 

when  188 =>
--#line  687
 
yyval := Create_Loop_Node (
yy.value_stack(yy.tos-8), 
yy.value_stack(yy.tos-7), 
yy.value_stack(yy.tos-5)); 

when  189 =>
--#line  694
 Must_Be_Boolean_Type (
yy.value_stack(yy.tos));
	     
yyval := 
yy.value_stack(yy.tos); 

when  190 =>
--#line  696
 
yyval := 
yy.value_stack(yy.tos); 

when  191 =>
--#line  701
 
yyval := Create_For_Node (
yy.value_stack(yy.tos-3), 
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos)); 

when  192 =>
--#line  709
Start_of_Block (Ace.Main_Thread_Context);

when  193 =>
--#line  715
 
yyval := Create_Block_Node (
yy.value_stack(yy.tos-9), 
yy.value_stack(yy.tos-7), 
yy.value_stack(yy.tos-5), 
yy.value_stack(yy.tos-4));
              End_of_Block (Ace.Main_Thread_Context);
            

when  194 =>
--#line  725
 
yyval := Create_Exit_Node (
yy.value_stack(yy.tos-3), 
yy.value_stack(yy.tos-2)); 

when  195 =>
--#line  732
 
yyval := Create_Return_Node (
yy.value_stack(yy.tos-2)); 

when  196 =>
--#line  739
 
yyval := Create_Goto_Node (
yy.value_stack(yy.tos-2)); 

when  197 =>
--#line  747
 
yyval := Build_Subprogram_Spec_Decl (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos-2));
		 Check_Statement (Ace.Main_Thread_Context, 
yyval);                  

when  198 =>
--#line  754
 
yyval := Create_Subprogram_Spec (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos), null); 

when  199 =>
--#line  756
 
yyval := Create_Subprogram_Spec (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos-3), 
yy.value_stack(yy.tos-2), 
yy.value_stack(yy.tos)); 

when  200 =>
--#line  760
  
yyval := 
yy.value_stack(yy.tos); 

when  201 =>
--#line  761
  
yyval := 
yy.value_stack(yy.tos); 

when  202 =>
--#line  765
 
yyval := 
yy.value_stack(yy.tos); 

when  203 =>
--#line  770
 
yyval := Link_Formal_Param_On_Front (
yy.value_stack(yy.tos-2), 
yy.value_stack(yy.tos-1)); 

when  204 =>
--#line  775
 
yyval := Build_Formal_Parameter (
yy.value_stack(yy.tos-4), 
yy.value_stack(yy.tos-2), 
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos)); 

when  205 =>
--#line  779
 
yyval := 
yy.value_stack(yy.tos); 

when  206 =>
--#line  781
 
yyval := Build_Param_Mode (Ace.Main_Thread_Context,
                 parserdefs.IN_TK, parserdefs.OUT_TK); 

when  207 =>
--#line  784
 
yyval := Build_Param_Mode (Ace.Main_Thread_Context,
                 parserdefs.OUT_TK); 

when  208 =>
--#line  795
 Start_Of_Subprogram_Body (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos-1));  

when  209 =>
--#line  803

              Build_Subprogram_Body_Decl
                 (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos-10), 
yy.value_stack(yy.tos-7), 
yy.value_stack(yy.tos-5), 
yy.value_stack(yy.tos-4), 
yy.value_stack(yy.tos-1), 
yyval);
	      Check_Statement (Ace.Main_Thread_Context, 
yyval);
            

when  210 =>
--#line  836
 Check_Statement (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos-1));
			  
yyval := 
yy.value_stack(yy.tos-1);             

when  211 =>
--#line  845
 Start_Of_Package (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos-2), False, 
yyval); 

when  212 =>
--#line  849
 Create_Package_Spec_Decl (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos-5), 
yy.value_stack(yy.tos-4), 
yy.value_stack(yy.tos), 
yyval); 

when  213 =>
--#line  856
 Start_Of_Package (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos-1), True, 
yyval); 

when  214 =>
--#line  860
 Create_Package_Body_Decl
               (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos-6), 
yy.value_stack(yy.tos-5), 
yy.value_stack(yy.tos-4), 
yy.value_stack(yy.tos-1), 
yyval);
	    Check_Statement (Ace.Main_Thread_Context, 
yyval); 

when  215 =>
--#line  871
 
yyval := 
yy.value_stack(yy.tos-7); 

when  216 =>
--#line  873
 
yyval := 
yy.value_stack(yy.tos-8); 

when  217 =>
--#line  885
 declare
               Tmp : Yystype :=
                    Link_Id_Onto_Front_Of_List (
yy.value_stack(yy.tos-3), 
yy.value_stack(yy.tos-2));
             begin
              Check_Expression (Ace.Main_Thread_Context, Tmp);
             
yyval := Create_Use_Node (Ace.Main_Thread_Context, Tmp);
             end;
            

when  218 =>
--#line  900
 
yyval := 
yy.value_stack(yy.tos-7); 

when  219 =>
--#line  902
 
yyval := 
yy.value_stack(yy.tos-7); 

when  220 =>
--#line  904
 
yyval := 
yy.value_stack(yy.tos-6); 

when  221 =>
--#line  906
 
yyval := 
yy.value_stack(yy.tos-5); 

when  222 =>
--#line  916
 
yyval := 
yy.value_stack(yy.tos-1); 

when  223 =>
--#line  922
 
yyval := 
yy.value_stack(yy.tos-5); 

when  224 =>
--#line  931
 
yyval := 
yy.value_stack(yy.tos-12); 

when  225 =>
--#line  937
 
yyval := 
yy.value_stack(yy.tos-5); 

when  226 =>
--#line  939
 
yyval := 
yy.value_stack(yy.tos-8); 

when  227 =>
--#line  943
 
yyval := 
yy.value_stack(yy.tos-2); 

when  228 =>
--#line  948
 
yyval := 
yy.value_stack(yy.tos-6); 

when  229 =>
--#line  951
 
yyval := 
yy.value_stack(yy.tos); 

when  230 =>
--#line  956
 
yyval := 
yy.value_stack(yy.tos-4); 

when  231 =>
--#line  961
 
yyval := 
yy.value_stack(yy.tos); 

when  232 =>
--#line  962
 
yyval := 
yy.value_stack(yy.tos); 

when  233 =>
--#line  962
 
yyval := 
yy.value_stack(yy.tos); 

when  234 =>
--#line  972
 
yyval := 
yy.value_stack(yy.tos-8); 

when  235 =>
--#line  976
 
yyval := 
yy.value_stack(yy.tos); 

when  236 =>
--#line  979
 
yyval := 
yy.value_stack(yy.tos); 

when  237 =>
--#line  980
 
yyval := 
yy.value_stack(yy.tos); 

when  238 =>
--#line  980
 
yyval := 
yy.value_stack(yy.tos); 

when  239 =>
--#line  984
 
yyval := 
yy.value_stack(yy.tos); 

when  240 =>
--#line  988
 
yyval := 
yy.value_stack(yy.tos); 

when  241 =>
--#line  991
 
yyval := 
yy.value_stack(yy.tos); 

when  242 =>
--#line  1002
 
yyval := 
yy.value_stack(yy.tos-9); 

when  243 =>
--#line  1014
 
yyval := 
yy.value_stack(yy.tos-9); 

when  244 =>
--#line  1019
 
yyval := 
yy.value_stack(yy.tos-5); 

when  245 =>
--#line  1026
 
yyval := 
yy.value_stack(yy.tos); 

when  246 =>
--#line  1033
 
yyval := Create_Compilation_Node (
yy.value_stack(yy.tos-2),
yy.value_stack(yy.tos));
          Pop_Context_Clause(Ace.Main_Thread_Context); 

when  247 =>
--#line  1038
 
yyval := Create_Compilation_Node (
yy.value_stack(yy.tos-2),
yy.value_stack(yy.tos));
          Pop_Context_Clause(Ace.Main_Thread_Context);

when  248 =>
--#line  1043
 
yyval := 
yy.value_stack(yy.tos); 

when  249 =>
--#line  1044
 
yyval := 
yy.value_stack(yy.tos); 

when  250 =>
--#line  1045
 Show_Error(3016, Fatal, "Generics"); 

when  251 =>
--#line  1046
 Show_Error(3016, Fatal, "Generics"); 

when  252 =>
--#line  1047
 Show_Error(3016, Fatal, "Renaming"); 

when  253 =>
--#line  1048
 
yyval := 
yy.value_stack(yy.tos); 

when  254 =>
--#line  1053
 
yyval := 
yy.value_stack(yy.tos); 

when  255 =>
--#line  1053
 
yyval := 
yy.value_stack(yy.tos); 

when  256 =>
--#line  1057
 
yyval := 
yy.value_stack(yy.tos); 

when  257 =>
--#line  1065
 
yyval := 
yy.value_stack(yy.tos); Push_Context_Clause(Ace.Main_Thread_Context); Check_Statement(Ace.Main_Thread_Context, 
yyval); 

when  258 =>
--#line  1070
 
yyval := Create_With_Node (Link_Id_Onto_Front_Of_List(
yy.value_stack(yy.tos-2),
yy.value_stack(yy.tos-1))); 

when  259 =>
--#line  1077
 
yyval := 
yy.value_stack(yy.tos-5); 

when  260 =>
--#line  1079
 
yyval := 
yy.value_stack(yy.tos-7); 

when  261 =>
--#line  1081
 
yyval := 
yy.value_stack(yy.tos-7); 

when  262 =>
--#line  1085
 
yyval := 
yy.value_stack(yy.tos-4); 

when  263 =>
--#line  1092
  Create_Exception_Declaration(Ace.Main_Thread_Context, 
yy.value_stack(yy.tos-4), 
yyval); 

when  264 =>
--#line  1101
 
yyval := Create_Exception_Alt_Node (
yy.value_stack(yy.tos-3), 
yy.value_stack(yy.tos-2), 
yy.value_stack(yy.tos)); 

when  265 =>
--#line  1106
 
yyval := 
yy.value_stack(yy.tos);

when  266 =>
--#line  1107
 
yyval := 
yy.value_stack(yy.tos); 

when  267 =>
--#line  1112
 
yyval := 
yy.value_stack(yy.tos-2); 

when  268 =>
--#line  1118
 
yyval := 
yy.value_stack(yy.tos-3); 

when  269 =>
--#line  1121
 
yyval := 
yy.value_stack(yy.tos-1); 

when  270 =>
--#line  1122
 
yyval := 
yy.value_stack(yy.tos-1); 

when  271 =>
--#line  1125
 
yyval := 
yy.value_stack(yy.tos-1); 

when  272 =>
--#line  1129
 
yyval := 
yy.value_stack(yy.tos-5); 

when  273 =>
--#line  1130
 
yyval := 
yy.value_stack(yy.tos-6); 

when  274 =>
--#line  1131
 
yyval := 
yy.value_stack(yy.tos); 

when  276 =>
--#line  1139
 
yyval := 
yy.value_stack(yy.tos-2); 

when  277 =>
--#line  1140
 
yyval := 
yy.value_stack(yy.tos-1); 

when  278 =>
--#line  1141
 
yyval := 
yy.value_stack(yy.tos-1); 

when  279 =>
--#line  1142
 
yyval := 
yy.value_stack(yy.tos-1); 

when  280 =>
--#line  1143
 
yyval := 
yy.value_stack(yy.tos); 

when  281 =>
--#line  1144
 
yyval := 
yy.value_stack(yy.tos); 

when  282 =>
--#line  1153
 
yyval := 
yy.value_stack(yy.tos-9); 

when  283 =>
--#line  1155
 
yyval := 
yy.value_stack(yy.tos-6); 

when  284 =>
--#line  1157
 
yyval := 
yy.value_stack(yy.tos-8); 

when  285 =>
--#line  1161
 
yyval := 
yy.value_stack(yy.tos-3); 

when  286 =>
--#line  1165
 
yyval := 
yy.value_stack(yy.tos); 

when  287 =>
--#line  1169
 
yyval := 
yy.value_stack(yy.tos); 

when  288 =>
--#line  1170
 
yyval := 
yy.value_stack(yy.tos); 

when  289 =>
--#line  1173
 
yyval := 
yy.value_stack(yy.tos); 

when  290 =>
--#line  1180
 
yyval := 
yy.value_stack(yy.tos); 

when  291 =>
--#line  1181
 
yyval := 
yy.value_stack(yy.tos); 

when  292 =>
--#line  1183
 
yyval := 
yy.value_stack(yy.tos); 

when  293 =>
--#line  1184
 
yyval := 
yy.value_stack(yy.tos); 

when  294 =>
--#line  1185
 
yyval := 
yy.value_stack(yy.tos); 

when  295 =>
--#line  1191

yyval := 
yy.value_stack(yy.tos-6); 

when  296 =>
--#line  1198
 
yyval := 
yy.value_stack(yy.tos-4); 

when  297 =>
--#line  1208
 
yyval := 
yy.value_stack(yy.tos-8); 

when  298 =>
--#line  1211
 
yyval := 
yy.value_stack(yy.tos-5); 

when  299 =>
--#line  1216
 
yyval := 
yy.value_stack(yy.tos-7); 

when  300 =>
--#line  1223
 
yyval := 
yy.value_stack(yy.tos-7); 

when  301 =>
--#line  1228
 
yyval := 
yy.value_stack(yy.tos-3); 

when  302 =>
--#line  1252
 
yyval := null; 

when  303 =>
--#line  1254
 
yyval := Create_Left_Stmt_Seq_Node (
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos), null); 

when  304 =>
--#line  1259
 
yyval := null; 

when  305 =>
--#line  1261
 
yyval := 
yy.value_stack(yy.tos-1); 

when  306 =>
--#line  1266
 
yyval := Link_Id_Onto_Rear_Of_List (
yy.value_stack(yy.tos), null); 

when  307 =>
--#line  1268
 
yyval := Link_Id_Onto_Rear_Of_List (
yy.value_stack(yy.tos), 
yy.value_stack(yy.tos-2)); 

when  308 =>
--#line  1281
 
yyval := null; 

when  309 =>
--#line  1283
 Check_Expression (Ace.Main_Thread_Context, 
yy.value_stack(yy.tos));
       
yyval := 
yy.value_stack(yy.tos); 

when  310 =>
--#line  1287
 
yyval := null; 

when  311 =>
--#line  1289
 
yyval := Link_Id_Onto_Rear_Of_List (
yy.value_stack(yy.tos), 
yy.value_stack(yy.tos-2)); 

when  312 =>
--#line  1294
 
yyval := null; 

when  313 =>
--#line  1296
 
yyval := 
yy.value_stack(yy.tos); 

when  314 =>
--#line  1304
 
yyval :=  
yy.value_stack(yy.tos); 

when  315 =>
--#line  1306
 
yyval := Create_Dot_Expr_Node (
yy.value_stack(yy.tos-2), 
yy.value_stack(yy.tos)); 

when  316 =>
--#line  1315
 
yyval := null; 

when  317 =>
--#line  1317
 
yyval := Link_Enum_Literal_On_End (
yy.value_stack(yy.tos-2), 
yy.value_stack(yy.tos)); 

when  318 =>
--#line  1322
 
yyval := null; 

when  319 =>
--#line  1323
 
yyval := 
yy.value_stack(yy.tos); 

when  320 =>
--#line  1327
 
yyval := null; 

when  321 =>
--#line  1328
 
yyval := 
yy.value_stack(yy.tos-2); 

when  322 =>
--#line  1335
 
yyval := null; 

when  323 =>
--#line  1336
 
yyval := Link_Id_Onto_Rear_Of_List
					  (
yy.value_stack(yy.tos), 
yy.value_stack(yy.tos-2)); 

when  324 =>
--#line  1345
 
yyval := null; 

when  325 =>
--#line  1347
 
yyval := Link_Record_Component_Entries (
yy.value_stack(yy.tos-2), 
yy.value_stack(yy.tos-1)); 

when  326 =>
--#line  1352
 
yyval := null; 

when  327 =>
--#line  1353
 
yyval := 
yy.value_stack(yy.tos-2); 

when  328 =>
--#line  1361
 
yyval := null; 

when  329 =>
--#line  1362
 
yyval := 
yy.value_stack(yy.tos-1); 

when  330 =>
--#line  1365
 
yyval := null; 

when  331 =>
--#line  1367
 
yyval := Link_Id_Onto_Rear_Of_List (
yy.value_stack(yy.tos), 
yy.value_stack(yy.tos-2)); 

when  332 =>
--#line  1379
 
yyval := 
yy.value_stack(yy.tos); 

when  333 =>
--#line  1381
 
yyval := Create_Left_Stmt_Seq_Node (
yy.value_stack(yy.tos-2), 
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos)); 

when  334 =>
--#line  1386
 
yyval := 
yy.value_stack(yy.tos); 

when  335 =>
--#line  1388
 
yyval := Create_Left_Stmt_Seq_Node (
yy.value_stack(yy.tos-2), 
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos)); 

when  336 =>
--#line  1413
 
yyval := null; 

when  337 =>
--#line  1415
 
yyval := Link_Id_Onto_Rear_Of_List (
yy.value_stack(yy.tos), 
yy.value_stack(yy.tos-2)); 

when  338 =>
--#line  1424
 
yyval := Create_Expr_Node (
yy.value_stack(yy.tos-2), parserdefs.AND_TK, 
yy.value_stack(yy.tos)); 

when  339 =>
--#line  1426
 
yyval := Create_Expr_Node (
yy.value_stack(yy.tos-2), parserdefs.AND_TK, 
yy.value_stack(yy.tos)); 

when  340 =>
--#line  1430
 
yyval := Create_Expr_Node (
yy.value_stack(yy.tos-2), parserdefs.OR_TK, 
yy.value_stack(yy.tos)); 

when  341 =>
--#line  1432
 
yyval := Create_Expr_Node (
yy.value_stack(yy.tos-2), parserdefs.OR_TK, 
yy.value_stack(yy.tos)); 

when  342 =>
--#line  1436
 
yyval := 
yy.value_stack(yy.tos); 

when  343 =>
--#line  1437
 
yyval := 
yy.value_stack(yy.tos); 

when  344 =>
--#line  1441
 
yyval := Create_Expr_Node (
yy.value_stack(yy.tos-2), parserdefs.XOR_TK, 
yy.value_stack(yy.tos)); 

when  345 =>
--#line  1443
 
yyval := Create_Expr_Node (
yy.value_stack(yy.tos-2), parserdefs.XOR_TK, 
yy.value_stack(yy.tos)); 

when  346 =>
--#line  1448
 
yyval := Create_Expr_Node (
yy.value_stack(yy.tos-3), parserdefs.AND_TK, parserdefs.THEN_TK, 
yy.value_stack(yy.tos-1)); 

when  347 =>
--#line  1450
 
yyval := Create_Expr_Node (
yy.value_stack(yy.tos-3), parserdefs.AND_TK, parserdefs.THEN_TK, 
yy.value_stack(yy.tos-1)); 

when  348 =>
--#line  1455
 
yyval := Create_Expr_Node (
yy.value_stack(yy.tos-3), parserdefs.OR_TK, parserdefs.ELSE_TK, 
yy.value_stack(yy.tos-1)); 

when  349 =>
--#line  1457
 
yyval := Create_Expr_Node (
yy.value_stack(yy.tos-3), parserdefs.OR_TK, parserdefs.ELSE_TK, 
yy.value_stack(yy.tos-1)); 

when  350 =>
--#line  1462
 
yyval := null; 

when  351 =>
--#line  1464
 
yyval := Create_Right_Expr_Node (
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos)); 

when  352 =>
--#line  1469
 
yyval := Create_Expr_Node (
yy.value_stack(yy.tos-3), 
yy.value_stack(yy.tos-2), parserdefs.IN_TK, 
yy.value_stack(yy.tos)); 

when  353 =>
--#line  1474
 
yyval := null; 

when  354 =>
--#line  1476
 
yyval := Create_Expr_Node (parserdefs.NOT_TK); 

when  355 =>
--#line  1480
 
yyval := 
yy.value_stack(yy.tos); 

when  356 =>
--#line  1481
 
yyval := Create_Unary_Add_Expr_Node (
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos)); 

when  357 =>
--#line  1483
 
yyval := Create_Expr_Node (
yy.value_stack(yy.tos-2), 
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos)); 

when  358 =>
--#line  1487
 
yyval := 
yy.value_stack(yy.tos); 

when  359 =>
--#line  1489
 
yyval := Create_Expr_Node (
yy.value_stack(yy.tos-2), 
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos)); 

when  360 =>
--#line  1494
 
yyval := null; 

when  361 =>
--#line  1496
 
yyval := Create_Expr_Node (parserdefs.EXP_TK, 
yy.value_stack(yy.tos)); 

when  362 =>
--#line  1501
 
yyval := Create_Expr_Node (
yy.value_stack(yy.tos-2), character'pos('''), 
yy.value_stack(yy.tos)); 

when  363 =>
--#line  1508
 
yyval := 
yy.value_stack(yy.tos); 

when  364 =>
--#line  1509
 
yyval := Create_Left_Stmt_Seq_Node (
yy.value_stack(yy.tos-2), 
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos)); 

when  365 =>
--#line  1515
 
yyval := null; 

when  366 =>
--#line  1516
 
yyval := 
yy.value_stack(yy.tos-1); 

when  367 =>
--#line  1521
 
yyval := null; 

when  368 =>
--#line  1524
 Must_Be_Boolean_Type (
yy.value_stack(yy.tos-1)); 

when  369 =>
--#line  1526
 
yyval := Create_If_Node (
yy.value_stack(yy.tos-3), 
yy.value_stack(yy.tos), 
yy.value_stack(yy.tos-5)); 

when  370 =>
--#line  1530
 
yyval := null; 

when  371 =>
--#line  1532
 
yyval := 
yy.value_stack(yy.tos); 

when  372 =>
--#line  1538
 
yyval := Create_Left_Case_Alt_Node (
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos)); 

when  373 =>
--#line  1543
 
yyval := null; 

when  374 =>
--#line  1545
 
yyval := Create_Right_Case_Alt_Node (
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos)); 

when  375 =>
--#line  1550
 
yyval := null; 

when  376 =>
--#line  1551
 
yyval := 
yy.value_stack(yy.tos-1); 

when  377 =>
--#line  1555
 
yyval := null; 

when  378 =>
--#line  1556
 
yyval := 
yy.value_stack(yy.tos); 

when  379 =>
--#line  1560
 
yyval := null; 

when  380 =>
--#line  1561
 
yyval := 
yy.value_stack(yy.tos); 

when  381 =>
--#line  1565
 
yyval := null; 

when  382 =>
--#line  1566
 
yyval := 
yy.value_stack(yy.tos); 

when  383 =>
--#line  1570
 
yyval := null; 

when  384 =>
--#line  1572
 
yyval := 
yy.value_stack(yy.tos); 

when  385 =>
--#line  1577
 
yyval := null; 

when  386 =>
--#line  1580
 
yyval := 
yy.value_stack(yy.tos); 

when  387 =>
--#line  1585
 
yyval := 
yy.value_stack(yy.tos); 

when  388 =>
--#line  1587
 
yyval := Create_Right_Exception_Alt_Node (
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos)); 

when  389 =>
--#line  1591
 
yyval := null; 

when  390 =>
--#line  1592
 
yyval := 
yy.value_stack(yy.tos); 

when  391 =>
--#line  1597
 
yyval := null; 

when  392 =>
--#line  1598
 Must_Be_Boolean_Type (
yy.value_stack(yy.tos)); 
yyval := 
yy.value_stack(yy.tos); 

when  393 =>
--#line  1603
 
yyval := null; 

when  394 =>
--#line  1604
 
yyval := 
yy.value_stack(yy.tos); 

when  395 =>
--#line  1612
 
yyval := null; 

when  396 =>
--#line  1613
 
yyval := 
yy.value_stack(yy.tos); 

when  397 =>
--#line  1618
 
yyval := null; 

when  398 =>
--#line  1620
 
yyval := Link_Formal_Param_On_Rear (
yy.value_stack(yy.tos), 
yy.value_stack(yy.tos-2)); 

when  399 =>
--#line  1625
 
yyval := Build_Param_Mode(Ace.Main_Thread_Context); 

when  400 =>
--#line  1627
 
yyval := Build_Param_Mode (Ace.Main_Thread_Context, parserdefs.IN_TK); 

when  401 =>
--#line  1630
 
yyval := 
yy.value_stack(yy.tos); 

when  402 =>
--#line  1636
 
yyval := null; 

when  403 =>
--#line  1637
 
yyval := 
yy.value_stack(yy.tos); 

when  404 =>
--#line  1645
 
yyval := null; 

when  405 =>
--#line  1647
 
yyval := 
yy.value_stack(yy.tos-1); 

when  406 =>
--#line  1652
 
yyval := null; 

when  407 =>
--#line  1656
 
yyval := Create_List_Item_Node (
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos)); 

when  408 =>
--#line  1661
 
yyval := null; 

when  409 =>
--#line  1662
 
yyval := 
yy.value_stack(yy.tos); 

when  410 =>
--#line  1666
 
yyval := null; 

when  411 =>
--#line  1668
 
yyval := Link_Id_Onto_Rear_Of_List (
yy.value_stack(yy.tos), 
yy.value_stack(yy.tos-2)); 

when  412 =>
--#line  1676
 
yyval := null; 

when  413 =>
--#line  1678
 
yyval := Create_Expr_Node (parserdefs.TYPE_TK); 

when  415 =>
--#line  1686
 
yyval := 
yy.value_stack(yy.tos-4); 

when  416 =>
--#line  1692
 
yyval := 
yy.value_stack(yy.tos); 

when  417 =>
--#line  1694
 
yyval := Create_Left_Stmt_Seq_Node (
yy.value_stack(yy.tos-2), 
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos)); 

when  418 =>
--#line  1699
 
yyval := null; 

when  419 =>
--#line  1701
 
yyval := Create_Left_Stmt_Seq_Node (
yy.value_stack(yy.tos-2), 
yy.value_stack(yy.tos-1), 
yy.value_stack(yy.tos)); 

when  420 =>
--#line  1706
 
yyval := 
yy.value_stack(yy.tos); 

when  421 =>
--#line  1707
 
yyval := 
yy.value_stack(yy.tos-3); 

when  422 =>
--#line  1711
 
yyval := null; 

when  423 =>
--#line  1714
 
yyval := 
yy.value_stack(yy.tos-3); 

when  424 =>
--#line  1718
 
yyval := null; 

when  425 =>
--#line  1719
 
yyval := 
yy.value_stack(yy.tos-2); 

when  426 =>
--#line  1722
 
yyval := 
yy.value_stack(yy.tos); 

when  427 =>
--#line  1724
 
yyval := 
yy.value_stack(yy.tos-3); 

when  428 =>
--#line  1727
 
yyval := 
yy.value_stack(yy.tos-2); 

when  429 =>
--#line  1730
 
yyval := 
yy.value_stack(yy.tos-2); 

when  430 =>
--#line  1732
 
yyval := 
yy.value_stack(yy.tos-3); 

when  431 =>
--#line  1735
 
yyval := 
yy.value_stack(yy.tos); 

when  432 =>
--#line  1736
 
yyval := 
yy.value_stack(yy.tos); 

when  433 =>
--#line  1740
 
yyval := null; 

when  434 =>
--#line  1741
 
yyval := 
yy.value_stack(yy.tos-2); 

when  435 =>
--#line  1748
 
yyval := 
yy.value_stack(yy.tos); 

when  436 =>
--#line  1750
 
yyval := 
yy.value_stack(yy.tos-1); 

when  437 =>
--#line  1753
 
yyval := 
yy.value_stack(yy.tos); 

when  438 =>
--#line  1760
 
yyval := null; 

when  439 =>
--#line  1762
 
yyval := Create_Left_Stmt_Seq_Node(
yy.value_stack(yy.tos-2),
yy.value_stack(yy.tos-1),
yy.value_stack(yy.tos)); 

when  440 =>
--#line  1767
 
yyval := 
yy.value_stack(yy.tos); 

when  441 =>
--#line  1769
 
yyval := Create_Left_Stmt_Seq_Node(
yy.value_stack(yy.tos-2),
yy.value_stack(yy.tos-1),
yy.value_stack(yy.tos)); 

when  442 =>
--#line  1774
 
yyval := null; 

when  443 =>
--#line  1776
 
yyval := Link_Id_Onto_Rear_Of_List (
yy.value_stack(yy.tos), 
yy.value_stack(yy.tos-2)); 

when  444 =>
--#line  1785
 
yyval := null; 

when  445 =>
--#line  1786
 
yyval := 
yy.value_stack(yy.tos-2); 

when  446 =>
--#line  1794
 
yyval := null; 

when  447 =>
--#line  1795
 
yyval := 
yy.value_stack(yy.tos-1); 

when  448 =>
--#line  1798
 
yyval := 
yy.value_stack(yy.tos); 

when  449 =>
--#line  1799
 
yyval := 
yy.value_stack(yy.tos-1); 

when  450 =>
--#line  1803
 
yyval := null; 

when  451 =>
--#line  1804
 
yyval := 
yy.value_stack(yy.tos-1); 

when  452 =>
--#line  1805
 
yyval := 
yy.value_stack(yy.tos-1); 

when  453 =>
--#line  1807
 
yyval := 
yy.value_stack(yy.tos-1); 

when  454 =>
--#line  1813
 
yyval := null; 

when  455 =>
--#line  1814
 
yyval := 
yy.value_stack(yy.tos); 

when  456 =>
--#line  1818
 
yyval := null; 

when  457 =>
--#line  1819
 
yyval := 
yy.value_stack(yy.tos-2); 

when  458 =>
--#line  1822
 
yyval := 
yy.value_stack(yy.tos); 

when  459 =>
--#line  1823
 
yyval := 
yy.value_stack(yy.tos-2); 

when  460 =>
--#line  1826
 
yyval := 
yy.value_stack(yy.tos); 

when  461 =>
--#line  1834
 
yyval := 
yy.value_stack(yy.tos-2); 

when  462 =>
--#line  1839
 
yyval := 
yy.value_stack(yy.tos); 

when  463 =>
--#line  1840
 
yyval := 
yy.value_stack(yy.tos-2); 

when  464 =>
--#line  1844
 
yyval := null; 

when  465 =>
--#line  1845
 
yyval := 
yy.value_stack(yy.tos-2); 

when  466 =>
--#line  1847
 
yyval := 
yy.value_stack(yy.tos); 

                when others => null;
            end case;

            -- Pop RHS states and goto next state
            yy.tos      := yy.tos - rule_length(yy.rule_id) + 1;
            yy.state_stack(yy.tos) := goto_state(yy.state_stack(yy.tos-1) ,
                                 get_lhs_rule(yy.rule_id));

            yy.value_stack(yy.tos) := yyval;

            if yy.debug then
                reduce_debug(yy.rule_id,
                    goto_state(yy.state_stack(yy.tos - 1),
                               get_lhs_rule(yy.rule_id)));
            end if;

        end if;
    end loop;

end yyparse;

end compilation_unit_parser;
