--|$Log:	csmisc.a,v $
--|Revision 8.0  91/01/17  10:44:18  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Subunit for Package Name - Semantic_Analyzer
--
---------------------------------------------------------------
--
--
-- Abstract
--
--  Contains Check_Infix_Operations (a separate unit).
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------

--==========================================================================
--==========================================================================
separate (Semantic_Analyzer.Check_Subprogram_Call)

function Create_Possible_List_Node return Ref_Possible_List_Node is
begin
   return (new Possible_List_Node'(Subpgm_Entry      => null,
                                   Next_Subpgm_Entry => null));
end Create_Possible_List_Node;

--==========================================================================
--==========================================================================
separate (Semantic_Analyzer.Check_Subprogram_Call)

procedure Save_Found_Entry (Node        : in out Ref_Possible_List_Node;
                            Found_Entry : in Ref_Symbol_Table_Entry) is
begin
   Node.Subpgm_Entry      := Found_Entry;
   Node.Next_Subpgm_Entry := Create_Possible_List_Node;
   Node                   := Node.Next_Subpgm_Entry;
end Save_Found_Entry;

--==========================================================================
--==========================================================================
separate (Semantic_Analyzer.Check_Subprogram_Call)

-- --------------------
-- check parameter modes
-- and
-- adjust for argument list for default parameter init code as/if necessary
--
procedure Check_Parameters (Symtbl_Entry : in Ref_Symbol_Table_Entry) is
   Current_Parameter        : Ref_Symbol_Table_Entry :=
      Get_Head_Of_Parameters_List (Symtbl_Entry);
   Trailing_Actual_Argument : Ref_Statement_Database := Node;
begin
   Actual_Argument := Get_Parameter_List (Node);
   while (not Is_Empty (Actual_Argument)) loop
      -----
      if Is_Mode_In (Current_Parameter) then
         -- must be able to read from actual argument
         if Is_Mode_Out (Actual_Argument) then
            Show_Error (1114, Fatal,
                        "an argument supplied to subprogram " &
                        Convert_To_String (Subprogram_Name) &
                        " must be of mode IN");
         end if;
      elsif Is_Mode_Out (Current_Parameter) then
         -- must be able to write to actual argument
         if Is_Mode_In (Actual_Argument) then
            Show_Error (1114, Fatal,
                        "an argument supplied to subprogram " &
                        Convert_To_String (Subprogram_Name) &
                        " must be of mode OUT");
         end if;
      elsif Is_Mode_In_Out (Current_Parameter) then
         -- must be able to read/write from/to actual argument
         if (Is_Mode_Out (Actual_Argument) xor Is_Mode_In (Actual_Argument))
             then
            Show_Error (1114, Fatal,
                        "an argument supplied to subprogram " &
                        Convert_To_String (Subprogram_Name) &
                        " must be of mode IN OUT");
         end if;
      else
         Show_Error (0, Internal, "can never be this");
      end if;
      -----
      Current_Parameter := Get_Next_Parameter (Current_Parameter);
      Trailing_Actual_Argument := Actual_Argument;
      Actual_Argument := Get_Next_List_Item (Actual_Argument);
   end loop;

   -----
   -- adjust argument list for default parameter init code as/if necessary
   --
   if ((not Is_Empty (Current_Parameter)) and
      Has_Default_Parameters (Symtbl_Entry)) then
      -- add default parameter(s) init expr code to subprogram call arg list
      while (not Is_Empty (Current_Parameter)) loop
         -- if there are no arguments in list, do differently
         if (Trailing_Actual_Argument = Node) then
            Set_Parameter_List
                (Node,
                 Create_Statement_Database_Item
                     (Create_Expression_Item
                          (List_Item,
                           Get_Initialization_Code (Current_Parameter))));
            Trailing_Actual_Argument := Get_Parameter_List (Node);
         else
            -- there _are_ arguments in list
            -- last arg is last arg
            Set_Next_List_Item
                (Trailing_Actual_Argument,
                 Create_Statement_Database_Item
                     (Create_Expression_Item
                          (List_Item,
                           Get_Initialization_Code (Current_Parameter))));
            Trailing_Actual_Argument :=
               Get_Next_List_Item (Trailing_Actual_Argument);
         end if;
         -- increment ref_count
         Increment_Reference_Count
             (Get_Initialization_Code (Current_Parameter));
         Current_Parameter := Get_Next_Parameter (Current_Parameter);
      end loop;
   end if;
   -----

end Check_Parameters;

--==========================================================================
--==========================================================================
separate (Semantic_Analyzer.Check_Subprogram_Call)

procedure Mark_As_Proper_Entry (Symtbl_Entry : in Ref_Symbol_Table_Entry) is
begin
   -----
   -- check mode of formal parameters w/ actual arguments
   if not Is_Empty (Symtbl_Entry) then
      Check_Parameters (Symtbl_Entry);
   end if;
   -----

   -----
   -- attach proper symbol table entry
   Set_Symbol_Table_Entry (Get_Subprogram_Info (Node), Symtbl_Entry);
   -----

   -----
   -- [for function calls only]
   -- set return type for node!
   if Is_Function_Call then
      if Is_Empty (Symtbl_Entry) then
         Set_Type (Node, null);
      else
         Set_Type (Node, Get_Return_Type (Symtbl_Entry));
      end if;
   end if;
   -----

   -----
   -- check if call is actually a use of a predefined infix operation
   if (not Is_Empty (Symtbl_Entry)) and then
      (Get_Built_In_Instance_Number (Symtbl_Entry) = Ace_Natural'FIRST) then
      Check_Infix_Operation (Node);
   end if;
   -----
end Mark_As_Proper_Entry;

--==========================================================================
--==========================================================================
separate (Semantic_Analyzer.Check_Subprogram_Call)

function Is_One_An_Infix_Operation (List_Of_Entries : in Ref_Possible_List_Node
                                    ) return Boolean is
   Current : Ref_Possible_List_Node := List_Of_Entries;
begin
   while Current.Subpgm_Entry /= null loop
      if (Get_Built_In_Instance_Number (Current.Subpgm_Entry) =
         Ace_Natural'FIRST) then
         return True;
      end if;
      Current := Current.Next_Subpgm_Entry;
   end loop;
   return False;
end Is_One_An_Infix_Operation;
