--|$Log:	ctree.a,v $
--|Revision 8.0  91/01/17  10:44:20  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Create_Statement_Database
--
---------------------------------------------------------------
--
--
-- Abstract
--
--   This package contains specific create routines for the
--   statement database.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
with Lexacts;

with Ace_Universal_Types;
use Ace_Universal_Types;

with Symbol_Table_Structures;
use Symbol_Table_Structures;

with Symbol_Table_Routines;
use Symbol_Table_Routines;

with Symtbl_Entry_Set_Routines;
use Symtbl_Entry_Set_Routines;

with Node_Create_Routines;
use Node_Create_Routines;

with Node_Set_Routines;
use Node_Set_Routines;

with Node_Get_Routines;
use Node_Get_Routines;

with Error_Handler;
use Error_Handler;

with Literals;
use Literals;

with Misc_Support_Routines;
use Misc_Support_Routines;

with Declaration_Routines;
use Declaration_Routines;

with String_Constants;

with Parserdefs;
package body Create_Statement_Database is

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Record_Statement
   --
   --  <Description>
   --
   --    The function is used to keep an array of all statements (a record
   --    of stmt database node).  Used by debugger "display (stmt)".
   --
   --==========================================================================
   function Record_Statement
       (Stmt : in Ref_Statement_Database) return Ref_Statement_Database is
   begin
      Stmt_List (Get_Statement_Number (Stmt)) := Stmt;
      return Stmt;
   end Record_Statement;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Pragma_Node
   --
   --  <Description>
   --
   --    Create a statement database node for pragmas.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Pragma_Node
       (Ident, List : Ref_Statement_Database) return Ref_Statement_Database is
   begin
      return
         Record_Statement
             (Create_Statement_Database_Item
                  (Create_Statement_Item (Kind  => Pragma_Statement,
                                          Item1 => Ident,
                                          Item2 => List)));
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Pragma_Node");
         raise Tree_Builder_Error;
   end Create_Pragma_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_With_Node
   --
   --  <Description>
   --
   --    Create a statement database node for with.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_With_Node
       (List : Ref_Statement_Database) return Ref_Statement_Database is
   begin
      return
         Record_Statement
             (Create_Statement_Database_Item
                  (Create_Statement_Item (Kind  => With_Statement,
                                          Item1 => List)));
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_With_Node");
         raise Tree_Builder_Error;
   end Create_With_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Case_Node
   --
   --  <Description>
   --
   --    Create a statement database node for case.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Case_Node
       (Expr, Altern : Ref_Statement_Database) return Ref_Statement_Database is
   begin
      return
         Record_Statement
             (Create_Statement_Database_Item
                  (Create_Statement_Item (Kind  => Case_Statement,
                                          Item1 => Expr,
                                          Item2 => Altern)));
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Case_Node");
         raise Tree_Builder_Error;
   end Create_Case_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Case_Alt_Node
   --
   --  <Description>
   --
   --    Create a statement database node for case alternatives.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Case_Alt_Node
       (Choice, Seq_Choice, Seq_Stmt : Ref_Statement_Database) return
         Ref_Statement_Database is
   begin
      return
         Record_Statement
             (Create_Statement_Database_Item
                  (Create_Statement_Item
                       (Kind  => Case_Alternative_Part,
                        Item1 =>
                           Create_Statement_Database_Item
                               (Create_Expression_Item (Kind  => List_Item,
                                                        Item1 => Choice,
                                                        Item2 => Seq_Choice)),
                        Item2 => Seq_Stmt)));
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Case_Alt_Node");
         raise Tree_Builder_Error;
   end Create_Case_Alt_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Exception_Alt_Node
   --
   --  <Description>
   --
   --    Create a statement database node for case alternatives.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Exception_Alt_Node
       (Choice, Seq_Choice, Seq_Stmt : Ref_Statement_Database) return
         Ref_Statement_Database is
   begin
      return
         Record_Statement
             (Create_Statement_Database_Item
                  (Create_Statement_Item
                       (Kind  => Exception_Alternative,
                        Item1 =>
                           Create_Statement_Database_Item
                               (Create_Expression_Item (Kind  => List_Item,
                                                        Item1 => Choice,
                                                        Item2 => Seq_Choice)),
                        Item2 => Seq_Stmt)));
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Exception_Alt_Node");
         raise Tree_Builder_Error;
   end Create_Exception_Alt_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Left_Exception_Alt_Node
   --
   --  <Description>
   --
   --    Create a statement database node for exception alternatives.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Left_Exception_Alt_Node
       (Altern, Seq_Altern : Ref_Statement_Database) return
         Ref_Statement_Database is
   begin
      Set_Next_Alternative (Altern, Seq_Altern);
      return Altern;
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Left_Exception_Alt_Node");
         raise Tree_Builder_Error;
   end Create_Left_Exception_Alt_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Left_Case_Alt_Node
   --
   --  <Description>
   --
   --    Create a statement database node for case alternatives.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Left_Case_Alt_Node
       (Altern, Seq_Altern : Ref_Statement_Database) return
         Ref_Statement_Database is
   begin
      Set_Next_Alternative (Altern, Seq_Altern);
      return Altern;
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Left_Case_Alt_Node");
         raise Tree_Builder_Error;
   end Create_Left_Case_Alt_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Right_Case_Alt_Node
   --
   --  <Description>
   --
   --    Create a statement database node for case alternatives.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Right_Case_Alt_Node
       (Seq_Altern, Altern : Ref_Statement_Database) return
         Ref_Statement_Database is
   begin
      if Is_Empty (Seq_Altern) then
         -- first one.
         Set_Holder (Altern, Altern);
         return Altern;
      else
         -- in the middle of the sequence.
         Set_Next_Alternative (Get_Holder (Seq_Altern), Altern);
         Set_Holder (Seq_Altern, Altern);
         return Seq_Altern;
      end if;
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Right_Case_Alt_Node");
         raise Tree_Builder_Error;
   end Create_Right_Case_Alt_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Right_Exception_Alt_Node
   --
   --  <Description>
   --
   --    Create a statement database node for exception alternatives.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Right_Exception_Alt_Node
       (Seq_Altern, Altern : Ref_Statement_Database) return
         Ref_Statement_Database is
   begin
      if Is_Empty (Seq_Altern) then
         -- first one.
         Set_Holder (Altern, Altern);
         return Altern;
      else
         -- in the middle of the sequence.
         Set_Next_Alternative (Get_Holder (Seq_Altern), Altern);
         Set_Holder (Seq_Altern, Altern);
         return Seq_Altern;
      end if;
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Right_Exception_Alt_Node");
         raise Tree_Builder_Error;
   end Create_Right_Exception_Alt_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Block_Node
   --
   --  <Description>
   --
   --    Create a statement database node for blocks.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Block_Node
       (Name, Declarations, Statements, Exceptions : Ref_Statement_Database)
       return Ref_Statement_Database is
   begin
      return
         Record_Statement
             (Create_Statement_Database_Item
                  (Create_Statement_Item (Kind  => Block_Statement,
                                          Item1 => Declarations,
                                          Item2 => Statements,
                                          Item3 => Name,
                                          Item4 => Exceptions)));
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Block_Node");
         raise Tree_Builder_Error;
   end Create_Block_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Procedure_Call_Node
   --
   --  <Description>
   --
   --    Create a statement database node for procedure calls.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Procedure_Call_Node
       (Name : Ref_Statement_Database) return Ref_Statement_Database is
   begin
      case Check_Expr_Node (Name) is
         when Terminal_Node    | Dot_Operator =>
            return
               Record_Statement
                   (Create_Statement_Database_Item
                        (Create_Statement_Item (Kind  => Procedure_Call,
                                                Item1 => Name)));
         when others =>
            return
               Record_Statement
                   (Create_Statement_Database_Item
                        (Create_Statement_Item
                             (Kind => Procedure_Call,
                              Item1 => Get_Object (Name),
                              Item2 => Get_List_Of_Items (Name))));
      end case;
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Procedure_Call_Node");
         raise Tree_Builder_Error;
   end Create_Procedure_Call_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Label_Node
   --
   --  <Description>
   --
   --    Create a statement database node for the labels.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Label_Node
       (Name : Ref_Statement_Database) return Ref_Statement_Database is
   begin
      return
         Record_Statement
             (Create_Statement_Database_Item
                  (Create_Statement_Item (Kind  => Labeled_Statement,
                                          Item1 => Name)));
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Label_Node");
         raise Tree_Builder_Error;
   end Create_Label_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Exit_Node
   --
   --  <Description>
   --
   --    Create a statement database node for exits.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Exit_Node
       (Name, Expr : Ref_Statement_Database) return Ref_Statement_Database is
   begin
      return
         Record_Statement
             (Create_Statement_Database_Item
                  (Create_Statement_Item (Kind  => Exit_Statement,
                                          Item1 => Name,
                                          Item2 => Expr)));
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Exit_Node");
         raise Tree_Builder_Error;
   end Create_Exit_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Goto_Node
   --
   --  <Description>
   --
   --    Create a statement database node for the goto.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Goto_Node
       (Name : Ref_Statement_Database) return Ref_Statement_Database is
   begin
      return
         Record_Statement
             (Create_Statement_Database_Item
                  (Create_Statement_Item (Kind  => Goto_Statement,
                                          Item1 => Name)));
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Goto_Node");
         raise Tree_Builder_Error;
   end Create_Goto_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Return_Node
   --
   --  <Description>
   --
   --    Create a statement database node for the return statement.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Return_Node
       (Expr : Ref_Statement_Database) return Ref_Statement_Database is
   begin
      return
         Record_Statement
             (Create_Statement_Database_Item
                  (Create_Statement_Item (Kind  => Return_Statement,
                                          Item1 => Expr)));
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Return_Node");
         raise Tree_Builder_Error;
   end Create_Return_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Loop_Node
   --
   --  <Description>
   --
   --    Create a statement database node for loops.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Loop_Node
       (Name, Scheme, Seq_Of_Stmt : Ref_Statement_Database) return
         Ref_Statement_Database is
   begin
      return
         Record_Statement
             (Create_Statement_Database_Item
                  (Create_Statement_Item (Kind  => Loop_Statement,
                                          Item1 => Scheme,
                                          Item2 => Seq_Of_Stmt,
                                          Item3 => Name)));
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Loop_Node");
         raise Tree_Builder_Error;
   end Create_Loop_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_For_Node
   --
   --  <Description>
   --
   --    Create a statement database node for fors.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_For_Node
       (Variable, Reverse_Flag, Dscr_Rng : Ref_Statement_Database) return
         Ref_Statement_Database is

      Switch_Temp : Ref_Statement_Database;

   begin
      if not Is_Empty (Reverse_Flag) then
         -- switch upper and lower
         Switch_Temp := Get_Lower_Bound (Dscr_Rng);
         Set_Lower_Bound (Dscr_Rng, Get_Upper_Bound (Dscr_Rng));
         Set_Upper_Bound (Dscr_Rng, Switch_Temp);
      end if;
      return
         Record_Statement
             (Create_Statement_Database_Item
                  (Create_Statement_Item (Kind  => For_Statement,
                                          Item1 => Variable,
                                          Item2 => Dscr_Rng)));
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_For_Node");
         raise Tree_Builder_Error;
   end Create_For_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Null_Stmt_Node
   --
   --  <Description>
   --
   --    Create a statement database node for the null.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Null_Stmt_Node return Ref_Statement_Database is
   begin
      return
         Record_Statement
             (Create_Statement_Database_Item
                  (Create_Statement_Item (Null_Statement)));
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Null_Stmt_Node");
         raise Tree_Builder_Error;
   end Create_Null_Stmt_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Left_Stmt_Seq_Node
   --
   --  <Description>
   --
   --    Hooks statement database nodes together.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Left_Stmt_Seq_Node (Current_Stmt_Seq, Statement,
                                       Optional_Pragma : Ref_Statement_Database
                                       ) return Ref_Statement_Database is
   begin
      if not Is_Empty (Current_Stmt_Seq) then
         if Is_Empty (Get_Holder (Current_Stmt_Seq)) then --jat
            -- first one
            if not Is_Empty (Optional_Pragma) then
               Set_Holder (Current_Stmt_Seq, Optional_Pragma);
               Set_Next_Statement (Statement, Optional_Pragma);
               Set_Next_Statement (Current_Stmt_Seq, Statement);
            else
               Set_Holder (Current_Stmt_Seq, Statement);
               Set_Next_Statement (Current_Stmt_Seq, Statement);
            end if;
            return Current_Stmt_Seq;
         else
            -- in the middle
            if not Is_Empty (Optional_Pragma) then
               Set_Next_Statement (Statement, Optional_Pragma);
               Set_Next_Statement (Get_Holder (Current_Stmt_Seq), Statement);
               Set_Holder (Current_Stmt_Seq, Optional_Pragma);
            else
               Set_Next_Statement (Get_Holder (Current_Stmt_Seq), Statement);
               Set_Holder (Current_Stmt_Seq, Statement);
            end if;
            return Current_Stmt_Seq;
         end if;
      else
         if not Is_Empty (Optional_Pragma) then
            Set_Holder (Statement, Optional_Pragma);
            Set_Next_Statement (Statement, Optional_Pragma);
            return Statement;
         else
            return Statement;
         end if;
      end if;
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Left_Stmt_Seq_Node");
         raise Tree_Builder_Error;
   end Create_Left_Stmt_Seq_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Right_Stmt_Seq_Node
   --
   --  <Description>
   --
   --    Hooks statement database nodes together.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Right_Stmt_Seq_Node
       (Optional_Pragma, Statement, Current_Stmt_Seq : Ref_Statement_Database)
       return Ref_Statement_Database is
   begin
      Set_Next_Statement (Statement, Current_Stmt_Seq);
      if not Is_Empty (Optional_Pragma) then
         if Is_Empty (Get_Holder (Optional_Pragma)) then
            Set_Next_Statement (Optional_Pragma, Statement);
         else
            Set_Next_Statement (Get_Holder (Optional_Pragma), Statement);
         end if;
         return Optional_Pragma;
      else
         return Statement;
      end if;
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Right_Stmt_Seq_Node");
         raise Tree_Builder_Error;
   end Create_Right_Stmt_Seq_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Compilation_Node
   --
   --  <Description>
   --
   --    Returns a statement database node that hooks a context clause
   --    into a compilation unit.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Compilation_Node
       (Context_Cl, Comp_Unit : Ref_Statement_Database) return
         Ref_Statement_Database is
      Compilation_Symbol_Entry : Ref_Symbol_Table_Entry;
   begin
      if not Is_Empty (Context_Cl) then
         -- Get symbol table entry for compilation unit.
         Compilation_Symbol_Entry := Get_Symbol_Table_Entry (Comp_Unit);
         Set_Context_Clause (Compilation_Symbol_Entry, Context_Cl);
      end if;
      return Comp_Unit;
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Stmt_Seq_Node");
         raise Tree_Builder_Error;
   end Create_Compilation_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Stmt_Seq_Node
   --
   --  <Description>
   --
   --    Returns a statement database node that hooks two nodes together.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Stmt_Seq_Node
       (Stmt1, Stmt2 : Ref_Statement_Database) return Ref_Statement_Database is
   begin
      if Is_Empty (Stmt1) then
         return Stmt2;
      else
         Set_Next_Statement (Stmt1, Stmt2);
         return Stmt1;
      end if;
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Stmt_Seq_Node");
         raise Tree_Builder_Error;
   end Create_Stmt_Seq_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Assign_Node
   --
   --  <Description>
   --
   --    Create a statement database node for assignment.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Assign_Node
       (Name, Expr : Ref_Statement_Database) return Ref_Statement_Database is
   begin
      return
         Record_Statement
             (Create_Statement_Database_Item
                  (Create_Statement_Item (Kind  => Assign_Statement,
                                          Item1 => Name,
                                          Item2 => Expr)));
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Assign_Node");
         raise Tree_Builder_Error;
   end Create_Assign_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_If_Node
   --
   --  <Description>
   --
   --    Create a statement database node for ifs.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_If_Node
       (Condition, Elsif_True_Stmts, Previous_Elsif_Stmt :
        Ref_Statement_Database) return Ref_Statement_Database is
      --
      -- This routine creates the sequence of elsif clauses
      -- That will be hooked together by another Create_If_Node routine.
      --
   begin
      if Is_Empty (Previous_Elsif_Stmt) then
         -- First elsif clause was reduced as epsilon.
         -- We must build the first elsif branch.
         return
            Record_Statement
                (Create_Statement_Database_Item
                     (Create_Statement_Item (If_Statement, Condition,
                                             Elsif_True_Stmts)));
      else
         --
         -- We are in the middle of the elsif clause.
         -- We must hook the current elsif clause at the bottom.
         -- This is done by keeping track of the last elsif node
         -- in the Holder field of the top elsif node.
         --
         if Is_Empty (Get_Holder (Previous_Elsif_Stmt)) then
            Set_Holder (Previous_Elsif_Stmt, Previous_Elsif_Stmt);
         end if;

         Set_Head_Of_False_Task_Statements_List
             (Get_Holder (Previous_Elsif_Stmt),
              Record_Statement
                  (Create_Statement_Database_Item
                       (Create_Statement_Item (If_Statement, Condition,
                                               Elsif_True_Stmts, null))));

         --
         -- Assign the new end of the elsif list to the Holder
         -- field of the top elsif node.
         --
         Set_Holder
             (Previous_Elsif_Stmt,
              Get_Head_Of_False_Task_Statements_List
                  (Get_Holder (Previous_Elsif_Stmt)));
         --
         -- return the entire list of elsifs.
         --
         return Previous_Elsif_Stmt;
      end if;
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_If_Node");
         raise Tree_Builder_Error;
   end Create_If_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_If_Node
   --
   --  <Description>
   --
   --    Create a statement database node for ifs.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_If_Node
       (Condition, If_True_Stmts, Elsif_Stmts, Final_Else_Part :
        Ref_Statement_Database) return Ref_Statement_Database is
      --
      -- This will construct the top if statement node in the current
      -- IF ELSIF ELSE sequence.
      --
   begin
      --
      -- If there are no elsif statements, the False_Task should
      -- be the else statement.
      --
      if Is_Empty (Elsif_Stmts) then
         return
            Record_Statement
                (Create_Statement_Database_Item
                     (Create_Statement_Item (If_Statement, Condition,
                                             If_True_Stmts, Final_Else_Part)));
      else
         --
         -- Assign the else clause to the False_Task field of the last
         -- Elsif clause, which if pointed to by the Statement_Node
         -- field of the top Elsif node.
         --
         if not Is_Empty (Get_Holder (Elsif_Stmts)) then
            Set_Head_Of_False_Task_Statements_List (Get_Holder (Elsif_Stmts),
                                                    Final_Else_Part);
         else
            Set_Head_Of_False_Task_Statements_List (Elsif_Stmts,
                                                    Final_Else_Part);
         end if;

         return
            Record_Statement
                (Create_Statement_Database_Item
                     (Create_Statement_Item (If_Statement, Condition,
                                             If_True_Stmts, Elsif_Stmts)));
      end if;
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_If_Node");
         raise Tree_Builder_Error;
   end Create_If_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Expr_Node
   --
   --  <Description>
   --
   --    Create a statement database expression node.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Expr_Node (Operator : Integer) return Ref_Statement_Database
                             is
      --
      -- This create_expr_node routine is used for building
      -- partial operator nodes, that will be passed up the tree
      -- and filled in later by another create_expr_node routine.
      --
      -- Operators are represented as functions.
      --
   begin
      --
      -- Create a function with arguments that will be filled in later.
      --
      if Operator = Parserdefs.Type_Tk then
         return
            Create_Statement_Database_Item
                (Create_Expression_Item
                     (Kind => Terminal_Node,
                      Str => Convert_To_Ref_String (Lexacts.Display (Operator))
                      ));
      else
         return
            Create_Statement_Database_Item
                (Create_Expression_Item
                     (Kind => Left_Parenthesis_Operator,
                      Item1 =>
                         Create_Statement_Database_Item
                             (Create_Expression_Item
                                  (Kind => Terminal_Node,
                                   Str =>
                                      Convert_To_Ref_String
                                          (Lexacts.Display (Operator))))));
      end if;
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Expr_Node");
         raise Tree_Builder_Error;
   end Create_Expr_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Attr_Expr_Node
   --
   --  <Description>
   --
   --    Create a statement database expression node for aggregates.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Attr_Expr_Node
       (Exp1 : Ref_Statement_Database;
        Exp2 : Ref_Statement_Database) return Ref_Statement_Database is
   begin
      return
         Create_Statement_Database_Item
             (Create_Expression_Item (Kind  => Attribute_Tick,
                                      Item1 => Exp1,
                                      Item2 => Exp2));
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Attr_Expr_Node");
         raise Tree_Builder_Error;
   end Create_Attr_Expr_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Range_Expr_Node
   --
   --  <Description>
   --
   --    Create a statement database expression node for ranges.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Range_Expr_Node
       (Exp1 : Ref_Statement_Database;
        Exp2 : Ref_Statement_Database) return Ref_Statement_Database is
   begin
      return
         Create_Statement_Database_Item
             (Create_Expression_Item (Kind  => Range_Expression,
                                      Item1 => Exp1,
                                      Item2 => Exp2));
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Range_Expr_Node");
         raise Tree_Builder_Error;
   end Create_Range_Expr_Node;

   --==========================================================================
   --  <SUBPROGRAM NAME>
   --
   --    CREATE_EXPR_NODE
   --
   --  <DESCRIPTION>
   --
   --    CREATE A STATEMENT DATABASE EXPRESSION NODE.
   --
   --  <EXCEPTIONS RAISED>
   --
   --    TREE_BUILDER_ERROR
   --
   --  <EXCEPTIONS HANDLED>
   --
   --    ALL EXCEPTIONS ARE TURNED INTO TREE_BUILDER_ERROR.
   --==========================================================================
   function Create_Expr_Node
       (Exp1     : Ref_Statement_Database;
        Operator : Integer;
        Exp2     : Ref_Statement_Database) return Ref_Statement_Database is
   begin
      return
         Create_Statement_Database_Item
             (Create_Expression_Item
                  (Kind => Left_Parenthesis_Operator,
                   Item1 =>
                      Create_Statement_Database_Item
                          (Create_Expression_Item
                               (Kind => Terminal_Node,
                                Str =>
                                   Convert_To_Ref_String
                                       (Lexacts.Display (Operator)))),
                   Item2 =>
                      Create_Statement_Database_Item
                          (Create_Expression_Item
                               (Kind  => List_Item,
                                Item1 => Exp1,
                                Item2 =>
                                   Create_Statement_Database_Item
                                       (Create_Expression_Item
                                            (Kind  => List_Item,
                                             Item1 => Exp2))))));
   exception
      when others    =>
         Show_Error (3005, Internal, "CREATE_EXPR_NODE");
         raise Tree_Builder_Error;
   end Create_Expr_Node;

   --==========================================================================
   --  <SUBPROGRAM NAME>
   --
   --    CREATE_EXPR_NODE
   --
   --  <DESCRIPTION>
   --
   --    CREATE A STATEMENT DATABASE EXPRESSION NODE.
   --
   --  <EXCEPTIONS RAISED>
   --
   --    TREE_BUILDER_ERROR
   --
   --  <EXCEPTIONS HANDLED>
   --
   --    ALL EXCEPTIONS ARE TURNED INTO TREE_BUILDER_ERROR.
   --==========================================================================
   function Create_Expr_Node
       (Exp1      : Ref_Statement_Database;
        Operator1 : Integer;
        Operator2 : Integer;
        Exp2      : Ref_Statement_Database) return Ref_Statement_Database is
      --
      -- THIS ROUTINE HANDLES THE AND THEN, OR ELSE OPERATORS
      --
   begin
      return
         Create_Statement_Database_Item
             (Create_Expression_Item
                  (Kind => Left_Parenthesis_Operator,
                   Item1 =>
                      Create_Statement_Database_Item
                          (Create_Expression_Item
                               (Kind => Terminal_Node,
                                Str =>
                                   Convert_To_Ref_String
                                       ("_" & Lexacts.Display (Operator1) &
                                        Lexacts.Display (Operator2)))),
                   Item2 =>
                      Create_Statement_Database_Item
                          (Create_Expression_Item
                               (Kind  => List_Item,
                                Item1 => Exp1,
                                Item2 =>
                                   Create_Statement_Database_Item
                                       (Create_Expression_Item
                                            (Kind  => List_Item,
                                             Item1 => Exp2))))));
   exception
      when others    =>
         Show_Error (3005, Internal, "CREATE_EXPR_NODE");
         raise Tree_Builder_Error;
   end Create_Expr_Node;

   --==========================================================================
   --  <SUBPROGRAM NAME>
   --
   --    CREATE_EXPR_NODE
   --
   --  <DESCRIPTION>
   --
   --    CREATE A STATEMENT DATABASE EXPRESSION NODE.
   --
   --  <EXCEPTIONS RAISED>
   --
   --    TREE_BUILDER_ERROR
   --
   --  <EXCEPTIONS HANDLED>
   --
   --    ALL EXCEPTIONS ARE TURNED INTO TREE_BUILDER_ERROR.
   --==========================================================================
   function Create_Expr_Node
       (Exp1 : Ref_Statement_Database;
        Exp2 : Ref_Statement_Database;
        Exp3 : Ref_Statement_Database) return Ref_Statement_Database is
      --
      -- EXP2 IS THE OPERATOR (I.E., EXP2(EXP1,EXP2) ).
      --
   begin
      Set_List_Of_Items
          (Exp2,
           Create_Statement_Database_Item
               (Create_Expression_Item
                    (Kind  => List_Item,
                     Item1 => Exp1,
                     Item2 =>
                        Create_Statement_Database_Item
                            (Create_Expression_Item (Kind  => List_Item,
                                                     Item1 => Exp3)))));
      return Exp2;
   exception
      when others    =>
         Show_Error (3005, Internal, "CREATE_EXPR_NODE");
         raise Tree_Builder_Error;
   end Create_Expr_Node;

   --
   --==========================================================================
   --  <SUBPROGRAM NAME>
   --
   --    CREATE_EXPR_NODE
   --
   --  <DESCRIPTION>
   --
   --    CREATE A STATEMENT DATABASE EXPRESSION NODE.
   --
   --  <EXCEPTIONS RAISED>
   --
   --    TREE_BUILDER_ERROR
   --
   --  <EXCEPTIONS HANDLED>
   --
   --    ALL EXCEPTIONS ARE TURNED INTO TREE_BUILDER_ERROR.
   --==========================================================================
   function Create_Expr_Node (Str      : String;
                              Str_Leng : Integer) return Ref_Statement_Database
                             is
   begin
      -- IDENTIFIER ETC.
      return
         Create_Statement_Database_Item
             (Create_Expression_Item
                  (Kind => Terminal_Node,
                   Str =>
                      Convert_To_Ref_String (Str (Str'FIRST ..
                                                  Str'FIRST + Str_Leng - 1))));
   exception
      when others    =>
         Show_Error (3005, Internal, "CREATE_EXPR_NODE");
         raise Tree_Builder_Error;
   end Create_Expr_Node;

   --
   --==========================================================================
   --  <SUBPROGRAM NAME>
   --
   --    CREATE_STRING_NODE
   --
   --  <DESCRIPTION>
   --
   --    CREATE A STATEMENT DATABASE NODE EXPRESSION FOR STRINGS.
   --
   --  <EXCEPTIONS RAISED>
   --
   --    TREE_BUILDER_ERROR
   --
   --  <EXCEPTIONS HANDLED>
   --
   --    ALL EXCEPTIONS ARE TURNED INTO TREE_BUILDER_ERROR.
   --==========================================================================
   function Create_String_Node
       (Str      : String;
        Str_Leng : Integer) return Ref_Statement_Database is
   begin
      -- STRING LITERAL
      return
         Create_Statement_Database_Item
             (Create_Expression_Item
                  (Kind => String_Node,
                   Str =>
                      Convert_To_Ref_String (Str (Str'FIRST ..
                                                  Str'FIRST + Str_Leng - 1))));
   exception
      when others    =>
         Show_Error (3005, Internal, "CREATE_STRING_NODE");
         raise Tree_Builder_Error;
   end Create_String_Node;

   --
   --==========================================================================
   --  <SUBPROGRAM NAME>
   --
   --    CREATE_CHARACTER_NODE
   --
   --  <DESCRIPTION>
   --
   --    CREATE A STATEMENT DATABASE EXPRESSION NODE FOR CHARACTERS.
   --
   --  <EXCEPTIONS RAISED>
   --
   --    TREE_BUILDER_ERROR
   --
   --  <EXCEPTIONS HANDLED>
   --
   --    ALL EXCEPTIONS ARE TURNED INTO TREE_BUILDER_ERROR.
   --==========================================================================
   function Create_Character_Node
       (Str      : String;
        Str_Leng : Integer) return Ref_Statement_Database is
   begin
      -- CHARACTER LITERAL
      return
         Create_Statement_Database_Item
             (Create_Expression_Item
                  (Kind => Character_Node,
                   Str =>
                      Convert_To_Ref_String (Str (Str'FIRST ..
                                                  Str'FIRST + Str_Leng - 1))));
   exception
      when others    =>
         Show_Error (3005, Internal, "CREATE_CHARACTER_NODE");
         raise Tree_Builder_Error;
   end Create_Character_Node;

   --
   --==========================================================================
   --  <SUBPROGRAM NAME>
   --
   --    CREATE_CONSTANT_NODE
   --
   --  <DESCRIPTION>
   --
   --    CREATE A STATEMENT DATABASE EXPRESSION NODE FOR CONSTANTS.
   --
   --  <EXCEPTIONS RAISED>
   --
   --    TREE_BUILDER_ERROR
   --
   --  <EXCEPTIONS HANDLED>
   --
   --    ALL EXCEPTIONS ARE TURNED INTO TREE_BUILDER_ERROR.
   --==========================================================================
   function Create_Constant_Node
       (Str      : String;
        Str_Leng : Integer) return Ref_Statement_Database is
   begin
      -- CONSTANT
      return
         Create_Statement_Database_Item
             (Create_Expression_Item
                  (Kind => Constant_Node,
                   Str =>
                      Convert_To_Ref_String (Str (Str'FIRST ..
                                                  Str'FIRST + Str_Leng - 1))));
   exception
      when others    =>
         Show_Error (3005, Internal, "CREATE_CONSTANT_NODE");
         raise Tree_Builder_Error;
   end Create_Constant_Node;

   --
   --==========================================================================
   --  <SUBPROGRAM NAME>
   --
   --    CREATE_DOT_EXPR_NODE
   --
   --  <DESCRIPTION>
   --
   --    CREATE A STATEMENT DATABASE EXPRESSION NODE FOR EXPANDED NAMES.
   --
   --  <EXCEPTIONS RAISED>
   --
   --    TREE_BUILDER_ERROR
   --
   --  <EXCEPTIONS HANDLED>
   --
   --    ALL EXCEPTIONS ARE TURNED INTO TREE_BUILDER_ERROR.
   --==========================================================================
   function Create_Dot_Expr_Node
       (Exp1 : Ref_Statement_Database;
        Exp2 : Ref_Statement_Database) return Ref_Statement_Database is

      Temp : Ref_Statement_Database := null;

   begin
      -- OPERATOR IS THE DOT OPERATOR.
      if Is_Empty (Get_Holder (Exp1)) then
         -- TERMINAL "." TERMINAL
         Temp :=
            Create_Statement_Database_Item
                (Create_Expression_Item (Kind  => Dot_Operator,
                                         Item1 => Exp1,
                                         Item2 => Exp2));
         -- SAVE THE POINTER TO THE LEFT BOTTOM CHILD FOR FAST ACCESS.
         Set_Holder (Temp, Temp);
         return Temp;
      else
         -- TREE "." TERMINAL
         -- FIND THE BOTTOM OF THE TREE.
         Temp := Get_Dot_Component (Get_Holder (Exp1));
         -- ASSIGN THE NEW "." NODE TO THE BE THE NEW BOTTOM
         Set_Dot_Component
             (Get_Holder (Exp1),
              Create_Statement_Database_Item
                  (Create_Expression_Item (Kind  => Dot_Operator,
         -- OLD BOTTOM BECOMES THE LEFT SIDE
                                           Item1 => Temp,
         -- NEW BOTTOM
                                           Item2 => Exp2)));
         -- KEEP TRACK OF THE BOTTOM
         Set_Holder (Exp1, Get_Dot_Component (Get_Holder (Exp1)));
         return Exp1; -- RETURN THE TREE
      end if;
   exception
      when others    =>
         Show_Error (3005, Internal, "CREATE_DOT_EXPR_NODE");
         raise Tree_Builder_Error;
   end Create_Dot_Expr_Node;

   --==========================================================================
   --  <SUBPROGRAM NAME>
   --
   --    CREATE_EXPR_NODE
   --
   --  <DESCRIPTION>
   --
   --    CREATE A STATEMENT DATABASE EXPRESSION NODE.
   --
   --  <EXCEPTIONS RAISED>
   --
   --    TREE_BUILDER_ERROR
   --
   --  <EXCEPTIONS HANDLED>
   --
   --    ALL EXCEPTIONS ARE TURNED INTO TREE_BUILDER_ERROR.
   --==========================================================================
   function Create_Expr_Node
       (Operator : Integer;
        Exp1     : Ref_Statement_Database) return Ref_Statement_Database is
   begin
      return
         Create_Statement_Database_Item
             (Create_Expression_Item
                  (Kind => Left_Parenthesis_Operator,
                   Item1 =>
                      Create_Statement_Database_Item
                          (Create_Expression_Item
                               (Kind => Terminal_Node,
                                Str =>
                                   Convert_To_Ref_String
                                       (Lexacts.Display (Operator)))),
                   Item2 =>
                      Create_Statement_Database_Item
                          (Create_Expression_Item
                               (Kind  => List_Item,
                                Item1 => null,
                                Item2 =>
                                   Create_Statement_Database_Item
                                       (Create_Expression_Item
                                            (Kind  => List_Item,
                                             Item1 => Exp1))))));
   exception
      when others    =>
         Show_Error (3005, Internal, "CREATE_EXPR_NODE");
         raise Tree_Builder_Error;
   end Create_Expr_Node;

   --==========================================================================
   --  <SUBPROGRAM NAME>
   --
   --    CREATE_UNARY_EXPR_NODE
   --
   --  <DESCRIPTION>
   --
   --    CREATE A STATEMENT DATABASE EXPRESSION NODE.
   --
   --  <EXCEPTIONS RAISED>
   --
   --    TREE_BUILDER_ERROR
   --
   --  <EXCEPTIONS HANDLED>
   --
   --    ALL EXCEPTIONS ARE TURNED INTO TREE_BUILDER_ERROR.
   --==========================================================================
   function Create_Unary_Expr_Node
       (Operator : Integer;
        Exp1     : Ref_Statement_Database) return Ref_Statement_Database is
   begin
      return
         Create_Statement_Database_Item
             (Create_Expression_Item
                  (Kind => Left_Parenthesis_Operator,
                   Item1 =>
                      Create_Statement_Database_Item
                          (Create_Expression_Item
                               (Kind => Terminal_Node,
                                Str =>
                                   Convert_To_Ref_String
                                       (Lexacts.Display (Operator)))),
                   Item2 =>
                      Create_Statement_Database_Item
                          (Create_Expression_Item (Kind  => List_Item,
                                                   Item1 => Exp1,
                                                   Item2 => null))));
   exception
      when others    =>
         Show_Error (3005, Internal, "CREATE_UNARY_EXPR_NODE");
         raise Tree_Builder_Error;
   end Create_Unary_Expr_Node;

   --==========================================================================
   --  <SUBPROGRAM NAME>
   --
   --    CREATE_LEFT_EXPR_NODE
   --
   --  <DESCRIPTION>
   --
   --    CREATE A STATEMENT DATABASE EXPRESSION NODE.
   --
   --  <EXCEPTIONS RAISED>
   --
   --    TREE_BUILDER_ERROR
   --
   --  <EXCEPTIONS HANDLED>
   --
   --    ALL EXCEPTIONS ARE TURNED INTO TREE_BUILDER_ERROR.
   --==========================================================================
   function Create_Left_Expr_Node
       (Exp1 : Ref_Statement_Database;
        Exp2 : Ref_Statement_Database) return Ref_Statement_Database is
      --
      -- EXP1 OR EXP2 CAN BE THE OPERATOR AND THE OTHER IS THE EXPRESSION
      -- SIM_EXPR .RELAL_OP__SIM_EXPR. (EXP,OP)
      --
   begin
      if not Is_Empty (Exp2) then
         Set_Expression (Get_List_Of_Items (Exp2), Exp1);
         return Exp2;
      else
         return Exp1;
      end if;
   exception
      when others    =>
         Show_Error (3005, Internal, "CREATE_LEFT_EXPR_NODE");
         raise Tree_Builder_Error;
   end Create_Left_Expr_Node;

   --==========================================================================
   --  <SUBPROGRAM NAME>
   --
   --    CREATE_UNARY_ADD_EXPR_NODE
   --
   --  <DESCRIPTION>
   --
   --    CREATE A STATEMENT DATABASE EXPRESSION NODE FOR UNARY OPERATORS.
   --
   --  <EXCEPTIONS RAISED>
   --
   --    TREE_BUILDER_ERROR
   --
   --  <EXCEPTIONS HANDLED>
   --
   --    ALL EXCEPTIONS ARE TURNED INTO TREE_BUILDER_ERROR.
   --==========================================================================
   function Create_Unary_Add_Expr_Node
       (Exp1 : Ref_Statement_Database;
        Exp2 : Ref_Statement_Database) return Ref_Statement_Database is
      --
      -- RELAL_OP SIM_EXPR (OP,EXP)
      -- UNARY_ADD_OP TERM (OP,EXP)
      --
   begin
      Set_List_Of_Items
          (Exp1,
           Create_Statement_Database_Item
               (Create_Expression_Item
                    (Kind => List_Item,
                     Item1 =>
                        Create_Statement_Database_Item
                            (Create_Expression_Item
                                 (Kind => Constant_Node,
                                  Str => Convert_To_Ref_String ("0"))),
                     Item2 =>
                        Create_Statement_Database_Item
                            (Create_Expression_Item (Kind  => List_Item,
                                                     Item1 => Exp2)))));
      return Exp1;
   exception
      when others    =>
         Show_Error (3005, Internal, "CREATE_UNARY_ADD_EXPR_NODE");
         raise Tree_Builder_Error;
   end Create_Unary_Add_Expr_Node;

   --==========================================================================
   --  <SUBPROGRAM NAME>
   --
   --    CREATE_RIGHT_EXPR_NODE
   --
   --  <DESCRIPTION>
   --
   --    CREATE A STATEMENT DATABASE EXPRESSION NODE.
   --
   --  <EXCEPTIONS RAISED>
   --
   --    TREE_BUILDER_ERROR
   --
   --  <EXCEPTIONS HANDLED>
   --
   --    ALL EXCEPTIONS ARE TURNED INTO TREE_BUILDER_ERROR.
   --==========================================================================
   function Create_Right_Expr_Node
       (Exp1 : Ref_Statement_Database;
        Exp2 : Ref_Statement_Database) return Ref_Statement_Database is
   begin
      Set_List_Of_Items
          (Exp1,
           Create_Statement_Database_Item
               (Create_Expression_Item
                    (Kind  => List_Item,
                     Item1 => null,
                     Item2 =>
                        Create_Statement_Database_Item
                            (Create_Expression_Item (Kind  => List_Item,
                                                     Item1 => Exp2)))));
      return Exp1;
   exception
      when others    =>
         Show_Error (3005, Internal, "CREATE_RIGHT_EXPR_NODE");
         raise Tree_Builder_Error;
   end Create_Right_Expr_Node;

   --==========================================================================
   --  <SUBPROGRAM NAME>
   --
   --    CREATE_EXPR_NODE
   --
   --  <DESCRIPTION>
   --
   --    CREATE A STATEMENT DATABASE EXPRESSION NODE.
   --
   --  <EXCEPTIONS RAISED>
   --
   --    TREE_BUILDER_ERROR
   --
   --  <EXCEPTIONS HANDLED>
   --
   --    ALL EXCEPTIONS ARE TURNED INTO TREE_BUILDER_ERROR.
   --==========================================================================
   function Create_Expr_Node
       (Exp1 : Ref_Statement_Database;
        Exp2 : Ref_Statement_Database) return Ref_Statement_Database is
      use String_Constants;
   begin
      -- "ROTATE" TREE IF IT'S AN ATTRIBUTE WITH A PARAMETER, OTHERWISE IT
      -- LOOKS LIKE A LEFT_PARENTHESIS_OPERATOR INSTEAD OF AN ATTRIBUTE_TICK
      if Is_Attribute_Tick (Exp1) then
         Set_Attribute
             (Exp1,
              Create_Statement_Database_Item
                  (Create_Expression_Item (Kind  => Left_Parenthesis_Operator,
                                           Item1 => Get_Attribute (Exp1),
                                           Item2 => Exp2)));
         return Exp1;
      else
         return
            Create_Statement_Database_Item
                (Create_Expression_Item (Kind  => Left_Parenthesis_Operator,
                                         Item1 => Exp1,
                                         Item2 => Exp2));
      end if;
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Expr_Node 2");
         raise Tree_Builder_Error;
   end Create_Expr_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Name_Range_Expr_Node
   --
   --  <Description>
   --
   --    Create a statement database expression node for ranges.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Name_Range_Expr_Node
       (Exp1, Exp2 : Ref_Statement_Database) return Ref_Statement_Database is
   begin
      Set_Name_Range (Exp2, Exp1);
      return Exp2;
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Name_Range_Expr_Node");
         raise Tree_Builder_Error;
   end Create_Name_Range_Expr_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_New_Expr_Node
   --
   --  <Description>
   --
   --    Create a statement database expression node for the New operator.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_New_Expr_Node
       (Exp1 : Ref_Statement_Database;
        Exp2 : Ref_Statement_Database) return Ref_Statement_Database is
   begin
      return
         Create_Statement_Database_Item
             (Create_Expression_Item
                  (Kind => Left_Parenthesis_Operator,
                   Item1 =>
                      Create_Statement_Database_Item
                          (Create_Expression_Item
                               (Kind => Terminal_Node,
                                Str => Convert_To_Ref_String ("new"))),
                   Item2 =>
                      Create_Statement_Database_Item
                          (Create_Expression_Item
                               (Kind  => List_Item,
                                Item1 => Exp1,
                                Item2 =>
                                   Create_Statement_Database_Item
                                       (Create_Expression_Item
                                            (Kind  => List_Item,
                                             Item1 => Exp2))))));
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_New_Expr_Node");
         raise Tree_Builder_Error;
   end Create_New_Expr_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Expr_Node
   --
   --  <Description>
   --
   --    Create a statement database expression node.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Expr_Node
       (Exp1     : Ref_Statement_Database;
        Exp2     : Ref_Statement_Database;
        Operator : Integer;
        Exp3     : Ref_Statement_Database) return Ref_Statement_Database is
   begin
      return null;
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Expr_Node");
         raise Tree_Builder_Error;
   end Create_Expr_Node;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create_Aggr_Expr_Node
   --
   --  <Description>
   --
   --    Create a statement database expression node for aggregates.
   --
   --  <Exceptions Raised>
   --
   --    Tree_Builder_Error
   --
   --  <Exceptions Handled>
   --
   --    All exceptions are turned into Tree_Builder_Error.
   --==========================================================================
   function Create_Aggr_Expr_Node
       (Exp1 : Ref_Statement_Database;
        Exp2 : Ref_Statement_Database;
        Exp3 : Ref_Statement_Database) return Ref_Statement_Database is
   begin
      return
         Create_Statement_Database_Item
             (Create_Expression_Item
                  (Kind  => Aggregate_Component,
                   Item1 => Link_Id_Onto_Front_Of_List (Exp1, Exp2),
                   Item2 => Exp3));
   exception
      when others    =>
         Show_Error (3005, Internal, "Create_Aggr_Expr_Node");
         raise Tree_Builder_Error;
   end Create_Aggr_Expr_Node;

end Create_Statement_Database;
