--|$Log:	denum.a,v $
--|Revision 8.0  91/01/17  10:44:45  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Subunit for Package - Declaration_Routines
--
---------------------------------------------------------------
--
--
-- Abstract
--
--   The following contains separate routines for the package
--   Declaration_Routines
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
-- --------------------------------------------------------------------------
--==========================================================================
--  <Subprogram Name>
--
--    Create_Enum_Type_Def_Entry
--
--  <Description>
--
--    This routine creates an enumeration type definition on the
--    symbol table.
--
--==========================================================================
separate (Declaration_Routines)

function Create_Enum_Type_Def_Entry
    (Context        : in Context_Control.Context_Definition;
     Enum_Type_Node : in Ref_Statement_Database) return Ref_Statement_Database
    is

   Symtbl_Entry, Last_Entry, Tmp_Entry : Ref_Symbol_Table_Entry;
   Count : Ace_Natural := 0;

begin

   -- Create a symbol table entry for the enumeration type definition
   Symtbl_Entry :=
      Create_Symbol_Table_Entry (Convert_To_Ref_String (Anonymous_Name),
                                 Type_Entry);
   Set_Type_To_Enumeration_Type (Symtbl_Entry);

   Set_First_Literal (Symtbl_Entry, Get_Symbol_Table_Entry (Enum_Type_Node));

   -- For the enumeration type definition, count the number of enumeration
   -- literals, locate the last literal, and check for duplicates within the
   -- list of enumeration literals.  Also set the static nesting level
   -- for each enumeration literal.
   Tmp_Entry := Get_First_Literal (Symtbl_Entry);
   Init_For_Duplicate_Check;
   while (not Is_Empty (Tmp_Entry)) loop
      Set_Level (Tmp_Entry, Get_Current_Level (Context));
      Set_Literal_Value (Tmp_Entry, Count);
      Count      := Count + 1;
      Last_Entry := Tmp_Entry;

      -- Check this enum literal entry with past entries
      if Check_For_Duplicates (Get_Symbolic_Name (Tmp_Entry)) then
         --E user error: two enumeration literals with same name
         Show_Error (2002, Fatal,
                     Convert_To_String (Get_Symbolic_Name (Tmp_Entry)));
      end if;

      Tmp_Entry := Get_Successor (Tmp_Entry);
   end loop;

   Set_Literal_Count (Symtbl_Entry, Count);
   Set_Last_Literal (Symtbl_Entry, Last_Entry);

   Tmp_Entry := Get_First_Literal (Symtbl_Entry);
   while (not Is_Empty (Tmp_Entry)) loop

      -----
      -- create a parameterless function call out of each literal...
      --
      declare

         New_Function_Entry : Ref_Symbol_Table_Entry :=
            Create_Symbol_Table_Entry (Get_Symbolic_Name (Tmp_Entry),
                                       Function_Entry);

         Return_Stmt : Ref_Statement_Database := Create_Node (Return_Statement);

         Enum_Lit_Expr : Ref_Statement_Database := Create_Node (Terminal_Node);

      begin
         -----
         -- convert to a parameterless function call!

         -- add items to new function_entry symtbl entry
         Set_Enumeration_Literal_Entry (New_Function_Entry, Tmp_Entry);

         -- make reference to parameterless function call symtbl entry
         Set_Function_Entry (Tmp_Entry, New_Function_Entry);

         -- == create stmt_db "return_stmt" node
         -- create return_expr_node
         Set_Symbolic_Name (Enum_Lit_Expr, Get_Symbolic_Name (Tmp_Entry));
         Set_Symbol_Table_Entry (Enum_Lit_Expr, Tmp_Entry);
         Set_Type (Enum_Lit_Expr, Get_Type (Tmp_Entry));
         -- assign return_expr_node to return_stmt_node
         Set_Expression (Return_Stmt, Enum_Lit_Expr);
         -- this should be ok
         Set_Subprogram_Reference (Return_Stmt, New_Function_Entry);
         -- record return_stmt_node in symtbl entry
         Set_Head_Of_Statements_List (New_Function_Entry, Return_Stmt);
         -- ==

         -- ***
         -- Set_Symbol_Table_Entry
         --    (Get_Subprogram_Info (Node), new_function_entry);

         --== ... and add function entry to symtbl as needed
         -- [oei] (done elsewhere)
         -- Add_Entry_To_Symbol_Table (New_Function_Entry);
         -----
      end;

      Tmp_Entry := Get_Successor (Tmp_Entry);
   end loop;

   -- Create and return a Statement database node that contains the
   -- enumeration type symbol table entry
   return (Create_Entry_Ref_Node (Symtbl_Entry));

end Create_Enum_Type_Def_Entry;

-- --------------------------------------------------------------------------
--==========================================================================
--  <Subprogram Name>
--
--    Link_Enum_Literal_On_Front
--
--  <Description>
--
--   The first enumeration literal (pred_node) is linked onto the front
--   of the rest of the enumeration literals that have already been
--   constructed at held in list_node.
--   Note that list_node may be null if the enumeration type consists of
--   only a single literal.
--
--==========================================================================
separate (Declaration_Routines)

function Link_Enum_Literal_On_Front
    (Pred_Node, List_Node : in Ref_Statement_Database) return
      Ref_Statement_Database is

begin

   -- Set the predecessor field of the List_Node to be the Pred_Node, and
   -- set the sucessor field of the Pred_Node to be the List_Node
   -- if List_Node exists
   if not Is_Empty (List_Node) then
      Set_Predecessor (Get_Symbol_Table_Entry (List_Node),
                       Get_Symbol_Table_Entry (Pred_Node));
      Set_Successor (Get_Symbol_Table_Entry (Pred_Node),
                     Get_Symbol_Table_Entry (List_Node));
   end if;

   -- Return the Pred_Node (which is already a statement database node).
   return (Pred_Node);

end Link_Enum_Literal_On_Front;

-- --------------------------------------------------------------------------

--==========================================================================
--  <Subprogram Name>
--
--    Link_Enum_Literal_On_End
--
--  <Description>
--
--   The enumeration literal (succ_node) is linked onto the end of the
--   current list of enumeration literals that have already been linked
--   together in list_node.
--   Note that list_node may be null when the list is beginning to be
--   built.
--==========================================================================
separate (Declaration_Routines)
function Link_Enum_Literal_On_End
    (List_Node, Succ_Node : in Ref_Statement_Database) return
      Ref_Statement_Database is

   -- Effects:

   -- Detail:
   --   Assume type xxx is (x1, x2, x3, x4);
   --   This routine will be call as follows:
   --      Link_Enum_Literal_On_End ( <empty list>, "x2")
   --      Link_Enum_Literal_On_End ( <x2>,         "x3")
   --      Link_Enum_Literal_On_End ( <x2 x3>,      "x4")
   --   and then Link_Enum_Literal_On_Front would be called with
   --      Link_Enum_Literal_On_Front ("x1", <x2 x3 x4>)

   --   To avoid having to search to the end of the list each time to link
   --   on the new enumeration literal, the field "Next_Entry" in the
   --   Symbol_Table_Entry (of the literal at the head of the list) will be
   --   used to hold a reference to the literal at the tail of the list.

   Last_Literal : Ref_Symbol_Table_Entry;

begin

   if not Is_Empty (List_Node) then
      -- acquire the  reference to the last literal in the list
      -- (stored in the Next_Entry field of the first element in the list)
      Last_Literal := Get_Next_Entry (Get_Symbol_Table_Entry (List_Node));

      -- Set the predecessor field of the Succ_Node to be the Last_Literal,
      -- and set the successor field of the Last_Literal to be the Succ_Node
      Set_Predecessor (Get_Symbol_Table_Entry (Succ_Node), Last_Literal);
      Set_Successor (Last_Literal, Get_Symbol_Table_Entry (Succ_Node));

      -- update the last_literal pointer stored in the first literal in
      -- the list
      Set_Next_Entry (Get_Symbol_Table_Entry (List_Node),
                      Get_Symbol_Table_Entry (Succ_Node));
   else
      -- The list was empty.  Predecessor and successor fields do not need
      -- to be set.  But now a list will been constructed (of one item - the
      -- Succ_Node), so set the Next_Entry field of the first item to
      -- point to the last item (itself).
      Set_Next_Entry (Get_Symbol_Table_Entry (Succ_Node),
                      Get_Symbol_Table_Entry (Succ_Node));
   end if;

   -- Return the List_Node (which is already a statement database node)
   -- if it exists, otherwise return the Succ_Node (which also is already
   -- a statement database node)
   if not Is_Empty (List_Node) then
      return (List_Node);
   else
      return (Succ_Node);
   end if;

end Link_Enum_Literal_On_End;

-- --------------------------------------------------------------------------
--==========================================================================
--  <Subprogram Name>
--
--    Create_Enum_Literal_Entry
--
--  <Description>
--
-- Check that enumeration literal name does not clash with other
-- identifiers
--
--==========================================================================
separate (Declaration_Routines)

function Create_Enum_Literal_Entry
    (Context           : in Context_Control.Context_Definition;
     Enum_Literal_Node : in Ref_Statement_Database) return
      Ref_Statement_Database is

begin

   -- Check that enumeration literal name does not clash with other
   -- identifiers
   -- NOTE: enumeration literals may be overloaded.  There may be multiple
   -- occurences of the id, but only as an enumeration literal.
   if Name_Clash_For_Enum_Literal (Context,
                                   Get_Symbolic_Name (Enum_Literal_Node)) then
      --E user error: redefinition/attempt to overload id
      Show_Error (2001, Fatal,
                  Convert_To_String (Get_Symbolic_Name (Enum_Literal_Node)));
   end if;

   -- Create an enumeration literal symbol table entry, and return a
   -- statement database node that contains the enumeration literal
   -- symbol table entry
   return
      (Create_Entry_Ref_Node
          (Create_Symbol_Table_Entry (Get_Symbolic_Name (Enum_Literal_Node),
                                      Enumeration_Literal_Entry)));

end Create_Enum_Literal_Entry;

-- --------------------------------------------------------------------------
