--|$Log:	drecs.a,v $
--|Revision 8.0  91/01/17  10:44:48  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Subunit for Package - Declaration_Routines
--
---------------------------------------------------------------
--
--
-- Abstract
--
--   The following routines are separate units for the package
--   Declaration_Routines.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
-- --------------------------------------------------------------------------
--==========================================================================
--  <Subprogram Name>
--
--    Create_Record_Type_Entry
--
--  <Description>
--
-- Create a statement database node which contains the record symbol
-- table entry
--
--
--==========================================================================
separate (Declaration_Routines)

function Create_Record_Type_Entry
    (Context                     : in Context_Control.Context_Definition;
     Record_Components_List_Node : in Ref_Statement_Database) return
      Ref_Statement_Database is

   Record_Symbol_Table    : Ref_Symbol_Table_Entry;
   Record_Component_Entry : Ref_Symbol_Table_Entry;

   -- Note: the Record_Components_List_Node may be null

begin

   -- Check that all record component identifiers are unique
   if not Is_Empty (Record_Components_List_Node) then
      Record_Component_Entry :=
         Get_Symbol_Table_Entry (Record_Components_List_Node);
      Init_For_Duplicate_Check;
      while not Is_Empty (Record_Component_Entry) loop
         if Check_For_Duplicates (Get_Symbolic_Name (Record_Component_Entry))
             then
            --E user error: duplicate record component names
            Show_Error
                (2001, Fatal,
                 Convert_To_String (Get_Symbolic_Name (Record_Component_Entry))
                 );
         end if;

         Record_Component_Entry :=
            Get_Next_Record_Component (Record_Component_Entry);
      end loop;
   end if;

   -- Create a Record Type symbol table entry at this point (list of
   -- components may be null)
   Record_Symbol_Table :=
      Create_Symbol_Table_Entry (Convert_To_Ref_String (Anonymous_Name),
                                 Type_Entry);
   Set_Type_To_Record_Type (Record_Symbol_Table);
   if not Is_Empty (Record_Components_List_Node) then
      Set_Record_Component
          (Record_Symbol_Table,
           Get_Symbol_Table_Entry (Record_Components_List_Node));
   end if;
   Set_Level (Record_Symbol_Table, Get_Current_Level (Context));

   -- Create a statement database node which contains the record symbol
   -- table entry
   return (Create_Entry_Ref_Node (Record_Symbol_Table));

end Create_Record_Type_Entry;

-- --------------------------------------------------------------------------
--==========================================================================
--  <Subprogram Name>
--
--    Link_Record_Component_Entries
--
--  <Description>
--
-- The component node (which may contain several component fields) are
-- linked onto the end of the component list node.  The "Next_Entry" field
-- of the first entry in the component list node points to the last entry
-- of the component list node (similarly for the component node).
--
--
--==========================================================================
separate (Declaration_Routines)

function Link_Record_Component_Entries
    (Component_List_Node : in Ref_Statement_Database;
     Component_Node      : in Ref_Statement_Database) return
      Ref_Statement_Database is

   -- Notes:
   -- Component_List_Node may be null

   Component_Node_First_Entry      : Ref_Symbol_Table_Entry;
   Component_List_Node_First_Entry : Ref_Symbol_Table_Entry;

begin
   -- If the list is empty, nothing needs to be done
   if Is_Empty (Component_List_Node) then
      return (Component_Node);
   end if;

   Component_Node_First_Entry := Get_Symbol_Table_Entry (Component_Node);
   Component_List_Node_First_Entry :=
      Get_Symbol_Table_Entry (Component_List_Node);

   -- Set the next record component link in the last record field in
   -- component list node.
   -- The first record field in component list node has a reference to the
   -- last record field retained in the Next_Entry field.
   Set_Next_Record_Component
       (Get_Next_Entry (Component_List_Node_First_Entry),
        Component_Node_First_Entry);

   -- Update the Next_Entry field in the first entry of the component list
   -- node to point to the last entry of the component node (which is
   -- retained in the Next_Entry field of the first entry of the component
   -- node).
   Set_Next_Entry (Component_List_Node_First_Entry,
                   Get_Next_Entry (Component_Node_First_Entry));

   return (Component_List_Node);

end Link_Record_Component_Entries;

-- --------------------------------------------------------------------------
--==========================================================================
--  <Subprogram Name>
--
--    Build_Record_Component
--
--  <Description>
--
-- A record component symbol table entry is built for each identifier
-- in the id list.  The symbol table entries are linked together and
-- returned as a single statement database node.
--
--
--==========================================================================
separate (Declaration_Routines)

function Build_Record_Component
    (Context          : in Context_Control.Context_Definition;
     Id_List_Node     : in Ref_Statement_Database;
     Subtype_Ind_Node : in Ref_Statement_Database;
     Init_Node        : in Ref_Statement_Database) return
      Ref_Statement_Database is

   -- Id_List: always id list (1 or more ids)
   -- Init_Node: may be null

   Current              : Ref_Statement_Database;

   Record_Comp_Name     : Ref_String;

   Record_Comp_Entry    : Ref_Symbol_Table_Entry;
   First_Record_Comp    : Ref_Symbol_Table_Entry := null;
   Previous_Record_Comp : Ref_Symbol_Table_Entry := null;

begin
   -- Determine whether the initialization code is appropriate for the
   -- type
   if not Is_Empty (Init_Node) then
      -- [TBD]
      null;
   end if;

   -- For each identifier in the id list, build up a record component
   -- symbol table entry.
   Current := Id_List_Node;
   while not Is_Empty (Current) loop

      -- Extract the record component name from the id list
      Record_Comp_Name := Get_Symbolic_Name (Get_Expression (Current));

      -- Build symbol table entry for the record component
      Record_Comp_Entry := Create_Symbol_Table_Entry (Record_Comp_Name,
                                                      Record_Component_Entry);

      -- Set the current level, type, and next record component fields
      Set_Type (Record_Comp_Entry, Get_Symbol_Table_Entry (Subtype_Ind_Node));
      Set_Level (Record_Comp_Entry, Get_Current_Level (Context));

      if not Is_Empty (Previous_Record_Comp) then
         Set_Next_Record_Component (Previous_Record_Comp, Record_Comp_Entry);
      end if;
      Previous_Record_Comp := Record_Comp_Entry;

      -- Set the initialization code, if it exists
      if not Is_Empty (Init_Node) then
         Set_Initialization_Code (Record_Comp_Entry, Init_Node);
      end if;

      -- Retain a reference to the first record component entry
      if Is_Empty (First_Record_Comp) then
         First_Record_Comp := Record_Comp_Entry;
      end if;

      -- Get next record component id from id_list
      Current := Get_Next_List_Item (Current);

   end loop;

   -- The next entry field of the first record component symbol table entry
   -- will be used as a holder to retain a reference to the last record
   -- component in the list
   Set_Next_Entry (First_Record_Comp, Record_Comp_Entry);

   -- Create a statement database node that contains the list of record
   -- component symbol table entries
   return (Create_Entry_Ref_Node (First_Record_Comp));

end Build_Record_Component;

-- --------------------------------------------------------------------------
