--|$Log:	drsup.a,v $
--|Revision 8.0  91/01/17  10:44:49  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Declarative_Region_Support_Routines
--
------------------------------------------------------------
--
--
-- Abstract
--
-- Support routines for manipulating Ada declarative regions
--
-- Update Log - :
--
-- End of Header
-------------------------------------------------------------------------
with Symbol_Table_Structures;
use Symbol_Table_Structures;

with Symtbl_Entry_Get_Routines;
use Symtbl_Entry_Get_Routines;

with Statement_Evaluator;

with Symtbl_Entry_Set_Routines;
use Symtbl_Entry_Set_Routines;

with Symbol_Table_Routines;
use Symbol_Table_Routines;

with Statement_Database;
use Statement_Database;

with Storage_Management_Routines;
use Storage_Management_Routines;

with Node_Get_Routines;
use Node_Get_Routines;

with Misc_Support_Routines;
use Misc_Support_Routines;

with Error_Handler;
use Error_Handler;

with Ace_Universal_Types;
use Ace_Universal_Types;

with Literals;
use Literals;

with Text_Io;
use Text_Io;

with Context_Control;

package body Declarative_Region_Support_Routines is

   -- These three should be removed when we
   -- implement "interpret" correctly. TBD
   At_Top_Level              : Boolean                := True;
   In_Aceadt_Compile         : Boolean                := False;
   Aceadt_Special_Case_Entry : Ref_Symbol_Table_Entry := null;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Declarative_Region_Entry
   --
   --  <Description>
   --
   --    This routine returns the position in the symbol table of
   --    the current declarative region.
   --
   --=======================================================================
   function Get_Declarative_Region_Entry
       (Context : in Context_Control.Context_Definition) return
         Ref_Symbol_Table_Entry is
   begin
      return
         (Context_Control.Get_Declarative_Region_Entry
             (Context, Context_Control.Get_Dregion_Count (Context)));
   end Get_Declarative_Region_Entry;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Valid_Return_Statement_Encountered
   --
   --  <Description>
   --
   --    This routine marks a declarative region that a return statement
   --    has been encounters.   We need this to be able to unwind
   --    the stack when executing a return.
   --=======================================================================
   procedure Valid_Return_Statement_Encountered
       (Context : in out Context_Control.Context_Definition) is
   begin
      Context_Control.Set_Have_Encountered_Valid_Return_Stmt
          (Context, Get_Current_Level (Context), True);
   end Valid_Return_Statement_Encountered;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Within_Declarative_Region
   --
   --  <Description>
   --
   --    This routine returns a boolean indicating whether or not we are
   --    currently in a declarative region.
   --
   --=======================================================================
   function Within_Declarative_Region
       (Context : in Context_Control.Context_Definition) return Boolean is
   begin
      return
         (Context_Control.Get_Dregion_Count (Context) > Ace_Natural'FIRST) or
         (Context_Control.Get_Declarative_Region_Entry (Context,
                                                        Ace_Natural'FIRST) /=
         null);
   end Within_Declarative_Region;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Set_Decl_Region_Top_Of_Symtbl
   --
   --  <Description>
   --
   --    This routine explicitly sets the declarative region.
   --    [not used? -- oei]
   --
   --=======================================================================
   procedure Set_Decl_Region_Top_Of_Symtbl
       (Context      : in out Context_Control.Context_Definition;
        Symtbl_Entry : in Ref_Symbol_Table_Entry) is
   begin
      Context_Control.Set_Declarative_Region
          (Context, Context_Control.Get_Dregion_Count (Context), Symtbl_Entry);
   end Set_Decl_Region_Top_Of_Symtbl;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Level_Zero_Entry
   --
   --  <Description>
   --
   --    This routine returns the position in the symbol table of
   --    the current declarative region.
   --
   --=======================================================================
   function Get_Level_Zero_Entry
       (Context : in Context_Control.Context_Definition) return
         Ref_Symbol_Table_Entry is
   begin
      return (Context_Control.Get_Declarative_Region_Entry (Context,
                                                            Ace_Natural'FIRST));
   end Get_Level_Zero_Entry;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Push_Declarative_Region
   --
   --  <Description>
   --
   --    This routine will push a declarative region, thus
   --    making it the current declarative region.
   --
   --=======================================================================
   procedure Push_Declarative_Region
       (Context      : in out Context_Control.Context_Definition;
        Symtbl_Entry : in Ref_Symbol_Table_Entry) is
   begin

      -- do not set the saved_main in this routine because
      -- an error in a package declaration will result in
      -- an empty package being declared.  The saved_main should
      -- only be set at the point where a prompt would be
      -- displayed.

      --if (Context_Control.Get_Dregion_Count (Context) = Ace_Natural'FIRST)
      --   then
      --   -- save this in case there is a "fatal" error later
      --   Context_Control.Set_Saved_Main_Level_Symbol_Table_Head
      --       (Context, Get_Top_Of_Symtbl (Context));
      --end if;

      begin
         Add_Entry_To_Symbol_Table (Context, Symtbl_Entry);
      exception
         when Duplicate_Symbol_Table_Entry_Error   =>
            null;
      end;

      Context_Control.Set_Dregion_Count
          (Context, Context_Control.Get_Dregion_Count (Context) + 1);
      Context_Control.Set_Declarative_Region
          (Context, Context_Control.Get_Dregion_Count (Context),
           Get_Top_Of_Symtbl (Context));

      Context_Control.Set_Declarative_Region_Entry
          (Context, Context_Control.Get_Dregion_Count (Context), Symtbl_Entry);

      case Get_Entry_Kind (Symtbl_Entry) is
         when Function_Entry   | Procedure_Entry   =>
            declare
               Formal_Parameters_List : Ref_Symbol_Table_Entry :=
                  Get_Head_Of_Parameters_List (Symtbl_Entry);
            begin
               Incr_Level (Context);
               -- also inits offset for new level; saves previous offset

               Context_Control.Set_Have_Encountered_Valid_Return_Stmt
                   (Context, Get_Current_Level (Context), False);

               -- push formal parameters onto the front of the symbol table
               while (not Is_Empty (Formal_Parameters_List)) loop
                  Add_Entry_To_Symbol_Table (Context, Formal_Parameters_List);
                  Formal_Parameters_List :=
                     Get_Next_Parameter (Formal_Parameters_List);
               end loop;

            end;
         when others =>
            null;
      end case;

   end Push_Declarative_Region;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Pop_Declarative_Region
   --
   --  <Description>
   --
   --    This routine will pop off the current declarative region, thus
   --    restoring the previous declarative region.
   --
   --=======================================================================
   procedure Pop_Declarative_Region
       (Context : in out Context_Control.Context_Definition) is

      Current_Entry       : Context_Control.Ref_List_Of_Entries :=
         Context_Control.Get_Hidden_Entries
             (Context, Context_Control.Get_Dregion_Count (Context));
      Corresponding_Entry : Ref_Symbol_Table_Entry := null;

   begin

      Corresponding_Entry :=
         Context_Control.Get_Declarative_Region_Entry
             (Context, Context_Control.Get_Dregion_Count (Context));

      case Get_Entry_Kind (Corresponding_Entry) is
         when Function_Entry   =>
            if (not Context_Control.Get_Have_Encountered_Valid_Return_Stmt
                   (Context, Get_Current_Level (Context))) then
               Show_Error
                   (1404, Warning,
                    Convert_To_String (Get_Symbolic_Name (Corresponding_Entry))
                    );
            end if;
            Decr_Level (Context); -- restores previous offset

         when Procedure_Entry  =>
            Decr_Level (Context); -- restores previous offset

         when others =>
            null;
      end case;

      -- reset current declarative region symtbl entry
      Context_Control.Set_Declarative_Region_Entry
          (Context, Context_Control.Get_Dregion_Count (Context), null);

      Set_Top_Of_Symtbl
          (Context,
           Context_Control.Get_Declarative_Region
               (Context, Context_Control.Get_Dregion_Count (Context)));
      Context_Control.Set_Declarative_Region
          (Context, Context_Control.Get_Dregion_Count (Context), null);

      -- reset entries that were hidden w/in this declarative region
      while (not Context_Control."=" (Current_Entry, null)) loop
         Make_Entry_Visible (Current_Entry.Hidden_Entry);
         Current_Entry := Current_Entry.Next_Hidden_Entry;
      end loop;
      -- [tbd] Free (Hidden_Entries (Get_Dregion_Count(Context)),
      --              Entire_Substructure);

      Context_Control.Set_Dregion_Count
          (Context, Context_Control.Get_Dregion_Count (Context) - 1);

   end Pop_Declarative_Region;

   --=====================================================================
   --  <Subprogram Name>
   --
   --    Is_In_AceAdt_Compile, Set_In_AceAdt_Compile, UnSet_In_AceAdt_Compile
   --
   --  <Description>
   --
   --    These routines provide access to a marker of when an ACE ADT
   --    "compile" routine is being executed.
   --
   --  These should be removed [tbd]
   --
   --=====================================================================
   function Is_In_Aceadt_Compile return Boolean is
   begin
      return In_Aceadt_Compile;
   end Is_In_Aceadt_Compile;

   procedure Set_In_Aceadt_Compile is
   begin
      null;
   end Set_In_Aceadt_Compile;

   procedure Unset_In_Aceadt_Compile is
   begin
      null;
   end Unset_In_Aceadt_Compile;

   --====================================================================
   --  <Subprogram Name>
   --
   --    Set_DR_To_Top_Level
   --
   --  <Description>
   --
   --    Sets the declarative region temporarily back to the top level
   --    (level 0) environment.
   --
   --=======================================================================
   procedure Set_Dr_To_Top_Level
       (Context : in out Context_Control.Context_Definition) is
   begin
      if (Context_Control.Get_Dregion_Count (Context) = Ace_Natural'FIRST) then
         At_Top_Level := True;

      else
         At_Top_Level := False;

         -- save position in DR stack
         Context_Control.Set_Dr_Count_Keep
             (Context, Context_Control.Get_Dregion_Count (Context));

         -- save level/offset for later
         Reset_Leveloffset_To_Top_Level (Context);

         -- set DR to level zero
         Context_Control.Set_Dregion_Count (Context, Ace_Natural'FIRST);

         -- set top of symtbl for level zero
         if (Context_Control.Get_Dr_Count_Keep (Context) > Ace_Natural'FIRST)
             then
            Symbol_Table_Routines.Set_Top_Of_Symtbl
                (Context,
                 Context_Control.Get_Declarative_Region (Context,
                                                         Ace_Natural'FIRST + 1)
                 );
         end if;
      end if;
   end Set_Dr_To_Top_Level;

   --====================================================================
   --  <Subprogram Name>
   --
   --    Reset_DR_To_Previous_Level
   --
   --  <Description>
   --
   --    Undoes the declarative region temporarily back to the top level
   --    (level 0) environment.
   --
   --====================================================================
   procedure Reset_Dr_To_Previous_Level
       (Context : in out Context_Control.Context_Definition) is
   begin
      if (not At_Top_Level) then
         -- reset "old" position in DR stack
         Context_Control.Set_Dregion_Count
             (Context, Context_Control.Get_Dr_Count_Keep (Context));

         -- reset previous level/offset
         Reset_Leveloffset_To_Previous_Dr_Level (Context);

         -- reset "old" symtbl head reference to new(est) top of symtbl
         Symbol_Table_Routines.Set_Saved_Main_Level_Symbol_Table_Head
             (Context, Get_Top_Of_Symtbl (Context));
         Context_Control.Set_Declarative_Region
             (Context, Ace_Natural'FIRST + 1, Get_Top_Of_Symtbl (Context));
      end if;
   end Reset_Dr_To_Previous_Level;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Declarative_Region_Mark
   --
   --  <Description>
   --
   --    This routine returns the position in the symbol table of
   --    the current declarative region.
   --
   --=======================================================================
   function Get_Declarative_Region_Mark
       (Context : in Context_Control.Context_Definition) return
         Ref_Symbol_Table_Entry is
   begin
      return
         (Context_Control.Get_Declarative_Region
             (Context, Context_Control.Get_Dregion_Count (Context)));
   end Get_Declarative_Region_Mark;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Push_Hidden_Entry
   --
   --  <Description>
   --
   --    This routine will add to a list of hidden entries, so that the
   --    entry may be made visible again at the end of the decalaritive
   --    region.
   --
   -- only entries actually hidden w/in this declarative region are received
   --   [i.e. entries hidden w/in other declarative regions are not received]
   --
   --=======================================================================
   procedure Push_Hidden_Entry
       (Context      : in out Context_Control.Context_Definition;
        Symtbl_Entry : in Ref_Symbol_Table_Entry) is
   begin
      if Within_Declarative_Region (Context) then
         Context_Control.Set_Hidden_Entries
             (Context, Context_Control.Get_Dregion_Count (Context),
              new Context_Control.List_Of_Entries'
                 (Hidden_Entry => Symtbl_Entry,
                  Next_Hidden_Entry =>
                     Context_Control.Get_Hidden_Entries
                         (Context, Context_Control.Get_Dregion_Count (Context))
                  ));
      end if;
   end Push_Hidden_Entry;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Reset_Declarative_Regions
   --
   --  <Description>
   --
   --    This routine will reset a declarative region, that is it will
   --    recover from an error that was in a declarative region.
   --
   --=======================================================================
   procedure Reset_Declarative_Regions
       (Context : in out Context_Control.Context_Definition) is
      Current_Entry : Context_Control.Ref_List_Of_Entries;
   begin
      for I in (Ace_Natural'FIRST + 1) ..
         Context_Control.Get_Dregion_Count (Context) loop
         Current_Entry := Context_Control.Get_Hidden_Entries (Context, I);
         while (not Context_Control."=" (Current_Entry, null)) loop
            Make_Entry_Visible (Current_Entry.Hidden_Entry);
            Current_Entry := Current_Entry.Next_Hidden_Entry;
         end loop;
         -- [tbd] Free (Get_Hidden_Entries (Context, I), Entire_Substructure);
      end loop;
      Context_Control.Set_Dregion_Count (Context, Ace_Natural'FIRST);
      Reset_Leveloffset_To_Top_Level (Context);
   end Reset_Declarative_Regions;

   -- --------------------------------------------------------------------

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Pop_Context_Clause
   --
   --    Last Mod:    Wed Nov  1 10:43:33 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --    Called before adding the symtbls associated with the context
   --    clause.
   --
   --  --------------------------------------------------------------------
   procedure Pop_Context_Clause
       (Context : in out Context_Control.Context_Definition) is
   begin
      Symbol_Table_Routines.Set_Top_Of_Symtbl
          (Context, Context_Control.Get_Context_Clause_Holder (Context));
   end Pop_Context_Clause;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Push_Context_Clause
   --
   --    Last Mod:    Wed Nov  1 10:43:39 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --    Called at the end of the scope associated with a context clause.
   --
   --  --------------------------------------------------------------------
   procedure Push_Context_Clause
       (Context : in out Context_Control.Context_Definition) is
   begin
      Context_Control.Set_Context_Clause_Holder (Context,
                                                 Get_Top_Of_Symtbl (Context));
   end Push_Context_Clause;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Start_Of_Block
   --
   --    Last Mod:    Wed Nov  1 10:43:43 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Start_Of_Block
       (Context : in out Context_Control.Context_Definition) is
   begin
      Context_Control.Set_Dregion_Count
          (Context, Context_Control.Get_Dregion_Count (Context) + 1);
      Context_Control.Set_Declarative_Region
          (Context, Context_Control.Get_Dregion_Count (Context),
           Get_Top_Of_Symtbl (Context));
      Context_Control.Set_Declarative_Region_Entry
          (Context, Context_Control.Get_Dregion_Count (Context),
           Get_Top_Of_Symtbl (Context));

      Incr_Level (Context);
   end Start_Of_Block;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    End_Of_Block
   --
   --    Last Mod:    Wed Nov  1 10:43:49 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure End_Of_Block (Context : in out Context_Control.Context_Definition)
                          is
      Current_Entry : Context_Control.Ref_List_Of_Entries :=
         Context_Control.Get_Hidden_Entries
             (Context, Context_Control.Get_Dregion_Count (Context));
   begin
      Decr_Level (Context); -- restores previous offset
      Context_Control.Set_Declarative_Region_Entry
          (Context, Context_Control.Get_Dregion_Count (Context), null);

      Context_Control.Set_Top_Of_Symtbl
          (Context,
           Context_Control.Get_Declarative_Region
               (Context, Context_Control.Get_Dregion_Count (Context)));
      Context_Control.Set_Declarative_Region
          (Context, Context_Control.Get_Dregion_Count (Context), null);

      -- reset entries that were hidden w/in this declarative region
      while (not Context_Control."=" (Current_Entry, null)) loop
         Make_Entry_Visible (Current_Entry.Hidden_Entry);
         Current_Entry := Current_Entry.Next_Hidden_Entry;
      end loop;
      -- [tbd] Free (Get_Hidden_Entries (Context, Get_Dregion_Count(Context)),
      --               Entire_Substructure);

      Context_Control.Set_Dregion_Count
          (Context, Context_Control.Get_Dregion_Count (Context) - 1);
   end End_Of_Block;

end Declarative_Region_Support_Routines;
