--|$Log:	dtyped.a,v $
--|Revision 8.0  91/01/17  10:44:56  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--==========================================================================
--  <Subprogram Name>
--
--    Create_Type_Decl
--
--  <Description>
--
--    A Statement_Record/Type_Decl is created for the identifier and its
--    associated type and discriminant information.
--
--    The discriminant node is optional.
--
--==========================================================================
with Derived_Subprogram_Support;

separate (Declaration_Routines)

function Create_Type_Decl
    (Context           : in Context_Control.Context_Definition;
     Id_Node           : in Ref_Statement_Database;
     Discriminant_Node : in Ref_Statement_Database;
     Type_Defn_Node    : in Ref_Statement_Database) return
      Ref_Statement_Database is

   -- Notes:
   --   Discriminants are not currently implemented; Discriminant_Node
   --   ignored

   Stmt_Db_Node      : Ref_Statement_Database;
   Type_Symtbl_Entry : Ref_Symbol_Table_Entry;

begin

   -- Verify that the type identifier does not clash with an existing
   -- identifier
   if Name_Clash (Context, Get_Symbolic_Name (Id_Node)) then
      --E user error: redefinition of existing identifier
      Show_Error (2001, Fatal, Convert_To_String (Get_Symbolic_Name (Id_Node)));
   end if;

   Type_Symtbl_Entry := Get_Symbol_Table_Entry (Type_Defn_Node);

   -- Assign the identifier to the type definition symbol table entry
   Set_Symbolic_Name (Type_Symtbl_Entry, Get_Symbolic_Name (Id_Node));

   -- Set the static nesting level of the type
   Set_Level (Type_Symtbl_Entry, Get_Current_Level (Context));

   -- If discriminant exists, then verify that the discriminant is
   -- appropriate for the type definition
   -- DISCRIMINANTS NOT YET IMPLEMENTED
   -- [TBD]

   -- If discriminant exists, incorporate it into the type declaration
   -- DISCRIMINANTS NOT YET IMPLEMENTED
   -- [TBD]

   -- For enumeration types, additional checking and linking of enumeration
   -- literal to enumeration type name must be performed.
   -- Also, create a parameterless function call for each literal
   -- and add to list of subpgms that need to be derived when needed. [oei]
   --
   if Get_Entry_Kind (Type_Symtbl_Entry) = Type_Entry and then
      Is_Type_An_Enumeration_Type (Type_Symtbl_Entry) then
      -- tasks to do:
      --    - check type clash of enum literal and enum type names
      --    - set enum_type of each literal to this base type
      declare
         Enum_Type_Entry, Tmp_Entry : Ref_Symbol_Table_Entry;
         Enum_Type_Name             : Ref_String;
      begin
         Enum_Type_Entry := Type_Symtbl_Entry;
         Enum_Type_Name := Get_Symbolic_Name (Enum_Type_Entry);
         Tmp_Entry := Get_First_Literal (Enum_Type_Entry);
         while (not Is_Empty (Tmp_Entry)) loop

            -- Check whether the enumeration type name matches the name
            -- of the enumeration literal
            if Are_Symbolic_Names_Equal (Get_Symbolic_Name (Tmp_Entry),
                                         Enum_Type_Name) then
               --E user error: enum type name matches enum literal
               Show_Error (2003, Fatal, Convert_To_String (Enum_Type_Name));
            end if;

            -- Set each enumeration literal so that it points back to the
            -- symbol table entry for the enumeration type
            Set_Type (Tmp_Entry, Enum_Type_Entry);

            Set_Return_Type (Get_Function_Entry (Tmp_Entry),
                             Get_Type (Tmp_Entry));

            -- add function entry to list of subpgms to derive when needed
            Derived_Subprogram_Support.Add_Subprogram_Entry_To_List
                (Get_Function_Entry (Tmp_Entry), Type_Symtbl_Entry);

            Tmp_Entry := Get_Successor (Tmp_Entry);
         end loop;
      end;
   end if;

   Stmt_Db_Node := Create_Node (Type_Declaration);

   -- Associate help info with the symbol table entry
   Set_Help (Type_Symtbl_Entry);
   Set_Symbol_Table_Entry (Stmt_Db_Node, Type_Symtbl_Entry);

   return (Stmt_Db_Node);

end Create_Type_Decl;
