--|$Log:	eattr.a,v $
--|Revision 8.0  91/01/17  10:45:02  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Subunits for Package - Statement_Evaluator
--
---------------------------------------------------------------
--
--
-- Abstract
--
--  The following routines are separate routines for
--  Statement_Evaluator.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------

--==========================================================================
--  <Subprogram Name>
--
--    Evaluate_Attribute
--
--  <Description>
--
--    This routine will apply a specific attribute to an expression.
--
--==========================================================================
with Text_Io;

with Node_Set_Routines;
use Node_Set_Routines;

with Symbol_Table_Routines;
use Symbol_Table_Routines;

with Statement_Evaluator;

with String_Constants;
use String_Constants;

separate (
   Statement_Evaluator.Evaluate_Statement.Evaluate_Individual_Statement.
      Evaluate_Expression)

--==========================================================================
--  <Subprogram Name>
--
--    Evaluate_Attribute
--
--  <Description>
--
--    Evaluates the given attribute node structure
--==========================================================================
function Evaluate_Attribute
    (Node : in Ref_Statement_Database) return Ref_Statement_Database is

   --   Result    : Ref_Statement_Database :=
   --      Evaluate_Expression (Get_Attribute_Prefix (Node));

   Attr_Name        : Ref_String;
   Attr_Node        : Ref_Statement_Database;
   Attr_Prefix_Node : Ref_Symbol_Table_Entry :=
      Get_Symbol_Table_Entry (Get_Attribute_Prefix (Node));
   Prefix_Type      : Basic_Type_Definitions :=
      Get_Type_Definition (Attr_Prefix_Node);

begin

   Attr_Node := Get_Attribute (Node);

   -- get attribute's name...
   if Check_Expr_Node (Attr_Node) = Left_Parenthesis_Operator then
      declare
         Attr_Param_Node : Ref_Statement_Database :=
            Get_List_Of_Items (Attr_Node);
      begin
         -- attribute has parameter, evaluate it while where there
         Attr_Name := Get_Symbolic_Name (Get_Object (Attr_Node));
         Set_Value
             (Attr_Param_Node,
              Copy_Value (Get_Value (Evaluate_Expression (Attr_Param_Node))));
      end;
   else
      Attr_Name :=
         Node_Get_Routines.Get_Symbolic_Name
             (Node_Get_Routines.Get_Attribute (Node));
   end if;

   if Are_Symbolic_Names_Equal (Attr_Name, Ref_First_String) then
      if Is_Type_An_Integer_Type (Get_Root_Type (Attr_Prefix_Node)) then
         if Are_Symbolic_Names_Equal (Get_Symbolic_Name (Attr_Prefix_Node),
                                      Ref_Character_String) then
            Set_Value (Node, Create_Storage_For_Character (Character'FIRST));
         else
            Set_Value (Node, Create_Storage_For_Integer (Ace_Integer'FIRST));
         end if;
      elsif Is_Type_An_Enumeration_Type (Get_Root_Type (Attr_Prefix_Node)) then
         Set_Value
             (Node,
              Create_Storage_For_Integer
                  (Get_Literal_Value
                       (Get_First_Literal (Get_Root_Type (Attr_Prefix_Node)))));
      else
         Show_Error (2000, Fatal, "Evaluate_Attribute");
      end if;
   elsif Are_Symbolic_Names_Equal (Attr_Name, Ref_Last_String) then
      if Is_Type_An_Integer_Type (Get_Root_Type (Attr_Prefix_Node)) then
         if Are_Symbolic_Names_Equal (Get_Symbolic_Name (Attr_Prefix_Node),
                                      Ref_Character_String) then
            Set_Value (Node, Create_Storage_For_Character (Character'LAST));
         else
            Set_Value (Node, Create_Storage_For_Integer (Ace_Integer'LAST));
         end if;
      elsif Is_Type_An_Enumeration_Type (Get_Root_Type (Attr_Prefix_Node)) then
         Set_Value
             (Node,
              Create_Storage_For_Integer
                  (Get_Literal_Value
                       (Get_Last_Literal (Get_Root_Type (Attr_Prefix_Node)))));
      else
         Show_Error (2000, Fatal, "Evaluate_Attribute");
      end if;

   elsif Are_Symbolic_Names_Equal (Attr_Name, Ref_Pos_String) then
      if Are_Symbolic_Names_Equal
             (Get_Symbolic_Name (Get_Root_Type (Attr_Prefix_Node)),
              Ref_Character_String) then
         Set_Value
             (Node,
              Create_Storage_For_Integer
                  (Character'POS (
                      Get_Character_Value
                          (Get_Value (Get_List_Of_Items (Attr_Node))))));
      else
         Set_Value (Node,
                    Copy_Value (Get_Value (Get_List_Of_Items (Attr_Node))));
      end if;

   elsif Are_Symbolic_Names_Equal (Attr_Name, Ref_Succ_String) then
      if Are_Symbolic_Names_Equal
             (Get_Symbolic_Name (Get_Root_Type (Attr_Prefix_Node)),
              Ref_Character_String) then
         Set_Value
             (Node,
              Create_Storage_For_Character
                  (Character'SUCC (
                      (Get_Character_Value
                          (Get_Value (Get_List_Of_Items (Attr_Node)))))));
      elsif Is_Type_An_Enumeration_Type (Get_Root_Type (Attr_Prefix_Node)) then
         Set_Value
             (Node,
              Create_Storage_For_Integer
                  (Get_Literal_Value
                       (Get_Successor
                            (Get_Enum_Node
                                 (Get_Root_Type (Attr_Prefix_Node),
                                  Get_Integer_Value
                                      (Get_Value
                                           (Get_List_Of_Items (Attr_Node)))))))
              );
      elsif Is_Type_An_Integer_Type (Get_Root_Type (Attr_Prefix_Node)) then
         Set_Value
             (Node,
              Create_Storage_For_Integer
                  (Get_Integer_Value
                       (Get_Value (Get_List_Of_Items (Attr_Node))) + 1));
      end if;

   elsif Are_Symbolic_Names_Equal (Attr_Name, Ref_Val_String) then
      if Are_Symbolic_Names_Equal
             (Get_Symbolic_Name (Get_Root_Type (Attr_Prefix_Node)),
              Ref_Character_String) then
         Set_Value
             (Node,
              Create_Storage_For_Character
                  (Character'VAL (
                      Get_Integer_Value
                          (Get_Value (Get_List_Of_Items (Attr_Node))))));
      elsif Is_Type_An_Integer_Type (Get_Root_Type (Attr_Prefix_Node)) then
         Set_Value
             (Node,
              Create_Storage_For_Integer
                  (Get_Integer_Value
                       (Get_Value (Get_List_Of_Items (Attr_Node)))));
      elsif Is_Type_An_Enumeration_Type (Get_Root_Type (Attr_Prefix_Node)) then
         Set_Value
             (Node,
              Create_Storage_For_Integer
                  (Get_Integer_Value
                       (Get_Value (Get_List_Of_Items (Attr_Node)))));
      end if;

   elsif Are_Symbolic_Names_Equal (Attr_Name, Ref_Pred_String) then
      if Are_Symbolic_Names_Equal
             (Get_Symbolic_Name (Get_Root_Type (Attr_Prefix_Node)),
              Ref_Character_String) then
         Set_Value
             (Node,
              Create_Storage_For_Character
                  (Character'PRED (
                      (Get_Character_Value
                          (Get_Value (Get_List_Of_Items (Attr_Node)))))));

      elsif Is_Type_An_Enumeration_Type (Get_Root_Type (Attr_Prefix_Node)) then
         Set_Value
             (Node,
              Create_Storage_For_Integer
                  (Get_Literal_Value
                       (Get_Predecessor
                            (Get_Enum_Node
                                 (Get_Root_Type (Attr_Prefix_Node),
                                  Get_Integer_Value
                                      (Get_Value
                                           (Get_List_Of_Items (Attr_Node)))))))
              );
      elsif Is_Type_An_Integer_Type (Get_Root_Type (Attr_Prefix_Node)) then
         Set_Value
             (Node,
              Create_Storage_For_Integer
                  (Get_Integer_Value
                       (Get_Value (Get_List_Of_Items (Attr_Node))) - 1));
      end if;
   elsif Are_Symbolic_Names_Equal (Attr_Name, Ref_Image_String) then
      if Are_Symbolic_Names_Equal
             (Get_Symbolic_Name (Get_Root_Type (Attr_Prefix_Node)),
              Ref_Character_String) then
         Set_Value
             (Node,
              Create_Storage_For_String
                  (Convert_To_Ref_String
                       ((Character'IMAGE (
                           Get_Character_Value
                               (Get_Value (Get_List_Of_Items (Attr_Node))))))));
      elsif Is_Type_An_Enumeration_Type (Get_Root_Type (Attr_Prefix_Node)) then
         Set_Value
             (Node,
              Create_Storage_For_String
                  (Convert_To_Ref_String
                       (Upper
                            (Convert_To_String
                                 (Get_Symbolic_Name
                                      (Get_Enum_Node
                                           (Get_Root_Type
                                                (Get_Type
                                                     (Get_List_Of_Items
                                                          (Attr_Node))),
                                            Get_Integer_Value
                                                (Get_Value
                                                     (Get_List_Of_Items
                                                          (Attr_Node))))))))));
      elsif Is_Type_An_Integer_Type (Get_Root_Type (Attr_Prefix_Node)) then
         Set_Value
             (Node,
              Create_Storage_For_String
                  (Convert_To_Ref_String
                       ((Ace_Integer'IMAGE (
                           Get_Integer_Value
                               (Get_Value (Get_List_Of_Items (Attr_Node))))))));
      end if;

   end if;
   return Node;

end Evaluate_Attribute;
