--|$Log:	eblock.a,v $
--|Revision 8.0  91/01/17  10:45:03  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--  -----------------------------------------------------------------------
--  <Subprogram Name>
--    Evaluate_Block
--
--    Last Mod:    Wed Nov  1 10:46:31 1989
--    Author:      William P Loftus <wpl@huey>
--
--  <Description>
--
--    Executes a block statement pointed to by Node
--
--  -----------------------------------------------------------------------

separate (Statement_Evaluator.Evaluate_Statement.Evaluate_Individual_Statement)

procedure Evaluate_Block (Node : in Ref_Statement_Database) is

   -- [tbd] change this to a linked list structure -- oei
   Actuals_Interim_Storage : array (Ace_Max_Levels) of Ref_Data_Storage :=
      (others => null);
   Count                   : Ace_Max_Levels := Ace_Max_Levels'FIRST;
   Level : Ace_Natural := Get_Level (Get_Top_Of_Symtbl (Context)) + 1;
   Previous_Scope          : Ref_Statement_Database;

begin

   -- First save the old scope
   Previous_Scope := Context_Control.Get_Enclosing_Scope (Context);
   -- Set the enclosing scope to the subprogram so we can handle an
   -- exception when it occurs.
   Context_Control.Set_Enclosing_Scope (Context, Node);

   -- prepare data storage stack for new environment
   Context_Control.Push_Display_For_Level (Context, Level);
   Context_Control.Push_Last_Storage_Location_Pointer (Context);

   Set_Display_For_Level
       (Context, Level,
        Context_Control.Get_Last_Storage_Location_Pointer (Context) + 1);

   -- keep pointer to current top of symbol table
   Context_Control.Push_Top_Of_Symtbl
       (Context, Context_Control.Get_Top_Of_Symtbl (Context));

   Context_Control.Increment_Call_Level (Context);

   Evaluate_Statement (Context, Get_Block_Declarations (Node));
   Evaluate_Statement (Context, Get_Head_Of_Statements_List (Node));

   if Context_Control.Get_Call_Level (Context) > 0 then
      Context_Control.Decrement_Call_Level (Context);
   end if;

   -- restore original pointer to symbol table
   -- (pop local variables off the front of the symbol table)
   Context_Control.Pop_Top_Of_Symtbl (Context);

   -- restore previous environment of data storage stack
   Context_Control.Pop_Display_For_Level (Context, Level);
   Context_Control.Pop_Last_Storage_Location_Pointer (Context);

   -- Deallocate_Locals (Get_Block_Declarations (Node));

   Context_Control.Set_Enclosing_Scope (Context, Previous_Scope);

exception

   when Return_Statement_Encountered     =>

      if Context_Control.Get_Call_Level (Context) > 0 then
         Context_Control.Decrement_Call_Level (Context);
      end if;

      Deallocate_Locals (Context, Get_Block_Declarations (Node));

      -- restore original pointer to symbol table
      -- (pop parameters and local variables off
      -- the front of the symbol table)
      Context_Control.Pop_Top_Of_Symtbl (Context);

      -- restore previous environment of data storage stack
      Context_Control.Pop_Display_For_Level (Context, Level);
      Context_Control.Pop_Last_Storage_Location_Pointer
          (Context);

      Context_Control.Set_Enclosing_Scope (Context, Previous_Scope);

      if Context_Control.Is_In_Subprogram_Call (Context) then
         raise ;
      else
         Show_Error (3020, Fatal, "Blocks");
         raise User_Error;
      end if;

   when others  =>

      if Context_Control.Get_Call_Level (Context) > 0 then
         Context_Control.Decrement_Call_Level (Context);
      end if;

      Deallocate_Locals (Context, Get_Block_Declarations (Node));

      -- restore original pointer to symbol table
      -- (pop parameters and local variables off
      -- the front of the symbol table)
      Context_Control.Pop_Top_Of_Symtbl (Context);

      -- restore previous environment of data storage stack

      Context_Control.Pop_Display_For_Level (Context, Level);
      Context_Control.Pop_Last_Storage_Location_Pointer
          (Context);

      Context_Control.Set_Enclosing_Scope (Context, Previous_Scope);

      raise ;

end Evaluate_Block;
