--|$Log:	ebuilt.x.a,v $
--|Revision 8.0  91/01/17  10:45:30  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Subunit for Package - Statement_Evaluator.Evaluate_Subprogram_Call
--
---------------------------------------------------------------
--
--
-- Abstract
--
--  The following routine is a separate unit for
--  Statement_Evaluator.Evaluate_Subprogram_Call.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------

--==========================================================================
-- <Subprogram Name>
--
--    Evaluate_Built_In_Subprogram
--
-- <Description>
--
--    This procedure will evaluate built in subprograms for all instance
--    numbers.  In essence, this is where the bodies for builtin subprograms are
--    located.  Most bodies for built in subprograms are not physically located
--    within this package.  For these subprograms, the parameters are extracted
--    from the ACE statement database node structure for function
--    invocation/procedure call, and are then passed to the built in
--    subprograms.
--
-- <Called By>
--
--    Statement_Evaluator
--
--==========================================================================
with Node_Set_Routines;
use Node_Set_Routines;

with Node_Get_Routines;
use Node_Get_Routines;

with Symbol_Table_Routines;
use Symbol_Table_Routines;

with Text_Io;
use Text_Io;

with Ooe;
use Ooe;

with String_Constants;
use String_Constants;

with System_Dependent_Routines;
use System_Dependent_Routines;

with Literals;
use Literals;

with Dump_Routines;
use Dump_Routines;

with Ace_File_System;

with Ace_Text_Io;

with Ace_Calendar;

with Ace_Program_Unit;

with Ace_Standard;

with Directory_Objects;

with Ace_Adt;

with Window_Objects;
use Window_Objects;

with X_Window_Adt;
use X_Window_Adt;

with Debugger;
use Debugger;

with Misc;

with Context_Control;

with System;

with Ace;

with Unchecked_Conversion;

with Expdname;

with Key_Bindings;

with Ace_Command_Line_Input;

with Intrinsics;
with Hp_Widgets;
with Ace_X_Windows;
with Ace_Widget_Package;
with Ace_Hp_Widgets;

separate (
   Statement_Evaluator.Evaluate_Statement.Evaluate_Individual_Statement.
      Evaluate_Subprogram_Call)

-- ----------
--
-- Information for Built-In Subprograms Authors:
--
--    - Formal_Parameter object below is the head of the formal parameters
--      list (if any, Is_Empty is true on object if none)
--    - to obtain subsequent parameters, call Get_Next_Parameter on preceding
--      formal parameter
--    - use appropriate Storage Management Routines to Get/Set values
--      (coerce types as needed) for parameters of mode IN and/or OUT
--
-- ----------

procedure Evaluate_Built_In_Subprogram
    (Symtbl_Entry : in Ref_Symbol_Table_Entry) is

   Subprogram_Name  : Ref_String := Get_Symbolic_Name (Symtbl_Entry);

   Formal_Parameter : Ref_Symbol_Table_Entry :=
      Get_Head_Of_Parameters_List (Symtbl_Entry);


   function Get_Upper_Bound_Of_Actual
       (Parameter_Pos : in Ace_Natural) return Integer is
      Actual_Argument : Ref_Statement_Database;
      Upper_Bound     : Ref_Statement_Database;
   begin
      Actual_Argument := Get_Parameter_List (Node);
      for I in 1 .. Parameter_Pos - 1 loop
         Actual_Argument := Get_Next_List_Item (Actual_Argument);
      end loop;
      Upper_Bound := Get_Upper_Bound (Get_Symbol_Table_Entry
      	      	      	(Get_Expression (Actual_Argument)));
      Upper_Bound := Evaluate_Expression (Upper_Bound);
      if Check_Expr_Node (Upper_Bound) = Constant_Node then
         return Integer (Get_Integer_Value (Get_Value (Upper_Bound)));
      else
         return Integer(Get_Integer_Value (Fetch_Value (Context,
	       	   Get_Symbol_Table_Entry (Upper_Bound))));
      end if;
   End Get_Upper_Bound_Of_Actual;

   function Get_Lower_Bound_Of_Actual
       (Parameter_Pos : in Ace_Natural) return Integer is
      Actual_Argument : Ref_Statement_Database;
      Lower_Bound     : Ref_Statement_Database;
   begin
      Actual_Argument := Get_Parameter_List (Node);
      for I in 1 .. Parameter_Pos - 1 loop
         Actual_Argument := Get_Next_List_Item (Actual_Argument);
      end loop;
      Lower_Bound := Get_Lower_Bound (Get_Symbol_Table_Entry
      	      	      	(Get_Expression (Actual_Argument)));
      Lower_Bound := Evaluate_Expression (Lower_Bound);
      if Check_Expr_Node (Lower_Bound) = Constant_Node then
        return Integer (Get_Integer_Value (Get_Value (Lower_Bound)));
      else
        return Integer (Get_Integer_Value (Fetch_Value (Context, 
        	        Get_Symbol_Table_Entry (Lower_Bound))));
      end if;
   end Get_Lower_Bound_Of_Actual;


begin
   -- Each built-in subprogram is identified by a unique positive integer.
   -- This value is stored in the symbol table entry associated with the
   -- subprogram and is used as the case selector value.

   case Get_Built_In_Instance_Number (Symtbl_Entry) is

      when 1    =>
         --   procedure Eval (Filename : in String);
         Eval
             (Convert_To_String
                  (Get_String_Value (Fetch_Value (Context, Formal_Parameter))));

      when 2    =>
         --   procedure Host (Command           : in Host_Os_String;
         --                   Command_Arguments : in Host_Os_String := "";
         --                   How               : Method_of_Execution
         --                                                   := Foreground);

         System_Dependent_Routines.System
             (Convert_To_String
                  (Get_String_Value (Fetch_Value (Context, Formal_Parameter))),
              Convert_To_String
                  (Get_String_Value
                       (Fetch_Value (Context,
                                     Get_Next_Parameter (Formal_Parameter)))),
              Ace_File_System.Execution_Mode'VAL (
                 Get_Integer_Value
                     (Fetch_Value
                          (Context,
                           Get_Next_Parameter
                               (Get_Next_Parameter (Formal_Parameter))))));

         -- =====================================
         -- >>> START of ACE Text_Io routines <<<
         -- =====================================

      when 9    =>
         --   procedure Create  (File : in out File_Type;
         --                      Mode : in File_Mode := Out_File;
         --                      Name : in String    := "";
         --                      Form : in String    := "");

         declare
            File           : Ace_Text_Io.File_Type := 1; -- don't care value
            File_Parameter : Ref_Symbol_Table_Entry;
            Mode_Parameter : Ref_Symbol_Table_Entry;
            Name_Parameter : Ref_Symbol_Table_Entry;
            Form_Parameter : Ref_Symbol_Table_Entry;

         begin
            -- Acquire parameter references
            File_Parameter := Formal_Parameter;
            Mode_Parameter := Get_Next_Parameter (File_Parameter);
            Name_Parameter := Get_Next_Parameter (Mode_Parameter);
            Form_Parameter := Get_Next_Parameter (Name_Parameter);

            Ace_Text_Io.Create
                (File,
                 Ace_Text_Io.File_Mode'VAL (
                    Get_Integer_Value (Fetch_Value (Context, Mode_Parameter))),
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Form_Parameter)))
                 );

            -- Store the File object
            Store_Value (Context, File_Parameter,
                         Create_Storage_For_Integer (Ace_Integer (File)));

         end;

      when 10   =>
         --   procedure Open (File : in out File_Type;
         --                   Mode : in File_Mode;
         --                   Name : in String;
         --                   Form : in String := "");

         declare
            File           : Ace_Text_Io.File_Type := 1; -- don't care value
            File_Parameter : Ref_Symbol_Table_Entry;
            Mode_Parameter : Ref_Symbol_Table_Entry;
            Name_Parameter : Ref_Symbol_Table_Entry;
            Form_Parameter : Ref_Symbol_Table_Entry;

         begin
            -- Acquire parameter references
            File_Parameter := Formal_Parameter;
            Mode_Parameter := Get_Next_Parameter (File_Parameter);
            Name_Parameter := Get_Next_Parameter (Mode_Parameter);
            Form_Parameter := Get_Next_Parameter (Name_Parameter);

            Ace_Text_Io.Open
                (File,
                 Ace_Text_Io.File_Mode'VAL (
                    Get_Integer_Value (Fetch_Value (Context, Mode_Parameter))),
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Form_Parameter)))
                 );

            -- Store the File object
            Store_Value (Context, File_Parameter,
                         Create_Storage_For_Integer (Ace_Integer (File)));

         end;

      when 11   =>
         --   function Is_Open (File : in File_Type) return Boolean;

         declare
            Result : Boolean;
         begin
            Result :=
               Ace_Text_Io.Is_Open
                   (Ace_Text_Io.File_Type
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))));

            -- Store the result
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Boolean'POS (Result)));
         end;

      when 12   =>
         --   function Mode (File : in File_Type) return File_Mode;

         declare
            Result : Ace_Text_Io.File_Mode;
         begin
            Result :=
               Ace_Text_Io.Mode
                   (Ace_Text_Io.File_Type
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))));

            -- Store the result
            Store_Value
                (Context, Symtbl_Entry,
                 Create_Storage_For_Integer
                     (Ace_Text_Io.File_Mode'POS (Result)));
         end;

      when 13   =>
         --   procedure Close (File : in out File_Type);

         declare
            File : Ace_Text_Io.File_Type;
         begin
            File :=
               Ace_Text_Io.File_Type
                   (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))
                    );
            Ace_Text_Io.Close (File);

            -- Store the result
            Store_Value (Context, Formal_Parameter,
                         Create_Storage_For_Integer (Ace_Integer (File)));
         end;

      when 14   =>
         --   procedure Delete (File : in out File_Type);

         declare
            File : Ace_Text_Io.File_Type;
         begin
            File :=
               Ace_Text_Io.File_Type
                   (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))
                    );
            Ace_Text_Io.Delete (File);

            -- Store the result
            Store_Value (Context, Formal_Parameter,
                         Create_Storage_For_Integer (Ace_Integer (File)));
         end;

      when 15   =>
         --   procedure Get (File : in File_Type;
         --                  Item : out Character);

         declare
            Item : Character;
         begin
            Ace_Text_Io.Get
                (Ace_Text_Io.File_Type
                     (Get_Integer_Value
                          (Fetch_Value (Context, Formal_Parameter))), Item);
            Store_Value (Context, Get_Next_Parameter (Formal_Parameter),
                         Create_Storage_For_Character (Item));
         end;

      when 16   =>
         --   procedure Get (Item : out Character);

         declare
            Item : Character;
         begin
            Ace_Text_Io.Get (Item);
            Store_Value (Context, Get_Next_Parameter (Formal_Parameter),
                         Create_Storage_For_Character (Item));
         end;

      when 17   =>
         --   procedure Get (File : in File_Type;
         --                  Item : out String);

         declare
            Item : String (Get_Lower_Bound_Of_Actual (2) ..
                            Get_Upper_Bound_Of_Actual (2));
         begin
            Ace_Text_Io.Get
                (Ace_Text_Io.File_Type
                     (Get_Integer_Value
                          (Fetch_Value (Context, Formal_Parameter))), Item);
            Store_Value
                (Context, Get_Next_Parameter (Formal_Parameter),
                 Create_Storage_For_String (Convert_To_Ref_String (Item)));
         end;

      when 18   =>
         --   procedure Get (Item : out String);
         declare
            Item : String (Get_Lower_Bound_Of_Actual (1) ..
                             Get_Upper_Bound_Of_Actual(1));
         begin
            Ace_Text_Io.Get (Item);
            Store_Value
                (Context, Formal_Parameter,
                 Create_Storage_For_String (Convert_To_Ref_String (Item)));
         end;

      when 19   =>
         --   procedure Get_Line (File : in File_Type;
         --                       Item : out String;
         --                       Last : out Natural);

         declare
            Item : String (Get_Lower_Bound_Of_Actual (2) ..
                             Get_Upper_Bound_Of_Actual(2));
            Last : Natural;
         begin
            Ace_Text_Io.Get_Line
                (Ace_Text_Io.File_Type
                     (Get_Integer_Value
                          (Fetch_Value (Context, Formal_Parameter))), Item,
                 Last);

            -- Store the Item object
            Formal_Parameter := Get_Next_Parameter (Formal_Parameter);
            Store_Value
                (Context, Formal_Parameter,
                 Create_Storage_For_String
                     (Convert_To_Ref_String (Item (1 .. Last))));

            -- Store the Last object
            Formal_Parameter := Get_Next_Parameter (Formal_Parameter);
            Store_Value (Context, Formal_Parameter,
                         Create_Storage_For_Integer (Ace_Integer (Last)));
         end;

      when 20   =>
         --   procedure Get_Line (Item : out String;
         --                       Last : out Natural);

         declare
            Item : String (Get_Lower_Bound_Of_Actual (1) ..
                             Get_Upper_Bound_Of_Actual(1));
            Last : Natural;
         begin
            Ace_Text_Io.Get_Line (Item, Last);

            -- Store the Item object
            Store_Value
                (Context, Formal_Parameter,
                 Create_Storage_For_String
                     (Convert_To_Ref_String (Item (1 .. Last))));

            -- Store the Last object
            Formal_Parameter := Get_Next_Parameter (Formal_Parameter);
            Store_Value (Context, Formal_Parameter,
                         Create_Storage_For_Integer (Ace_Integer (Last)));
         end;

      when 21   =>
         --   procedure Skip_Line (File    : in File_Type;
         --                        Spacing : in Positive_Count := 1);

         Ace_Text_Io.Skip_Line
             (Ace_Text_Io.File_Type
                  (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))),
              Ace_Text_Io.Positive_Count
                  (Get_Integer_Value
                       (Fetch_Value (Context,
                                     Get_Next_Parameter (Formal_Parameter)))));

      when 22   =>
         --   procedure Skip_Line (Spacing : in Positive_Count := 1);

         Ace_Text_Io.Skip_Line
             (Ace_Text_Io.Positive_Count
                  (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter)))
              );

      when 23   =>
         --   procedure Put (File : in File_Type;
         --                  Item : in Character);

         Ace_Text_Io.Put
             (Ace_Text_Io.File_Type
                  (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))),
              Get_Character_Value
                  (Fetch_Value (Context, Get_Next_Parameter (Formal_Parameter))
                   ));

      when 24   =>
         --   procedure Put (Item : in Character);

         Ace_Text_Io.Put (Get_Character_Value (Fetch_Value (Context,
                                                            Formal_Parameter)));

      when 25   =>
         --   procedure Put (File : in File_Type;
         --                  Item : in String);

         Ace_Text_Io.Put
             (Ace_Text_Io.File_Type
                  (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))),
              Convert_To_String
                  (Get_String_Value
                       (Fetch_Value (Context,
                                     Get_Next_Parameter (Formal_Parameter)))));

      when 26   =>
         --   procedure Put (Item : in String);

         Ace_Text_Io.Put
             (Convert_To_String
                  (Get_String_Value (Fetch_Value (Context, Formal_Parameter))));

      when 27   =>
         --   procedure Put_Line (File : in File_Type;
         --                       Item : in String);

         Ace_Text_Io.Put_Line
             (Ace_Text_Io.File_Type
                  (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))),
              Convert_To_String
                  (Get_String_Value
                       (Fetch_Value (Context,
                                     Get_Next_Parameter (Formal_Parameter)))));

      when 28   =>
         --   procedure Put_Line (Item : out String);

         Ace_Text_Io.Put_Line
             (Convert_To_String
                  (Get_String_Value (Fetch_Value (Context, Formal_Parameter))));

      when 29   =>
         --   procedure New_Line (File    : in File_Type;
         --                       Spacing : in Positive_Count := 1);

         Ace_Text_Io.New_Line
             (Ace_Text_Io.File_Type
                  (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))),
              Ace_Text_Io.Positive_Count
                  (Get_Integer_Value
                       (Fetch_Value (Context,
                                     Get_Next_Parameter (Formal_Parameter)))));

      when 30   =>
         --   procedure New_Line (Spacing : in Positive_Count := 1);

         Ace_Text_Io.New_Line
             (Ace_Text_Io.Positive_Count
                  (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter)))
              );

      when 31   =>
         --   function End_Of_Line (File : in File_Type) return Boolean;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Boolean'POS (
                      Ace_Text_Io.End_Of_Line
                          (Ace_Text_Io.File_Type
                               (Get_Integer_Value
                                    (Fetch_Value (Context, Formal_Parameter))))
                      )));

      when 32   =>
         --   function End_Of_Line return Boolean;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Boolean'POS (Ace_Text_Io.End_Of_Line)));

      when 33   =>
         --   function End_Of_File (File : in File_Type) return Boolean;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Boolean'POS (
                      Ace_Text_Io.End_Of_File
                          (Ace_Text_Io.File_Type
                               (Get_Integer_Value
                                    (Fetch_Value (Context, Formal_Parameter))))
                      )));

      when 34   =>
         --   function End_Of_File return Boolean;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Boolean'POS (Ace_Text_Io.End_Of_File)));

         -- ===================================
         -- >>> END of ACE Text_Io routines <<<
         -- ===================================

         -- ==============================================
         -- >>> START of File System built-in routines <<<
         -- ==============================================

      when 35   =>
         --   procedure Get (File : in File_Type;
         --                  Item : out Integer;
         --                  Width : in Field :=0);

         declare
            Item           : Integer;
            Item_Parameter : Ref_Symbol_Table_Entry;
         begin
            Item_Parameter := Get_Next_Parameter (Formal_Parameter);
            Ace_Text_Io.Integer_Io.Get
                (Ace_Text_Io.File_Type
                     (Get_Integer_Value
                          (Fetch_Value (Context, Formal_Parameter))), Item,
                 Ace_Text_Io.Field
                     (Get_Integer_Value
                          (Fetch_Value (Context,
                                        Get_Next_Parameter (Item_Parameter)))));
            Store_Value (Context, Item_Parameter,
                         Create_Storage_For_Integer (Ace_Integer (Item)));
         end;

      when 36   =>
         --   procedure Get (Item  : out Integer;
         --                  Width : in Field :=0);

         declare
            Item           : Integer;
            Item_Parameter : Ref_Symbol_Table_Entry;
         begin
            Item_Parameter := Formal_Parameter;
            Ace_Text_Io.Integer_Io.Get
                (Item,
                 Ace_Text_Io.Field
                     (Get_Integer_Value
                          (Fetch_Value (Context,
                                        Get_Next_Parameter (Item_Parameter)))));
            Store_Value (Context, Item_Parameter,
                         Create_Storage_For_Integer (Ace_Integer (Item)));
         end;

      when 37   =>
         --   procedure Get (From : in String;
         --                  Item : out Integer;
         --                  Last : out Positive);

         declare
            Item           : Integer;
            Last           : Positive := 1; -- don't care value
            Item_Parameter : Ref_Symbol_Table_Entry;
            Last_Parameter : Ref_Symbol_Table_Entry;
         begin
            Item_Parameter := Get_Next_Parameter (Formal_Parameter);
            Last_Parameter := Get_Next_Parameter (Item_Parameter);
            Ace_Text_Io.Integer_Io.Get
                (Convert_To_String
                     (Get_String_Value
                          (Fetch_Value (Context, Formal_Parameter))), Item,
                 Last);

            Store_Value (Context, Item_Parameter,
                         Create_Storage_For_Integer (Ace_Integer (Item)));

            Store_Value (Context, Last_Parameter,
                         Create_Storage_For_Integer (Ace_Integer (Last)));
         end;

      when 38   =>
         --   procedure Put (File  : in File_Type;
         --                  Item  : in Integer;
         --                  Width : in Field       := Default_Width;
         --                  Base  : in Number_Base := Default_Base);

         declare
            File_Parameter  : Ref_Symbol_Table_Entry := Formal_Parameter;
            Item_Parameter  : Ref_Symbol_Table_Entry;
            Width_Parameter : Ref_Symbol_Table_Entry;
            Base_Parameter  : Ref_Symbol_Table_Entry;
         begin
            Item_Parameter := Get_Next_Parameter (File_Parameter);
            Width_Parameter := Get_Next_Parameter (Item_Parameter);
            Base_Parameter := Get_Next_Parameter (Width_Parameter);
            Ace_Text_Io.Integer_Io.Put
                (Ace_Text_Io.File_Type
                     (Get_Integer_Value (Fetch_Value (Context, File_Parameter))
                      ),
                 Integer (Get_Integer_Value (Fetch_Value (Context,
                                                          Item_Parameter))),
                 Ace_Text_Io.Field
                     (Get_Integer_Value
                          (Fetch_Value (Context, Width_Parameter))),
                 Ace_Text_Io.Number_Base
                     (Get_Integer_Value (Fetch_Value (Context, Base_Parameter))
                      ));
         end;

      when 39   =>
         --   procedure Put (Item  : in Integer;
         --                  Width : in Field       := Default_Width;
         --                  Base  : in Number_Base := Default_Base);

         declare
            Item_Parameter  : Ref_Symbol_Table_Entry := Formal_Parameter;
            Width_Parameter : Ref_Symbol_Table_Entry;
            Base_Parameter  : Ref_Symbol_Table_Entry;
         begin
            Width_Parameter := Get_Next_Parameter (Item_Parameter);
            Base_Parameter := Get_Next_Parameter (Width_Parameter);
            Ace_Text_Io.Integer_Io.Put
                (Integer (Get_Integer_Value (Fetch_Value (Context,
                                                          Item_Parameter))),
                 Ace_Text_Io.Field
                     (Get_Integer_Value
                          (Fetch_Value (Context, Width_Parameter))),
                 Ace_Text_Io.Number_Base
                     (Get_Integer_Value (Fetch_Value (Context, Base_Parameter))
                      ));
         end;

      when 40   =>
         --   procedure Put (To    : out String;
         --                  Item  : in Integer;
         --                  Base  : in Number_Base := Default_Base);

         declare
            To_Parameter   : Ref_Symbol_Table_Entry := Formal_Parameter;
            Item_Parameter : Ref_Symbol_Table_Entry;
            Base_Parameter : Ref_Symbol_Table_Entry;
            To : String (Get_Lower_Bound_Of_Actual (1) ..
                             Get_Upper_Bound_Of_Actual(1));
         begin
            Item_Parameter := Get_Next_Parameter (To_Parameter);
            Base_Parameter := Get_Next_Parameter (Item_Parameter);
            Ace_Text_Io.Integer_Io.Put
                (To,
                 Integer (Get_Integer_Value (Fetch_Value (Context,
                                                          Item_Parameter))),
                 Ace_Text_Io.Number_Base
                     (Get_Integer_Value (Fetch_Value (Context, Base_Parameter))
                      ));
            Store_Value
                (Context, To_Parameter,
                 Create_Storage_For_String (Convert_To_Ref_String (To)));
         end;

      when 41   =>
         --   procedure Copy (From : in File_Object;
         --                   To   : in File_Object);

         declare
            File1 : Ace_File_System.File_Object;
            File2 : Ace_File_System.File_Object;
         begin
            File1 :=
               Ace_File_System.File_Object
                   (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))
                    );
            File2 :=
               Ace_File_System.File_Object
                   (Get_Integer_Value
                        (Fetch_Value (Context,
                                      Get_Next_Parameter (Formal_Parameter))));

            Ace_File_System.Copy (File1, File2);

            Store_Value (Context, Formal_Parameter,
                         Create_Storage_For_Integer (Ace_Integer (File1)));
            Store_Value (Context, Get_Next_Parameter (Formal_Parameter),
                         Create_Storage_For_Integer (Ace_Integer (File2)));
         end;

      when 42   =>
         --    function Exists (Obj : in File_System_String) return Boolean;

         declare
            Obj    : constant String :=
               Convert_To_String
                   (Get_String_Value (Fetch_Value (Context, Formal_Parameter)));
            Status : Boolean;
            File   : Ace_Text_Io.File_Type := 1;
         begin
            begin
               Ace_Text_Io.Open (File, Ace_Text_Io.In_File, Obj);
               Status := True;
               Ace_Text_Io.Close (File);
            exception
               when others     =>
                  Status := False;
            end;

            -- Store the result
            Store_Value
                (Context, Symtbl_Entry,
                 Create_Storage_For_Integer
                     (Ace_Integer (Boolean'POS (Status))));
         end;

      when 43   =>
         --   procedure Rename (From : in File_Object;
         --                     To   : in File_Object);

         declare
            File1 : Ace_File_System.File_Object;
            File2 : Ace_File_System.File_Object;
         begin
            File1 :=
               Ace_File_System.File_Object
                   (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))
                    );
            File2 :=
               Ace_File_System.File_Object
                   (Get_Integer_Value
                        (Fetch_Value (Context,
                                      Get_Next_Parameter (Formal_Parameter))));

            Ace_File_System.Rename (File1, File2);

            Store_Value (Context, Formal_Parameter,
                         Create_Storage_For_Integer (Ace_Integer (File1)));
            Store_Value (Context, Get_Next_Parameter (Formal_Parameter),
                         Create_Storage_For_Integer (Ace_Integer (File2)));
         end;

      when 44   =>
         --   procedure Create  (Obj  : in out File_Object;
         --                      Mode : in Object_Mode        := Out_Object;
         --                      Name : in File_System_String := "";
         --                      Form : in File_System_String := "");

         declare
            File           : Ace_File_System.File_Object := 1;
            -- don't care value
            File_Parameter : Ref_Symbol_Table_Entry;
            Mode_Parameter : Ref_Symbol_Table_Entry;
            Name_Parameter : Ref_Symbol_Table_Entry;
            Form_Parameter : Ref_Symbol_Table_Entry;

         begin
            -- Acquire parameter references
            File_Parameter := Formal_Parameter;
            Mode_Parameter := Get_Next_Parameter (File_Parameter);
            Name_Parameter := Get_Next_Parameter (Mode_Parameter);
            Form_Parameter := Get_Next_Parameter (Name_Parameter);

            Ace_File_System.Create
                (File,
                 Ace_File_System.Object_Mode'VAL (
                    Get_Integer_Value (Fetch_Value (Context, Mode_Parameter))),
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Form_Parameter)))
                 );

            -- Store the File object
            Store_Value (Context, File_Parameter,
                         Create_Storage_For_Integer (Ace_Integer (File)));

         end;

      when 45   =>
         --   procedure Open (Obj  : in out File_Object;
         --                   Mode : in Object_Mode;
         --                   Name : in File_System_String;
         --                   Form : in File_System_String := "");

         declare
            File           : Ace_File_System.File_Object := 1;
            -- don't care value
            File_Parameter : Ref_Symbol_Table_Entry;
            Mode_Parameter : Ref_Symbol_Table_Entry;
            Name_Parameter : Ref_Symbol_Table_Entry;
            Form_Parameter : Ref_Symbol_Table_Entry;

         begin
            -- Acquire parameter references
            File_Parameter := Formal_Parameter;
            Mode_Parameter := Get_Next_Parameter (File_Parameter);
            Name_Parameter := Get_Next_Parameter (Mode_Parameter);
            Form_Parameter := Get_Next_Parameter (Name_Parameter);

            Ace_File_System.Open
                (File,
                 Ace_File_System.Object_Mode'VAL (
                    Get_Integer_Value (Fetch_Value (Context, Mode_Parameter))),
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Form_Parameter)))
                 );

            -- Store the File object
            Store_Value (Context, File_Parameter,
                         Create_Storage_For_Integer (Ace_Integer (File)));

         end;

      when 46   =>
         --   procedure Close (File : in out File_Object);

         declare
            File : Ace_File_System.File_Object;
         begin
            File :=
               Ace_File_System.File_Object
                   (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))
                    );
            Ace_File_System.Close (File);

            -- Store the result
            Store_Value (Context, Formal_Parameter,
                         Create_Storage_For_Integer (Ace_Integer (File)));
         end;

      when 47   =>
         --   procedure List (Directory : in Directory_Object :=
         --                                     Current_Directory);

         Directory_Objects.List
             (Directory_Objects.Directory_Object
                  (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter)))
              );

      when 48   =>
         --   procedure Set_Directory (Directory : in Directory_Object :=
         --                                              Home_Directory);

         Directory_Objects.Set_Directory
             (Directory_Objects.Directory_Object
                  (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter)))
              );

      when 49   =>
         --   procedure List (Text : in Text_Object);

         Ace_File_System.List
             (Ace_File_System.File_Object
                  (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))),
              Get_Built_In_Instance_Number (Symtbl_Entry));

      when 50   =>
         --   procedure Edit (Text : in out Text_Object;
         --                   How  : in Method_Of_Execution := Foreground);

         declare
            File : Ace_File_System.File_Object;
         begin
            File :=
               Ace_File_System.File_Object
                   (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))
                    );

            System_Dependent_Routines.Edit
                (File,
                 Ace_File_System.Execution_Mode'VAL (
                    Get_Integer_Value
                        (Fetch_Value (Context,
                                      Get_Next_Parameter (Formal_Parameter)))));

            -- Store the result
            Store_Value (Context, Formal_Parameter,
                         Create_Storage_For_Integer (Ace_Integer (File)));
         end;

      when 51   =>
         --   procedure Compile (Prog : in Program_Text;
         --                      Form : in Host_Os_String := "");

         Ace_File_System.Compile
             (Ace_File_System.File_Object
                  (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))),
              Convert_To_String
                  (Get_String_Value
                       (Fetch_Value (Context,
                                     Get_Next_Parameter (Formal_Parameter)))));

      when 52   =>
         --   procedure Stub (From_Spec : in Spec_Program;
         --                   To_Body   : in out Body_Program);

         declare
            Object1 : Ace_File_System.File_Object :=
               Ace_File_System.File_Object
                   (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))
                    );
            Object2 : Ace_File_System.File_Object :=
               Ace_File_System.File_Object
                   (Get_Integer_Value
                        (Fetch_Value (Context,
                                      Get_Next_Parameter (Formal_Parameter))));
         begin
            Ace_File_System.Stubber (Object1, Object2);

            Store_Value (Context, Formal_Parameter,
                         Create_Storage_For_Integer (Ace_Integer (Object1)));

            Store_Value (Context, Get_Next_Parameter (Formal_Parameter),
                         Create_Storage_For_Integer (Ace_Integer (Object2)));
         end;

         -- ============================================
         -- >>> END of File System built-in routines <<<
         -- ============================================

      when 53   =>
         --  procedure Create (Ace_Package : in out Package_Type;
         --                    Mode        : in Program_Unit_Mode
         --                                            := Out_Prog_Unit;
         --                    Name        : in Interpreter_String := "";
         --                    Form        : in Interpreter_String := "");

         declare
            Package_Item      : Ace_Program_Unit.Package_Type := 1;
            --    input value is "don't care"
            Package_Parameter : Ref_Symbol_Table_Entry;
            Mode_Parameter    : Ref_Symbol_Table_Entry;
            Name_Parameter    : Ref_Symbol_Table_Entry;
            Form_Parameter    : Ref_Symbol_Table_Entry;
            Package_Name      : Ref_String;

         begin
            -- Acquire parameter references
            Package_Parameter := Formal_Parameter;
            Mode_Parameter := Get_Next_Parameter (Package_Parameter);
            Name_Parameter := Get_Next_Parameter (Mode_Parameter);
            Form_Parameter := Get_Next_Parameter (Name_Parameter);

            Package_Name := Get_String_Value (Fetch_Value (Context,
                                                           Name_Parameter));
            Literals.Ada_Id (Package_Name);

            Ace_Program_Unit.Create
                (Context, Package_Item,
                 Ace_Program_Unit.Program_Unit_Mode'VAL (
                    Get_Integer_Value (Fetch_Value (Context, Mode_Parameter))),
                 Convert_To_String (Package_Name),
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Form_Parameter)))
                 );

            -- Store the Package object
            Store_Value
                (Context, Package_Parameter,
                 Create_Storage_For_Integer (Ace_Integer (Package_Item)));

         end;

      when 54   =>
         --   procedure Open (Ace_Package : in out Package_Type;
         --                   Mode        : in Program_Unit_Mode;
         --                   Name        : in Interpreter_String;
         --                   Form        : in Interpreter_String := "");

         declare
            Package_Item      : Ace_Program_Unit.Package_Type := 1;
            --    input value is "don't care"
            Package_Parameter : Ref_Symbol_Table_Entry;
            Mode_Parameter    : Ref_Symbol_Table_Entry;
            Name_Parameter    : Ref_Symbol_Table_Entry;
            Form_Parameter    : Ref_Symbol_Table_Entry;
            Package_Name      : Ref_String;

         begin
            -- Acquire parameter references
            Package_Parameter := Formal_Parameter;
            Mode_Parameter := Get_Next_Parameter (Package_Parameter);
            Name_Parameter := Get_Next_Parameter (Mode_Parameter);
            Form_Parameter := Get_Next_Parameter (Name_Parameter);

            Package_Name := Get_String_Value (Fetch_Value (Context,
                                                           Name_Parameter));
            Literals.Ada_Id (Package_Name);

            Ace_Program_Unit.Open
                (Context, Package_Item,
                 Ace_Program_Unit.Program_Unit_Mode'VAL (
                    Get_Integer_Value (Fetch_Value (Context, Mode_Parameter))),
                 Convert_To_String (Package_Name),
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Form_Parameter)))
                 );

            -- Store the Package object
            Store_Value
                (Context, Package_Parameter,
                 Create_Storage_For_Integer (Ace_Integer (Package_Item)));

         end;

      when 55   =>
         --   procedure Open (Ace_Subprogram : in out Subprogram_Type;
         --                   Mode           : in Program_Unit_Mode;
         --                   Name           : in Interpreter_String;
         --                   Form           : in Interpreter_String := "");

         declare
            Subprogram_Item      : Ace_Program_Unit.Subprogram_Type := 1;
            --    input value is "don't care"
            Subprogram_Parameter : Ref_Symbol_Table_Entry;
            Mode_Parameter       : Ref_Symbol_Table_Entry;
            Name_Parameter       : Ref_Symbol_Table_Entry;
            Form_Parameter       : Ref_Symbol_Table_Entry;
            Subprogram_Name_1    : Ref_String;

         begin
            -- Acquire parameter references
            Subprogram_Parameter := Formal_Parameter;
            Mode_Parameter := Get_Next_Parameter (Subprogram_Parameter);
            Name_Parameter := Get_Next_Parameter (Mode_Parameter);
            Form_Parameter := Get_Next_Parameter (Name_Parameter);

            Subprogram_Name_1 :=
               Get_String_Value (Fetch_Value (Context, Name_Parameter));
            Literals.Ada_Id (Subprogram_Name_1);

            Ace_Program_Unit.Open
                (Context, Subprogram_Item,
                 Ace_Program_Unit.Program_Unit_Mode'VAL (
                    Get_Integer_Value (Fetch_Value (Context, Mode_Parameter))),
                 Convert_To_String (Subprogram_Name_1),
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Form_Parameter)))
                 );

            -- Store the Subprogram object
            Store_Value
                (Context, Subprogram_Parameter,
                 Create_Storage_For_Integer (Ace_Integer (Subprogram_Item)));

         end;

      when 56   =>
         --   function Is_Open (Ace_Package : in Package_Type) return Boolean;

         declare
            Result : Boolean;
         begin
            Result :=
               Ace_Program_Unit.Is_Open
                   (Ace_Program_Unit.Package_Type
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))));

            -- Store the result
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Boolean'POS (Result)));
         end;

      when 57   =>
         --   function Is_Open (Ace_Subprogram : in Subprogram_Type)
         --                   return Boolean;

         declare
            Result : Boolean;
         begin
            Result :=
               Ace_Program_Unit.Is_Open
                   (Ace_Program_Unit.Subprogram_Type
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))));

            -- Store the result
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Boolean'POS (Result)));
         end;

      when 58   =>
         --   function Mode (Ace_Package : in Package_Type)
         --                return Program_Unit_Mode;

         declare
            Result : Ace_Program_Unit.Program_Unit_Mode;
         begin
            Result :=
               Ace_Program_Unit.Mode
                   (Ace_Program_Unit.Package_Type
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))));

            -- Store the result
            Store_Value
                (Context, Symtbl_Entry,
                 Create_Storage_For_Integer
                     (Ace_Program_Unit.Program_Unit_Mode'POS (Result)));
         end;

      when 59   =>
         --   function Mode (Ace_Subprogram : in Subprogram_Type)
         --                return Program_Unit_Mode;

         declare
            Result : Ace_Program_Unit.Program_Unit_Mode;
         begin
            Result :=
               Ace_Program_Unit.Mode
                   (Ace_Program_Unit.Subprogram_Type
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))));

            -- Store the result
            Store_Value
                (Context, Symtbl_Entry,
                 Create_Storage_For_Integer
                     (Ace_Program_Unit.Program_Unit_Mode'POS (Result)));
         end;

      when 60   =>
         --   procedure Close (Ace_Package : in out Package_Type);

         declare
            Package_Item : Ace_Program_Unit.Package_Type;
         begin
            Package_Item :=
               Ace_Program_Unit.Package_Type
                   (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))
                    );
            Ace_Program_Unit.Close (Package_Item);

            -- Store the result
            Store_Value
                (Context, Formal_Parameter,
                 Create_Storage_For_Integer (Ace_Integer (Package_Item)));
         end;

      when 61   =>
         --   procedure Close (Ace_Subprogram : in out Subprogram_Type);

         declare
            Subprogram_Item : Ace_Program_Unit.Subprogram_Type;
         begin
            Subprogram_Item :=
               Ace_Program_Unit.Subprogram_Type
                   (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))
                    );
            Ace_Program_Unit.Close (Subprogram_Item);

            -- Store the result
            Store_Value
                (Context, Formal_Parameter,
                 Create_Storage_For_Integer (Ace_Integer (Subprogram_Item)));
         end;

      when 62   =>
         --   procedure Delete (Ace_Package : in out Package_Type);

         declare
            Package_Item : Ace_Program_Unit.Package_Type;
         begin
            Package_Item :=
               Ace_Program_Unit.Package_Type
                   (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))
                    );
            Ace_Program_Unit.Delete (Package_Item);

            -- Store the result
            Store_Value
                (Context, Formal_Parameter,
                 Create_Storage_For_Integer (Ace_Integer (Package_Item)));
         end;

      when 63   =>
         --   procedure Delete (Ace_Subprogram : in out Subprogram_Type);

         declare
            Subprogram_Item : Ace_Program_Unit.Subprogram_Type;
         begin
            Subprogram_Item :=
               Ace_Program_Unit.Subprogram_Type
                   (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))
                    );
            Ace_Program_Unit.Delete (Subprogram_Item);

            -- Store the result
            Store_Value
                (Context, Formal_Parameter,
                 Create_Storage_For_Integer (Ace_Integer (Subprogram_Item)));
         end;

      when 64   =>
         --   procedure Put (Subprogram_Item : in Subprogram_Type;
         --                  Into_Package    : in out Package_Type;
         --                  Action          : in Execution_Type;
         --                  Visibility      : in Visibility_Type);

         declare
            Subprogram_Parameter : Ref_Symbol_Table_Entry;
            Package_Parameter    : Ref_Symbol_Table_Entry;
            Action_Parameter     : Ref_Symbol_Table_Entry;
            Visibility_Parameter : Ref_Symbol_Table_Entry;

            Subprogram_Item      : Ace_Program_Unit.Subprogram_Type;
            Package_Item         : Ace_Program_Unit.Package_Type;
         begin
            -- Extract parameter positions
            Subprogram_Parameter := Formal_Parameter;
            Package_Parameter := Get_Next_Parameter (Subprogram_Parameter);
            Action_Parameter := Get_Next_Parameter (Package_Parameter);
            Visibility_Parameter := Get_Next_Parameter (Action_Parameter);

            Subprogram_Item :=
               Ace_Program_Unit.Subprogram_Type
                   (Get_Integer_Value (Fetch_Value (Context,
                                                    Subprogram_Parameter)));
            Package_Item :=
               Ace_Program_Unit.Package_Type
                   (Get_Integer_Value
                        (Fetch_Value (Context, Package_Parameter)));

            -- Invoke Put procedure
            Ace_Program_Unit.Put
                (Context, Subprogram_Item, Package_Item,
                 Ace_Program_Unit.Action_Type'VAL (
                    Get_Integer_Value (Fetch_Value (Context, Action_Parameter))
                    ),
                 Ace_Program_Unit.Visibility_Type'VAL (
                    Get_Integer_Value (Fetch_Value (Context,
                                                    Visibility_Parameter))));
         end;

      when 65   =>
         --   procedure List (Ace_Package : in Package_Type;
         --                   List_Method : in List_Method := Specification);

         declare
            Package_Parameter     : Ref_Symbol_Table_Entry;
            List_Method_Parameter : Ref_Symbol_Table_Entry;

            Package_Item          : Ace_Program_Unit.Package_Type;
            List_Method_Item      : Dump_Routines.List_Mode;
         begin
            -- Extract parameter positions
            Package_Parameter := Formal_Parameter;
            List_Method_Parameter := Get_Next_Parameter (Package_Parameter);

            Package_Item :=
               Ace_Program_Unit.Package_Type
                   (Get_Integer_Value
                        (Fetch_Value (Context, Package_Parameter)));
            List_Method_Item  :=
               Dump_Routines.List_Mode'VAL (
                  Get_Integer_Value (Fetch_Value (Context,
                                                  List_Method_Parameter)));
            Ace_Program_Unit.List (Package_Item, List_Method_Item);
         end;

      when 66   =>
         --   procedure List (Ace_Subprogram : in Subprogram_Type;
         --                   List_Method    : in List_Method := Both);

         declare
            Subprogram_Parameter  : Ref_Symbol_Table_Entry;
            List_Method_Parameter : Ref_Symbol_Table_Entry;

            Subprogram_Item       : Ace_Program_Unit.Subprogram_Type;
            List_Method_Item      : Dump_Routines.List_Mode;
         begin
            -- Extract parameter positions
            Subprogram_Parameter := Formal_Parameter;
            List_Method_Parameter := Get_Next_Parameter (Subprogram_Parameter);

            Subprogram_Item :=
               Ace_Program_Unit.Subprogram_Type
                   (Get_Integer_Value (Fetch_Value (Context,
                                                    Subprogram_Parameter)));
            List_Method_Item     :=
               Dump_Routines.List_Mode'VAL (
                  Get_Integer_Value (Fetch_Value (Context,
                                                  List_Method_Parameter)));
            Ace_Program_Unit.List (Subprogram_Item, List_Method_Item);
         end;

      when 67   =>
         --   procedure Delete (Name : in Interpreter_String;
         --                     Form : in Interpreter_String := "");

         declare
            Object_Item : Ref_String :=
               Get_String_Value (Fetch_Value (Context, Formal_Parameter));
            Form        : constant String :=
               Convert_To_String
                   (Get_String_Value
                        (Fetch_Value (Context,
                                      Get_Next_Parameter (Formal_Parameter))));
         begin
            Ada_Id (Object_Item);
            Ace_Standard.Delete (Context, Object_Item, Form);
         end;

      when 68   =>
         --   procedure Deuse (Package_Name : in Interpreter_String;
         --                    Form         : Interpreter_String := "");

         declare
            Package_Item : Ref_String :=
               Get_String_Value (Fetch_Value (Context, Formal_Parameter));
            Form         : constant String :=
               Convert_To_String
                   (Get_String_Value
                        (Fetch_Value (Context,
                                      Get_Next_Parameter (Formal_Parameter))));
         begin
            Ada_Id (Package_Item);
            Ace_Standard.Defuse (Context, Package_Item, Lower (Form));
         end;

      when 69   =>
         Show_Error (3012, Warning, "Undelete"); -- for Undelete;

      when 70   =>
         --   function Name (Ace_Package : in Package_Type)
         --                return Interpreter_String;

         declare
            Package_Item : Ace_Program_Unit.Package_Type;
         begin
            Package_Item :=
               Ace_Program_Unit.Package_Type
                   (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))
                    );
            Store_Value
                (Context, Symtbl_Entry,
                 Create_Storage_For_String
                     (Convert_To_Ref_String
                          (Ace_Program_Unit.Name (Package_Item))));
         end;

      when 71   =>
         --   function Name (Ace_Subprogram : in Subprogram_Type)
         --                return Interpreter_String;

         declare
            Subprogram_Item : Ace_Program_Unit.Subprogram_Type;
         begin
            Subprogram_Item :=
               Ace_Program_Unit.Subprogram_Type
                   (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))
                    );
            Store_Value
                (Context, Symtbl_Entry,
                 Create_Storage_For_String
                     (Convert_To_Ref_String
                          (Ace_Program_Unit.Name (Subprogram_Item))));
         end;

      when 72   =>
         --   function Form (Ace_Package : in Package_Type)
         --                return Interpreter_String;

         declare
            Package_Item : Ace_Program_Unit.Package_Type;
         begin
            Package_Item :=
               Ace_Program_Unit.Package_Type
                   (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))
                    );
            Store_Value
                (Context, Symtbl_Entry,
                 Create_Storage_For_String
                     (Convert_To_Ref_String
                          (Ace_Program_Unit.Form (Package_Item))));
         end;

      when 73   =>
         --   function Form (Ace_Subprogram : in Subprogram_Type)
         --                return Interpreter_String;

         declare
            Subprogram_Item : Ace_Program_Unit.Subprogram_Type;
         begin
            Subprogram_Item :=
               Ace_Program_Unit.Subprogram_Type
                   (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))
                    );
            Store_Value
                (Context, Symtbl_Entry,
                 Create_Storage_For_String
                     (Convert_To_Ref_String
                          (Ace_Program_Unit.Form (Subprogram_Item))));
         end;

      when 100  =>
         --   procedure Set_Input (File : in File_Type);

         Ace_Text_Io.Set_Input
             (Ace_Text_Io.File_Type
                  (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter)))
              );

      when 101  =>
         --   procedure Set_Output (File : in File_Type);

         Ace_Text_Io.Set_Output
             (Ace_Text_Io.File_Type
                  (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter)))
              );

      when 102  =>
         --   function Standard_Input return File_Type;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Integer ((Ace_Text_Io.Standard_Input))));

      when 103  =>
         --   function Standard_Output return File_Type;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Integer ((Ace_Text_Io.Standard_Output))));

      when 104  =>
         --   function Current_Input return File_Type;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Integer ((Ace_Text_Io.Current_Input))));

      when 105  =>
         --   function Current_Output return File_Type;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Integer ((Ace_Text_Io.Current_Output))));

      when 106  =>
         --   procedure Set_Ace_Input (File : in File_Type);

         Ace_Text_Io.Set_Ace_Input
             (Ace_Text_Io.File_Type
                  (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter)))
              );

      when 107  =>
         --   procedure Set_Ace_Output (File : in File_Type);

         Ace_Text_Io.Set_Ace_Output
             (Ace_Text_Io.File_Type
                  (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter)))
              );

      when 108  =>
         --   function Standard_Ace_Input return File_Type;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Integer ((Ace_Text_Io.Standard_Ace_Input))));

      when 109  =>
         --   function Standard_Ace_Output return File_Type;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Integer ((Ace_Text_Io.Standard_Ace_Output))));

      when 110  =>
         --   function Current_Ace_Input return File_Type;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Integer ((Ace_Text_Io.Current_Ace_Input))));

      when 111  =>
         --   function Current_Ace_Output return File_Type;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Integer ((Ace_Text_Io.Current_Ace_Output))));

      when 112  =>
         --   function Name (File : in File_Type) return String;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_String
                  (Convert_To_Ref_String
                       (Ace_Text_Io.Name
                            (Ace_Text_Io.File_Type
                                 (Get_Integer_Value
                                      (Fetch_Value (Context, Formal_Parameter))
                                  )))));

      when 200  =>
         --   procedure Set_Line_Length (File : in File_Type;
         --                              To   : in Count);

         Ace_Text_Io.Set_Line_Length
             (Ace_Text_Io.File_Type
                  (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))),
              Ace_Text_Io.Count
                  (Get_Integer_Value
                       (Fetch_Value (Context,
                                     Get_Next_Parameter (Formal_Parameter)))));

      when 201  =>
         --   procedure Set_Line_Length (To : in Count);

         Ace_Text_Io.Set_Line_Length
             (Ace_Text_Io.Count
                  (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter)))
              );

      when 202  =>
         --   procedure Set_Page_Length (File : in File_Type;
         --                              To   : in Count);

         Ace_Text_Io.Set_Page_Length
             (Ace_Text_Io.File_Type
                  (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))),
              Ace_Text_Io.Count
                  (Get_Integer_Value
                       (Fetch_Value (Context,
                                     Get_Next_Parameter (Formal_Parameter)))));
      when 203  =>
         --   procedure Set_Page_Length (To : in Count);

         Ace_Text_Io.Set_Page_Length
             (Ace_Text_Io.Count
                  (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter)))
              );

      when 204  =>
         --   function Line_Length (File : in File_Type) return Count;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Integer
                       ((Ace_Text_Io.Line_Length
                            (Ace_Text_Io.File_Type
                                 (Get_Integer_Value
                                      (Fetch_Value (Context, Formal_Parameter))
                                  ))))));

      when 205  =>
         --   function Line_Length return Count;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Integer (Ace_Text_Io.Line_Length)));

      when 206  =>
         --   function Page_Length (File : in File_Type) return Count;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Integer
                       ((Ace_Text_Io.Page_Length
                            (Ace_Text_Io.File_Type
                                 (Get_Integer_Value
                                      (Fetch_Value (Context, Formal_Parameter))
                                  ))))));

      when 207  =>
         --   function Page_Length return Count;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Integer (Ace_Text_Io.Page_Length)));

      when 208  =>
         --   procedure New_Page (File : in File_Type);

         Ace_Text_Io.New_Page
             (Ace_Text_Io.File_Type
                  (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter)))
              );

      when 209  =>
         --   procedure New_Page;

         Ace_Text_Io.New_Page;

      when 210  =>
         --   procedure Skip_Page (File : in File_Type);

         Ace_Text_Io.Skip_Page
             (Ace_Text_Io.File_Type
                  (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter)))
              );

      when 211  =>
         --   procedure Skip_Page;

         Ace_Text_Io.Skip_Page;

      when 212  =>
         --   function End_Of_Page (File : in File_Type) return Boolean;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Boolean'POS (
                      (Ace_Text_Io.End_Of_Page
                          (Ace_Text_Io.File_Type
                               (Get_Integer_Value
                                    (Fetch_Value (Context, Formal_Parameter))))
                      ))));

      when 213  =>
         --   function End_Of_Page return Boolean;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Boolean'POS ((Ace_Text_Io.End_Of_Page))));

      when 214  =>
         --   procedure Set_Col (File : in File_Type;
         --                      To   : in Positive_Count);

         Ace_Text_Io.Set_Col
             (Ace_Text_Io.File_Type
                  (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))),
              Ace_Text_Io.Positive_Count
                  (Get_Integer_Value
                       (Fetch_Value (Context,
                                     Get_Next_Parameter (Formal_Parameter)))));

      when 215  =>
         --   procedure Set_Col (To : in Positive_Count);

         Ace_Text_Io.Set_Col
             (Ace_Text_Io.Positive_Count
                  (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter)))
              );

      when 216  =>
         --   procedure Set_Line (File : in File_Type;
         --                       To   : in Positive_Count);

         Ace_Text_Io.Set_Line
             (Ace_Text_Io.File_Type
                  (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))),
              Ace_Text_Io.Positive_Count
                  (Get_Integer_Value
                       (Fetch_Value (Context,
                                     Get_Next_Parameter (Formal_Parameter)))));

      when 217  =>
         --   procedure Set_Line (To : in Positive_Count);

         Ace_Text_Io.Set_Line
             (Ace_Text_Io.Positive_Count
                  (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter)))
              );

      when 218  =>
         --   function Col (File : in File_Type) return Positive_Count;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Integer
                       (Ace_Text_Io.Col
                            (Ace_Text_Io.File_Type
                                 (Get_Integer_Value
                                      (Fetch_Value (Context, Formal_Parameter))
                                  )))));

      when 219  =>
         --   function Col return Positive;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer (Ace_Integer (Ace_Text_Io.Col)));

      when 220  =>
         --   function Line (File : in File_Type) return Positive_Count;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Integer
                       (Ace_Text_Io.Line
                            (Ace_Text_Io.File_Type
                                 (Get_Integer_Value
                                      (Fetch_Value (Context, Formal_Parameter))
                                  )))));

      when 221  =>
         --   function Line return Positive_Count;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer (Ace_Integer (Ace_Text_Io.Line)));

      when 222  =>
         --   function Page (File : in File_Type) return Positive_Count;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Integer
                       (Ace_Text_Io.Page
                            (Ace_Text_Io.File_Type
                                 (Get_Integer_Value
                                      (Fetch_Value (Context, Formal_Parameter))
                                  )))));

      when 223  =>
         --   function Page return Positive_Count;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer (Ace_Integer (Ace_Text_Io.Page)));

      when 224  =>
         --   function Name (File : in File_Type) return String;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_String
                  (Convert_To_Ref_String
                       (Ace_Text_Io.Name
                            (Ace_Text_Io.File_Type
                                 (Get_Integer_Value
                                      (Fetch_Value (Context, Formal_Parameter))
                                  )))));

      when 225  =>
         --   function Form (File : in File_Type) return String;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_String
                  (Convert_To_Ref_String
                       ((Ace_Text_Io.Form
                            (Ace_Text_Io.File_Type
                                 (Get_Integer_Value
                                      (Fetch_Value (Context, Formal_Parameter))
                                  ))))));

      when 226  =>
         --   procedure Reset (File : in out File_Type;
         --                    Mode : in File_Mode);

         declare
            File : Ace_Text_Io.File_Type :=
               Ace_Text_Io.File_Type
                   (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))
                    );
         begin
            Ace_Text_Io.Reset
                (File,
                 Ace_Text_Io.File_Mode'VAL (
                    Get_Integer_Value
                        (Fetch_Value (Context,
                                      Get_Next_Parameter (Formal_Parameter)))));

            Store_Value (Context, Formal_Parameter,
                         Create_Storage_For_Integer (Ace_Integer (File)));
         end;

      when 227  =>
         --   procedure Reset (File : in out File_Type);

         declare
            File : Ace_Text_Io.File_Type :=
               Ace_Text_Io.File_Type
                   (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))
                    );
         begin
            Ace_Text_Io.Reset (File);

            Store_Value (Context, Formal_Parameter,
                         Create_Storage_For_Integer (Ace_Integer (File)));
         end;

      when 228  =>
         --   function Home_Directory return Directory_Object;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer (Ace_Integer (Home_Directory)));

      when 229  =>
         --   function Current_Directory return Directory_Object;
         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer (Ace_Integer (Current_Directory)));

      when 230  =>
         --   procedure Create  (Directory  : in out Directory_Object;
         --                      Name : in String    := "";
         --                      Form : in String    := "");

         declare
            File           : Directory_Objects.Directory_Object :=
               Directory_Objects.Directory_Object'FIRST;
            File_Parameter : Ref_Symbol_Table_Entry;
            Name_Parameter : Ref_Symbol_Table_Entry;
            Form_Parameter : Ref_Symbol_Table_Entry;

         begin
            -- Acquire parameter references
            File_Parameter := Formal_Parameter;
            Name_Parameter := Get_Next_Parameter (File_Parameter);
            Form_Parameter := Get_Next_Parameter (Name_Parameter);

            Directory_Objects.Create
                (File,
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Form_Parameter)))
                 );

            -- Store the File object
            Store_Value (Context, File_Parameter,
                         Create_Storage_For_Integer (Ace_Integer (File)));
         end;

      when 231  =>
         --   procedure Open (Directory : in out Directory_Object;
         --                   Name      : in String;
         --                   Form      : in String := "");

         declare
            File           : Directory_Objects.Directory_Object :=
               Directory_Objects.Directory_Object'FIRST;
            -- don't care value
            File_Parameter : Ref_Symbol_Table_Entry;
            Name_Parameter : Ref_Symbol_Table_Entry;
            Form_Parameter : Ref_Symbol_Table_Entry;

         begin
            -- Acquire parameter references
            File_Parameter := Formal_Parameter;
            Name_Parameter := Get_Next_Parameter (File_Parameter);
            Form_Parameter := Get_Next_Parameter (Name_Parameter);

            Directory_Objects.Open
                (File,
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Form_Parameter)))
                 );

            -- Store the File object
            Store_Value (Context, File_Parameter,
                         Create_Storage_For_Integer (Ace_Integer (File)));
         end;

      when 232  =>
         --   procedure Close (Directory : in out Directory_Object);

         declare
            File : Directory_Objects.Directory_Object :=
               Directory_Objects.Directory_Object'FIRST;
            -- don't care value;

         begin
            File :=
               Directory_Objects.Directory_Object
                   (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))
                    );
            Directory_Objects.Close (File);

            -- Store the result
            Store_Value (Context, Formal_Parameter,
                         Create_Storage_For_Integer (Ace_Integer (File)));
         end;

      when 233  =>
         --   function Is_Open (File : in Directory_Object) return Boolean;

         declare
            Result : Boolean;
         begin
            Result :=
               Directory_Objects.Is_Open
                   (Directory_Objects.Directory_Object
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))));

            -- Store the result
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Boolean'POS (Result)));
         end;

      when 234  =>
         --   function Name (File : in Directory_Object) return String;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_String
                  (Convert_To_Ref_String
                       (Directory_Objects.Name
                            (Directory_Objects.Directory_Object
                                 (Get_Integer_Value
                                      (Fetch_Value (Context, Formal_Parameter))
                                  )))));

      when 235  =>
         --   function Form (File : in Directory_Object) return String;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_String
                  (Convert_To_Ref_String
                       (Directory_Objects.Form
                            (Directory_Objects.Directory_Object
                                 (Get_Integer_Value
                                      (Fetch_Value (Context, Formal_Parameter))
                                  )))));

      when 500  =>
         --   procedure Compile (Str   : in String;
         --                      Tree  : in out Ace_Statement_Database;
         --                      Error : out Boolean);

         declare
            function Convert is new Unchecked_Conversion
                (Ace_Adt.Ace_Statement_Database, Ace_Integer);
            Str   : constant String :=
               Convert_To_String
                   (Get_String_Value (Fetch_Value (Context, Formal_Parameter)));
            Tree  : Ace_Adt.Ace_Statement_Database;
            Error : Boolean := False;
         begin
            Ace_Adt.Compile (Context, Str, Tree, Error);
            Store_Value (Context, Get_Next_Parameter (Formal_Parameter),
                         Create_Storage_For_Integer (Convert (Tree)));
            Store_Value
                (Context,
                 Get_Next_Parameter (Get_Next_Parameter (Formal_Parameter)),
                 Create_Storage_For_Integer (Boolean'POS (Error)));
         end;

      when 501  =>
         --   procedure Compile (File  : in File_Type;
         --                      Tree  : in out Ace_Statement_Database;
         --                      Error : out Boolean);

         declare
            function Convert is new Unchecked_Conversion
                (Ace_Adt.Ace_Statement_Database, Ace_Integer);
            File  : Ace_Text_Io.File_Type :=
               Ace_Text_Io.File_Type
                   (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))
                    );
            Tree  : Ace_Adt.Ace_Statement_Database;
            Error : Boolean := False;
         begin
            Ace_Adt.Compile (Context, File, Tree, Error);
            Store_Value (Context, Get_Next_Parameter (Formal_Parameter),
                         Create_Storage_For_Integer ((Convert (Tree))));
            Store_Value
                (Context,
                 Get_Next_Parameter (Get_Next_Parameter (Formal_Parameter)),
                 Create_Storage_For_Integer (Boolean'POS (Error)));
         end;

      when 502  =>
         --   procedure Delete (Tree : in Ace_Statement_Database);

         declare

            function Convert is new Unchecked_Conversion
                (Ace_Integer, Ace_Adt.Ace_Statement_Database);

            Tree : Ace_Adt.Ace_Statement_Database :=
               Convert (Get_Integer_Value (Fetch_Value (Context,
                                                        Formal_Parameter)));

         begin
            Ace_Adt.Delete (Tree);
         end;

      when 503  =>
         --   procedure Interpret (Tree  : in Ace_Statement_Database;
         --                        Error : out Boolean);

         declare

            function Convert is new Unchecked_Conversion
                (Ace_Integer, Ace_Adt.Ace_Statement_Database);

            Tree  : Ace_Adt.Ace_Statement_Database :=
               Convert (Get_Integer_Value (Fetch_Value (Context,
                                                        Formal_Parameter)));

            Error : Boolean := False;
         begin
            Ace_Adt.Interpret (Context, Tree, Error);
            Store_Value (Context, Get_Next_Parameter (Formal_Parameter),
                         Create_Storage_For_Integer (Boolean'POS (Error)));
         end;

      when 504  =>
         --   procedure Interpret (Str   : in String;
         --                        Error : out Boolean);

         declare
            Str   : constant String :=
               Convert_To_String
                   (Get_String_Value (Fetch_Value (Context, Formal_Parameter)));
            Error : Boolean := False;
         begin
            Ace_Adt.Interpret (Context, Str, Error);
            Store_Value (Context, Get_Next_Parameter (Formal_Parameter),
                         Create_Storage_For_Integer (Boolean'POS (Error)));
         end;

      when 505  =>
         --   procedure Interpret (File  : in File_Type;
         --                        Error : out Boolean);

         declare
            File  : Ace_Text_Io.File_Type :=
               Ace_Text_Io.File_Type
                   (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))
                    );
            Error : Boolean := False;
         begin
            Ace_Adt.Interpret (Context, File, Error);
            Store_Value (Context, Get_Next_Parameter (Formal_Parameter),
                         Create_Storage_For_Integer (Boolean'POS (Error)));
         end;

      when 506  =>
         --   procedure Interpret_File (File_Name   : in String);

         declare
            File_Name : constant String :=
               Convert_To_String
                   (Get_String_Value (Fetch_Value (Context, Formal_Parameter)));
         begin
            Ace_Adt.Interpret_File (Context, File_Name);
         end;

      when 600  =>
         --    procedure Help ( Name : in Interpreter_String := "help"
         --                     Form : in Interpreter_String := "");

         declare
            Name            : Ref_String :=
               Get_String_Value (Fetch_Value (Context, Formal_Parameter));
            Ref             : Ref_Symbol_Table_Entry;
            Form            : Ref_Symbol_Table_Entry :=
               Get_Next_Parameter (Formal_Parameter);
            Ref_Form_Str    : Ref_String :=
               Get_String_Value (Fetch_Value (Context, Form));
            Found_Something : Boolean := False;

            Enclosing_Pkg   : Ref_Symbol_Table_Entry;

            Form_Str : constant String := Convert_To_String (Ref_Form_Str);

            use Symtbl_Entry_Get_Routines;
            use Node_Get_Routines;

            procedure Dump_Info (Symtbl_Entry : Ref_Symbol_Table_Entry) is
            begin
               Dump_Routines.Dump_Enclosing_Pkgs (Symtbl_Entry, Current_Output);
               Dump_Routines.Dump_Help_Info
                   (Symtbl_Entry, True, Current_Output);
               Dump_Routines.Dump_Symtbl (Symtbl_Entry, Current_Output,
                                          Dump_Help => False);

            end Dump_Info;

            procedure Dump_If_Form_Matches (Ref      : Ref_Symbol_Table_Entry;
                                            Name     : Ref_String;
                                            Form_Str : String) is
            begin
               if Are_Symbolic_Names_Equal (Get_Symbolic_Name (Ref), Name)
                   and then Expdname.Do_Forms_Match (Ref, Form_Str) then
                  Dump_Info (Ref);
                  Found_Something := True;
               end if;
            end Dump_If_Form_Matches;

         begin
            Ada_Id (Name);
            Ada_Id (Ref_Form_Str);

            Expdname.Handle_Expanded_Name (Context, Name, Enclosing_Pkg);
            if Is_Empty (Enclosing_Pkg) then
               if Convert_To_String (Ref_Form_Str) /= "" then

                  Ref := Find_Matching_Name (Context, Name);
                  if Is_Empty (Ref) then
                     raise Name_Error;
                  else
                     loop
                        Dump_If_Form_Matches (Ref, Name, Form_Str);
                        Ref := Find_Next_Matching_Name (Ref);
                        exit  when Is_Empty (Ref);
                     end loop;

                  end if;
               else
                  Ref := Find_Symbol_Table_Entry (Context, Name);
                  if Is_Empty (Ref) then
                     raise Name_Error;
                  else
                     Dump_Info (Ref);
                     Found_Something := True;
                  end if;
               end if;
            else -- was an expanded name
               if Get_Entry_Kind (Enclosing_Pkg) = Package_Entry then
                  declare
                     Last_Stmt      : Ref_Statement_Database;
                     Curr_Decl_Stmt : Ref_Statement_Database :=
                        Get_Package_Spec_Decls (Enclosing_Pkg);
                  begin
                     Expdname.Find_Matching_Name
                         (Context, Enclosing_Pkg, Name, Form_Str, Ref,
                          Last_Stmt);
                     while not Is_Empty (Ref) loop
                        Dump_If_Form_Matches (Ref, Name, Form_Str);
                        Expdname.Find_Next_Matching_Name
                            (Context, Curr_Decl_Stmt, Name, Form_Str, Ref);
                     end loop;

                  end;

               else
                  Show_Error
                      (4206, Fatal,
                       Convert_To_String (Get_Symbolic_Name (Enclosing_Pkg)));
                  null;
               end if;
            end if;
            if (not Found_Something) then
               raise Name_Error;
            end if;
         exception
            when Constraint_Error   =>
               Show_Error
                   (3018, Fatal,
                    Convert_To_String (Get_String_Value (Fetch_Value (Context,
                                                                      Form))));
         end;

      when 601  =>
         -- procedure List_Objs (Obj_Type  : Obj_Declarations := Any;
         --                      Specifier : Interpreter_String := "");
         declare
            Specifier : Ref_String :=
               Get_String_Value
                   (Fetch_Value
                        (Context, Get_Next_Parameter (Formal_Parameter)));
         begin
            Ada_Id (Specifier);
            Dump_Routines.Dump_Objs
                (Context,
                 Dump_Routines.Obj_Declarations'VAL (
                    Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))
                    ), Specifier, Current_Output);
         end;

      when 710  =>
         -- function Clock return Time;

         declare
            Result : Ace_Calendar.Time;
         begin
            Result := Ace_Calendar.Clock;

            -- store the result
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Ace_Integer (Result)));
         end;

      when 711  =>
         -- function Year (Date : Time) return Year_Number;

         declare
            Result : Ace_Calendar.Year_Number;
         begin
            Result :=
               Ace_Calendar.Year
                   (Ace_Calendar.Time
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))));

            -- store the result
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Ace_Integer (Result)));
         end;

      when 712  =>
         -- function Month (Date : Time) return Month_Number;

         declare
            Result : Ace_Calendar.Month_Number;
         begin
            Result :=
               Ace_Calendar.Month
                   (Ace_Calendar.Time
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))));

            -- store the result
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Ace_Integer (Result)));
         end;

      when 713  =>
         -- function Day (Date : Time) return Day_Number;

         declare
            Result : Ace_Calendar.Day_Number;
         begin
            Result :=
               Ace_Calendar.Day
                   (Ace_Calendar.Time
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))));

            -- store the result
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Ace_Integer (Result)));
         end;

      when 714  =>
         -- function Seconds (Date : Time) return Day_Duration;

         declare
            Result : Ace_Calendar.Day_Duration;
         begin
            Result :=
               Ace_Calendar.Seconds
                   (Ace_Calendar.Time
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))));

            -- store the result
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Ace_Integer (Result)));
         end;

      when 715  =>
         --   procedure Split (Date    : in Time;
         --                    Year    : out Year_Number;
         --                    Month   : out Month_Number;
         --                    Day     : out Day_Number;
         --                    Seconds : out Day_Duration);

         declare
            Year    : Ace_Calendar.Year_Number;
            Month   : Ace_Calendar.Month_Number;
            Day     : Ace_Calendar.Day_Number;
            Seconds : Ace_Calendar.Day_Duration;
         begin
            Ace_Calendar.Split
                (Ace_Calendar.Time
                     (Get_Integer_Value
                          (Fetch_Value (Context, Formal_Parameter))), Year,
                 Month, Day, Seconds);

            -- store the result
            Formal_Parameter := Get_Next_Parameter (Formal_Parameter);
            Store_Value (Context, Formal_Parameter,
                         Create_Storage_For_Integer (Ace_Integer (Year)));
            Formal_Parameter := Get_Next_Parameter (Formal_Parameter);
            Store_Value (Context, Formal_Parameter,
                         Create_Storage_For_Integer (Ace_Integer (Month)));
            Formal_Parameter := Get_Next_Parameter (Formal_Parameter);
            Store_Value (Context, Formal_Parameter,
                         Create_Storage_For_Integer (Ace_Integer (Day)));
            Formal_Parameter := Get_Next_Parameter (Formal_Parameter);
            Store_Value (Context, Formal_Parameter,
                         Create_Storage_For_Integer (Ace_Integer (Seconds)));
         end;

      when 716  =>
         --   function Time_Of (Year    : Year_Number;
         --                     Month   : Month_Number;
         --                     Day     : Day_Number;
         --                     Seconds : Day_Duration := 0) return Time;

         declare
            Result  : Ace_Calendar.Time;
            Year    : Ref_Symbol_Table_Entry := Formal_Parameter;
            Month   : Ref_Symbol_Table_Entry := Get_Next_Parameter (Year);
            Day     : Ref_Symbol_Table_Entry := Get_Next_Parameter (Month);
            Seconds : Ref_Symbol_Table_Entry := Get_Next_Parameter (Day);
         begin
            Result :=
               Ace_Calendar.Time_Of
                   (Ace_Calendar.Year_Number
                        (Get_Integer_Value (Fetch_Value (Context, Year))),
                    Ace_Calendar.Month_Number
                        (Get_Integer_Value (Fetch_Value (Context, Month))),
                    Ace_Calendar.Day_Number
                        (Get_Integer_Value (Fetch_Value (Context, Day))),
                    Ace_Calendar.Day_Duration
                        (Get_Integer_Value (Fetch_Value (Context, Seconds))));
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Ace_Integer (Result)));
         end;

      when 717  =>
         --   function "+" (Left  : Time;
         --                 Right : Duration) return Time;
         declare
            Result : Ace_Calendar.Time;
         begin
            Result :=
               Ace_Calendar."+"
                   (Ace_Calendar.Time
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))),
                    Ace_Calendar.Ace_Duration
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))));
            -- store the result
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Ace_Integer (Result)));
         end;

      when 718  =>
         --   function "+" (Left  : Duration;
         --                 Right : Time) return Time;
         null;
         declare
            Result : Ace_Calendar.Time;
         begin
            Result :=
               Ace_Calendar."+"
                   (Ace_Calendar.Ace_Duration
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))),
                    Ace_Calendar.Time
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))));
            -- store the result
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Ace_Integer (Result)));
         end;

      when 719  =>
         --   function "-" (Left  : Time;
         --                 Right : Duration) return Time;
         null;
         declare
            Result : Ace_Calendar.Time;
         begin
            Result :=
               Ace_Calendar."+"
                   (Ace_Calendar.Time
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))),
                    Ace_Calendar.Ace_Duration
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))));
            -- store the result
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Ace_Integer (Result)));
         end;

      when 720  =>
         --   function "-" (Left  : Time;
         --                 Right : Time) return Duration;

         declare
            Result : Ace_Calendar.Ace_Duration;
         begin
            Result :=
               Ace_Calendar."-"
                   (Ace_Calendar.Time
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))),
                    Ace_Calendar.Time
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))));
            -- store the result
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Ace_Integer (Result)));
         end;

      when 721  =>
         --   function "<" (Left, Right : Time) return Boolean;

         declare
            Result : Boolean;
         begin
            Result :=
               Ace_Calendar."<"
                   (Ace_Calendar.Time
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))),
                    Ace_Calendar.Time
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))));
            -- store the result
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Boolean'POS (Result)));
         end;

      when 722  =>
         --   function "<=" (Left, Right : Time) return Boolean;

         declare
            Result : Boolean;
         begin
            Result :=
               Ace_Calendar."<="
                   (Ace_Calendar.Time
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))),
                    Ace_Calendar.Time
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))));
            -- store the result
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Boolean'POS (Result)));
         end;

      when 723  =>
         --   function ">" (Left, Right : Time) return Boolean;

         declare
            Result : Boolean;
         begin
            Result :=
               Ace_Calendar.">"
                   (Ace_Calendar.Time
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))),
                    Ace_Calendar.Time
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))));
            -- store the result
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Boolean'POS (Result)));
         end;

      when 724  =>
         --   function ">=" (Left, Right : Time) return Boolean;

         declare
            Result : Boolean;
         begin
            Result :=
               Ace_Calendar.">="
                   (Ace_Calendar.Time
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))),
                    Ace_Calendar.Time
                        (Get_Integer_Value (Fetch_Value (Context,
                                                         Formal_Parameter))));
            -- store the result
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Boolean'POS (Result)));
         end;

         -- beginning of string routines
      when 800  =>
         -- function Slice (Str       : in String;
         --                 Start_Pos : in Integer;
         --                 Stop_Pos  : in Integer) return String is
         declare

            Str_Ref       : Ref_Symbol_Table_Entry := Formal_Parameter;
            Start_Pos_Ref : Ref_Symbol_Table_Entry :=
               Get_Next_Parameter (Formal_Parameter);
            Stop_Pos_Ref  : Ref_Symbol_Table_Entry :=
               Get_Next_Parameter (Start_Pos_Ref);

            Str           : constant String :=
               Convert_To_String (Get_String_Value (Fetch_Value (Context,
                                                                 Str_Ref)));
            Start_Pos     : constant Integer :=
               Integer
                   (Get_Integer_Value (Fetch_Value (Context, Start_Pos_Ref)));
            Stop_Pos      : constant Integer :=
               Integer
                   (Get_Integer_Value (Fetch_Value (Context, Stop_Pos_Ref)));

         begin

            Store_Value
                (Context, Symtbl_Entry,
                 Create_Storage_For_String
                     (Convert_To_Ref_String
                          (Str ((Str'FIRST + Start_Pos - 1) ..
                                (Str'FIRST + Stop_Pos - 1)))));
         end;

      when 801  =>
         -- function Slice (Str : in String;
         --                 Pos : in Integer) return Character is
         declare

            Str_Ref : Ref_Symbol_Table_Entry := Formal_Parameter;
            Pos_Ref : Ref_Symbol_Table_Entry :=
               Get_Next_Parameter (Formal_Parameter);

            Str     : constant String :=
               Convert_To_String (Get_String_Value (Fetch_Value (Context,
                                                                 Str_Ref)));
            Pos     : constant Integer :=
               Integer (Get_Integer_Value (Fetch_Value (Context, Pos_Ref)));

         begin

            Store_Value
                (Context, Symtbl_Entry,
                 Create_Storage_For_Character (Str (Str'FIRST + Pos - 1)));
         end;

      when 802  =>
         --   function Length (Str : in String) return Integer is
         declare

            Str_Ref : Ref_Symbol_Table_Entry := Formal_Parameter;
            Str     : constant String :=
               Convert_To_String (Get_String_Value (Fetch_Value (Context,
                                                                 Str_Ref)));
         begin

            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Str'LENGTH));

         end;

         -- end of string routines

      when 1000 =>
         --   procedure Dump_Symbol_Table;

         Dump_Entire_Symtbl (Context);

      when 1001 =>
         --   procedure Dump_Symbol_Table
         --      (Display_Kind : in Display_Enumeration);

         case Get_Integer_Value (Fetch_Value (Context, Formal_Parameter)) is
            when 0   =>
               Dump_Entire_Symtbl (Context, Both);
            when 1   =>
               Dump_Entire_Symtbl (Context, Specification);
            when 2   =>
               Dump_Entire_Symtbl (Context, Implementation);
            when others   =>
               raise Program_Error;
         end case;

      when 1003 =>
         --   procedure Dump_Dss;

         Dump_Dss (Context);

      when 1004 =>
         --   procedure Dump_Display;

         Dump_Display (Context);

      when 1005 =>
         --   procedure Break;

         Debugger.Break (Context_Control.Get_Current_Statement (Context));

      when 1006 =>
         --   procedure Continue;

         Debugger.Continue;

      when 1007 =>
         --   procedure Set_Break (Statement_Number : in Natural);

         declare
            Item : Ref_Symbol_Table_Entry := Formal_Parameter;
         begin
            Debugger.Set_Break
                (Ace_Natural (Get_Integer_Value (Fetch_Value (Context, Item))));
         end;

      when 1008 =>
         --   procedure Set_Break (Name : in Interpreter_String);
         --     [Name is either a subprogram or label]

         declare
            Item : Ref_Symbol_Table_Entry := Formal_Parameter;
         begin
            Debugger.Set_Break
                (Convert_To_String
                     (Ref_String
                          (Get_String_Value (Fetch_Value (Context, Item)))));
         end;

      when 1009 =>
         --   procedure Set_Break;
         --      [at "current" statement]

         Debugger.Set_Break
             (Get_Statement_Number
                  (Context_Control.Get_Current_Statement (Context)));

      when 1010 =>
         --   procedure Clear_Break (Statement_Number : in Natural);

         declare
            Item : Ref_Symbol_Table_Entry := Formal_Parameter;
         begin
            Debugger.Clear_Break
                (Ace_Natural (Get_Integer_Value (Fetch_Value (Context, Item))));
         end;

      when 1011 =>
         --   procedure Clear_Break (Name : in Interpreter_String);
         --     [Name is either a subprogram or label]

         declare
            Item : Ref_Symbol_Table_Entry := Formal_Parameter;
         begin
            Debugger.Clear_Break
                (Convert_To_String
                     (Ref_String
                          (Get_String_Value (Fetch_Value (Context, Item)))));
         end;

      when 1012 =>
         --   procedure Clear_Break;
         --     [clear breakpoint at "current" breakpoint statement]

         Debugger.Clear_Break
             (Get_Statement_Number
                  (Context_Control.Get_Current_Statement (Context)));

      when 1013 =>
         --   procedure Display (Statement_Number : in Natural);

         declare
            Item : Ref_Symbol_Table_Entry := Formal_Parameter;
         begin
            Debugger.Display
                (Ace_Natural (Get_Integer_Value (Fetch_Value (Context, Item))));
         end;

      when 1014 =>
         --   procedure Display (Name : in Interpreter_String);
         --     [Name is a subprogram, package or label]

         declare
            Item : Ref_Symbol_Table_Entry := Formal_Parameter;
         begin
            Debugger.Display
                (Convert_To_String
                     (Ref_String
                          (Get_String_Value (Fetch_Value (Context, Item)))));
         end;

      when 1015 =>
         --   procedure Display;
         --   procedure Display_Current_Statement;
         --     [Two names for same routine, since renames is not yet
         --      implemented]

         Debugger.Display
             (Get_Statement_Number
                  (Context_Control.Get_Current_Statement (Context)));

      when 1016 =>
         --   procedure Display_Next;

         Debugger.Display_Next
             (Get_Statement_Number
                  (Context_Control.Get_Current_Statement (Context)));

      when 1017 =>
         --   procedure Display_Previous;

         Debugger.Display_Previous
             (Get_Statement_Number
                  (Context_Control.Get_Current_Statement (Context)));

      when 1018 =>
         --   procedure Display_Call_Stack;
         Put_Line ("unimplemented");

      when 1019 =>
         --   procedure List (Name      : in Interpreter_String;
         --                   List_Kind : in List_Mode := Specification;
         --                   Form      : in Interpreter_String := "");

         declare
            Name          : Ref_String :=
               Get_String_Value (Fetch_Value (Context, Formal_Parameter));
            Form          : Ref_Symbol_Table_Entry :=
               Get_Next_Parameter (Get_Next_Parameter (Formal_Parameter));
            Ref           : Ref_Symbol_Table_Entry;
            Ref_Form_Str  : Ref_String :=
               Get_String_Value (Fetch_Value (Context, Form));

            Enclosing_Pkg : Ref_Symbol_Table_Entry;

            use Expdname;

            procedure List (Dump_Mode : List_Mode) is
               Form_Str : constant String := Convert_To_String (Ref_Form_Str);
               Found_Something : Boolean := False;
            begin
               if Is_Empty (Enclosing_Pkg) then
                  -- not an expanded name
                  if (Form_Str /= "") then
                     Ref := Find_Matching_Name (Context, Name);
                     if (Is_Empty (Ref)) then
                        raise Name_Error;
                     else
                        loop
                           if (Form_Str = "all") then
                              if (Is_Visible (Ref)) then
                                 Dump_Symtbl (Ref, Current_Output, Dump_Mode);
                                 Found_Something := True;
                              end if;
                           elsif (Form_Str = "hidden") then
                              if (Is_Hidden (Ref)) then
                                 Dump_Symtbl (Ref, Current_Output, Dump_Mode);
                                 Found_Something := True;
                              end if;
                           elsif (Form_Str = "deleted") then
                              if (Is_Removed (Ref)) then
                                 Dump_Symtbl (Ref, Current_Output, Dump_Mode);
                                 Found_Something := True;
                              end if;
                           elsif (Get_Statement_Number (Ref) =
                                 Ace_Integer'VALUE (Form_Str)) then
                              Dump_Symtbl (Ref, Current_Output, Dump_Mode);
                              Found_Something := True;
                           end if;

                           Ref := Find_Next_Matching_Name (Ref);
                           exit  when Is_Empty (Ref);

                        end loop;
                     end if;
                     if (not Found_Something) then
                        raise Name_Error;
                     end if;
                  else
                     Ref := Find_Symbol_Table_Entry (Context, Name);
                     if (Is_Empty (Ref)) then
                        raise Name_Error;
                     end if;
                     Dump_Symtbl (Ref, Current_Output, Dump_Mode);
                  end if;
               else -- was an expanded name
                  if Get_Entry_Kind (Enclosing_Pkg) = Package_Entry then
                     declare
                        Curr_Decl_Stmt : Ref_Statement_Database;
                     begin
                        Find_Matching_Name (Context, Enclosing_Pkg, Name,
                                            Form_Str, Ref, Curr_Decl_Stmt);
                        while not Is_Empty (Ref) loop
                           if Are_Symbolic_Names_Equal
                                  (Get_Symbolic_Name (Ref), Name) and
                              Do_Forms_Match (Ref, Form_Str) then
                              Dump_Symtbl (Ref, Current_Output, Dump_Mode);
                              Found_Something := True;
                           end if;
                           Find_Next_Matching_Name (Context, Curr_Decl_Stmt,
                                                    Name, Form_Str, Ref);
                        end loop;

                     end;

                  else
                     Show_Error
                         (4206, Fatal,
                          Convert_To_String (Get_Symbolic_Name (Enclosing_Pkg))
                          );
                     null;
                  end if;
                  if (not Found_Something) then
                     raise Name_Error;
                  end if;
               end if;

            exception
               when Constraint_Error     =>
                  Show_Error
                      (3018, Fatal,
                       Convert_To_String
                           (Get_String_Value (Fetch_Value (Context, Form))));
            end List;

         begin
            Ada_Id (Name);
            Ada_Id (Ref_Form_Str);

            Handle_Expanded_Name (Context, Name, Enclosing_Pkg);

            case Get_Integer_Value
                (Fetch_Value (Context, Get_Next_Parameter (Formal_Parameter)))
                is
               when 0     =>
                  List (Both);
               when 1     =>
                  List (Specification);
               when 2     =>
                  List (Implementation);
               when others     =>
                  raise Program_Error;
            end case;
         end;

         -- =====================================================
         -- >>> More debugger operations start at number 1030 <<<
         -- =====================================================

         -- ================================
         -- >>> X Window System routines <<<
         -- ================================

      when 1020 =>
         --   procedure Create_Window
         --                    (Window              : in out Window_Type;
         --                     Horizontal_Position : in Coordinate;
         --                     Vertical_Position   : in Coordinate;
         --                     Horizontal_Size     : in Pixels;
         --                     Vertical_Size       : in Pixels);

         declare
            Window              : Ref_Symbol_Table_Entry := Formal_Parameter;
            Horizontal_Position : Ref_Symbol_Table_Entry :=
               Get_Next_Parameter (Formal_Parameter);
            Vertical_Position   : Ref_Symbol_Table_Entry :=
               Get_Next_Parameter (Horizontal_Position);
            Horizontal_Size     : Ref_Symbol_Table_Entry :=
               Get_Next_Parameter (Vertical_Position);
            Vertical_Size       : Ref_Symbol_Table_Entry :=
               Get_Next_Parameter (Horizontal_Size);
            Window_Object       : Ace_Window_Type :=
               Ace_Window_Type
                   (Get_Integer_Value (Fetch_Value (Context, Window)));
         begin
            X_Window_Adt.Create_Window
                (Window_Object,
                 Ace_Coordinate
                     (Get_Integer_Value (Fetch_Value (Context,
                                                      Horizontal_Position))),
                 Ace_Coordinate
                     (Get_Integer_Value (Fetch_Value (Context,
                                                      Vertical_Position))),
                 Ace_Pixels (Get_Integer_Value (Fetch_Value (Context,
                                                             Horizontal_Size))),
                 Ace_Pixels (Get_Integer_Value (Fetch_Value (Context,
                                                             Vertical_Size))));
            Store_Value
                (Context, Window,
                 Create_Storage_For_Integer (Ace_Integer (Window_Object)));
         end;

      when 1021 =>
         --   procedure Draw_Line (Window         : in Window_Type;
         --                        X1, Y1, X2, Y2 : in Coordinate);

         declare
            Window        : Ref_Symbol_Table_Entry := Formal_Parameter;

            X1_In         : Ref_Symbol_Table_Entry :=
               Get_Next_Parameter (Formal_Parameter);
            Y1_In : Ref_Symbol_Table_Entry := Get_Next_Parameter (X1_In);
            X2_In : Ref_Symbol_Table_Entry := Get_Next_Parameter (Y1_In);
            Y2_In : Ref_Symbol_Table_Entry := Get_Next_Parameter (X2_In);

            X1            : Ace_Integer :=
               Get_Integer_Value (Fetch_Value (Context, X1_In));
            Y1            : Ace_Integer :=
               Get_Integer_Value (Fetch_Value (Context, Y1_In));
            X2            : Ace_Integer :=
               Get_Integer_Value (Fetch_Value (Context, X2_In));
            Y2            : Ace_Integer :=
               Get_Integer_Value (Fetch_Value (Context, Y2_In));

            Window_Object : Ace_Window_Type :=
               Ace_Window_Type
                   (Get_Integer_Value (Fetch_Value (Context, Window)));
         begin
            X_Window_Adt.Draw_Line (Window_Object, Ace_Coordinate (X1),
                                    Ace_Coordinate (Y1), Ace_Coordinate (X2),
                                    Ace_Coordinate (Y2));
         end;

      when 1022 =>
         --   procedure Draw_Text (Window : in Window_Type;
         --                        X, Y   : in Coordinate;
         --                        Text   : in String);

         declare
            Window        : Ref_Symbol_Table_Entry := Formal_Parameter;
            X1            : Ref_Symbol_Table_Entry :=
               Get_Next_Parameter (Formal_Parameter);
            Y1            : Ref_Symbol_Table_Entry := Get_Next_Parameter (X1);
            Text          : Ref_Symbol_Table_Entry := Get_Next_Parameter (Y1);
            Window_Object : Ace_Window_Type :=
               Ace_Window_Type
                   (Get_Integer_Value (Fetch_Value (Context, Window)));
         begin
            X_Window_Adt.Draw_Text
                (Window_Object,
                 Ace_Coordinate (Get_Integer_Value (Fetch_Value (Context, X1))),
                 Ace_Coordinate (Get_Integer_Value (Fetch_Value (Context, Y1))),
                 Convert_To_String (Get_String_Value (Fetch_Value (Context,
                                                                   Text))));
         end;

      when 1023 =>
         --   procedure Draw_Rectangle_Builtin
         --                      (Window        : in Window_Type;
         --                       X, Y          : in Coordinate;
         --                       Width, Height : in Pixels);

         declare
            Window        : Ref_Symbol_Table_Entry := Formal_Parameter;
            X1            : Ref_Symbol_Table_Entry :=
               Get_Next_Parameter (Formal_Parameter);
            Y1            : Ref_Symbol_Table_Entry := Get_Next_Parameter (X1);
            Width         : Ref_Symbol_Table_Entry := Get_Next_Parameter (Y1);
            Height : Ref_Symbol_Table_Entry := Get_Next_Parameter (Width);
            Window_Object : Ace_Window_Type :=
               Ace_Window_Type
                   (Get_Integer_Value (Fetch_Value (Context, Window)));
         begin
            X_Window_Adt.Draw_Rectangle_Builtin
                (Window_Object,
                 Ace_Coordinate (Get_Integer_Value (Fetch_Value (Context, X1))),
                 Ace_Coordinate (Get_Integer_Value (Fetch_Value (Context, Y1))),
                 Ace_Pixels (Get_Integer_Value (Fetch_Value (Context, Width))),
                 Ace_Pixels (Get_Integer_Value (Fetch_Value (Context, Height)))
                 );
         end;

      when 1024 =>
         --   procedure Destroy_Window (Window : in out Window_Type);

         declare
            Window        : Ref_Symbol_Table_Entry := Formal_Parameter;
            Window_Object : Ace_Window_Type :=
               Ace_Window_Type
                   (Get_Integer_Value (Fetch_Value (Context, Window)));
         begin
            X_Window_Adt.Destroy_Window (Window_Object);
            Store_Value
                (Context, Window,
                 Create_Storage_For_Integer (Ace_Integer (Window_Object)));
         end;

      when 1025 =>
         --   procedure Clear_Window (Window : in out Window_Type);

         declare
            Window        : Ref_Symbol_Table_Entry := Formal_Parameter;
            Window_Object : Ace_Window_Type :=
               Ace_Window_Type
                   (Get_Integer_Value (Fetch_Value (Context, Window)));
         begin
            X_Window_Adt.Clear_Window (Window_Object);
         end;

      when 1027 =>
         --   procedure Draw_Dashed_Line (Window         : in Window_Type;
         --                               X1, Y1, X2, Y2 : in Coordinate);

         declare
            Window        : Ref_Symbol_Table_Entry := Formal_Parameter;

            X1_In         : Ref_Symbol_Table_Entry :=
               Get_Next_Parameter (Formal_Parameter);
            Y1_In : Ref_Symbol_Table_Entry := Get_Next_Parameter (X1_In);
            X2_In : Ref_Symbol_Table_Entry := Get_Next_Parameter (Y1_In);
            Y2_In : Ref_Symbol_Table_Entry := Get_Next_Parameter (X2_In);

            X1            : Ace_Integer :=
               Get_Integer_Value (Fetch_Value (Context, X1_In));
            Y1            : Ace_Integer :=
               Get_Integer_Value (Fetch_Value (Context, Y1_In));
            X2            : Ace_Integer :=
               Get_Integer_Value (Fetch_Value (Context, X2_In));
            Y2            : Ace_Integer :=
               Get_Integer_Value (Fetch_Value (Context, Y2_In));

            Window_Object : Ace_Window_Type :=
               Ace_Window_Type
                   (Get_Integer_Value (Fetch_Value (Context, Window)));
         begin
            X_Window_Adt.Draw_Dashed_Line (Window_Object, Ace_Coordinate (X1),
                                           Ace_Coordinate (Y1),
                                           Ace_Coordinate (X2),
                                           Ace_Coordinate (Y2));
         end;

         ----------

      when 1026 =>
         --   procedure Delay_For
         --                 (Whole_Second, Fraction_Of_A_Second : in Integer);

         declare
            Whole_Second         : Ref_Symbol_Table_Entry := Formal_Parameter;
            Fraction_Of_A_Second : Ref_Symbol_Table_Entry :=
               Get_Next_Parameter (Formal_Parameter);
         begin
            delay
               (Duration
                   (Get_Integer_Value (Fetch_Value (Context, Whole_Second))) +
               (Duration
                   (Get_Integer_Value (Fetch_Value (Context,
                                                    Fraction_Of_A_Second))) /
               100));
         end;

      when 1030 =>
         --   procedure Display_Next (Statement_Number : in Natural);

         declare
            Item : Ref_Symbol_Table_Entry := Formal_Parameter;
         begin
            Debugger.Display_Next
                (Ace_Natural (Get_Integer_Value (Fetch_Value (Context, Item))));
         end;

      when 1031 =>
         --   procedure Display_Previous (Statement_Number : in Natural);

         declare
            Item : Ref_Symbol_Table_Entry := Formal_Parameter;
         begin
            Debugger.Display_Previous
                (Ace_Natural (Get_Integer_Value (Fetch_Value (Context, Item))));
         end;

      when 1032 =>
         --   procedure Step (Count : Positive := 1);

         declare
            Count : Ref_Symbol_Table_Entry := Formal_Parameter;
         begin
            Debugger.Step
                (Ace_Positive
                     (Get_Integer_Value (Fetch_Value (Context, Count))));
         end;

      when 1033 =>
         --   procedure List_Breakpoints;

         Debugger.List_Breakpoints;

      when 1034 =>
         -- function Clock return Time;

         begin
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Ace_Integer (Cpu.Clock)));
         end;

      when 1035 =>
         -- function "-" (Stop_Time, Start_Time : Time) return Time;
         declare
            Stop_Time  : Ref_Symbol_Table_Entry := Formal_Parameter;
            Start_Time : Ref_Symbol_Table_Entry :=
               Get_Next_Parameter (Formal_Parameter);
            -- [TBD] workaround for telesoft conversion bug
            Tmp_Time1  : Cpu.Time;
            Tmp_Time2  : Cpu.Time;
         begin
            Tmp_Time1 := Cpu.Time (Get_Integer_Value (Fetch_Value (Context,
                                                                   Stop_Time)));

            Tmp_Time2 :=
               Cpu.Time (Get_Integer_Value (Fetch_Value (Context, Start_Time)));

            Store_Value
                (Context, Symtbl_Entry,
                 Create_Storage_For_Integer
                     (Ace_Integer (Cpu.Time'(Cpu."-" (Tmp_Time1, Tmp_Time2)))));
         end;

      when 1036 =>
         -- procedure Put_Time (A_Time : in Time);
         declare
            A_Time : Ref_Symbol_Table_Entry := Formal_Parameter;
         begin
            Cpu.Put_Time (Cpu.Time (Get_Integer_Value (Fetch_Value (Context,
                                                                    A_Time))));
         end;

      when 1037 =>
         -- procedure Console (Message : String);
         begin
            Ace_Text_Io.Put
                (Convert_To_String
                     (Get_String_Value
                          (Fetch_Value (Context, Formal_Parameter))) &
                 Ascii.Esc & "[K" & Ascii.Cr);
         end;

      when 1500 =>
         --  function Opendir (Filename : String) return Ace_Integer;
         begin
            Store_Value
                (Context, Symtbl_Entry,
                 Create_Storage_For_Integer
                     (System_Dependent_Routines.Opendir
                          (Convert_To_String
                               (Get_String_Value
                                    (Fetch_Value (Context, Formal_Parameter))))
                      ));
         end;

      when 1501 =>
         --  function Readdir (Dirfd : Ace_Integer) return String;
         begin
            Store_Value
                (Context, Symtbl_Entry,
                 Create_Storage_For_String
                     (Convert_To_Ref_String
                          (System_Dependent_Routines.Readdir
                               (Get_Integer_Value
                                    (Fetch_Value (Context, Formal_Parameter))))
                      ));
         end;

      when 1502 =>
         --  function Closedir (Dirfd : Ace_Integer) return Ace_Integer;
         begin
            Store_Value
                (Context, Symtbl_Entry,
                 Create_Storage_For_Integer
                     (System_Dependent_Routines.Closedir
                          (Get_Integer_Value (Fetch_Value (Context,
                                                           Formal_Parameter))))
                 );
         end;

      when 1600 =>
         -- procedure Make_Binding (Char_Seq        : in String;
         --                         Binding         : in Commands;
         --                         Optional_String : in String := "");
         declare
            Char_Seq        : Ref_Symbol_Table_Entry := Formal_Parameter;
            Binding         : Ref_Symbol_Table_Entry :=
               Get_Next_Parameter (Formal_Parameter);
            Optional_String : Ref_Symbol_Table_Entry :=
               Get_Next_Parameter (Binding);
         begin
            Key_Bindings.Make_Binding
                (Convert_To_String (Get_String_Value (Fetch_Value (Context,
                                                                   Char_Seq))),
                 Key_Bindings.Commands'VAL (
                    Get_Integer_Value (Fetch_Value (Context, Binding))),
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Optional_String))
                      ));
         end;

      when 1601 =>
         -- procedure Make_Binding (C               : in Character;
         --                         Binding         : in Commands;
         --                         Optional_String : in String := "");
         declare
            C               : Ref_Symbol_Table_Entry := Formal_Parameter;
            Binding         : Ref_Symbol_Table_Entry :=
               Get_Next_Parameter (Formal_Parameter);
            Optional_String : Ref_Symbol_Table_Entry :=
               Get_Next_Parameter (Binding);
         begin
            Key_Bindings.Make_Binding
                (Get_Character_Value (Fetch_Value (Context, C)),
                 Key_Bindings.Commands'VAL (
                    Get_Integer_Value (Fetch_Value (Context, Binding))),
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Optional_String))
                      ));
         end;

      when 1603 =>
         -- procedure History_Back;
         Ace_Command_Line_Input.Command_Input.Do_History_Back;
      when 1604 =>
         -- procedure History_Forward;
         Ace_Command_Line_Input.Command_Input.Do_History_Forward;
      when 1605 =>
         -- procedure Move_Left;
         Ace_Command_Line_Input.Command_Input.Move_Left;
      when 1606 =>
         -- procedure Move_Right;
         Ace_Command_Line_Input.Command_Input.Move_Right;
      when 1607 =>
         -- procedure Goto_Beg_Of_Line;
         Ace_Command_Line_Input.Command_Input.Goto_Beg_Of_Line;
      when 1608 =>
         -- procedure Goto_Start_Of_History;
         Ace_Command_Line_Input.Command_Input.Goto_Start_Of_History;
      when 1609 =>
         -- procedure Goto_End_Of_History;
         Ace_Command_Line_Input.Command_Input.Goto_End_Of_History;
      when 1610 =>
         -- procedure Show_History;
         Ace_Command_Line_Input.Command_Input.Show_History;
      when 1611 =>
         -- procedure Delete;
         Ace_Command_Line_Input.Command_Input.Delete;
      when 1612 =>
         -- procedure Delete_This_Char;
         Ace_Command_Line_Input.Command_Input.Delete_This_Char;
      when 1614 =>
         -- procedure Kill_Line;
         Ace_Command_Line_Input.Command_Input.Kill_Line;
      when 1615 =>
         -- procedure Insert_String (Str : String);
         Ace_Command_Line_Input.Command_Input.Insert_String
             (Convert_To_String
                  (Get_String_Value (Fetch_Value (Context, Formal_Parameter))));
      when 1616 =>
         -- procedure Show_History_Limit;
         Ace_Command_Line_Input.Command_Input.Show_History_Limit;
      when 1617 =>
         -- procedure Set_History_Limit;
         Ace_Command_Line_Input.Command_Input.Set_History_Limit;
      when 1618 =>
         -- procedure Quoted_Insert;
         Ace_Command_Line_Input.Command_Input.Quoted_Insert;
      when 1619 =>
         -- procedure Refresh_Current_Line;
         Ace_Command_Line_Input.Command_Input.Refresh_Current_Line;
      when 1620 =>
         -- procedure Refresh_Current_Line_And_Prompt;
         Ace_Command_Line_Input.Command_Input.Refresh_Current_Line_And_Prompt;
      when 1621 =>
         -- procedure Rewrite_Current_Line (Line : String);
         Ace_Command_Line_Input.Command_Input.Rewrite_Current_Line 
             (Convert_To_String
                  (Get_String_Value (Fetch_Value (Context, Formal_Parameter))));
      when 1622 =>
         -- procedure Rewrite_Current_Line_And_Prompt (Line : String);
         Ace_Command_Line_Input.Command_Input.Rewrite_Current_Line_And_Prompt
             (Convert_To_String
                  (Get_String_Value (Fetch_Value (Context, Formal_Parameter))));
      when 1623 =>
         -- procedure Get_Current_Line (Line : out String; Length : out Natural);

         declare
            Line_Parameter	    : Ref_Symbol_Table_Entry;
            Length_Parameter	    : Ref_Symbol_Table_Entry;
            Line_Parameter_Result   : String 
                (Get_Lower_Bound_Of_Actual (1) .. Get_Upper_Bound_Of_Actual(1));
            Length_Parameter_Result : Natural;

         begin
            Line_Parameter := Formal_Parameter;
            Length_Parameter := Get_Next_Parameter (Line_Parameter);

            Ace_Command_Line_Input.Command_Input.Get_Current_Line
                (Line_Parameter_Result,
                 Length_Parameter_Result);

            Store_Value (Context, Line_Parameter,
                         Create_Storage_For_String (Convert_To_Ref_String
                            (Line_Parameter_Result)));
            Store_Value (Context, Length_Parameter,
                         Create_Storage_For_Integer 
                            (Ace_Integer(Length_Parameter_Result)));
         end;

      when 1624 =>
         -- function Get_Current_Character return Character;
         
         declare
            Result : Character;
         begin
            Result := Ace_Command_Line_Input.Command_Input.Get_Current_Character;
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Character (Result));
         end;

      when 1625 =>
         --function Get_Current_Column return Natural;

         declare
            Result : Natural;
         begin
            Result := Ace_Command_Line_Input.Command_Input.Get_Current_Column;
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Ace_Integer (Result)));
         end;

      when 1626 =>
         -- procedure Goto_End_Of_Line;
         Ace_Command_Line_Input.Command_Input.Goto_End_Of_Line;

      when 1627 =>
         -- procedure Terminal_Set;
         Ace_Command_Line_Input.Command_Input.Terminal_Set;

      when 1628 =>
         -- procedure Terminal_Reset;
         Ace_Command_Line_Input.Command_Input.Terminal_Reset;

      when 1629 =>
         -- function Maximum_Line_Length return Natural;
         declare
            Result : Natural;
         begin
            Result := Ace_Command_Line_Input.Command_Input.Maximum_Line_Length;
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Ace_Integer (Result)));
         end;


      -- 2000 to 2099 is reserved for CAIS-A (wpl)

         -- =====================================
         -- >>> START of ACE Xt toolkit routines <<<
         -- =====================================


      when 2101 =>
         --       function Ascent (F : in Font_Record) return Pixels;

         declare
            Result : Ace_X_Windows.Pixels;
         begin
            Result :=
               Ace_X_Windows.Ascent
                   (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))
                    );
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Ace_Integer (Result)));
         end;

      when 2102 =>
         --       function Descent (F : in Font_Record) return Pixels;

         declare
            Result : Ace_X_Windows.Pixels;
         begin
            Result :=
               Ace_X_Windows.Descent
                   (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))
                    );
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Ace_Integer (Result)));
         end;

      when 2103 =>
         --       function X_Text_Width (Font_Info : in Font_Record;
         --                              Text      : in String) return Pixels;

         declare
            Result              : Ace_X_Windows.Pixels;
            Font_Info_Parameter : Ref_Symbol_Table_Entry;
            Text_Parameter      : Ref_Symbol_Table_Entry;
         begin
            Font_Info_Parameter := Formal_Parameter;
            Text_Parameter      := Get_Next_Parameter (Font_Info_Parameter);

            Result :=
               Ace_Widget_Package.X_Text_Width
                   (Get_Integer_Value (Fetch_Value (Context,
                                                    Font_Info_Parameter)),
                    Convert_To_String
                        (Get_String_Value
                             (Fetch_Value (Context, Text_Parameter))));
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Ace_Integer (Result)));
         end;

      when 2201 =>
         --    function Null_Xrm_Option_List
         --        return X_Windows.Resource_Manager.Xrm_Option_Desc_List;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Widget_Package.Null_Xrm_Option_List));

      when 2202 =>
         --    function Null_Xt_Arg_List return Xt_Arg_List;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer (Ace_Widget_Package.Null_Xt_Arg_List));

      when 2203 =>
         --    function Make_Xt_String (S : in String)
         --        return Renamed_Xlib_Types.Xt_String;

         declare
            Result : Ace_Integer;
         begin
            Result :=
               Ace_Widget_Package.Make_Xt_String
                   (Convert_To_String
                        (Get_String_Value (Fetch_Value (Context,
                                                        Formal_Parameter))));
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Result));
         end;

      when 2204 =>
         --    procedure Xt_Initialize
         --        (Out_Value : out Intrinsics.Widget;
         --         Name      : in String;
         --         Classname : in String;
         --         Urlist    : in
         --               X_Windows.Resource_Manager.Xrm_Option_Desc_List;
         --         Num_Urs   : in Intrinsics.Cardinal;
         --         Argc      : in out Intrinsics.Cardinal;
         --         Argv      : in out U_Env.Argv_Ptr);

         declare

            Out_Value_Parameter : Ref_Symbol_Table_Entry;
            Name_Parameter      : Ref_Symbol_Table_Entry;
            Classname_Parameter : Ref_Symbol_Table_Entry;
            Urlist_Parameter    : Ref_Symbol_Table_Entry;
            Num_Urs_Parameter   : Ref_Symbol_Table_Entry;
            Argc_Parameter      : Ref_Symbol_Table_Entry;
            Argv_Parameter      : Ref_Symbol_Table_Entry;

            Out_Value_Result : Ace_Integer;
            Argc_Result      : Ace_Integer;
            Argv_Result      : Ace_Integer;

         begin

            Out_Value_Parameter := Formal_Parameter;
            Name_Parameter      := Get_Next_Parameter (Out_Value_Parameter);
            Classname_Parameter := Get_Next_Parameter (Name_Parameter);
            Urlist_Parameter    := Get_Next_Parameter (Classname_Parameter);
            Num_Urs_Parameter   := Get_Next_Parameter (Urlist_Parameter);
            Argc_Parameter      := Get_Next_Parameter (Num_Urs_Parameter);
            Argv_Parameter      := Get_Next_Parameter (Argc_Parameter);

            Argc_Result := Get_Integer_Value (Fetch_Value (Context,
                                                           Argc_Parameter));
            Argv_Result := Get_Integer_Value (Fetch_Value (Context,
                                                           Argv_Parameter));

            Ace_Widget_Package.Xt_Initialize
                (Out_Value_Result,
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context,
                                                     Classname_Parameter))),
                 Get_Integer_Value (Fetch_Value (Context, Urlist_Parameter)),
                 Get_Integer_Value (Fetch_Value (Context, Num_Urs_Parameter)),
                 Argc_Result, Argv_Result);

            Store_Value (Context, Out_Value_Parameter,
                         Create_Storage_For_Integer (Out_Value_Result));
            Store_Value (Context, Argc_Parameter,
                         Create_Storage_For_Integer (Argc_Result));
            Store_Value (Context, Argv_Parameter,
                         Create_Storage_For_Integer (Argv_Result));
         end;

      when 2205 =>
         --    procedure Xt_Set_Arg (Arg   : out
         --                             Intrinsics.Xt_Ancillary_Types.Xt_Arg;
         --                          Name  : in String;
         --                          Value : in Boolean);

         declare

            Arg_Parameter   : Ref_Symbol_Table_Entry;
            Name_Parameter  : Ref_Symbol_Table_Entry;
            Value_Parameter : Ref_Symbol_Table_Entry;
            Arg_Result      : Ace_Integer;

         begin
            Arg_Parameter   := Formal_Parameter;
            Name_Parameter  := Get_Next_Parameter (Arg_Parameter);
            Value_Parameter := Get_Next_Parameter (Name_Parameter);

            Ace_Widget_Package.Xt_Set_Arg
                (Arg_Result,
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Boolean'VAL (Get_Integer_Value (Fetch_Value (Context,
                                                              Value_Parameter))
                              ));
            Store_Value (Context, Arg_Parameter,
                         Create_Storage_For_Integer (Arg_Result));
         end;

      when 2206 =>
         --    procedure Xt_Set_Arg (Arg   : out
         --                             Intrinsics.Xt_Ancillary_Types.Xt_Arg;
         --                          Name  : in String;
         --                          Value : in String);

         declare

            Arg_Parameter   : Ref_Symbol_Table_Entry;
            Name_Parameter  : Ref_Symbol_Table_Entry;
            Value_Parameter : Ref_Symbol_Table_Entry;
            Arg_Result      : Ace_Integer;

         begin
            Arg_Parameter   := Formal_Parameter;
            Name_Parameter  := Get_Next_Parameter (Arg_Parameter);
            Value_Parameter := Get_Next_Parameter (Name_Parameter);

            Ace_Widget_Package.Xt_Set_Arg
                (Arg_Result,
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Value_Parameter))
                      ));
            Store_Value (Context, Arg_Parameter,
                         Create_Storage_For_Integer (Arg_Result));
         end;

      when 2207 =>
         --    procedure Xt_Set_Arg (Arg   : out
         --                             Intrinsics.Xt_Ancillary_Types.Xt_Arg;
         --                          Name  : in String;
         --                          Value : in Integer);

         declare

            Arg_Parameter   : Ref_Symbol_Table_Entry;
            Name_Parameter  : Ref_Symbol_Table_Entry;
            Value_Parameter : Ref_Symbol_Table_Entry;
            Arg_Result      : Ace_Integer;

         begin
            Arg_Parameter   := Formal_Parameter;
            Name_Parameter  := Get_Next_Parameter (Arg_Parameter);
            Value_Parameter := Get_Next_Parameter (Name_Parameter);

            Ace_Widget_Package.Xt_Set_Arg
                (Arg_Result,
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Integer (Get_Integer_Value (Fetch_Value (Context,
                                                          Value_Parameter))));
            Store_Value (Context, Arg_Parameter,
                         Create_Storage_For_Integer (Arg_Result));
         end;

      when 2208 =>
         --    procedure Xt_Set_Arg (Arg   : out
         --                             Intrinsics.Xt_Ancillary_Types.Xt_Arg;
         --                          Name  : in String;
         --                          Value : in System.Address);

         declare

            Arg_Parameter   : Ref_Symbol_Table_Entry;
            Name_Parameter  : Ref_Symbol_Table_Entry;
            Value_Parameter : Ref_Symbol_Table_Entry;
            Arg_Result      : Ace_Integer;

         begin
            Arg_Parameter   := Formal_Parameter;
            Name_Parameter  := Get_Next_Parameter (Arg_Parameter);
            Value_Parameter := Get_Next_Parameter (Name_Parameter);

            Ace_Widget_Package.Xt_Set_Arg_Address
                (Arg_Result,
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Get_Integer_Value (Fetch_Value (Context, Value_Parameter)));
            Store_Value (Context, Arg_Parameter,
                         Create_Storage_For_Integer (Arg_Result));
         end;

      when 2209 =>
         --    procedure Xt_Set_Arg (Arg   : out
         --                             Intrinsics.Xt_Ancillary_Types.Xt_Arg;
         --                          Name  : in String;
         --                          Value : in Intrinsics.Dimension);

         declare

            Arg_Parameter   : Ref_Symbol_Table_Entry;
            Name_Parameter  : Ref_Symbol_Table_Entry;
            Value_Parameter : Ref_Symbol_Table_Entry;
            Arg_Result      : Ace_Integer;

         begin
            Arg_Parameter   := Formal_Parameter;
            Name_Parameter  := Get_Next_Parameter (Arg_Parameter);
            Value_Parameter := Get_Next_Parameter (Name_Parameter);

            Ace_Widget_Package.Xt_Set_Arg
                (Arg_Result,
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Intrinsics.Dimension
                     (Get_Integer_Value
                          (Fetch_Value (Context, Value_Parameter))));
            Store_Value (Context, Arg_Parameter,
                         Create_Storage_For_Integer (Arg_Result));
         end;

      when 2210 =>
         --    procedure Xt_Set_Arg (Arg   : out
         --                             Intrinsics.Xt_Ancillary_Types.Xt_Arg;
         --                          Name  : in String;
         --                          Value : in Intrinsics.Position);

         declare

            Arg_Parameter   : Ref_Symbol_Table_Entry;
            Name_Parameter  : Ref_Symbol_Table_Entry;
            Value_Parameter : Ref_Symbol_Table_Entry;
            Arg_Result      : Ace_Integer;

         begin
            Arg_Parameter   := Formal_Parameter;
            Name_Parameter  := Get_Next_Parameter (Arg_Parameter);
            Value_Parameter := Get_Next_Parameter (Name_Parameter);

            Ace_Widget_Package.Xt_Set_Arg
                (Arg_Result,
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Intrinsics.Position
                     (Get_Integer_Value
                          (Fetch_Value (Context, Value_Parameter))));
            Store_Value (Context, Arg_Parameter,
                         Create_Storage_For_Integer (Arg_Result));
         end;

      when 2211 =>
         --    procedure Xt_Set_Arg (Arg   : out
         --                             Intrinsics.Xt_Ancillary_Types.Xt_Arg;
         --                          Name  : in String;
         --                          Value : in X_Windows.Events.Gravity_Type);

         declare

            Arg_Parameter   : Ref_Symbol_Table_Entry;
            Name_Parameter  : Ref_Symbol_Table_Entry;
            Value_Parameter : Ref_Symbol_Table_Entry;
            Arg_Result      : Ace_Integer;

         begin
            Arg_Parameter   := Formal_Parameter;
            Name_Parameter  := Get_Next_Parameter (Arg_Parameter);
            Value_Parameter := Get_Next_Parameter (Name_Parameter);

            Ace_Widget_Package.Xt_Set_Arg
                (Arg_Result,
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Ace_X_Windows.Gravity_Type'VAL (
                    Get_Integer_Value (Fetch_Value (Context, Value_Parameter)))
                 );
            Store_Value (Context, Arg_Parameter,
                         Create_Storage_For_Integer (Arg_Result));
         end;

      when 2212 =>
         --    procedure Create (Arg_List : out Xt_Arg_List;
         --                      First    : in Integer;
         --                      Last     : in Integer);

         declare

            Arg_List_Parameter : Ref_Symbol_Table_Entry;
            First_Parameter    : Ref_Symbol_Table_Entry;
            Last_Parameter     : Ref_Symbol_Table_Entry;
            Arg_List_Result    : Ace_Integer;

         begin
            Arg_List_Parameter := Formal_Parameter;
            First_Parameter    := Get_Next_Parameter (Arg_List_Parameter);
            Last_Parameter     := Get_Next_Parameter (First_Parameter);

            Ace_Widget_Package.Create_Xt_Arg_List
                (Arg_List_Result,
                 Get_Integer_Value (Fetch_Value (Context, First_Parameter)),
                 Get_Integer_Value (Fetch_Value (Context, Last_Parameter)));
            Store_Value (Context, Arg_List_Parameter,
                         Create_Storage_For_Integer (Arg_List_Result));
         end;

      when 2213 =>
         --    procedure Put (Arg_List  : out Xt_Arg_List;
         --                   Subscript : in Integer;
         --                   Value     : in
         --                         Intrinsics.Xt_Ancillary_Types.Xt_Arg);

         declare

            Arg_List_Parameter  : Ref_Symbol_Table_Entry;
            Subscript_Parameter : Ref_Symbol_Table_Entry;
            Value_Parameter     : Ref_Symbol_Table_Entry;
            Arg_List_Result     : Ace_Integer;

         begin
            Arg_List_Parameter  := Formal_Parameter;
            Subscript_Parameter := Get_Next_Parameter (Arg_List_Parameter);
            Value_Parameter     := Get_Next_Parameter (Subscript_Parameter);
            Arg_List_Result     :=
               Get_Integer_Value (Fetch_Value (Context, Arg_List_Parameter));

            Ace_Widget_Package.Put_Xt_Arg_List
                (Arg_List_Result,
                 Get_Integer_Value (Fetch_Value (Context, Subscript_Parameter)),
                 Get_Integer_Value (Fetch_Value (Context, Value_Parameter)));
            Store_Value (Context, Arg_List_Parameter,
                         Create_Storage_For_Integer (Arg_List_Result));
         end;

      when 2214 =>
         --    function Get (Arg_List  : in Xt_Arg_List;
         --                  Subscript : in Integer)
         --          return Intrinsics.Xt_Ancillary_Types.Xt_Arg;

         declare

            Arg_List_Parameter  : Ref_Symbol_Table_Entry;
            Subscript_Parameter : Ref_Symbol_Table_Entry;
            Result              : Ace_Integer;

         begin
            Arg_List_Parameter  := Formal_Parameter;
            Subscript_Parameter := Get_Next_Parameter (Arg_List_Parameter);

            Result :=
               Ace_Widget_Package.Get_Xt_Arg_List
                   (Get_Integer_Value
                        (Fetch_Value (Context, Arg_List_Parameter)),
                    Get_Integer_Value (Fetch_Value (Context,
                                                    Subscript_Parameter)));
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Result));
         end;

      when 2215 =>
         --    function Get_Xt_Arg_List (Arg_List : in Xt_Arg_List;
         --                              First    : in Integer;
         --                              Last     : in Integer)
         --          return Xt_Arg_List;

         declare

            Arg_List_Parameter : Ref_Symbol_Table_Entry;
            First_Parameter    : Ref_Symbol_Table_Entry;
            Last_Parameter     : Ref_Symbol_Table_Entry;
            Result             : Ace_Integer;

         begin
            Arg_List_Parameter := Formal_Parameter;
            First_Parameter    := Get_Next_Parameter (Arg_List_Parameter);
            Last_Parameter     := Get_Next_Parameter (First_Parameter);

            Result :=
               Ace_Widget_Package.Get_Xt_Arg_List
                   (Get_Integer_Value
                        (Fetch_Value (Context, Arg_List_Parameter)),
                    Get_Integer_Value (Fetch_Value (Context, First_Parameter)),
                    Get_Integer_Value (Fetch_Value (Context, Last_Parameter)));
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Result));
         end;

      when 2216 =>
         --    procedure Xt_Set_Values (W        : in Intrinsics.Widget;
         --                             Arg_List : in Xt_Arg_List);

         declare

            W_Parameter        : Ref_Symbol_Table_Entry;
            Arg_List_Parameter : Ref_Symbol_Table_Entry;

         begin
            W_Parameter        := Formal_Parameter;
            Arg_List_Parameter := Get_Next_Parameter (W_Parameter);

            Ace_Widget_Package.Xt_Set_Values
                (Get_Integer_Value (Fetch_Value (Context, W_Parameter)),
                 Get_Integer_Value (Fetch_Value (Context, Arg_List_Parameter)));
         end;

      when 2217 =>
         --    function Xt_Get_Value (W    : in Intrinsics.Widget;
         --                           Name : in String) return Boolean;

         declare

            W_Parameter    : Ref_Symbol_Table_Entry;
            Name_Parameter : Ref_Symbol_Table_Entry;
            Result         : Boolean;

         begin
            W_Parameter    := Formal_Parameter;
            Name_Parameter := Get_Next_Parameter (W_Parameter);

            Result :=
               Ace_Widget_Package.Xt_Get_Value
                   (Get_Integer_Value (Fetch_Value (Context, W_Parameter)),
                    Convert_To_String
                        (Get_String_Value
                             (Fetch_Value (Context, Name_Parameter))));
            Store_Value
                (Context, Symtbl_Entry,
                 Create_Storage_For_Integer
                     (Ace_Integer (Boolean'POS (Result))));
         end;

      when 2218 =>
         --    function Xt_Get_Value (W    : in Intrinsics.Widget;
         --                           Name : in String)
         --          return Intrinsics.Dimension;

         declare

            W_Parameter    : Ref_Symbol_Table_Entry;
            Name_Parameter : Ref_Symbol_Table_Entry;
            Result         : Intrinsics.Dimension;

         begin
            W_Parameter    := Formal_Parameter;
            Name_Parameter := Get_Next_Parameter (W_Parameter);

            Result :=
               Ace_Widget_Package.Xt_Get_Value
                   (Get_Integer_Value (Fetch_Value (Context, W_Parameter)),
                    Convert_To_String
                        (Get_String_Value
                             (Fetch_Value (Context, Name_Parameter))));
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Ace_Integer (Result)));
         end;

      when 2219 =>
         --    function Xt_Get_Value (W    : in Intrinsics.Widget;
         --                           Name : in String)
         --          return X_Windows.Fonts.Font_Record;

         declare

            W_Parameter    : Ref_Symbol_Table_Entry;
            Name_Parameter : Ref_Symbol_Table_Entry;
            Result         : Ace_Integer;

         begin
            W_Parameter    := Formal_Parameter;
            Name_Parameter := Get_Next_Parameter (W_Parameter);

            Result :=
               Ace_Widget_Package.Xt_Get_Value_Font_Record
                   (Get_Integer_Value (Fetch_Value (Context, W_Parameter)),
                    Convert_To_String
                        (Get_String_Value
                             (Fetch_Value (Context, Name_Parameter))));
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Result));
         end;

      when 2220 =>
         --    procedure Xt_Add_Callback (W             : in Intrinsics.Widget;
         --                               Name          : in String;
         --                               Callback_Proc : in System.Address;
         --                               Closure       : in System.Address);

         declare

            W_Parameter             : Ref_Symbol_Table_Entry;
            Name_Parameter          : Ref_Symbol_Table_Entry;
            Callback_Proc_Parameter : Ref_Symbol_Table_Entry;
            Closure_Parameter       : Ref_Symbol_Table_Entry;

         begin
            W_Parameter             := Formal_Parameter;
            Name_Parameter          := Get_Next_Parameter (W_Parameter);
            Callback_Proc_Parameter := Get_Next_Parameter (Name_Parameter);
            Closure_Parameter := Get_Next_Parameter (Callback_Proc_Parameter);

            Ace_Widget_Package.Xt_Add_Callback
                (Get_Integer_Value (Fetch_Value (Context, W_Parameter)),
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Get_Integer_Value (Fetch_Value (Context,
                                                 Callback_Proc_Parameter)),
                 Get_Integer_Value (Fetch_Value (Context, Closure_Parameter)));
         end;

      when 2221 =>
         --    procedure Xt_Add_Callback (W             : in Intrinsics.Widget;
         --                               Name          : in String;
         --                               Callback_Proc : in System.Address;
         --                               Closure       : in Integer);

         declare

            W_Parameter             : Ref_Symbol_Table_Entry;
            Name_Parameter          : Ref_Symbol_Table_Entry;
            Callback_Proc_Parameter : Ref_Symbol_Table_Entry;
            Closure_Parameter       : Ref_Symbol_Table_Entry;

         begin
            W_Parameter             := Formal_Parameter;
            Name_Parameter          := Get_Next_Parameter (W_Parameter);
            Callback_Proc_Parameter := Get_Next_Parameter (Name_Parameter);
            Closure_Parameter := Get_Next_Parameter (Callback_Proc_Parameter);

            Ace_Widget_Package.Xt_Add_Callback
                (Get_Integer_Value (Fetch_Value (Context, W_Parameter)),
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Get_Integer_Value (Fetch_Value (Context,
                                                 Callback_Proc_Parameter)),
                 Get_Integer_Value (Fetch_Value (Context, Closure_Parameter)));
         end;

      when 2222 =>
         --    procedure Xt_Create_Widget (Out_Value   : out Intrinsics.Widget;
         --                                Name        : in String;
         --                                Widgetclass : in
         --                                   Intrinsics.Widget_Class;
         --                                Parent      : in Intrinsics.Widget;
         --                                Arg_List    : in Xt_Arg_List);

         declare

            Out_Value_Parameter   : Ref_Symbol_Table_Entry;
            Name_Parameter        : Ref_Symbol_Table_Entry;
            Widgetclass_Parameter : Ref_Symbol_Table_Entry;
            Parent_Parameter      : Ref_Symbol_Table_Entry;
            Arg_List_Parameter    : Ref_Symbol_Table_Entry;
            Out_Value_Result      : Ace_Integer;

         begin
            Out_Value_Parameter   := Formal_Parameter;
            Name_Parameter        := Get_Next_Parameter (Out_Value_Parameter);
            Widgetclass_Parameter := Get_Next_Parameter (Name_Parameter);
            Parent_Parameter      := Get_Next_Parameter (Widgetclass_Parameter);
            Arg_List_Parameter    := Get_Next_Parameter (Parent_Parameter);

            Ace_Widget_Package.Xt_Create_Widget
                (Out_Value_Result,
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Get_Integer_Value
                     (Fetch_Value (Context, Widgetclass_Parameter)),
                 Get_Integer_Value (Fetch_Value (Context, Parent_Parameter)),
                 Get_Integer_Value (Fetch_Value (Context, Arg_List_Parameter)));
            Store_Value (Context, Out_Value_Parameter,
                         Create_Storage_For_Integer (Out_Value_Result));
         end;

      when 2223 =>
         --    procedure Xt_Create_Managed_Widget (Out_Value   : out
         --                                           Intrinsics.Widget;
         --                                        Name        : in String;
         --                                        Widgetclass : in
         --                                           Intrinsics.Widget_Class;
         --                                        Parent      : in
         --                                           Intrinsics.Widget;
         --                                        Arg_List    : in
         --                                           Xt_Arg_List);

         declare

            Out_Value_Parameter   : Ref_Symbol_Table_Entry;
            Name_Parameter        : Ref_Symbol_Table_Entry;
            Widgetclass_Parameter : Ref_Symbol_Table_Entry;
            Parent_Parameter      : Ref_Symbol_Table_Entry;
            Arg_List_Parameter    : Ref_Symbol_Table_Entry;
            Out_Value_Result      : Ace_Integer;

         begin
            Out_Value_Parameter   := Formal_Parameter;
            Name_Parameter        := Get_Next_Parameter (Out_Value_Parameter);
            Widgetclass_Parameter := Get_Next_Parameter (Name_Parameter);
            Parent_Parameter      := Get_Next_Parameter (Widgetclass_Parameter);
            Arg_List_Parameter    := Get_Next_Parameter (Parent_Parameter);

            Ace_Widget_Package.Xt_Create_Managed_Widget
                (Out_Value_Result,
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Get_Integer_Value
                     (Fetch_Value (Context, Widgetclass_Parameter)),
                 Get_Integer_Value (Fetch_Value (Context, Parent_Parameter)),
                 Get_Integer_Value (Fetch_Value (Context, Arg_List_Parameter)));
            Store_Value (Context, Out_Value_Parameter,
                         Create_Storage_For_Integer (Out_Value_Result));
         end;

      when 2224 =>
         --    procedure Xt_Realize_Widget (W : in Intrinsics.Widget);

         Ace_Widget_Package.Xt_Realize_Widget
             (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter)));

      when 2225 =>
         --    procedure Xt_Main_Loop;

         Ace_Widget_Package.Xt_Main_Loop;

      when 2226 =>
         --    procedure Xt_Destroy_Widget (W : in Intrinsics.Widget);

         Ace_Widget_Package.Xt_Destroy_Widget
             (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter)));

      when 2227 =>
         --    function X_Text_Width
         --          (F : in X_Windows.Fonts.Font_Record;
         --           S : in String) return Intrinsics.Dimension;

         declare
            Result              : Ace_X_Windows.Pixels;
            Font_Info_Parameter : Ref_Symbol_Table_Entry;
            Text_Parameter      : Ref_Symbol_Table_Entry;
         begin
            Font_Info_Parameter := Formal_Parameter;
            Text_Parameter      := Get_Next_Parameter (Font_Info_Parameter);

            Result :=
               Ace_Widget_Package.X_Text_Width
                   (Get_Integer_Value (Fetch_Value (Context,
                                                    Font_Info_Parameter)),
                    Convert_To_String
                        (Get_String_Value
                             (Fetch_Value (Context, Text_Parameter))));
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Ace_Integer (Result)));
         end;

      when 2228 =>
         --    function Xt_Parse_Translation_Table (S : in String)
         --          return Xt_Translations;

         declare
            Result : Ace_Integer;
         begin
            Result :=
               Ace_Widget_Package.Xt_Parse_Translation_Table
                   (Convert_To_String
                        (Get_String_Value (Fetch_Value (Context,
                                                        Formal_Parameter))));
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Result));
         end;

      when 2229 =>
         --    procedure Xt_Override_Translations (W : in Intrinsics.Widget;
         --                                        T : in Xt_Translations);

         declare

            W_Parameter : Ref_Symbol_Table_Entry;
            T_Parameter : Ref_Symbol_Table_Entry;

         begin
            W_Parameter := Formal_Parameter;
            T_Parameter := Get_Next_Parameter (W_Parameter);

            Ace_Widget_Package.Xt_Override_Translations
                (Get_Integer_Value (Fetch_Value (Context, W_Parameter)),
                 Get_Integer_Value (Fetch_Value (Context, T_Parameter)));
         end;

      when 2230 =>
         --    procedure Xt_Augment_Translations (W : in Intrinsics.Widget;
         --                                       T : in Xt_Translations);

         declare

            W_Parameter : Ref_Symbol_Table_Entry;
            T_Parameter : Ref_Symbol_Table_Entry;

         begin
            W_Parameter := Formal_Parameter;
            T_Parameter := Get_Next_Parameter (W_Parameter);

            Ace_Widget_Package.Xt_Augment_Translations
                (Get_Integer_Value (Fetch_Value (Context, W_Parameter)),
                 Get_Integer_Value (Fetch_Value (Context, T_Parameter)));
         end;

      when 2231 =>
         --    procedure Xt_Add_Actions (Arg_List : in Xt_Arg_List);

         Ace_Widget_Package.Xt_Add_Actions
             (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter)));

      when 2232 =>
         --    function Action_Procedure_Pointer (Name: in String)
         --          return System.Address;

         declare
            Result : Ace_Integer;
         begin
            Result :=
               Ace_Widget_Package.Action_Procedure_Pointer
                   (Context,
                    Convert_To_String
                        (Get_String_Value (Fetch_Value (Context,
                                                        Formal_Parameter))));
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Result));
         end;

      when 2233 =>
         --    function Callback_Procedure_Pointer (Name: in String)
         --          return System.Address;

         declare
            Result : Ace_Integer;
         begin
            Result :=
               Ace_Widget_Package.Callback_Procedure_Pointer
                   (Context,
                    Convert_To_String
                        (Get_String_Value (Fetch_Value (Context,
                                                        Formal_Parameter))));
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Result));
         end;

      when 2234 =>
         --    function Xt_Default_App_Context return Intrinsics.Xt_App_Context;

         declare
            Result : Ace_Integer;
         begin
            Result := Ace_Widget_Package.Xt_Default_App_Context;
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Result));
         end;

      when 2235 =>
         --    function Xt_App_Next_Event (App : in Intrinsics.Xt_App_Context)
         --          return X_Windows.Events.Event;

         declare
            Result : Ace_Integer;
         begin
            Result :=
               Ace_Widget_Package.Xt_App_Next_Event
                   (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))
                    );
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Result));
         end;

      when 2236 =>
         --    procedure Xt_Dispatch_Event (E : in X_Windows.Events.Event);

         Ace_Widget_Package.Xt_Dispatch_Event
             (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter)));

      when 2301 =>
         -- function Xw_Arrow_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer (Ace_Hp_Widgets.Xw_Arrow_Widget_Class)
              );

      when 2302 =>
      -- function Xw_Bulletin_Board_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Bulletin_Board_Widget_Class));

      when 2303 =>
         -- function Xw_Bulletin_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Bulletin_Widget_Class));

      when 2304 =>
         -- function Xw_Button_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Button_Widget_Class));

      when 2305 =>
         -- function Xw_Cascade_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Cascade_Widget_Class));

      when 2306 =>
         -- function Xw_Form_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer (Ace_Hp_Widgets.Xw_Form_Widget_Class));

      when 2307 =>
         -- function Xw_Image_Edit_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Image_Edit_Widget_Class));

      when 2308 =>
         -- function Xw_List_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer (Ace_Hp_Widgets.Xw_List_Widget_Class));

      when 2309 =>
         -- function Xw_Listrow_Col_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Listrow_Col_Widget_Class));

      when 2310 =>
         -- function Xw_Manager_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Manager_Widget_Class));

      when 2311 =>
         -- function Xw_Menu_Button_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Menu_Button_Widget_Class));

      when 2312 =>
         -- function Xw_Menu_Sep_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Menu_Sep_Widget_Class));

      when 2313 =>
         -- function Xw_Menubutton_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Menubutton_Widget_Class));

      when 2314 =>
         -- function Xw_Menumgr_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Menumgr_Widget_Class));

      when 2315 =>
         -- function Xw_Menupane_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Menupane_Widget_Class));

      when 2316 =>
         -- function Xw_Panel_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer (Ace_Hp_Widgets.Xw_Panel_Widget_Class)
              );

      when 2317 =>
         -- function Xw_Popup_Mgr_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Popup_Mgr_Widget_Class));

      when 2318 =>
         -- function Xw_Popupmgr_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Popupmgr_Widget_Class));

      when 2319 =>
         -- function Xw_Primitive_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Primitive_Widget_Class));

      when 2320 =>
         -- function Xw_Push_Button_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Push_Button_Widget_Class));

      when 2321 =>
         -- function Xw_Row_Col_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Row_Col_Widget_Class));

      when 2322 =>
         -- function Xw_Sash_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer (Ace_Hp_Widgets.Xw_Sash_Widget_Class));

      when 2323 =>
         -- function Xw_Scroll_Bar_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Scroll_Bar_Widget_Class));

      when 2324 =>
         -- function Xw_Scrollbar_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Scrollbar_Widget_Class));

      when 2325 =>
     -- function Xw_Scrolled_Window_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Scrolled_Window_Widget_Class));

      when 2326 =>
         -- function Xw_Sraster_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Sraster_Widget_Class));

      when 2327 =>
       -- function Xw_Static_Raster_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Static_Raster_Widget_Class));

      when 2328 =>
         -- function Xw_Static_Text_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Static_Text_Widget_Class));

      when 2329 =>
         -- function Xw_Statictext_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Statictext_Widget_Class));

      when 2330 =>
         -- function Xw_Swindow_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Swindow_Widget_Class));

      when 2331 =>
         -- function Xw_Text_Edit_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Text_Edit_Widget_Class));

      when 2332 =>
         -- function Xw_Textedit_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Textedit_Widget_Class));

      when 2333 =>
         -- function Xw_Title_Bar_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Title_Bar_Widget_Class));

      when 2334 =>
         -- function Xw_Titlebar_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Titlebar_Widget_Class));

      when 2335 =>
         -- function Xw_Toggle_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Toggle_Widget_Class));

         -- Using Xw_V_Paned_Widget_Class creates a demands for a procedure
         -- called _XtCreateFontCursor, which does not seem to be in any
         -- X library (rhp, 5/2/90)

         -- when 2336 =>
         -- function Xw_V_Paned_Widget_Class return Intrinsics.Widget_Class;

         --    Store_Value
         --        (Context, Symtbl_Entry,
         --         Create_Storage_For_Integer
         --             (Ace_Hp_Widgets.Xw_V_Paned_Widget_Class));

      when 2337 =>
         -- function Xw_Valuator_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Valuator_Widget_Class));

      when 2338 =>
         -- function Xw_Work_Space_Widget_Class return Intrinsics.Widget_Class;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_Integer
                  (Ace_Hp_Widgets.Xw_Work_Space_Widget_Class));

      when 2401 =>
         --    procedure Xw_Text_Clear_Buffer (W : in Widget);

         Ace_Hp_Widgets.Xw_Text_Clear_Buffer
             (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter)));

      when 2402 =>
         --     function Xw_Text_Copy_Buffer (W : in Widget) return String;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_String
                  (Convert_To_Ref_String
                       (Ace_Hp_Widgets.Xw_Text_Copy_Buffer
                            (Get_Integer_Value (Fetch_Value (Context,
                                                             Formal_Parameter))
                             ))));

      when 2403 =>
         --     function Xw_Text_Copy_selection (W : in Widget) return String;

         Store_Value
             (Context, Symtbl_Entry,
              Create_Storage_For_String
                  (Convert_To_Ref_String
                       (Ace_Hp_Widgets.Xw_Text_Copy_Selection
                            (Get_Integer_Value (Fetch_Value (Context,
                                                             Formal_Parameter))
                             ))));

      when 2404 =>
         --    procedure Xw_Text_Read_Sub_String (W           : in Widget;
         --                                       Start_Pos   : in Integer;
         --                                       End_Pos     : in Integer;
         --                                       Target      : out String;
         --                                       Target_Used : out Integer;
         --                                       Source_Used : out Integer);

         declare

            W_Parameter           : Ref_Symbol_Table_Entry;
            Start_Pos_Parameter   : Ref_Symbol_Table_Entry;
            End_Pos_Parameter     : Ref_Symbol_Table_Entry;
            Target_Parameter      : Ref_Symbol_Table_Entry;
            Target_Used_Parameter : Ref_Symbol_Table_Entry;
            Source_Used_Parameter : Ref_Symbol_Table_Entry;
            Target_Used_Result    : Ace_Integer;
            Source_Used_Result    : Ace_Integer;
            Start_Pos_Value       : Ace_Integer;
            End_Pos_Value         : Ace_Integer;

         begin
            W_Parameter           := Formal_Parameter;
            Start_Pos_Parameter   := Get_Next_Parameter (W_Parameter);
            End_Pos_Parameter     := Get_Next_Parameter (Start_Pos_Parameter);
            Target_Parameter      := Get_Next_Parameter (End_Pos_Parameter);
            Target_Used_Parameter := Get_Next_Parameter (Target_Parameter);
            Source_Used_Parameter := Get_Next_Parameter (Target_Used_Parameter);

            Start_Pos_Value :=
               Get_Integer_Value (Fetch_Value (Context, Start_Pos_Parameter));
            End_Pos_Value :=
               Get_Integer_Value (Fetch_Value (Context, End_Pos_Parameter));

            declare

               Target_Result : String
                (Integer (Start_Pos_Value + 1) .. Integer (End_Pos_Value + 1));

               -- TBD fix strings

            begin
               Ace_Hp_Widgets.Xw_Text_Read_Sub_String
                   (Get_Integer_Value (Fetch_Value (Context, W_Parameter)),
                    Start_Pos_Value, End_Pos_Value, Target_Result,
                    Target_Used_Result, Source_Used_Result);
               Store_Value
                   (Context, Target_Parameter,
                    Create_Storage_For_String
                        (Convert_To_Ref_String (Target_Result)));
               Store_Value (Context, Target_Used_Parameter,
                            Create_Storage_For_Integer (Target_Used_Result));
               Store_Value (Context, Source_Used_Parameter,
                            Create_Storage_For_Integer (Source_Used_Result));
            end;
         end;

      when 2405 =>
         --    procedure Xw_Text_Unset_Selection (W : in Widget);

         Ace_Hp_Widgets.Xw_Text_Unset_Selection
             (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter)));

      when 2406 =>
         --    procedure Xw_Text_Set_Selection (W     : in Widget;
         --                                     Left  : in Xw_Text_Position;
         --                                     Right : in Xw_Text_Position);

         declare

            W_Parameter     : Ref_Symbol_Table_Entry;
            Left_Parameter  : Ref_Symbol_Table_Entry;
            Right_Parameter : Ref_Symbol_Table_Entry;

         begin
            W_Parameter     := Formal_Parameter;
            Left_Parameter  := Get_Next_Parameter (W_Parameter);
            Right_Parameter := Get_Next_Parameter (Left_Parameter);

            Ace_Hp_Widgets.Xw_Text_Set_Selection
                (Get_Integer_Value (Fetch_Value (Context, W_Parameter)),
                 Get_Integer_Value (Fetch_Value (Context, Left_Parameter)),
                 Get_Integer_Value (Fetch_Value (Context, Right_Parameter)));
         end;

      when 2407 =>
         --    function Xw_Text_Replace
         --          (W         : in Widget;
         --           Start_Pos : in Xw_Text_Position;
         --           End_Pos   : in Xw_Text_Position;
         --           S         : in String) return Xw_Edit_Result;

         declare

            W_Parameter         : Ref_Symbol_Table_Entry;
            Start_Pos_Parameter : Ref_Symbol_Table_Entry;
            End_Pos_Parameter   : Ref_Symbol_Table_Entry;
            S_Parameter         : Ref_Symbol_Table_Entry;

            Result : Hp_Widgets.Xw_Edit_Result;

         begin
            W_Parameter         := Formal_Parameter;
            Start_Pos_Parameter := Get_Next_Parameter (W_Parameter);
            End_Pos_Parameter   := Get_Next_Parameter (Start_Pos_Parameter);
            S_Parameter         := Get_Next_Parameter (End_Pos_Parameter);

            Result :=
               Ace_Hp_Widgets.Xw_Text_Replace
                   (Get_Integer_Value (Fetch_Value (Context, W_Parameter)),
                    Get_Integer_Value (Fetch_Value (Context,
                                                    Start_Pos_Parameter)),
                    Get_Integer_Value
                        (Fetch_Value (Context, End_Pos_Parameter)),
                    Convert_To_String
                        (Get_String_Value (Fetch_Value (Context, S_Parameter)))
                    );
            Store_Value
                (Context, Symtbl_Entry,
                 Create_Storage_For_Integer
                     (Hp_Widgets.Xw_Edit_Result'POS (Result)));
         end;

      when 2408 =>
         --    procedure Xw_Text_Redraw (W : in Widget);

         Ace_Hp_Widgets.Xw_Text_Redraw
             (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter)));

      when 2409 =>
         --       procedure Xw_Text_Update (W      : in Widget;
         --                                 Status : in Boolean);

         declare

            W_Parameter      : Ref_Symbol_Table_Entry;
            Status_Parameter : Ref_Symbol_Table_Entry;

         begin
            W_Parameter      := Formal_Parameter;
            Status_Parameter := Get_Next_Parameter (W_Parameter);

            Ace_Hp_Widgets.Xw_Text_Update
                (Get_Integer_Value (Fetch_Value (Context, W_Parameter)),
                 Boolean'VAL (
                    Get_Integer_Value (Fetch_Value (Context, Status_Parameter))
                    ));
         end;

      when 2410 =>
         --       procedure Xw_Text_Insert (W : in Widget;
         --                                 S : in String);

         declare

            W_Parameter : Ref_Symbol_Table_Entry;
            S_Parameter : Ref_Symbol_Table_Entry;

         begin
            W_Parameter := Formal_Parameter;
            S_Parameter := Get_Next_Parameter (W_Parameter);

            Ace_Hp_Widgets.Xw_Text_Insert
                (Get_Integer_Value (Fetch_Value (Context, W_Parameter)),
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, S_Parameter))));
         end;

      when 2411 =>
         --    function Xw_Text_Get_Last_Pos
         --          (W : in Widget) return Xw_Text_Position;

         declare

            Result : Ace_Integer;

         begin
            Result :=
               Ace_Hp_Widgets.Xw_Text_Get_Last_Pos
                   (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))
                    );
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Result));
         end;

      when 2412 =>
         --    procedure Xw_Text_Get_Selection_Pos
         --          (W     : in Widget;
         --           Left  : out Xw_Text_Position;
         --           Right : out Xw_Text_Position);

         declare

            W_Parameter     : Ref_Symbol_Table_Entry;
            Left_Parameter  : Ref_Symbol_Table_Entry;
            Right_Parameter : Ref_Symbol_Table_Entry;
            Left_Result     : Ace_Integer;
            Right_Result    : Ace_Integer;

         begin
            W_Parameter     := Formal_Parameter;
            Left_Parameter  := Get_Next_Parameter (W_Parameter);
            Right_Parameter := Get_Next_Parameter (Left_Parameter);

            Ace_Hp_Widgets.Xw_Text_Get_Selection_Pos
                (Get_Integer_Value (Fetch_Value (Context, W_Parameter)),
                 Left_Result, Right_Result);

            Store_Value (Context, Left_Parameter,
                         Create_Storage_For_Integer (Left_Result));
            Store_Value (Context, Right_Parameter,
                         Create_Storage_For_Integer (Right_Result));
         end;

      when 2413 =>
         --    procedure Xw_Text_Set_Insert_Pos (W   : in Widget;
         --                                      Pos : in Xw_Text_Position);

         declare

            W_Parameter   : Ref_Symbol_Table_Entry;
            Pos_Parameter : Ref_Symbol_Table_Entry;

         begin
            W_Parameter   := Formal_Parameter;
            Pos_Parameter := Get_Next_Parameter (W_Parameter);

            Ace_Hp_Widgets.Xw_Text_Set_Insert_Pos
                (Get_Integer_Value (Fetch_Value (Context, W_Parameter)),
                 Get_Integer_Value (Fetch_Value (Context, Pos_Parameter)));
         end;

      when 2414 =>
         --    function Xw_Text_Get_Insert_Pos
         --          (W : in Widget) return Xw_Text_Position;

         declare

            Result : Ace_Integer;

         begin
            Result :=
               Ace_Hp_Widgets.Xw_Text_Get_Insert_Pos
                   (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter))
                    );
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Result));
         end;

      when 2415 =>
         --    procedure Xw_Text_Set_Source (W         : in Widget;
         --                                  Source    : in Xw_Text_Source_Ptr;
         --                                  Start_Pos : in Xw_Text_Position);

         declare

            W_Parameter         : Ref_Symbol_Table_Entry;
            Source_Parameter    : Ref_Symbol_Table_Entry;
            Start_Pos_Parameter : Ref_Symbol_Table_Entry;

         begin
            W_Parameter         := Formal_Parameter;
            Source_Parameter    := Get_Next_Parameter (W_Parameter);
            Start_Pos_Parameter := Get_Next_Parameter (Source_Parameter);

            Ace_Hp_Widgets.Xw_Text_Set_Source
                (Get_Integer_Value (Fetch_Value (Context, W_Parameter)),
                 Get_Integer_Value (Fetch_Value (Context, Source_Parameter)),
                 Get_Integer_Value (Fetch_Value (Context, Start_Pos_Parameter))
                 );
         end;

      when 2416 =>
         --    function Xw_Ascii_Sink_Create
         --          (W    : in Widget;
         --           Args : in Xt_Arg_List) return Xw_Text_Sink_Ptr;

         declare

            W_Parameter    : Ref_Symbol_Table_Entry;
            Args_Parameter : Ref_Symbol_Table_Entry;
            Result         : Ace_Integer;

         begin
            W_Parameter    := Formal_Parameter;
            Args_Parameter := Get_Next_Parameter (W_Parameter);

            Result :=
               Ace_Hp_Widgets.Xw_Ascii_Sink_Create
                   (Get_Integer_Value (Fetch_Value (Context, W_Parameter)),
                    Get_Integer_Value (Fetch_Value (Context, Args_Parameter)));
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Result));
         end;

      when 2417 =>
         --    function Xw_Disk_Source_Create
         --          (W    : in Widget;
         --           Args : in Xt_Arg_List) return Xw_Text_Source_Ptr;

         declare

            W_Parameter    : Ref_Symbol_Table_Entry;
            Args_Parameter : Ref_Symbol_Table_Entry;
            Result         : Ace_Integer;

         begin
            W_Parameter    := Formal_Parameter;
            Args_Parameter := Get_Next_Parameter (W_Parameter);

            Result :=
               Ace_Hp_Widgets.Xw_Disk_Source_Create
                   (Get_Integer_Value (Fetch_Value (Context, W_Parameter)),
                    Get_Integer_Value (Fetch_Value (Context, Args_Parameter)));
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Result));
         end;

      when 2418 =>
         --    procedure Xw_Disk_Source_Destroy (Src : in Xw_Text_Source_Ptr);

         Ace_Hp_Widgets.Xw_Disk_Source_Destroy
             (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter)));

      when 2419 =>
         --    function Xw_String_Source_Create
         --          (W    : in Widget;
         --           Args : in Xt_Arg_List) return Xw_Text_Source_Ptr;

         declare

            W_Parameter    : Ref_Symbol_Table_Entry;
            Args_Parameter : Ref_Symbol_Table_Entry;
            Result         : Ace_Integer;

         begin
            W_Parameter    := Formal_Parameter;
            Args_Parameter := Get_Next_Parameter (W_Parameter);

            Result :=
               Ace_Hp_Widgets.Xw_String_Source_Create
                   (Get_Integer_Value (Fetch_Value (Context, W_Parameter)),
                    Get_Integer_Value (Fetch_Value (Context, Args_Parameter)));
            Store_Value (Context, Symtbl_Entry,
                         Create_Storage_For_Integer (Result));
         end;

      when 2420 =>
         --    procedure Xw_String_Source_Destroy (Src : in Xw_Text_Source_Ptr);

         Ace_Hp_Widgets.Xw_String_Source_Destroy
             (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter)));

      when 2421 =>
         --    procedure Xw_Move_Focus (W : in Widget);

         Ace_Hp_Widgets.Xw_Move_Focus
             (Get_Integer_Value (Fetch_Value (Context, Formal_Parameter)));

      when 2422 =>
         --    procedure Xt_Set_Arg (Arg   : out Xt_Arg;
         --                          Name  : in String;
         --                          Value : in Xw_Alignment);

         declare

            Arg_Parameter   : Ref_Symbol_Table_Entry;
            Name_Parameter  : Ref_Symbol_Table_Entry;
            Value_Parameter : Ref_Symbol_Table_Entry;
            Arg_Result      : Ace_Integer;

         begin
            Arg_Parameter   := Formal_Parameter;
            Name_Parameter  := Get_Next_Parameter (Arg_Parameter);
            Value_Parameter := Get_Next_Parameter (Name_Parameter);

            Ace_Hp_Widgets.Xt_Set_Arg
                (Arg_Result,
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Hp_Widgets.Xw_Alignment'VAL (
                    Get_Integer_Value (Fetch_Value (Context, Value_Parameter)))
                 );
            Store_Value (Context, Arg_Parameter,
                         Create_Storage_For_Integer (Arg_Result));
         end;

      when 2423 =>
         --    procedure Xt_Set_Arg (Arg   : out Xt_Arg;
         --                          Name  : in String;
         --                          Value : in Xw_Widget_Type);

         declare

            Arg_Parameter   : Ref_Symbol_Table_Entry;
            Name_Parameter  : Ref_Symbol_Table_Entry;
            Value_Parameter : Ref_Symbol_Table_Entry;
            Arg_Result      : Ace_Integer;

         begin
            Arg_Parameter   := Formal_Parameter;
            Name_Parameter  := Get_Next_Parameter (Arg_Parameter);
            Value_Parameter := Get_Next_Parameter (Name_Parameter);

            Ace_Hp_Widgets.Xt_Set_Arg
                (Arg_Result,
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Hp_Widgets.Xw_Widget_Type'VAL (
                    Get_Integer_Value (Fetch_Value (Context, Value_Parameter)))
                 );
            Store_Value (Context, Arg_Parameter,
                         Create_Storage_For_Integer (Arg_Result));
         end;

      when 2424 =>
         --    procedure Xt_Set_Arg (Arg   : out Xt_Arg;
         --                          Name  : in String;
         --                          Value : in Xw_Scan_Direction);

         declare

            Arg_Parameter   : Ref_Symbol_Table_Entry;
            Name_Parameter  : Ref_Symbol_Table_Entry;
            Value_Parameter : Ref_Symbol_Table_Entry;
            Arg_Result      : Ace_Integer;

         begin
            Arg_Parameter   := Formal_Parameter;
            Name_Parameter  := Get_Next_Parameter (Arg_Parameter);
            Value_Parameter := Get_Next_Parameter (Name_Parameter);

            Ace_Hp_Widgets.Xt_Set_Arg
                (Arg_Result,
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Hp_Widgets.Xw_Scan_Direction'VAL (
                    Get_Integer_Value (Fetch_Value (Context, Value_Parameter)))
                 );
            Store_Value (Context, Arg_Parameter,
                         Create_Storage_For_Integer (Arg_Result));
         end;

      when 2425 =>
         --    procedure Xt_Set_Arg (Arg   : out Xt_Arg;
         --                          Name  : in String;
         --                          Value : in Xw_Scan_Type);

         declare

            Arg_Parameter   : Ref_Symbol_Table_Entry;
            Name_Parameter  : Ref_Symbol_Table_Entry;
            Value_Parameter : Ref_Symbol_Table_Entry;
            Arg_Result      : Ace_Integer;

         begin
            Arg_Parameter   := Formal_Parameter;
            Name_Parameter  := Get_Next_Parameter (Arg_Parameter);
            Value_Parameter := Get_Next_Parameter (Name_Parameter);

            Ace_Hp_Widgets.Xt_Set_Arg
                (Arg_Result,
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Hp_Widgets.Xw_Scan_Type'VAL (
                    Get_Integer_Value (Fetch_Value (Context, Value_Parameter)))
                 );
            Store_Value (Context, Arg_Parameter,
                         Create_Storage_For_Integer (Arg_Result));
         end;

      when 2426 =>
         --    procedure Xt_Set_Arg (Arg   : out Xt_Arg;
         --                          Name  : in String;
         --                          Value : in Xw_Edit_Type);

         declare

            Arg_Parameter   : Ref_Symbol_Table_Entry;
            Name_Parameter  : Ref_Symbol_Table_Entry;
            Value_Parameter : Ref_Symbol_Table_Entry;
            Arg_Result      : Ace_Integer;

         begin
            Arg_Parameter   := Formal_Parameter;
            Name_Parameter  := Get_Next_Parameter (Arg_Parameter);
            Value_Parameter := Get_Next_Parameter (Name_Parameter);

            Ace_Hp_Widgets.Xt_Set_Arg
                (Arg_Result,
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Hp_Widgets.Xw_Edit_Type'VAL (
                    Get_Integer_Value (Fetch_Value (Context, Value_Parameter)))
                 );
            Store_Value (Context, Arg_Parameter,
                         Create_Storage_For_Integer (Arg_Result));
         end;

      when 2427 =>
         --    procedure Xt_Set_Arg (Arg   : out Xt_Arg;
         --                          Name  : in String;
         --                          Value : in Xw_Edit_Result);

         declare

            Arg_Parameter   : Ref_Symbol_Table_Entry;
            Name_Parameter  : Ref_Symbol_Table_Entry;
            Value_Parameter : Ref_Symbol_Table_Entry;
            Arg_Result      : Ace_Integer;

         begin
            Arg_Parameter   := Formal_Parameter;
            Name_Parameter  := Get_Next_Parameter (Arg_Parameter);
            Value_Parameter := Get_Next_Parameter (Name_Parameter);

            Ace_Hp_Widgets.Xt_Set_Arg
                (Arg_Result,
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Hp_Widgets.Xw_Edit_Result'VAL (
                    Get_Integer_Value (Fetch_Value (Context, Value_Parameter)))
                 );
            Store_Value (Context, Arg_Parameter,
                         Create_Storage_For_Integer (Arg_Result));
         end;

      when 2428 =>
         --    procedure Xt_Set_Arg (Arg   : out Xt_Arg;
         --                          Name  : in String;
         --                          Value : in Xw_Verify_Op_Type);

         declare

            Arg_Parameter   : Ref_Symbol_Table_Entry;
            Name_Parameter  : Ref_Symbol_Table_Entry;
            Value_Parameter : Ref_Symbol_Table_Entry;
            Arg_Result      : Ace_Integer;

         begin
            Arg_Parameter   := Formal_Parameter;
            Name_Parameter  := Get_Next_Parameter (Arg_Parameter);
            Value_Parameter := Get_Next_Parameter (Name_Parameter);

            Ace_Hp_Widgets.Xt_Set_Arg
                (Arg_Result,
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Hp_Widgets.Xw_Verify_Op_Type'VAL (
                    Get_Integer_Value (Fetch_Value (Context, Value_Parameter)))
                 );
            Store_Value (Context, Arg_Parameter,
                         Create_Storage_For_Integer (Arg_Result));
         end;

      when 2429 =>
         --    procedure Xt_Set_Arg (Arg   : out Xt_Arg;
         --                          Name  : in String;
         --                          Value : in Xw_Source_Type);

         declare

            Arg_Parameter   : Ref_Symbol_Table_Entry;
            Name_Parameter  : Ref_Symbol_Table_Entry;
            Value_Parameter : Ref_Symbol_Table_Entry;
            Arg_Result      : Ace_Integer;

         begin
            Arg_Parameter   := Formal_Parameter;
            Name_Parameter  := Get_Next_Parameter (Arg_Parameter);
            Value_Parameter := Get_Next_Parameter (Name_Parameter);

            Ace_Hp_Widgets.Xt_Set_Arg
                (Arg_Result,
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Hp_Widgets.Xw_Source_Type'VAL (
                    Get_Integer_Value (Fetch_Value (Context, Value_Parameter)))
                 );
            Store_Value (Context, Arg_Parameter,
                         Create_Storage_For_Integer (Arg_Result));
         end;

      when 2430 =>
         --    procedure Xt_Set_Arg (Arg   : out Xt_Arg;
         --                          Name  : in String;
         --                          Value : in Xw_Wrap);

         declare

            Arg_Parameter   : Ref_Symbol_Table_Entry;
            Name_Parameter  : Ref_Symbol_Table_Entry;
            Value_Parameter : Ref_Symbol_Table_Entry;
            Arg_Result      : Ace_Integer;

         begin
            Arg_Parameter   := Formal_Parameter;
            Name_Parameter  := Get_Next_Parameter (Arg_Parameter);
            Value_Parameter := Get_Next_Parameter (Name_Parameter);

            Ace_Hp_Widgets.Xt_Set_Arg
                (Arg_Result,
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Hp_Widgets.Xw_Wrap'VAL (
                    Get_Integer_Value (Fetch_Value (Context, Value_Parameter)))
                 );
            Store_Value (Context, Arg_Parameter,
                         Create_Storage_For_Integer (Arg_Result));
         end;

      when 2431 =>
         --    procedure Xt_Set_Arg (Arg   : out Xt_Arg;
         --                          Name  : in String;
         --                          Value : in Xw_Wrap_Form);

         declare

            Arg_Parameter   : Ref_Symbol_Table_Entry;
            Name_Parameter  : Ref_Symbol_Table_Entry;
            Value_Parameter : Ref_Symbol_Table_Entry;
            Arg_Result      : Ace_Integer;

         begin
            Arg_Parameter   := Formal_Parameter;
            Name_Parameter  := Get_Next_Parameter (Arg_Parameter);
            Value_Parameter := Get_Next_Parameter (Name_Parameter);

            Ace_Hp_Widgets.Xt_Set_Arg
                (Arg_Result,
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Hp_Widgets.Xw_Wrap_Form'VAL (
                    Get_Integer_Value (Fetch_Value (Context, Value_Parameter)))
                 );
            Store_Value (Context, Arg_Parameter,
                         Create_Storage_For_Integer (Arg_Result));
         end;

      when 2432 =>
         --    procedure Xt_Set_Arg (Arg   : out Xt_Arg;
         --                          Name  : in String;
         --                          Value : in Xw_Wrap_Break);

         declare

            Arg_Parameter   : Ref_Symbol_Table_Entry;
            Name_Parameter  : Ref_Symbol_Table_Entry;
            Value_Parameter : Ref_Symbol_Table_Entry;
            Arg_Result      : Ace_Integer;

         begin
            Arg_Parameter   := Formal_Parameter;
            Name_Parameter  := Get_Next_Parameter (Arg_Parameter);
            Value_Parameter := Get_Next_Parameter (Name_Parameter);

            Ace_Hp_Widgets.Xt_Set_Arg
                (Arg_Result,
                 Convert_To_String
                     (Get_String_Value (Fetch_Value (Context, Name_Parameter))),
                 Hp_Widgets.Xw_Wrap_Break'VAL (
                    Get_Integer_Value (Fetch_Value (Context, Value_Parameter)))
                 );
            Store_Value (Context, Arg_Parameter,
                         Create_Storage_For_Integer (Arg_Result));
         end;

         -- ..... END OF BUILT-IN ROUTINES .....

      when others    =>
         Show_Error (1990, Internal, "Evaluate_Built_In_Subprogram");

   end case;

end Evaluate_Built_In_Subprogram;

