--|$Log:	eexpr.a,v $
--|Revision 8.0  91/01/17  10:45:33  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Statement_Evaluator
--
---------------------------------------------------------------
--
--
-- Abstract
--
--  The following routines are separate routines for
--  Statement_Evaluator.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
--==========================================================================
--  <Subprogram Name>
--
--    Evaluate_Expression
--
--  <Description>
--
--    Evaluates the given expression node structures.
--
--==========================================================================
with Node_Set_Routines;
use Node_Set_Routines;

separate (Statement_Evaluator.Evaluate_Statement.Evaluate_Individual_Statement)

function Evaluate_Expression
    (Node : in Ref_Statement_Database) return Ref_Statement_Database is

   Subprogram_Name : constant String := "Evaluate_Expression";

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Evaluate_Attribute
   --
   --  <Description>
   --
   --    Evaluates the given expression with repect to an attribute.
   --
   --==========================================================================
   function Evaluate_Attribute
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
   separate;

begin

   case Check_Expr_Node (Node) is

      --------------------

      when Terminal_Node  =>
         declare
            Tmp_Value : Ref_Data_Storage :=
               Fetch_Value (Context, Get_Symbol_Table_Entry (Node));
         begin
            Set_Value (Node, Copy_Value (Tmp_Value));
            -- If this value was created for an enumerated value then we must
            -- free it.
            if Get_Entry_Kind (Get_Symbol_Table_Entry (Node)) =
               Enumeration_Literal_Entry then
               Free (Tmp_Value);
            end if;
         end;

      when Dot_Operator   =>
         if Get_Entry_Kind (Get_Symbol_Table_Entry (Node)) = Function_Entry
             then
            return (Evaluate_Subprogram_Call (Get_Database_Node (Node)));
         else
            -- copy node.symtbl_entry.value into node.value
            -- we must Copy_Value here!
            Set_Value
                (Node,
                 Copy_Value (Fetch_Value (Context,
                                          Get_Symbol_Table_Entry (Node))));
            -- afterwards, node_value = value of terminal
         end if;

      when Constant_Node  =>
         -- it's a constant,
         --    and node.value already contains the constant's value
         null;

      when String_Node    =>
         -- it's a string literal,
         --    and node.value already contains the string's "value"
         null;

      when Character_Node =>
         -- it's a character literal,
         --    and node.value already contains the character's "value"
         null;

      when Function_Call  =>
         return (Evaluate_Subprogram_Call (Node));
         -- afterwards, node_value = returned value of function call

      when Type_Conversion     =>
         -- we must Copy_Value here!
         Set_Value
             (Node,
              Copy_Value
                  (Get_Value
                       (Evaluate_Expression
                            (Get_Expression (Get_List_Of_Items (Node))))));
         -- afterwards, node_value = value of converted object type

      when List_Item =>
         -- evaluate each item
         -- set node_value to last list expr evaluated
         --    [need to do this because of the way lists are built
         --     (instead of normal expr nodes) in parenthesized exprs]
         declare
            List_Expr_Result, List_Node_Item : Ref_Statement_Database;
            List_Node : Ref_Statement_Database := Node;
         begin
            while (not Is_Empty (List_Node)) loop
               List_Node_Item := Get_Expression (List_Node);
               List_Expr_Result := Evaluate_Expression (List_Node_Item);
               List_Node := Get_Next_List_Item (List_Node);
            end loop;
            -- we must Copy_Value here!
            Set_Value (Node, Copy_Value (Get_Value (List_Expr_Result)));
         end;
         -- afterwards, node_value = value of last list expr evaluated

      when Aggregate_Component =>
         -- eval aggr expr + ??
         -- for named exprs ??
         null;
         -- afterwards, node_value = ??

      when Range_Expression    =>
         -- more to do than evaluate UB and LB ??
         declare
            Ignored_Result : Ref_Statement_Database;
         begin
            Ignored_Result := Evaluate_Expression (Get_Lower_Bound (Node));
            Ignored_Result := Evaluate_Expression (Get_Upper_Bound (Node));
         end;
         -- afterwards, node_value = null (no access to it will occur)

         --------------------

      when Attribute_Tick =>
         -- Evaluate the expression and apply the specific attribute to the
         -- expression.
         return Evaluate_Attribute (Node);

      when Array_Reference     =>
         -- not implementing these
         Show_Error (1999, Internal, Subprogram_Name);

         --------------------

      when Record_Reference    | Dot_Notation_Variable_Reference  =>
         -- can't be these here!
         -- (semantic analyzer resolved/reduced this to terminal_node ref)
         Show_Error (1000, Internal, Subprogram_Name);

      when Left_Parenthesis_Operator     =>
         -- can't be these here!
         Show_Error (1000, Internal, Subprogram_Name);

         --------------------

      when Unknown   =>
         Show_Error (1003, Internal, Subprogram_Name);
   end case;

   --------------------
   return (Node);
   --------------------

end Evaluate_Expression;
