--|$Log:	einfix.a,v $
--|Revision 8.0  91/01/17  10:45:34  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Subunit for Package - Statement_Evaluator.Evaluate_Statement.
--                       Evaluate_Individual_Statement.Evaluate_Subprogram_Call
--
---------------------------------------------------------------
--
--
-- Abstract
--
--   The following routine is a separate routine for
--   Statement_Evaluator.Evaluate_Statement.
--   Evaluate_Individual_Statement.Evaluate_Subprogram_Call.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
--==========================================================================
--  <Subprogram Name>
--
--    Evaluate_Infix_Operation
--
--  <Description>
--
--    Evaluates predefined infix operations.
--
--==========================================================================
with Node_Set_Routines;
use Node_Set_Routines;

with String_Constants;
use String_Constants;

separate (
   Statement_Evaluator.Evaluate_Statement.Evaluate_Individual_Statement.
      Evaluate_Subprogram_Call)

function Evaluate_Infix_Operation
    (Node : in Ref_Statement_Database) return Ref_Statement_Database is

   Subprogram_Name : Ref_String :=
      Get_Symbolic_Name (Get_Subprogram_Info (Node));
   -- Subprogram_Symtbl_Entry : Ref_Symbol_Table_Entry :=
   -- Get_Symbol_Table_Entry (Get_Subprogram_Info (Node));

   Current_Node    : Ref_Statement_Database := Get_Parameter_List (Node);

   -- [tbd] make value_result a linked list
   subtype Value_Result_Range is Ace_Natural range 0 .. 100;
   Top             : constant Value_Result_Range := Value_Result_Range'FIRST;
   Value_Result    : array (Value_Result_Range) of Ref_Data_Storage;
   Argument_Count  : Value_Result_Range          := Top;

begin

   -----
   -- evaluate arguments
   ---- Note:
   -- use of Actuals_Interim_Storage and this subprogram constitute
   -- a slight-of-hand use of Ada's block structured facility
   -- as ACE's run-time storage stack -- providing implicit
   -- data storage for recursion and dynamic scoping of ACE subprograms
   -- [this is done in Evaluate_Subprogram_Call also]
   ----
   while (not Is_Empty (Current_Node)) loop
      Argument_Count := Argument_Count + 1;
      Value_Result (Argument_Count) :=
         Copy_Value
             (Get_Value (Evaluate_Expression (Get_Expression (Current_Node))));
      Current_Node := Get_Next_List_Item (Current_Node);
   end loop;
   -----

   -----
   -- perform infix operation
   -----

   Subprogram_Name := Get_Symbolic_Name (Get_Subprogram_Info (Node));
   if (Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Plus_String)) then
      Set_Value
          (Node,
           Create_Storage_For_Integer
               (Get_Integer_Value (Value_Result (Top + 1)) +
                Get_Integer_Value (Value_Result (Top + 2))));

   elsif (Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Minus_String)) then
      Set_Value
          (Node,
           Create_Storage_For_Integer
               (Get_Integer_Value (Value_Result (Top + 1)) -
                Get_Integer_Value (Value_Result (Top + 2))));

   elsif (Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Times_String)) then
      Set_Value
          (Node,
           Create_Storage_For_Integer
               (Get_Integer_Value (Value_Result (Top + 1)) *
                Get_Integer_Value (Value_Result (Top + 2))));

   elsif (Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Divide_String)) then
      Set_Value
          (Node,
           Create_Storage_For_Integer
               (Get_Integer_Value (Value_Result (Top + 1)) /
                Get_Integer_Value (Value_Result (Top + 2))));

   elsif (Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Mod_String)) then
      Set_Value
          (Node,
           Create_Storage_For_Integer
               (Get_Integer_Value (Value_Result (Top + 1)) mod
                Get_Integer_Value (Value_Result (Top + 2))));

   elsif (Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Rem_String)) then
      Set_Value
          (Node,
           Create_Storage_For_Integer
               (Get_Integer_Value (Value_Result (Top + 1)) rem
                Get_Integer_Value (Value_Result (Top + 2))));

   elsif (Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Exp_String)) then
      Set_Value
          (Node,
           Create_Storage_For_Integer
               (Ace_Integer
                    (Integer (Get_Integer_Value (Value_Result (Top + 1))) **
                     Integer (Get_Integer_Value (Value_Result (Top + 2))))));

   elsif (Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Abs_String)) then
      Set_Value
          (Node,
           Create_Storage_For_Integer
               (abs Get_Integer_Value (Value_Result (Top + 1))));

   elsif (Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Concat_String)) then

      case Get_Data_Type (Value_Result (Top + 1)) is
         when String_Data =>
            case Get_Data_Type (Value_Result (Top + 2)) is
               when String_Data     =>
                  Set_Value
                      (Node,
                       Create_Storage_For_String
                           (Convert_To_Ref_String
                                (Convert_To_String
                                     (Get_String_Value (Value_Result (Top + 1))
                                      ) &
                                 Convert_To_String
                                     (Get_String_Value (Value_Result (Top + 2))
                                      ))));
               when Char_Data  =>
                  Set_Value
                      (Node,
                       Create_Storage_For_String
                           (Convert_To_Ref_String
                                (Convert_To_String
                                     (Get_String_Value (Value_Result (Top + 1))
                                      ) &
                                 Get_Character_Value (Value_Result (Top + 2))))
                       );
               when others     =>
                  Show_Error (1111, Fatal, "&");
            end case;
         when Char_Data   =>
            case Get_Data_Type (Value_Result (Top + 2)) is
               when String_Data     =>
                  Set_Value
                      (Node,
                       Create_Storage_For_String
                           (Convert_To_Ref_String
                                ((Get_Character_Value (Value_Result (Top + 1)))
                                  &
                                 (Convert_To_String
                                     (Get_String_Value (Value_Result (Top + 2))
                                      )))));
               when Char_Data  =>
                  Set_Value
                      (Node,
                       Create_Storage_For_String
                           (Convert_To_Ref_String
                                ((Get_Character_Value (Value_Result (Top + 1)))
                                  &
                                 (Get_Character_Value (Value_Result (Top + 2)))
                                 )));
               when others     =>
                  Show_Error (1111, Fatal, "&");
            end case;
         when others =>
            Show_Error (1111, Fatal, "&");
      end case;

   elsif (Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Equal_String)) then
      case Get_Data_Type (Value_Result (Top + 1)) is

         when Integer_Data     =>
            if (Get_Integer_Value (Value_Result (Top + 1)) =
               Get_Integer_Value (Value_Result (Top + 2))) then
               Set_Value (Node, Create_Storage_For_Integer (1));
            else
               Set_Value (Node, Create_Storage_For_Integer (0));
            end if;
         when String_Data =>
            if (Get_String_Value (Value_Result (Top + 1)) =
               Get_String_Value (Value_Result (Top + 2))) then
               Set_Value (Node, Create_Storage_For_Integer (1));
            else
               Set_Value (Node, Create_Storage_For_Integer (0));
            end if;
         when Char_Data   =>
            if (Get_Character_Value (Value_Result (Top + 1)) =
               Get_Character_Value (Value_Result (Top + 2))) then
               Set_Value (Node, Create_Storage_For_Integer (1));
            else
               Set_Value (Node, Create_Storage_For_Integer (0));
            end if;
         when Float_Data  =>
            --         if (Get_Float_Value (Value_Result (Top + 1)) =
            --            Get_Float_Value (Value_Result (Top + 2))) then
            --            Set_Value (Node, Create_Storage_For_Integer (1));
            --         else
            --            Set_Value (Node, Create_Storage_For_Integer (0));
            --         end if;
            null;
         when Array_Data => null;
         when Void   =>
            null;
      end case;

   elsif (Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Not_Equal_String))
          then
      case Get_Data_Type (Value_Result (Top + 1)) is
         when Integer_Data     =>
            if (Get_Integer_Value (Value_Result (Top + 1)) /=
               Get_Integer_Value (Value_Result (Top + 2))) then
               Set_Value (Node, Create_Storage_For_Integer (1));
            else
               Set_Value (Node, Create_Storage_For_Integer (0));
            end if;
         when String_Data =>
            if (Get_String_Value (Value_Result (Top + 1)) /=
               Get_String_Value (Value_Result (Top + 2))) then
               Set_Value (Node, Create_Storage_For_Integer (1));
            else
               Set_Value (Node, Create_Storage_For_Integer (0));
            end if;
         when Char_Data   =>
            if (Get_Character_Value (Value_Result (Top + 1)) /=
               Get_Character_Value (Value_Result (Top + 2))) then
               Set_Value (Node, Create_Storage_For_Integer (1));
            else
               Set_Value (Node, Create_Storage_For_Integer (0));
            end if;
         when Float_Data  =>
            --         if (Get_Float_Value (Value_Result (Top + 1)) /=
            --            Get_Float_Value (Value_Result (Top + 2))) then
            --            Set_Value (Node, Create_Storage_For_Integer (1));
            --         else
            --            Set_Value (Node, Create_Storage_For_Integer (0));
            --         end if;
            null;
         when Array_Data => null;
         when Void   =>
            null;
      end case;

   elsif (Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Lessthan_String)) then
      case Get_Data_Type (Value_Result (Top + 1)) is
         when Integer_Data     =>
            if (Get_Integer_Value (Value_Result (Top + 1)) <
               Get_Integer_Value (Value_Result (Top + 2))) then
               Set_Value (Node, Create_Storage_For_Integer (1));
            else
               Set_Value (Node, Create_Storage_For_Integer (0));
            end if;
         when String_Data =>
            if (Convert_To_String (Get_String_Value (Value_Result (Top + 1)))
                < Convert_To_String (Get_String_Value (Value_Result (Top + 2)))
               ) then
               Set_Value (Node, Create_Storage_For_Integer (1));
            else
               Set_Value (Node, Create_Storage_For_Integer (0));
            end if;
         when Char_Data   =>
            if (Get_Character_Value (Value_Result (Top + 1)) <
               Get_Character_Value (Value_Result (Top + 2))) then
               Set_Value (Node, Create_Storage_For_Integer (1));
            else
               Set_Value (Node, Create_Storage_For_Integer (0));
            end if;
         when Float_Data  =>
            --         if (Get_Float_Value (Value_Result (Top + 1)) <
            --            Get_Float_Value (Value_Result (Top + 2))) then
            --            Set_Value (Node, Create_Storage_For_Integer (1));
            --         else
            --            Set_Value (Node, Create_Storage_For_Integer (0));
            --         end if;
            null;
         when Array_Data => null;
         when Void   =>
            null;
      end case;

   elsif (Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Greaterthan_String))
          then
      case Get_Data_Type (Value_Result (Top + 1)) is
         when Integer_Data     =>
            if (Get_Integer_Value (Value_Result (Top + 1)) >
               Get_Integer_Value (Value_Result (Top + 2))) then
               Set_Value (Node, Create_Storage_For_Integer (1));
            else
               Set_Value (Node, Create_Storage_For_Integer (0));
            end if;
         when String_Data =>
            if (Convert_To_String (Get_String_Value (Value_Result (Top + 1)))
                > Convert_To_String (Get_String_Value (Value_Result (Top + 2)))
               ) then
               Set_Value (Node, Create_Storage_For_Integer (1));
            else
               Set_Value (Node, Create_Storage_For_Integer (0));
            end if;
         when Char_Data   =>
            if (Get_Character_Value (Value_Result (Top + 1)) >
               Get_Character_Value (Value_Result (Top + 2))) then
               Set_Value (Node, Create_Storage_For_Integer (1));
            else
               Set_Value (Node, Create_Storage_For_Integer (0));
            end if;
         when Float_Data  =>
            --         if (Get_Float_Value (Value_Result (Top + 1)) >
            --            Get_Float_Value (Value_Result (Top + 2))) then
            --            Set_Value (Node, Create_Storage_For_Integer (1));
            --         else
            --            Set_Value (Node, Create_Storage_For_Integer (0));
            --         end if;
            null;
         when Array_Data => null;
         when Void   =>
            null;
      end case;

   elsif (Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Less_Eq_String)) then
      case Get_Data_Type (Value_Result (Top + 1)) is
         when Integer_Data     =>
            if (Get_Integer_Value (Value_Result (Top + 1)) <=
               Get_Integer_Value (Value_Result (Top + 2))) then
               Set_Value (Node, Create_Storage_For_Integer (1));
            else
               Set_Value (Node, Create_Storage_For_Integer (0));
            end if;
         when String_Data =>
            if (Convert_To_String (Get_String_Value (Value_Result (Top + 1)))
                <=
               Convert_To_String (Get_String_Value (Value_Result (Top + 2))))
                then
               Set_Value (Node, Create_Storage_For_Integer (1));
            else
               Set_Value (Node, Create_Storage_For_Integer (0));
            end if;
         when Char_Data   =>
            if (Get_Character_Value (Value_Result (Top + 1)) <=
               Get_Character_Value (Value_Result (Top + 2))) then
               Set_Value (Node, Create_Storage_For_Integer (1));
            else
               Set_Value (Node, Create_Storage_For_Integer (0));
            end if;
         when Float_Data  =>
            --         if (Get_Float_Value (Value_Result (Top + 1)) <=
            --            Get_Float_Value (Value_Result (Top + 2))) then
            --            Set_Value (Node, Create_Storage_For_Integer (1));
            --         else
            --            Set_Value (Node, Create_Storage_For_Integer (0));
            --         end if;
            null;
         when Array_Data => null;
         when Void   =>
            null;
      end case;

   elsif (Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Greater_Eq_String))
          then
      case Get_Data_Type (Value_Result (Top + 1)) is
         when Integer_Data     =>
            if (Get_Integer_Value (Value_Result (Top + 1)) >=
               Get_Integer_Value (Value_Result (Top + 2))) then
               Set_Value (Node, Create_Storage_For_Integer (1));
            else
               Set_Value (Node, Create_Storage_For_Integer (0));
            end if;
         when String_Data =>
            if (Convert_To_String (Get_String_Value (Value_Result (Top + 1)))
                >=
               Convert_To_String (Get_String_Value (Value_Result (Top + 2))))
                then
               Set_Value (Node, Create_Storage_For_Integer (1));
            else
               Set_Value (Node, Create_Storage_For_Integer (0));
            end if;
         when Char_Data   =>
            if (Get_Character_Value (Value_Result (Top + 1)) >=
               Get_Character_Value (Value_Result (Top + 2))) then
               Set_Value (Node, Create_Storage_For_Integer (1));
            else
               Set_Value (Node, Create_Storage_For_Integer (0));
            end if;
         when Float_Data  =>
            --         if (Get_Float_Value (Value_Result (Top + 1)) >=
            --            Get_Float_Value (Value_Result (Top + 2))) then
            --            Set_Value (Node, Create_Storage_For_Integer (1));
            --         else
            --            Set_Value (Node, Create_Storage_For_Integer (0));
            --         end if;
            null;
         when Array_Data => null;
         when Void   =>
            null;
      end case;

   elsif (Are_Symbolic_Names_Equal (Subprogram_Name, Ref_And_String)) then
      if ((Get_Integer_Value (Value_Result (Top + 1)) = 0) or
         (Get_Integer_Value (Value_Result (Top + 2)) = 0)) then
         Set_Value (Node, Create_Storage_For_Integer (0));
      else
         Set_Value (Node, Create_Storage_For_Integer (1));
      end if;

   elsif (Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Or_String)) then
      if ((Get_Integer_Value (Value_Result (Top + 1)) = 1) or
         (Get_Integer_Value (Value_Result (Top + 2)) = 1)) then
         Set_Value (Node, Create_Storage_For_Integer (1));
      else
         Set_Value (Node, Create_Storage_For_Integer (0));
      end if;

   elsif (Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Xor_String)) then
      if (Get_Integer_Value (Value_Result (Top + 1)) =
         Get_Integer_Value (Value_Result (Top + 2))) then
         Set_Value (Node, Create_Storage_For_Integer (0));
      else
         Set_Value (Node, Create_Storage_For_Integer (1));
      end if;

   elsif (Are_Symbolic_Names_Equal (Subprogram_Name, Ref_Not_String)) then
      if (Get_Integer_Value (Value_Result (Top + 1)) = 1) then
         Set_Value (Node, Create_Storage_For_Integer (0));
      else
         Set_Value (Node, Create_Storage_For_Integer (1));
      end if;

   else
      Show_Error (1990, Internal,
                  "Evaluate_Infix_Operation: " &
                  Convert_To_String (Subprogram_Name));
   end if;

   return (Node); -- afterwards, node_value = result of operation

end Evaluate_Infix_Operation;
