--|$Log:	eistmt.a,v $
--|Revision 8.0  91/01/17  10:45:35  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Subunit for Package - Statement_Evaluator
--
---------------------------------------------------------------
--
--
-- Abstract
--
-- The following routine is a separate routine for
-- Statement_Evaluator.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
--==========================================================================
--  <Subprogram Name>
--
--    Evaluate_Individual_Statement
--
--  <Description>
--
--    Evaluates a statement or basic decl node.
--
--==========================================================================
with Symbol_Table_Routines;
use Symbol_Table_Routines;

with Derived_Subprogram_Support;
use Derived_Subprogram_Support;

with Debugger;
use Debugger;

with Create_Statement_Database;
use Create_Statement_Database;

with Node_Set_Routines;
use Node_Set_Routines;

with Node_Create_Routines;
use Node_Create_Routines;

with Declaration_Routines;
use Declaration_Routines;

with Semantic_Analyzer;
use Semantic_Analyzer;

with Context_Control;

separate (Statement_Evaluator.Evaluate_Statement)

procedure Evaluate_Individual_Statement (Node : in Ref_Statement_Database) is

   Subprogram_Name : constant String := "Evaluate_Individual_Statement";

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Evaluate_Subprogram_Call
   --
   --  <Description>
   --
   --    Evaluates a procedure or function call.
   --
   --==========================================================================
   function Evaluate_Subprogram_Call
       (Node : in Ref_Statement_Database) return Ref_Statement_Database;

   --=====================================================================
   --  <Subprogram Name>
   --
   --    Evaluate_Expression
   --
   --  <Description>
   --
   --    Evaluates the given expression node structures.
   --
   --=====================================================================
   -- --------------------------------------------------------------------
   function Evaluate_Expression
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
   separate;

   --==========================================================================
   function Evaluate_Subprogram_Call
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
   separate;

   procedure Evaluate_Object (Node : in Ref_Statement_Database) is separate;

   procedure Evaluate_Block (Node : in Ref_Statement_Database) is separate;

   procedure Allocate_And_Init_Storage_For_Package_Decls
       (Context : in out Context_Control.Context_Definition;
        Node    : Ref_Statement_Database) is separate;

   procedure Evaluate_Package_Body (Node : in Ref_Statement_Database) is
   separate;

begin
   case Check_Stmt_Node (Node) is

      when Exception_Alternative    =>
         null;

      when Null_Statement =>
         null;
         --------------------

      when Assign_Statement    =>
         declare
            Rhs_Expression   : Ref_Statement_Database :=
               Evaluate_Expression (Get_Rhs (Node));
            -- assume Lhs is simply a terminal_node reference
            -- (not yet implementing access and array references)
            Lhs_Symtbl_Entry : Ref_Symbol_Table_Entry :=
               Get_Symbol_Table_Entry (Get_Lhs (Node));
         begin
            Store_Value (Context, Lhs_Symtbl_Entry,
                         Copy_Value (Get_Value (Rhs_Expression)));
         end;

      when If_Statement   =>
         if (Is_True (Evaluate_Expression (Get_Condition (Node)))) then
            Evaluate_Statement (Context,
                                Get_Head_Of_True_Task_Statements_List (Node));
         else
            Evaluate_Statement (Context,
                                Get_Head_Of_False_Task_Statements_List (Node));
         end if;

      when For_Statement  =>
         null;

      when Loop_Statement =>
         begin
            if (Is_Empty (Get_Loop_Scheme (Node))) then
               -- endless LOOP
               loop
                  Evaluate_Statement (Context,
                                      Get_Head_Of_Statements_List (Node));
               end loop;
            elsif (Is_Expression_Node (Get_Loop_Scheme (Node))) then
               -- WHILE loop
               while (Is_True (Evaluate_Expression (Get_Loop_Scheme (Node))))
                loop
                  Evaluate_Statement (Context,
                                      Get_Head_Of_Statements_List (Node));
               end loop;
            elsif (Is_Statement_Node (Get_Loop_Scheme (Node))) then
               -- FOR stmt
               null; -- [TBD]
            else
               Show_Error (1001, Internal, "Loop_Statement/Loop_Scheme");
               -- raise some exception
            end if;

         exception
            when Exit_Statement_Encountered   =>
               -- need to handle "named" exits here also! [tbd]
               null;
         end;

      when Procedure_Call =>
         declare
            Ignored_Result : Ref_Statement_Database;
         begin
            Ignored_Result := Evaluate_Subprogram_Call (Node);
            --Free(
         end;

      when Case_Statement =>
         declare
            Choice_Expression_Result, Expression_Result :
            Ref_Statement_Database; -- for expressions
            List_Node : Ref_Statement_Database := Get_Alternatives_List (Node);
         begin
            Expression_Result := Evaluate_Expression (Get_Expression (Node));
            while (not Is_Empty (List_Node)) loop
               Choice_Expression_Result :=
                  Evaluate_Expression (Get_Alternative_Choice (List_Node));
               -- need to handle when multiple-ORed expr for a choice (??)
               -- need to handle "when OTHERS =>"
               -- [fix the "get_value" calls here?? --oei]
               if (Is_Equal (Get_Value (Expression_Result),
                             Get_Value (Choice_Expression_Result))) then
                  Evaluate_Statement (Context,
                                      Get_Alternative_Statements (List_Node));
                  exit ;
               else
                  List_Node := Get_Next_Alternative (List_Node);
               end if;
            end loop;
         end;

      when Return_Statement    =>
         if (not Is_Empty (Get_Expression (Node))) then
            -- this is a function call return
            -- place return expression into dss (function_value_offset)
            declare
               Tmp_Value : Ref_Data_Storage :=
                  Get_Value (Evaluate_Expression (Get_Expression (Node)));
            begin
               Store_Value (Context, Get_Subprogram_Reference (Node),
                            Copy_Value (Tmp_Value));
               case Check_Expr_Node (Get_Expression (Node)) is
                  when Constant_Node     | String_Node  | Character_Node    =>
                     null;
                  when others  =>
                     Free (Tmp_Value);
               end case;
            end;
         end if;
         raise Return_Statement_Encountered;

      when Exit_Statement =>
         if ((not Is_Empty (Get_Name (Node))) and then
            (Are_Symbolic_Names_Equal (Get_Symbolic_Name (Get_Name (Node)),
                                       Convert_To_Ref_String ("ace_main"))))
             then
            raise Super_Terminate;
         elsif ((not Is_Empty (Get_Name (Node))) and then
               (Are_Symbolic_Names_Equal (Get_Symbolic_Name (Get_Name (Node)),
                                          Convert_To_Ref_String ("ace_level")))
               ) then
            raise Terminate_Current_Ace;
         elsif ((not Is_Empty (Get_Name (Node))) and then
               (Are_Symbolic_Names_Equal (Get_Symbolic_Name (Get_Name (Node)),
                                          Convert_To_Ref_String ("ace")))) then
            raise End_Ace;
         elsif Is_Empty (Get_Condition (Node)) then
            raise Exit_Statement_Encountered;
         else
            if (Is_True (Evaluate_Expression (Get_Condition (Node)))) then
               raise Exit_Statement_Encountered;
            end if;
         end if;

      when Block_Statement     =>

         Evaluate_Block (Node);

      when Exception_Declaration    =>
         declare
            List_Node : Ref_Statement_Database := Get_Name_List (Node);
         begin
            while (not Is_Empty (List_Node)) loop
               declare
                  Exception_Symtbl_Entry : Ref_Symbol_Table_Entry :=
                     Get_Symbol_Table_Entry (Get_Expression (List_Node));
               begin
                  if Context_Control.Is_In_Subprogram_Call (Context) then
                     if Is_Symbol_Table_Debugging_On then
                        Add_Duplicate_Of_Entry_To_Symbol_Table
                            (Context, Exception_Symtbl_Entry);
                     end if;
                  else
                     Add_Entry_To_Symbol_Table
                         (Context, Exception_Symtbl_Entry);
                  end if;
                  --
                  Observe_Object_If_Objects_Observation_Is_On
                      (Exception_Symtbl_Entry);
                  --
                  List_Node := Get_Next_List_Item (List_Node);
               end;
            end loop;
         end;

      when Object_Declaration  =>

         Evaluate_Object (Node);

      when Type_Declaration    | Subtype_Declaration    =>
         declare
            Symtbl_Entry : Ref_Symbol_Table_Entry :=
               Get_Symbol_Table_Entry (Node);
         begin
            -- as appropriate "allocate storage" also
            Allocate_Storage (Context, Symtbl_Entry);
            --
            if Context_Control.Is_In_Subprogram_Call (Context) then
               -- for debugging visibility
               if Is_Symbol_Table_Debugging_On then
                  Add_Duplicate_Of_Entry_To_Symbol_Table
                      (Context, Symtbl_Entry);
               end if;
            else
               Add_Entry_To_Symbol_Table (Context, Symtbl_Entry);
               if Get_Entry_Kind (Symtbl_Entry) = Derived_Type_Entry then
                  Derive_Subprograms (Context, Symtbl_Entry);
               end if;
            end if;
            --
            Observe_Object_If_Objects_Observation_Is_On
                (Symtbl_Entry);
         end;

      when Procedure_Spec_Declaration |
           Procedure_Declaration      |
           Function_Spec_Declaration  |
           Function_Declaration     =>
         -- put symtbl entry on symtbl
         --    but only if the entry is not already there
         -- [for resolving real references (ACE top level)
         --  or "debugging" visibility (w/in subpgms)]
         begin
            -- for debugging visibility
            if Context_Control.Is_In_Subprogram_Call (Context) then
               if Is_Symbol_Table_Debugging_On then
                  Add_Duplicate_Of_Entry_To_Symbol_Table
                      (Context, Get_Symbol_Table_Entry (Node));
               end if;
            else
               --
               Evaluate_Statement
                   (Context, Get_Context_Clause (Get_Symbol_Table_Entry (Node))
                    );
               Add_Entry_To_Symbol_Table (Context,
                                          Get_Symbol_Table_Entry (Node));
            end if;
            --
         exception
            when Duplicate_Symbol_Table_Entry_Error     =>
               if Is_Symbol_Table_Debugging_On then
                  Add_Duplicate_Of_Entry_To_Symbol_Table
                      (Context, Get_Symbol_Table_Entry (Node));
               end if;
         end;

         Observe_Object_If_Objects_Observation_Is_On
             (Get_Symbol_Table_Entry (Node));

         --------------------

      when Package_Declaration =>
         declare
            Package_Symtbl_Entry : Ref_Symbol_Table_Entry :=
               Get_Symbol_Table_Entry (Node);
         begin
            Evaluate_Statement
                (Context, Get_Context_Clause (Get_Symbol_Table_Entry (Node)));
            Allocate_And_Init_Storage_For_Package_Decls
                (Context, Get_Package_Spec_Decls (Package_Symtbl_Entry));
            if Context_Control.Is_In_Subprogram_Call (Context) then
               if Is_Symbol_Table_Debugging_On then
                  Add_Duplicate_Of_Entry_To_Symbol_Table
                      (Context, Package_Symtbl_Entry);
               end if;
            else
               begin
                  Add_Entry_To_Symbol_Table (Context, Package_Symtbl_Entry);
               exception
                  when Duplicate_Symbol_Table_Entry_Error    =>
                     if Is_Symbol_Table_Debugging_On then
                        Add_Duplicate_Of_Entry_To_Symbol_Table
                            (Context, Package_Symtbl_Entry);
                     end if;
               end;
            end if;
            Observe_Object_If_Objects_Observation_Is_On
                (Package_Symtbl_Entry);
         end;

      when Package_Body_Declaration =>

         Evaluate_Package_Body (Node);

      when With_Statement =>
         -- eval package_stmts (currently ignores the "with" statement,
         -- but executes the "use" statements if any in the context
         -- clause.
         declare
            List_Node : Ref_Statement_Database := Get_Next_Statement (Node);
         begin
            while not Is_Empty (List_Node) loop
               if Check_Stmt_Node (List_Node) = Use_Statement then
                  Evaluate_Statement (Context, List_Node);
               end if;
               List_Node := Get_Next_Statement (List_Node);
            end loop;
         end;

         -- trap Duplicate_Symbol_Table_Entry_Error exception

      when Use_Statement  =>
         if Is_Symbol_Table_Debugging_On and
            Context_Control.Is_In_Subprogram_Call (Context) then
            -- put USE-ed package's symtbl entries on symtbl for debugging-type
            -- visibility
            declare
               List_Node : Ref_Statement_Database := Get_Name_List (Node);
            begin
               while (not Is_Empty (List_Node)) loop
                  begin
                     Add_Package_Decls_To_Symbol_Table
                         (Context,
                          Get_Symbol_Table_Entry (Get_Expression (List_Node)));
                  exception
                     when Duplicate_Symbol_Table_Entry_Error =>
                        -- raised by add_entry_to_symtbl
                        -- [means that package is already USE-ed]
                        null;
                  end;
                  List_Node := Get_Next_List_Item (List_Node);
               end loop;
            end;
         end if;

         --------------------

      when Labeled_Statement   =>
         -- ignore these
         null;

         --------------------

      when Pragma_Statement    =>
         -- do nothing here (PRAGMAs handled during semantic analysis)
         null;

         --------------------

      when Case_Alternative_Part    =>
         -- can't be this kind here!
         Show_Error (1000, Internal, Subprogram_Name);

         --------------------

      when Goto_Statement =>
         -- not implementing these
         Show_Error (1999, Internal, Subprogram_Name);

         --------------------

      when Unknown   =>
         Show_Error (1002, Internal, Subprogram_Name);
   end case;

end Evaluate_Individual_Statement;
