--|$Log:	eprag.a,v $
--|Revision 8.0  91/01/17  10:45:37  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Subunit for Package - Statement_Evaluator
--
---------------------------------------------------------------
--
--
-- Abstract
--
--  The following routine is a separate for Statement_Evaluator.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------

--==========================================================================
--  <Subprogram Name>
--
--    Evaluate_Pragma_Statement
--
--  <Description>
--
--    Evaluates PRAGMA statements.
--
--  <Called By>
--
--    Semantic_Analyzer (Check_Statement)
--
--==========================================================================
with Symtbl_Entry_Set_Routines;
use Symtbl_Entry_Set_Routines;

with Storage_Management_Routines;
use Storage_Management_Routines;

with Semantic_Analyzer;
use Semantic_Analyzer;

with Ace_Universal_Types;
use Ace_Universal_Types;

with Debugger;
use Debugger;

with Dump_Routines;
use Dump_Routines;

with User_Pkg;
use User_Pkg;

with Literals;
use Literals;

with Misc;

with Ace;

separate (Semantic_Analyzer.Check_Statement)

procedure Evaluate_Pragma_Statement (Node : in Ref_Statement_Database) is

   Arguments       : Ref_Statement_Database := Get_Argument_List (Node);
   Pragma_Name     : Ref_String := Get_Symbolic_Name (Get_Name (Node));

   -- [tbd] change these to be get_symbolic_name (ref_string)
   First_Argument  : Ref_Statement_Database;
   Second_Argument : Ref_Statement_Database;

   procedure No_Arguments is
   begin
      if Is_Empty (Arguments) then
         null;
      else
         Show_Error (1980, Fatal, Convert_To_String (Pragma_Name));
         return ;
      end if;
   end No_Arguments;

   procedure Must_Have_At_Least_One_Argument is
   begin
      if Is_Empty (Arguments) then
         Show_Error (1980, Fatal, Convert_To_String (Pragma_Name));
         return ;
      end if;
      First_Argument := Get_Expression (Arguments);
   end Must_Have_At_Least_One_Argument;

   procedure Must_Have_At_Least_Two_Arguments is
   begin
      Must_Have_At_Least_One_Argument;
      if Is_Empty (Get_Next_List_Item (Arguments)) then
         Show_Error (1980, Fatal, Convert_To_String (Pragma_Name));
         return ;
      end if;
      Second_Argument := Get_Expression (Get_Next_List_Item (Arguments));
   end Must_Have_At_Least_Two_Arguments;

begin

   if (Are_Symbolic_Names_Equal
          (Pragma_Name, Convert_To_Ref_String ("builtin"))) then
      declare
         Builtin_Entry    : Ref_Symbol_Table_Entry;
         Formal_Parameter : Ref_Symbol_Table_Entry;
         -- assume offset 0 reserved,
         --    just in case built-in is a function
         Offset           : Ace_Integer := 1;
      begin
         -- mark symtbl entry of given arg as "built-in"
         Must_Have_At_Least_Two_Arguments;
         Builtin_Entry :=
            Find_Symbol_Table_Entry (Ace.Main_Thread_Context,
                                     Get_Symbolic_Name (First_Argument));
         if Is_Empty (Builtin_Entry) then
            Show_Error (1981, Internal,
                        Convert_To_String (Get_Symbolic_Name (First_Argument)));
            return ;
         end if;
         Check_Expression (Ace.Main_Thread_Context, Second_Argument);

         -- Pragma builtin can only handle an integer as
         -- it's second parameter.
         Set_As_Built_In_Subprogram
             (Builtin_Entry, Get_Integer_Value (Get_Value (Second_Argument)));

         -- assign level and offset to formal params,
         --    if built-in is a subpgm
         case Get_Entry_Kind (Builtin_Entry) is
            when Function_Entry     | Procedure_Entry   =>
               Formal_Parameter := Get_Head_Of_Parameters_List (Builtin_Entry);
               while (not Is_Empty (Formal_Parameter)) loop
                  Set_Level (Formal_Parameter, Get_Current_Level (Context) + 1);
                  Set_Value_Offset (Formal_Parameter, Offset);
                  Formal_Parameter := Get_Next_Parameter (Formal_Parameter);
                  Offset := Offset + 1;
               end loop;
            when others   =>
               Show_Error
                   (1982, Internal,
                    Convert_To_String (Get_Symbolic_Name (Builtin_Entry)));
         end case;
      end;

   elsif (Are_Symbolic_Names_Equal
             (Pragma_Name, Convert_To_Ref_String ("list_statement_numbers")))
          then
      Must_Have_At_Least_One_Argument;
      if (Are_Symbolic_Names_Equal (Get_Symbolic_Name (First_Argument),
                                    Convert_To_Ref_String ("off"))) then
         Dump_Routines.Dump_Statement_Numbers (False);
      elsif (Are_Symbolic_Names_Equal (Get_Symbolic_Name (First_Argument),
                                       Convert_To_Ref_String ("on"))) then
         Dump_Routines.Dump_Statement_Numbers (True);
      else
         Show_Error (3004, Warning,
                     Convert_To_String (Get_Symbolic_Name (First_Argument)));
      end if;

   elsif (Are_Symbolic_Names_Equal
             (Pragma_Name, Convert_To_Ref_String ("dump"))) then
      Must_Have_At_Least_One_Argument;
      if (Are_Symbolic_Names_Equal (Get_Symbolic_Name (First_Argument),
                                    Convert_To_Ref_String ("off"))) then
         Dump_Routines.Dump (False);
      elsif (Are_Symbolic_Names_Equal (Get_Symbolic_Name (First_Argument),
                                       Convert_To_Ref_String ("on"))) then
         Dump_Routines.Dump (True);
      else
         Show_Error (3004, Warning,
                     Convert_To_String (Get_Symbolic_Name (First_Argument)));
      end if;

   elsif (Are_Symbolic_Names_Equal
             (Pragma_Name, Convert_To_Ref_String ("echo"))) then
      Must_Have_At_Least_One_Argument;
      if (Are_Symbolic_Names_Equal (Get_Symbolic_Name (First_Argument),
                                    Convert_To_Ref_String ("off"))) then
         Echo (False);
      elsif (Are_Symbolic_Names_Equal (Get_Symbolic_Name (First_Argument),
                                       Convert_To_Ref_String ("on"))) then
         Echo (True);
      else
         Show_Error (3004, Warning,
                     Convert_To_String (Get_Symbolic_Name (First_Argument)));
      end if;

   elsif (Are_Symbolic_Names_Equal (Pragma_Name,
                                    Convert_To_Ref_String ("trace"))) then
      Must_Have_At_Least_One_Argument;
      if (Are_Symbolic_Names_Equal (Get_Symbolic_Name (First_Argument),
                                    Convert_To_Ref_String ("off"))) then
         Set_Statements_Trace_Off;
      elsif (Are_Symbolic_Names_Equal (Get_Symbolic_Name (First_Argument),
                                       Convert_To_Ref_String ("on"))) then
         Set_Statements_Trace_On;
      else
         Show_Error (3004, Warning,
                     Convert_To_String (Get_Symbolic_Name (First_Argument)));
      end if;

   elsif (Are_Symbolic_Names_Equal (Pragma_Name,
                                    Convert_To_Ref_String ("debug"))) then
      Must_Have_At_Least_One_Argument;
      if (Are_Symbolic_Names_Equal (Get_Symbolic_Name (First_Argument),
                                    Convert_To_Ref_String ("off"))) then
         Set_Overall_Debugging_Off;
      elsif (Are_Symbolic_Names_Equal (Get_Symbolic_Name (First_Argument),
                                       Convert_To_Ref_String ("on"))) then
         Set_Overall_Debugging_On;
      else
         Show_Error (3004, Warning,
                     Convert_To_String (Get_Symbolic_Name (First_Argument)));
      end if;

   elsif (Are_Symbolic_Names_Equal (Pragma_Name,
                                    Convert_To_Ref_String ("set_delay"))) then
      Must_Have_At_Least_One_Argument;
      if (Are_Symbolic_Names_Equal (Get_Symbolic_Name (First_Argument),
                                    Convert_To_Ref_String ("off"))) then
         Do_Delay := False;
      elsif (Are_Symbolic_Names_Equal (Get_Symbolic_Name (First_Argument),
                                       Convert_To_Ref_String ("on"))) then
         Do_Delay := True;
      else
         Show_Error (3004, Warning,
                     Convert_To_String (Get_Symbolic_Name (First_Argument)));
      end if;

   elsif (Are_Symbolic_Names_Equal (Pragma_Name,
                                    Convert_To_Ref_String ("exit_on_error")))
          then
      Must_Have_At_Least_One_Argument;
      if (Are_Symbolic_Names_Equal (Get_Symbolic_Name (First_Argument),
                                    Convert_To_Ref_String ("off"))) then
         Misc.Set_Exit_On_First_Error (False);
      elsif (Are_Symbolic_Names_Equal (Get_Symbolic_Name (First_Argument),
                                       Convert_To_Ref_String ("on"))) then
         Misc.Set_Exit_On_First_Error (True);
      else
         Show_Error (3004, Warning,
                     Convert_To_String (Get_Symbolic_Name (First_Argument)));
      end if;

   elsif (Are_Symbolic_Names_Equal (Pragma_Name,
                                    Convert_To_Ref_String ("main_prompt")))
          then
      Must_Have_At_Least_One_Argument;
      Misc.Set_Main_Prompt
          (Convert_To_String (Get_Symbolic_Name (First_Argument)));

   elsif (Are_Symbolic_Names_Equal (Pragma_Name,
                                    Convert_To_Ref_String ("continue_prompt")))
          then
      Must_Have_At_Least_One_Argument;
      Misc.Set_Continuation_Prompt
          (Convert_To_String (Get_Symbolic_Name (First_Argument)));

   elsif (Are_Symbolic_Names_Equal (Pragma_Name,
                                    Convert_To_Ref_String ("observe"))) then
      declare
         Is_On : Boolean;
      begin
         Must_Have_At_Least_Two_Arguments;
         if (Are_Symbolic_Names_Equal (Get_Symbolic_Name (Second_Argument),
                                       Convert_To_Ref_String ("off"))) then
            Is_On := False;
         elsif (Are_Symbolic_Names_Equal (Get_Symbolic_Name (Second_Argument),
                                          Convert_To_Ref_String ("on"))) then
            Is_On := True;
         else
            Show_Error (1984, Fatal,
                        Convert_To_String (Get_Symbolic_Name (First_Argument)));
         end if;
         if (Are_Symbolic_Names_Equal (Get_Symbolic_Name (First_Argument),
                                       Convert_To_Ref_String ("objects"))) then
            if Is_On then
               Set_Observe_Objects_On;
            else
               Set_Observe_Objects_Off;
            end if;
         elsif (Are_Symbolic_Names_Equal (Get_Symbolic_Name (First_Argument),
                                          Convert_To_Ref_String ("statements"))
               ) then
            if Is_On then
               Set_Observe_Statements_On;
            else
               Set_Observe_Statements_Off;
            end if;
         elsif (Are_Symbolic_Names_Equal
                   (Get_Symbolic_Name (First_Argument),
                    Convert_To_Ref_String ("subprograms"))) then
            if Is_On then
               Set_Observe_Subprograms_On;
            else
               Set_Observe_Subprograms_Off;
            end if;
         elsif (Are_Symbolic_Names_Equal (Get_Symbolic_Name (First_Argument),
                                          Convert_To_Ref_String ("tasks")))
                then
            if Is_On then
               Set_Observe_Tasks_On;
            else
               Set_Observe_Tasks_Off;
            end if;
         else
            Show_Error (1985, Warning,
                        Convert_To_String (Get_Symbolic_Name (First_Argument)));
         end if;
      end;

   else
      Show_Error (1983, Warning, Convert_To_String (Pragma_Name));
   end if;

end Evaluate_Pragma_Statement;
