--|$Log:	error.a,v $
--|Revision 8.0  91/01/17  10:45:38  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Error_Handler
--
---------------------------------------------------------------
--
--
-- Abstract
--
--    This package provides the display of error messages (fatal, warnings,
--    and internal), as well as routines to restore internal ACE data
--    structures when errors occur within the middle of Ada constructs.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------

with Symbol_Table_Routines;
use Symbol_Table_Routines;

with Declarative_Region_Support_Routines;
use Declarative_Region_Support_Routines;

with Ace_Universal_Types;
use Ace_Universal_Types;

with Ace_Text_Io;
use Ace_Text_Io;

with Help_Pkg;

with Misc;

package body Error_Handler is

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Show_Error
   --
   --  <Description>
   --
   --    This procedure displays the text associated with an error message,
   --    followed by any optional message text.  Errors that are fatal user
   --    errors then raise a User_Error exception which is the mechanism
   --    for indicating that error recovery should be performed by ACE.
   --
   --  <Exceptions Raised>
   --
   --    User_Error is raised if the message level of the error is a Fatal_Error
   --
   --==========================================================================
   procedure Show_Error (Error_Number     : in Ace_Integer;
                         Message_Level    : in Message_Level_Enumeration;
                         Optional_Message : in String := "") is

      -- This exception is only raised when we should exit ACE to the
      -- host O/S.  The pragma Exit_On_Error controls this exception.
      Non_Recoverable_Exception : exception;

   begin

      case Message_Level is

         when Warning     =>
            Put (Current_Ace_Output, "Warning: ");

         when Fatal  =>
            Put (Current_Ace_Output, "Error: ");

         when Internal    =>
            Put (Current_Ace_Output, "Internal Error: ");

         when Diagnostic  =>
            Put (Current_Ace_Output, "Diagnostic: ");

         when others =>
            Put (Current_Ace_Output,
                 "Error Handler Internal Error: Unknown Message Level: ");
            Put_Line (Current_Ace_Output,
                      Message_Level_Enumeration'IMAGE (Message_Level));

      end case; -- message_level

      case Error_Number is

         when 0 => -- oei
            Put (Current_Ace_Output, "Undefined error (0)");

            -- ========== OEI ==========

         when 1000   => -- oei
            Put (Current_Ace_Output,
                 "Enumeration kind should not have been encountered");

         when 1001   => -- oei
            Put (Current_Ace_Output, "Unexpected Node_Kind");

         when 1002   => -- oei
            Put (Current_Ace_Output, "Unexpected Statement_Kind");

         when 1003   => -- oei
            Put (Current_Ace_Output, "Unexpected Expression_Kind");

         when 1004   => -- oei
            Put (Current_Ace_Output, "Unexpected Type_Entry_Enumeration");

         when 1005   => -- oei
            Put (Current_Ace_Output, "Unexpected Dot_Node_Enumeration");

         when 1006   => -- oei
            Put (Current_Ace_Output, "Unexpected Lp_Node_Enumeration");

         when 1007   => -- oei
            Put (Current_Ace_Output,
                 "Unexpected Possible_Attributes_Enumeration");

         when 1008   => -- oei
            Put (Current_Ace_Output,
                 "Unexpected Statement_Database_Enumeration");

         when 1009   => -- oei
            Put (Current_Ace_Output, "Unexpected Lp_Node_Enumeration");

         when 1010   => -- oei
            Put (Current_Ace_Output, "Unimplemented Statement_Kind");

         when 1011   => -- oei
            Put (Current_Ace_Output, "Unimplemented Expression_Kind");

         when 1051   => -- oei
            Put (Current_Ace_Output, "Subprogram overload resolution");

         when 1100   => -- oei
            Put (Current_Ace_Output, "Undeclared identifier");

         when 1111   => -- oei
            Put (Current_Ace_Output, "Type mismatch");

         when 1112   => -- oei
            Put (Current_Ace_Output,
                 "Could not find a valid subprogram specification");

         when 1114   => -- oei
            Put (Current_Ace_Output, "Invalid mode");

         when 1115   => -- oei
            Put (Current_Ace_Output,
                 "Multiple valid subprogram specifications found");

         when 1116   => -- oei
            Put (Current_Ace_Output, "Invalid expression");

         when 1117   => -- oei
            Put (Current_Ace_Output,
                 "Type mismatch in initialization code" &
                 " for formal parameter of type");

         when 1200   => -- oei
            Put (Current_Ace_Output,
                 "Constraint error in Storage_Management_Routines");

         when 1401   => -- oei
            Put (Current_Ace_Output,
                 "Cannot return an expression from procedure");

         when 1402   => -- oei
            Put (Current_Ace_Output, "Statement invalid here");

         when 1403   => -- oei
            Put (Current_Ace_Output,
                 "Return statement within functions must return an expression");

         when 1404   => -- oei
            Put (Current_Ace_Output,
                 "Functions must contain at least one valid return statement");

         when 1405   => -- oei
            Put (Current_Ace_Output, "Subprogram body not available");

         when 1510   => -- oei
            Put (Current_Ace_Output, "No such statement to display");

         when 1511   => -- oei
            Put (Current_Ace_Output, "No breakpoint set at statement number");

         when 1512   => -- oei
            Put (Current_Ace_Output, "Subprogram body has no statements");

         when 1514   => -- oei
            Put (Current_Ace_Output, "Breakpoint cannot be set at named item");

         when 1515   => -- oei
            Put (Current_Ace_Output,
                 "Breakpoint cannot be cleared at named item");

         when 1513   => -- oei
            Put (Current_Ace_Output,
                 "Statement cannot be displayed for named item");

         when 1980   => -- oei
            Put (Current_Ace_Output, "Invalid number of arguments to PRAGMA");

         when 1981   => -- oei
            Put (Current_Ace_Output,
                 "Subprogram named does not exist in PRAGMA BuiltIn");

         when 1982   => -- oei
            Put (Current_Ace_Output,
                 "PRAGMA BuiltIn only valid for subprograms");

         when 1983   => -- oei
            Put (Current_Ace_Output, "Unimplemented PRAGMA");

         when 1984   => -- oei
            Put (Current_Ace_Output,
                 "Second argument to PRAGMA observe must be on or off");

         when 1985   => -- oei
            Put (Current_Ace_Output,
                 "First argument to PRAGMA observe must be " &
                 "objects, statements or subprograms");

         when 1986   => -- oei
            Put (Current_Ace_Output,
                 "Only one argument allowed for type conversion");

         when 1987   => -- oei
            Put (Current_Ace_Output, "Not running under an windowing system");

         when 1988   => -- oei
            Put (Current_Ace_Output, "Not yet supported");

         when 1990   => -- oei
            Put (Current_Ace_Output, "Unimplemented built-in subprogram");

         when 1991   => -- wpl
            Put (Current_Ace_Output, "Illegal prefix");

         when 1999   => -- oei
            Put (Current_Ace_Output, "Unimplemented feature");

            -- ========== JAT ==========

         when 2000   => -- jat
            Put (Current_Ace_Output, "Constraint error in get/set routines");

         when 2001   => -- jat
            Put (Current_Ace_Output,
                 "Invalid attempt to redefine an existing identifier");

         when 2002   => -- jat
            Put (Current_Ace_Output, "Invalid reuse of identifier");

         when 2003   => -- jat
            Put (Current_Ace_Output, "Invalid reuse of type and literal name");

         when 2004   => -- jat
            Put (Current_Ace_Output,
                 "Invalid reference of an undefined identifier");

         when 2005   => -- jat
            Put (Current_Ace_Output,
                 "Type/subtype/derived type required, instead of");

         when 2006   => -- jat
            Put (Current_Ace_Output,
                 "Designator after END does not match subprogram spec");

         when 2007   => -- jat
            Put (Current_Ace_Output,
                 "Function parameters must be of mode IN " &
                 "(implicit or explicit)");

         when 2008   => -- jat
            Put (Current_Ace_Output,
                 "Default expression only allowed for parameter of mode IN " &
                 "(implicit or explicit)");

         when 2009   => -- jat
            Put (Current_Ace_Output, "Subprogram body replaced with new copy");

         when 2010   => -- jat
            Put (Current_Ace_Output,
       --                 "No package specification exists within this scope " &
            --                 "for the package body");
                 "Package specification not found");

         when 2011   => -- jat
            Put (Current_Ace_Output,
                 "Identifier after END does not match package name");

         when 2012   => -- jat
            Put (Current_Ace_Output,
    --                 "Identifier after USE is either undefined within this " &
            --                 "scope, or is not the name of a package");
                 "Package in USE statement not found");

         when 2013   => -- jat
            Put (Current_Ace_Output, "Package body replaced with new copy");

         when 2014   => -- jat
            Put (Current_Ace_Output,
                 "No body declared within the package for the specification");

         when 2015   => -- jat
            Put (Current_Ace_Output,
                 "Declaration is not made directly visible for");

         when 2016   => -- jat
            Put (Current_Ace_Output,
    --                 "Declaration that was previously visible is no longer " &
            --                 "directly visible");
                 "Declaraion no longer direcly visible");

         when 2017   => -- jat
            Put (Current_Ace_Output,
                 "Constant object declaration requires " &
                 "explicit initialization");

         when 2018   => -- jat
            Put (Current_Ace_Output,
                 "Subprogram name is overloaded; use FORM to qualify name");

            -- ========== WPL ==========

         when 3001   => -- wpl
            Put (Current_Ace_Output, "Unterminated String in input");

         when 3002   => -- wpl
            Put (Current_Ace_Output, "File doesn't exist");

         when 3003   => -- wpl
            Put (Current_Ace_Output, "Unknown problem with file");

         when 3004   => -- wpl
            Put (Current_Ace_Output,
                 "Arguments to pragma are ""on"", or ""off""");

         when 3005   => -- wpl
            Put (Current_Ace_Output, "Exception in Create_Statement_Database");

         when 3006   => -- wpl
            Put (Current_Ace_Output, "Unhandled exception in ACE main");

         when 3007   => -- wpl
            Put (Current_Ace_Output, "Underlying Host O/S error");

         when 3008   => -- wpl
            Put (Current_Ace_Output, "Unhandled exception in Tree Builder");

         when 3009   => -- wpl
            Put (Current_Ace_Output, "Unexpected case alternative");

         when 3010   => -- wpl
            Put (Current_Ace_Output, "Input line too long -- Skipped");

         when 3011   => -- wpl
            Put (Current_Ace_Output, "Illegal token entered");

         when 3012   => -- wpl
            Put (Current_Ace_Output, "Subprogram is unimplemented in phase 1");

         when 3013   => -- wpl
            Put (Current_Ace_Output, "Feature is unimplemented in phase 1");

         when 3014   => -- wpl
            Put (Current_Ace_Output, "Error processing data");

         when 3015   => -- wpl
            Put (Current_Ace_Output, "Invalid token starting command");

         when 3016   => -- wpl
            Put (Current_Ace_Output, "Unimplemented Ada Feature");

         when 3017   => -- wpl
            Put (Current_Ace_Output, "No command associated with");

         when 3018   => -- wpl
            Put (Current_Ace_Output, "Illegal Form parameter");

         when 3019   => -- wpl
            Put (Current_Ace_Output, "Can not recall command");

         when 3020   => -- wpl
            Put (Current_Ace_Output, "Return statement invalid");

         when 3021   => -- wpl
            Put (Current_Ace_Output,
                 "Identifier after END does not match task name");

         when 3022   => -- wpl
            Put (Current_Ace_Output, "Task specification not found");

         when 3023   => -- wpl
            Put (Current_Ace_Output, "Task body replaced with new copy");

         when 3024   => -- wpl
            Put (Current_Ace_Output,
                 "Illegal use of a procedure call in expression");

         when 3025   => -- wpl
            Put (Current_Ace_Output, "Illegal use in expression");

         when 3026   => -- wpl
            Put (Current_Ace_Output, "Illegal use of expanded name");

         when 3027   => -- wpl
            Put (Current_Ace_Output, "Could not find file");

            -- ========== xxx ==========

         when 4000   => -- raf
            Put (Current_Ace_Output, "Help already associated with object");

         when 4001   => -- raf
            Put (Current_Ace_Output, "No help available");

         when 4100   => -- raf
            Put (Current_Ace_Output, "Type Mark expected");

         when 4101   => -- raf
            Put (Current_Ace_Output,
                 "Base Type of parameter does not " &
                 "correspond to attribute prefix");

         when 4102   => -- raf
            Put (Current_Ace_Output, "Parameter must be of some integer type");

         when 4103   => -- raf
            Put (Current_Ace_Output, "Attribute not currently supported");

         when 4104   => -- raf
            Put (Current_Ace_Output, "Unknown attribute");

         when 4105   => -- raf
            Put (Current_Ace_Output, "Parameter not allowed for attribute");

         when 4106   => -- raf
            Put
                (Current_Ace_Output, "Attribute should have only one parameter"
                 );

         when 4107   => --raf
            Put (Current_Ace_Output, "Parameter expected for attribute");

         when 4108   => -- raf
            Put (Current_Ace_Output,
                 "Prefix must denote a discrete type or subtype");

         when 4200   => -- raf
            Put (Current_Ace_Output, "Subprogram has been deleted");

         when 4205   => -- raf
            Put (Current_Ace_Output, "Package not used");

         when 4206   => -- raf
            Put (Current_Ace_Output, "Package not found");

            -- ========== Xt ==========

         when 5001   => -- rhp
            Put_Line (Current_Ace_Output, "Unknown parameter order");
            Put (Current_Ace_Output,
                 "(Xt callbacks may not function properly)");

            -- ========== end ==========

         when others =>
            Put (Current_Ace_Output,
                 "Error Handler Internal Error: Unknown Error Code: ");
            Put (Current_Ace_Output, Ace_Integer'IMAGE (Error_Number));

      end case; -- error_number

      if Optional_Message /= "" then
         Put (Current_Ace_Output, ": ");
         Put_Line (Current_Ace_Output, Optional_Message);
      else
         New_Line (Current_Ace_Output);
      end if;

      if Misc.Exit_On_First_Error and (Message_Level in Fatal .. Internal) then
         raise Non_Recoverable_Exception;
      end if;

      if (Message_Level = Fatal) then
         raise User_Error;
      end if;

      return ;

   end Show_Error;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Reset_From_Subprogram_Or_Package_Declaration
   --
   --  <Description>
   --
   --    This procedure performs restoring actions for the command langauge
   --    interpreter when ACE needs to be reset.  Actions taken are dependent
   --    upon whether parsing is occuring within a declarative region when
   --    restoration is required.
   --
   --==========================================================================
   procedure Reset_From_Subprogram_Or_Package_Declaration
       (Context : in out Context_Control.Context_Definition) is

   begin
      if Within_Declarative_Region (Context) then
         Reset_Declarative_Regions (Context);
         Set_Top_Of_Symtbl (Context,
                            Get_Saved_Main_Level_Symbol_Table_Head (Context));
      end if;
      Help_Pkg.Clear_Help;
   end Reset_From_Subprogram_Or_Package_Declaration;

   -- -------------------------------------------------------------------------

end Error_Handler;
