--|$Log:	estmt.a,v $
--|Revision 8.0  91/01/17  10:45:40  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Statement_Evaluator
--
---------------------------------------------------------------
--
--
-- Abstract
--
--   The following routine is a separate for Statement_Evaluator
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
--==========================================================================
--  <Subprogram Name>
--
--    Evaluate_Statement
--
--  <Description>
--
--    Evaluates the list of statement (including declaration) node structures.
--
--==========================================================================
-- --------------------------------------------------------------------------

with Dump_Routines;
use Dump_Routines;

with Debugger;
use Debugger;

with System_Dependent_Routines;

with Text_Io;

with Context_Control;

separate (Statement_Evaluator)

procedure Evaluate_Statement
    (Context : in out Context_Control.Context_Definition;
     Node    : in Ref_Statement_Database) is

   Current : Ref_Statement_Database := Node;

   -- --------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Evaluate_Individual_Statement
   --
   --  <Description>
   --
   --    This routine evaluates a single Ada statement.
   --
   --=======================================================================
   procedure Evaluate_Individual_Statement (Node : in Ref_Statement_Database)
                                           is separate;

begin

   -- while there are statements
   --
   while (not Is_Empty (Current)) loop

      if Is_Statements_Trace_On then
         Dump_Statement_Numbers (True);
         -- should this be ace_standard_out or something?? [oei]
         Dump_For_Statement_Trace (Current, Text_Io.Standard_Output);
         Dump_Statement_Numbers (False);
      end if;

      Observe_Statement_If_Statements_Observation_Is_On
          (Current);

      if (not Within_Breakpoint) then
         Context_Control.Set_Current_Statement (Context, Current);
      end if;

      if ((Is_Breakpoint_Set (Current)) or (Is_Stepping_Through)) then
         Break (Current);
      end if;

      if System_Dependent_Routines.Abort_Check then
         System_Dependent_Routines.Flush;
         -- because of the ioctl in flag_abort there is
         -- a new_line that isn't thrown away by flush.
         -- Since abort_check is true this new_line must be there.
         Text_Io.Skip_Line (Text_Io.Standard_Input);
         Break (Current, "Interrupt encountered");
      end if;

      Evaluate_Individual_Statement (Current);

      Current := Get_Next_Statement (Current);

   end loop;

end Evaluate_Statement;
