--|$Log:	esubprm.a,v $
--|Revision 8.0  91/01/17  10:45:40  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Subunit of Package - Statement_Evaluator
--
---------------------------------------------------------------
--
--
-- Abstract
--
--  The following routine is a unit of Statement_Evaluator.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
--==========================================================================
--  <Subprogram Name>
--
--    Evaluate_Subprogram_Call
--
--  <Description>
--
--    Evaluates procedure and function calls.
--
--==========================================================================
with Symtbl_Entry_Set_Routines;
use Symtbl_Entry_Set_Routines;

with Ace_Universal_Types;
use Ace_Universal_Types;

separate (Statement_Evaluator.Evaluate_Statement.Evaluate_Individual_Statement)

function Evaluate_Subprogram_Call
    (Node : in Ref_Statement_Database) return Ref_Statement_Database is

   Subprogram_Name         : Ref_String :=
      Get_Symbolic_Name (Get_Subprogram_Info (Node));
   Subprogram_Symtbl_Entry : Ref_Symbol_Table_Entry :=
      Get_Symbol_Table_Entry (Get_Subprogram_Info (Node));

   ----------

   procedure Evaluate_Built_In_Subprogram
       (Symtbl_Entry : in Ref_Symbol_Table_Entry) is separate;

   function Evaluate_User_Defined_Subprogram_Call
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
   separate;

   function Evaluate_Infix_Operation
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
   separate;

   function Is_Infix_Operation
       (Symtbl_Entry : in Ref_Symbol_Table_Entry) return Boolean is
   begin
      return (Get_Built_In_Instance_Number (Symtbl_Entry) = Ace_Natural'FIRST);
   end Is_Infix_Operation;

   ----------

begin

   --   if ((Are_Symbolic_Names_Equal (Subprogram_Name,
   --                                  Convert_To_Ref_String ("break"))) or
   --      (Are_Symbolic_Names_Equal (Subprogram_Name,
   --                                 Convert_To_Ref_String ("continue")))) then
   --      Evaluate_Built_In_Subprogram (Subprogram_Symtbl_Entry);
   --      return (Node);
   --   end if;

   if Is_Removed (Subprogram_Symtbl_Entry) then
      Show_Error (4200, Fatal, Convert_To_String (Subprogram_Name));
      --        raise Invalid_Reference;
   end if;

   if Is_Infix_Operation (Subprogram_Symtbl_Entry) then
      return Evaluate_Infix_Operation (Node);
   end if;

   if Is_A_Derived_Subprogram (Subprogram_Symtbl_Entry) then
      declare
         Parent_Subprogram        : Ref_Symbol_Table_Entry :=
            Get_Derived_Subprogram_Parent (Subprogram_Symtbl_Entry);
         Parent_Formal_Parameter  : Ref_Symbol_Table_Entry :=
            Get_Head_Of_Parameters_List (Parent_Subprogram);
         Derived_Formal_Parameter : Ref_Symbol_Table_Entry :=
            Get_Head_Of_Parameters_List (Subprogram_Symtbl_Entry);
      begin
         ----------
         -- copy pertinent info from the parent entry
         -- [return_type and parameter_count should be correct]
         -- [derived entry is never a built-in subprogram]
         -- [derived entry is never derived from another derived entry]
         Set_Head_Of_Declarations_List
             (Subprogram_Symtbl_Entry,
              Get_Head_Of_Declarations_List (Parent_Subprogram));
         Set_Head_Of_Statements_List
             (Subprogram_Symtbl_Entry,
              Get_Head_Of_Statements_List (Parent_Subprogram));
         Set_Level (Subprogram_Symtbl_Entry, Get_Level (Parent_Subprogram));
         Set_Value_Offset (Subprogram_Symtbl_Entry,
                           Get_Value_Offset (Parent_Subprogram));
         -----
         -- copy offset values from each parent formal parameter
         while not Is_Empty (Parent_Formal_Parameter) loop
            Set_Level (Derived_Formal_Parameter,
                       Get_Level (Parent_Formal_Parameter));
            Set_Value_Offset (Derived_Formal_Parameter,
                              Get_Value_Offset (Parent_Formal_Parameter));
            Parent_Formal_Parameter :=
               Get_Next_Parameter (Parent_Formal_Parameter);
            Derived_Formal_Parameter :=
               Get_Next_Parameter (Derived_Formal_Parameter);
         end loop;
         -----
         ----------
      end;
   end if;

   if ((not Is_Subprogram_Built_In (Subprogram_Symtbl_Entry)) and
      (Is_Empty (Get_Head_Of_Statements_List (Subprogram_Symtbl_Entry)))) then
      Show_Error (1405, Fatal, Convert_To_String (Subprogram_Name));
   end if;

   return Evaluate_User_Defined_Subprogram_Call (Node);

end Evaluate_Subprogram_Call;
