--|$Log:	euser.a,v $
--|Revision 8.0  91/01/17  10:45:41  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Subunit for Package - Statement_Evaluator.Evaluate_Subprogram_Call
--
---------------------------------------------------------------
--
--
-- Abstract
--
--   The following routine is a separate unit of
--   Statement_Evaluator.Evaluate_Subprogram_Call
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
--==========================================================================
--  <Subprogram Name>
--
--    Evaluate_User_Defined_Subprogram_Call
--
--  <Description>
--
--    Evaluates procedure and function calls.
--
--==========================================================================

with Node_Set_Routines;
use Node_Set_Routines;

with Symbol_Table_Routines;
use Symbol_Table_Routines;

with Debugger;
use Debugger;

with Declarative_Region_Support_Routines;
use Declarative_Region_Support_Routines;

with Context_Control;

separate (
   Statement_Evaluator.Evaluate_Statement.Evaluate_Individual_Statement.
      Evaluate_Subprogram_Call)

function Evaluate_User_Defined_Subprogram_Call
    (Node : in Ref_Statement_Database) return Ref_Statement_Database is

   Subprogram_Name         : Ref_String :=
      Get_Symbolic_Name (Get_Subprogram_Info (Node));
   Subprogram_Symtbl_Entry : Ref_Symbol_Table_Entry :=
      Get_Symbol_Table_Entry (Get_Subprogram_Info (Node));

   -- [tbd] change this to a linked list structure -- oei
   Actuals_Interim_Storage : array (Ace_Max_Levels) of Ref_Data_Storage;
   -- := (others => null); -- We assume these will be null.
   Count                   : Ace_Max_Levels := Ace_Max_Levels'FIRST;

   Temp                    : Ref_Data_Storage;

   Formal_Parameter        : Ref_Symbol_Table_Entry;
   Actual_Argument         : Ref_Statement_Database;
   Previous_Scope          : Ref_Statement_Database;

   Current_Entry           : Context_Control.Ref_List_Of_Entries;

   Actuals_Evaluated       : Boolean        := False;

   procedure Start_Of_Call is

      procedure Evaluate_Actual_Parameters is
      begin
         -- evaluate actual arguments in "previous" environment
         ---- Note:
         -- use of Actuals_Interim_Storage and this subprogram constitute
         -- a slight-of-hand use of Ada's block structured facility
         -- as ACE's run-time storage stack -- providing implicit
         -- data storage for recursion and dynamic scoping of ACE subprograms
         -- [this is done in Evaluate_Infix_Operation also]
         ----
         Actual_Argument := Get_Parameter_List (Node);
         while (not Is_Empty (Actual_Argument)) loop
            -- evaluate actual and save for later (even OUT mode formals)
            declare
               Tmp_Value : Ref_Data_Storage :=
                  Get_Value
                      (Evaluate_Expression (Get_Expression (Actual_Argument)));
            begin
               Actuals_Interim_Storage (Count) := Copy_Value (Tmp_Value);
               -- if value came from a function then we must free it...
               if Check_Expr_Node (Get_Expression (Actual_Argument)) =
                  Function_Call then
                  Free (Tmp_Value);
               end if;
            end;
            Actual_Argument := Get_Next_List_Item (Actual_Argument);
            Count := Count + 1;
         end loop;
      end Evaluate_Actual_Parameters;
      pragma INLINE (Evaluate_Actual_Parameters);

      procedure Setup_Formal_Parameters is
      begin
         --[ perform before-call operations on (formal & actual) parameters ]--
         --    - push parameters onto the front of the symbol table
         --      (so params are visible for debugger-type purposes)
         --    - allocate storage for formal parameters
         --    - associate actual parameter values to formal parameters
         Formal_Parameter :=
            Get_Head_Of_Parameters_List (Subprogram_Symtbl_Entry);
         Count := Ace_Max_Levels'FIRST;
         while (not Is_Empty (Formal_Parameter)) loop
           -- add copy of formal parameter entry for "debugging"-type visibility
            if Is_Symbol_Table_Debugging_On then
               Add_Duplicate_Of_Entry_To_Symbol_Table (Context,
                                                       Formal_Parameter);
            end if;
            Observe_Object_If_Objects_Observation_Is_On
                (Formal_Parameter);

            -- [actually, here we do not need to "new" a storage location,
           --    just update the "last_dss_sp"], since we are going to assign it
           -- the value Actuals_Interim_Storage (Count).  So for now we free the
            -- locate right away.
            Allocate_Storage (Context, Formal_Parameter);
            declare
               Not_Needed : Ref_Data_Storage := Fetch_Value (Context,
                                                             Formal_Parameter);
            begin
               Free (Not_Needed);
            end;

            -- get stored values of actuals and assign to formals
            --   (even OUT mode formals)
            Store_Value (Context, Formal_Parameter,
                         Actuals_Interim_Storage (Count));
            -- Actuals_Interim_Storage get used or free-ed later
            -- (eval any init assignments for parameters) [tbd]
            Formal_Parameter := Get_Next_Parameter (Formal_Parameter);
            Count := Count + 1;
         end loop;
      end Setup_Formal_Parameters;
      pragma INLINE (Setup_Formal_Parameters);

   begin
      -- First  save the old scope
      Previous_Scope := Context_Control.Get_Enclosing_Scope (Context);
      -- Set the enclosing scope to the subprogram so we can handle an
      -- exception when it occurs.
      Context_Control.Set_Enclosing_Scope (Context, Get_Subprogram_Info (Node));

      -- Evaluation must be done before we setup new environment.
      Evaluate_Actual_Parameters;
      Actuals_Evaluated := True;

      -- prepare data storage stack for new environment
      Context_Control.Push_Display_For_Level
          (Context, Get_Level (Subprogram_Symtbl_Entry) + 1);

      Context_Control.Push_Last_Storage_Location_Pointer
          (Context);

      Context_Control.Set_Display_For_Level
          (Context, Get_Level (Subprogram_Symtbl_Entry) + 1,
           Context_Control.Get_Last_Storage_Location_Pointer (Context) + 1);

      -- keep pointer to current top of symbol table
      Context_Control.Push_Top_Of_Symtbl
          (Context, Context_Control.Get_Top_Of_Symtbl (Context));

      -- if subpgm is w/in a package,
      --    make package body entries visible for "debugging" puposes
      --    [only add to symtbl body specific entries -- subpgm specs w/
      --     separate bodies are already on symtbl]
      if (not Is_Empty (Get_Enclosing_Package (Subprogram_Symtbl_Entry))) then
         if Is_Symbol_Table_Debugging_On then
            Add_Package_Decls_To_Symbol_Table
                (Context, Get_Enclosing_Package (Subprogram_Symtbl_Entry),
                 Pkg_Body);
         end if;
         if Is_Objects_Observe_On then
            null;
            -- [tbd] add pkg body entries to observation of objects file
         end if;
      end if;

      Setup_Formal_Parameters;

      Context_Control.Increment_Call_Level (Context);
      Context_Control.Set_In_Subprogram_Call (Context);

      if Is_Symbol_Table_Debugging_On then
         Context_Control.Set_Dregion_Count
             (Context, Context_Control.Get_Dregion_Count (Context) + 1);
      end if;

   end Start_Of_Call;
   -- pragma INLINE (Start_Of_Call);

   procedure End_Of_Call is
   begin
      if Actuals_Evaluated then

         Context_Control.Unset_In_Subprogram_Call (Context);


         if Context_Control.Get_Call_Level (Context) > 0 then
            Context_Control.Decrement_Call_Level (Context);
         end if;

         -- [for functions only]
       --    "copy" function value (in func symtbl entry) to current_node.value;
         if (Get_Entry_Kind (Subprogram_Symtbl_Entry) = Function_Entry) then
            Set_Value (Node, Fetch_Value (Context, Subprogram_Symtbl_Entry));
         end if;

         -- Copy OUT and IN_OUT formal parameters to actuals holding area
         Formal_Parameter :=
            Get_Head_Of_Parameters_List (Subprogram_Symtbl_Entry);
         Count := Ace_Max_Levels'FIRST;
         while (not Is_Empty (Formal_Parameter)) loop
            if (Is_Mode_Out (Formal_Parameter) or else
               Is_Mode_In_Out (Formal_Parameter)) then
               Actuals_Interim_Storage (Count) :=
                  Fetch_Value (Context, Formal_Parameter);
            else
               Free (Actuals_Interim_Storage (Count)); -- it's an IN mode formal
            end if;
            Formal_Parameter := Get_Next_Parameter (Formal_Parameter);
            Count := Count + 1;
         end loop;

      end if;

      Deallocate_Locals
          (Context, Get_Head_Of_Declarations_List (Subprogram_Symtbl_Entry));

      Context_Control.Set_Enclosing_Scope (Context, Previous_Scope);

      if Actuals_Evaluated then

         -- restore original pointer to symbol table
         -- (pop parameters and local variables off the front of the symtbl)
         Context_Control.Pop_Top_Of_Symtbl (Context);

         -- restore previous environment of data storage stack
         Context_Control.Pop_Display_For_Level
             (Context, Get_Level (Subprogram_Symtbl_Entry) + 1);

         Context_Control.Pop_Last_Storage_Location_Pointer
             (Context);

         if Is_Symbol_Table_Debugging_On then
            -- reset entries that were hidden w/in this declarative region
            while (not Context_Control."=" (Current_Entry, null)) loop
               Make_Entry_Visible (Current_Entry.Hidden_Entry);
               Current_Entry := Current_Entry.Next_Hidden_Entry;
            end loop;
            -- [tbd] Free (Hidden_Entries (Get_Dregion_Count(Context)),
            --              Entire_Substructure);

            Context_Control.Set_Dregion_Count
                (Context, Context_Control.Get_Dregion_Count (Context) - 1);
         end if;
      end if;
   end End_Of_Call;
   pragma INLINE (End_Of_Call);

begin

   Start_Of_Call;

   if (Is_Subprogram_Built_In (Subprogram_Symtbl_Entry)) then

      -- do a built-in
      Evaluate_Built_In_Subprogram (Subprogram_Symtbl_Entry);

   else

      -- interpret the subprogram declarations & statements
      Initialize_Statements_Observation_If_Statements_Observation_Is_On
          (Subprogram_Symtbl_Entry);

      Evaluate_Statement
          (Context, Get_Head_Of_Declarations_List (Subprogram_Symtbl_Entry));

      Evaluate_Statement
          (Context, Get_Head_Of_Statements_List (Subprogram_Symtbl_Entry));

   end if;

   -- for built-in's and "return"-less user-defined procedures
   raise Return_Statement_Encountered;

exception

   when Return_Statement_Encountered     =>

      End_Of_Call;

      -- return OUT and IN_OUT mode formals to actuals (in "previous" environ)
      Actual_Argument := Get_Parameter_List (Node);
      Count := Ace_Max_Levels'FIRST;
      while (not Is_Empty (Actual_Argument)) loop
         if (not Is_Empty (Actuals_Interim_Storage (Count))) then
            -- mode for formal is OUT or IN_OUT
            Store_Value
                (Context,
                 (Get_Symbol_Table_Entry (Get_Expression (Actual_Argument))),
                 Actuals_Interim_Storage (Count));
         end if;
         Actual_Argument := Get_Next_List_Item (Actual_Argument);
         Count := Count + 1;
      end loop;

      if (not Is_Subprogram_Built_In (Subprogram_Symtbl_Entry)) then
         Reset_Statements_Observation_If_Statements_Observation_Is_On
            ;
      end if;

      -- return result of operation (meaningful for functions only)

      return (Node);

   when others  =>

      End_Of_Call;

      raise ;

end Evaluate_User_Defined_Subprogram_Call;
