--|$Log:	expand.a,v $
--|Revision 8.0  91/01/17  10:45:42  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Expanded_Names_Support
--
---------------------------------------------------------------
--
--
-- Abstract
--
--   Contains routines to support expanded names for object names passed
--  to subroutines via parameter strings.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------

with Literals;
use Literals;

with Statement_Database;
use Statement_Database;

with Symbol_Table_Structures;
use Symbol_Table_Structures;

with Create_Statement_Database;

with Context_Control;
use Context_Control;

with Symtbl_Entry_Get_Routines;
use Symtbl_Entry_Get_Routines;

with Symbol_Table_Routines;
use Symbol_Table_Routines;

with Symtbl_Entry_Get_Routines;
use Symtbl_Entry_Get_Routines;

with Node_Get_Routines;
use Node_Get_Routines;

with Misc_Support_Routines;
use Misc_Support_Routines;

with Semantic_Analyzer;

with Ace_Universal_Types;

with Error_Handler;
use Error_Handler;

package body Expdname is


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Handle_Expanded_Name
   --
   --  <Description>
   --
   --    When passed an object name, will return the symbol table entry of
   --    the enclosing unit and the simple name of the object.
   --==========================================================================

   procedure Handle_Expanded_Name (Context        : in Context_Definition;
                                   Object_Name    : in out Ref_String;
                                   Enclosing_Unit : out Ref_Symbol_Table_Entry)
                                  is

      Object_Name_Str : constant String := Convert_To_String (Object_Name);
      Temp_Name       : String (1 .. Object_Name_Str'LENGTH) := Object_Name_Str;
      Dot_Index       : Integer := 0;
      String_Start    : Integer := Temp_Name'FIRST;
      String_End      : constant Integer                     := Temp_Name'LAST;
      Name_Node       : Ref_Statement_Database               := null;
      -- need dup_context 'cause some functions expect it to be in,
      --   others need it in out. Since I don't modify it, in is good 'nough
      Dup_Context     : Context_Definition                   := Context;

      function Build_Name (Name_Str : String) return Ref_Statement_Database is
         New_Name_Node : Ref_Statement_Database;
      begin
         New_Name_Node :=
            Create_Statement_Database.Create_Expr_Node (Name_Str,
                                                        Name_Str'LENGTH);
         if Misc_Support_Routines.Is_Empty (Name_Node) then
            return New_Name_Node;

         else
            return
               Create_Statement_Database.Create_Dot_Expr_Node (Name_Node,
                                                               New_Name_Node);
         end if;
      end Build_Name;

   begin

      for I in String_Start .. String_End loop
         if Temp_Name (I) = '.' then
            Name_Node := Build_Name (Temp_Name (String_Start .. (I - 1)));
            String_Start := I + 1;
         end if;
      end loop;

      Object_Name := Convert_To_Ref_String (Temp_Name (String_Start ..
                                                       String_End));
      if not Misc_Support_Routines.Is_Empty (Name_Node) then
         Semantic_Analyzer.Check_Expression (Dup_Context, Name_Node);
         Enclosing_Unit := Node_Get_Routines.Get_Symbol_Table_Entry (Name_Node);
      else
         Enclosing_Unit := null;
      end if;

   end Handle_Expanded_Name;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Is_Declaration_Node
   --
   --  <Description>
   --
   --    Returns whether the given object is a declaration node (except for
   --    exceptions, which aren't supported yet anyway)
   --
   --==========================================================================

   function Is_Declaration (Node : in Ref_Statement_Database) return Boolean is
   begin
      if Node.Node_Kind = Statement_Node_Type then
         case Check_Stmt_Node (Node) is
            when Type_Declaration           |
            -- Exception_Declaration      | -- not fully implemented yet
                 Subtype_Declaration        |
                 Procedure_Spec_Declaration |
                 Procedure_Declaration      |
                 Function_Spec_Declaration  |
                 Function_Declaration       |
                 Package_Declaration        |
                 Package_Body_Declaration     =>
               return True;
            when others   =>
               return False;
         end case;
      else
         return False;
      end if;
   end Is_Declaration;


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Do_Forms_Match
   --
   --  <Description>
   --
   --    will return wheter a symbol table entry matches a form string
   --==========================================================================

   function Do_Forms_Match (Ref      : Ref_Symbol_Table_Entry;
                            Form_Str : String) return Boolean is
      use Ace_Universal_Types;
   begin
      if (Form_Str = "") then
         return True;
      elsif (Form_Str = "all") then
         return (Is_Visible (Ref));
      elsif (Form_Str = "hidden") then
         return (Is_Hidden (Ref));
      elsif (Form_Str = "deleted") then
         return (Is_Removed (Ref));
      else
         return (Get_Statement_Number (Ref) = Ace_Integer'VALUE (Form_Str));
      end if;

   end Do_Forms_Match;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Find_Next_Matching_Name
   --
   --  <Description>
   --
   --    Find_Next_Matching_Name traverses the statement database for the
   --    next occurrence of the name, starting at the current statement
   --    (curr_stmt).  This routine will look at all names, even those not
   --    directly visible.  If a null form string is given the next occurrence
   --    is returned via the Found_Node parameter; otherwise the next node
   --    matching the form string is returned. When invoked with the type
   --    of the name (Entry_Type), only the name defined by the specified type
   --    will be found.
   --==========================================================================
   procedure Find_Next_Matching_Name
       (Context    : in Context_Definition;
        Curr_Stmt  : in out Ref_Statement_Database;
        Name       : in Ref_String;
        Form_Str   : in String;
        Found_Node : out Ref_Symbol_Table_Entry;
        Entry_Type : in Type_Entry_Enumeration := Unknown) is
      Curr_Decl_Node : Ref_Statement_Database := Curr_Stmt;

   begin
      while not Is_Empty (Curr_Decl_Node) loop
         if Is_Declaration (Curr_Decl_Node) and then
				Are_Symbolic_Names_Equal(Get_Symbolic_Name
				(get_Symbol_Table_Entry(Curr_decl_Node)), Name)
					 then
            Found_Node := Get_Symbol_Table_Entry (Curr_Decl_Node);
            Curr_Stmt := Get_Next_Statement (Curr_Decl_Node);
            return ;
         elsif (Check_Stmt_Node (Curr_Decl_Node) = Object_Declaration) then
            declare
               Curr_Name_Node : Ref_Statement_Database :=
                  Get_Name_List (Curr_Decl_Node);
            begin
               while not Is_Empty (Curr_Name_Node) loop
                  if Are_Symbolic_Names_Equal
                         (Get_Symbolic_Name
                              (Get_Symbol_Table_Entry
                                   (Get_Expression (Curr_Name_Node))), Name)
                      and
                     (Entry_Type = Unknown or else
                     Entry_Type =
                     Get_Entry_Kind
                         (Get_Symbol_Table_Entry
                              (Get_Expression (Curr_Name_Node)))) then
                     Found_Node :=
                        Get_Symbol_Table_Entry
                            (Get_Expression (Curr_Name_Node));
                     Curr_Stmt := Get_Next_Statement (Curr_Decl_Node);
                     return ;
                  else
                     Curr_Name_Node := Get_Next_List_Item (Curr_Name_Node);
                  end if;
               end loop;
            end;
         end if;
         Curr_Decl_Node := Get_Next_Statement (Curr_Decl_Node);
      end loop;

      if not (Is_Empty (Curr_Decl_Node)) then
         Found_Node := Get_Symbol_Table_Entry (Curr_Decl_Node);
      else
         Found_Node := null;
      end if;
   exception
      when Constraint_Error    =>
         Show_Error (3018, Fatal, Form_Str);


   end Find_Next_Matching_Name;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Find_Matching_Name
   --
   --  <Description>
   --
   --    This version of Find_Matching_name searches for a Symbol_Table_entry
   --     for a given expanded name, that is, within some enclosing package
   --   which might not be currently on the Symbol_Table. This routine will
   --   look at any name in the enclosing package, including a hidden or
   --   deleted one by traversing the statement database. If a form_string is




   --   provided, the first entry matching it will be returned, otherwise the
   --     first occurance will be returned.
   --   The symbol table node will be returned via the "found_node" parameter.
   --   The last statement checked will be returned via the Last_Stmt parameter.
   --==========================================================================
   procedure Find_Matching_Name (Context       : in Context_Definition;
                                 Enclosing_Pkg : in Ref_Symbol_Table_Entry;
											Name          : in Ref_String;
                                 Form_Str      : in String;
                                 Found_Node    : out Ref_Symbol_Table_Entry;
                                 Last_Stmt     : out Ref_Statement_Database;
                                 Entry_Type    : in Type_Entry_Enumeration :=
      Unknown) is
      Curr_Decl_Node : Ref_Statement_Database;
   begin
      if Get_Entry_Kind (Enclosing_Pkg) = Package_Entry then
         Curr_Decl_Node := Get_Package_Spec_Decls (Enclosing_Pkg);
         Find_Next_Matching_Name (Context, Curr_Decl_Node, Name, Form_Str,
                                  Found_Node, Entry_Type);
			Last_Stmt := Curr_Decl_Node;
      else
         Show_Error (4206, Fatal,
                     Convert_To_String (Get_Symbolic_Name (Enclosing_Pkg)));
      end if;

   end Find_Matching_Name;


end Expdname;
