--|$Log:	files.a,v $
--|Revision 8.0  91/01/17  10:45:43  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Ace_File_System
--
---------------------------------------------------------------
--
--
-- Abstract
--
--  This package contains the builtin routines for the ACE file
--  system.
--
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
with Text_Io;
use Text_Io;

with Literals;
use Literals;

with System_Dependent_Routines;
use System_Dependent_Routines;

with Ooe;
use Ooe;

--with Stubber;
package body Ace_File_System is

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Create
   --
   --  <Description>
   --
   --    Create routine for File_Objects (similar to Text_Io.)
   --
   --  <Exceptions Raised>
   --
   --    Same as Text_Io Create.
   --
   --==========================================================================
   procedure Create (Obj  : in out File_Object;
                     Mode : in Object_Mode := Out_Object;
                     Name : in String      := "";
                     Form : in String      := "") is
   begin

      for I in File_Object'FIRST + 1 .. File_Object'LAST - 2 loop

         -- find an unused Obj entry

         if not Text_Io.Is_Open (File_Entries (I)) then

            Create (Files (I), Text_Io.File_Mode'VAL (Object_Mode'POS (Mode)),
                    Name, Form);
            Obj := I;
            return ;

         end if;

      end loop;

      -- reached the limit on number of files
      raise Ace_Text_Io.Use_Error;
   end Create;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Open
   --
   --  <Description>
   --
   --    Open routine for File_Objects (similar to Text_Io.)
   --
   --  <Exceptions Raised>
   --
   --    Same as Text_Io Open.
   --
   --==========================================================================
   procedure Open (Obj  : in out File_Object;
                   Mode : in Object_Mode;
                   Name : in String;
                   Form : in String := "") is

   begin

      for I in File_Object'FIRST + 1 .. File_Object'LAST - 2 loop

         -- find an unused Obj entry

         if not Text_Io.Is_Open (File_Entries (I)) then

            Open (Files (I), Text_Io.File_Mode'VAL (Object_Mode'POS (Mode)),
                  Name, Form);
            Obj := I;
            return ;

         end if;

      end loop;

      -- reached the limit on number of files

      raise Ace_Text_Io.Use_Error;

   end Open;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Delete
   --
   --  <Description>
   --
   --    Delete routine for File_Objects (similar to Text_Io.)
   --
   --  <Exceptions Raised>
   --
   --    Same as Text_Io Delete.
   --
   --==========================================================================
   procedure Delete (Obj : in out File_Object) is
   begin
      Delete (Files (Obj));
   end Delete;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Close
   --
   --  <Description>
   --
   --    Close routine for File_Objects (similar to Text_Io.)
   --
   --  <Exceptions Raised>
   --
   --    Same as Text_Io Close.
   --
   --==========================================================================
   procedure Close (Obj : in out File_Object) is
   begin
      Close (Files (Obj));
      Obj := File_Object'FIRST;
   end Close;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Rename
   --
   --  <Description>
   --
   --    This routine will rename a file from one name to another.
   --
   --  <Exceptions Raised>
   --
   --    Text_Io exceptions
   --
   --==========================================================================
   procedure Rename (Obj1, Obj2 : in out File_Object) is

      To_Ref : Ref_String := Convert_To_Ref_String (Name (File_Entries (Obj2)));
      From_Ref : Ref_String :=
         Convert_To_Ref_String (Name (File_Entries (Obj1)));

   begin
      Delete (Obj2);
      Rename (From_Ref, To_Ref);
      Open (Obj2, In_Object, Convert_To_String (To_Ref), "");
      Close (Obj1);
   end Rename;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Copy
   --
   --  <Description>
   --
   --    This routine will copy the contents of one file to another file.
   --
   --  <Exceptions Raised>
   --
   --    Text_Io exceptions
   --
   --==========================================================================
   procedure Copy (Obj1, Obj2 : in out File_Object) is
      Object1 : Ref_String :=
         Convert_To_Ref_String (Name (File_Entries (Obj1)));
      Object2 : Ref_String :=
         Convert_To_Ref_String (Name (File_Entries (Obj2)));
   begin
      Close (Obj1);
      Close (Obj2);
      System_Dependent_Routines.Copy (Object1, Object2);
      Ace_File_System.Open (Obj1, In_Object, Convert_To_String (Object1));
      Ace_File_System.Open (Obj2, In_Object, Convert_To_String (Object2));
   end Copy;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    List
   --
   --  <Description>
   --
   --    This routine will list the contents of an object (directory, or
   --    text);
   --
   --  <Exceptions Raised>
   --
   --    Text_Io exceptions
   --
   --==========================================================================
   procedure List (Obj   : in File_Object;
                   Which : in Ace_Integer) is

      Line    : String (1 .. 133);
      Last    : Natural;
      Ref_Dir : Ref_String;

   begin
      -- File_Lister(Obj);
      Reset (Files (Obj));
      while not End_Of_File (File_Entries (Obj)) loop
         Get_Line (File_Entries (Obj), Line, Last);
         if Last /= 0 then
            Ace_Text_Io.Put_Line (Line (1 .. Last));
         else
            Ace_Text_Io.New_Line;
         end if;
      end loop;
   end List;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Change_Directory
   --
   --  <Description>
   --
   --    The following routine will change the directory to which the users
   --    actions will default.  (i.e., if the user doesn't specify the full
   --    path name on an object the system will use the current working
   --    directory).
   --
   --  <Exceptions Raised>
   --
   --    Text_Io exceptions
   --
   --==========================================================================
   procedure Change_Directory (Obj : in File_Object) is

      Dir_Ref : Ref_String := Convert_To_Ref_String (Name (File_Entries (Obj)));

   begin
      Chdir (Dir_Ref);
   end Change_Directory;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Edit
   --
   --  <Description>
   --
   --    Edit will call the host operating system edit command on
   --    a specified file.
   --
   --  <Exceptions Raised>
   --
   --    Text_Io exceptions
   --
   --==========================================================================
   procedure Edit (Obj : in out File_Object;
                   How : in Execution_Mode) is
   begin
      System_Dependent_Routines.Edit (Obj, How);
   end Edit;

   --==========================================================================
   --+--  <Subprogram Name>
   --
   --    Compile
   --
   --  <Description>
   --
   --    Compile will call the host operating system compile command on
   --    a specified file.
   --
   --  <Exceptions Raised>
   --
   --    Text_Io exceptions
   --
   --==========================================================================
   procedure Compile (Obj  : in File_Object;
                      Form : in String) is
   begin
      System_Dependent_Routines.Compile (Obj, Form);
   end Compile;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Stubber
   --
   --  <Description>
   --
   --    This is the stubber that is found in the repository.
   --    It has been modified slightly to become integrated with the
   --    ACE File_System.
   --
   --==========================================================================
   -- procedure Stubber (Obj1, Obj2 : in out File_Object) is
   --    -- Call the repository stubber
   --    File1 : constant String := Name (File_Entries (Obj1));
   --    File2 : constant String := Name (File_Entries (Obj2));
   -- begin
   --    Delete (Files (Obj2));
   --    Close (Files (Obj1));
   --    Stubber (File1, File2);
   --    Text_Io.Open (Files (Obj2), In_File, File2);
   --    Text_Io.Open (Files (Obj1), In_File, File1);
   -- end Stubber;

   -- This is a stub for the stubber, which was removed from the ACE
   -- environment in an attempt to stream line the environment.
   procedure Stubber (Obj1, Obj2 : in out File_Object) is
   begin
      null;
   end Stubber;


end Ace_File_System;
