--|$Log:	get.a,v $
--|Revision 8.0  91/01/17  10:45:44  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Node_Get_Routines
--
------------------------------------------------------------
--
--
-- Abstract
--
--    This package contains a set of GET routines that provide access
--    to statement database nodes.  These routines support access to both
--    statement kind and expression kind statement database nodes.
--
--
-- Update Log - :
--
-- End of Header
-------------------------------------------------------------------------

with Statement_Database;
use Statement_Database;

with Symbol_Table_Structures;
use Symbol_Table_Structures;

with Symtbl_Entry_Get_Routines;
use Symtbl_Entry_Get_Routines;

with Misc_Support_Routines;
use Misc_Support_Routines;

with Storage_Management;
use Storage_Management;

with Storage_Management_Routines;
use Storage_Management_Routines;

with Ace_Universal_Types;
use Ace_Universal_Types;

with Error_Handler;
use Error_Handler;

package body Node_Get_Routines is

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Get_Database_Node
   --
   --    Last Mod:    Thu Sep  7 09:44:22 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --    Returns the expanded name's real statement_database node.
   --
   --  --------------------------------------------------------------------
   function Get_Database_Node
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
   begin
      return Node.Expression_Node.Expn_Terminal_Sdb;
   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Database_Node");
         raise ;
   end Get_Database_Node;
   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Is_Statement_Node
   --
   --  <Description>
   --
   --    Returns TRUE if the given NODE is a Statement Node
   --
   --=======================================================================
   function Is_Statement_Node (Node : in Ref_Statement_Database) return Boolean
                              is
   begin
      return (Node.Node_Kind = Statement_Node_Type);

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Is_Statement_Node");
         raise ;

   end Is_Statement_Node;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Is_Expression_Node
   --
   --  <Description>
   --
   --    Returns TRUE if the given NODE is an Expression Node
   --
   --=======================================================================
   function Is_Expression_Node
       (Node : in Ref_Statement_Database) return Boolean is
   begin
      return (Node.Node_Kind = Expression_Node_Type);

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Is_Expression_Node");
         raise ;

   end Is_Expression_Node;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Is_Attribte_Tick
   --
   --  <Description>
   --
   --    Returns TRUE if the given NODE is an Attribute_Tick expression
   --
   --=======================================================================
   function Is_Attribute_Tick (Node : in Ref_Statement_Database) return Boolean
                              is
   begin
      return (Node.Expression_Node.Expression_Kind = Attribute_Tick);

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Is_Attribte_Tick");
         raise ;

   end Is_Attribute_Tick;

   -- ----------------------------------------------------------------------

   function Is_Declaration (Node : in Ref_Statement_Database) return Boolean is
   begin
      if Node.Node_Kind = Statement_Node_Type then
         case Node.Statement_Node.Statement_Kind is
            when Type_Declaration           |
            -- Exception_Declaration      | -- not fully implemented yet
                 Subtype_Declaration        |
                 Procedure_Spec_Declaration |
                 Procedure_Declaration      |
                 Function_Spec_Declaration  |
                 Function_Declaration       |
                 Package_Declaration        |
                 Package_Body_Declaration     =>
               return True;
            when others   =>
               return False;
         end case;
      else
         return False;
      end if;
   end Is_Declaration;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Is_Removed
   --
   --  <Description>
   --
   --   Return whether the symbol table entry associated with node has
   --    been "removed" from the symbol table
   --=======================================================================
   function Is_Removed (Node : in Ref_Statement_Database) return Boolean is
   begin
      if (Is_Declaration (Node)) then
         return (Is_Removed (Get_Symbol_Table_Entry (Node)));
      else
         return False;
      end if;
   end Is_Removed;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Next_Statement
   --
   --  <Description>
   --
   --    Returns the Next Statement Database Node
   --
   --=======================================================================
   function Get_Next_Statement
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Next_Statement
   begin
      return Node.Statement_Node.Next_Statement;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Next_Statement");
         raise ;

   end Get_Next_Statement;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Comments_List
   --
   --  <Description>
   --
   --    Returns the comments list for the current Node
   --
   --=======================================================================
   function Get_Comments_List
       (Node : in Ref_Statement_Database) return Ref_Comment is
   begin
      return Node.Comment_Holder;
   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Comments_List1");
         raise ;
   end Get_Comments_List;

   function Get_Comments_List (Node : in Ref_Comment) return Ref_Comment is
   begin
      return Node.Next;
   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Comments_List2");
         raise ;
   end Get_Comments_List;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Comment
   --
   --  <Description>
   --
   --    Returns the comment(s) for the current Node
   --
   --=======================================================================
   function Get_Comment (Node : in Ref_Comment) return String is
   begin
      return Node.Comment.all;
   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Comments_List2");
         raise ;
   end Get_Comment;

   -- -----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Holder
   --
   --  <Description>
   --
   --    Returns the "holder" (temporary) node of the current Node
   --
   --=======================================================================
   function Get_Holder
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Holder
      -- Next_Expression

   begin
      case Node.Node_Kind is
         when Statement_Node_Type   =>
            return Node.Statement_Node.Holder;

         when Expression_Node_Type  =>
            return Node.Expression_Node.Next_Expression;

         when others =>
            Show_Error (2000, Internal, "Get_Holder1");

      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Holder");
         raise ;

   end Get_Holder;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Should_Suspend_Evaluation_At_This_Statement
   --
   --  <Description>
   --
   --    Returns the whether evaluation should be suspend at current Node
   --
   --=======================================================================
   function Should_Suspend_Evaluation_At_This_Statement
       (Node : in Ref_Statement_Database) return Boolean is
      -- Suspend_Evaluation_At_This_Statement
   begin
      return Node.Statement_Node.Suspend_Evaluation_At_This_Statement;
   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Suspend_Evaluation_At_This_Statement");
         raise ;
   end Should_Suspend_Evaluation_At_This_Statement;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Statement_Number
   --
   --  <Description>
   --
   --    Returns the statement number for the current Node
   --
   --=======================================================================
   function Get_Statement_Number
       (Node : in Ref_Statement_Database) return Ace_Natural is
      -- Statement_Number
   begin
      return Node.Statement_Node.Statement_Number;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Statement_Number");
         raise ;

   end Get_Statement_Number;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Subprogram_Info
   --
   --  <Description>
   --
   --    Returns the subprogram information node for the current Node
   --
   --=======================================================================
   function Get_Subprogram_Info
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Procedure_Info
      -- Lp_Object (for function calls)

   begin
      case Node.Node_Kind is
         when Statement_Node_Type   =>
            case Node.Statement_Node.Statement_Kind is
               when Procedure_Call  =>
                  return Node.Statement_Node.Procedure_Info;
               when others     =>
                  Show_Error (2000, Internal, "Get_Subprogram_Info1");
            end case;

         when Expression_Node_Type  =>
            case Node.Expression_Node.Expression_Kind is
               when Left_Parenthesis_Operator =>
                  return Node.Expression_Node.Lp_Object;
               when others     =>
                  Show_Error (2000, Internal, "Get_Subprogram_Info2");
            end case;

         when others =>
            Show_Error (2000, Internal, "Get_Subprogram_Info3");

      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Subprogram_Info");
         raise ;

   end Get_Subprogram_Info;

   -- ----------------------------------------------------------------------

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Symbol_Table_Entry
   --
   --  <Description>
   --
   --    Returns the symbol table entry for the current Node
   --
   --=======================================================================
   function Get_Symbol_Table_Entry
       (Node : in Ref_Statement_Database) return Ref_Symbol_Table_Entry is
      -- ExpN_Terminal_Symbol
      -- Symbol_Table_Entry
      -- Terminal_Symbol

   begin
      case Node.Node_Kind is
         when Statement_Node_Type   =>
            case Node.Statement_Node.Statement_Kind is
               when Exception_Declaration      |
                    Type_Declaration           |
                    Subtype_Declaration        |
                    Procedure_Spec_Declaration |
                    Procedure_Declaration      |
                    Function_Spec_Declaration  |
                    Function_Declaration       |
                    Package_Declaration        |
                    Package_Body_Declaration  =>
                  return Node.Statement_Node.Symbol_Table_Entry;
               when others     =>
                  Show_Error (2000, Internal, "Get_Symbol_Table_Entry1");
            end case;

         when Expression_Node_Type  =>
            case Node.Expression_Node.Expression_Kind is
               when Dot_Operator    =>
                  return Node.Expression_Node.Expn_Terminal_Symbol;
               when Terminal_Node   =>
                  return Node.Expression_Node.Terminal_Symbol;
               when Left_Parenthesis_Operator =>
                  return
                     Node.Expression_Node.Lp_Object.Expression_Node.
                        Terminal_Symbol;
               when others     =>
                  Show_Error (2000, Internal, "Get_Symbol_Table_Entry2");
            end case;

         when others =>
            Show_Error (2000, Internal, "Get_Symbol_Table_Entry3");

      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Symbol_Table_Entry");
         raise ;

   end Get_Symbol_Table_Entry;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Lhs
   --
   --  <Description>
   --
   --    Returns the LHS for the current Assignment Statement Node
   --
   --=======================================================================
   function Get_Lhs
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Lhs

   begin
      return Node.Statement_Node.Lhs;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Lhs");
         raise ;

   end Get_Lhs;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Rhs
   --
   --  <Description>
   --
   --    Returns the RHS for the current Assignment Statement Node
   --
   --=======================================================================
   function Get_Rhs
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Rhs

   begin
      return Node.Statement_Node.Rhs;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Rhs");
         raise ;

   end Get_Rhs;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Condition
   --
   --  <Description>
   --
   --    Returns the condition expression for the current Node
   --
   --=======================================================================
   function Get_Condition
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- If_Condition
      -- Exit_Condition

   begin
      case Node.Statement_Node.Statement_Kind is
         when If_Statement     =>
            return Node.Statement_Node.If_Condition;

         when Exit_Statement   =>
            return Node.Statement_Node.Exit_Condition;

         when others =>
            Show_Error (2000, Internal, "Get_Condition1");

      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Condition");
         raise ;

   end Get_Condition;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Loop_Scheme
   --
   --  <Description>
   --
   --    Returns the loop scheme for the current Node
   --=======================================================================

   function Get_Loop_Scheme
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Loop_Scheme

   begin
      return Node.Statement_Node.Loop_Scheme;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Loop_Scheme");
         raise ;

   end Get_Loop_Scheme;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_For_Variable
   --
   --  <Description>
   --
   --    Returns the FOR variable for the current Node
   --
   --=======================================================================
   function Get_For_Variable
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- For_Variable

   begin
      return Node.Statement_Node.For_Variable;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_For_Variable");
         raise ;

   end Get_For_Variable;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_For_Range
   --
   --  <Description>
   --
   --    Returns the FOR range for the current Node
   --
   --=======================================================================
   function Get_For_Range
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- For_Range

   begin
      return Node.Statement_Node.For_Range;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_For_Range");
         raise ;

   end Get_For_Range;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Name_Range
   --
   --  <Description>
   --
   --    Returns the name range for the current Node
   --
   --=======================================================================
   function Get_Name_Range
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Name_Range

   begin
      return Node.Expression_Node.Range_Name;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Name_Range");
         raise ;

   end Get_Name_Range;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Head_Of_Statements_List
   --
   --  <Description>
   --
   --    Returns the head of the statements list for the current Node
   --
   --=======================================================================
   function Get_Head_Of_Statements_List
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Loop_Body
      -- Block_Statements

   begin
      case Node.Statement_Node.Statement_Kind is
         when Loop_Statement   =>
            return Node.Statement_Node.Loop_Body;

         when Block_Statement  =>
            return Node.Statement_Node.Block_Statements;

         when others =>
            Show_Error (2000, Internal, "Get_Head_Of_Statements_List1");

      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Head_Of_Statements_List");
         raise ;

   end Get_Head_Of_Statements_List;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Head_Of_True_Task_Statements_List
   --
   --  <Description>
   --
   --    Returns the head of the true task statement for the current Node
   --
   --=======================================================================
   function Get_Head_Of_True_Task_Statements_List
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- True_Task

   begin
      return Node.Statement_Node.True_Task;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Head_Of_True_Task_Statements_List");
         raise ;

   end Get_Head_Of_True_Task_Statements_List;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Head_Of_False_Task_Statements_List
   --
   --  <Description>
   --
   --    Returns the head of the false task statement for the current Node
   --
   --=======================================================================
   function Get_Head_Of_False_Task_Statements_List
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- False_Task

   begin
      return Node.Statement_Node.False_Task;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Head_Of_False_Task_Statements_List");
         raise ;

   end Get_Head_Of_False_Task_Statements_List;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Expression
   --
   --  <Description>
   --
   --    Returns the expression for the current Node
   --
   --=======================================================================
   function Get_Expression
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Init_Expression
      -- Case_Expression
      -- Return_Expression
      -- Item_Expression

   begin
      case Node.Node_Kind is
         when Statement_Node_Type   =>
            case Node.Statement_Node.Statement_Kind is
               when Object_Declaration   =>
                  return Node.Statement_Node.Init_Expression;

               when Case_Statement  =>
                  return Node.Statement_Node.Case_Expression;

               when Return_Statement     =>
                  return Node.Statement_Node.Return_Expression;

               when others     =>
                  Show_Error (2000, Internal, "Get_Expression1");
            end case;

         when Expression_Node_Type  =>
            case Node.Expression_Node.Expression_Kind is
               when List_Item  =>
                  return Node.Expression_Node.Item_Expression;
               when others     =>
                  Show_Error (2000, Internal, "Get_Expression2");
            end case;

         when others =>
            Show_Error (2000, Internal, "Get_Expression3");

      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Expression");
         raise ;

   end Get_Expression;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Subprogram_Reference
   --
   --  <Description>
   --
   --    Returns the subprogram reference for the current Node
   --
   --=======================================================================
   function Get_Subprogram_Reference
       (Node : in Ref_Statement_Database) return Ref_Symbol_Table_Entry is
      -- Subprogram_Reference

   begin
      return Node.Statement_Node.Subprogram_Reference;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "get_subprogram_reference");
         raise ;

   end Get_Subprogram_Reference;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Alternatives_List
   --
   --  <Description>
   --
   --    Returns the CASE alternatives list for the current Node
   --
   --=======================================================================
   function Get_Alternatives_List
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Case_Alternatives_List

   begin
      return Node.Statement_Node.Case_Alternatives_List;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Alternatives_List");
         raise ;

   end Get_Alternatives_List;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Alternative_Choice
   --
   --  <Description>
   --
   --    Returns the CASE alternative choice(s) for the current Node
   --
   --=======================================================================
   function Get_Alternative_Choice
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Case_Alternative_Choice

   begin
      case Node.Statement_Node.Statement_Kind is
         when Case_Alternative_Part =>
            return Node.Statement_Node.Case_Alternative_Choice;
         when Exception_Alternative =>
            return Node.Statement_Node.Exception_Alternative_Choice;
         when others =>
            Show_Error (2000, Internal, "Get_Alternative_Choice 1");
      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Alternative_Choice 2");
         raise ;

   end Get_Alternative_Choice;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Alternative_Statements
   --
   --  <Description>
   --
   --    Returns the CASE alternative statements for the current Node
   --
   --=======================================================================
   function Get_Alternative_Statements
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Case_Alternative_Statements
   begin
      case Node.Statement_Node.Statement_Kind is
         when Case_Alternative_Part =>
            return Node.Statement_Node.Case_Alternative_Statements;
         when Exception_Alternative =>
            return Node.Statement_Node.Exception_Alternative_Statements;
         when others =>
            Show_Error (2000, Internal, "Get_Alternative_Statements 1");
      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Alternative_Statements 2");
         raise ;

   end Get_Alternative_Statements;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Next_Alternative
   --
   --  <Description>
   --
   --    Returns the next CASE alternative for the current Node
   --
   --=======================================================================
   function Get_Next_Alternative
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Next_Case_Alternative

   begin
      case Node.Statement_Node.Statement_Kind is
         when Case_Alternative_Part =>
            return Node.Statement_Node.Next_Case_Alternative;
         when Exception_Alternative =>
            return Node.Statement_Node.Next_Exception_Alternative;
         when others =>
            Show_Error (2000, Internal, "Set_Next_Alternative 1");
      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Next_Alternative");
         raise ;

   end Get_Next_Alternative;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Label_Entry
   --
   --  <Description>
   --
   --    Returns the label entry for the current Node
   --
   --=======================================================================
   function Get_Label_Entry
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Label_Entry

   begin
      return Node.Statement_Node.Label_Entry;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Label_Entry");
         raise ;

   end Get_Label_Entry;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Goto_Label
   --
   --  <Description>
   --
   --    Returns the goto label for the current Node
   --
   --=======================================================================
   function Get_Goto_Label
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Goto_Label

   begin
      return Node.Statement_Node.Goto_Label;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Goto_Label");
         raise ;

   end Get_Goto_Label;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Name
   --
   --  <Description>
   --
   --    Returns the name for the current Node
   --
   --=======================================================================
   function Get_Name
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Loop_Name
      -- Exit_Loop_Name
      -- Pragma_Name

   begin
      case Node.Statement_Node.Statement_Kind is
         when Loop_Statement   =>
            return Node.Statement_Node.Loop_Name;

         when Exit_Statement   =>
            return Node.Statement_Node.Exit_Loop_Name;

         when Pragma_Statement =>
            return Node.Statement_Node.Pragma_Name;

         when others =>
            Show_Error (2000, Internal, "Get_Name1");
      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Name");
         raise ;

   end Get_Name;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Block_Declarations
   --
   --  <Description>
   --
   --    Returns the Block Declarations for the current Node
   --
   --=======================================================================
   function Get_Block_Declarations
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Block_Declarations

   begin
      return Node.Statement_Node.Block_Declarations;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Block_Declarations");
         raise ;

   end Get_Block_Declarations;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Exception_Handler
   --
   --  <Description>
   --
   --    Returns the Block Declarations for the current Node
   --
   --=======================================================================
   function Get_Exception_Handler
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Exception_Handler

   begin
      return Node.Statement_Node.Block_Exceptions;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Exception_Handler");
         raise ;

   end Get_Exception_Handler;
   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Name_List
   --
   --  <Description>
   --
   --    Returns the head of the name list for the current Node
   --
   --=======================================================================
   function Get_Name_List
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Name_List
      -- Unit_Name_List

   begin
      case Node.Statement_Node.Statement_Kind is
         when Object_Declaration    =>
            return Node.Statement_Node.Name_List;

         when Exception_Declaration =>
            return Node.Statement_Node.Exception_Name_List;

         when With_Statement   | Use_Statement     =>
            return Node.Statement_Node.Unit_Name_List;

         when others =>
            Show_Error (2000, Internal, "Get_Name_List1");

      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Name_List");
         raise ;

   end Get_Name_List;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Parameter_List
   --
   --  <Description>
   --
   --    Returns the parameter list for the current Node
   --
   --=======================================================================
   function Get_Parameter_List
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Actual_Procedure_Parameters_List
      -- Lp_Items_List (for function calls)

   begin
      case Node.Node_Kind is
         when Statement_Node_Type   =>
            case Node.Statement_Node.Statement_Kind is
               when Procedure_Call  =>
                  return Node.Statement_Node.Actual_Procedure_Parameters_List;
               when others     =>
                  Show_Error (2000, Internal, "Get_Parameter_List1");
            end case;

         when Expression_Node_Type  =>
            case Node.Expression_Node.Expression_Kind is
               when Left_Parenthesis_Operator =>
                  return Node.Expression_Node.Lp_Items_List;
               when others     =>
                  Show_Error (2000, Internal, "Get_Parameter_List2");
            end case;

         when others =>
            Show_Error (2000, Internal, "Get_Parameter_List3");

      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Parameter_List");
         raise ;

   end Get_Parameter_List;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Argument_List
   --
   --  <Description>
   --
   --    Returns the argument list for the current Node
   --
   --=======================================================================
   function Get_Argument_List
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Pragma_Arg

   begin
      return Node.Statement_Node.Pragma_Arg;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Pragma_Arg");
         raise ;

   end Get_Argument_List;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Value
   --
   --  <Description>
   --
   --    Returns the value for the current Node
   --
   --=======================================================================
   function Get_Value
       (Node : in Ref_Statement_Database) return Ref_Data_Storage is
      -- Node_Value

   begin
      case Node.Expression_Node.Expression_Kind is
         when Left_Parenthesis_Operator  =>
            case Node.Expression_Node.Lp_Node_Kind is
               when Array_Reference =>
                  -- what are we returning a slice or a single value?
                  case Get_Expression_Kind
                      (Get_Expression (Get_List_Of_Items (Node))) is
                     when Range_Expression    =>
                        -- TBD must return a slice here.
                        null;
                     when Terminal_Node  | Constant_Node     =>
                        return
                           Fetch_Value
                               (Get_Array_Value
                                    (Node.Expression_Node.Node_Value),
                                Index (Get_Integer_Value
                                    (Get_Value
                                         (Get_Expression
                                              (Get_List_Of_Items (Node))))));
                     when others    =>
                        null;
                  end case;

               when others     =>
                  return Node.Expression_Node.Node_Value;
            end case;
         when others =>
            return Node.Expression_Node.Node_Value;
      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Value");
         raise ;

   end Get_Value;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Type
   --
   --  <Description>
   --
   --    Returns the type for the current Node
   --
   --=======================================================================
   function Get_Type
       (Node : in Ref_Statement_Database) return Ref_Symbol_Table_Entry is
      -- Node_Type

   begin
      return Node.Expression_Node.Node_Type;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Type");
         raise ;

   end Get_Type;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Is_A_Default_Expression
   --
   --  <Description>
   --
   --    Returns TRUE if the given NODE is a default expression
   --
   --=======================================================================
   function Is_A_Default_Expression
       (Node : in Ref_Statement_Database) return Boolean is
   begin
      return Node.Expression_Node.Is_Default_Expression;
   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Is_A_Default_Expression");
         raise ;
   end Is_A_Default_Expression;
   -- Is_Default_Expression

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Is_Type_Universal
   --
   --  <Description>
   --
   --    Returns TRUE if the given NODE is a universal type
   --
   --=======================================================================
   function Is_Type_Universal (Node : in Ref_Statement_Database) return Boolean
                              is
      -- Is_Node_Type_Universal

   begin
      return Node.Expression_Node.Is_Node_Type_Universal;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Is_Type_Universal");
         raise ;

   end Is_Type_Universal;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Symbolic_Name
   --
   --  <Description>
   --
   --    Returns the symbolic name for the current Node
   --
   --=======================================================================
   function Get_Symbolic_Name
       (Node : in Ref_Statement_Database) return Ref_String is
      -- Symbol_Name
      -- Character_Symbol
      -- String_Symbol
      -- Constant_Symbol

   begin
      case Node.Expression_Node.Expression_Kind is
         when Dot_Operator     =>
            return Get_Symbolic_Name (Get_Symbol_Table_Entry (Node));
         when Terminal_Node    =>
            return Node.Expression_Node.Symbol_Name;
         when String_Node =>
            return Node.Expression_Node.String_Symbol;
         when Character_Node   =>
            return
               Convert_To_Ref_String
                   (Character'IMAGE (Node.Expression_Node.Character_Symbol) (
                       2 .. 2));
         when Constant_Node    =>
            return Node.Expression_Node.Constant_Symbol;

         when Left_Parenthesis_Operator  =>
            return Get_Symbolic_Name (Node.Expression_Node.Lp_Object);

         when others =>
            Show_Error (2000, Internal, "Get_Symbolic_Name1");
      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Symbolic_Name");
         raise ;

   end Get_Symbolic_Name;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Node_Kind
   --
   --  <Description>
   --
   --    Returns the kind of the current Node
   --
   --=======================================================================
   function Get_Node_Kind
       (Node : in Ref_Statement_Database) return Expression_Enumeration is
      -- Dot_Node_Kind
      -- Lp_Node_Kind

   begin
      case Node.Expression_Node.Expression_Kind is
         when Dot_Operator     =>
            case Node.Expression_Node.Dot_Node_Kind is
               when Record_Reference     =>
                  return Record_Reference;
               when Dot_Notation_Variable_Reference     =>
                  return Dot_Notation_Variable_Reference;
               when Unknown    =>
                  return Unknown;
            end case;

         when Left_Parenthesis_Operator  =>
            case Node.Expression_Node.Lp_Node_Kind is
               when Array_Reference =>
                  return Array_Reference;
               when Function_Call   =>
                  return Function_Call;
               when Type_Conversion =>
                  return Type_Conversion;
               when Unknown    =>
                  return Unknown;
            end case;

         when others =>
            Show_Error (2000, Internal, "Get_Node_Kind1");
      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Node_Kind");
         raise ;

   end Get_Node_Kind;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Object
   --
   --  <Description>
   --
   --    Returns the object for the current Node
   --
   --=======================================================================
   function Get_Object
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Dot_Object
      -- Lp_Object

   begin
      case Node.Expression_Node.Expression_Kind is
         when Dot_Operator     =>
            return Node.Expression_Node.Dot_Object;

         when Left_Parenthesis_Operator  =>
            return Node.Expression_Node.Lp_Object;

         when Attribute_Tick   =>
            return Node.Expression_Node.Attribute_Prefix;

         when others =>
            Show_Error (2000, Internal, "Get_Object1");

      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Object");
         raise ;

   end Get_Object;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Dot_Component
   --
   --  <Description>
   --
   --    Returns the dot component for the current Node
   --
   --=======================================================================
   function Get_Dot_Component
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Dot_Component

   begin
      return Node.Expression_Node.Dot_Component;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Dot_Component");
         raise ;

   end Get_Dot_Component;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_List_Of_Items
   --
   --  <Description>
   --
   --    Returns the head of the list of items for the current Node
   --
   --=======================================================================
   function Get_List_Of_Items
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Lp_Items_List

   begin
      return Node.Expression_Node.Lp_Items_List;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_List_Of_Items");
         raise ;

   end Get_List_Of_Items;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Next_List_Item
   --
   --  <Description>
   --
   --    Returns the next List Item for the current Node
   --
   --=======================================================================
   function Get_Next_List_Item
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Next_List_Item

   begin
      return Node.Expression_Node.Next_List_Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Next_List_Item");
         raise ;

   end Get_Next_List_Item;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Aggregate_Choice
   --
   --  <Description>
   --
   --    Returns the aggregate choice for the current Node
   --
   --=======================================================================
   function Get_Aggregate_Choice
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Aggregate_Choice

   begin
      return Node.Expression_Node.Aggregate_Choice;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Aggregate_Choice");
         raise ;

   end Get_Aggregate_Choice;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Aggregate_Expression
   --
   --  <Description>
   --
   --    Returns the Aggregate Expression for the current Node
   --
   --=======================================================================
   function Get_Aggregate_Expression
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Aggregate_Expression

   begin
      return Node.Expression_Node.Aggregate_Expression;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Aggregate_Expression");
         raise ;

   end Get_Aggregate_Expression;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Lower_Bound
   --
   --  <Description>
   --
   --    Returns the Lower Bound for the current Node
   --
   --=======================================================================
   function Get_Lower_Bound
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Lower_Bound

   begin
      return Node.Expression_Node.Lower_Bound;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Lower_Bound");
         raise ;

   end Get_Lower_Bound;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Upper_Bound
   --
   --  <Description>
   --
   --    Returns the Upper Bound for the current Node
   --
   --=======================================================================
   function Get_Upper_Bound
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Upper_Bound

   begin
      return Node.Expression_Node.Upper_Bound;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Upper_Bound");
         raise ;

   end Get_Upper_Bound;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Attribute_Prefix
   --
   --  <Description>
   --
   --    Returns the Attribute Prefix Node for the current Node
   --
   --=======================================================================
   function Get_Attribute_Prefix
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Attribute_Prefix

   begin
      return Node.Expression_Node.Attribute_Prefix;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Attribute_Prefix");
         raise ;

   end Get_Attribute_Prefix;

   -- ----------------------------------------------------------------------
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Attribute
   --
   --  <Description>
   --
   --    Returns the Attribute Statement Database Node for the current Node
   --
   --=======================================================================
   function Get_Attribute
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Attribute

   begin
      return Node.Expression_Node.Attribute;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Attribute");
         raise ;

   end Get_Attribute;

   -- ----------------------------------------------------------------------

   --
   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Parent_Node
   --
   --  <Description>
   --
   --    Returns the Parent Statement Database Node for the current Node
   --
   --=======================================================================
   function Get_Parent_Node
       (Node : in Ref_Statement_Database) return Ref_Statement_Database is
      -- Parent_Node
   begin
      return Node.Parent_Node;
   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Parent_Node");
         raise ;
   end Get_Parent_Node;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Expression_Kind
   --
   --  <Description>
   --
   --    Returns the Expression_Kind Statement Database Node
   --    for the current Node
   --
   --=======================================================================
   function Get_Expression_Kind
       (Node : in Ref_Statement_Database) return Expression_Enumeration is
      -- Expression_Kind
   begin
      return Node.Expression_Node.Expression_Kind;
   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Expression_Kind");
         raise ;
   end Get_Expression_Kind;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Get_Reference_Count
   --
   --  <Description>
   --
   --  Returns the number of references made to Node
   --=======================================================================
   function Get_Reference_Count
       (Node : in Ref_Statement_Database) return Ace_Positive is
      -- Reference_Count
   begin
      return Node.Expression_Node.Reference_Count;
   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Get_Reference_Count");
         raise ;
   end Get_Reference_Count;

end Node_Get_Routines;
