--|$Log:	initsym.a,v $
--|Revision 8.0  91/01/17  10:45:47  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Subunit for Package - Symbol_Table_Routines
--
---------------------------------------------------------------
--
--
-- Abstract
--
--  The following routine is a separate unit for Symbol_Table_Routines
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------

with Statement_Database;
use Statement_Database;

with Node_Set_Routines;
use Node_Set_Routines;

with Node_Create_Routines;
use Node_Create_Routines;

with Derived_Subprogram_Support;

with Ace;

--==========================================================================
--  <Subprogram Name>
--
--    Initialize_Symbol_Table
--
--  <Description>
--
--    The symbol table is initialized with entries for predefined types
--    and subprograms.
--
--==========================================================================
separate (Symbol_Table_Routines)

procedure Initialize_Symbol_Table is

   Current_Entry : Ref_Symbol_Table_Entry;

begin

   ----------
   -- INTEGER
   ----------
   begin
      Integer_Entry :=
         Create_Symbol_Table_Entry (Convert_To_Ref_String ("integer"),
                                    Type_Entry);
      Set_Type_To_Integer_Type (Integer_Entry);
      Set_Lower_Bound (Integer_Entry, Ace_Integer'FIRST);
      Set_Upper_Bound (Integer_Entry, Ace_Integer'LAST);
      Add_Entry_To_Symbol_Table (Ace.Main_Thread_Context, Integer_Entry);

      Current_Entry :=
         Create_Symbol_Table_Entry (Convert_To_Ref_String ("natural"),
                                    Subtype_Entry);
      Set_Entry_Base_Type (Current_Entry, Integer_Entry);
      Add_Entry_To_Symbol_Table (Ace.Main_Thread_Context, Current_Entry);

      Current_Entry :=
         Create_Symbol_Table_Entry (Convert_To_Ref_String ("positive"),
                                    Subtype_Entry);
      Set_Entry_Base_Type (Current_Entry, Integer_Entry);
      Add_Entry_To_Symbol_Table (Ace.Main_Thread_Context, Current_Entry);
   end;

   ----------
   -- STRING
   ----------
   String_Entry :=
      Create_Symbol_Table_Entry (Convert_To_Ref_String ("string"), Type_Entry);
   Set_Type_To_Real_Type (String_Entry); -- TBD this was Array, but
                                         -- was changed while real arrays
                                         -- were being implemented.  After
                                         -- arrays are completed String should
                                         -- be implemented correctly, i.e., as
                                         -- an array.
   Add_Entry_To_Symbol_Table (Ace.Main_Thread_Context, String_Entry);

   ----------
   -- CHARACTER
   ----------
   Character_Entry :=
      Create_Symbol_Table_Entry (Convert_To_Ref_String ("character"),
                                 Type_Entry);
   Set_Type_To_Integer_Type (Character_Entry);
   Add_Entry_To_Symbol_Table (Ace.Main_Thread_Context, Character_Entry);

   ----------
   -- BOOLEAN
   ----------
   declare
      False_Lit          : Ref_Symbol_Table_Entry :=
         Create_Symbol_Table_Entry (Convert_To_Ref_String ("false"),
                                    Enumeration_Literal_Entry);
      True_Lit           : Ref_Symbol_Table_Entry :=
         Create_Symbol_Table_Entry (Convert_To_Ref_String ("true"),
                                    Enumeration_Literal_Entry);

      New_Function_Entry : Ref_Symbol_Table_Entry;
      Return_Stmt        : Ref_Statement_Database;
      Enum_Lit_Expr      : Ref_Statement_Database;

   begin
      Boolean_Entry :=
         Create_Symbol_Table_Entry (Convert_To_Ref_String ("boolean"),
                                    Type_Entry);
      Set_Type_To_Enumeration_Type (Boolean_Entry);

      Set_Literal_Value (False_Lit, 0);
      Set_Literal_Value (True_Lit, 1);
      Set_Type (False_Lit, Boolean_Entry);
      Set_Type (True_Lit, Boolean_Entry);
      Set_Successor (False_Lit, True_Lit);
      Set_Predecessor (True_Lit, False_Lit);

      Set_Literal_Count (Boolean_Entry, 2);
      Set_First_Literal (Boolean_Entry, False_Lit);
      Set_Last_Literal (Boolean_Entry, True_Lit);

      -----
      -- add also as parameterless function call symtbl entry
      --

      --== for FALSE
      --
      -- create and init new nodes
      New_Function_Entry :=
         Create_Symbol_Table_Entry (Get_Symbolic_Name (False_Lit),
                                    Function_Entry);
      Return_Stmt := Create_Node (Return_Statement);
      Enum_Lit_Expr := Create_Node (Terminal_Node);
      -- add items to new function_entry symtbl entry
      Set_Enumeration_Literal_Entry (New_Function_Entry, False_Lit);
      Set_Return_Type (New_Function_Entry, Boolean_Entry);
      --
      -- make reference to parameterless function call symtbl entry
      Set_Function_Entry (False_Lit, New_Function_Entry);
      --
      -- == create stmt_db "return_stmt" node
      -- create return_expr_node
      Set_Symbolic_Name (Enum_Lit_Expr, Get_Symbolic_Name (False_Lit));
      Set_Symbol_Table_Entry (Enum_Lit_Expr, False_Lit);
      Set_Type (Enum_Lit_Expr, Boolean_Entry);
      -- assign return_expr_node to return_stmt_node
      Set_Expression (Return_Stmt, Enum_Lit_Expr);
      -- this should be ok
      Set_Subprogram_Reference (Return_Stmt, New_Function_Entry);
      -- record return_stmt_node in symtbl entry
      Set_Head_Of_Statements_List (New_Function_Entry, Return_Stmt);

      -- add function entry to list of subpgms to derive when needed
      Derived_Subprogram_Support.Add_Subprogram_Entry_To_List
          (New_Function_Entry, Boolean_Entry);

      --== for TRUE
      --
      -- create and init new nodes
      New_Function_Entry :=
         Create_Symbol_Table_Entry (Get_Symbolic_Name (True_Lit),
                                    Function_Entry);
      Return_Stmt := Create_Node (Return_Statement);
      Enum_Lit_Expr := Create_Node (Terminal_Node);
      -- add items to new function_entry symtbl entry
      Set_Enumeration_Literal_Entry (New_Function_Entry, True_Lit);
      Set_Return_Type (New_Function_Entry, Boolean_Entry);
      --
      -- make reference to parameterless function call symtbl entry
      Set_Function_Entry (True_Lit, New_Function_Entry);
      --
      -- == create stmt_db "return_stmt" node
      -- create return_expr_node
      Set_Symbolic_Name (Enum_Lit_Expr, Get_Symbolic_Name (True_Lit));
      Set_Symbol_Table_Entry (Enum_Lit_Expr, True_Lit);
      Set_Type (Enum_Lit_Expr, Boolean_Entry);
      -- assign return_expr_node to return_stmt_node
      Set_Expression (Return_Stmt, Enum_Lit_Expr);
      -- this should be ok
      Set_Subprogram_Reference (Return_Stmt, New_Function_Entry);
      -- record return_stmt_node in symtbl entry
      Set_Head_Of_Statements_List (New_Function_Entry, Return_Stmt);

      -- add function entry to list of subpgms to derive when needed
      Derived_Subprogram_Support.Add_Subprogram_Entry_To_List
          (New_Function_Entry, Boolean_Entry);

      -----

      Add_Entry_To_Symbol_Table (Ace.Main_Thread_Context, Boolean_Entry);
   end;

   ----------
   -- Predefined Exceptions
   -- Constraint_Error, Numeric_Error, Program_Error, Storage_Error,
   -- and Tasking_Error
   ----------

   declare
      Constraint_Error_Entry : Ref_Symbol_Table_Entry :=
         Create_Symbol_Table_Entry
             (Convert_To_Ref_String ("constraint_error"), Exception_Entry);
      Numeric_Error_Entry    : Ref_Symbol_Table_Entry :=
         Create_Symbol_Table_Entry (Convert_To_Ref_String ("numeric_error"),
                                    Exception_Entry);
      Program_Error_Entry    : Ref_Symbol_Table_Entry :=
         Create_Symbol_Table_Entry (Convert_To_Ref_String ("program_error"),
                                    Exception_Entry);
      Storage_Error_Entry    : Ref_Symbol_Table_Entry :=
         Create_Symbol_Table_Entry (Convert_To_Ref_String ("storage_error"),
                                    Exception_Entry);
      Tasking_Error_Entry    : Ref_Symbol_Table_Entry :=
         Create_Symbol_Table_Entry (Convert_To_Ref_String ("tasking_error"),
                                    Exception_Entry);
   begin
      Add_Entry_To_Symbol_Table (Ace.Main_Thread_Context,
                                 Constraint_Error_Entry);
      Add_Entry_To_Symbol_Table (Ace.Main_Thread_Context, Numeric_Error_Entry);
      Add_Entry_To_Symbol_Table (Ace.Main_Thread_Context, Program_Error_Entry);
      Add_Entry_To_Symbol_Table (Ace.Main_Thread_Context, Storage_Error_Entry);
      Add_Entry_To_Symbol_Table (Ace.Main_Thread_Context, Tasking_Error_Entry);
   end;

   ----------
   -- ACE Labels
   -- used for the following EXIT stmts ...
   --    EXIT Ace;
   --    EXIT Ace_Main;
   --    EXIT Ace_Level;
   ----------
   Current_Entry := Create_Symbol_Table_Entry (Convert_To_Ref_String ("ace"),
                                               Label_Entry);
   Add_Entry_To_Symbol_Table (Ace.Main_Thread_Context, Current_Entry);
   Current_Entry :=
      Create_Symbol_Table_Entry (Convert_To_Ref_String ("ace_level"),
                                 Label_Entry);
   Add_Entry_To_Symbol_Table (Ace.Main_Thread_Context, Current_Entry);
   Current_Entry :=
      Create_Symbol_Table_Entry (Convert_To_Ref_String ("ace_main"),
                                 Label_Entry);
   Add_Entry_To_Symbol_Table (Ace.Main_Thread_Context, Current_Entry);

   ----------
   -- EVAL
   ----------
   --   procedure Eval (Filename : in String);
   --      pragma Builtin (Eval, 1);

   declare
      Parameter : Ref_Symbol_Table_Entry;
   begin
      Parameter :=
         Create_Symbol_Table_Entry (Convert_To_Ref_String ("filename"),
                                    Formal_Param_Entry);
      Set_Type (Parameter, String_Entry);
      Set_Level (Parameter, 1);

      Current_Entry :=
         Create_Symbol_Table_Entry (Convert_To_Ref_String ("eval"),
                                    Procedure_Entry);
      Set_Head_Of_Parameters_List (Current_Entry, Parameter);
      Set_Tail_Of_Parameters_List (Current_Entry, Parameter);
      Set_Parameter_Count (Current_Entry, 1);
      Set_As_Built_In_Subprogram (Current_Entry, 1);
      Add_Entry_To_Symbol_Table (Ace.Main_Thread_Context, Current_Entry);
   end;

   ----------
   -- Interpret_File
   ----------
   --   procedure Interpret_File (File_name : in String);
   --      pragma Builtin (Interpret_File, 506);

   declare
      Parameter : Ref_Symbol_Table_Entry;
   begin
      Parameter :=
         Create_Symbol_Table_Entry (Convert_To_Ref_String ("file_name"),
                                    Formal_Param_Entry);

      Set_Type (Parameter, String_Entry);
      Set_Level (Parameter, 1);

      Current_Entry :=
         Create_Symbol_Table_Entry (Convert_To_Ref_String ("interpret_file"),
                                    Procedure_Entry);
      Set_Head_Of_Parameters_List (Current_Entry, Parameter);
      Set_Tail_Of_Parameters_List (Current_Entry, Parameter);
      Set_Parameter_Count (Current_Entry, 1);
      Set_As_Built_In_Subprogram (Current_Entry, 506);
      Add_Entry_To_Symbol_Table (Ace.Main_Thread_Context, Current_Entry);
   end;

   ----------
   -- Console
   ----------
   --   procedure Console (Message : in String);
   --      pragma Builtin (Console, 1037);

   declare
      Parameter : Ref_Symbol_Table_Entry;
   begin
      Parameter :=
         Create_Symbol_Table_Entry (Convert_To_Ref_String ("message"),
                                    Formal_Param_Entry);
      Set_Type (Parameter, String_Entry);
      Set_Level (Parameter, 1);

      Current_Entry :=
         Create_Symbol_Table_Entry (Convert_To_Ref_String ("console"),
                                    Procedure_Entry);
      Set_Head_Of_Parameters_List (Current_Entry, Parameter);
      Set_Tail_Of_Parameters_List (Current_Entry, Parameter);
      Set_Parameter_Count (Current_Entry, 1);
      Set_As_Built_In_Subprogram (Current_Entry, 1037);
      Add_Entry_To_Symbol_Table (Ace.Main_Thread_Context, Current_Entry);
   end;

end Initialize_Symbol_Table;
