--|$Log:	lexact.a,v $
--|Revision 8.0  91/01/17  10:45:50  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Lexacts
--
---------------------------------------------------------------
--
--
-- Abstract
--
--   This package provides the interface from the scanner to
--   the parser (with the exception of the token returned from
--   the scanner).  Routines included deal with the textual
--   representation of tokens, and dealing with the CR which
--   in ACE is a special token (it's meaning changes while the
--   user is typing, and is used to help make ACE interactive).
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
with Parserdefs;
use Parserdefs;
package body Lexacts is

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Display
   --
   --  <Description>
   --
   --    This routine is used to convert the internal representation of
   --    tokens into the string equivalent.
   --
   --==========================================================================

   function Display (Tok : Integer) return String is
   begin
      case Tok is
         when Abort_Tk    =>
            return ("abort");
         when Abs_Tk =>
            return ("abs");
         when Accept_Tk   =>
            return ("accept");
         when Access_Tk   =>
            return ("assess");
         when All_Tk =>
            return ("all");
         when And_Tk =>
            return ("and");
         when Array_Tk    =>
            return ("array");
         when At_Tk  =>
            return ("at");
         when Begin_Tk    =>
            return ("begin");
         when Body_Tk     =>
            return ("body");
         when Case_Tk     =>
            return ("case");
         when Constant_Tk =>
            return ("constant");
         when Declare_Tk  =>
            return ("declare");
         when Delay_Tk    =>
            return ("delay");
         when Delta_Tk    =>
            return ("delta");
         when Digits_Tk   =>
            return ("digits");
         when Do_Tk  =>
            return ("do");
         when Else_Tk     =>
            return ("else");
         when Elsif_Tk    =>
            return ("elsif");
         when End_Tk =>
            return ("end");
         when Entry_Tk    =>
            return ("entry");
         when Exception_Tk     =>
            return ("exception");
         when Exit_Tk     =>
            return ("exit");
         when For_Tk =>
            return ("for");
         when Function_Tk =>
            return ("function");
         when Generic_Tk  =>
            return ("generic");
         when Goto_Tk     =>
            return ("goto");
         when If_Tk  =>
            return ("if");
         when In_Tk  =>
            return ("in");
         when Is_Tk  =>
            return ("is");
         when Limited_Tk  =>
            return ("limited");
         when Loop_Tk     =>
            return ("loop");
         when Mod_Tk =>
            return ("mod");
         when New_Tk =>
            return ("new");
         when Not_Tk =>
            return ("not");
         when Null_Tk     =>
            return ("null");
         when Of_Tk  =>
            return ("of");
         when Or_Tk  =>
            return ("or");
         when Others_Tk   =>
            return ("others");
         when Out_Tk =>
            return ("out");
         when Package_Tk  =>
            return ("package");
         when Pragma_Tk   =>
            return ("pragma");
         when Private_Tk  =>
            return ("private");
         when Procedure_Tk     =>
            return ("procedure");
         when Raise_Tk    =>
            return ("raise");
         when Range_Tk    =>
            return ("range");
         when Record_Tk   =>
            return ("record");
         when Rem_Tk =>
            return ("rem");
         when Renames_Tk  =>
            return ("renames");
         when Return_Tk   =>
            return ("return");
         when Reverse_Tk  =>
            return ("reverse");
         when Select_Tk   =>
            return ("select");
         when Separate_Tk =>
            return ("separate");
         when Subtype_Tk  =>
            return ("subtype");
         when Task_Tk     =>
            return ("task");
         when Terminate_Tk     =>
            return ("terminate");
         when Then_Tk     =>
            return ("then");
         when Type_Tk     =>
            return ("type");
         when Use_Tk =>
            return ("use");
         when When_Tk     =>
            return ("when");
         when While_Tk    =>
            return ("while");
         when With_Tk     =>
            return ("with");
         when Xor_Tk =>
            return ("xor");
         when Arrow_Tk    =>
            return ("arrow");
         when Dbldot_Tk   =>
            return ("..");
         when Exp_Tk =>
            return ("**");
         when Assign_Tk   =>
            return (":=");
         when Noteql_Tk   =>
            return ("/=");
         when Gteql_Tk    =>
            return (">=");
         when Lteq_Tk     =>
            return ("<=");
         when L_Lbl_Tk    =>
            return ("<<");
         when R_Lbl_Tk    =>
            return (">>");
         when Box_Tk =>
            return ("<>");
         when Character'POS ('&')   =>
            return ("&");
         when Character'POS ('(')   =>
            return ("(");
         when Character'POS (')')   =>
            return (")");
         when Character'POS ('*')   =>
            return ("*");
         when Character'POS ('+')   =>
            return ("+");
         when Character'POS (',')   =>
            return (",");
         when Character'POS ('-')   =>
            return ("-");
         when Character'POS ('.')   =>
            return (".");
         when Character'POS ('/')   =>
            return ("/");
         when Character'POS (':')   =>
            return (":");
         when Character'POS (';')   =>
            return (";");
         when Character'POS ('<')   =>
            return ("<");
         when Character'POS ('=')   =>
            return ("=");
         when Character'POS ('>')   =>
            return (">");
         when Character'POS ('|')   =>
            return ("|");
         when Character'POS (''')   =>
            return ("'");
         when Identifier  =>
            return ("Identifier");
         when Numeric_Literal  =>
            return ("Numeric-Literal");
         when String_Literal   =>
            return ("String-Literal");
         when Character_Literal     =>
            return ("Character-Literal");
         when others =>
            return ("INVALID_TOKEN");
      end case;
   end Display;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Pop_Eol
   --
   --  <Description>
   --
   --    This routine is used to keep track of syntatic nesting
   --    of constructs.  So that ACE can tell when a construct is
   --    finished. This routine is called everytime
   --    the user exits a construct.
   --    When the stack is empty and the use hit a newline,
   --    then the constuct is over.
   --
   --==========================================================================
   procedure Pop_Eol is
   begin
      Eol_Flag := Eol_Flag - 1;
   end Pop_Eol;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Push_Eol
   --
   --  <Description>
   --
   --    This routine is used to keep track of syntatic nesting
   --    of constructs.  So that ACE can tell when a construct is
   --    finished. This routine is called everytime
   --    the user enters a construct.
   --    When the stack is empty and the use hit a newline,
   --    then the constuct is over.
   --
   --==========================================================================
   procedure Push_Eol is
   begin
      Eol_Flag := Eol_Flag + 1;
   end Push_Eol;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Reset_Eol
   --
   --  <Description>
   --
   --    This routine is used to initialize the stack used in the previous
   --    two procedures.
   --==========================================================================
   procedure Reset_Eol is
   begin
      Eol_Flag := 0;
   end Reset_Eol;

begin
   Reset_Eol;
end Lexacts;
