--|$Log:	literl.a,v $
--|Revision 8.0  91/01/17  10:45:51  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Literals
--
--
-- Abstract
--
--   This package manages the literal pool.  Only one copy of a literal
-- is stored.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
package body Literals is

   Literal_Tree : Ref_String := Create;

   function "<" (Lhs, Rhs : in String_Acc) return boolean is
     -- Telesoft returns the wrong lexigraphical ordering for
     -- "<" of access types of Stings.  Telesoft compares only the 
     -- the number of characters that is common to both strings,
     -- e.g., "AA" < "AAA" returns false, because "AA" is not less than "AA."
     Lhs1 : constant String := Lhs.all;
     Rhs1 : constant String := Rhs.all;
   begin
     return Lhs1 < Rhs1;
   end;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Convert_To_Ref_String
   --
   --  <Description>
   --
   --    Adds a literal to the literal pool (will not enter duplicate
   -- strings, but return the position of the existing string).
   --==========================================================================
   function Convert_To_Ref_String (Str : String) return Ref_String is
      Position : Ref_String;
      Temp     : String_Acc := New String'(Str);
   begin
      -- Insert a node in the B Tree and return the position
      Insertnode (Temp, Literal_Tree, Position);
      return Position;
   end Convert_To_Ref_String;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Convert_To_String
   --
   --  <Description>
   --
   --    Given a position will return the literal string at that position
   --==========================================================================
   function Convert_To_String (Position : Ref_String) return String is
   begin
      -- return the string associated with the position.
      return Get_Info (Position).all;
   end Convert_To_String;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Are_Symbolic_Names_Equal
   --
   --  <Description>
   --
   --    return true iff Name1 and Name2 are equal
   --==========================================================================
   function Are_Symbolic_Names_Equal (Name1, Name2 : Ref_String) return Boolean
                                     is
   begin
      return (Name1 = Name2);
   end Are_Symbolic_Names_Equal;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Lower
   --
   --  <Description>
   --
   --    return a string the same as Str, with but all lower case letters
   --==========================================================================
   function Lower (Str : in String) return String is
      Str1 : String (Str'FIRST .. Str'LAST) := Str;
      I    : Positive;
   begin
      I := Str1'FIRST;
      while I <= Str1'LAST loop
         if Str1 (I) >= 'A' and Str1 (I) <= 'Z' then
            Str1 (I) := Character'VAL (Character'POS (Str1 (I)) +
                                       Character'POS ('a') -
                                       Character'POS ('A'));
         end if;
         I := I + 1;
      end loop;
      return Str1;
   end Lower;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Upper
   --
   --  <Description>
   --
   --    return a string the same as Str, with but all upper case letters
   --==========================================================================
   function Upper (Str : in String) return String is
      Str1 : String (Str'FIRST .. Str'LAST) := Str;
      I    : Positive;
   begin
      I := Str1'FIRST;
      while I <= Str1'LAST loop
         if Str1 (I) >= 'a' and Str1 (I) <= 'z' then
            Str1 (I) := Character'VAL (Character'POS (Str1 (I)) -
                                       Character'POS ('a') +
                                       Character'POS ('A'));
         end if;
         I := I + 1;
      end loop;
      return Str1;
   end Upper;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Ada_Id
   --
   --  <Description>
   --
   --    Ada_Id will change a string to follow the Ada rules for
   -- case (i.e., case insensitive).
   --==========================================================================
   procedure Ada_Id (Id : in out Ref_String) is
   begin
      Id := Convert_To_Ref_String (Lower (Convert_To_String (Id)));
   end Ada_Id;

end Literals;
