--|$Log:	misc.a,v $
--|Revision 8.0  91/01/17  10:45:52  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Misc
--
---------------------------------------------------------------
--
--
-- Abstract
--
--   This routine handles miscellaneous tasks (such as changing the
--   users prompt from "ACE> " to "    |" when a construct is being
--   continued on more than one line).
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
with Ace_Text_Io;

with User_Pkg;

with Lexacts;

with Ace_Universal_Types;

with Ace;

with System_Dependent_Routines;
use System_Dependent_Routines;

package body Misc is

   Blanks : constant String (1 .. 256) := (others => ' ');

   Current_Prompt : String (1 .. 256);

   Continuation_Prompt_Str : String (1 .. 256) := (1        => ' ',
                                                   2        => ' ',
                                                   3        => ' ',
                                                   4        => '|',
                                                   5        => ' ',
                                                   6 .. 256 => ' ');

   Main_Prompt_Str : String (1 .. 256) := (1        => 'A',
                                           2        => 'C',
                                           3        => 'E',
                                           4        => '>',
                                           5        => ' ',
                                           6 .. 256 => ' ');

   Continuation_Prompt_Length, Main_Prompt_Length : Natural := 5;
   Current_Prompt_Length : Natural := 0;

   Prompt_User : Boolean := False;
   Exit_On_First_Error_Flag : Boolean := False;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Prompt_Off
   --
   --  <Description>
   --
   --    Set a flag so the that the prompt will not be given.
   --
   --==========================================================================
   procedure Set_Prompt_Off is
   begin
      Prompt_User := False;
   end Set_Prompt_Off;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Prompt_On
   --
   --  <Description>
   --
   --    Set a flag so the that the prompt will be send to the user's screen.
   --
   --==========================================================================
   procedure Set_Prompt_On is
   begin
      Prompt_User := True;
   end Set_Prompt_On;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    prompt
   --
   --  <Description>
   --
   --    Prompts the users with the given parameter.  This routine also
   --    depends on Ace.Ace_Level (if non-zero then the level will be
   --    displayed in parenthesis).
   --
   --==========================================================================
   procedure Prompt (Str : String) is
   begin
      if Abort_Check then
         Ace_Text_Io.Put_Line (Ace_Text_Io.Current_Ace_Output, "Aborted");
         -- The abort has left a new_line in the input stream that we
         -- should throw away if we are not reading from standard input
         if Ace_Text_Io."/=" (Ace_Text_Io.Current_Ace_Input,
                              Ace_Text_Io.Standard_Ace_Input) then
            Ace_Text_Io.Skip_Line;
         end if;
         raise Ace_Universal_Types.Parser_Error;
      end if;
      if Prompt_User then
         if Ace.Ace_Level = 0 then
            Current_Prompt_Length := Str'Length;
            Current_Prompt (1 .. Current_Prompt_Length) :=  Str;
            Ace_Text_Io.Put (Ace_Text_Io.Current_Ace_Output, Str);
         else
            declare
              Temp_Str : constant String := "(" &
                 Natural'IMAGE (Ace.Ace_Level) (
                    2 .. Natural'IMAGE (Ace.Ace_Level)'LAST) & ")" & Str;
            begin
              Current_Prompt_Length := Temp_Str'Length;
              Current_Prompt (1 .. Current_Prompt_Length) :=  Temp_Str;
              Ace_Text_Io.Put
                  (Ace_Text_Io.Current_Ace_Output, Temp_Str);
            end;
         end if;
      end if;
   end Prompt;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Prompt
   -- 
   --    Last Mod:    Thu May 24 11:56:01 1990
   --    Author:      William P Loftus <wpl@huey>
   -- 
   --  <Description> 
   --    returns a string that represents the current prompt
   -- 
   --  --------------------------------------------------------------------
   function Prompt return String is
   begin
     return Current_Prompt (1 .. Current_Prompt_Length);
   end;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Main_Prompt
   --
   --  <Description>
   --
   --    Returns a string containing the Main prompt.
   --
   --==========================================================================
   function Main_Prompt return String is
   begin
      return Main_Prompt_Str (1 .. Main_Prompt_Length);
   end Main_Prompt;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Main_Prompt
   --
   --  <Description>
   --
   --    Sets the main user prompt.
   --
   --==========================================================================
   procedure Set_Main_Prompt (Str : String) is
   begin
      Main_Prompt_Length := Str'LENGTH;
      Main_Prompt_Str := Str & Blanks (Main_Prompt_Length + 1 .. Blanks'LAST);
   end Set_Main_Prompt;

   --==========================================================================
   --+--  <Subprogram Name>
   --
   --    Continuation_Prompt
   --
   --  <Description>
   --
   --    Returns a string containing the continuation prompt.
   --
   --==========================================================================
   function Continuation_Prompt return String is
   begin
      return Continuation_Prompt_Str (1 .. Continuation_Prompt_Length);
   end Continuation_Prompt;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Continuation_Prompt
   --
   --  <Description>
   --
   --    Sets the continuation prompt.
   --
   --==========================================================================
   procedure Set_Continuation_Prompt (Str : String) is
   begin
      Continuation_Prompt_Length := Str'LENGTH;
      Continuation_Prompt_Str    := Str &
                                 Blanks (Continuation_Prompt_Length + 1 ..
                                         Blanks'LAST);
   end Set_Continuation_Prompt;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Set_Exit_On_First_Error
   --
   --  <Description>
   --
   --   Set a flag that if true will cause ACE to exit to the host O/S
   --   when the first error is detected.
   --
   --==========================================================================
   procedure Set_Exit_On_First_Error (Switch : Boolean) is
   begin
      Exit_On_First_Error_Flag := Switch;
   end Set_Exit_On_First_Error;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Exit_On_First_Error
   --
   --  <Description>
   --
   --   If true the first error should cause ACE to exit to the host O/S.
   --
   --==========================================================================
   function Exit_On_First_Error return Boolean is
   begin
      return Exit_On_First_Error_Flag;
   end Exit_On_First_Error;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Reset_Input
   --
   --  <Description>
   --
   --   Flushes out the input left in the user's buffer and resets
   --   the parser and lexical analyzer.
   --
   --==========================================================================
   procedure Reset_Input is
      use Ace_Text_Io;
   begin
      Set_Ace_Input (Standard_Ace_Input);
      Set_Ace_Output (Standard_Ace_Output);
      Set_Input (Standard_Input);
      Set_Output (Standard_Output);
      Flush;
      User_Pkg.Reset_Buffer;
      Lexacts.Reset_Eol;
   end Reset_Input;

end Misc;
