--|$Log:	ooe.a,v $
--|Revision 8.0  91/01/17  10:45:55  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Ooe
--
---------------------------------------------------------------
--
--
-- Abstract
--  The Object Oriented Environment (Ooe) primatives (that are not provide by
--  Ada are located in this package).
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
with User_Pkg;
use User_Pkg;

with Unchecked_Conversion;

with System_Dependent_Routines;
use System_Dependent_Routines;

with Ace_Text_Io;

pragma ELABORATE (Ace_Text_Io);
package body Ooe is

   --==========================================================================
   --  <Subprogram Name>
   --
   --    File_Entries
   --
   --  <Description>
   --
   --    This function converts Ace_Text_IO.file_type to Text_Io.file_type.
   --
   --  <Called By>
   --
   --    Ace_Text_Io body
   --
   --==========================================================================
   function File_Entries (File : File_Type) return Text_Io.File_Type is
   begin
      if File = File_Type'LAST then
         return Text_Io.Standard_Input;
      elsif File = File_Type'LAST - 1 then
         return Text_Io.Standard_Output;
      else
         return Files (File);
      end if;
   end File_Entries;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Host
   --
   --  <Description>
   --
   --    This procedure executes a string on the host operating system.
   --    This is the system independent part (it does not need to be recoded
   --    for porting).
   --
   --  <Called By>
   --
   --    Various
   --
   --  <Exceptions Handled>
   --
   --    All.
   --
   --==========================================================================
   procedure Host (Command : String) is
   begin
      System (Command);
   exception
      when others    => -- don't do it
         null;
   end Host;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Eval
   --
   --  <Description>
   --
   --    This procedure set the input of ACE to a particular file.
   --
   --  <Called By>
   --
   --    Various
   --
   --==========================================================================
   procedure Eval (File : String) is
      Tmp : Ace_Text_Io.File_Type;
   begin
      Ace_Text_Io.Open (Tmp, Ace_Text_Io.In_File, File, "");
      Ace_Text_Io.Set_Ace_Input (Tmp);
   end Eval;

end Ooe;
