-----------------------------------------------------------------------------
--									    
--			 A LALR(1) grammar for ANSI Ada		   
--									     
--			 Adapted for YACC (UNIX) Inputs 		     
--									     
--                                                                           
--                             Herman Fischer                                
--                           Litton Data Systems                             
--                       8000 Woodley Ave., ms 44-30                         
--                              Van Nuys, CA                                 
--                                                                           
--                              818/902-5139                                 
--                           HFischer@eclb.arpa                              
--                        {cepu,trwrb}!litvax!fischer                        
--                                                                           
--                             March 26, 1984                                
--									     
--                   A Contribution to the Public Domain                     
--                                   for                                     
--            Research, Development, and Training Purposes Only              
--                                                                           
--       Any Corrections or Problems are to be Reported to the Author        
--                                                                           
--                                                                           
--                                                                           
--                              adapted from                                 
--                              the grammar                                  
--                                  by:                                      
--									     
--            Gerry Fisher                         Philippe Charles          
--                                                                           
--    Computer Sciences Corporation     &             Ada  Project           
--         4045 Hancock Street                     New York University       
--         San Diego, CA 92121                      251 Mercer Street        
--                                               New York, New York 10012    
--									     
--									     
--    This grammar is organized in the same order as the syntax summary     
-- in appendix E of the ANSI Ada Reference Manual.   All reserved words     
-- are	 written  in  upper  case  letters.    The  lexical  categories     
-- numeric_literal, string_literal, etc, are viewed as terminals.   The     
-- rules  for  pragmas as  stated in  chapter 2,  section 8,  have been     
-- incorporated in the grammar.   Comments are included wherever we had     
-- to deviate from the syntax given in appendix E. Different symbols	    
-- used here (to comply with yacc requirements) are of note:		    
--	{,something}	is denoted ...something..			     
--	{something}	is denoted ..something..			     
--	[something]	is denoted .something.				    
-- Constructs involving                                                     
-- meta brackets, e.g., ...identifier.. are represented by a nonterminal    
-- formed  by  concatenating  the  construct symbols ( as ...identifier..   
-- in the example)   for  which  the  rules are given at the end.  When     
-- reading  this  grammar,  it	is  important to  note that all symbols     
-- appearing  in the  rules are  separated by  one or  more  blanks.  A     
-- string  such  as   'identifier_type_mark   is  actually  a   single      
-- nonterminal symbol defined at the end of the rules.
--                                                                          
--     This grammar is presented here in a form suitable for input to a     
-- yacc parser generator.  It has  been  processed  by the Bell System      
-- III lex/yacc combination, and tested against over 400 ACVC tests.        
--									    
-----------------------------------------------------------------------------
--%terminals -----------------------------------------------------------     

%token ABORT_TK ABS_TK ACCEPT_TK  ACCESS_TK ALL_TK  AND_TK ARRAY_TK
%token AT_TK BEGIN_TK  BODY_TK CASE_TK  CONSTANT_TK
%token DECLARE_TK DELAY_TK DELTA_TK DIGITS_TK DO_TK ELSE_TK ELSIF_TK
%token END_TK  ENTRY_TK EXCEPTION_TK EXIT_TK FOR_TK
%token FUNCTION_TK GENERIC_TK  GOTO_TK IF_TK IN_TK  IS_TK LIMITED_TK
%token LOOP_TK MOD_TK  NEW_TK NOT_TK  NULL_TK OF_TK OR_TK
%token OTHERS_TK OUT_TK  PACKAGE_TK PRAGMA_TK  PRIVATE_TK
%token PROCEDURE_TK  RAISE_TK  RANGE_TK RECORD_TK REM_TK
%token RENAMES_TK RETURN_TK REVERSE_TK  SELECT_TK  SEPARATE_TK
%token SUBTYPE_TK TASK_TK TERMINATE_TK THEN_TK
%token TYPE_TK USE_TK WHEN_TK WHILE_TK WITH_TK XOR_TK

%token identifier numeric_literal string_literal character_literal
%token ARROW_TK DBLDOT_TK EXP_TK ASSIGN_TK NOTEQL_TK GTEQL_TK LTEQ_TK
%token L_LBL_TK R_LBL_TK BOX_TK

%with Statement_Database;

{ subtype Yystype is Statement_Database.Ref_Statement_Database; }

%start strt

%%

-- ACE specific rules

eol_push : { Push_Eol; } ;

eol_pop : { Pop_Eol; } ;

save_comment : { NQ_Pkg_Help(Get_Help); } ;

strt : compilation  { Lval := $1; } ;

-- 2.8 

prag	:  eol_push PRAGMA_TK identifier .arg_ascs ';' eol_pop
           { $$ := Create_Pragma_Node($3, $4);
	     Check_Statement (Ace.Main_Thread_Context, $$); }
   ;

arg_asc :
	expr { $$ := $1; }
   |	identifier ARROW_TK expr
        { Show_Error(3016, Fatal, "Named Association"); }
        -- WPL { $$ := Create_Aggr_Expr_Node ($1, $2, $4); }
   ;


-- 3.1

basic_d  :
      object_d	        { Check_Statement (Ace.Main_Thread_Context, $1);
                          $$ := $1; }
   |  ty_d		{ Check_Statement (Ace.Main_Thread_Context, $1);
                          $$ := $1; }
   |  subty_d		{ Check_Statement (Ace.Main_Thread_Context, $1);
                          $$ := $1; }
   |  subprg_d		{ $$ := $1; }
   |  pkg_d		{ $$ := $1; }
   |  task_d            { Show_Error(3016, Fatal, "Tasking"); }
   |  gen_d             { Show_Error(3016, Fatal, "Generics"); }
   |  excptn_d          { Check_Statement (Ace.Main_Thread_Context, $1);
                          $$ := $1; }
   |  gen_inst          { Show_Error(3016, Fatal, "Generics"); }
   |  renaming_d        { Show_Error(3016, Fatal, "Renaming"); }
   |  number_d          { $$ := $1; }
   ;


-- 3.2 

object_d  :
    eol_push idents ':'           subty_ind ._ASN_expr. eol_pop ';'
    { Create_Object_Decl (Ace.Main_Thread_Context, $2, FALSE, $4, $5, $$); }
 |  eol_push idents ':'  CONSTANT_TK subty_ind ._ASN_expr. eol_pop ';'
    { Create_Object_Decl (Ace.Main_Thread_Context, $2, TRUE, $5, $6, $$); }
 |  eol_push idents ':'           c_arr_def ._ASN_expr. eol_pop ';'
    { Create_Object_Decl (Ace.Main_Thread_Context, $2, FALSE, $4, $5, $$); }
 |  eol_push idents ':'  CONSTANT_TK c_arr_def ._ASN_expr. eol_pop ';'
    { Create_Object_Decl (Ace.Main_Thread_Context, $2, TRUE, $5, $6, $$); }
  ;


number_d  :
        eol_push idents ':' CONSTANT_TK ASSIGN_TK expr eol_pop ';'
    {  Create_Number_Decl (Ace.Main_Thread_Context, $2, $6, $$);
       Check_Statement (Ace.Main_Thread_Context, $$); } ;


idents   :  identifier ...ident..
   { $$ := Link_Id_Onto_Front_Of_List ($1, $2); } ;



-- 3.3.1 

ty_d  :
        full_ty_d       { $$ := $1; }
   |    incomplete_ty_d { Show_Error(3016, Fatal, "Incomplete Declarations"); }
   |    priv_ty_d       { Show_Error(3016, Fatal, "Private Declarations"); }
   ;


full_ty_d  :
    eol_push TYPE_TK identifier IS_TK ty_def ';' eol_pop
     { $$ := Create_Type_Decl (Ace.Main_Thread_Context, $3, null, $5); }
   | eol_push TYPE_TK identifier discr_part IS_TK ty_def ';' eol_pop
     { $$ := Create_Type_Decl (Ace.Main_Thread_Context, $3, $4, $6); }
    ;



ty_def   :
      enum_ty_def
       { $$ := Create_Enum_Type_Def_Entry (Ace.Main_Thread_Context, $1); }
   |  integer_ty_def    { $$ := $1; }
   |  real_ty_def       { $$ := $1; }
   |  array_ty_def      { $$ := $1; }
   |  rec_ty_def        { Show_Error(3016, Fatal, "Records"); }
   |  access_ty_def     { Show_Error(3016, Fatal, "Access"); }
   |  derived_ty_def    { $$ := $1; } ;


-- 3.3.2

subty_d  :
        eol_push SUBTYPE_TK identifier IS_TK subty_ind eol_pop ';'
     { $$ := Create_Subtype_Decl (Ace.Main_Thread_Context, $3, $5); } ;


subty_ind  :    ty_mk .constrt.
     { $$ := Build_Subtype_Indication ($1, $2); } ;


ty_mk  :  expanded_n {  Check_Expression(Ace.Main_Thread_Context, $1); 
                        $$ := Resolve_Type_Mark ($1);
}

   ;
    


constrt  :
        rng_c  { Show_Error(3016, Warning, "Constraints"); }
   |    fltg_point_c { Show_Error(3016, Fatal, "Floating Point"); }
   |    fixed_point_c { Show_Error(3016, Fatal, "Fixed Point"); }
   |    aggr  { $$ := $1; Check_Expression (Ace.Main_Thread_Context, $$);  }
   ;


-- 3.4

derived_ty_def   :  NEW_TK subty_ind
      { $$ := Mark_Derived_Type ($2); } ;
-- 3.5 

rng_c  :  RANGE_TK rng
       { ---$$ := Build_Range_Constraint ($2);
         ---THIS IS STILL UNDER DEVELOPMENT
         ---COMPLETE IMPLEMENTATION NOT YET DONE  [TBD]
         $$ := $2;
       };


rng  :
        name { $$ := $1; }
   |    sim_expr DBLDOT_TK sim_expr { $$ := Create_Range_Expr_Node ($2, $3); }
   ;

-- 3.5.1

enum_ty_def :
        '(' enum_lit_spec ...enum_lit_spec.. ')'
      { $$ := Link_Enum_Literal_On_Front ($2, $3); } ;


enum_lit_spec  :  enum_lit
      { $$ := Create_Enum_Literal_Entry (Ace.Main_Thread_Context, $1); } ;


enum_lit  : identifier   { $$ := $1; }  -- terminal_node built already
   |  character_literal  { $$ := $1; }  -- terminal_node built already
    ;


-- 3.5.4

integer_ty_def   :  rng_c
      { $$ := Build_Integer_Type ($1); } ;


-- 3.5.6 

real_ty_def	      :
	fltg_point_c  { Show_Error(3016, Fatal, "Float"); }
   |    fixed_point_c { Show_Error(3016, Fatal, "Fixed Point"); }
   ;


-- 3.5.7 

fltg_point_c  :
	fltg_accuracy_def .rng_c. { $$ := $1; };


fltg_accuracy_def	  :
	DIGITS_TK sim_expr { $$ := $1; };


-- 3.5.9 

fixed_point_c	:
	fixed_accuracy_def .rng_c. { $$ := $1; };


fixed_accuracy_def	   :
	DELTA_TK sim_expr { $$ := $1; };


-- 3.6 

array_ty_def	       :
	uncnstrnd_array_def { $$ := $1; }
   |    c_arr_def { $$ := $1; }
   ;


uncnstrnd_array_def		:
        ARRAY_TK '(' idx_subty_def ...idx_subty_def.. ')' OF_TK
					 subty_ind 
        { $$ := Build_Unconstrained_Array_Decl ($3, $4, $6); }
        ;


c_arr_def	      :
	ARRAY_TK idx_c OF_TK subty_ind 
        { $$ := Build_Constrained_Array_Decl ($2, $4); }
        ;


idx_subty_def     :     name  RANGE_TK BOX_TK { $$ := $1; };


idx_c  :  '(' dscr_rng ...dscr_rng.. ')'
          { $$ := Link_Id_Onto_Rear_Of_List ($2, $3); };

dscr_rng	:
	rng { $$ := $1; }
   |	name rng_c { $$ := $1;
                     --Show_Error(3016, Warning, "Constrained Ranges");
                    };

-- 3.7 

rec_ty_def      :
        RECORD_TK
            cmpons
        END_TK RECORD_TK
        { $$ := Create_Record_Type_Entry (Ace.Main_Thread_Context, $2); };


cmpons  :
        ..prag.. ..cmpon_d.. cmpon_d ..prag..
      { $$ := Link_Record_Component_Entries ($2, $3); }
   |    ..prag.. ..cmpon_d.. variant_part ..prag.. { $$ := $1; }
   |    ..prag.. NULL_TK ';' ..prag..
      { $$ := null; }
   ;
-- JAT !!! Ignore variants and pragmas for now. !!!


cmpon_d  :
     eol_push idents ':' cmpon_subty_def ._ASN_expr. eol_pop ';'
     { $$ := Build_Record_Component (Ace.Main_Thread_Context, $2, $4, $5); } ;


cmpon_subty_def       :  subty_ind
      { $$ := $1; };


-- 3.7.1 

discr_part  :
        '(' discr_spec ...discr_spec.. ')' { $$ := $1; };


discr_spec  :
	idents ':' ty_mk ._ASN_expr. { $$ := $1; };


-- 3.7.2 

--discr_c	:						     
--      '(' discr_asc ... discr_asc.. ')' ;                
--									     
--discr_asc	    :						     
--  .discr_sim_n ..or_discrim_sim_n.. ARROW. expressi 
--	 ;								     
--"discr_c" is included under "aggr"		     


-- 3.7.3 

variant_part  :
	eol_push CASE_TK sim_n IS_TK
	      ..prag.. variant ..variant..
        END_TK CASE_TK eol_pop ';' { $$ := $1; };


variant  :
	WHEN_TK choice ..or_choice.. ARROW_TK
	    cmpons { $$ := $1; };


choice  :
        sim_expr   { $$ := $1; }
   |    name rng_c { $$ := Create_Expr_Node ($1, $2); }
   |    sim_expr DBLDOT_TK sim_expr
                   { $$ := Create_Range_Expr_Node ($1, $3); }
   |    OTHERS_TK  { $$ := Create_Expr_Node (parserdefs.OTHERS_TK); }
   ;


-- 3.8 

access_ty_def 	:  ACCESS_TK subty_ind { $$ := $1; };


-- 3.8.1 

incomplete_ty_d  :
        eol_push TYPE_TK identifier eol_pop ';' { $$ := $1; }
   |    eol_push TYPE_TK identifier discr_part eol_pop ';'  { $$ := $1; };

-- 3.9 

decl_part	  :
	..basic_decl_item..
      { $$ := $1; }
   |    ..basic_decl_item.. body ..later_decl_item..
      { $$ := Create_Right_Stmt_Seq_Node ($1, $2, $3); }
   ;

basic_decl_item 	:
	basic_d   { $$ := $1; }
   |	rep_cl    { $$ := $1; }
   |    use_cl	  { $$ := $1; Check_Statement(Ace.Main_Thread_Context, $$);}
   ;


later_decl_item 	:
        body      { $$ := $1; }
   |	subprg_d  { $$ := $1; }
   |    pkg_d     { $$ := $1; }
   |	task_d    { Show_Error(3016, Fatal, "Tasking"); }
   |    gen_d     { Show_Error(3016, Fatal, "Generics"); }
   |	use_cl	  { $$ := $1; Check_Statement(Ace.Main_Thread_Context, $$);}
   |    gen_inst  { Show_Error(3016, Fatal, "Generics"); }
   ;

body  :  proper_body { $$ := $1; }
      |  body_stub { Show_Error(3016, Fatal, "Separate"); }
      ;

proper_body  :
	subprg_body  { $$ := $1; }
   |    pkg_body     { $$ := $1; }
   |    task_body    { Show_Error(3016, Fatal, "Tasking"); }
   ;

-- 4.1 

name  : sim_n              { $$ := $1; }
   |    character_literal  { $$ := $1; }
   |    op_symbol          { $$ := $1; }
   |    idxed_cmpon        { $$ := $1; }
   |    selected_cmpon     { $$ := $1; }
--   |    attribute          { Check_Expression (Ace.Main_Thread_Context, $1); $$ := $1; }
   |    attribute          {  $$ := $1; }
   ;

sim_n  :  identifier  { $$ := $1; }
   ;



-- 4.1.1 

idxed_cmpon  :
        prefix aggr { $$ := Create_Expr_Node ($1, $2); }
   ;


-- 4.1.2 

-- slice  :  prefix '(' dscr_rng ')' ;       
--						   
-- included under "idxed_cmpon".		   


-- 4.1.3 

selected_cmpon  :  prefix '.' selector
             { $$ := Create_Dot_Expr_Node ($1, $3); }
   ;

selector  :  sim_n        { $$ := $1; }
   |    character_literal { $$ := $1; }
   |    op_symbol         { $$ := $1; }
   |    ALL_TK            { $$ := Create_Expr_Node (parserdefs.ALL_TK); }
   ;


-- 4.1.4 

attribute  :  prefix ''' attribute_designator
                          { $$ := Create_Attr_Expr_Node ($1, $3); } ;

attribute_designator :
        sim_n     { $$ := $1; }
--   |    DIGITS_TK { $$ := Create_Expr_Node (DIGITS_TK); } -- delete (wpl)
--   |    DELTA_TK  { $$ := Create_Expr_Node (DELTA_TK); }  -- 2/13/89
--   |    RANGE_TK  { $$ := Create_Expr_Node (RANGE_TK); }
   ;

prefix  :  name  { $$ := $1; }
   ;


-- 4.3 

aggr  :
        '(' cmpon_asc ...cmpon_asc.. ')'
       { $$ := Link_Id_Onto_Front_Of_List ($2,$3);
       }

   ;

cmpon_asc        :
        expr { $$ := $1; }
   |    choice ..or_choice.. ARROW_TK expr
             { Show_Error(3016, Fatal, "Named Association"); }
   |    sim_expr DBLDOT_TK sim_expr
             { $$ := Create_Range_Expr_Node ($1, $3); }
   |    name rng_c
             { $$ := Create_Name_Range_Expr_Node ($1, $2); }
   ;


-- 4.4 
expr  :
        rel..AND__rel..       { $$ := $1; }
   |    rel..AND__THEN__rel.. { $$ := $1; }
   |    rel..OR__rel..        { $$ := $1; }
   |    rel..OR__ELSE__rel..  { $$ := $1; }
   |    rel..XOR__rel..       { $$ := $1; }
   ;

rel  :
        sim_expr .relal_op__sim_expr. { $$ := Create_Left_Expr_Node ($1, $2); }
   |    sim_expr.NOT.IN__rng_or_sim_expr.NOT.IN__ty_mk { $$ := $1; }
   ;

sim_expr  :
        .unary_add_op.term..binary_add_op__term..  { $$ := $1; }
   ;

term  :  factor..mult_op__factor.. { $$ := $1; }
   ;

factor  :  pri ._EXP___pri. { $$ := Create_Left_Expr_Node ($1, $2); }
   |    ABS_TK pri          { $$ := Create_Unary_Expr_Node (parserdefs.ABS_TK, $2); }
   |    NOT_TK pri          { $$ := Create_Unary_Expr_Node (parserdefs.NOT_TK, $2); }
   ;

pri  :
        numeric_literal { $$ := $1; }
   |    NULL_TK         { $$ := Create_Expr_Node (parserdefs.NULL_TK); }
   |    allocator       { $$ := $1; }
   |    qualified_expr  { $$ := $1; }
   |    name            { $$ := $1; }
   |    aggr            { $$ := $1; }
   ;

relal_op  :  '=' { $$ := Create_Expr_Node (character'pos('=')); }
   |  NOTEQL_TK  { $$ := Create_Expr_Node (parserdefs.NOTEQL_TK);          }
   |  '<'        { $$ := Create_Expr_Node (character'pos('<')); }
   |  LTEQ_TK    { $$ := Create_Expr_Node (parserdefs.LTEQ_TK);            }
   |  '>'        { $$ := Create_Expr_Node (character'pos('>')); }
   |  GTEQL_TK   { $$ := Create_Expr_Node (parserdefs.GTEQL_TK);           }
   ;

binary_add_op  :  '+' { $$ := Create_Expr_Node (character'pos('+')); }
   |  '-'             { $$ := Create_Expr_Node (character'pos('-')); }
   |  '&'             { $$ := Create_Expr_Node (character'pos('&')); }
   ;

unary_add_op  :  '+' { $$ := Create_Expr_Node (character'pos('+')); }
   |  '-'            { $$ := Create_Expr_Node (character'pos('-')); }
   ;

mult_op  :  '*'  { $$ := Create_Expr_Node (character'pos('*')); }
   |  '/'        { $$ := Create_Expr_Node (character'pos('/')); }
   |  MOD_TK     { $$ := Create_Expr_Node (parserdefs.MOD_TK);             }
   |  REM_TK     { $$ := Create_Expr_Node (parserdefs.REM_TK);             }
   ;

qualified_expr  :
        ty_mkaggr_or_ty_mkPexprP_TK { $$ := $1; }
   ;

allocator  :
        NEW_TK ty_mk
        { $$ := Create_Expr_Node (parserdefs.NEW_TK, $2); }
   |    NEW_TK ty_mk aggr      { $$ := Create_New_Expr_Node ($2, $3); }
   |    NEW_TK ty_mk ''' aggr { $$ := Create_New_Expr_Node ($2, $4); }
   ;
-- 5.1 

seq_of_stmts	:
         ..prag.. stmt ..stmt.. 
	 { $$ := Create_Right_Stmt_Seq_Node ($1, $2, $3); }
   ;


stmt  : ..label.. sim_stmt
	    { $$ := Create_Stmt_Seq_Node ($1,$2); }
   | ..label.. compound_stmt
	    { $$ := Create_Stmt_Seq_Node ($1,$2); }
   ;

sim_stmt  : null_stmt       { $$ := $1; }
   |        assignment_stmt
            { Check_Statement (Ace.Main_Thread_Context, $1); $$ := $1; }
   |        exit_stmt
            { Check_Statement (Ace.Main_Thread_Context, $1); $$ := $1; }
   |        return_stmt
            { Check_Statement (Ace.Main_Thread_Context, $1); $$ := $1; }
   |        goto_stmt       { Show_Error(3016, Fatal, "Goto"); }
   |        delay_stmt      { Show_Error(3016, Fatal, "Delay"); }
   |        abort_stmt      { Show_Error(3016, Fatal, "Abort"); }
   |        raise_stmt      { Show_Error(3016, Fatal, "Raise"); }
   |        code_stmt       { Show_Error(3016, Fatal, "Code"); }
   |        eol_push name eol_pop ';'
            { $$ := Create_Procedure_Call_Node ($2);
	      Check_Statement (Ace.Main_Thread_Context, $$); }
   |        error 
            { $$ := Create_Null_Stmt_Node; }
   ;

-- Procedure and ent call stmts are included under "name".   


compound_stmt  :
	if_stmt      { $$ := $1; }
   |    case_stmt    { Check_Statement (Ace.Main_Thread_Context, $1); $$ := $1; }
   |	loop_stmt    { Check_Statement (Ace.Main_Thread_Context, $1); $$ := $1; }
   |    block_stmt   { Check_Statement (Ace.Main_Thread_Context, $1); $$ := $1; }
   |	accept_stmt  { Show_Error(3016, Fatal, "Accept"); }
   |	select_stmt  { Show_Error(3016, Fatal, "Select"); }
   ;


label  :  L_LBL_TK sim_n R_LBL_TK { Show_Error(3016, Warning, "Label");
                                    $$ := Create_Label_Node ($2); }
   ;


null_stmt  :  NULL_TK ';'  { $$ := Create_Null_Stmt_Node; };


-- 5.2 

assignment_stmt  :  eol_push name ASSIGN_TK expr eol_pop ';'
                    { $$ := Create_Assign_Node ($2, $4); }
     ;


-- 5.3 

if_stmt  :
        eol_push 
        IF_TK cond THEN_TK
	    { Must_Be_Boolean_Type ($3); }
		seq_of_stmts
        ..ELSIF__cond__THEN__seq_of_stmts..
        .ELSE__seq_of_stmts.
        END_TK IF_TK ';' eol_pop 
        { $$ := Create_If_Node ($3, $6, $7, $8); }
   ;

cond  :  expr { $$ := $1; };


-- 5.4 

case_stmt	:
	eol_push CASE_TK expr IS_TK
	    { Check_Expression (Ace.Main_Thread_Context, $3); }
		case_stmt_alt..case_stmt_alt..
        END_TK CASE_TK ';' eol_pop  
        { $$ := Create_Case_Node ($3, $6); }
   ;


case_stmt_alt  :
	WHEN_TK choice ..or_choice.. ARROW_TK
	    seq_of_stmts
        { $$ := Create_Case_Alt_Node ($2, $3, $5); }
    ;


-- 5.5	

loop_stmt	:
         eol_push
	.sim_nC.  --simple name colon 
	    .iteration_scheme. LOOP_TK 
		seq_of_stmts
            END_TK LOOP_TK .sim_n. eol_pop ';'
         { $$ := Create_Loop_Node ($2, $3, $5); }
   ;



iteration_scheme  :
	WHILE_TK cond
	   { Must_Be_Boolean_Type ($2);
	     $$ := $2; }
   |	FOR_TK loop_prm_spec { $$ := $2; } ;


loop_prm_spec  :
	identifier IN_TK .REVERSE. dscr_rng
          { $$ := Create_For_Node ($1, $3, $4); };


-- 5.6 

block_stmt  :
        eol_push
	.sim_nC.
            {Start_of_Block (Ace.Main_Thread_Context);}
	    .DECLARE__decl_part.
	    BEGIN_TK
		seq_of_stmts
	    .EXCEPTION__excptn_handler..excptn_handler...
            END_TK .sim_n. eol_pop ';'
            { $$ := Create_Block_Node ($2, $4, $6, $7);
              End_of_Block (Ace.Main_Thread_Context);
            }
    ;


-- 5.7 

exit_stmt	:
        eol_push EXIT_TK .expanded_n. .WHEN__cond. eol_pop ';'
                { $$ := Create_Exit_Node ($3, $4); }
   ;


-- 5.8 

return_stmt  :  eol_push RETURN_TK .expr. eol_pop ';' 
                { $$ := Create_Return_Node ($3); }
   ;


-- 5.9 

goto_stmt  :  eol_push GOTO_TK expanded_n eol_pop ';'
                { $$ := Create_Goto_Node ($3); }
   ;



-- 6.1 

subprg_d  :  eol_push subprg_spec eol_pop ';'
	       { $$ := Build_Subprogram_Spec_Decl (Ace.Main_Thread_Context, $2);
		 Check_Statement (Ace.Main_Thread_Context, $$);                  }
   ;


subprg_spec  :
	PROCEDURE_TK identifier .fml_part.
	       { $$ := Create_Subprogram_Spec (Ace.Main_Thread_Context, $2, $3, null); }
   |	FUNCTION_TK  designator .fml_part. RETURN_TK ty_mk 
	       { $$ := Create_Subprogram_Spec (Ace.Main_Thread_Context, $2, $3, $5); }
   ;


designator  :  identifier {  $$ := $1; }
	    |  op_symbol  {  $$ := $1; }
   ;


op_symbol  :  string_literal { $$ := $1; }
   ;


fml_part  :  '(' prm_spec .._.prm_spec.. ')'
		{ $$ := Link_Formal_Param_On_Front ($2, $3); }
   ;


prm_spec  :  idents ':' mode ty_mk ._ASN_expr. 
	       { $$ := Build_Formal_Parameter ($1, $3, $4, $5); }
   ;


mode  :  .IN.         { $$ := $1; }
      |  IN_TK OUT_TK
         { $$ := Build_Param_Mode (Ace.Main_Thread_Context,
                 parserdefs.IN_TK, parserdefs.OUT_TK); }
      |  OUT_TK 
         { $$ := Build_Param_Mode (Ace.Main_Thread_Context,
                 parserdefs.OUT_TK); }
   ;



-- 6.3 

subprg_body  :
	eol_push subprg_spec IS_TK

	    { Start_Of_Subprogram_Body (Ace.Main_Thread_Context, $2);  }

	    .decl_part.
	BEGIN_TK
	    seq_of_stmts
	.EXCEPTION__excptn_handler..excptn_handler...
        END_TK .designator. eol_pop ';'

	    {
              Build_Subprogram_Body_Decl
                 (Ace.Main_Thread_Context, $2, $5, $7, $8, $11, $$);
	      Check_Statement (Ace.Main_Thread_Context, $$);
            }
     ;


-- 6.4 

-- procedure_call_stmt	:						    
--      procedure_n .act_prm_part. ';' ;                        
--									    
-- func_call  :							    
--	func_n .act_prm.  ;				    
--									    
-- act_prm_part  :						    
--      '(' prm_asc ... prm_asc .._paren ;                  
--									    
-- prm_asc	  :						    
--	.fml_prm ARROW. act_prm	;		    
--									    
-- fml_prm  : sim_n  ; 			    
--									    
-- act_prm  :							    
--      expr  |  name  |  ty_mk '(' name ')';   
--									    
-- "procedure_call_stmt" and "func_call" are included under "name".



-- 7.1 

pkg_d  :  pkg_spec ';'  { Check_Statement (Ace.Main_Thread_Context, $1);
			  $$ := $1;             }
	 ;

pkg_spec  :
	eol_push PACKAGE_TK identifier
          save_comment  -- Will save PDL help comments for package
        IS_TK
          -- This is the value of $6
	  { Start_Of_Package (Ace.Main_Thread_Context, $3, False, $$); }
	    ..basic_decl_item..
	.PRIVATE..basic_decl_item...       -- Ignored for now [TBD]
	END_TK .sim_n. eol_pop 
	  { Create_Package_Spec_Decl (Ace.Main_Thread_Context, $6, $7, $11, $$); }
    ;


pkg_body  :
	eol_push PACKAGE_TK BODY_TK sim_n IS_TK
          -- This is the value of $6
	  { Start_Of_Package (Ace.Main_Thread_Context, $4, True, $$); }
	    .decl_part.
	    .BEGIN__seq_of_stmts.EXCEPTION__excptn_handler..excptn_handler...
	END_TK .sim_n. eol_pop ';'
	  { Create_Package_Body_Decl
               (Ace.Main_Thread_Context, $6, $7, $8, $11, $$);
	    Check_Statement (Ace.Main_Thread_Context, $$); }
    ;



-- 7.4	

priv_ty_d  :
	eol_push TYPE_TK identifier IS_TK .LIMITED. PRIVATE_TK eol_pop ';'
        { $$ := $1; }
      | eol_push TYPE_TK identifier discr_part
        IS_TK .LIMITED. PRIVATE_TK eol_pop ';' { $$ := $1; } ;

-- defer_const_d	  :		     
--      idents : CONSTANT_TK ty_mk ';'       ;  
--						     
-- Included under "object_d". 	     



-- 8.4 

use_cl  :  eol_push USE_TK expanded_n ...expanded_n.. eol_pop ';'
           { declare
               Tmp : Yystype :=
                    Link_Id_Onto_Front_Of_List ($3, $4);
             begin
              Check_Expression (Ace.Main_Thread_Context, Tmp);
             $$ := Create_Use_Node (Ace.Main_Thread_Context, Tmp);
             end;
            }
       ;



-- 8.5 

renaming_d  :
        eol_push idents ':' ty_mk      RENAMES_TK name eol_pop ';'{ $$ := $1; }
   |    eol_push idents ':' EXCEPTION_TK      RENAMES_TK expanded_n eol_pop ';'
        { $$ := $1; }
   |    eol_push PACKAGE_TK identifier        RENAMES_TK expanded_n eol_pop ';'
        { $$ := $1; }
   |    eol_push subprg_spec  RENAMES_TK name eol_pop ';'  
        { $$ := $1; }
   ;

-- idents in the two above rule must contain only one 
-- identifier.						       


-- 9.1 

task_d  :  task_spec ';'  
          { $$ := $1; } ;

task_spec  :
	eol_push TASK_TK .TYPE. identifier
	    .IS..ent_d_..rep_cl_END.sim_n.
	    eol_pop 
	{ $$ := $1; } ;

task_body  :
	eol_push TASK_TK BODY_TK sim_n IS_TK
	    .decl_part.
	BEGIN_TK
	    seq_of_stmts
	.EXCEPTION__excptn_handler..excptn_handler...
        END_TK .sim_n. eol_pop ';' 
	{ $$ := $1; } ;


-- 9.5 

ent_d  :
	eol_push ENTRY_TK identifier .fml_part. eol_pop ';' { $$ := $1; }
   |    eol_push ENTRY_TK identifier '(' dscr_rng ')' .fml_part. eol_pop ';' 
        { $$ := $1; } ;


ent_call_stmt  :
        ..prag.. name ';' { $$ := $1; };


accept_stmt  :
	eol_push ACCEPT_TK sim_n .Pent_idx_P..fml_part.
            .DO__seq_of_stmts__END.sim_n.. eol_pop ';' { $$ := $1; };


ent_idx  :	expr { $$ := $1; }  ;


-- 9.6 

delay_stmt  :  eol_push DELAY_TK sim_expr eol_pop ';' { $$ := $1; };


-- 9.7 

select_stmt  :	selec_wait { $$ := $1; }
   |	condal_ent_call	{ $$ := $1; } |  timed_ent_call { $$ := $1; } ;


-- 9.7.1 

selec_wait	:
	eol_push SELECT_TK
	    select_alt
	    ..OR__select_alt..
	    .ELSE__seq_of_stmts.
        END_TK SELECT_TK eol_pop ';' { $$ := $1; };


select_alt  :
	.WHEN__condARROW.selec_wait_alt  { $$ := $1; };


selec_wait_alt  :  accept_alt { $$ := $1; }
   |	delay_alt { $$ := $1; } |  terminate_alt  { $$ := $1; };


accept_alt  :
	accept_stmt.seq_of_stmts. { $$ := $1; } ;


delay_alt  :
	delay_stmt.seq_of_stmts.  { $$ := $1; };


terminate_alt  :  TERM_stmt   { $$ := $1; };


-- 9.7.2 

condal_ent_call	:
	eol_push SELECT_TK
	    ent_call_stmt
	    .seq_of_stmts.
	ELSE_TK
	    seq_of_stmts
        END_TK SELECT_TK eol_pop ';' { $$ := $1; };



-- 9.7.3 

timed_ent_call  :
	eol_push SELECT_TK
	    ent_call_stmt
	    .seq_of_stmts.
	OR_TK
	    delay_alt
        END_TK SELECT_TK eol_pop ';' { $$ := $1; };


-- 9.10  

abort_stmt  :  eol_push ABORT_TK name ...name.. eol_pop ';' { $$ := $1; } ;


-- 10.1 


compilation  :
	..compilation_unit.. { $$ := $1; } ;


compilation_unit  :
	context_cl
        eol_pop
        library_unit
	{ $$ := Create_Compilation_Node ($1,$3);
          Pop_Context_Clause(Ace.Main_Thread_Context); }
   |    context_cl
        eol_pop
        secondary_unit 
	{ $$ := Create_Compilation_Node ($1,$3);
          Pop_Context_Clause(Ace.Main_Thread_Context);}
   ;

library_unit  :
	subprg_d	{ $$ := $1; }
   |    pkg_d           { $$ := $1; }
   |	gen_d           { Show_Error(3016, Fatal, "Generics"); }
   |    gen_inst        { Show_Error(3016, Fatal, "Generics"); }
   |    renaming_d      { Show_Error(3016, Fatal, "Renaming"); }
   |	subprg_body     { $$ := $1; }
   ;


secondary_unit	:
	library_unit_body { $$ := $1; } |  subunit	{ $$ := $1; };


library_unit_body  :
	pkg_body_or_subprg_body  { $$ := $1; }
    ;



-- 10.1.1  

context_cl      : ..with_cl..use_cl.... 
                  { $$ := $1; Push_Context_Clause(Ace.Main_Thread_Context); Check_Statement(Ace.Main_Thread_Context, $$); }
   ;


with_cl  :  WITH_TK sim_n ...sim_n.. ';' 
            { $$ := Create_With_Node (Link_Id_Onto_Front_Of_List($2,$3)); }
   ;


-- 10.2 

body_stub  :
       eol_push subprg_spec IS_TK SEPARATE_TK eol_pop ';' { $$ := $1; }
   |   eol_push PACKAGE_TK BODY_TK sim_n IS_TK SEPARATE_TK eol_pop ';'
       { $$ := $1; }
   |   eol_push TASK_TK BODY_TK sim_n    IS_TK SEPARATE_TK eol_pop ';' 
       { $$ := $1; }
   ;


subunit  :  SEPARATE_TK '(' expanded_n ')' proper_body  { $$ := $1; } ;



-- 11.1 

excptn_d  :  eol_push idents ':' EXCEPTION_TK eol_pop ';'
       {  Create_Exception_Declaration(Ace.Main_Thread_Context, $2, $$); }
    ;


-- 11.2 

excptn_handler	:
	WHEN_TK excptn_choice ..or_excptn_choice.. ARROW_TK
	    seq_of_stmts 
        { $$ := Create_Exception_Alt_Node ($2, $3, $5); }
    ;


excptn_choice  :  expanded_n 
                    { $$ := $1;}
    |	OTHERS_TK { $$ := $1; } ;


-- 11.3 

raise_stmt  :  RAISE_TK .expanded_n. ';' { $$ := $1; } ;



-- 12.1 

gen_d  :  eol_push gen_spec eol_pop ';'  { $$ := $1; };

gen_spec  :
	gen_fml_part subprg_spec { $$ := $1; }
   |	gen_fml_part pkg_spec  { $$ := $1; };


gen_fml_part  :	GENERIC_TK ..gen_prm_d..  { $$ := $1; };


gen_prm_d  :
	idents ':' .IN.OUT.. ty_mk ._ASN_expr. ';' { $$ := $1; }
   |    eol_push TYPE_TK identifier IS_TK  gen_ty_def eol_pop ';' { $$ := $1; }
   |	priv_ty_d { $$ := $1; }
   |    WITH_TK subprg_spec .IS_BOX_. ';' ;
-- |    WITH_TK subprg_spec .IS_TK name. ';'  
--						     
-- This rule is included in the previous one.	     


gen_ty_def	 : 
        '(' BOX_TK ')'  { $$ := $1; }
   |  RANGE_TK BOX_TK   { $$ := $1; }
   |  DIGITS_TK BOX_TK  { $$ := $1; }
   |  DELTA_TK BOX_TK   { $$ := $1; }
   |  array_ty_def      { $$ := $1; }
   |  access_ty_def	{ $$ := $1; }   ;


-- 12.3  

gen_inst  :
	eol_push PACKAGE_TK identifier
          save_comment -- Will save PDL help comments for generic package [TBD]
        IS_TK
            NEW_TK expanded_n .gen_act_part. eol_pop ';' { $$ := $1; }
   |	eol_push PROCEDURE__ident__IS_TK
            NEW_TK expanded_n .gen_act_part. eol_pop ';' { $$ := $1; }
   |	eol_push FUNCTION_TK  designator IS_TK
	    NEW_TK expanded_n .gen_act_part. eol_pop ';' { $$ := $1; } ;


gen_act_part  :
        '(' gen_asc ...gen_asc.. ')'     { $$ := $1; } ;


gen_asc	     :
	.gen_fml_prmARROW.gen_act_prm { $$ := $1; };


gen_fml_prm  :
	sim_n   { $$ := $1; }
     |  op_symbol { $$ := $1; }  ;

gen_act_prm  :
  expr_or_name_or_subprg_n_or_ent_n_or_ty_mk { $$ := $1; }
	 ;


-- 13.1 

rep_cl  :
	ty_rep_cl  { $$ := $1; }
   |  address_cl { $$ := $1; } ;

ty_rep_cl  :  length_cl { $$ := $1; }
   |	enum_rep_cl { $$ := $1; }
   |	rec_rep_cl	{ $$ := $1; }    ;


-- 13.2 

length_cl  :  eol_push FOR_TK attribute USE_TK sim_expr eol_pop ';'
    {$$ := $1; };



-- 13.3 

enum_rep_cl  :
        eol_push FOR__ty_sim_n__USE_TK aggr eol_pop ';' { $$ := $1; };



-- 13.4  

rec_rep_cl	:
	eol_push FOR__ty_sim_n__USE_TK
	    RECORD_TK .algt_cl.
		..cmpon_cl..
            END_TK RECORD_TK eol_pop ';'   { $$ := $1; };


algt_cl  :     eol_push AT_TK MOD_TK sim_expr eol_pop ';' { $$ := $1; };


cmpon_cl  :
        name eol_push  AT_TK sim_expr RANGE_TK rng eol_pop ';'
        { $$ := $1; };



-- 13.5 

address_cl  :  eol_push FOR_TK sim_n USE_TK AT_TK sim_expr eol_pop ';'
               { $$ := $1; } ;


-- 13.8 

code_stmt  :  eol_push ty_mk_rec_aggr eol_pop ';' { $$ := $1; };


------------------------------------------------------------------------

-- The following rules define semantically qualified symbols under more
-- general categories.

-- ty_n_or_subty_n  :  expanded_n ;
--								      
-- An "expanded_n" is used for names that can be written using only
-- selectors.							      

--     ... these have been replaced logically to reduce the number    
--         of YACC_TK nonterminal "rules"                                

-- The following rules expand the concatenated constructs and define the 
-- specially added syntactical classes. 				 


-- 2.1 

..prag..  :
       -- empty 
       { $$ := null; }
   |	..prag.. prag
       { $$ := Create_Left_Stmt_Seq_Node ($1, $2, null); }
   ;

.arg_ascs :
       -- empty 
       { $$ := null; }
   |    '(' arg_ascs ')'
       { $$ := $2; }
   ;

arg_ascs :
	arg_asc
     { $$ := Link_Id_Onto_Rear_Of_List ($1, null); }
   |    arg_ascs ',' arg_asc 
     { $$ := Link_Id_Onto_Rear_Of_List ($3, $1); }
   ;

--                                      
-- "name" is included under "expr"	


-- 3.1

--
--  "defer_const_d" is included under "object_d".

._ASN_expr.     :
     { $$ := null; }  -- empty
   |    ASSIGN_TK expr
     { Check_Expression (Ace.Main_Thread_Context, $2);
       $$ := $2; } ;

...ident..  :
     { $$ := null; }  -- empty
   |    ...ident.. ',' identifier
     { $$ := Link_Id_Onto_Rear_Of_List ($3, $1); } ;
     -- Link_Id_Onto_Rear_Of_List expects a single id as its first argument and
     -- a list (possibly null) of ids as the second argument

.constrt.  :
     { $$ := null; }  -- empty
   |    constrt
     { $$ := $1; } ;

--									 
-- "idx_c" and "discr_c" are included under	 
-- the class "aggr".						 

expanded_n  :
        identifier
          { $$ :=  $1; }
   |    expanded_n '.' identifier
          { $$ := Create_Dot_Expr_Node ($1, $3); }
   ;

--								       
--  This expansion generalizes "rng" so that it may include ty and 
--  subty names.						       

...enum_lit_spec.. :
       -- empty 
       { $$ := null; }
   |    ...enum_lit_spec.. ',' enum_lit_spec
       { $$ := Link_Enum_Literal_On_End ($1, $3); }
   ;

.rng_c.  :
       -- empty 
       { $$ := null; }
   |	rng_c	{ $$ := $1; } ;

...idx_subty_def..	       :
       -- empty 
       { $$ := null; }
   |    ...idx_subty_def.. ',' idx_subty_def { $$ := $1; } ;

--								   
-- To avoid conflicts, the more general class "name" is used.	   

...dscr_rng.. :
       -- empty 
       { $$ := null; }
   |    ...dscr_rng.. ',' dscr_rng  { $$ := Link_Id_Onto_Rear_Of_List
					  ($3, $1); } ;


-- A dscr subty ind given as a ty mk is included under rng


..cmpon_d.. :
       -- empty 
       { $$ := null; }
   |    ..cmpon_d.. cmpon_d ..prag..
       { $$ := Link_Record_Component_Entries ($1, $2); }
   ;

...discr_spec..	:
       -- empty 
       { $$ := null; }
   |    ...discr_spec.. ';' discr_spec  { $$ := $1; } ;


-- Pragmas that can appear between two consecutive variants are picked	
-- up in the cmpons part of the variants themselves.		

..variant..  :
       -- empty 
       { $$ := null; }
   |	..variant.. variant { $$ := $1; } ;

..or_choice.. :
       { $$ := null; } -- empty
   |    ..or_choice.. '|' choice
       { $$ := Link_Id_Onto_Rear_Of_List ($3, $1); }
   ;

-- The "sim_expr" by itself may be a "dscr_rng" or a  
-- "cmpon_sim_n".					     

-- A body is the only later_decl_item that is not also a	   
-- basic_decl_item.	    It is therefore used as the dividing   
-- point between the two lists of decl items.			   

..basic_decl_item..	    :
	..prag..
      { $$ := $1; }
   |	..basic_decl_item.. basic_decl_item ..prag..
      { $$ := Create_Left_Stmt_Seq_Node ($1, $2, $3); }
   ;

..later_decl_item..      :
	..prag..
      { $$ := $1; }
   |    ..later_decl_item.. later_decl_item ..prag.. 
      { $$ := Create_Left_Stmt_Seq_Node ($1, $2, $3); }
   ;


-- 4.1 


-- "slice" is included under "idxed_cmpon".  

-- The def of "name" includes "func_call".				
-- A prmless func call is recognized as a sim name or a	
-- selectedcmpon.	A func call with prms is recognized	
-- as an idxed cmpon.						


-- Reserved word attribute designators are included in the rules as a	     
-- convenience.  Alternativly, since an attribute designator is always preced
-- by an apostrophe, as noted in the RR_TK 4.1.4, such usage may be detected    
-- during lexical analysis thus obviating the need for special rules.	     
--									     
-- The univ static expr of an attribute designator is reduced     
-- as an "idxed_cmpon".						     

...cmpon_asc..      :
       -- empty
       { $$ := null; }
   |    ...cmpon_asc.. ',' cmpon_asc
       { $$ := Link_Id_Onto_Rear_Of_List ($3, $1); }
   ;


-- Component ascs are generalized to include dscr rngs. 	   
-- Thus, an "aggr" can be used for slices and idx and discr  
-- constrts.							   

rel..AND__rel..  :
        rel AND_TK rel { $$ := Create_Expr_Node ($1, parserdefs.AND_TK, $3); }
   |    rel..AND__rel.. AND_TK rel
                       { $$ := Create_Expr_Node ($1, parserdefs.AND_TK, $3); }
   ;

rel..OR__rel.. :
        rel OR_TK rel  { $$ := Create_Expr_Node ($1, parserdefs.OR_TK, $3); }
   |    rel..OR__rel.. OR_TK rel 
                       { $$ := Create_Expr_Node ($1, parserdefs.OR_TK, $3); }
   ;

rel..XOR__rel..  :
        rel            { $$ := $1; }
   |    ..XOR__rel..   { $$ := $1; }
   ;

..XOR__rel..  :
        rel XOR_TK rel { $$ := Create_Expr_Node ($1, parserdefs.XOR_TK, $3); }
   |    ..XOR__rel.. XOR_TK rel 
                       { $$ := Create_Expr_Node ($1, parserdefs.XOR_TK, $3); }
   ;

rel..AND__THEN__rel..  :
        rel AND_TK THEN_TK rel
     { $$ := Create_Expr_Node ($1, parserdefs.AND_TK, parserdefs.THEN_TK, $3); }
   |    rel..AND__THEN__rel.. AND_TK THEN_TK rel
     { $$ := Create_Expr_Node ($1, parserdefs.AND_TK, parserdefs.THEN_TK, $3); }
   ;

rel..OR__ELSE__rel..    :
        rel OR_TK ELSE_TK rel
      { $$ := Create_Expr_Node ($1, parserdefs.OR_TK, parserdefs.ELSE_TK, $3); }
   |    rel..OR__ELSE__rel.. OR_TK ELSE_TK rel 
      { $$ := Create_Expr_Node ($1, parserdefs.OR_TK, parserdefs.ELSE_TK, $3); }
   ;

.relal_op__sim_expr.  :
       -- empty 
       { $$ := null; }
   |    relal_op sim_expr
       { $$ := Create_Right_Expr_Node ($1, $2); }
   ;

sim_expr.NOT.IN__rng_or_sim_expr.NOT.IN__ty_mk  :
        sim_expr .NOT. IN_TK rng
        { $$ := Create_Expr_Node ($1, $2, parserdefs.IN_TK, $4); }
   ;

.NOT.  :
       -- empty 
       { $$ := null; }
   |    NOT_TK
       { $$ := Create_Expr_Node (parserdefs.NOT_TK); }
   ;

.unary_add_op.term..binary_add_op__term..  :
        term              { $$ := $1; }
   |    unary_add_op term { $$ := Create_Unary_Add_Expr_Node ($1, $2); }
   |    .unary_add_op.term..binary_add_op__term..  binary_add_op term
                          { $$ := Create_Expr_Node ($1, $2, $3); }
   ;

factor..mult_op__factor..       :
        factor            { $$ := $1; }
   |    factor..mult_op__factor.. mult_op factor
                          { $$ := Create_Expr_Node ($1, $2, $3); }
   ;

._EXP___pri.  :
       -- empty 
       { $$ := null; }
   |    EXP_TK pri
       { $$ := Create_Expr_Node (parserdefs.EXP_TK, $2); }
   ;

ty_mkaggr_or_ty_mkPexprP_TK  :
        prefix ''' aggr
       { $$ := Create_Expr_Node ($1, character'pos('''), $3); }
       -- should be a type mark node.
   ;
-- 5.1 


..stmt..  :
	..prag.. { $$ := $1; }
   |	..stmt.. stmt ..prag.. { $$ := Create_Left_Stmt_Seq_Node ($1, $2, $3); }
   ;


..label..   :
       -- empty 
       { $$ := null; }
   |	..label.. label { $$ := $1; } ;


..ELSIF__cond__THEN__seq_of_stmts..  :
       -- empty  
        { $$ := null; }  
   |    ..ELSIF__cond__THEN__seq_of_stmts..
                ELSIF_TK cond THEN_TK
	   { Must_Be_Boolean_Type ($3); }
                seq_of_stmts    
        { $$ := Create_If_Node ($3, $6, $1); };

.ELSE__seq_of_stmts.    :
       -- empty  
        { $$ := null; }  
   |    ELSE_TK seq_of_stmts 
        { $$ := $2; };

case_stmt_alt..case_stmt_alt.. :
	    ..prag..  -- forget this pragma...
	    case_stmt_alt
	    ..case_stmt_alt..
        { $$ := Create_Left_Case_Alt_Node ($2, $3); }
   ;

..case_stmt_alt..	:
       -- empty 
       { $$ := null; }
   |	..case_stmt_alt.. case_stmt_alt
       { $$ := Create_Right_Case_Alt_Node ($1, $2); }
   ;

.sim_nC.	:
        -- empty 
       { $$ := null; }
   |    sim_n ':' { $$ := $1; } ;

.sim_n. :
       -- empty 
       { $$ := null; }
   |	sim_n  { $$ := $1; } ;

.iteration_scheme.  :
       -- empty 
       { $$ := null; }
   |	iteration_scheme  { $$ := $1; };

.REVERSE. :
       -- empty 
       { $$ := null; }
   |	REVERSE_TK   { $$ := $1; } ;

.DECLARE__decl_part.	     :
       -- empty 
       { $$ := null; }
   |	DECLARE_TK decl_part
       { $$ := $2; }
   ;

.EXCEPTION__excptn_handler..excptn_handler... :
       -- empty 
       { $$ := null; }
   |	EXCEPTION_TK
	    ..prag.. excptn_handlers
       { $$ := $3; }
   ;

excptn_handlers	:
	excptn_handler
       { $$ := $1; }
   |	excptn_handlers excptn_handler  
       { $$ := Create_Right_Exception_Alt_Node ($1, $2); } ;

.expanded_n.  :
       -- empty
       { $$ := null; }
   |	expanded_n  { $$ := $1; }
   ;

.WHEN__cond.  :
       -- empty
       { $$ := null; }
   |	WHEN_TK cond  { Must_Be_Boolean_Type ($2); $$ := $2; }
   ;

.expr.	:
       -- empty
       { $$ := null; }
   |	expr    { $$ := $1; }
   ;


-- 6.1 

.fml_part.  :
       -- empty
       { $$ := null; }
   |	fml_part  { $$ := $1; }
   ;

.._.prm_spec..  :
       -- empty 
       { $$ := null; }
   |    .._.prm_spec.. ';' prm_spec
       { $$ := Link_Formal_Param_On_Rear ($3, $1); }
   ;

.IN.  :
       -- empty
        { $$ := Build_Param_Mode(Ace.Main_Thread_Context); }
   |	IN_TK
        { $$ := Build_Param_Mode (Ace.Main_Thread_Context, parserdefs.IN_TK); }
   ;

.decl_part.    :  decl_part  { $$ := $1; }  ;

-- A "decl_part" may be empty.		

.designator.  :
       -- empty
       { $$ := null; }
   |    designator { $$ := $1; }
   ;


-- 7.1	

.PRIVATE..basic_decl_item... :
       -- empty 
       { $$ := null; }
   |	PRIVATE_TK
	    ..basic_decl_item.. { $$ := $1; } ;

.BEGIN__seq_of_stmts.EXCEPTION__excptn_handler..excptn_handler...
				:
       -- empty 
       { $$ := null; }
   |	BEGIN_TK
	    seq_of_stmts
	.EXCEPTION__excptn_handler..excptn_handler...
      { $$ := Create_List_Item_Node ($2, $3); }
   ;

.LIMITED.  :
       -- empty 
       { $$ := null; }
   |	LIMITED_TK    { $$ := $1; } ;

...expanded_n.. :
       -- empty 
       { $$ := null; }
   |    ...expanded_n.. ',' expanded_n
       { $$ := Link_Id_Onto_Rear_Of_List ($3, $1); }
   ;


-- 9.1 

.TYPE.	:
       -- empty 
       { $$ := null; }
   |	TYPE_TK   
       { $$ := Create_Expr_Node (parserdefs.TYPE_TK); }
   ;

.IS..ent_d_..rep_cl_END.sim_n.  :
       -- empty 
   |	IS_TK
	    ..ent_d..
	    ..rep_cl..
	END_TK .sim_n. { $$ := $1; }
    ;


..ent_d..  :
	..prag..
      { $$ := $1; }
   |	..ent_d.. ent_d ..prag..
      { $$ := Create_Left_Stmt_Seq_Node ($1, $2, $3); }
	;

..rep_cl..  :
       -- empty 
       { $$ := null; }
   |	..rep_cl.. rep_cl ..prag..
       { $$ := Create_Left_Stmt_Seq_Node ($1, $2, $3); }
	;


.Pent_idx_P..fml_part.  :
	.fml_part. { $$ := $1; }
   |    '(' ent_idx ')' .fml_part.  { $$ := $1; };

.DO__seq_of_stmts__END.sim_n..  :
       -- empty 
       { $$ := null; }
   |	DO_TK
	  seq_of_stmts
	END_TK .sim_n.  { $$ := $1; };

..OR__select_alt..  :
       -- empty 
       { $$ := null; }
   |	..OR__select_alt.. OR_TK select_alt	{ $$ := $1; };

.WHEN__condARROW.selec_wait_alt  :
	selec_wait_alt { $$ := $1; }
   |	WHEN_TK cond ARROW_TK selec_wait_alt
	    { $$ := $1; };

accept_stmt.seq_of_stmts.  :
	..prag.. accept_stmt .seq_of_stmts.  { $$ := $1; };

delay_stmt.seq_of_stmts.  :
	..prag.. delay_stmt .seq_of_stmts. { $$ := $1; } ;

TERM_stmt   :  ..prag.. TERMINATE_TK ';' ..prag..  { $$ := $1; };

.seq_of_stmts.	:
	..prag.. { $$ := $1; }
   |	seq_of_stmts { $$ := $1; }	;

...name..	:
       -- empty 
       { $$ := null; }
   |    ...name.. ',' name { $$ := $1; } ;



-- 10.1 

..compilation_unit..  :
	..prag.. { $$ := $1; }
   |	..compilation_unit.. compilation_unit ..prag.. 
      { $$ := $2; }	-- [TBD] seq of comp units
   ;

pkg_body_or_subprg_body   :  pkg_body  { $$ := $1; }
   ;

-- "subprg_body" is already contained in the class "library_unit". 

..with_cl..use_cl.... :
       eol_push -- empty 
       { $$ := null; }
   |   ..with_cl..use_cl.... with_cl use_cls 
         { $$ := Create_Left_Stmt_Seq_Node($1,$2,$3); }
   ;


use_cls  :
	..prag.. { $$ := $1; }
   |	use_cls use_cl ..prag..  
         { $$ := Create_Left_Stmt_Seq_Node($1,$2,$3); }
   ;

...sim_n..  :
       -- empty 
       { $$ := null; }
   |    ...sim_n.. ',' sim_n 
       { $$ := Link_Id_Onto_Rear_Of_List ($3, $1); }
   ;



-- 11.1  

..or_excptn_choice.. :
       -- empty 
       { $$ := null; }
   |    ..or_excptn_choice.. '|' excptn_choice { $$ := $1; } ;



-- 12.1 

..gen_prm_d..  :
       -- empty 
       { $$ := null; }
   |	..gen_prm_d.. gen_prm_d { $$ := $1; } ;

.IN.OUT..  :
	.IN. { $$ := $1; }
   |	IN_TK OUT_TK  { $$ := $1; };

.IS_BOX_.	:
       -- empty 
       { $$ := null; }
   |	IS_TK name { $$ := $1; }
   |	IS_TK BOX_TK { $$ := $1; };

PROCEDURE__ident__IS_TK    :  subprg_spec IS_TK { $$ := $1; } ;

-- To avoid conflicts, the more general "subprg_spec" is used.  

.gen_act_part.  :
       -- empty 
       { $$ := null; }
   |	gen_act_part { $$ := $1; } ;

...gen_asc.. :
       -- empty 
       { $$ := null; }
   |    ...gen_asc.. ',' gen_asc { $$ := $1; } ;

.gen_fml_prmARROW.gen_act_prm  :
	gen_act_prm { $$ := $1; }
   |	gen_fml_prm ARROW_TK gen_act_prm { $$ := $1; } ;

expr_or_name_or_subprg_n_or_ent_n_or_ty_mk
				:  expr { $$ := $1; } ;

-- The above alts are included under "expr". 


-- 13.1 

FOR__ty_sim_n__USE_TK  :
	FOR_TK sim_n USE_TK { $$ := $1; };

-- The "sim_n" must be a "ty_sim_n". 

.algt_cl.  :
	..prag.. { $$ := $1; }
   |    ..prag.. algt_cl ..prag.. { $$ := $1; }  ;

..cmpon_cl..  :
       -- empty 
       { $$ := null; }
   |    ..cmpon_cl.. cmpon_cl ..prag..  { $$ := $1; };

ty_mk_rec_aggr  :  qualified_expr     { $$ := $1; } ;

-- The qualified expr must contain a rec aggr. 

%%

with Yyval_Error;
use Yyval_Error;

with User_Pkg;
use User_Pkg;

with Lexacts;
use Lexacts;

with Error_Handler;
use Error_Handler;

with Help_Pkg;
use Help_Pkg;

with Declaration_Routines;
use Declaration_Routines;

with Declarative_Region_Support_Routines;
use Declarative_Region_Support_Routines;

with Semantic_Analyzer;
use Semantic_Analyzer;

with Create_Statement_Database;
use Create_Statement_Database;

with Ace_Text_Io;
use  Ace_Text_Io;

with Parserdefs; --  Remove later [TBD] wpl

with Ace;

-- Parser packages

with Common_Parser_Routines;
use  Common_Parser_Routines;

with Tokens_Definition, compilation_unit_shift_reduce, compilation_unit_goto;

package body compilation_unit_parser is

  ## -- YYParse will be inserted here.

end compilation_unit_parser;
