--|$Log:	rwace.a,v $
--|Revision 8.0  91/01/17  10:45:57  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Reader_Writer
--
---------------------------------------------------------------
--
--
-- Abstract
--    This package provides most of the routines to write a representation
--   of the ACE internal information. Since the many of the definitions
--   within context_definition are either private or hidden, some dump routines
--   will be found in the context_control package.
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
with Symbol_Table_Structures;
use Symbol_Table_Structures;
with Statement_Database;
use Statement_Database;
with Misc_Support_Routines;
use Misc_Support_Routines;
with Symtbl_Entry_Get_Routines;
use Symtbl_Entry_Get_Routines;
with Node_Get_Routines;

with Literals;

with Storage_Management;
with Storage_Management_Routines;
with Hash_Table;
with Ace_Universal_Types;

with Text_Io;
use Text_Io;

package body Reader_Writer is

   package Int_Io is new Text_Io.Integer_Io (Integer);
   package Ace_Int_Io is new Text_Io.Integer_Io
       (Ace_Universal_Types.Ace_Integer);

   package Symtbl_Hash is new Hash_Table (Ref_Symbol_Table_Entry, Natural, 129);
   package Sdb_Hash is new Hash_Table (Ref_Statement_Database, Natural, 129);
   package Refstr_Hash is new Hash_Table (Literals.Ref_String, Natural, 129);

   package Datastor_Hash is new Hash_Table
       (Storage_Management.Ref_Data_Storage, Natural, 129);

   use Int_Io;
   use Ace_Int_Io;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Dump_Ref_String
   --
   --  <Description>
   --
   --    Will dump out ref_strings and keep track of them via the hash table.
   --
   --==========================================================================
   procedure Dump_Ref_String (Outfile : File_Type;
                              Str     : Literals.Ref_String) is
      use Refstr_Hash;

      Node_Num : Node_Id;
   begin
      if (Is_Empty (Str)) then
         Put (" *0 ");
      else
         Node_Num := Save_Rec (Str, 0);
         if (Node_Num < 0) then
            Put (Outfile, " *");
            Put (Outfile, - Integer (Node_Num), 1);
            Put (Outfile, " ");

         else
            Put (Outfile, """");
            Put (Outfile, Literals.Convert_To_String (Str));
            Put (Outfile, """ ");
         end if;
      end if;
   end Dump_Ref_String;

   procedure Dump_Statement_Dbase (Outfile   : File_Type;
                                   Curr_Node : Ref_Statement_Database);

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Dump_Statement_Node
   --
   --  <Description>
   --
   --    Will dump out statement_database statement nodes.
   --
   --==========================================================================
   procedure Dump_Statement_Node (Outfile : File_Type;
                                  Node    : Ref_Statement_Database) is
      use Node_Get_Routines;
   begin
      Put
          (Integer
               (Statement_Enumeration'POS (Node.Statement_Node.Statement_Kind))
           );

      Dump_Statement_Dbase (Outfile, Get_Next_Statement (Node));
      Dump_Statement_Dbase (Outfile, Get_Holder (Node));
      Put (Outfile, Node.Statement_Node.Statement_Number);
      Put (Outfile, " ");
      Put
          (Integer'IMAGE (
              Boolean'POS (

                 Node.Statement_Node.Suspend_Evaluation_At_This_Statement)));
      Put (Outfile, " ");

      case Node.Statement_Node.Statement_Kind is
         when Object_Declaration    | Exception_Declaration  =>
            Dump_Statement_Dbase (Outfile, Get_Name_List (Node));
         when Type_Declaration           |
              Subtype_Declaration        |
              Procedure_Spec_Declaration |
              Procedure_Declaration      |
              Function_Spec_Declaration  |
              Function_Declaration       |
              Package_Declaration        |
              Package_Body_Declaration   =>
            Dump_Symbol_Table_Node (Outfile, Get_Symbol_Table_Entry (Node));

         when Assign_Statement =>
            Dump_Statement_Dbase (Outfile, Get_Lhs (Node));
            Dump_Statement_Dbase (Outfile, Get_Rhs (Node));
         when Loop_Statement   =>
            Dump_Statement_Dbase (Outfile, Get_Name (Node));
            Dump_Statement_Dbase (Outfile, Get_Loop_Scheme (Node));
            Dump_Statement_Dbase (Outfile, Get_Head_Of_Statements_List (Node));
         when For_Statement    =>
            Dump_Statement_Dbase (Outfile, Get_For_Variable (Node));
            Dump_Statement_Dbase (Outfile, Get_For_Range (Node));
         when If_Statement     =>
            Dump_Statement_Dbase (Outfile, Get_Condition (Node));
            Dump_Statement_Dbase (Outfile,
                                  Get_Head_Of_True_Task_Statements_List (Node));
            Dump_Statement_Dbase
                (Outfile, Get_Head_Of_False_Task_Statements_List (Node));
         when Case_Statement   =>
            Dump_Statement_Dbase (Outfile, Get_Expression (Node));
            Dump_Statement_Dbase (Outfile, Get_Alternatives_List (Node));

         when Case_Alternative_Part =>
            Dump_Statement_Dbase (Outfile, Get_Alternative_Choice (Node));
            Dump_Statement_Dbase (Outfile, Get_Alternative_Statements (Node));
            Dump_Statement_Dbase (Outfile, Get_Next_Alternative (Node));

         when Exception_Alternative =>
            Dump_Statement_Dbase (Outfile, Get_Alternative_Choice (Node));
            Dump_Statement_Dbase (Outfile, Get_Alternative_Statements (Node));
            Dump_Statement_Dbase (Outfile, Get_Next_Alternative (Node));

         when Labeled_Statement     =>
            Dump_Statement_Dbase (Outfile, Get_Label_Entry (Node));

         when Goto_Statement   =>
            Dump_Statement_Dbase (Outfile, Get_Goto_Label (Node));

         when Exit_Statement   =>
            Dump_Statement_Dbase (Outfile, Get_Name (Node));
            Dump_Statement_Dbase (Outfile, Get_Condition (Node));

         when Return_Statement =>
            Dump_Statement_Dbase (Outfile, Get_Expression (Node));
            Dump_Symbol_Table_Node (Outfile, Get_Subprogram_Reference (Node));
         when Block_Statement  =>
            Dump_Statement_Dbase (Outfile, Get_Name (Node));
            Dump_Statement_Dbase (Outfile, Get_Block_Declarations (Node));
            Dump_Statement_Dbase (Outfile, Get_Head_Of_Statements_List (Node));
            Dump_Statement_Dbase (Outfile, Get_Exception_Handler (Node));

         when With_Statement   | Use_Statement     =>
            Dump_Statement_Dbase (Outfile, Get_Name_List (Node));

         when Procedure_Call   =>
            Dump_Statement_Dbase (Outfile, Get_Subprogram_Info (Node));
            Dump_Statement_Dbase (Outfile, Get_Parameter_List (Node));

         when Pragma_Statement =>
            Dump_Statement_Dbase (Outfile, Get_Name (Node));
            Dump_Statement_Dbase (Outfile, Get_Argument_List (Node));

         when Null_Statement   | Unknown =>
            null;
      end case;

   end Dump_Statement_Node;


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Dump_Ref_Data_Storage
   --
   --  <Description>
   --
   --    Will dump out data_storage no.des
   --
   --==========================================================================
   procedure Dump_Ref_Data_Storage
       (Outfile    : File_Type;
        Data_Stack : Storage_Management.Ref_Data_Storage) is
      use Storage_Management;
      use Storage_Management_Routines;
   begin
      if (not (Is_Empty (Data_Stack))) then
         Put (Outfile, "|");
         Dump_Ref_Data_Storage (Outfile, Get_Next_Item (Data_Stack));
         Dump_Ref_Data_Storage (Outfile, Get_Next_Component (Data_Stack));

         case Get_Data_Type (Data_Stack) is
            when Integer_Data  =>
               Put (Outfile, Get_Integer_Value (Data_Stack));
            when Char_Data     =>
               Put (Outfile, Get_Character_Value (Data_Stack));
            when String_Data   =>
               Put (Outfile,
                    Literals.Convert_To_String (Get_String_Value (Data_Stack)));
            when others   =>
               null;
         end case;
         Put (Outfile, "|");
      end if;
   end Dump_Ref_Data_Storage;


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Dump_Expression_Node
   --
   --  <Description>
   --
   --    Will dump out statement_database expression nodes.
   --
   --==========================================================================
   procedure Dump_Expression_Node (Outfile : File_Type;
                                   Node    : Ref_Statement_Database) is
      use Node_Get_Routines;
   begin
      Dump_Ref_Data_Storage (Outfile, Get_Value (Node));
      Dump_Symbol_Table_Node (Outfile, Get_Type (Node));
      Put (Outfile, Integer (Boolean'POS (Is_Type_Universal (Node))));
      Put (Outfile, " ");
      Dump_Statement_Dbase (Outfile, Get_Holder (Node));
      Put (Outfile, Integer (Boolean'POS (Is_A_Default_Expression (Node))));
      Put (Outfile, " ");
      Put (Outfile, Get_Reference_Count (Node));
      Put (Outfile, " ");
      case Get_Expression_Kind (Node) is
         when Dot_Operator     =>
            Put
                (Integer
                     (Dot_Node_Enumeration'POS (
                         Node.Expression_Node.Dot_Node_Kind)));
            Dump_Statement_Dbase (Outfile, Get_Object (Node));
            Dump_Statement_Dbase (Outfile, Get_Dot_Component (Node));
            Dump_Symbol_Table_Node (Outfile, Get_Symbol_Table_Entry (Node));
            Dump_Statement_Dbase (Outfile, Get_Database_Node (Node));
         when Left_Parenthesis_Operator  =>
            Put
                (Integer
                     (Lp_Node_Enumeration'POS (
                         Node.Expression_Node.Lp_Node_Kind)));
            Put (Outfile, " ");
            Dump_Statement_Dbase (Outfile, Get_Parameter_List (Node));
         when List_Item   =>
            Dump_Statement_Dbase (Outfile, Get_Expression (Node));
            Dump_Statement_Dbase (Outfile, Get_Next_List_Item (Node));
         when Aggregate_Component   =>
            Dump_Statement_Dbase (Outfile, Get_Aggregate_Choice (Node));
            Dump_Statement_Dbase (Outfile, Get_Aggregate_Expression (Node));
         when Range_Expression =>
            Dump_Statement_Dbase (Outfile, Get_Name_Range (Node));
            Dump_Statement_Dbase (Outfile, Get_Lower_Bound (Node));
            Dump_Statement_Dbase (Outfile, Get_Upper_Bound (Node));
         when Attribute_Tick   =>
            Dump_Statement_Dbase (Outfile, Get_Attribute_Prefix (Node));
            Dump_Statement_Dbase (Outfile, Get_Attribute (Node));
         when String_Node =>
            Dump_Ref_String (Outfile, Get_Symbolic_Name (Node));
         when Character_Node   =>
            Dump_Ref_String (Outfile, Get_Symbolic_Name (Node));
         when Constant_Node    =>
            Dump_Ref_String (Outfile, Get_Symbolic_Name (Node));
         when Terminal_Node    =>
            Dump_Ref_String (Outfile, Get_Symbolic_Name (Node));
            Dump_Symbol_Table_Node (Outfile, Get_Symbol_Table_Entry (Node));
         when Array_Reference  | Function_Call     | Type_Conversion   |
         -- these next 2 are placeholders for "." operator
              Record_Reference | Dot_Notation_Variable_Reference  | Unknown =>
            null;

      end case;
   end Dump_Expression_Node;



   --==========================================================================
   --  <Subprogram Name>
   --
   --    Dump_Statement_Dbase
   --
   --  <Description>
   --
   --    Will dump out statement database nodes and keep track of them via
   --     a hash lookup table.
   --
   --==========================================================================
   procedure Dump_Statement_Dbase (Outfile   : File_Type;
                                   Curr_Node : Ref_Statement_Database) is
      use Sdb_Hash;
      Node_Num : Node_Id;
   begin
      if Is_Empty (Curr_Node) then
         Put (Outfile, " %0  ");
      else
         Node_Num := Save_Rec (Curr_Node, 0);
         if (Node_Num < 0) then
            Put (Outfile, " %");
            Put (Outfile, - Integer (Node_Num), 1);
            Put (Outfile, " ");
         else
            Put (Outfile, "[");
            Put
                (Integer
                     (Statement_Database_Enumeration'POS (Curr_Node.Node_Kind))
                 );
            case Curr_Node.Node_Kind is
               when Statement_Node_Type  =>
                  Dump_Statement_Node (Outfile, Curr_Node);
               when Expression_Node_Type =>
                  Dump_Expression_Node (Outfile, Curr_Node);
               when Unknown    =>
                  null;
            end case;
            Put (Outfile, "]");
         end if;
      end if;
   end Dump_Statement_Dbase;


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Dump_Subprogram_List
   --
   --  <Description>
   --
   --    Will recursively dump out subprogram list nodes.
   --
   --==========================================================================
   procedure Dump_Subprogram_List (Outfile  : File_Type;
                                   Sub_List : Ref_List_Of_Subprograms) is
   begin
      if (Sub_List = null) then
         Put (Outfile, "0");
      else
         Dump_Symbol_Table_Node (Outfile, Sub_List.Subprogram_Entry);
         Dump_Subprogram_List (Outfile, Sub_List.Next_Subprogram_Entry);
      end if;
   end Dump_Subprogram_List;


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Dump_Symbol_Table_Node
   --
   --  <Description>
   --
   --    Will dump out symbol table nodes and keep track of them via a hashed
   --  lookup table.
   --
   --==========================================================================
   procedure Dump_Symbol_Table_Node (Outfile : File_Type;
                                     Node    : Ref_Symbol_Table_Entry) is
      use Symtbl_Hash;
      Node_Num : Node_Id;
   begin
      if Is_Empty (Node) then
         Put (Outfile, " !0 ");
      else
         Node_Num := Save_Rec (Node, 0);
         if (Node_Num < 0) then
            Put (Outfile, " !");
            Put (Outfile, - Integer (Node_Num));
            Put (Outfile, " ");
         else
            Put (Outfile, "(");
            Put (Outfile,
                 Integer (Type_Entry_Enumeration'POS (Get_Entry_Kind (Node))));
            Put (Outfile, " ");
            Dump_Symbol_Table_Node (Outfile, Get_Next_Entry (Node));
            Dump_Ref_String (Outfile, Get_Symbolic_Name (Node));
            Put (Outfile, Integer (Get_Level (Node)));
            Put (Outfile, " ");
            Dump_Symbol_Table_Node (Outfile, Get_Enclosing_Package (Node));
            Put (Outfile,
                 Integer (Name_Visibility_Status'POS (Node.Visibility)));
            Put (Outfile, " ");
            Put (Outfile, Integer (Get_Statement_Number (Node)));
            Put (Outfile, " ");
            case Get_Entry_Kind (Node) is
               when Package_Entry   =>
                  Dump_Statement_Dbase (Outfile, Get_Context_Clause (Node));
                  Dump_Statement_Dbase
                      (Outfile, Get_Context_Clause_Body (Node));

                  Dump_Statement_Dbase (Outfile, Get_Package_Spec_Decls (Node));
                  Dump_Statement_Dbase (Outfile, Get_Package_Body_Decls (Node));
                  Put (Outfile, Integer (Boolean'POS (Has_A_Body (Node))));
                  Put (Outfile, " ");
                  Dump_Statement_Dbase (Outfile,
                                        Get_Head_Of_Statements_List (Node));
                  -- Dump_Help(Outfile, Node.Pkg_Help);
               when Procedure_Entry | Function_Entry    =>
                  Put (Outfile, Get_Parameter_Count (Node));
                  Put (Outfile, " ");
                  Put (Outfile, Get_Number_Of_Default_Parameters (Node));
                  Put (Outfile, " ");
                  Dump_Symbol_Table_Node (Outfile, Node.Head_Param_List);
                  Dump_Symbol_Table_Node (Outfile, Node.Tail_Param_List);
                  Dump_Symbol_Table_Node (Outfile, Node.Return_Type);
                  Dump_Statement_Dbase (Outfile, Get_Context_Clause (Node));
                  --dump_help(Outfile, Subprg_help);
                  Dump_Statement_Dbase (Outfile,
                                        Get_Head_Of_Declarations_List (Node));
                  Dump_Statement_Dbase (Outfile,
                                        Get_Head_Of_Statements_List (Node));
           --dump_statement_dbase(Outfile, get_Head_of_Exception_handler(Node));
                  Put (Outfile, Get_Value_Offset (Node));
                  Put (Outfile, " ");
                  Put (Outfile, Node.Is_Built_In_Subprogram);
                  Put (Outfile, " ");
                  Put (Outfile, Integer (Boolean'POS (Node.Has_Explicit_Decl)));
                  Put (Outfile, " ");
                  Dump_Symbol_Table_Node (Outfile,
                                          Node.Derived_Subprogram_Parent);
                  Dump_Symbol_Table_Node (Outfile, Node.Enum_Lit_Entry);
               when Exception_Entry =>
                  null;
               when Formal_Param_Entry   =>
                  Dump_Symbol_Table_Node (Outfile, Node.Param_Type);
                  Put (Outfile,
                       Integer (Mode_Enumeration'POS (Node.Param_Mode)));
                  Put (Outfile, " ");
                  Dump_Statement_Dbase
                      (Outfile, Get_Initialization_Code (Node));
                  Dump_Symbol_Table_Node (Outfile, Node.Next_Param);
                  Put (Outfile, Node.Param_Value_Offset);
                  Put (Outfile, " ");
                  Put (Outfile, Integer (Boolean'POS (Node.Preceded_By_Comma)));
                  Put (Outfile, " ");
               when Type_Entry =>
                  Put (Outfile,
                       Integer (Type_State_Attr'POS (Node.Concreteness)));
                  Put (Outfile, " ");
                  case Get_Type_Definition (Node) is
                     when Enumeration_Type    =>
                        Put (Outfile, Integer (Get_Literal_Count (Node)));
                        Put (Outfile, " ");
                        Dump_Symbol_Table_Node (Outfile,
                                                Get_First_Literal (Node));
                        Dump_Symbol_Table_Node (Outfile,
                                                Get_Last_Literal (Node));

                     when Integer_Type   =>
                        Put (Outfile, Get_Lower_Bound (Node));
                        Put (Outfile, " ");
                        Put (Outfile, Get_Upper_Bound (Node));

                     when Real_Type =>
                        null;

                     when Record_Type    =>
                        null;

                     when Access_Type    =>
                        null;

                     when Array_Type     =>
                        Put (Outfile,
                             Integer (Boolean'POS (Is_Constrained (Node))));
                        Put (Outfile, " ");
                        Dump_Statement_Dbase (Outfile, Get_Index_Type (Node));
                        Dump_Statement_Dbase (Outfile, Get_Element_Type (Node));
                  end case;


                  Dump_Subprogram_List (Outfile,
                                        Get_Derived_Subprogram_List (Node));
                  -- dump_help(Outfile, type_help);

               when Subtype_Entry   | Derived_Type_Entry     =>
                  Dump_Symbol_Table_Node (Outfile, Node.Base_Type);
                  -- dump_help (Outfile, other_type_help);
                  Dump_Symbol_Table_Node (Outfile, Node.Constraint_Info);
                  if (Get_Entry_Kind (Node) = Derived_Type_Entry) then
                     Dump_Subprogram_List (Outfile,
                                           Get_Derived_Subprogram_List (Node));
                  else
                     Dump_Subprogram_List (Outfile, null);
                  end if;
               when Enumeration_Literal_Entry =>
                  Put (Outfile, Node.Literal_Value);
                  Put (Outfile, " ");
                  Dump_Symbol_Table_Node (Outfile, Node.Enumeration_Type);
                  Dump_Symbol_Table_Node (Outfile, Node.Succ_Enum_Literal);
                  Dump_Symbol_Table_Node (Outfile, Node.Pred_Enum_Literal);
                  Dump_Symbol_Table_Node (Outfile, Node.Func_Entry);

               when Label_Entry     =>
                  Dump_Statement_Dbase (Outfile,
                                        Get_Head_Of_Statements_List (Node));
               when Record_Component_Entry    =>
                  Dump_Symbol_Table_Node (Outfile, Node.Component_Type);
                  Dump_Statement_Dbase
                      (Outfile, Get_Initialization_Code (Node));
                  Dump_Symbol_Table_Node (Outfile, Node.Next_Record_Component);
               when Simple_Name_Entry    =>
                  Dump_Statement_Dbase (Outfile,
                                        Get_Simple_Name_Construct (Node));
               when others     =>

                  null;
            end case;
            Put (Outfile, ") ");
         end if;
      end if;
   end Dump_Symbol_Table_Node;


   --==========================================================================
   --  <Subprogram Name>
   --
   --    Dump_Context
   --
   --  <Description>
   --
   --    Will dump out then context definition node. By dumping the top-most
   --  context node, the entire ACE environment will be dumped, theoretically
   --  at least.
   --
   --==========================================================================
   procedure Dump_Context (Outfile : File_Type;
                           Context : Context_Definition) is
      use Datastor_Hash;
      Dummy : Ref_Statement_Database;

   begin
      Put (Outfile, "{ ");

      if (Is_Empty (Context)) then
         Put (Outfile, "0}");
      else

         Put (Outfile, "1 ");
         Put (Outfile, Integer (Boolean'POS (Get_In_Interpret (Context))));
         Put (Outfile, " ");
         -- Dump_Top_Of_Symtbl_Stack (Outfile, Context);
         Dump_Top_Of_Symtbl_Save (Outfile, Context);
         Dump_Symbol_Table_Node
             (Outfile, Context_Control.Get_Top_Of_Symtbl_Save (Context));
         Dump_Symbol_Table_Node
             (Outfile, Get_Saved_Main_Level_Symbol_Table_Head (Context));
         Put (Outfile, " ");
         Put (Outfile, Get_Call_Level (Context));
         Put (Outfile, Integer (Get_Call_Level_Save (Context)));
         Dump_Statement_Dbase (Outfile, Get_Current_Statement (Context));
         Dump_Subprogram_Call_Stack (Outfile, Context);
         Dump_Symbol_Table_Node (Outfile, Get_Context_Clause_Holder (Context));
         Put (Outfile, " ");
         Put (Outfile, Get_Dregion_Count (Context));
         Put (Outfile, " ");
         Put (Outfile, Get_Dr_Count_Keep (Context));
         Dump_Dregion (Outfile, Context);
         Dump_Dregion_Entry (Outfile, Context);
         Put (Outfile, " ");
         Put (Outfile, Get_Current_Level (Context));
         Put (Outfile, " ");
         Put (Outfile, Get_Saved_Level (Context));
         Put (Outfile, " ");
         Put (Outfile, Get_Current_Offset (Context));
         Dump_Offset_Stack (Outfile, Context);
         Put (Outfile, " ");
         Put (Outfile, Get_Saved_Offset (Context));
         Dump_Saved_Offset_Stack (Outfile, Context);
         Put (Outfile, " ");
         Put (Outfile, Get_Last_Storage_Location_Pointer (Context));
         Put (Outfile, " ");
         Put (Outfile, Get_Threshold (Context));
         Dump_Dss (Outfile, Context);
         Dump_Dss_Save (Outfile, Context);
         Dump_Storage_Pointers_Stack (Outfile, Context);
         -- display_items_stack
         -- displays
         Dump_Context (Outfile, Get_Previous_Context (Context));
         Dump_Statement_Dbase (Outfile, Get_Enclosing_Scope (Context));
         -- valid returns
         -- hidden_entries
         Put (Outfile, "}");
      end if;
   end Dump_Context;



   --==========================================================================
   --  <Subprogram Name>
   --
   --    Dump_Symbol_Table
   --
   --  <Description>
   --
   --    An old routine subroutine used for debugging.. should now be removed
   --  and replaced with Dump_Context, since that is what it does anyway...
   --
   --==========================================================================

   procedure Dump_Symbol_Table (Outfile : File_Type;
                                Context : Context_Definition) is
      Curr_Node : Ref_Symbol_Table_Entry;
   begin
      Dump_Context (Outfile, Context);
      Curr_Node := Get_Top_Of_Symtbl (Context);
      -- just some diagnostic stuff to see how things went
      Symtbl_Hash.Check_Hash;
      Sdb_Hash.Check_Hash;

      -- clean up after ourselves...

      Sdb_Hash.Destroy_Table;
      Symtbl_Hash.Destroy_Table;
   end Dump_Symbol_Table;


begin
   Int_Io.Default_Width := 1;
end Reader_Writer;
