%w Parserdefs, Text_Io, Lexacts, Yyval_Error
%w Misc, Create_Statement_Database, Help_Pkg
%w Ace_Universal_Types
%w Error_Handler
%w Node_Create_Routines

-- 2.1

-- excluding the returns
SPACE_AND_FORMAT_EFFECTORS 	([ \t\f\013])
RETURN_FORMAT_EFFECTORS 	([\n\r])
WHITESPACE 			({SPACE_AND_FORMAT_EFFECTORS})
WHITESPACE_RETURNS 		({RETURN_FORMAT_EFFECTORS})
DIGIT 	 			([0-9])
LETTER 	 			([a-z])

-- 2.3

LETTER_OR_DIGIT 	 	({LETTER}|{DIGIT})
IDENTIFIER 	 		({LETTER}(_?{LETTER_OR_DIGIT})*)

-- 2.4.1

INTEGER 	 		({DIGIT}(_?{DIGIT})*)
EXPONENT 	 		(e[-+]?{INTEGER})
DECIMAL_LITERAL 	 	({INTEGER}([.]{INTEGER})?({EXPONENT})?)

-- 2.4.2

EXTENDED_DIGIT 	 		([0-9a-f])
BASED_INTEGER 	 		({EXTENDED_DIGIT}(_?{EXTENDED_DIGIT})*)
BASE 	 			({INTEGER})
BASED_LITERAL_POUND_1		({BASE}#{BASED_INTEGER}([.]{BASED_INTEGER})?#)
BASED_LITERAL_POUND 	 	({BASED_LITERAL_POUND_1}({EXPONENT})?)
BASED_LITERAL_COLON_1		({BASE}:{BASED_INTEGER}([.]{BASED_INTEGER})?:)
BASED_LITERAL_COLON 	 	({BASED_LITERAL_COLON_1}({EXPONENT})?)
BASED_LITERAL 	 		({BASED_LITERAL_POUND}|{BASED_LITERAL_COLON})

-- 2.4

NUMERIC_LITERAL 	 	({DECIMAL_LITERAL}|{BASED_LITERAL})

-- 2.5

CHARACTER_LITERAL 	 	(\'.\')

-- 2.6

NO_QUOTE 	 		([^"\n])
NO_QUOTE_OR_PERCENT 	 	([^"%\n])
DOUBLE_PERCENT 	 		([%][%])
DOUBLE_QUOTE 	 		(\"\")
STRING_LITERAL_1                (\"({NO_QUOTE}*({DOUBLE_QUOTE})*)*\")
STRING_LITERAL_2		(%({NO_QUOTE_OR_PERCENT}*({DOUBLE_PERCENT})*)*%)
STRING_LITERAL 	 		({STRING_LITERAL_1}|{STRING_LITERAL_2})

-- 2.7

COMMENT 	 		("--".*)
HELP_COMMENT 	 		("--?".*)

%{
Id_String : STRING(1..200);            -- identifier being processed
Char_Lit : STRING(1..200);             -- character or string literal
Num_Lit : STRING(1..200);              -- numeric literal
%}

%e      1500  -- tree nodes array size
%p      4500  -- positions
%a      4000  -- transitions array size
%k      1000  -- packed character classes
%o      6000  -- output array size

%START IDENT Z Z1

-- 2.9 and 2.2
%%
abort           {NULL; START(Z); return(ABORT_TK);}
abs             {NULL; START(Z); return(ABS_TK);}
accept          {NULL; START(Z); return(ACCEPT_TK);}
access          {NULL; START(Z); return(ACCESS_TK);}
all             {NULL; START(IDENT); return(ALL_TK);}
and             {NULL; START(Z); return(AND_TK);}
array           {NULL; START(Z); return(ARRAY_TK);}
at              {NULL; START(Z); return(AT_TK);}
begin           {NULL; START(Z); return(BEGIN_TK);}
body            {NULL; START(Z); return(BODY_TK);}
case            {NULL; START(Z); return(CASE_TK);}
constant        {NULL; START(Z); return(CONSTANT_TK);}
declare         {NULL; START(Z); return(DECLARE_TK);}
delay           {NULL; START(Z); return(DELAY_TK);}
<Z,IDENT>delta           {NULL; START(Z); return(DELTA_TK);}
<Z,IDENT>digits          {NULL; START(Z); return(DIGITS_TK);}
do              {NULL; START(Z); return(DO_TK);}
else            {NULL; START(Z); return(ELSE_TK);}
elsif           {NULL; START(Z); return(ELSIF_TK);}
end             {NULL; START(Z); return(END_TK);}
entry           {NULL; START(Z); return(ENTRY_TK);}
exception       {NULL; START(Z); return(EXCEPTION_TK);}
exit            {NULL; START(Z); return(EXIT_TK);}
for             {NULL; START(Z); return(FOR_TK);}
function        {NULL; START(Z); return(FUNCTION_TK);}
generic         {NULL; START(Z); return(GENERIC_TK);}
goto            {NULL; START(Z); return(GOTO_TK);}
if              {NULL; START(Z); return(IF_TK);}
in              {NULL; START(Z); return(IN_TK);}
is              {NULL; START(Z); return(IS_TK);}
limited         {NULL; START(Z); return(LIMITED_TK);}
loop            {NULL; START(Z); return(LOOP_TK);}
mod             {NULL; START(Z); return(MOD_TK);}
new             {NULL; START(Z); return(NEW_TK);}
not             {NULL; START(Z); return(NOT_TK);}
null            {NULL; START(Z); return(null_TK);}
of              {NULL; START(Z); return(OF_TK);}
or              {NULL; START(Z); return(OR_TK);}
others          {NULL; START(Z); return(OTHERS_TK);}
out             {NULL; START(Z); return(OUT_TK);}
package         {NULL; START(Z); return(PACKAGE_TK);}
pragma          {NULL; START(Z); return(PRAGMA_TK);}
private         {NULL; START(Z); return(PRIVATE_TK);}
procedure       {NULL; START(Z); return(PROCEDURE_TK);}
raise           {NULL; START(Z); return(RAISE_TK);}
<Z,IDENT>range  {NULL; START(Z); return(RANGE_TK);}
record          {NULL; START(Z); return(RECORD_TK);}
rem             {NULL; START(Z); return(REM_TK);}
renames         {NULL; START(Z); return(RENAMES_TK);}
return          {NULL; START(Z); return(RETURN_TK);}
reverse         {NULL; START(Z); return(REVERSE_TK);}
select          {NULL; START(Z); return(SELECT_TK);}
separate        {NULL; START(Z); return(SEPARATE_TK);}
subtype         {NULL; START(Z); return(SUBTYPE_TK);}
task            {NULL; START(Z); return(TASK_TK);}
terminate       {NULL; START(Z); return(TERMINATE_TK);}
then            {NULL; START(Z); return(THEN_TK);}
type            {NULL; START(Z); return(TYPE_TK);}
use             {NULL; START(Z); return(USE_TK);}
when            {NULL; START(Z); return(WHEN_TK);}
while           {NULL; START(Z); return(WHILE_TK);}
with            {NULL; START(Z); return(WITH_TK);}
xor             {NULL; START(Z); return(XOR_TK);}

"=>"            {NULL; START(Z); return(ARROW_TK);}
".."            {NULL; START(Z); return(DBLDOT_TK);}
"**"            {NULL; START(Z); return(EXP_TK);}
":="            {NULL; START(Z); return(ASSIGN_TK);}
"/="            {NULL; START(Z); return(NOTEQL_TK);}
">="            {NULL; START(Z); return(GTEQL_TK);}
"<="            {NULL; START(Z); return(LTEQ_TK);}
"<<"            {NULL; START(Z); return(L_LBL_TK);}
">>"            {NULL; START(Z); return(R_LBL_TK);}
"<>"            {NULL; START(Z); return(BOX_TK);}
"&"             {NULL; START(Z); return(Character'Pos('&')); }
"("             {NULL; START(Z); return(Character'Pos('(')); }
")"             {NULL; START(IDENT); return(Character'Pos(')')); }
"*"             {NULL; START(Z); return(Character'Pos('*')); }
"+"             {NULL; START(Z); return(Character'Pos('+')); }
","             {NULL; START(Z); return(Character'Pos(',')); }
"-"             {NULL; START(Z); return(Character'Pos('-')); }
"."             {NULL; START(Z); return(Character'Pos('.')); }
"/"             {NULL; START(Z); return(Character'Pos('/')); }
":"             {NULL; START(Z); return(Character'Pos(':')); }
";"             {NULL; START(Z); return(Character'Pos(';')); }
"<"             {NULL; START(Z); return(Character'Pos('<')); }
"="             {NULL; START(Z); return(Character'Pos('=')); }
">"             {NULL; START(Z); return(Character'Pos('>')); }

[|!]            {NULL; START(Z); return(Character'Pos('|')); }

<IDENT>\'       {NULL; START(Z1); return(Character'Pos('''));}


{IDENTIFIER}       {
                    NULL;
                    START(IDENT);
                    Lval := Create_Expr_Node (Yytext, Yyleng);
                    return (Identifier);
                   }

{NUMERIC_LITERAL}	{
                         NULL;
                         START(Z);
                         Num_Lit := Yytext;
                         Lval := Create_Constant_Node (Yytext, Yyleng);
                         return(Numeric_Literal);
                        }

{STRING_LITERAL}	{
                          NULL;
                          START(Z);
                          declare
                            Num_Quotes : Natural := 0;
                          begin
                            for I in 2 .. Yyleng-1 loop
                              if Yytext(I) = '"' then
                                 Yytext(I+1..Yyleng-1) :=
                                   Yytext(I+2..Yyleng-1) & ' ';
                                 Num_Quotes := Num_Quotes + 1;
                              end if;
                            end loop;
                            Lval := Create_String_Node( Yytext(2 .. Yyleng-1),
                              Yyleng-2-Num_Quotes);
                          end;
                          return(String_Literal);
                        }

<Z,Z1>{CHARACTER_LITERAL}	{
               NULL;
               START(IDENT);
               Lval := Create_Character_Node (Yytext (2 .. Yyleng-1), Yyleng-2);
               return(Character_Literal);
               }


{WHITESPACE_RETURNS}	{
                          NULL;
                          if Lexacts.Eol_Flag = 0 then
                            Yyline_Number := 0;
                            return (0);
                          else
                            Prompt(Continuation_Prompt);
                            Yyline_Number := Yyline_Number +1;
                          end if;
                        }

{WHITESPACE}	NULL;

{HELP_COMMENT}	{ NULL; Process_Help (Yytext(1 .. Yyleng)); }


{COMMENT}	{ NULL; Process_Comments (Yytext(1 .. Yyleng)); }

.	{ NULL;
          Show_Error (3011, Fatal, Yytext(1..Yyleng));
          raise Ace_Universal_Types.Scanner_Error; }
