--|$Log:	set.a,v $
--|Revision 8.0  91/01/17  10:46:19  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Node_Set_Routines
--
---------------------------------------------------------------
--
--
-- Abstract
--
--    These routines are statment database node SET routines
--    for each node/component kind of the statement database.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------
-- ==========================================================================

with Statement_Database;
use Statement_Database;

with Symbol_Table_Structures;
use Symbol_Table_Structures;

with Misc_Support_Routines;
use Misc_Support_Routines;

with Node_Create_Routines;
use Node_Create_Routines;

with Symtbl_Entry_Get_Routines;
use Symtbl_Entry_Get_Routines;

with Ace_Universal_Types;
use Ace_Universal_Types;

with Error_Handler;
use Error_Handler;

with Storage_Management_Routines;
use Storage_Management_Routines;

package body Node_Set_Routines is

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_Database_Node
   --
   --    Last Mod:    Thu Sep  7 09:49:04 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --    Set the real statement database node for an expanded name.
   --
   --  --------------------------------------------------------------------
   procedure Set_Database_Node (Node : in Ref_Statement_Database;
                                Item : in Ref_Statement_Database) is
   begin
      Node.Expression_Node.Expn_Terminal_Sdb := Item;
   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Database_Node");
   end Set_Database_Node;

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_Next_Statement
   --
   --    Last Mod:    Thu Sep  7 09:51:03 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Set_Next_Statement (Node : in Ref_Statement_Database;
                                 Item : in Ref_Statement_Database) is
      -- Next_Statement
   begin
      Node.Statement_Node.Next_Statement := Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Next_Statement");

   end Set_Next_Statement;

   -- -------------------------------------------------------------------------

   --  --------------------------------------------------------------------
   --  <Subprogram Name>
   --    Set_Holder
   --
   --    Last Mod:    Thu Sep  7 09:51:50 1989
   --    Author:      William P Loftus <wpl@huey>
   --
   --  <Description>
   --
   --  --------------------------------------------------------------------
   procedure Set_Holder (Node : in Ref_Statement_Database;
                         Item : in Ref_Statement_Database) is
      -- Holder
      -- Next_Expression

   begin
      case Node.Node_Kind is
         when Statement_Node_Type   =>
            Node.Statement_Node.Holder := Item;

         when Expression_Node_Type  =>
            Node.Expression_Node.Next_Expression := Item;

         when others =>
            Show_Error (2000, Internal, "Set_Holder1");

      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Holder");

   end Set_Holder;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    UnSet_Suspend_Evaluation_At_This_Statement
   --
   --  <Description>
   --
   --    Sets that evaluation should be suspended at current Node
   --
   --=======================================================================
   procedure Unset_Suspend_Evaluation_At_This_Statement
       (Node : in Ref_Statement_Database) is
      -- Suspend_Evaluation_At_This_Statement
   begin
      Node.Statement_Node.Suspend_Evaluation_At_This_Statement := False;
   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Suspend_Evaluation_At_This_Statement");
   end Unset_Suspend_Evaluation_At_This_Statement;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Set_Suspend_Evaluation_At_This_Statement
   --
   --  <Description>
   --
   --    Sets that evaluation should be suspended at current Node
   --
   --=======================================================================
   procedure Set_Suspend_Evaluation_At_This_Statement
       (Node : in Ref_Statement_Database) is
      -- Suspend_Evaluation_At_This_Statement
   begin
      Node.Statement_Node.Suspend_Evaluation_At_This_Statement := True;
   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Suspend_Evaluation_At_This_Statement");
   end Set_Suspend_Evaluation_At_This_Statement;

   -- -------------------------------------------------------------------------

   procedure Set_Statement_Number (Node : in Ref_Statement_Database;
                                   Item : in Ace_Natural) is
      -- Statement_Number
   begin
      Node.Statement_Node.Statement_Number := Item;
   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Statement_Number");
   end Set_Statement_Number;

   -- -------------------------------------------------------------------------

   procedure Set_Subprogram_Info (Node : in Ref_Statement_Database;
                                  Item : in Ref_Statement_Database) is
      -- Procedure_Info
      -- Lp_Object (for function calls)

   begin
      case Node.Node_Kind is
         when Statement_Node_Type   =>
            case Node.Statement_Node.Statement_Kind is
               when Procedure_Call  =>
                  Node.Statement_Node.Procedure_Info := Item;
               when others     =>
                  Show_Error (2000, Internal, "Set_Subprogram_Info1");
            end case;

         when Expression_Node_Type  =>
            case Node.Expression_Node.Expression_Kind is
               when Left_Parenthesis_Operator =>
                  Node.Expression_Node.Lp_Object := Item;
               when others     =>
                  Show_Error (2000, Internal, "Set_Subprogram_Info2");
            end case;

         when others =>
            Show_Error (2000, Internal, "Set_Subprogram_Info3");

      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Subprogram_Info");

   end Set_Subprogram_Info;

   -- -------------------------------------------------------------------------

   procedure Set_Symbol_Table_Entry (Node : in Ref_Statement_Database;
                                     Item : in Ref_Symbol_Table_Entry) is
      -- ExpN_Terminal_Symbol
      -- Symbol_Table_Entry
      -- Terminal_Symbol

   begin
      case Node.Node_Kind is
         when Statement_Node_Type   =>
            case Node.Statement_Node.Statement_Kind is
               when Exception_Declaration      |
                    Type_Declaration           |
                    Subtype_Declaration        |
                    Procedure_Spec_Declaration |
                    Procedure_Declaration      |
                    Function_Spec_Declaration  |
                    Function_Declaration       |
                    Package_Declaration        |
                    Package_Body_Declaration  =>
                  Node.Statement_Node.Symbol_Table_Entry := Item;
               when others     =>
                  Show_Error (2000, Internal, "Set_Symbol_Table_Entry1");
            end case;

         when Expression_Node_Type  =>
            case Node.Expression_Node.Expression_Kind is
               when Dot_Operator    =>
                  Node.Expression_Node.Expn_Terminal_Symbol := Item;
               when Terminal_Node   =>
                  Node.Expression_Node.Terminal_Symbol := Item;
               when others     =>
                  Show_Error (2000, Internal, "Set_Symbol_Table_Entry2");
            end case;

         when others =>
            Show_Error (2000, Internal, "Set_Symbol_Table_Entry3");

      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Symbol_Table_Entry");

   end Set_Symbol_Table_Entry;

   -- -------------------------------------------------------------------------

   procedure Set_Lhs (Node : in Ref_Statement_Database;
                      Item : in Ref_Statement_Database) is
      -- Lhs

   begin
      Node.Statement_Node.Lhs := Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Lhs");

   end Set_Lhs;

   -- -------------------------------------------------------------------------

   procedure Set_Rhs (Node : in Ref_Statement_Database;
                      Item : in Ref_Statement_Database) is
      -- Rhs

   begin
      Node.Statement_Node.Rhs := Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Rhs");

   end Set_Rhs;

   -- -------------------------------------------------------------------------

   procedure Set_Condition (Node : in Ref_Statement_Database;
                            Item : in Ref_Statement_Database) is
      -- If_Condition
      -- Exit_Condition

   begin
      case Node.Statement_Node.Statement_Kind is
         when If_Statement     =>
            Node.Statement_Node.If_Condition := Item;

         when Exit_Statement   =>
            Node.Statement_Node.Exit_Condition := Item;

         when others =>
            Show_Error (2000, Internal, "Set_Condition1");

      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Condition");

   end Set_Condition;

   -- -------------------------------------------------------------------------

   procedure Set_Loop_Scheme (Node : in Ref_Statement_Database;
                              Item : in Ref_Statement_Database) is
      -- Loop_Scheme

   begin
      Node.Statement_Node.Loop_Scheme := Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Loop_Scheme");

   end Set_Loop_Scheme;

   -- -------------------------------------------------------------------------

   procedure Set_For_Variable (Node : in Ref_Statement_Database;
                               Item : in Ref_Statement_Database) is
      -- For_Variable

   begin
      Node.Statement_Node.For_Variable := Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_For_Variable");

   end Set_For_Variable;

   -- -------------------------------------------------------------------------

   procedure Set_For_Range (Node : in Ref_Statement_Database;
                            Item : in Ref_Statement_Database) is
      -- For_Range

   begin
      Node.Statement_Node.For_Range := Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_For_Range");

   end Set_For_Range;

   -- -------------------------------------------------------------------------

   procedure Set_Name_Range (Node : in Ref_Statement_Database;
                             Item : in Ref_Statement_Database) is
      -- Range_Name

   begin
      Node.Expression_Node.Range_Name := Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Name_Range");

   end Set_Name_Range;

   -- -------------------------------------------------------------------------

   procedure Set_Head_Of_Statements_List (Node : in Ref_Statement_Database;
                                          Item : in Ref_Statement_Database) is
      -- Loop_Body
      -- Block_Statements

   begin
      case Node.Statement_Node.Statement_Kind is
         when Loop_Statement   =>
            Node.Statement_Node.Loop_Body := Item;

         when Block_Statement  =>
            Node.Statement_Node.Block_Declarations := Item;

         when others =>
            Show_Error (2000, Internal, "Set_Head_Of_Statements_List1");

      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Head_Of_Statements_List");

   end Set_Head_Of_Statements_List;

   -- -------------------------------------------------------------------------

   procedure Set_Head_Of_True_Task_Statements_List
       (Node : in Ref_Statement_Database;
        Item : in Ref_Statement_Database) is
      -- True_Task

   begin
      Node.Statement_Node.True_Task := Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Head_Of_True_Task_Statements_List");

   end Set_Head_Of_True_Task_Statements_List;

   -- -------------------------------------------------------------------------

   procedure Set_Head_Of_False_Task_Statements_List
       (Node : in Ref_Statement_Database;
        Item : in Ref_Statement_Database) is
      -- False_Task

   begin
      Node.Statement_Node.False_Task := Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Head_Of_False_Task_Statements_List");

   end Set_Head_Of_False_Task_Statements_List;

   -- -------------------------------------------------------------------------

   procedure Set_Expression (Node : in Ref_Statement_Database;
                             Item : in Ref_Statement_Database) is
      -- Init_Expression
      -- Case_Expression
      -- Return_Expression
      -- Item_Expression

   begin
      case Node.Node_Kind is
         when Statement_Node_Type   =>
            case Node.Statement_Node.Statement_Kind is
               when Object_Declaration   =>
                  Node.Statement_Node.Init_Expression := Item;

               when Case_Statement  =>
                  Node.Statement_Node.Case_Expression := Item;

               when Return_Statement     =>
                  Node.Statement_Node.Return_Expression := Item;

               when others     =>
                  Show_Error (2000, Internal, "Set_Expression1");
            end case;

         when Expression_Node_Type  =>
            case Node.Expression_Node.Expression_Kind is
               when List_Item  =>
                  Node.Expression_Node.Item_Expression := Item;
               when others     =>
                  Show_Error (2000, Internal, "Set_Expression2");
            end case;

         when others =>
            Show_Error (2000, Internal, "Set_Expression3");

      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Expression");

   end Set_Expression;

   -- -------------------------------------------------------------------------

   procedure Set_Subprogram_Reference (Node : in Ref_Statement_Database;
                                       Item : Ref_Symbol_Table_Entry) is
      -- Subprogram_Reference

   begin
      Node.Statement_Node.Subprogram_Reference := Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "set_subprogram_reference");

   end Set_Subprogram_Reference;

   -- -------------------------------------------------------------------------

   procedure Set_Alternatives_List (Node : in Ref_Statement_Database;
                                    Item : in Ref_Statement_Database) is
      -- Case_Alternatives_List

   begin
      Node.Statement_Node.Case_Alternatives_List := Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Alternatives_List");

   end Set_Alternatives_List;

   -- -------------------------------------------------------------------------

   procedure Set_Alternative_Choice (Node : in Ref_Statement_Database;
                                     Item : in Ref_Statement_Database) is
      -- Case_Alternative_Choice

   begin
      case Node.Statement_Node.Statement_Kind is
         when Case_Alternative_Part =>
            Node.Statement_Node.Case_Alternative_Choice := Item;
         when Exception_Alternative =>
            Node.Statement_Node.Exception_Alternative_Choice := Item;
         when others =>
            Show_Error (2000, Internal, "Set_Alternative_Choice 1");
      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Alternative_Choice 2");

   end Set_Alternative_Choice;

   -- -------------------------------------------------------------------------

   procedure Set_Alternative_Statements (Node : in Ref_Statement_Database;
                                         Item : in Ref_Statement_Database) is
      -- Case_Alternative_Statements

   begin
      case Node.Statement_Node.Statement_Kind is
         when Case_Alternative_Part =>
            Node.Statement_Node.Case_Alternative_Statements := Item;
         when Exception_Alternative =>
            Node.Statement_Node.Exception_Alternative_Statements := Item;
         when others =>
            Show_Error (2000, Internal, "Set_Alternative_Statements 1");
      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Alternative_Statements 2");

   end Set_Alternative_Statements;

   -- -------------------------------------------------------------------------

   procedure Set_Next_Alternative (Node : in Ref_Statement_Database;
                                   Item : in Ref_Statement_Database) is
      -- Next_Case_Alternative

   begin
      case Node.Statement_Node.Statement_Kind is
         when Case_Alternative_Part =>
            Node.Statement_Node.Next_Case_Alternative := Item;
         when Exception_Alternative =>
            Node.Statement_Node.Next_Exception_Alternative := Item;
         when others =>
            Show_Error (2000, Internal, "Set_Next_Alternative 1");
      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Next_Alternative 2");

   end Set_Next_Alternative;

   -- -------------------------------------------------------------------------

   procedure Set_Label_Entry (Node : in Ref_Statement_Database;
                              Item : in Ref_Statement_Database) is
      -- Label_Entry

   begin
      Node.Statement_Node.Label_Entry := Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Label_Entry");

   end Set_Label_Entry;

   -- -------------------------------------------------------------------------

   procedure Set_Goto_Label (Node : in Ref_Statement_Database;
                             Item : in Ref_Statement_Database) is
      -- Goto_Label

   begin
      Node.Statement_Node.Goto_Label := Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Goto_Label");

   end Set_Goto_Label;

   -- -------------------------------------------------------------------------

   procedure Set_Name (Node : in Ref_Statement_Database;
                       Item : in Ref_Statement_Database) is
      -- Loop_Name
      -- Exit_Loop_Name
      -- Pragma_Name

   begin
      case Node.Statement_Node.Statement_Kind is
         when Loop_Statement   =>
            Node.Statement_Node.Loop_Name := Item;

         when Exit_Statement   =>
            Node.Statement_Node.Exit_Loop_Name := Item;

         when Pragma_Statement =>
            Node.Statement_Node.Pragma_Name := Item;

         when others =>
            Show_Error (2000, Internal, "Set_Name1");
      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Name");

   end Set_Name;

   -- -------------------------------------------------------------------------

   procedure Set_Block_Declarations (Node : in Ref_Statement_Database;
                                     Item : in Ref_Statement_Database) is
      -- Block_Declarations

   begin
      Node.Statement_Node.Block_Declarations := Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Block_Declarations");

   end Set_Block_Declarations;

   -- -------------------------------------------------------------------------
   --  <Subprogram Name>
   --
   --    Set_Exception_Handler
   --
   --  <Description>
   --
   --    Returns the Block Declarations for the current Node
   --
   --=======================================================================
   procedure Set_Exception_Handler (Node : in Ref_Statement_Database;
                                    Item : in Ref_Statement_Database) is
      -- Exception_Handler

   begin
      Node.Statement_Node.Block_Exceptions := Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Exception_Handler");
         raise ;
   end Set_Exception_Handler;

   procedure Set_Name_List (Node : in Ref_Statement_Database;
                            Item : in Ref_Statement_Database) is
      -- Name_List
      -- Unit_Name_List

   begin
      case Node.Statement_Node.Statement_Kind is
         when Object_Declaration    =>
            Node.Statement_Node.Name_List := Item;

         when Exception_Declaration =>
            Node.Statement_Node.Exception_Name_List := Item;

         when With_Statement   | Use_Statement     =>
            Node.Statement_Node.Unit_Name_List := Item;

         when others =>
            Show_Error (2000, Internal, "Set_Name_List1");

      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Name_List");

   end Set_Name_List;

   -- -------------------------------------------------------------------------

   procedure Set_Parameter_List (Node : in Ref_Statement_Database;
                                 Item : in Ref_Statement_Database) is
      -- Actual_Procedure_Parameters_List
      -- Lp_Items_List (for function calls)

   begin
      case Node.Node_Kind is
         when Statement_Node_Type   =>
            case Node.Statement_Node.Statement_Kind is
               when Procedure_Call  =>
                  Node.Statement_Node.Actual_Procedure_Parameters_List := Item;
               when others     =>
                  Show_Error (2000, Internal, "Set_Parameter_List1");
            end case;

         when Expression_Node_Type  =>
            case Node.Expression_Node.Expression_Kind is
               when Left_Parenthesis_Operator =>
                  Node.Expression_Node.Lp_Items_List := Item;
               when others     =>
                  Show_Error (2000, Internal, "Set_Parameter_List2");
            end case;

         when others =>
            Show_Error (2000, Internal, "Set_Parameter_List3");

      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Parameter_List");

   end Set_Parameter_List;

   -- -------------------------------------------------------------------------

   procedure Set_Argument_List (Node : in Ref_Statement_Database;
                                Item : in Ref_Statement_Database) is
      -- Pragma_Arg

   begin
      Node.Statement_Node.Pragma_Arg := Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Pragma_Arg");

   end Set_Argument_List;

   -- -------------------------------------------------------------------------

   procedure Set_Value (Node : in Ref_Statement_Database;
                        Item : in Ref_Data_Storage) is
      -- Node_Value

   begin
      -- Free (Node.Expression_Node.Node_Value);
      Node.Expression_Node.Node_Value := (Item);

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Value");
   end Set_Value;

   -- -------------------------------------------------------------------------

   procedure Set_Type (Node : in Ref_Statement_Database;
                       Item : in Ref_Symbol_Table_Entry) is
      -- Node_Type

   begin
      Node.Expression_Node.Node_Type := Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Type");
   end Set_Type;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Set_Is_A_Default_Expression
   --
   --  <Description>
   --
   --    Marks the given NODE as being a default expression
   --
   --=======================================================================
   procedure Set_Is_A_Default_Expression (Node : in Ref_Statement_Database) is
      -- Is_Default_Expression
   begin
      Node.Expression_Node.Is_Default_Expression := True;
   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Is_A_Default_Expression");
   end Set_Is_A_Default_Expression;

   -- -------------------------------------------------------------------------

   procedure Set_As_Universal_Type (Node : in Ref_Statement_Database) is
      -- Is_Node_Type_Universal

   begin
      Node.Expression_Node.Is_Node_Type_Universal := True;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_As_Universal_Type");
   end Set_As_Universal_Type;

   -- -------------------------------------------------------------------------

   procedure Set_Symbolic_Name (Node : in Ref_Statement_Database;
                                Item : in Ref_String) is
      -- Symbol_Name
      -- Character_Symbol
      -- String_Symbol
      -- Constant_Symbol

   begin
      case Node.Expression_Node.Expression_Kind is
         when Terminal_Node    =>
            Node.Expression_Node.Symbol_Name := Item;
         when String_Node =>
            Node.Expression_Node.String_Symbol := Item;
         when Character_Node   =>
            Node.Expression_Node.Character_Symbol :=
               Convert_To_String (Item) (1);
         when Constant_Node    =>
            Node.Expression_Node.Constant_Symbol := Item;

         when others =>
            Show_Error (2000, Internal, "Set_Symbolic_Name1");
      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Symbolic_Name");

   end Set_Symbolic_Name;

   -- -------------------------------------------------------------------------

   procedure Set_Node_Kind (Node : in Ref_Statement_Database;
                            Item : in Expression_Enumeration) is
      -- Dot_Node_Kind
      -- Lp_Node_Kind

   begin
      case Node.Expression_Node.Expression_Kind is
         when Dot_Operator     =>
            case Item is
               when Record_Reference     =>
                  Node.Expression_Node.Dot_Node_Kind := Record_Reference;
               when Dot_Notation_Variable_Reference     =>
                  Node.Expression_Node.Dot_Node_Kind :=
                     Dot_Notation_Variable_Reference;
               when Unknown    =>
                  Node.Expression_Node.Dot_Node_Kind := Unknown;
               when others     =>
                  Show_Error (2000, Internal, "Set_Node_Kind1");
            end case;

         when Left_Parenthesis_Operator  =>
            case Item is
               when Array_Reference =>
                  Node.Expression_Node.Lp_Node_Kind := Array_Reference;
               when Function_Call   =>
                  Node.Expression_Node.Lp_Node_Kind := Function_Call;
               when Type_Conversion =>
                  Node.Expression_Node.Lp_Node_Kind := Type_Conversion;
               when Unknown    =>
                  Node.Expression_Node.Lp_Node_Kind := Unknown;
               when others     =>
                  Show_Error (2000, Internal, "Set_Node_Kind2");
            end case;

         when Terminal_Node    =>
            case Item is
               when Function_Call   =>
                  declare
                     Function_Node : Ref_Statement_Database;
                  begin
                     Function_Node := Create_Node (Left_Parenthesis_Operator);
                     Set_Node_Kind (Function_Node, Function_Call);
                     Set_Type (Function_Node, Node.Expression_Node.Node_Type);
                     Set_Value (Function_Node, Node.Expression_Node.Node_Value);
                     Set_Subprogram_Info (Function_Node, Duplicate_Node (Node));
                     Node.Expression_Node := Function_Node.Expression_Node;
                  end;

               when others     =>
                  Show_Error (2000, Internal, "Set_Node_Kind3");
            end case;

         when others =>
            Show_Error (2000, Internal, "Set_Node_Kind4");
      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Node_Kind");

   end Set_Node_Kind;

   -- -------------------------------------------------------------------------

   procedure Set_Object (Node : in Ref_Statement_Database;
                         Item : in Ref_Statement_Database) is
      -- Dot_Object
      -- Lp_Object

   begin
      case Node.Expression_Node.Expression_Kind is
         when Dot_Operator     =>
            Node.Expression_Node.Dot_Object := Item;

         when Left_Parenthesis_Operator  =>
            Node.Expression_Node.Lp_Object := Item;

         when others =>
            Show_Error (2000, Internal, "Set_Object1");

      end case;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Object");

   end Set_Object;

   -- -------------------------------------------------------------------------

   procedure Set_Dot_Component (Node : in Ref_Statement_Database;
                                Item : in Ref_Statement_Database) is
      -- Dot_Component

   begin
      Node.Expression_Node.Dot_Component := Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Dot_Component");

   end Set_Dot_Component;

   -- -------------------------------------------------------------------------

   procedure Set_List_Of_Items (Node : in Ref_Statement_Database;
                                Item : in Ref_Statement_Database) is
      -- Lp_Items_List

   begin
      Node.Expression_Node.Lp_Items_List := Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_List_Of_Items");

   end Set_List_Of_Items;

   -- -------------------------------------------------------------------------

   procedure Set_Next_List_Item (Node : in Ref_Statement_Database;
                                 Item : in Ref_Statement_Database) is
      -- Next_List_Item

   begin
      Node.Expression_Node.Next_List_Item := Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Next_List_Item");

   end Set_Next_List_Item;

   -- -------------------------------------------------------------------------

   procedure Set_Aggregate_Choice (Node : in Ref_Statement_Database;
                                   Item : in Ref_Statement_Database) is
      -- Aggregate_Choice

   begin
      Node.Expression_Node.Aggregate_Choice := Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Aggregate_Choice");

   end Set_Aggregate_Choice;

   -- -------------------------------------------------------------------------

   procedure Set_Aggregate_Expression (Node : in Ref_Statement_Database;
                                       Item : in Ref_Statement_Database) is
      -- Aggregate_Expression

   begin
      Node.Expression_Node.Aggregate_Expression := Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Aggregate_Expression");

   end Set_Aggregate_Expression;

   -- -------------------------------------------------------------------------

   procedure Set_Lower_Bound (Node : in Ref_Statement_Database;
                              Item : in Ref_Statement_Database) is
      -- Lower_Bound

   begin
      Node.Expression_Node.Lower_Bound := Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Lower_Bound");

   end Set_Lower_Bound;

   -- -------------------------------------------------------------------------

   procedure Set_Upper_Bound (Node : in Ref_Statement_Database;
                              Item : in Ref_Statement_Database) is
      -- Upper_Bound

   begin
      Node.Expression_Node.Upper_Bound := Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Upper_Bound");

   end Set_Upper_Bound;

   -- -------------------------------------------------------------------------

   procedure Set_Attribute_Prefix (Node : in Ref_Statement_Database;
                                   Item : in Ref_Statement_Database) is
      -- Attribute_Prefix

   begin
      Node.Expression_Node.Attribute_Prefix := Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Attribute_Prefix");

   end Set_Attribute_Prefix;

   -- -------------------------------------------------------------------------

   procedure Set_Attribute (Node : in Ref_Statement_Database;
                            Item : in Ref_Statement_Database) is
      -- Attribute

   begin
      Node.Expression_Node.Attribute := Item;

   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Attribute");

   end Set_Attribute;

   -- -------------------------------------------------------------------------

   -----

   procedure Set_Parent_Node (Node : in Ref_Statement_Database;
                              Item : in Ref_Statement_Database) is
      -- Parent_Node
   begin
      Node.Parent_Node := Item;
   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Set_Parent_Node");
   end Set_Parent_Node;

   --=======================================================================
   --  <Subprogram Name>
   --
   --    Increment_Reference_Count
   --
   --  <Description>
   --
   --
   --=======================================================================
   procedure Increment_Reference_Count (Node : in Ref_Statement_Database) is
      -- Reference_Count
   begin
      Node.Expression_Node.Reference_Count :=
         Node.Expression_Node.Reference_Count + 1;
   exception
      when Constraint_Error    =>
         Show_Error (2000, Internal, "Increment_Reference_Count");
   end Increment_Reference_Count;

end Node_Set_Routines;
