--|$Log:	stmtev.a,v $
--|Revision 8.0  91/01/17  10:46:28  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Program Name - Statement_Evaluator
--
-- Package Name -
--
---------------------------------------------------------------
--
--
-- Abstract
--
--   The statement evaluator for the ACE ACLI.
--   Evaluates Ada statements, declarations and expressions.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------

with Statement_Database;
use Statement_Database;

with Symbol_Table_Structures;
use Symbol_Table_Structures;

with Storage_Management;
use Storage_Management;

with Node_Get_Routines;
use Node_Get_Routines;

with Symtbl_Entry_Get_Routines;
use Symtbl_Entry_Get_Routines;

with Misc_Support_Routines;
use Misc_Support_Routines;

with Storage_Management_Routines;
use Storage_Management_Routines;

with Error_Handler;
use Error_Handler;

with Ace_Universal_Types;
use Ace_Universal_Types;

with Literals;
use Literals;

package body Statement_Evaluator is

   Anon_Id_Counter              : Integer := 0;
   -----
   -- to properly handle RETURN stmts
   -----
   Return_Statement_Encountered : exception;
   -----
   -- to properly handle exceptions
   -- This might have to move to the top level of ACE, so it can be
   -- handled by global exception handlers. [TBD]
   -----
   Exception_Encountered        : exception;

   -- --------------------------------------------------------------------------
   -- evaluator support routines
   -- [private to package]
   --

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Is_True
   --
   --  <Description>
   --
   --    Returns true iff the value stored at the given node is true.
   --
   --==========================================================================
   -- tests if (expr.val = "true")
   function Is_True (Node : in Ref_Statement_Database) return Boolean is
   separate;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Is_Equal
   --
   --  <Description>
   --
   --    Returns true iff the values stored at each node are equal.
   --
   --==========================================================================
   -- node1, node2 are exprs
   function Is_Equal (Node_1, Node_2 : in Ref_Data_Storage) return Boolean is
   separate;

   --==========================================================================
   --  <Subprogram Name>
   --
   --    Deallocate_Locals
   --
   --  <Description>
   --
   --    Deallocate_Locals
   --
   --==========================================================================
   procedure Deallocate_Locals
       (Context : in out Context_Control.Context_Definition;
        Node    : in Ref_Statement_Database) is
      Subprogram_Name : constant String        := "Deallocate_Locals";

      Current         : Ref_Statement_Database := Node;

      procedure Deallocate_Storage_For_Package_Decls
          (Node : Ref_Statement_Database) is
         Current : Ref_Statement_Database := Node;
      begin
         while (not (Is_Empty (Current))) loop
            case Check_Stmt_Node (Current) is

               when Object_Declaration   =>
                  declare
                     List_Node : Ref_Statement_Database :=
                        Get_Name_List (Current);
                  begin
                     while (not Is_Empty (List_Node)) loop
                        declare
                           Object_Symtbl_Entry : Ref_Symbol_Table_Entry :=
                              Get_Symbol_Table_Entry
                                  (Get_Expression (List_Node));
                        begin
                           Deallocate_Storage (Context, Object_Symtbl_Entry);
                           List_Node := Get_Next_List_Item (List_Node);
                        end;
                     end loop;
                  end;

               when Type_Declaration     | Subtype_Declaration    =>
                  Deallocate_Storage
                      (Context, Get_Symbol_Table_Entry (Current));

               when Package_Declaration  =>
                  -- recurse on this routine for packages within packages.
                  Deallocate_Storage_For_Package_Decls
                      (Get_Package_Spec_Decls
                           (Get_Symbol_Table_Entry (Current)));

               when others     =>
                  null; -- ignore
            end case;

            Current := Get_Next_Statement (Current);
         end loop;
      end Deallocate_Storage_For_Package_Decls;

   begin
      while (not (Is_Empty (Current))) loop
         case Check_Stmt_Node (Current) is

            when Object_Declaration =>
               declare
                  List_Node : Ref_Statement_Database :=
                     Get_Name_List (Current);
               begin
                  while (not Is_Empty (List_Node)) loop
                     declare
                        Object_Symtbl_Entry : Ref_Symbol_Table_Entry :=
                           Get_Symbol_Table_Entry (Get_Expression (List_Node));
                     begin
                        Deallocate_Storage (Context, Object_Symtbl_Entry);

                        -- UN-Observe_Object_If_Objects_Observation_Is_On
                        --    (Object_Symtbl_Entry);

                        List_Node := Get_Next_List_Item (List_Node);
                     end;
                  end loop;
               end;

            when Type_Declaration   | Subtype_Declaration    =>
               declare
                  Symtbl_Entry : Ref_Symbol_Table_Entry :=
                     Get_Symbol_Table_Entry (Current);
               begin
                  -- as appropriate "deallocate storage" also
                  Deallocate_Storage (Context, Symtbl_Entry);

                  -- UN-Observe_Object_If_Objects_Observation_Is_On
                  --    (Symtbl_Entry);
               end;

            when Procedure_Spec_Declaration |
                 Procedure_Declaration      |
                 Function_Spec_Declaration  |
                 Function_Declaration    =>
               null;
               -- UN-Observe_Object_If_Objects_Observation_Is_On
               --    (Get_Symbol_Table_Entry (current));

               --------------------

            when Package_Declaration     =>
               Deallocate_Storage_For_Package_Decls
                   (Get_Package_Spec_Decls (Get_Symbol_Table_Entry (Current)));

               -- UN-Observe_Object_If_Objects_Observation_Is_On
               --    (Package_Symtbl_Entry);

            when Package_Body_Declaration     =>
               Deallocate_Storage_For_Package_Decls
                   (Get_Package_Body_Decls (Get_Symbol_Table_Entry (Current)));

               -- UN- [tbd] observe package body entry ???

               --------------------

            when others   =>
               -- ignore others :-)
               null;
         end case;
         Current := Get_Next_Statement (Current);
      end loop;
   end Deallocate_Locals;

   -- --------------------------------------------------------------------
   --=====================================================================
   --  <Subprogram Name>
   --
   --    Evaluate_Statement
   --
   --  <Description>
   --
   --    Evaluates the list of statement (including declaration) node
   --    structures.
   --
   --=====================================================================
   procedure Evaluate_Statement
       (Context : in out Context_Control.Context_Definition;
        Node    : in Ref_Statement_Database) is separate;

end Statement_Evaluator;
