--|$Log:	yyerr.a,v $
--|Revision 8.0  91/01/17  10:46:35  cm
--|
--|
--|Copyright (c) 1990, Unisys Corporation.
--|
--|Ada Command Environment (ACE).
--|
--|This software, developed under the Software Technology for Adaptable,
--|Reliable Systems (STARS) program, is approved for release under Distribution
--|"A" of the Scientific and Technical Information Program Classification 
--|Scheme (DoD Directive 5230.24) unless otherwise indicated.  Sponsored by the
--|U.S. Defense Advanced Research Projects Agency (DARPA) under contract 
--|F19628-88-D-0031, the STARS program is supported by the military services, 
--|SEI, and MITRE, with the U.S. Air Force as the executive contracting agent.
--|
--|Permission to use, copy, modify, and comment on this software and its
--|documentation for purposes stated under Distribution "A" and without fee
--|is hereby granted.  This software retains Contractor indemnification to
--|the Government regarding copyrights pursuant to the above referenced
--|STARS contract.  The Government disclaims all responsibility against
--|liability, including costs and expenses for violation of proprietary
--|rights, or copyrights arising out of the creation or use of this
--|software.
--|
--|In addition, the Government and Unisys disclaim all warranties with
--|regard to this software, including all implied warranties of
--|merchantability and fitness, and in no event shall the Government or
--|Unisys be liable for any special, indirect or consequential damages or
--|any damages whatsoever resulting from the loss of use, data, or profits,
--|whether in action of contract, negligence or other tortuous action,
--|arising in connection with the use or performance of this software.
--|
--
-- Package Name - Yyval_Error
--
---------------------------------------------------------------
--
-- Abstract
--
--   This package contains the error routines for the parsing routines.
--
-- Update Log - :
--
-- End of Header
----------------------------------------------------------------------------

with Ace_Text_Io;

with Lexacts;

with Ace_Universal_Types;

package body Yyval_Error is

   use Ascii;

   --===========================================================================
   --  <Subprogram Name>
   --
   --     Error
   --
   --  <Description>
   --
   --     Fatal error routine for the parser to call.
   --
   --  <Exceptions Raised>
   --
   --     Program_Error always.
   --
   --===========================================================================
   procedure Error (S : String) is
   begin
      Ace_Text_Io.Put_Line (Ace_Text_Io.Current_Ace_Output, S);
      -- raise Program_Error;
   end Error;

   --==========================================================================
   --  <Subprogram Name>
   --
   --     Serror
   --
   --  <Description>
   --
   --    Routine for reporting syntax errors that resets the
   --    input buffer, prints an error message, and raises the exception
   --    "Parser_Error" that is handled by the main routine.
   --
   --  <Exceptions Raised>
   --
   --     Ace_Universal_Types.Parser_error
   --
   --==========================================================================
   procedure Serror (Line            : Integer;
                     Scaned_Token    : String;
                     Expected_Tokens : String) is

      I : Natural := Expected_Tokens'FIRST;
      J : Natural := 0;

   begin
      Ace_Text_Io.New_Line (Ace_Text_Io.Current_Ace_Output);
      Ace_Text_Io.Put_Line (Ace_Text_Io.Current_Ace_Output,
                            "Syntax error " & "on or near the token: " & '"' &
                            Scaned_Token & '"');
      Ace_Text_Io.Put (Ace_Text_Io.Current_Ace_Output, "Expecting: ");

      while (I + J <= Expected_Tokens'LAST) loop
         Ace_Text_Io.Put (Ace_Text_Io.Current_Ace_Output,
                          Expected_Tokens (I + J));
         if I > 60 and Expected_Tokens (I + J) = ' ' then
            J := J + I - 1;
            I := 0;
            Ace_Text_Io.New_Line (Ace_Text_Io.Current_Ace_Output);
            Ace_Text_Io.Put (Ace_Text_Io.Current_Ace_Output, "           ");
         end if;
         I := I + 1;
      end loop;

      Ace_Text_Io.New_Line (Ace_Text_Io.Current_Ace_Output);

      Lexacts.Reset_Eol;
      raise Ace_Universal_Types.Parser_Error;
   end Serror;

end Yyval_Error;
